using System;
using System.Text;
using System.Windows.Forms;

namespace Scan_Measurement_Example
{
    public class MeasurementManager
    {
        private Instrument m_instrument;

        public ApplicationStatus m_applicationStatus;
        public SettingsManager m_settingsManager;
        public DataManager m_dataManager;

        public MeasurementManager()
        {
            m_applicationStatus = new ApplicationStatus();
            m_settingsManager = new SettingsManager();
            m_dataManager = new DataManager();
        }

        ~MeasurementManager()
        {

        }


        /* Manages the scan measurement */
        public void runMeasurementManager()
        {
            /* Accomplishes the scan measurement */
            try
            {
                m_applicationStatus.setMeasurementIsRunning(true);

                initializeDriver();
                configureMeasurement();
                runScanMeasurement();
                readScanMeasurement();
                configurePeakSearch();
                searchPeaks();
                readPeakList();

                m_applicationStatus.setMeasurementIsRunning(false);
            }

            /* Catches occurring errors*/
            catch (Exception ex)
            {
                int errorCode = 0;

                if (null != m_instrument)
                    m_instrument.error_query(out errorCode, m_dataManager.m_instrumentData.getErrorMessage());

                MessageBox.Show(ex.Message + " " + m_dataManager.m_instrumentData.getErrorMessage(), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /* Initialize the instrument driver */
        public void initializeDriver()
        {
            if (m_settingsManager.m_remoteControlSetting.getInstrumentType() == InstrumentType.m_ESU || m_settingsManager.m_remoteControlSetting.getInstrumentType() == InstrumentType.m_ESCI || m_settingsManager.m_remoteControlSetting.getInstrumentType() == InstrumentType.m_ESPI)
            {
                m_instrument = new rsesci_Instrument(m_settingsManager.m_remoteControlSetting.getResourceDescriptor(), m_settingsManager.m_remoteControlSetting.getIdQuery(), m_settingsManager.m_remoteControlSetting.getResetDevice());
            }
            else if (m_settingsManager.m_remoteControlSetting.getInstrumentType() == InstrumentType.m_ESL)
            {
                m_instrument = new rsspecan_Instrument(m_settingsManager.m_remoteControlSetting.getResourceDescriptor(), m_settingsManager.m_remoteControlSetting.getIdQuery(), m_settingsManager.m_remoteControlSetting.getResetDevice());
            }
        }

        /* Configures the receiver measurement */
        public void configureMeasurement()
        {
            if (m_settingsManager.m_remoteControlSetting.getResetDevice() == true)
            {
                m_instrument.resetInstrument(m_settingsManager.m_remoteControlSetting.getResetDevice());
            }

            m_instrument.receiverMode();

            m_instrument.configureDisplayUpdate(m_settingsManager.m_remoteControlSetting.getDisplayEnable());

            m_instrument.configureScanTable(m_settingsManager.m_remoteControlSetting.getInstrumentType(), m_settingsManager.m_measurementSettings.getStepMode(), m_settingsManager.m_measurementSettings.getStartFrequency(), m_settingsManager.m_measurementSettings.getStopFrequency(), m_settingsManager.m_measurementSettings.getStepSize(), m_settingsManager.m_advancedMeasruementSettings.getResBW(), m_settingsManager.m_advancedMeasruementSettings.getMeasTime(), m_settingsManager.m_advancedMeasruementSettings.getRfAttn());

            m_instrument.configureAcquisition();

            m_instrument.configureFeedControl();

            m_instrument.configureReceiverDetector(m_settingsManager.m_advancedMeasruementSettings.getDetectorType());

            m_instrument.configurePowerUnit();
        }

        /* Runs the receiver measurement */
        public void runScanMeasurement()
        {
            m_instrument.runReceiverMeasurement();
        }

        /* Returns the result of the scan measurement */
        public void readScanMeasurement()
        {
            int traceStatus = 0;
            int statusReg = 0;
            int noOfPoints = 0;

            while ((traceStatus & 0x1000) == 0)
            {
                m_instrument.getStatusRegister(1, out statusReg);

                if ((statusReg & 512) != 0)
                {
                    m_instrument.readScanMeasurement(out traceStatus, out noOfPoints, m_dataManager.m_instrumentData.getStatusData(), m_dataManager.m_measurementData.getTrace1(), m_dataManager.m_measurementData.getTrace2(), m_dataManager.m_measurementData.getTrace3(), m_dataManager.m_measurementData.getTrace4(), m_dataManager.m_measurementData.getTrace5(), m_dataManager.m_measurementData.getTrace6());

                    m_dataManager.m_measurementData.getTrace1().CopyTo(m_dataManager.m_measurementData.getTraceData(), m_dataManager.m_instrumentData.getPointsToPrint());

                    m_dataManager.m_instrumentData.setPointsToPrint(noOfPoints);
                }
            }
        }

        /* Configures the peak search */
        public void configurePeakSearch()
        {
            m_instrument.configurePeakSearch(m_settingsManager.m_peakSearchSettings.getPeakSearchMethod(), m_settingsManager.m_peakSearchSettings.getNumberOfPeaksSubranges());
        }

        /* Generates a peak list from the scan results */
        public void searchPeaks()
        {
            m_instrument.searchPeaks();
        }

        /* Returns the results of the peak search */
        public void readPeakList()
        {
            m_instrument.readPeakList(m_dataManager.m_measurementData.getPeakFrequency(), m_dataManager.m_measurementData.getPeakLevel(), m_dataManager.m_measurementData.getPeakDeltaLevel(), m_settingsManager.m_peakSearchSettings.getNumberOfPeaksSubranges());
        }

        /* Specify the method of communication you will use to control the measurement hardware */
        public void setRemoteControlSettings(int instrumentModel, string resourceDescriptor, bool idQuery, bool resetDevice, bool displayEnable)
        {
            m_settingsManager.setRemoteControlSettings(instrumentModel, resourceDescriptor, idQuery, resetDevice, displayEnable);
        }

        /* Configures the start and stop frequency from the measurement */
        public void setMeasurementSettings(double startFreq, double stopFreq, int stepMode, double stepSize)
        {
            m_settingsManager.setMeasurementSettings(startFreq, stopFreq, stepMode, stepSize);
        }

        /* Adapts the measurement to certain needs */
        public void setAdvancedMeasurementSettings(int detector, double resBW, double measTime, double rfAttn)
        {
            m_settingsManager.setAdvancedMeasurementSettings(detector, resBW, measTime, rfAttn);
        }

        /* Configure the peak search after the scan measurement */
        public void setPeakSearchSettings(bool methodPeaks, bool methodSubranges, int numberOfPeaksSubranges)
        {
            m_settingsManager.setPeakSearchSettings(methodPeaks, methodSubranges, numberOfPeaksSubranges);
        }
    }
}