namespace InstrumentDrivers
{
    using System;
    using System.Runtime.InteropServices;


    public class rsesci : object, System.IDisposable
    {

        private System.Runtime.InteropServices.HandleRef _handle;

        private bool _disposed = true;

        ~rsesci() { Dispose(false); }


        /// <summary>
        /// This function creates an IVI instrument driver session, typically using the C session instrument handle.
        /// </summary>
        /// <param name="Instrument_Handle">
        /// The instrument handle that is used to create an IVI instrument driver session.
        /// </param>
        public rsesci(System.IntPtr Instrument_Handle)
        {
            this._handle = new System.Runtime.InteropServices.HandleRef(this, Instrument_Handle);
            this._disposed = false;
        }

        /// <summary>
        /// This function performs the following initialization actions:
        /// 
        /// - Opens a session to the Default Resource Manager resource and a session to the specified device using the interface and address specified in the Resource_Name control.
        /// 
        /// - Performs an identification query on the Instrument.
        /// 
        /// - Resets the instrument to a known state.
        /// 
        /// - Sends initialization commands to the instrument that set any necessary programmatic variables to the state necessary for the operation of the instrument driver.
        /// 
        /// - Returns an Instrument Handle which is used to differentiate between different sessions of this instrument driver.
        /// 
        /// - Each time this function is invoked a Unique Session is opened.  It is possible to have more than one session open for the same resource.
        /// 
        /// </summary>
        /// <param name="Resource_Name">
        /// This control specifies the interface and address of the device that is to be initialized (Instrument Descriptor). The exact grammar to be used in this control is shown in the note below. 
        /// 
        /// Default Value:  "GPIB::20"
        /// 
        /// Notes:
        /// 
        /// (1) Based on the Instrument Descriptor, this operation establishes a communication session with a device.  The grammar for the Instrument Descriptor is shown below.  Optional parameters are shown in square brackets ([]).
        /// 
        /// Interface   Grammar
        /// ------------------------------------------------------
        /// Serial      ASRL&lt;port&gt;[::INSTR]
        /// GPIB        GPIB[board]::primary address[::secondary address]
        ///             [::INSTR]
        /// RSIB        RSIB::remote_host::INSTR 
        /// VXI-11      TCPIP::remote_host::INSTR
        ///             
        /// The ASRL keyword is used for serial interface.
        /// The GPIB keyword is used for GPIB interface.
        /// The RSIB keyword is used for RSIB interface.
        /// The TCPIP keyword is used for VXI-11 interface.
        /// 
        /// Examples:
        /// (1) Serial - "ASRL1"
        /// (2) GPIB   - "GPIB::20"
        /// (3) RSIB   - "RSIB::192.168.1.33::INSTR"
        /// (4) VXI-11    - "TCPIP::192.168.1.33::INSTR"
        /// 
        /// The default value for optional parameters are shown below.
        /// 
        /// Optional Parameter          Default Value
        /// -----------------------------------------
        /// board                       0
        /// secondary address           none - 31
        /// 
        /// </param>
        /// <param name="ID_Query">
        /// This control specifies if an ID Query is sent to the instrument during the initialization procedure.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Skip Query
        /// VI_TRUE  (1) - Do Query (Default Value)
        /// 
        /// Notes:
        ///    
        /// (1) Under normal circumstances the ID Query ensures that the instrument initialized is the type supported by this driver. However circumstances may arise where it is undesirable to send an ID Query to the instrument.  In those cases; set this control to "Skip Query" and this function will initialize the selected interface, without doing an ID Query.
        /// 
        /// </param>
        /// <param name="Reset_Device">
        /// This control specifies if the instrument is to be reset to its power-on settings during the initialization procedure.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Don't Reset
        /// VI_TRUE  (1) - Reset Device (Default Value)
        /// 
        /// Notes:
        /// 
        /// (1) If you do not want the instrument reset. Set this control to "Don't Reset" while initializing the instrument.
        /// 
        /// </param>
        public rsesci(string Resource_Name, bool ID_Query, bool Reset_Device)
        {
            System.IntPtr instrumentHandle;
            int pInvokeResult = PInvoke.init(Resource_Name, System.Convert.ToUInt16(ID_Query), System.Convert.ToUInt16(Reset_Device), out instrumentHandle);
            this._handle = new System.Runtime.InteropServices.HandleRef(this, instrumentHandle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            this._disposed = false;
        }

        /// <summary>
        /// Gets the instrument handle.
        /// </summary>
        /// <value>
        /// The value is the IntPtr that represents the handle to the instrument.
        /// </value>
        public System.IntPtr Handle
        {
            get
            {
                return this._handle.Handle;
            }
        }

        /// <summary>
        /// This function is an simple example how to use instrument driver functions.
        /// 
        /// This function allows:
        /// 
        ///  - set up start and stop frequencies
        ///  - set up continuous or single sweep mode and trigger it
        ///  - read out measured trace data
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]FREQuency:OFFSet
        /// [SENSe&lt;1|2&gt;:]FREQuency:STARt
        /// [SENSe&lt;1|2&gt;:]FREQuency:STOP
        /// [SENSe&lt;1|2&gt;:]SWEep:POINts
        /// FORMat[:DATA] ASCii | REAL
        /// TRACe&lt;1|2&gt;:DATA? TRACE1
        /// INITiate&lt;1|2&gt;:CONTinuous ON | OFF
        /// INITiate&lt;1|2&gt;[:IMMediate]
        /// *OPC?
        /// </summary>
        /// <param name="Measurement_Mode">
        /// This control sets the measurement mode.
        /// 
        /// Valid Range:
        /// 0 - Set Up Frequencies
        /// 1 - Continual Measurement
        /// 2 - Single Measurement
        /// 3 - Transfer Data Only
        /// 
        /// Default Value: 0
        /// 
        /// Notes:
        /// 
        /// (1) Set Up Frequencies:
        /// Allows to set up start and stop frequencies.
        /// 
        /// (2) Continual Measurement:
        /// Sets the continual sweeping.
        /// 
        /// (3) Single Measurement:
        /// Sets the single sweeping and immediately sweeps.
        /// 
        /// (4) Transfer Data Only:
        /// Tranfers data from the instrument without reconfiguring it.
        /// 
        /// </param>
        /// <param name="Frequency_Parameter">
        /// Specifies which parameter will be set.
        /// 
        /// Valid Range:
        /// 0 - Start And Stop (Default Value)
        /// 1 - Start Only
        /// 2 - Stop Only
        /// 
        /// </param>
        /// <param name="Start_Frequency">
        /// Sets the start frequency of the analyzer.
        /// 
        /// Valid Range: 0.0 Hz to 3.0e9 Hz (depends on the model)
        /// 
        /// Default Value: 1.0e9 Hz
        /// 
        /// Note:
        /// 
        /// Frequency offset is set to 0.0 Hz.
        /// 
        /// </param>
        /// <param name="Stop_Frequency">
        /// Sets the stop frequency of the analyzer.
        /// 
        /// Valid Range: 0.0 Hz to 3.0e9 Hz (depends on the model)
        /// 
        /// Default Value: 2.0e9 Hz
        /// 
        /// Note:
        /// 
        /// Frequency offset is set to 0.0 Hz.
        /// 
        /// </param>
        /// <param name="Samples_Returned">
        /// Returns the number of trace data points.
        /// 
        /// </param>
        /// <param name="Trace_Data">
        /// Returns the trace data.
        /// 
        /// Note:
        /// 
        /// (1) The array must contain at least 501 elements of ViReal64[] data type.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int ReadSpectrum(int Measurement_Mode, int Frequency_Parameter, double Start_Frequency, double Stop_Frequency, out int Samples_Returned, double[] Trace_Data)
        {
            int pInvokeResult = PInvoke.ReadSpectrum(this._handle, Measurement_Mode, Frequency_Parameter, Start_Frequency, Stop_Frequency, out Samples_Returned, Trace_Data);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the receiver frequency or the center frequency in analyzer mode.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]FREQuency:CENTer
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Center_Frequency">
        /// This control sets the receiver frequency or the center frequency.
        /// 
        /// Valid Range: 0.0 to 7.0e9 Hz (depends on the model)
        /// 
        /// Default Value: 1.5e9 Hz
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetCenterFrequency(int Window, double Center_Frequency)
        {
            int pInvokeResult = PInvoke.SetCenterFrequency(this._handle, Window, Center_Frequency);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the receiver frequency or the center frequency in analyzer mode.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]FREQuency:CENTer?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Center_Frequency">
        /// This control returns the receiver frequency or the center frequency.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetCenterFrequency(int Window, out double Center_Frequency)
        {
            int pInvokeResult = PInvoke.GetCenterFrequency(this._handle, Window, out Center_Frequency);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the start frequency of the overall scan in receiver mode and defines the start frequency of the sweep in analyzer mode.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis (frequency domain only)
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]FREQuency:STARt
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Start_Frequency">
        /// This control sets the start frequency of the overall scan in receiver mode and sets the start frequency of the sweep in analyzer mode.
        /// 
        /// Valid Range: 0.0 to 7.0e9 Hz (depends on the model)
        /// 
        /// Default Value: 0.0 Hz
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetStartFrequency(int Window, double Start_Frequency)
        {
            int pInvokeResult = PInvoke.SetStartFrequency(this._handle, Window, Start_Frequency);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the start frequency of the overall scan in receiver mode and returns the start frequency of the sweep in analyzer mode.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis (frequency domain only)
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]FREQuency:STARt?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Start_Frequency">
        /// This control returns the start frequency of the overall scan in receiver mode and returns the start frequency of the sweep in analyzer mode.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetStartFrequency(int Window, out double Start_Frequency)
        {
            int pInvokeResult = PInvoke.GetStartFrequency(this._handle, Window, out Start_Frequency);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the stop frequency of the overall scan in receiver mode and defines the stop frequency of the sweep in analyzer mode.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis (frequency domain only)
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]FREQuency:STOP
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Stop_Frequency">
        /// This control sets the stop frequency of the overall scan in receiver mode and sets the stop frequency of the sweep in analyzer mode.
        /// 
        /// Valid Range: 0.0 to 7.0e9 Hz (depends on the model)
        /// 
        /// Default Value: 7.0e9 Hz
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetStopFrequency(int Window, double Stop_Frequency)
        {
            int pInvokeResult = PInvoke.SetStopFrequency(this._handle, Window, Stop_Frequency);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the stop frequency of the overall scan in receiver mode and returns the stop frequency of the sweep in analyzer mode.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis (frequency domain only)
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]FREQuency:STOP?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Stop_Frequency">
        /// This control returns the stop frequency of the overall scan in receiver mode and returns the stop frequency of the sweep in analyzer mode.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetStopFrequency(int Window, out double Stop_Frequency)
        {
            int pInvokeResult = PInvoke.GetStopFrequency(this._handle, Window, out Stop_Frequency);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the frequency offset of the instrument.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]FREQuency:OFFSet
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Frequency_Offset">
        /// This control sets the frequency offset of the instrument.
        /// 
        /// Valid Range: -100.0e9 to 100.0e9 Hz
        /// 
        /// Default Value: 0.0 Hz
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetFrequencyOffset(int Window, double Frequency_Offset)
        {
            int pInvokeResult = PInvoke.SetFrequencyOffset(this._handle, Window, Frequency_Offset);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the frequency offset of the instrument.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]FREQuency:OFFSet?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Frequency_Offset">
        /// This control returns the frequency offset of the instrument.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetFrequencyOffset(int Window, out double Frequency_Offset)
        {
            int pInvokeResult = PInvoke.GetFrequencyOffset(this._handle, Window, out Frequency_Offset);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches between single frequency measurement (CW) and scan (SCAN), if in EMI receiver operating mode. 
        /// 
        /// In SPECTRUM mode it switches between frequency domain (SWEep) and time domain (CW | FIXed).
        /// 
        /// In SCAN mode the frequency setup is performed using the commands FREQuency:STARt, FREQuency:STOP in the SENSe:SCAN subsystem.
        /// 
        /// Note(s):
        /// 
        /// (1) The parameter TDOMain is available in the receiver mode only with the R&amp;S ESU instrument and option R&amp;S ESU-K53.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]FREQuency:MODE FIXed | SWEep | SCAN | TDOMain
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Frequency_Mode">
        /// This control switches between single frequency measurement (CW) and scan (SCAN), if in EMI receiver operating mode. In SPECTRUM mode it switches between frequency domain (SWEep) and
        /// time domain (CW | FIXed). In the receiver mode, this command switches between frequency scan (SCAN) and time domain scan (TDOMain).
        /// 
        /// Valid Values:
        /// RSESCI_FREQ_FIXED (0) - Fixed
        /// RSESCI_FREQ_SWEEP (1) - Sweep
        /// RSESCI_FREQ_SCAN  (2) - Scan
        /// RSESCI_FREQ_TDOM  (3) - Time Domain
        /// 
        /// Default Value: RSESCI_FREQ_FIXED (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetFrequencyMode(int Window, int Frequency_Mode)
        {
            int pInvokeResult = PInvoke.SetFrequencyMode(this._handle, Window, Frequency_Mode);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns frequency measurement mode or receiver operating mode.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]FREQuency:MODE?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Frequency_Mode">
        /// This control returns frequency measurement mode or receiver operating mode.
        /// 
        /// Valid Values:
        /// RSESCI_FREQ_FIXED (0) - Fixed
        /// RSESCI_FREQ_SWEEP (1) - Sweep
        /// RSESCI_FREQ_SCAN  (2) - Scan
        /// RSESCI_FREQ_TDOM  (3) - Time Domain
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetFrequencyMode(int Window, out int Frequency_Mode)
        {
            int pInvokeResult = PInvoke.GetFrequencyMode(this._handle, Window, out Frequency_Mode);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the step width of the center frequency.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]FREQuency:CENTer:STEP
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Step_Width">
        /// This control sets the step width of the center frequency.
        /// 
        /// Valid Range: 0.0 to 40.0e9 Hz
        /// 
        /// Default Value: 0.0 Hz
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetCenterFrequencyStepWidth(int Window, double Step_Width)
        {
            int pInvokeResult = PInvoke.SetCenterFrequencyStepWidth(this._handle, Window, Step_Width);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the step width of the center frequency.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]FREQuency:CENTer:STEP?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Step_Width">
        /// This control returns the step width of the center frequency.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetCenterFrequencyStepWidth(int Window, out double Step_Width)
        {
            int pInvokeResult = PInvoke.GetCenterFrequencyStepWidth(this._handle, Window, out Step_Width);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function couples the step width of the center frequency to span (span &gt;0) or to the resolution bandwidth (span = 0) or cancels the couplings.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]FREQuency:CENTer:STEP:LINK SPAN | RBW | OFF
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Step_Link">
        /// This control couples the step width of the center frequency to span or to the resolution bandwidth or cancels the couplings.
        /// 
        /// Valid Values:
        /// RSESCI_FREQ_STEP_LINK_OFF  (0) - Off
        /// RSESCI_FREQ_STEP_LINK_RBW  (1) - RBW
        /// RSESCI_FREQ_STEP_LINK_SPAN (2) - Span
        /// 
        /// Default Value:
        /// RSESCI_FREQ_STEP_LINK_SPAN (2)
        /// 
        /// 
        /// Note(s):
        /// 
        /// (1) Span:
        /// Coupling to frequency display range (for span &gt; 0).
        /// 
        /// (2) RBW:
        /// Coupling to resolution bandwidth (for span = 0).
        /// 
        /// (3) Off:
        /// Manual input, no coupling.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetCenterFrequencyStepLink(int Window, int Step_Link)
        {
            int pInvokeResult = PInvoke.SetCenterFrequencyStepLink(this._handle, Window, Step_Link);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the step link.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]FREQuency:CENTer:STEP:LINK?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Step_Link">
        /// This control returns the step link.
        /// 
        /// Valid Values:
        /// RSESCI_FREQ_STEP_LINK_OFF  (0) - Off
        /// RSESCI_FREQ_STEP_LINK_RBW  (1) - RBW
        /// RSESCI_FREQ_STEP_LINK_SPAN (2) - Span
        /// 
        /// Note(s):
        /// 
        /// (1) Span:
        /// Coupling to frequency display range (for span &gt; 0).
        /// 
        /// (2) RBW:
        /// Coupling to resolution bandwidth (for span = 0).
        /// 
        /// (3) Off:
        /// Manual input, no coupling.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetCenterFrequencyStepLink(int Window, out int Step_Link)
        {
            int pInvokeResult = PInvoke.GetCenterFrequencyStepLink(this._handle, Window, out Step_Link);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function couples the step width of the center frequency with a factor to the span (span &gt;0) or to the resolution bandwidth (span = 0).
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]FREQuency:CENTer:STEP:LINK:FACTor
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Step_Link_Factor">
        /// This control couples the step width of the center frequency with a factor to the span or to the resolution bandwidth.
        /// 
        /// Valid Range: 1 to 100 PCT
        /// 
        /// Default Value: 1 PCT
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetCenterFrequencyStepLinkFactor(int Window, int Step_Link_Factor)
        {
            int pInvokeResult = PInvoke.SetCenterFrequencyStepLinkFactor(this._handle, Window, Step_Link_Factor);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the step link factor.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]FREQuency:CENTer:STEP:LINK:FACTor?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Step_Link_Factor">
        /// This control returns the step link factor.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetCenterFrequencyStepLinkFactor(int Window, out int Step_Link_Factor)
        {
            int pInvokeResult = PInvoke.GetCenterFrequencyStepLinkFactor(this._handle, Window, out Step_Link_Factor);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches the signal-track function on or off for the selected measurement window.
        /// 
        /// The function is independent of the selected marker.
        /// 
        /// With signal track activated, the maximum signal is determined after each frequency sweep and the center frequency is set to the frequency of this signal. Thus with drifting signals the center frequency follows the signal.
        /// 
        /// Mode(s):
        /// Spectrum Analysis (frequency domain only)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:STRack[:STATe] ON | OFF
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Signal_Tracking">
        /// This control switches the signal-track function on or off for the selected measurement window.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetSignalTracking(int Window, bool Signal_Tracking)
        {
            int pInvokeResult = PInvoke.SetSignalTracking(this._handle, Window, System.Convert.ToUInt16(Signal_Tracking));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the signal-track function state for the selected measurement window.
        /// 
        /// The function is independent of the selected marker.
        /// 
        /// With signal track activated, the maximum signal is determined after each frequency sweep and the center frequency is set to the frequency of this signal. Thus with drifting signals the center frequency follows the signal.
        /// 
        /// Mode(s):
        /// Spectrum Analysis (frequency domain only)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:STRack:STATe?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Signal_Tracking">
        /// This control returns the signal-track function state for the selected measurement window.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetSignalTracking(int Window, out bool Signal_Tracking)
        {
            ushort Signal_TrackingAsUShort;
            int pInvokeResult = PInvoke.GetSignalTracking(this._handle, Window, out Signal_TrackingAsUShort);
            Signal_Tracking = System.Convert.ToBoolean(Signal_TrackingAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function define the bandwidth around the center frequency within which the largest signal is searched.
        /// 
        /// The function is independent of the selected marker. It is only available in the frequency domain (span &gt; 0).
        /// 
        /// Mode(s):
        /// Spectrum Analysis (frequency domain only)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:STRack:BANDwidth|BWIDth
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Signal_Tracking_Bandwidth">
        /// This control sets the bandwidth around the center frequency within which the largest signal is searched.
        /// 
        /// Valid Range: 10.0 to 40.0e9 Hz
        /// 
        /// Default Value: 1.0e6 Hz
        /// 
        /// Note(s):
        /// 
        /// (1) The entry of the search bandwidth is only possible if the Signal Track function is switched on (CALC:MARK:FUNC:STR ON).
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetSignalTrackingBandwidth(int Window, double Signal_Tracking_Bandwidth)
        {
            int pInvokeResult = PInvoke.SetSignalTrackingBandwidth(this._handle, Window, Signal_Tracking_Bandwidth);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the bandwidth around the center frequency within which the largest signal is searched.
        /// 
        /// The function is independent of the selected marker. It is only available in the frequency domain (span &gt; 0).
        /// 
        /// Mode(s):
        /// Spectrum Analysis (frequency domain only)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:STRack:BANDwidth|BWIDth?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Signal_Tracking_Bandwidth">
        /// This control returns the bandwidth around the center frequency within which the largest signal is searched.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetSignalTrackingBandwidth(int Window, out double Signal_Tracking_Bandwidth)
        {
            int pInvokeResult = PInvoke.GetSignalTrackingBandwidth(this._handle, Window, out Signal_Tracking_Bandwidth);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the threshold above which the largest signal is searched for in the selected measurement window. 
        /// 
        /// The function is independent of the selected marker. It is only available in the frequency domain (span &gt; 0).
        /// 
        /// Mode(s):
        /// Spectrum Analysis (frequency domain only)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:STRack:THReshold
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Signal_Tracking_Threshold">
        /// This control sets the threshold above which the largest signal is searched for in the selected measurement window. 
        /// 
        /// Valid Range: -330.0 to +30.0 dBm
        /// 
        /// Default Value: -120.0 dBm
        /// 
        /// Note(s):
        /// 
        /// (1) The entry of the search bandwidth is only possible if the Signal Track function is switched on (CALC:MARK:FUNC:STR ON).
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetSignalTrackingThreshold(int Window, double Signal_Tracking_Threshold)
        {
            int pInvokeResult = PInvoke.SetSignalTrackingThreshold(this._handle, Window, Signal_Tracking_Threshold);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the threshold above which the largest signal is searched for in the selected measurement window. 
        /// 
        /// The function is independent of the selected marker. It is only available in the frequency domain (span &gt; 0).
        /// 
        /// Mode(s):
        /// Spectrum Analysis (frequency domain only)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:STRack:THReshold?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Signal_Tracking_Threshold">
        /// This control returns the threshold above which the largest signal is searched for in the selected measurement window.
        /// 
        /// Note(s):
        /// 
        /// (1) The response unit depends on the settings defined with CALC:UNIT.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetSignalTrackingThreshold(int Window, out double Signal_Tracking_Threshold)
        {
            int pInvokeResult = PInvoke.GetSignalTrackingThreshold(this._handle, Window, out Signal_Tracking_Threshold);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the trace on which the largest signal is searched for in the selected measurement window. 
        /// 
        /// The function is independent of the selected marker. It is only available in the frequency domain (span &gt; 0).
        /// 
        /// Mode(s):
        /// Spectrum Analysis (frequency domain only)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:STRack:TRACe
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Signal_Tracking_Trace">
        /// This control sets the trace on which the largest signal is searched for in the selected measurement window. 
        /// 
        /// Valid Range: 1 to 3
        /// 
        /// Default Value: 1
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetSignalTrackingTrace(int Window, int Signal_Tracking_Trace)
        {
            int pInvokeResult = PInvoke.SetSignalTrackingTrace(this._handle, Window, Signal_Tracking_Trace);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the trace on which the largest signal is searched for in the selected measurement window. 
        /// 
        /// The function is independent of the selected marker. It is only available in the frequency domain (span &gt; 0).
        /// 
        /// Mode(s):
        /// Spectrum Analysis (frequency domain only)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:STRack:TRACe?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Signal_Tracking_Trace">
        /// This control returns the trace on which the largest signal is searched for in the selected measurement window.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetSignalTrackingTrace(int Window, out int Signal_Tracking_Trace)
        {
            int pInvokeResult = PInvoke.GetSignalTrackingTrace(this._handle, Window, out Signal_Tracking_Trace);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the frequency span in the analyzer mode.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]FREQuency:SPAN
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Frequency_Span">
        /// This control sets the frequency span in the analyzer mode.
        /// 
        /// Valid Range: 0.0 to 7.0e9 Hz (depends on the model)
        /// 
        /// Default Value: 0.0 Hz
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetFrequencySpan(int Window, double Frequency_Span)
        {
            int pInvokeResult = PInvoke.SetFrequencySpan(this._handle, Window, Frequency_Span);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the frequency span in the analyzer mode.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]FREQuency:SPAN?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Frequency_Span">
        /// This control returns the frequency span in the analyzer mode.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetFrequencySpan(int Window, out double Frequency_Span)
        {
            int pInvokeResult = PInvoke.GetFrequencySpan(this._handle, Window, out Frequency_Span);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the frequency span in the analyzer mode to its maximum.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]FREQuency:SPAN:FULL
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetFrequencyFullSpan(int Window)
        {
            int pInvokeResult = PInvoke.SetFrequencyFullSpan(this._handle, Window);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the reference level in the selected measurement window.
        /// 
        /// Note(s):
        /// 
        /// (1) Depending on the coupling of the measurement windows, it is valid for both screens (INSTrument:COUPle ALL) or only for the selected measurement window (INSTrument:COUPle NONE). With the reference level offset &lt;&gt; 0 the indicated value range of the reference level is modified by the offset.
        /// 
        /// (2) The unit depends on the setting defined with CALCulate:UNIT. 
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1...3&gt;:Y[:SCALe]:RLEVel
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Reference_Level">
        /// This control sets the reference level in the selected measurement window.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: -20.0 dBm
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetReferenceLevel(int Window, double Reference_Level)
        {
            int pInvokeResult = PInvoke.SetReferenceLevel(this._handle, Window, Reference_Level);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the reference level in the selected measurement window.
        /// 
        /// Note(s):
        /// 
        /// (1) Depending on the coupling of the measurement windows, it is valid for both screens (INSTrument:COUPle ALL) or only for the selected measurement window (INSTrument:COUPle NONE). With the reference level offset &lt;&gt; 0 the indicated value range of the reference level is modified by the offset.
        /// 
        /// (2) The unit depends on the setting defined with CALCulate:UNIT. 
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1...3&gt;:Y[:SCALe]:RLEVel?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Reference_Level">
        /// This control returns the reference level in the selected measurement window.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetReferenceLevel(int Window, out double Reference_Level)
        {
            int pInvokeResult = PInvoke.GetReferenceLevel(this._handle, Window, out Reference_Level);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the offset of the reference level in the selected measurement window.
        /// 
        /// Note(s):
        /// 
        /// (1) Depending on the coupling of the measurement windows, it is valid for both screens (INSTrument:COUPle ALL) or only for the selected measurement window (INSTrument:COUPle NONE).
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1...3&gt;:Y[:SCALe]:RLEVel:OFFSet
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Reference_Level_Offset">
        /// This control sets the offset of the reference level in the selected measurement window.
        /// 
        /// Valid Range: -200.0 to 200.0 dB
        /// 
        /// Default Value: 0.0 dB
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetReferenceLevelOffset(int Window, double Reference_Level_Offset)
        {
            int pInvokeResult = PInvoke.SetReferenceLevelOffset(this._handle, Window, Reference_Level_Offset);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the offset of the reference level in the selected measurement window.
        /// 
        /// Note(s):
        /// 
        /// (1) Depending on the coupling of the measurement windows, it is valid for both screens (INSTrument:COUPle ALL) or only for the selected measurement window (INSTrument:COUPle NONE).
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1...3&gt;:Y[:SCALe]:RLEVel:OFFSet?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Reference_Level_Offset">
        /// This control returns the offset of the reference level in the selected measurement window.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetReferenceLevelOffset(int Window, out double Reference_Level_Offset)
        {
            int pInvokeResult = PInvoke.GetReferenceLevelOffset(this._handle, Window, out Reference_Level_Offset);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the position of the reference value in the selected measurement window.
        /// 
        /// Note(s):
        /// 
        /// (1) In operating mode NETWORK (Tracking Generator /Ext. Generator Option R&amp;S FSP-B9/B10) with active normalization, RPOSition defines the reference point for the output of the normalized measurement results.
        /// 
        /// (2) With the FM demodulator (FS-K7) switched on, the reference position for result display RF POWER and SPECTRUM is kept separate from that for result display FM.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1...3&gt;:Y[:SCALe]:RPOSition
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Reference_Level_Position">
        /// This control sets the reference level in the selected measurement window.
        /// 
        /// Valid Range: 0 to 100 PCT
        /// 
        /// Default Value: 100 PCT
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetReferenceLevelPosition(int Window, double Reference_Level_Position)
        {
            int pInvokeResult = PInvoke.SetReferenceLevelPosition(this._handle, Window, Reference_Level_Position);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the position of the reference value in the selected measurement window.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Note(s):
        /// 
        /// (1) In operating mode NETWORK (Tracking Generator /Ext. Generator Option R&amp;S FSP-B9/B10) with active normalization, RPOSition defines the reference point for the output of the normalized measurement results.
        /// 
        /// (2) With the FM demodulator (FS-K7) switched on, the reference position for result display RF POWER and SPECTRUM is kept separate from that for result display FM.
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1...3&gt;:Y[:SCALe]:RPOSition?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Reference_Level_Position">
        /// This control returns the reference level in the selected measurement window.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetReferenceLevelPosition(int Window, out double Reference_Level_Position)
        {
            int pInvokeResult = PInvoke.GetReferenceLevelPosition(this._handle, Window, out Reference_Level_Position);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// If the tracking generator option or the external generator control option (R&amp;S FSP-B9/B10) is mounted and the normalization in the NETWORK mode is activated, this value defines the power value assigned to the reference position in the selected measurement window. This value corresponds to the parameter REFERENCE VALUE in manual operation.
        /// 
        /// Note(s):
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FSP-B9, R&amp;S FSP-B10 or R&amp;S FS-K7.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACE&lt;1...3&gt;:Y[:SCALe]:RVALue
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Reference_Level_Position">
        /// This control sets the reference level in the selected measurement window.
        /// 
        /// Valid Range: 0 to 100 PCT
        /// 
        /// Default Value: 100 PCT
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetReferenceLevelPositionValue(int Window, double Reference_Level_Position)
        {
            int pInvokeResult = PInvoke.SetReferenceLevelPositionValue(this._handle, Window, Reference_Level_Position);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the position of the reference value in the selected measurement window.
        /// 
        /// Note(s):
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FSP-B9, R&amp;S FSP-B10 or R&amp;S FS-K7.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACE&lt;1...3&gt;:Y[:SCALe]:RVALue?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Reference_Level_Position">
        /// This control returns the reference level in the selected measurement window.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetReferenceLevelPositionValue(int Window, out double Reference_Level_Position)
        {
            int pInvokeResult = PInvoke.GetReferenceLevelPositionValue(this._handle, Window, out Reference_Level_Position);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the scaling of the Y axis in the current unit.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is only valid in conjunction with the option R&amp;S FS-K7.
        /// 
        /// Mode(s):
        /// Frequency Modulation
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACE:Y[:SCALe]:PDIVision &lt;numeric_value&gt;
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Y_Scale_Div">
        /// This control defines the scaling of the Y axis in the current unit.
        /// 
        /// Valid Range: 
        /// 0.1 to 20.0 dB
        /// 
        /// Default Value: 1.0
        /// </param>
        /// <returns>
        /// Returns the status code of this operation.  The status code  either indicates success or describes an error or warning condition.  You examine the status code from each call to an instrument driver function to determine if an error occurred.
        /// 
        /// To obtain a text description of the status code, call the RsSpecAn_error_message function.  To obtain additional information about the error condition, call the RsSpecAn_GetError function.  To clear the error information from the driver, call the RsSpecAn_ClearError function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// WARNINGS:
        /// 3FFA2001  The instrument was in an uncalibrated state
        ///           when the measurement was taken.
        /// 3FFA2002  The measurement taken was over the instrument??s
        ///           range.
        ///   
        /// ERRORS:
        /// BFFA2001  The Active Marker is not enabled.
        /// BFFA2002  The Active Marker is not a delta marker.
        /// BFFA2003  The maximum waiting time for this operation was
        ///           exceeded.
        ///           
        /// This instrument driver also returns errors and warnings defined by other sources.  The following table defines the ranges of additional status codes that this driver can return.  The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFA0000 to 3FFA1FFF     IVI    Warnings
        /// 3FFF0000 to 3FFFFFFF     VISA   Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP Driver Warnings
        ///           
        /// BFFA0000 to BFFA1FFF     IVI    Errors
        /// BFFF0000 to BFFFFFFF     VISA   Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP Driver Errors
        /// 
        /// </returns>
        public int SetYAxisScaleDivision(int Window, double Y_Scale_Div)
        {
            int pInvokeResult = PInvoke.SetYAxisScaleDivision(this._handle, Window, Y_Scale_Div);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the scaling of the Y axis in the current unit.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is only valid in conjunction with the option R&amp;S FS-K7.
        /// 
        /// Mode(s):
        /// Frequency Modulation
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACE:Y[:SCALe]:PDIVision?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Y_Scale_Div">
        /// This control returns the scaling of the Y axis in the current unit.
        /// 
        /// Valid Range: 
        /// 0.1 to 20.0 dB
        /// </param>
        /// <returns>
        /// Returns the status code of this operation.  The status code  either indicates success or describes an error or warning condition.  You examine the status code from each call to an instrument driver function to determine if an error occurred.
        /// 
        /// To obtain a text description of the status code, call the RsSpecAn_error_message function.  To obtain additional information about the error condition, call the RsSpecAn_GetError function.  To clear the error information from the driver, call the RsSpecAn_ClearError function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// WARNINGS:
        /// 3FFA2001  The instrument was in an uncalibrated state
        ///           when the measurement was taken.
        /// 3FFA2002  The measurement taken was over the instrument??s
        ///           range.
        ///   
        /// ERRORS:
        /// BFFA2001  The Active Marker is not enabled.
        /// BFFA2002  The Active Marker is not a delta marker.
        /// BFFA2003  The maximum waiting time for this operation was
        ///           exceeded.
        ///           
        /// This instrument driver also returns errors and warnings defined by other sources.  The following table defines the ranges of additional status codes that this driver can return.  The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFA0000 to 3FFA1FFF     IVI    Warnings
        /// 3FFF0000 to 3FFFFFFF     VISA   Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP Driver Warnings
        ///           
        /// BFFA0000 to BFFA1FFF     IVI    Errors
        /// BFFF0000 to BFFFFFFF     VISA   Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP Driver Errors
        /// 
        /// </returns>
        public int GetYAxisScaleDivision(int Window, out double Y_Scale_Div)
        {
            int pInvokeResult = PInvoke.GetYAxisScaleDivision(this._handle, Window, out Y_Scale_Div);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the display range of the Y axis (level axis) in the selected measurement window with logarithmic scaling (DISP:TRAC:Y:SPAC LOG).
        /// 
        /// Note(s):
        /// 
        /// (1) For linear scaling, (DISP:TRAC:Y:SPAC LIN | PERC) the display range is fixed and cannot be modified. 
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1...3&gt;:Y[:SCALe]
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Level_Axis_Range">
        /// This control sets the display range of the Y axis (level axis) in the selected measurement window with logarithmic scaling.
        /// 
        /// Valid Range: 10.0 to 200.0 dB
        /// 
        /// Default Value: 100.0 dB
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetLevelAxisRange(int Window, double Level_Axis_Range)
        {
            int pInvokeResult = PInvoke.SetLevelAxisRange(this._handle, Window, Level_Axis_Range);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the display range of the Y axis (level axis) in the selected measurement window with logarithmic scaling (DISP:TRAC:Y:SPAC LOG).
        /// 
        /// Note(s):
        /// 
        /// (1) For linear scaling, (DISP:TRAC:Y:SPAC LIN | PERC) the display range is fixed and cannot be modified. 
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1...3&gt;:Y[:SCALe]?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Level_Axis_Range">
        /// This control returns the display range of the Y axis (level axis) in the selected measurement window with logarithmic scaling.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetLevelAxisRange(int Window, out double Level_Axis_Range)
        {
            int pInvokeResult = PInvoke.GetLevelAxisRange(this._handle, Window, out Level_Axis_Range);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the minimum grid level in the current unit for the scan display in the receiver mode.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1...3&gt;:Y[:SCALe]:BOTTom
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Level_Axis_Minimum">
        /// This control sets the minimum grid level in the current unit for the scan display in the receiver mode.
        /// 
        /// Valid Range: none
        /// 
        /// Default Value: 0.0
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetLevelAxisMinimum(int Window, double Level_Axis_Minimum)
        {
            int pInvokeResult = PInvoke.SetLevelAxisMinimum(this._handle, Window, Level_Axis_Minimum);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the minimum grid level in the current unit for the scan display in the receiver mode.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1...3&gt;:Y[:SCALe]:BOTTom?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Level_Axis_Minimum">
        /// This control returns the minimum grid level in the current unit for the scan display in the receiver mode.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetLevelAxisMinimum(int Window, out double Level_Axis_Minimum)
        {
            int pInvokeResult = PInvoke.GetLevelAxisMinimum(this._handle, Window, out Level_Axis_Minimum);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function toggles between linear and logarithmic display in the selected measurement window.
        /// 
        /// Note(s):
        /// 
        /// (1) On a linear scale, switchover between the unit % (command DISP:WIND:TRAC:Y:SPAC LIN) and the unit dB (command DISP:WIND:TRAC:Y:SPAC LDB) is also possible.
        /// 
        /// (2) If the FM demodulator (FS-K7) is active and result display AF spectrum of FM is selected, only the parameters LINear and LOGarithmic are permissible.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1...3&gt;:Y:SPACing LINear | LOGarithmic | LDB
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Level_Axis_Range_Spacing">
        /// This control selects linear or logarithmic display in the selected measurement window.
        /// 
        /// Valid Values:
        /// RSESCI_SPACING_LIN (0) - Linear
        /// RSESCI_SPACING_LOG (1) - Logarithmic
        /// RSESCI_SPACING_LDB (2) - LDB Demodulator
        /// 
        /// Default Value: RSESCI_SPACING_LOG (1)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetLevelAxisRangeSpacing(int Window, int Level_Axis_Range_Spacing)
        {
            int pInvokeResult = PInvoke.SetLevelAxisRangeSpacing(this._handle, Window, Level_Axis_Range_Spacing);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns display type in the selected measurement window.
        /// 
        /// Note(s):
        /// 
        /// (1) On a linear scale, switchover between the unit % (command DISP:WIND:TRAC:Y:SPAC LIN) and the unit dB (command DISP:WIND:TRAC:Y:SPAC LDB) is also possible.
        /// 
        /// (2) If the FM demodulator (FS-K7) is active and result display AF spectrum of FM is selected, only the parameters LINear and LOGarithmic are permissible.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1...3&gt;:Y:SPACing?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Level_Axis_Range_Spacing">
        /// This control returns display type in the selected measurement window.
        /// 
        /// Valid Values:
        /// RSESCI_SPACING_LIN (0) - Linear            RSESCI_SPACING_LOG (1) - Logarithmic
        /// RSESCI_SPACING_LDB (2) - LDB Demodulator
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetLevelAxisRangeSpacing(int Window, out int Level_Axis_Range_Spacing)
        {
            int pInvokeResult = PInvoke.GetLevelAxisRangeSpacing(this._handle, Window, out Level_Axis_Range_Spacing);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the unit for power in the selected measurement window.
        /// 
        /// Note(s):
        /// 
        /// (1) In receiver mode, the units DBM, DBUV, DBUA, DBPW, DBUV_MHZ, DBMV_MHZ and DBUA_MHZ are available.
        /// 
        /// (2) The units DBUV_MHZ, DBUA_MHZ, DBMV_MHZ are available with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:UNIT:POWer DBM | V | A | W | DBPW | WATT | DBUV | DBMV | VOLT | DBUA | AMPere| DBPT| DBUV_M | DBUA_M | DBUV_MHZ | DBUA_MHZ | DBMV_MHZ
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Level_Axis_Unit">
        /// This control selects the unit for power in the selected measurement window.
        /// 
        /// Valid Values:
        /// RSESCI_UNIT_DBM       (0) - dBm
        /// RSESCI_UNIT_VOLT      (1) - Volt
        /// RSESCI_UNIT_AMP       (2) - Ampere
        /// RSESCI_UNIT_WATT      (3) - Watt
        /// RSESCI_UNIT_DBPW      (4) - dBpW
        /// RSESCI_UNIT_DBUV      (5) - dBuV
        /// RSESCI_UNIT_DBMV      (6) - dBmV
        /// RSESCI_UNIT_DBUA      (7) - dBuA
        /// RSESCI_UNIT_DBPT      (8) - dBpT
        /// RSESCI_UNIT_DBUV_M    (9)  - dBuV/m
        /// RSESCI_UNIT_DBUA_M    (10) - dBuA/m
        /// RSESCI_UNIT_DBUV_MHZ  (18) - dBuV/MHz
        /// RSESCI_UNIT_DBMV_MHZ  (19) - dBmV/MHz
        /// RSESCI_UNIT_DBUA_MHZ  (20) - dBuA/MHz
        /// 
        /// Default Value: RSESCI_UNIT_DBM (0)
        /// 
        /// Note(s):
        /// 
        /// (1) In receiver mode the units DBM, DBUV, DBUA, DBPW, DBT, DBUV_MHZ, DBMV_MHZ and DBUA_MHZ are available.
        /// 
        /// (2) In analyzer mode all units are available except for DBPT.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetLevelAxisUnit(int Window, int Level_Axis_Unit)
        {
            int pInvokeResult = PInvoke.SetLevelAxisUnit(this._handle, Window, Level_Axis_Unit);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the unit for power in the selected measurement window.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:UNIT:POWer?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Level_Axis_Unit">
        /// This control returns the unit for power in the selected measurement window.
        /// 
        /// Valid Values:
        /// RSESCI_UNIT_DBM       (0) - dBm
        /// RSESCI_UNIT_VOLT      (1) - Volt
        /// RSESCI_UNIT_AMP       (2) - Ampere
        /// RSESCI_UNIT_WATT      (3) - Watt
        /// RSESCI_UNIT_DBPW      (4) - dBpW
        /// RSESCI_UNIT_DBUV      (5) - dBuV
        /// RSESCI_UNIT_DBMV      (6) - dBmV
        /// RSESCI_UNIT_DBUA      (7) - dBuA
        /// RSESCI_UNIT_DBPT      (8) - dBpT
        /// RSESCI_UNIT_DBUV_M    (9)  - dBuV/m
        /// RSESCI_UNIT_DBUA_M    (10) - dBuA/m
        /// RSESCI_UNIT_DBUV_MHZ  (18) - dBuV/MHz
        /// RSESCI_UNIT_DBMV_MHZ  (19) - dBmV/MHz
        /// RSESCI_UNIT_DBUA_MHZ  (20) - dBuA/MHz
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetLevelAxisUnit(int Window, out int Level_Axis_Unit)
        {
            int pInvokeResult = PInvoke.GetLevelAxisUnit(this._handle, Window, out Level_Axis_Unit);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the scale type of the Y axis (absolute or relative) in the selected measurement window.
        /// 
        /// Note(s):
        /// 
        /// (1) When SYSTem:DISPlay:UPDate is set to OFF, this command has no immediate effect on the screen. 
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1...3&gt;:Y[:SCALe]:MODE ABSolute | RELative
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Level_Axis_Scaling">
        /// This control sets the scale type of the Y axis in the selected measurement window.
        /// 
        /// Valid Values:
        /// RSESCI_ABS (0) - Absolute
        /// RSESCI_REL (1) - Relative
        /// 
        /// Default Value: RSESCI_ABS (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetLevelAxisScaling(int Window, int Level_Axis_Scaling)
        {
            int pInvokeResult = PInvoke.SetLevelAxisScaling(this._handle, Window, Level_Axis_Scaling);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the scale type of the Y axis (absolute or relative) in the selected measurement window.
        /// 
        /// Note(s):
        /// 
        /// (1) When SYSTem:DISPlay is set to OFF, this command has no immediate effect on the screen. 
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1...3&gt;:Y[:SCALe]:MODE?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Level_Axis_Scaling">
        /// This control returns the scale type of the Y axis in the selected measurement window.
        /// 
        /// Valid Values:
        /// RSESCI_ABS (0) - Absolute
        /// RSESCI_REL (1) - Relative
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetLevelAxisScaling(int Window, out int Level_Axis_Scaling)
        {
            int pInvokeResult = PInvoke.GetLevelAxisScaling(this._handle, Window, out Level_Axis_Scaling);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches the input coupling of the RF input between AC and DC.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// INPut&lt;1|2&gt;:COUPling AC | DC
        /// </summary>
        /// <param name="RF_Input_Coupling">
        /// This control switches the input coupling of the RF input between AC and DC.
        /// 
        /// Valid Values:
        /// RSESCI_COUPLING_AC (0) - AC
        /// RSESCI_COUPLING_DC (1) - DC
        /// 
        /// Default Value: RSESCI_COUPLING_AC (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetRFInputCoupling(int RF_Input_Coupling)
        {
            int pInvokeResult = PInvoke.SetRFInputCoupling(this._handle, RF_Input_Coupling);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the input coupling of the RF input.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// INPut:COUPling?
        /// </summary>
        /// <param name="RF_Input_Coupling">
        /// This control returns the input coupling of the RF input.
        /// 
        /// Valid Values:
        /// RSESCI_COUPLING_AC (0) - AC
        /// RSESCI_COUPLING_DC (1) - DC
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetRFInputCoupling(out int RF_Input_Coupling)
        {
            int pInvokeResult = PInvoke.GetRFInputCoupling(this._handle, out RF_Input_Coupling);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function programs the input attenuator. 
        /// 
        /// Note(s):
        /// 
        /// (1) To protect the input mixer against damage from overloads, the setting 0 dB can be obtained by entering numerals, not by using the command DEC.
        /// 
        /// (2) In the default state with analyzer mode, the attenuation set on the step attenuator is coupled to the reference level of the instrument. If the attenuation is programmed directly, the coupling to the reference level is switched off.
        /// 
        /// (3) With receiver mode, if the attenuation is programmed directly, the auto range function is switched off if necessary. Settings below 10 dB can be prevented by activating the protection function (INPut:ATTenuation:PROTection ON).
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// INPut&lt;1|2&gt;:ATTenuation
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Attenuation">
        /// This control sets the input attenuator. The step width is 5 dB.
        /// 
        /// Valid Range: 0.0 to 75.0 dB
        /// 
        /// Default Value: 10.0 dB
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetRFInputAttenuation(int Window, double Attenuation)
        {
            int pInvokeResult = PInvoke.SetRFInputAttenuation(this._handle, Window, Attenuation);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the input attenuator. 
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// INPut&lt;1|2&gt;:ATTenuation?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Attenuation">
        /// This control returns the input attenuator.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetRFInputAttenuation(int Window, out double Attenuation)
        {
            int pInvokeResult = PInvoke.GetRFInputAttenuation(this._handle, Window, out Attenuation);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// In analyzer mode, this function automatically couples the input attenuation to the reference level (state ON) or switches the input attenuation to manual entry (state OFF). The minimum input attenuation set with the coupling switched on is 10 dB.
        /// 
        /// In receiver mode, this function automatically sets the attenuation so that a good S/N ratio is obtained without the receiver stages being overdriven (state ON).
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// INPut&lt;1|2&gt;:ATTenuation:AUTO ON | OFF
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Attenuation_Auto">
        /// This control sets auto attenuation.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_TRUE (1)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetRFInputAttenuationAuto(int Window, bool Attenuation_Auto)
        {
            int pInvokeResult = PInvoke.SetRFInputAttenuationAuto(this._handle, Window, System.Convert.ToUInt16(Attenuation_Auto));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns auto attenuation state.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// INPut&lt;1|2&gt;:ATTenuation:AUTO?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Attenuation_Auto">
        /// This control returns auto attenuation state.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetRFInputAttenuationAuto(int Window, out bool Attenuation_Auto)
        {
            ushort Attenuation_AutoAsUShort;
            int pInvokeResult = PInvoke.GetRFInputAttenuationAuto(this._handle, Window, out Attenuation_AutoAsUShort);
            Attenuation_Auto = System.Convert.ToBoolean(Attenuation_AutoAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines whether the 0 dB position of the attenuator is to be used in manual or automatic adjustment.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// INPut&lt;1|2&gt;:ATTenuation:PROTection[:STATe] ON | OFF
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Attenuation_Protection">
        /// This control defines whether the 0 dB position of the attenuator is to be used in manual or automatic adjustment.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetRFInputAttenuationProtection(int Window, bool Attenuation_Protection)
        {
            int pInvokeResult = PInvoke.SetRFInputAttenuationProtection(this._handle, Window, System.Convert.ToUInt16(Attenuation_Protection));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns whether the 0 dB position of the attenuator is to be used in manual or automatic adjustment.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// INPut&lt;1|2&gt;:ATTenuation:PROTection[:STATe]?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Attenuation_Protection">
        /// This control returns whether the 0 dB position of the attenuator is to be used in manual or automatic adjustment.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetRFInputAttenuationProtection(int Window, out bool Attenuation_Protection)
        {
            ushort Attenuation_ProtectionAsUShort;
            int pInvokeResult = PInvoke.GetRFInputAttenuationProtection(this._handle, Window, out Attenuation_ProtectionAsUShort);
            Attenuation_Protection = System.Convert.ToBoolean(Attenuation_ProtectionAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function resets the attenuator into the state that it had before the overload condition was detected. It re-connects the RF input with the input mixer.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with R&amp;S ESU instrument.
        /// 
        /// (2) The R&amp;S ESU is equipped with an overload protection mechanism. This mechanism becomes active as soon as the power at the input mixer exceeds a value of 27 dBm. It ensures that the connection between RF input and input mixer is cut off.
        /// 
        /// (3) This function comes into effect only if the reason for the overload condition has been eliminated. Otherwise the connection between RF input and input mixer is left open.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// INPut&lt;1|2&gt;:ATTenuation:PROTection:RESet
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int ResetRFInputAttenuationProtection(int Window)
        {
            int pInvokeResult = PInvoke.ResetRFInputAttenuationProtection(this._handle, Window);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the nominal input impedance of the instrument. 
        /// 
        /// Note(s):
        /// 
        /// (1) The set impedance is taken into account in all level indications of results.
        /// 
        /// (2) The setting 75 ohm should be selected, if the 50 ohm input impedance is transformed to a higher impedance using a 75 ohm adapter of the RAZ type (= 25 ohm in series to the input impedance of the instrument). The correction value in this case is 1.76 dB = 10 log ( 75 ohm / 50 ohm).
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// INPut&lt;1|2&gt;:IMPedance 50 | 75
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="RF_Input_Impedance">
        /// This control sets the nominal input impedance of the instrument. 
        /// 
        /// Valid Values:
        /// RSESCI_INPUT_IMP_50 (0) - 50 Ohm RSESCI_INPUT_IMP_75 (1) - 75 Ohm
        /// 
        /// Default Value: RSESCI_INPUT_IMP_50 (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetRFInputImpedance(int Window, int RF_Input_Impedance)
        {
            int pInvokeResult = PInvoke.SetRFInputImpedance(this._handle, Window, RF_Input_Impedance);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the nominal input impedance of the instrument. 
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// INPut&lt;1|2&gt;:IMPedance?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="RF_Input_Impedance">
        /// This control returns the nominal input impedance of the instrument. 
        /// 
        /// Valid Values:
        /// RSESCI_INPUT_IMP_50 (0) - 50 Ohm RSESCI_INPUT_IMP_75 (1) - 75 Ohm
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetRFInputImpedance(int Window, out int RF_Input_Impedance)
        {
            int pInvokeResult = PInvoke.GetRFInputImpedance(this._handle, Window, out RF_Input_Impedance);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the signal input. 
        /// 
        /// Note(s):
        /// 
        /// (1) This function is only availabe with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// INPut&lt;1|2&gt;:TYPE INPUT1 | INPUT2
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Input_Type">
        /// This control selects the signal input. 
        /// 
        /// Valid Values:
        /// RSESCI_INPUT_TYPE_1 (0) - RF input 1 
        /// RSESCI_INPUT_TYPE_2 (1) - RF input 2
        /// 
        /// Default Value: RSESCI_INPUT_TYPE_1 (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetRFInputType(int Window, int Input_Type)
        {
            int pInvokeResult = PInvoke.SetRFInputType(this._handle, Window, Input_Type);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the signal input. 
        /// 
        /// Note(s):
        /// 
        /// (1) This function is only availabe with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// INPut&lt;1|2&gt;:TYPE?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Input_Type">
        /// This control returns the signal input. 
        /// 
        /// Valid Values:
        /// RSESCI_INPUT_TYPE_1 (0) - RF input 1 
        /// RSESCI_INPUT_TYPE_2 (1) - RF input 2
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetRFInputType(int Window, out int Input_Type)
        {
            int pInvokeResult = PInvoke.GetRFInputType(this._handle, Window, out Input_Type);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function includes the preamplifier into the autoranging function of the receiver.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// INPut&lt;1|2&gt;:GAIN:AUTO ON | OFF
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Preamplifier_Auto">
        /// This control sets the preamplifier into the autoranging function of the receiver.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetPreamplifierAuto(int Window, bool Preamplifier_Auto)
        {
            int pInvokeResult = PInvoke.SetPreamplifierAuto(this._handle, Window, System.Convert.ToUInt16(Preamplifier_Auto));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the preamplifier autoranging function state.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// INPut&lt;1|2&gt;:GAIN:AUTO?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Preamplifier_Auto">
        /// This control returns returns the preamplifier autoranging function state.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetPreamplifierAuto(int Window, out bool Preamplifier_Auto)
        {
            ushort Preamplifier_AutoAsUShort;
            int pInvokeResult = PInvoke.GetPreamplifierAuto(this._handle, Window, out Preamplifier_AutoAsUShort);
            Preamplifier_Auto = System.Convert.ToBoolean(Preamplifier_AutoAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches on the preamplifier for the instrument. 
        /// 
        /// Notes(s):
        /// 
        /// (1) The switchable gain is fixed to 20 dB.
        /// 
        /// (2) In analyzer mode, the command is only available, if the preselector measurement has been activated.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// INPut&lt;1|2&gt;:GAIN:STATe ON | OFF
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Preamplifier">
        /// This control switches on the preamplifier for the instrument. 
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetPreamplifier(int Window, bool Preamplifier)
        {
            int pInvokeResult = PInvoke.SetPreamplifier(this._handle, Window, System.Convert.ToUInt16(Preamplifier));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the preamplifier state for the instrument. 
        /// 
        /// Notes(s):
        /// 
        /// (1) The switchable gain is fixed to 20 dB.
        /// 
        /// (2) In analyzer mode, the command is only available, if the preselector measurement has been activated.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// INPut&lt;1|2&gt;:GAIN:STATe?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Preamplifier">
        /// This control returns the preamplifier state for the instrument.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetPreamplifier(int Window, out bool Preamplifier)
        {
            ushort PreamplifierAsUShort;
            int pInvokeResult = PInvoke.GetPreamplifier(this._handle, Window, out PreamplifierAsUShort);
            Preamplifier = System.Convert.ToBoolean(PreamplifierAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures input mixer of the analyzer.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// INPut&lt;1|2&gt;:MIXer[:POWer]
        /// INPut&lt;1|2&gt;:MIXer:AUTO ON | OFF
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Automatic_Setup">
        /// Enables/disables the automatic setup of the mixer level.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_TRUE (1)
        /// </param>
        /// <param name="Power">
        /// Defines the desired power at the input mixer of the analyzer. On any change to the reference level the RF attenuation will be adjusted in a way that makes the difference between reference level and RF attenuation come as close to the desired mixer level as possible. 
        /// 
        /// Valid Values:
        /// -100.0 to 5.0 dBm
        /// 
        /// Default Value: -25.0 dBm
        /// </param>
        /// <returns>
        /// Returns the status code of this operation.  The status code  either indicates success or describes an error or warning condition.  You examine the status code from each call to an instrument driver function to determine if an error occurred.
        /// 
        /// To obtain a text description of the status code, call the rsspecan_error_message function.  To obtain additional information about the error condition, call the rsspecan_GetError function.  To clear the error information from the driver, call the rsspecan_ClearError function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// WARNINGS:
        /// 3FFA2001  The instrument was in an uncalibrated state
        ///           when the measurement was taken.
        /// 3FFA2002  The measurement taken was over the instrument??s
        ///           range.
        ///   
        /// ERRORS:
        /// BFFA2001  The Active Marker is not enabled.
        /// BFFA2002  The Active Marker is not a delta marker.
        /// BFFA2003  The maximum waiting time for this operation was
        ///           exceeded.
        ///           
        /// This instrument driver also returns errors and warnings defined by other sources.  The following table defines the ranges of additional status codes that this driver can return.  The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFA0000 to 3FFA1FFF     IVI    Warnings
        /// 3FFF0000 to 3FFFFFFF     VISA   Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP Driver Warnings
        ///           
        /// BFFA0000 to BFFA1FFF     IVI    Errors
        /// BFFF0000 to BFFFFFFF     VISA   Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP Driver Errors
        /// 
        /// </returns>
        public int ConfigureInputMixerLevel(int Window, bool Automatic_Setup, double Power)
        {
            int pInvokeResult = PInvoke.ConfigureInputMixerLevel(this._handle, Window, System.Convert.ToUInt16(Automatic_Setup), Power);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function enables/disables the automatic setup of the mixer level.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// INPut&lt;1|2&gt;:MIXer:AUTO ON | OFF
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Automatic_Setup">
        /// Enables/disables the automatic setup of the mixer level.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_TRUE (1)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation.  The status code  either indicates success or describes an error or warning condition.  You examine the status code from each call to an instrument driver function to determine if an error occurred.
        /// 
        /// To obtain a text description of the status code, call the rsspecan_error_message function.  To obtain additional information about the error condition, call the rsspecan_GetError function.  To clear the error information from the driver, call the rsspecan_ClearError function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// WARNINGS:
        /// 3FFA2001  The instrument was in an uncalibrated state
        ///           when the measurement was taken.
        /// 3FFA2002  The measurement taken was over the instrument??s
        ///           range.
        ///   
        /// ERRORS:
        /// BFFA2001  The Active Marker is not enabled.
        /// BFFA2002  The Active Marker is not a delta marker.
        /// BFFA2003  The maximum waiting time for this operation was
        ///           exceeded.
        ///           
        /// This instrument driver also returns errors and warnings defined by other sources.  The following table defines the ranges of additional status codes that this driver can return.  The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFA0000 to 3FFA1FFF     IVI    Warnings
        /// 3FFF0000 to 3FFFFFFF     VISA   Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP Driver Warnings
        ///           
        /// BFFA0000 to BFFA1FFF     IVI    Errors
        /// BFFF0000 to BFFFFFFF     VISA   Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP Driver Errors
        /// 
        /// </returns>
        public int SetInputMixerLevelState(int Window, bool Automatic_Setup)
        {
            int pInvokeResult = PInvoke.SetInputMixerLevelState(this._handle, Window, System.Convert.ToUInt16(Automatic_Setup));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the state of the automatic setup of the mixer level.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// INPut&lt;1|2&gt;:MIXer:AUTO?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Automatic_Setup">
        /// Returns the state of the automatic setup of the mixer level.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation.  The status code  either indicates success or describes an error or warning condition.  You examine the status code from each call to an instrument driver function to determine if an error occurred.
        /// 
        /// To obtain a text description of the status code, call the rsspecan_error_message function.  To obtain additional information about the error condition, call the rsspecan_GetError function.  To clear the error information from the driver, call the rsspecan_ClearError function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// WARNINGS:
        /// 3FFA2001  The instrument was in an uncalibrated state
        ///           when the measurement was taken.
        /// 3FFA2002  The measurement taken was over the instrument??s
        ///           range.
        ///   
        /// ERRORS:
        /// BFFA2001  The Active Marker is not enabled.
        /// BFFA2002  The Active Marker is not a delta marker.
        /// BFFA2003  The maximum waiting time for this operation was
        ///           exceeded.
        ///           
        /// This instrument driver also returns errors and warnings defined by other sources.  The following table defines the ranges of additional status codes that this driver can return.  The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFA0000 to 3FFA1FFF     IVI    Warnings
        /// 3FFF0000 to 3FFFFFFF     VISA   Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP Driver Warnings
        ///           
        /// BFFA0000 to BFFA1FFF     IVI    Errors
        /// BFFF0000 to BFFFFFFF     VISA   Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP Driver Errors
        /// 
        /// </returns>
        public int GetInputMixerLevelState(int Window, out bool Automatic_Setup)
        {
            ushort Automatic_SetupAsUShort;
            int pInvokeResult = PInvoke.GetInputMixerLevelState(this._handle, Window, out Automatic_SetupAsUShort);
            Automatic_Setup = System.Convert.ToBoolean(Automatic_SetupAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the desired power at the input mixer of the analyzer. On any change to the reference level the RF attenuation will be adjusted in a way that makes the difference between reference level and RF attenuation come as close to the desired mixer level as possible.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// INPut&lt;1|2&gt;:MIXer[:POWer]
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Power">
        /// Defines the desired power at the input mixer of the analyzer. On any change to the reference level the RF attenuation will be adjusted in a way that makes the difference between reference level and RF attenuation come as close to the desired mixer level as possible. 
        /// 
        /// Valid Values:
        /// -100.0 to 5.0 dBm
        /// 
        /// Default Value: -25.0 dBm
        /// </param>
        /// <returns>
        /// Returns the status code of this operation.  The status code  either indicates success or describes an error or warning condition.  You examine the status code from each call to an instrument driver function to determine if an error occurred.
        /// 
        /// To obtain a text description of the status code, call the rsspecan_error_message function.  To obtain additional information about the error condition, call the rsspecan_GetError function.  To clear the error information from the driver, call the rsspecan_ClearError function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// WARNINGS:
        /// 3FFA2001  The instrument was in an uncalibrated state
        ///           when the measurement was taken.
        /// 3FFA2002  The measurement taken was over the instrument??s
        ///           range.
        ///   
        /// ERRORS:
        /// BFFA2001  The Active Marker is not enabled.
        /// BFFA2002  The Active Marker is not a delta marker.
        /// BFFA2003  The maximum waiting time for this operation was
        ///           exceeded.
        ///           
        /// This instrument driver also returns errors and warnings defined by other sources.  The following table defines the ranges of additional status codes that this driver can return.  The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFA0000 to 3FFA1FFF     IVI    Warnings
        /// 3FFF0000 to 3FFFFFFF     VISA   Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP Driver Warnings
        ///           
        /// BFFA0000 to BFFA1FFF     IVI    Errors
        /// BFFF0000 to BFFFFFFF     VISA   Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP Driver Errors
        /// 
        /// </returns>
        public int SetInputMixerPower(int Window, double Power)
        {
            int pInvokeResult = PInvoke.SetInputMixerPower(this._handle, Window, Power);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the power at the input mixer of the analyzer.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// INPut&lt;1|2&gt;:MIXer[:POWer]?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Power">
        /// Returns the power at the input mixer of the analyzer.
        /// 
        /// Valid Values:
        /// -100.0 to 5.0 dBm
        /// </param>
        /// <returns>
        /// Returns the status code of this operation.  The status code  either indicates success or describes an error or warning condition.  You examine the status code from each call to an instrument driver function to determine if an error occurred.
        /// 
        /// To obtain a text description of the status code, call the rsspecan_error_message function.  To obtain additional information about the error condition, call the rsspecan_GetError function.  To clear the error information from the driver, call the rsspecan_ClearError function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// WARNINGS:
        /// 3FFA2001  The instrument was in an uncalibrated state
        ///           when the measurement was taken.
        /// 3FFA2002  The measurement taken was over the instrument??s
        ///           range.
        ///   
        /// ERRORS:
        /// BFFA2001  The Active Marker is not enabled.
        /// BFFA2002  The Active Marker is not a delta marker.
        /// BFFA2003  The maximum waiting time for this operation was
        ///           exceeded.
        ///           
        /// This instrument driver also returns errors and warnings defined by other sources.  The following table defines the ranges of additional status codes that this driver can return.  The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFA0000 to 3FFA1FFF     IVI    Warnings
        /// 3FFF0000 to 3FFFFFFF     VISA   Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP Driver Warnings
        ///           
        /// BFFA0000 to BFFA1FFF     IVI    Errors
        /// BFFF0000 to BFFFFFFF     VISA   Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP Driver Errors
        /// 
        /// </returns>
        public int GetInputMixerPower(int Window, out double Power)
        {
            int pInvokeResult = PInvoke.GetInputMixerPower(this._handle, Window, out Power);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the bandwidth of the main PLL of the instrument synthesizer.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]BANDwidth|BWIDth:PLL AUTO | HIGH | MEDium | LOW
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="PLL_Bandwidth">
        /// This control defines the bandwidth of the main PLL of the instrument synthesizer.
        /// 
        /// Valid Values:
        /// RSESCI_PLL_BWIDTH_AUTO  (0) - Auto
        /// RSESCI_PLL_BWIDTH_HIGH  (1) - High
        /// RSESCI_PLL_BWIDTH_MED   (2) - Medium
        /// RSESCI_PLL_BWIDTH_LOW   (3) - Low
        /// 
        /// Default Value: RSESCI_PLL_BWIDTH_AUTO (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetMainPLLBandwidth(int Window, int PLL_Bandwidth)
        {
            int pInvokeResult = PInvoke.SetMainPLLBandwidth(this._handle, Window, PLL_Bandwidth);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the bandwidth of the main PLL of the instrument synthesizer.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]BANDwidth|BWIDth:PLL?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="PLL_Bandwidth">
        /// This control returns the bandwidth of the main PLL of the instrument synthesizer.
        /// 
        /// Valid Values:
        /// RSESCI_PLL_BWIDTH_WIDE   (0) - Wide (High)
        /// RSESCI_PLL_BWIDTH_NORMAL (1) - Normal (Medium)
        /// RSESCI_PLL_BWIDTH_NARROW (2) - Narrow (Low)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetMainPLLBandwidth(int Window, out int PLL_Bandwidth)
        {
            int pInvokeResult = PInvoke.GetMainPLLBandwidth(this._handle, Window, out PLL_Bandwidth);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function controls the PLL bandwidth used to synchronize with the external reference signal and is available only with an external reference frequency of 10 MHz. By default, it is set to NORMal.
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]ROSCillator:EXTernal:PLL
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="PLL_Bandwidth">
        /// This control select the PLL bandwidth used to synchronize with the external reference signal and is available only with an external reference frequency of 10 MHz.
        /// 
        /// Valid Values:
        /// RSESCI_VAL_NORM (0) - Normal
        /// RSESCI_VAL_WIDE (1) - Wide
        /// 
        /// Default Value: RSESCI_NORM (0)
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetPLLBandwidthSynchronization(int Window, int PLL_Bandwidth)
        {
            int pInvokeResult = PInvoke.SetPLLBandwidthSynchronization(this._handle, Window, PLL_Bandwidth);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the PLL bandwidth used to synchronize with the external reference signal and is available only with an external reference frequency of 10 MHz. By default, it is set to NORMal.
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]ROSCillator:EXTernal:PLL?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="PLL_Bandwidth">
        /// This control returns the selected PLL bandwidth used to synchronize with the external reference signal and is available only with an external reference frequency of 10 MHz.
        /// 
        /// Valid Values:
        /// RSESCI_VAL_NORM (0) - Normal
        /// RSESCI_VAL_WIDE (1) - Wide
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetPLLBandwidthSynchronization(int Window, out int PLL_Bandwidth)
        {
            int pInvokeResult = PInvoke.GetPLLBandwidthSynchronization(this._handle, Window, out PLL_Bandwidth);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function specifies the FFT filter.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]BANDwidth|BWIDth:FFT WIDE | AUTO | NARR
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="FFT_Filter_Mode">
        /// This control specifies the FFT filter mode.
        /// 
        /// Valid Values:
        /// RSESCI_FFT_FILTER_AUTO  (0) - Auto
        /// RSESCI_FFT_FILTER_WIDE  (1) - Wide
        /// RSESCI_FFT_FILTER_NARR  (2) - Narrow
        /// 
        /// Default Value: RSESCI_FFT_FILTER_AUTO (0)
        /// 
        /// Note(s):
        /// 
        /// (1) WIDE: The FFT filters with the greater partial span (this is the span which is covered with one FFT analysis) are used always.
        /// 
        /// (2) AUTO: The firmware decides whether to use wide or narrow filters to achieve the best performance of the measurement.
        /// 
        /// (3) NARROW: The FFT filters with the smaller partial span are used, this allows measurements near a carrier with reduced reference level, because of a more narrow analog prefilter.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetFFTFilterMode(int Window, int FFT_Filter_Mode)
        {
            int pInvokeResult = PInvoke.SetFFTFilterMode(this._handle, Window, FFT_Filter_Mode);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the FFT filter.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]BANDwidth|BWIDth:FFT?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="FFT_Filter_Mode">
        /// This control returns the FFT filter mode.
        /// 
        /// Valid Values:
        /// RSESCI_FFT_FILTER_AUTO  (0) - Auto
        /// RSESCI_FFT_FILTER_WIDE  (1) - Wide
        /// RSESCI_FFT_FILTER_NARR  (2) - Narrow
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetFFTFilterMode(int Window, out int FFT_Filter_Mode)
        {
            int pInvokeResult = PInvoke.GetFFTFilterMode(this._handle, Window, out FFT_Filter_Mode);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the resolution bandwidth of the IF analysis. Digitally implemented resolution filters from 10 Hz to 100 kHz in steps of 1, 3, and 10 are available (each with 6 dB bandwidths). The reliable value range depends on the selected frequency span of the IF analysis.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]BANDwidth|BWIDth:IF
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="IF_Bandwidth">
        /// This control sets the resolution bandwidth of the IF analysis.
        /// 
        /// Valid Range: 10.0 to 100.0e3 Hz
        /// 
        /// Default Value: 3.0e3 Hz
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetIFBandwidth(int Window, double IF_Bandwidth)
        {
            int pInvokeResult = PInvoke.SetIFBandwidth(this._handle, Window, IF_Bandwidth);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function gets the resolution bandwidth of the IF analysis. 
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]BANDwidth|BWIDth:IF?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="IF_Bandwidth">
        /// This control gets the resolution bandwidth of the IF analysis.
        /// 
        /// Valid Range: 10.0 to 100.0e3 Hz
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetIFBandwidth(int Window, out double IF_Bandwidth)
        {
            int pInvokeResult = PInvoke.GetIFBandwidth(this._handle, Window, out IF_Bandwidth);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the demodulation bandwidth of the instrument for analog demodulation. The required sampling rate is automatically set depending on the selected demodulation bandwidth. The available demodulation bandwidths are determined by the existing sampling rates.
        /// 
        /// Note(s):
        /// 
        /// (1) This command is only available with option R&amp;S FS-K7 (FM Demodulator).
        /// 
        /// Mode(s):
        /// Frequency Modulation 
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]BANDwidth|BWIDth:DEMod &lt;numeric_value&gt;
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Demodulation_Bandwidth">
        /// This control defines the demodulation bandwidth of the instrument for analog demodulation.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: 5.0e6 Hz
        /// 
        /// Note(s):
        /// 
        /// (1)
        /// Rounded            Sampling 
        /// demodulation       rate
        /// bandwidth
        /// 
        ///   10 MHz            32 MHz
        ///    8 MHz            16 MHz
        ///    5 MHz             8 MHz
        ///    3 MHz             4 MHz
        ///  1.6 MHz             2 MHz
        ///  800 kHz             1 MHz
        ///  400 kHz           500 kHz
        ///  200 kHz           250 kHz
        ///  100 kHz           125 kHz
        ///   50 kHz          62.5 kHz
        ///   25 kHz         31.25 kHz
        /// 12.5 kHz        15.625 kHz
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetDemodulationBandwidth(int Window, double Demodulation_Bandwidth)
        {
            int pInvokeResult = PInvoke.SetDemodulationBandwidth(this._handle, Window, Demodulation_Bandwidth);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the demodulation bandwidth of the instrument for analog demodulation. 
        /// 
        /// Note(s):
        /// 
        /// (1) This command is only available with option R&amp;S FS-K7 (FM Demodulator).
        /// 
        /// Mode(s):
        /// Frequency Modulation 
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]BANDwidth|BWIDth:DEMod?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Demodulation_Bandwidth">
        /// This control returns the demodulation bandwidth of the instrument for analog demodulation.
        /// 
        /// Valid Range: not checked
        /// 
        /// Note(s):
        /// 
        /// (1)
        /// Rounded            Sampling 
        /// demodulation       rate
        /// bandwidth
        /// 
        ///   10 MHz            32 MHz
        ///    8 MHz            16 MHz
        ///    5 MHz             8 MHz
        ///    3 MHz             4 MHz
        ///  1.6 MHz             2 MHz
        ///  800 kHz             1 MHz
        ///  400 kHz           500 kHz
        ///  200 kHz           250 kHz
        ///  100 kHz           125 kHz
        ///   50 kHz          62.5 kHz
        ///   25 kHz         31.25 kHz
        /// 12.5 kHz        15.625 kHz
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetDemodulationBandwidth(int Window, out double Demodulation_Bandwidth)
        {
            int pInvokeResult = PInvoke.GetDemodulationBandwidth(this._handle, Window, out Demodulation_Bandwidth);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the receiver IF bandwidth or, in SPECTRUM mode, the analyzer's resolution bandwidth. If the resolution bandwidth is modified in SPECTRUM mode, the coupling to the span is automatically switched off. If the resolution bandwidth is modified in FM DEMOD mode, the coupling to the demodulation bandwidth is automatically switched off.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]BANDwidth|BWIDth[:RESolution]
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Resolution_Bandwidth">
        /// This control sets the resolution bandwidth value.
        /// 
        /// Valid Range: 10.0 to 10.0E6 Hz
        /// 
        /// Default Value: 100.0e3 Hz
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetResolutionBandwidth(int Window, double Resolution_Bandwidth)
        {
            int pInvokeResult = PInvoke.SetResolutionBandwidth(this._handle, Window, Resolution_Bandwidth);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the receiver IF bandwidth or, in SPECTRUM mode, the analyzer's resolution bandwidth.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]BANDwidth|BWIDth[:RESolution]?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Resolution_Bandwidth">
        /// This control returns the value of resolution bandwidth.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetResolutionBandwidth(int Window, out double Resolution_Bandwidth)
        {
            int pInvokeResult = PInvoke.GetResolutionBandwidth(this._handle, Window, out Resolution_Bandwidth);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// In RECEIVER mode with activated quasi peak detector, this function either automatically couples the IF bandwidth of the receiver to the frequency range or cancels the coupling.
        /// In SPECTRUM mode, this function either automatically couples the resolution bandwidth of the instrument to the span or cancels the coupling.
        /// In FM DEMOD mode, this function either automatically couples the IF bandwidth of the instrument to the demodulation bandwidth or cancels the coupling (BWRBW = 10 * BWDemod).
        /// In FM DEMOD mode, the coupling refers to the demodulation bandwidth (BWRBW = 10 * BWDemod).
        /// The automatic coupling adapts the resolution bandwidth to the currently set frequency span according to the relationship between frequency span and resolution bandwidth. The 6 dB bandwidths 200 Hz, 9 kHz and 120 kHz and the channel filters available from Version 1.10 are not set by the automatic coupling.
        /// The ratio resolution bandwidth/span can be modified with the function rsesci_SetResolutionBandwidthCouplingRatio().
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis (frequency domain only)
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]BANDwidth|BWIDth[:RESolution]:AUTO ON | OFF
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Auto_Resolution_Bandwidth">
        /// This control enables or disables automatic setting of resolution bandwidth.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_TRUE  (1)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetResolutionBandwidthAuto(int Window, bool Auto_Resolution_Bandwidth)
        {
            int pInvokeResult = PInvoke.SetResolutionBandwidthAuto(this._handle, Window, System.Convert.ToUInt16(Auto_Resolution_Bandwidth));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the setting of auto resolution bandwidth.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis (frequency domain only)
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]BANDwidth|BWIDth[:RESolution]:AUTO?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Auto_Resolution_Bandwidth">
        /// This control returns the setting of auto resolution bandwidth.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetResolutionBandwidthAuto(int Window, out bool Auto_Resolution_Bandwidth)
        {
            ushort Auto_Resolution_BandwidthAsUShort;
            int pInvokeResult = PInvoke.GetResolutionBandwidthAuto(this._handle, Window, out Auto_Resolution_BandwidthAsUShort);
            Auto_Resolution_Bandwidth = System.Convert.ToBoolean(Auto_Resolution_BandwidthAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the ratio resolution bandwidth (Hz) / span (Hz). The ratio to be entered is reciprocal to the ratio span/RBW used in manual control.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]BANDwidth|BWIDth[:RESolution]:RATio
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Ratio">
        /// This control sets the ratio resolution bandwidth (Hz) / span (Hz).
        /// 
        /// Valid Range: 0.0001 to 1
        /// 
        /// Default Value: 0.02
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetResolutionBandwidthCouplingRatio(int Window, double Ratio)
        {
            int pInvokeResult = PInvoke.SetResolutionBandwidthCouplingRatio(this._handle, Window, Ratio);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the ratio resolution bandwidth (Hz) / span (Hz).
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]BANDwidth|BWIDth[:RESolution]:RATio?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Ratio">
        /// This control returns the ratio resolution bandwidth (Hz) / span (Hz).
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetResolutionBandwidthCouplingRatio(int Window, out double Ratio)
        {
            int pInvokeResult = PInvoke.GetResolutionBandwidthCouplingRatio(this._handle, Window, out Ratio);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches the filter type for the resolution bandwidth between "normal" analog or FIR filters in 1, 3, 10 steps and the FFT filtering for bandwidths &lt;100 kHz.
        /// 
        /// FFT filters are only suitable for periodic signals, and they are only available for span &gt; 0 Hz.
        /// 
        /// The filter type 'Pulse' selects the 6 dB bandwidths for EMI measurements rather than the 3 dB bandwidths for spectrum analysis, which is selected with type 'Noise'.
        /// 
        /// Steep-edged channel filters and filters with RRC (Root Raised Cosine) characteristic are available.
        /// 
        /// The possible combinations of filter type and filter bandwidth are listed in the table "List of available channel filters" in chapter 4, section "Setting Bandwidths and Sweep Time - Key BW".
        /// 
        /// Note(s):
        /// 
        /// (1) When changing the filter type, the next larger filter bandwidth is selected if the same filter bandwidth is not available for the new filter type.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]BANDwidth|BWIDth[:RESolution]:TYPE NORMal | FFT | CFILter | RRC | NOISe | PULSe
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Filter_Type">
        /// This control switches the filter type for the resolution bandwidth.
        /// 
        /// Valid Values:
        /// RSESCI_FILTER_NORMAL (0) - Normal
        /// RSESCI_FILTER_FFT    (1) - FFT
        /// RSESCI_FILTER_CFIL   (2) - Channel Filter
        /// RSESCI_FILTER_RRC    (3) - RRC (Root Raised Cosine)
        /// RSESCI_FILTER_NOISE  (4) - Noise
        /// RSESCI_FILTER_PULSE  (5) - Pulse
        /// 
        /// Default Value: RSESCI_FILTER_NORMAL (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetResolutionBandwidthFilterType(int Window, int Filter_Type)
        {
            int pInvokeResult = PInvoke.SetResolutionBandwidthFilterType(this._handle, Window, Filter_Type);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the filter type for the resolution bandwidth.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]BANDwidth|BWIDth[:RESolution]:TYPE?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Filter_Type">
        /// This control returns the filter type for the resolution bandwidth.
        /// 
        /// Valid Values:
        /// RSESCI_FILTER_NORMAL (0) - Normal
        /// RSESCI_FILTER_FFT    (1) - FFT
        /// RSESCI_FILTER_CFIL   (2) - Channel Filter
        /// RSESCI_FILTER_RRC    (3) - RRC (Root Raised Cosine)
        /// RSESCI_FILTER_NOISE  (4) - Noise
        /// RSESCI_FILTER_PULSE  (5) - Pulse
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetResolutionBandwidthFilterType(int Window, out int Filter_Type)
        {
            int pInvokeResult = PInvoke.GetResolutionBandwidthFilterType(this._handle, Window, out Filter_Type);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the instrument's video bandwidth. Bandwidths from 10 Hz to 10 MHz in 1, 3, 10 steps are available. The function is not available if FFT filtering is switched on and the set bandwidth is &lt;= 30 kHz or if the quasi-peak detector is switched on.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]BANDwidth|BWIDth:VIDeo
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Video_Bandwidth">
        /// This control sets the instrument's video bandwidth.
        /// 
        /// Valid Range: 1.0 to 10.0E6 Hz
        /// 
        /// Default Value: 300.0e3 Hz
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetVideoBandwidth(int Window, double Video_Bandwidth)
        {
            int pInvokeResult = PInvoke.SetVideoBandwidth(this._handle, Window, Video_Bandwidth);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the instrument's video bandwidth.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]BANDwidth|BWIDth:VIDeo?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Video_Bandwidth">
        /// This control returns the instrument's video bandwidth.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetVideoBandwidth(int Window, out double Video_Bandwidth)
        {
            int pInvokeResult = PInvoke.GetVideoBandwidth(this._handle, Window, out Video_Bandwidth);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function either automatically couples the instrument's video bandwidth to the resolution bandwidth or cancels the coupling.
        /// 
        /// The ratio video bandwidth/resolution bandwidth can be modified with the function SetVideoBandwidthCouplingRatio().
        /// 
        /// Note(s):
        /// 
        /// (1) In receiver mode, the video bandwidth is set to ten times the resolution bandwidth.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]BANDwidth|BWIDth:VIDeo:AUTO ON | OFF
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Auto_Video_Bandwidth">
        /// This control enables or disables automatic setting of video bandwidth.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_TRUE  (1)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetVideoBandwidthAuto(int Window, bool Auto_Video_Bandwidth)
        {
            int pInvokeResult = PInvoke.SetVideoBandwidthAuto(this._handle, Window, System.Convert.ToUInt16(Auto_Video_Bandwidth));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns whether automatically couples the instrument's video bandwidth to the resolution bandwidth or cancels the coupling.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]BANDwidth|BWIDth:VIDeo:AUTO?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Auto_Video_Bandwidth">
        /// This control returns automatic setting of video bandwidth.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetVideoBandwidthAuto(int Window, out bool Auto_Video_Bandwidth)
        {
            ushort Auto_Video_BandwidthAsUShort;
            int pInvokeResult = PInvoke.GetVideoBandwidthAuto(this._handle, Window, out Auto_Video_BandwidthAsUShort);
            Auto_Video_Bandwidth = System.Convert.ToBoolean(Auto_Video_BandwidthAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the ratio video bandwidth (Hz) / resolution bandwidth (Hz). The ratio to be entered is reciprocal to the ratio RBW/VBW used in manual control.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]BANDwidth|BWIDth:VIDeo:RATio
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Ratio">
        /// This control defines the ratio video bandwidth (Hz) / resolution bandwidth (Hz).
        /// 
        /// Valid Range: 0.01 to 1000
        /// 
        /// Default Value: 3.0
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetVideoBandwidthCouplingRatio(int Window, double Ratio)
        {
            int pInvokeResult = PInvoke.SetVideoBandwidthCouplingRatio(this._handle, Window, Ratio);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the value of ratio video bandwidth (Hz) / resolution bandwidth (Hz).
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]BANDwidth|BWIDth:VIDeo:RATio?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Ratio">
        /// This control returns the ratio video bandwidth (Hz) / resolution bandwidth (Hz).
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetVideoBandwidthCouplingRatio(int Window, out double Ratio)
        {
            int pInvokeResult = PInvoke.GetVideoBandwidthCouplingRatio(this._handle, Window, out Ratio);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the position of the video filter in the signal path, provided that the resolution bandwidth is &lt;= 100 kHz.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]BANDwidth|BWIDth:VIDeo:TYPE LINear | LOGarithmic
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Bandwidth_Mode">
        /// This control selects the position of the video filter in the signal path, provided that the resolution bandwidth is &lt;=100 kHz.
        /// 
        /// The essential difference between the two modes is the transient response at falling signal edges.
        /// 
        /// Valid Values:
        /// RSESCI_LIN (0) - Linear
        /// RSESCI_LOG (1) - Logarithmic
        /// 
        /// Default Value: RSESCI_LIN (0)
        /// 
        /// Note(s):
        /// 
        /// (1) If Linear is selected, the video filter is connected ahead of the logarithmic amplifier. The measurement with logarithmic level scaling yields a much "flatter" falling edge than Logarithmic. This behaviour is due to the conversion of linear power into logarithmic level. If the linear power is halved, the level decreases by only 3 dB.
        /// 
        /// (2) If Logarithmic is selected, the video filter follows the logarithmic amplifier.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetVideoBandwidthMode(int Window, int Bandwidth_Mode)
        {
            int pInvokeResult = PInvoke.SetVideoBandwidthMode(this._handle, Window, Bandwidth_Mode);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the position of the video filter in the signal path.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]BANDwidth|BWIDth:VIDeo:TYPE?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Bandwidth_Mode">
        /// This control returns the position of the video filter in the signal path.
        /// 
        /// Valid Values:
        /// RSESCI_LIN (0) - Linear
        /// RSESCI_LOG (1) - Logarithmic
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetVideoBandwidthMode(int Window, out int Bandwidth_Mode)
        {
            int pInvokeResult = PInvoke.GetVideoBandwidthMode(this._handle, Window, out Bandwidth_Mode);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// When a scan is automatically interrupted by a transducer interrupt point, this function continues the measurement at the current receiver frequency. The function is useful especially for trace functions MAXHold, MINHold and AVERage if the previous
        /// results are not to be cleared with Sweep Count &gt; 0 or Average Count &gt; 0 on restarting the measurement (INIT:IMMediate resets the previous results on restarting the measurement). The single-sweep mode is automatically switched on. In the continuous-sweep mode, synchronization to the sweep end is not possible since the overall measurement "never" ends.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// INITiate&lt;1|2&gt;:CONMeas
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int ContinueSingleSweep(int Window)
        {
            int pInvokeResult = PInvoke.ContinueSingleSweep(this._handle, Window);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the number of scans started with single scan in receiver mode. In analyzer mode, this function queries the number of sweeps started with single sweep.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SWEep:COUNt:CURRent?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Started_Scans">
        /// This control returns the number of started scans.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetNumberStartedSweeps(int Window, out int Started_Scans)
        {
            int pInvokeResult = PInvoke.GetNumberStartedSweeps(this._handle, Window, out Started_Scans);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function determines whether the trigger system is continuously initiated (continuous) or performs single measurements (single).
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// INITiate&lt;1|2&gt;:CONTinuous ON | OFF
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Continuous_Sweep">
        /// This control determines whether the trigger system is continuously initiated (continuous) or performs single measurements (single).
        /// 
        /// Valid Values:
        /// RSESCI_SINGLE_SWEEP     (0) - Single
        /// RSESCI_CONTINUOUS_SWEEP (1) - Continuous
        /// 
        /// Default Value: RSESCI_CONTINUOUS_SWEEP (1)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetSweepMode(int Window, int Continuous_Sweep)
        {
            int pInvokeResult = PInvoke.SetSweepMode(this._handle, Window, Continuous_Sweep);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns whether the trigger system is continuously initiated (continuous) or performs single measurements (single).
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// INITiate&lt;1|2&gt;:CONTinuous?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Continuous_Sweep">
        /// This control returns whether the trigger system is continuously initiated (continuous) or performs single measurements (single).
        /// 
        /// Valid Values:
        /// RSESCI_SINGLE_SWEEP     (0) - Single
        /// RSESCI_CONTINUOUS_SWEEP (1) - Continuous
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetSweepMode(int Window, out int Continuous_Sweep)
        {
            int pInvokeResult = PInvoke.GetSweepMode(this._handle, Window, out Continuous_Sweep);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the scan or sweep time.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SWEep:TIME
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Sweep_Time">
        /// This control defines the scan or sweep time.
        /// 
        /// Valid Range: 1.0e-6 to 16000 s
        /// 
        /// Default Value: 10.0 s
        /// 
        /// Note(s):
        /// 
        /// (1) For receiver single scan, the available time value range is 100 us to 100 s in 1-2-5 steps.
        /// 
        /// (2) In analyzer mode, the available time values are different in the frequency domain (2.5 ms to 16000s with span &gt; 0) and in the time domain (1 us to 16000 s with span = 0).
        /// 
        /// (3) If Sweep Time is directly programmed, automatic coupling to resolution bandwidth and video bandwidth is switched off.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetSweepTime(int Window, double Sweep_Time)
        {
            int pInvokeResult = PInvoke.SetSweepTime(this._handle, Window, Sweep_Time);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the scan or sweep time
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SWEep:TIME?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Sweep_Time">
        /// This control returns the scan or sweep time.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetSweepTime(int Window, out double Sweep_Time)
        {
            int pInvokeResult = PInvoke.GetSweepTime(this._handle, Window, out Sweep_Time);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function controls the automatic coupling of the sweep time to the frequency span and bandwidth settings.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SWEep:TIME:AUTO ON | OFF
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Auto_Sweep_Time">
        /// This control enables or disables automatic coupling of the sweep time to the frequency span and bandwidth settings.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_TRUE  (1)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetSweepTimeAuto(int Window, bool Auto_Sweep_Time)
        {
            int pInvokeResult = PInvoke.SetSweepTimeAuto(this._handle, Window, System.Convert.ToUInt16(Auto_Sweep_Time));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the state of automatic coupling of the sweep time to the frequency span and bandwidth settings.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SWEep:TIME:AUTO?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Auto_Sweep_Time">
        /// This control returns the state of automatic coupling of the sweep time to the frequency span and bandwidth settings.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetSweepTimeAuto(int Window, out bool Auto_Sweep_Time)
        {
            ushort Auto_Sweep_TimeAsUShort;
            int pInvokeResult = PInvoke.GetSweepTimeAuto(this._handle, Window, out Auto_Sweep_TimeAsUShort);
            Auto_Sweep_Time = System.Convert.ToBoolean(Auto_Sweep_TimeAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the number of scans started with single scan in receiver mode. In analyzer mode, this function defines the number of sweeps started with single sweep, which are
        /// used for calculating the average or maximum value. In average mode, the value 0 defines a continuous averaging of measurement data over 10 sweeps.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SWEep:COUNt
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Sweep_Count">
        /// This control defines the number of scans started with single scan in receiver mode. In analyzer mode, this control defines the number of sweeps started with single sweep, which are
        /// used for calculating the average or maximum value. In average mode, the value 0 defines a continuous averaging of measurement data over 10 sweeps.
        /// 
        /// Valid Range: 0 to 32767
        /// 
        /// Default Value: 0
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetSweepCount(int Window, int Sweep_Count)
        {
            int pInvokeResult = PInvoke.SetSweepCount(this._handle, Window, Sweep_Count);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the number of scans started with single scan in receiver mode. In analyzer mode, this function returns the number of sweeps started with single sweep, which are
        /// used for calculating the average or maximum value.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SWEep:COUNt?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Sweep_Count">
        /// This control returns the number of scans started with single scan in receiver mode.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetSweepCount(int Window, out int Sweep_Count)
        {
            int pInvokeResult = PInvoke.GetSweepCount(this._handle, Window, out Sweep_Count);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the number of measurement points for one sweep run.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SWEep:POINts
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Sweep_Points">
        /// This control defines the number of measurement points for one sweep run.
        /// 
        /// Valid Values:
        /// 125, 251, 501, 1001, 2001, 4001, 8001 - for R&amp;S ESCI
        /// 155, 313, 625, 1251, 1999, 2501, 5001, 10001, 20001, 30001 - for R&amp;S ESU
        /// 
        /// Default Value: 501
        /// 
        /// Note(s):
        /// 
        /// (1) Passed value will be coerced to nearest valid value.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetSweepPoints(int Window, int Sweep_Points)
        {
            int pInvokeResult = PInvoke.SetSweepPoints(this._handle, Window, Sweep_Points);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the number of measurement points for one sweep run.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SWEep:POINts?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Sweep_Points">
        /// This control returns the number of measurement points for one sweep run.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetSweepPoints(int Window, out int Sweep_Points)
        {
            int pInvokeResult = PInvoke.GetSweepPoints(this._handle, Window, out Sweep_Points);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the behavior of the display during a single sweep.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// INITiate&lt;1|2&gt;:DISPlay ON | OFF
        /// </summary>
        /// <param name="Display_State">
        /// This control configures the behavior of the display during a single sweep. Value 'Off' means that the display is switched off during the measurement, 'On' means that the display is switched on during the measurement.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_TRUE  (1)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetSingleSweepDisplay(bool Display_State)
        {
            int pInvokeResult = PInvoke.SetSingleSweepDisplay(this._handle, System.Convert.ToUInt16(Display_State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the state of the display during a single sweep.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// INITiate:DISPlay?
        /// </summary>
        /// <param name="Display_State">
        /// This control returns the state of the display during a single sweep.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetSingleSweepDisplay(out bool Display_State)
        {
            ushort Display_StateAsUShort;
            int pInvokeResult = PInvoke.GetSingleSweepDisplay(this._handle, out Display_StateAsUShort);
            Display_State = System.Convert.ToBoolean(Display_StateAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches between linear, logarthmic and automatically selected linear step size in the receiver.
        /// 
        /// In the receiver mode, this parameter has no impact on the graphical representation of the frequency axis.
        /// 
        /// In the analyzer, this function toggles between linear and logarithmic sweep. The frequency axis is represented linearly or logarithmically depending on the sweep.
        /// 
        /// Note(s):
        /// 
        /// (1) The parameters APULse and ACW are available only with instrument R&amp;S ESU and option R&amp;S ESU-K53.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SWEep:SPACing LINear | LOGarithmic | AUTO | APULse | ACW
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Sweep_Spacing">
        /// This control switches between linear, logarthmic and automatically selected linear step size in the receiver.
        /// 
        /// Valid Values:
        /// RSESCI_LIN  (0) - Linear
        /// RSESCI_LOG  (1) - Logarithmic
        /// RSESCI_AUTO (2) - Auto
        /// RSESCI_APUL (3) - Auto Pulse
        /// RSESCI_ACW  (4) - Auto CW
        /// 
        /// Default Value: RSESCI_LIN (0)
        /// 
        /// Note(s):
        /// 
        /// (1) The parameters RSESCI_APUL and RSESCI_ACW are available only with instrument R&amp;S ESU and option R&amp;S ESU-K53.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetSweepSpacing(int Window, int Sweep_Spacing)
        {
            int pInvokeResult = PInvoke.SetSweepSpacing(this._handle, Window, Sweep_Spacing);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns sweep spacing mode.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SWEep:SPACing?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Sweep_Spacing">
        /// This control returns value of sweep spacing mode.
        /// 
        /// Valid Values:
        /// RSESCI_LIN  (0) - Linear
        /// RSESCI_LOG  (1) - Logarithmic
        /// RSESCI_AUTO (2) - Auto
        /// RSESCI_APUL (3) - Auto Pulse
        /// RSESCI_ACW  (4) - Auto CW
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetSweepSpacing(int Window, out int Sweep_Spacing)
        {
            int pInvokeResult = PInvoke.GetSweepSpacing(this._handle, Window, out Sweep_Spacing);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function aborts a current measurement and resets the trigger system.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// ABORt
        /// </summary>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int Abort()
        {
            int pInvokeResult = PInvoke.Abort(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function interrupts a current scan measurement.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// HOLD
        /// </summary>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int Hold()
        {
            int pInvokeResult = PInvoke.Hold(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function initiates a new sweep in the indicated measurement window.
        /// In receiver mode with 'Single' selected, the R&amp;S ESCI performs a single scan and stops at the end frequency. With 'Continuous' selected, the scan is performed continuously until it is deliberately stopped.
        /// In analyzer mode, with Sweep Count &gt; 0 or Average Count &gt; 0, this means a restart of the indicated number of measurements. With trace functions 'Max Hold', 'Min Hold' and 'Average', the previous results are reset on restarting the measurement.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// INITiate&lt;1|2&gt;[:IMMediate]
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SendTrigger(int Window)
        {
            int pInvokeResult = PInvoke.SendTrigger(this._handle, Window);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function triggers all actions waiting for a trigger event in the selected window and waits for operation completed (OPC) before returning the status code.
        /// 
        /// Remote-control command(s):
        /// INITiate&lt;1|2&gt;[:IMMediate]
        /// *OPC?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Timeout">
        /// Sets the timeout for the triggering routine to be finished. If the length of time required for triggering exceeds the timeout value, then the function will return with a timeout error.
        /// 
        /// Valid Range: 0 ms to 600000 ms
        /// 
        /// Default Value: 15000 ms
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SendTriggerWaitOPC(int Window, int Timeout)
        {
            int pInvokeResult = PInvoke.SendTriggerWaitOPC(this._handle, Window, Timeout);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the trigger source for the start of a sweep.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Note(s):
        /// 
        /// (1)The selection 'RF Power' or 'TV' is only available with option R&amp;S FSP-B6 (TV and RF Trigger).
        /// 
        /// (2)The selection 'AF', 'AM', 'AMR', 'FM' and 'PM' is only available with option R&amp;S FS-K7 (FM Demodulator).
        /// 
        /// Remote-control command(s):
        /// TRIGger&lt;1|2&gt;[:SEQuence]:SOURce IMMediate | EXTernal | VIDeo | IFPower | RFPower | TV | FM | AM | PM | AMR
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Trigger_Source">
        /// This control selects the trigger source for the start of a sweep.
        /// 
        /// Valid Values:
        /// RSESCI_TRIG_IMM     (0) - Immediate
        /// RSESCI_TRIG_EXT     (1) - External
        /// RSESCI_TRIG_VID     (2) - Video
        /// RSESCI_TRIG_IFP     (3) - IF Power
        /// RSESCI_TRIG_RFP     (4) - RF Power
        /// RSESCI_TRIG_TV      (5) - TV
        /// RSESCI_TRIG_FM      (6) - FM
        /// RSESCI_TRIG_AM      (7) - AM
        /// RSESCI_TRIG_PM      (8) - PM
        /// RSESCI_TRIG_AM_REL  (10) - AM Relative
        /// 
        /// Default Value: RSESCI_TRIG_IMM (0)
        /// 
        /// Note(s):
        /// 
        /// (1) Immediate - automatic triggering the next measurement at the end of the previous one. The value 'Immediate' corresponds to the FREE RUN setting.
        /// 
        /// (2) External - the next measurement is triggered by the signal at the external trigger input.
        /// 
        /// (3) Video - the next measurement is triggered by the detection of a signal at the video filter output (not available for FM DEMOD mode).
        /// 
        /// (4) IF Power - the next measurement is triggered by the detection of a signal at the instrument IF (10 MHz bandwidth)
        /// 
        /// (5) RF Power - the next measurement is triggered by the detection of a signal at the instrument RF (80 MHz bandwidth)
        /// 
        /// (6) TV - the next measurement is triggered by the detection of a TV signal according to the settings of the Trigger Video Subsystem (not available for FM DEMOD mode)
        /// 
        /// (7) FM - the next measurement is triggered by the detection of a FM modulated signal (AF is synonymous with FM)
        /// 
        /// (8) AM - the next measurement is triggered by the detection of a
        /// audio signal after AM demodulation
        /// 
        /// (9) PM - the next measurement is triggered by the detection of a
        /// audio signal after PM demodulation
        /// 
        /// (10) AM Relative - the next measurement is triggered by the detection of a audio signal after relative AM demodulation (%)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetTriggerSource(int Window, int Trigger_Source)
        {
            int pInvokeResult = PInvoke.SetTriggerSource(this._handle, Window, Trigger_Source);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the trigger source for the start of a sweep.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// TRIGger&lt;1|2&gt;[:SEQuence]:SOURce?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Trigger_Source">
        /// This control returns the trigger source for the start of a sweep.
        /// 
        /// Valid Values:
        /// RSESCI_TRIG_IMM     (0) - Immediate
        /// RSESCI_TRIG_EXT     (1) - External
        /// RSESCI_TRIG_VID     (2) - Video
        /// RSESCI_TRIG_IFP     (3) - IF Power
        /// RSESCI_TRIG_RFP     (4) - RF Power
        /// RSESCI_TRIG_TV      (5) - TV
        /// RSESCI_TRIG_FM      (6) - FM
        /// RSESCI_TRIG_AM      (7) - AM
        /// RSESCI_TRIG_PM      (8) - PM
        /// RSESCI_TRIG_AM_REL  (10) - AM Relative
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetTriggerSource(int Window, out int Trigger_Source)
        {
            int pInvokeResult = PInvoke.GetTriggerSource(this._handle, Window, out Trigger_Source);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the trigger level for selected signal.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// TRIGger&lt;1|2&gt;[:SEQuence]:LEVel[:EXTernal]
        /// TRIGger&lt;1|2&gt;[:SEQuence]:LEVel:AM
        /// TRIGger&lt;1|2&gt;[:SEQuence]:LEVel:FM
        /// TRIGger&lt;1|2&gt;[:SEQuence]:LEVel:PM
        /// TRIGger&lt;1|2&gt;[:SEQuence]:LEVel:IFPower
        /// TRIGger&lt;1|2&gt;[:SEQuence]:LEVel:RFPower
        /// TRIGger&lt;1|2&gt;[:SEQuence]:LEVel:VIDeo
        /// TRIGger&lt;1|2&gt;[:SEQuence]:LEVel:AM:REL
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Signal_Type">
        /// This control selects the signal type.
        /// 
        /// Valid Values:
        /// RSESCI_TRIG_EXT     (1) - External
        /// RSESCI_TRIG_VID     (2) - Video
        /// RSESCI_TRIG_IFP     (3) - IF Power
        /// RSESCI_TRIG_RFP     (4) - RF Power
        /// RSESCI_TRIG_FM      (6) - FM
        /// RSESCI_TRIG_AM      (7) - AM
        /// RSESCI_TRIG_PM      (8) - PM
        /// RSESCI_TRIG_AM_REL  (10) - AM Relative
        /// 
        /// Default Value: RSESCI_TRIG_VID (2)
        /// </param>
        /// <param name="Trigger_Level">
        /// This control sets the trigger level for selected signal type.
        /// 
        /// Valid Range:
        /// - For External   0.5   to   3.5   V
        /// - For Video      0.0   to 100.0   PCT
        /// - For IF Power -30.0   to -10.0   dBm
        /// - For RF Power -50.0   to -10.0   dBm
        /// - For FM       -10E6   to +10E6   Hz
        /// - For AM       -100.0  to +30.0   dBm
        /// - For PM       -1000.0 to +1000.0 RAD
        /// - For AM Relative 0.0  to 100.0   PCT
        /// 
        /// Default Value: 0.0
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetTriggerLevel(int Window, int Signal_Type, double Trigger_Level)
        {
            int pInvokeResult = PInvoke.SetTriggerLevel(this._handle, Window, Signal_Type, Trigger_Level);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the trigger level for selected signal.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// TRIGger&lt;1|2&gt;[:SEQuence]:LEVel[:EXTernal]?
        /// TRIGger&lt;1|2&gt;[:SEQuence]:LEVel:AM?
        /// TRIGger&lt;1|2&gt;[:SEQuence]:LEVel:AF?
        /// TRIGger&lt;1|2&gt;[:SEQuence]:LEVel:FM?
        /// TRIGger&lt;1|2&gt;[:SEQuence]:LEVel:PM?
        /// TRIGger&lt;1|2&gt;[:SEQuence]:LEVel:IFPower?
        /// TRIGger&lt;1|2&gt;[:SEQuence]:LEVel:RFPower?
        /// TRIGger&lt;1|2&gt;[:SEQuence]:LEVel:VIDeo?
        /// TRIGger&lt;1|2&gt;[:SEQuence]:LEVel:AM:REL?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Signal_Type">
        /// This control selects the signal type.
        /// 
        /// Valid Values:
        /// RSESCI_TRIG_EXT     (1) - External
        /// RSESCI_TRIG_VID     (2) - Video
        /// RSESCI_TRIG_IFP     (3) - IF Power
        /// RSESCI_TRIG_RFP     (4) - RF Power
        /// RSESCI_TRIG_FM      (6) - FM
        /// RSESCI_TRIG_AM      (7) - AM
        /// RSESCI_TRIG_PM      (8) - PM
        /// RSESCI_TRIG_AM_REL  (10) - AM Relative
        /// 
        /// Default Value: RSESCI_TRIG_VID (2)
        /// </param>
        /// <param name="Trigger_Level">
        /// This control returns the trigger level for selected signal type.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetTriggerLevel(int Window, int Signal_Type, out double Trigger_Level)
        {
            int pInvokeResult = PInvoke.GetTriggerLevel(this._handle, Window, Signal_Type, out Trigger_Level);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the length of the trigger delay.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// TRIGger&lt;1|2&gt;[:SEQuence]:HOLDoff
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Trigger_Delay">
        /// This control defines the length of the trigger delay.
        /// 
        /// Valid Range: -100.0 to 100.0 s
        /// 
        /// Default Value: 0.0 s
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetTriggerOffset(int Window, double Trigger_Delay)
        {
            int pInvokeResult = PInvoke.SetTriggerOffset(this._handle, Window, Trigger_Delay);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the length of the trigger delay.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// TRIGger&lt;1|2&gt;[:SEQuence]:HOLDoff?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Trigger_Delay">
        /// This control returns the length of the trigger delay.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetTriggerOffset(int Window, out double Trigger_Delay)
        {
            int pInvokeResult = PInvoke.GetTriggerOffset(this._handle, Window, out Trigger_Delay);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines whether the group delay of the bandwidth filters for the external trigger is compensated (ON) or not (OFF). If a burst signal is analyzed in the time domain mode, the rising edge will remain at the same time position if compensation is on and the bandwidth changes.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// TRIGger&lt;1|2&gt;[:SEQuence]:HOLDoff:ADJust:AUTO ON | OFF
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Offset_Adjust">
        /// This control defines whether the group delay of the bandwidth filters for the external trigger is compensated (ON) or not (OFF). 
        /// 
        /// Valid Range: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetTriggerOffsetAdjust(int Window, bool Offset_Adjust)
        {
            int pInvokeResult = PInvoke.SetTriggerOffsetAdjust(this._handle, Window, System.Convert.ToUInt16(Offset_Adjust));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the state of the group delay of the bandwidth filters for the external trigger is compensated (ON) or not (OFF). If a burst signal is analyzed in the time domain mode, the rising edge will remain at the same time position if compensation is on and the bandwidth changes.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// TRIGger&lt;1|2&gt;[:SEQuence]:HOLDoff:ADJust:AUTO?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Offset_Adjust">
        /// This control returns the state of the group delay of the bandwidth filters for the external trigger is compensated (ON) or not (OFF). 
        /// 
        /// Valid Range: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetTriggerOffsetAdjust(int Window, out bool Offset_Adjust)
        {
            ushort Offset_AdjustAsUShort;
            int pInvokeResult = PInvoke.GetTriggerOffsetAdjust(this._handle, Window, out Offset_AdjustAsUShort);
            Offset_Adjust = System.Convert.ToBoolean(Offset_AdjustAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the slope of the trigger signal. The selected trigger slope applies to all trigger signal sources.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// TRIGger&lt;1|2&gt;[:SEQuence]:SLOPe POSitive | NEGative
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Polarity">
        /// This control selects the slope of the trigger signal. The selected trigger slope applies to all trigger signal sources.
        /// 
        /// Valid Values:
        /// RSESCI_POL_POS (0) - Positive
        /// RSESCI_POL_NEG (1) - Negative
        /// 
        /// Default Value: RSESCI_POL_POS (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetTriggerSourcePolarity(int Window, int Polarity)
        {
            int pInvokeResult = PInvoke.SetTriggerSourcePolarity(this._handle, Window, Polarity);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the slope of the trigger signal. The selected trigger slope applies to all trigger signal sources.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// TRIGger&lt;1|2&gt;[:SEQuence]:SLOPe?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Polarity">
        /// This control returns the slope of the trigger signal. The selected trigger slope applies to all trigger signal sources.
        /// 
        /// Valid Values:
        /// RSESCI_POL_POS (0) - Positive
        /// RSESCI_POL_NEG (1) - Negative
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetTriggerSourcePolarity(int Window, out int Polarity)
        {
            int pInvokeResult = PInvoke.GetTriggerSourcePolarity(this._handle, Window, out Polarity);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches on/off the sweep control by an external gate signal. If the external gate is selected the trigger source is automatically switched to 'External' as well.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SWEep:EGATe ON | OFF
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Gated_Trigger">
        /// This control switches on/off the sweep control by an external gate signal.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetGatedTrigger(int Window, bool Gated_Trigger)
        {
            int pInvokeResult = PInvoke.SetGatedTrigger(this._handle, Window, System.Convert.ToUInt16(Gated_Trigger));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns setting of the sweep control by an external gate signal.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SWEep:EGATe?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Gated_Trigger">
        /// This control returns setting of the sweep control by an external gate signal.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetGatedTrigger(int Window, out bool Gated_Trigger)
        {
            ushort Gated_TriggerAsUShort;
            int pInvokeResult = PInvoke.GetGatedTrigger(this._handle, Window, out Gated_TriggerAsUShort);
            Gated_Trigger = System.Convert.ToBoolean(Gated_TriggerAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function toggles between external gate signal and IF power signal as a signal source for the gate mode. If an IF power signal is used, the gate is opened as soon as a signal at &gt; -20 dBm is detected within the IF path bandwidth (10 MHz).
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Note(s):
        /// 
        /// (1) Selection RFPower is only available with option R&amp;S FSP-B6 (TV and RF Trigger). 
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SWEep:EGATe:SOURce EXTernal | IFPower | RFPower
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Trigger_Source">
        /// This control toggles between external gate signal and IF power signal as a signal source for the gate mode.
        /// 
        /// Valid Values:
        /// RSESCI_TRIG_EXT (1) - External
        /// RSESCI_TRIG_IFP (3) - IF Power
        /// RSESCI_TRIG_RFP (4) - RF Power
        /// 
        /// Default Value: RSESCI_TRIG_IFP (3)
        /// 
        /// Note(s):
        /// 
        /// (1) Selection RFPower is only available with option R&amp;S FSP-B6 (TV and RF Trigger).
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetGatedTriggerSource(int Window, int Trigger_Source)
        {
            int pInvokeResult = PInvoke.SetGatedTriggerSource(this._handle, Window, Trigger_Source);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns selected signal source for the gate mode.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SWEep:EGATe:SOURce?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Trigger_Source">
        /// This control returns selected signal source for the gate mode.
        /// 
        /// Valid Values:
        /// RSESCI_TRIG_EXT (1) - External
        /// RSESCI_TRIG_IFP (3) - IF Power
        /// RSESCI_TRIG_RFP (4) - RF Power
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetGatedTriggerSource(int Window, out int Trigger_Source)
        {
            int pInvokeResult = PInvoke.GetGatedTriggerSource(this._handle, Window, out Trigger_Source);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the type of triggering (level or edge) by the external gate signal.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SWEep:EGATe:TYPE LEVel | EDGE
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Trigger_Mode">
        /// This control sets the type of triggering (level or edge) by the external gate signal.
        /// 
        /// Valid Values:
        /// RSESCI_TRIG_LEVEL (0) - Level
        /// RSESCI_TRIG_EDGE  (1) - Edge
        /// 
        /// Default Value: RSESCI_TRIG_LEVEL (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetGatedTriggerMode(int Window, int Trigger_Mode)
        {
            int pInvokeResult = PInvoke.SetGatedTriggerMode(this._handle, Window, Trigger_Mode);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the type of triggering (level or edge) by the external gate signal.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SWEep:EGATe:TYPE?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Trigger_Mode">
        /// This control returns the type of triggering (level or edge) by the external gate signal. 
        /// 
        /// Valid Values:
        /// RSESCI_TRIG_LEVEL (0) - Level
        /// RSESCI_TRIG_EDGE  (1) - Edge
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetGatedTriggerMode(int Window, out int Trigger_Mode)
        {
            int pInvokeResult = PInvoke.GetGatedTriggerMode(this._handle, Window, out Trigger_Mode);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function determines the polarity of the external gate signal. The setting is valid both for the edge of an edge-triggered signal and the level of a level-triggered signal.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SWEep:EGATe:POLarity POSitive | NEGative
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Trigger_Polarity">
        /// This control determines the polarity of the external gate signal. The setting is valid both for the edge of an edge-triggered signal and the level of a level-triggered signal.
        /// 
        /// Valid Values:
        /// RSESCI_POS (0) - Positive
        /// RSESCI_NEG (1) - Negative
        /// 
        /// Default Value: RSESCI_POS (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetGatedTriggerPolarity(int Window, int Trigger_Polarity)
        {
            int pInvokeResult = PInvoke.SetGatedTriggerPolarity(this._handle, Window, Trigger_Polarity);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the polarity of the external gate signal.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SWEep:EGATe:POLarity?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Trigger_Polarity">
        /// This control returns the polarity of the external gate signal.
        /// 
        /// Valid Values:
        /// RSESCI_POS (0) - Positive
        /// RSESCI_NEG (1) - Negative
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetGatedTriggerPolarity(int Window, out int Trigger_Polarity)
        {
            int pInvokeResult = PInvoke.GetGatedTriggerPolarity(this._handle, Window, out Trigger_Polarity);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the delay time between the external gate signal and the continuation of the sweep.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SWEep:EGATe:HOLDoff
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Trigger_Delay">
        /// This control defines the delay time between the external gate signal and the continuation of the sweep.
        /// 
        /// Valid Range: 125e-9 to 100 s
        /// 
        /// Default Value: 0.0 s
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetGatedTriggerDelay(int Window, double Trigger_Delay)
        {
            int pInvokeResult = PInvoke.SetGatedTriggerDelay(this._handle, Window, Trigger_Delay);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the delay time between the external gate signal and the continuation of the sweep.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SWEep:EGATe:HOLDoff?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Trigger_Delay">
        /// This control returns the delay time between the external gate signal and the continuation of the sweep.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetGatedTriggerDelay(int Window, out double Trigger_Delay)
        {
            int pInvokeResult = PInvoke.GetGatedTriggerDelay(this._handle, Window, out Trigger_Delay);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// In case of edge triggering, this function determines the time interval during which the instrument sweeps.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SWEep:EGATe:LENGth
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Trigger_Length">
        /// In case of edge triggering, this control determines the time interval during which the instrument sweeps.
        /// 
        /// Valid Range: 0 to 100 s
        /// 
        /// Default Value: 0.0 s
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetGatedTriggerLength(int Window, double Trigger_Length)
        {
            int pInvokeResult = PInvoke.SetGatedTriggerLength(this._handle, Window, Trigger_Length);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// In case of edge triggering, this function returns the time interval during which the instrument sweeps.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SWEep:EGATe:LENGth?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Trigger_Length">
        /// In case of edge triggering, this control returns the time interval during which the instrument sweeps.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetGatedTriggerLength(int Window, out double Trigger_Length)
        {
            int pInvokeResult = PInvoke.GetGatedTriggerLength(this._handle, Window, out Trigger_Length);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function enables/disables gate range in gate range table.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SWEep:EGATe:TRACe&lt;1...3&gt;:STATe&lt;1...3&gt; ON | OFF
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Range">
        /// This control indicates the range.
        /// 
        /// Valid Values:
        /// 1 to 3
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Trace">
        /// This control indicates the trace.
        /// 
        /// Valid Values:
        /// 1 to 3
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Gate_Range_Table_State">
        /// This control enables/disables gate range in gate range table.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetGatedTriggerState(int Window, int Range, int Trace, bool Gate_Range_Table_State)
        {
            int pInvokeResult = PInvoke.SetGatedTriggerState(this._handle, Window, Range, Trace, System.Convert.ToUInt16(Gate_Range_Table_State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries state of the gate range in gate range table.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SWEep:EGATe:TRACe&lt;1...3&gt;:STATe&lt;1...3&gt;?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Range">
        /// This control indicates the range.
        /// 
        /// Valid Values:
        /// 1 to 3
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Trace">
        /// This control indicates the trace.
        /// 
        /// Valid Values:
        /// 1 to 3
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Gate_Range_Table_State">
        /// This control returns state of gate range in gate range table.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetGatedTriggerState(int Window, int Range, int Trace, out bool Gate_Range_Table_State)
        {
            ushort Gate_Range_Table_StateAsUShort;
            int pInvokeResult = PInvoke.GetGatedTriggerState(this._handle, Window, Range, Trace, out Gate_Range_Table_StateAsUShort);
            Gate_Range_Table_State = System.Convert.ToBoolean(Gate_Range_Table_StateAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the start and stop values of a gate range in the gate range table.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SWEep:EGATe:TRACe&lt;1...3&gt;:STARt&lt;1...3&gt;
        /// [SENSe&lt;1|2&gt;:]SWEep:EGATe:TRACe&lt;1...3&gt;:STOP&lt;1...3&gt;
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Range">
        /// This control indicates the range.
        /// 
        /// Valid Values:
        /// 1 to 3
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Trace">
        /// This control indicates the trace.
        /// 
        /// Valid Values:
        /// 1 to 3
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Start">
        /// This control sets the start value of a gate range in the gate range table.
        /// 
        /// Valid Values:
        /// 0.0 to 70000.0 s
        /// 
        /// Default Value: 0.0
        /// </param>
        /// <param name="Stop">
        /// This control sets the stop value of a gate range in the gate range table.
        /// 
        /// Valid Values:
        /// 0.0 to 70000.0 s
        /// 
        /// Default Value: 0.0
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetGatedTriggerRange(int Window, int Range, int Trace, double Start, double Stop)
        {
            int pInvokeResult = PInvoke.SetGatedTriggerRange(this._handle, Window, Range, Trace, Start, Stop);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the start and stop values of a gate range in the gate range table.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SWEep:EGATe:TRACe&lt;1...3&gt;:STARt&lt;1...3&gt;?
        /// [SENSe&lt;1|2&gt;:]SWEep:EGATe:TRACe&lt;1...3&gt;:STOP&lt;1...3&gt;?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Range">
        /// This control indicates the range.
        /// 
        /// Valid Values:
        /// 1 to 3
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Trace">
        /// This control indicates the trace.
        /// 
        /// Valid Values:
        /// 1 to 3
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Start">
        /// This control returns the start value of a gate range in the gate range table.
        /// 
        /// Valid Values:
        /// not checked
        /// 
        /// Default Value: 0.0
        /// </param>
        /// <param name="Stop">
        /// This control returns the stop value of a gate range in the gate range table.
        /// 
        /// Valid Values:
        /// not checked
        /// 
        /// Default Value: 0.0
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetGatedTriggerRange(int Window, int Range, int Trace, out double Start, out double Stop)
        {
            int pInvokeResult = PInvoke.GetGatedTriggerRange(this._handle, Window, Range, Trace, out Start, out Stop);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the period of the signal for the gate range table.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SWEep:EGATe:TRACe&lt;1..3&gt;:PERiod
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Trace">
        /// This control indicates the trace.
        /// 
        /// Valid Values:
        /// 1 to 3
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Period">
        /// This control sets the period of the signal for the gate range table.
        /// 
        /// Valid Values:
        /// 0.0 to 70000.0 s
        /// 
        /// Default Value: 0.0
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetGatedTriggerPeriod(int Window, int Trace, double Period)
        {
            int pInvokeResult = PInvoke.SetGatedTriggerPeriod(this._handle, Window, Trace, Period);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the period of the signal for the gate range table.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SWEep:EGATe:TRACe&lt;1..3&gt;:PERiod?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Trace">
        /// This control indicates the trace.
        /// 
        /// Valid Values:
        /// 1 to 3
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Period">
        /// This control returns the period of the signal for the gate range table.
        /// 
        /// Valid Values:
        /// not checked
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetGatedTriggerPeriod(int Window, int Trace, out double Period)
        {
            int pInvokeResult = PInvoke.GetGatedTriggerPeriod(this._handle, Window, Trace, out Period);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the comment string in the gate range table.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SWEep:EGATe:TRACe&lt;1..3&gt;:COMMent
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Trace">
        /// This control indicates the trace.
        /// 
        /// Valid Values:
        /// 1 to 3
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Comment">
        /// This control sets the comment string in the gate range table.
        /// 
        /// Valid Values:
        /// not checked
        /// 
        /// Default Value: "Comment"
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetGatedTriggerComment(int Window, int Trace, string Comment)
        {
            int pInvokeResult = PInvoke.SetGatedTriggerComment(this._handle, Window, Trace, Comment);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the comment string in the gate range table.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SWEep:EGATe:TRACe&lt;1..3&gt;:COMMent?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Trace">
        /// This control indicates the trace.
        /// 
        /// Valid Values:
        /// 1 to 3
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Array_Size">
        /// This control defines the size of array in argument 'Comment'.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: 100
        /// </param>
        /// <param name="Comment">
        /// This control retruns the comment string in the gate range table.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetGatedTriggerComment(int Window, int Trace, int Array_Size, System.Text.StringBuilder Comment)
        {
            int pInvokeResult = PInvoke.GetGatedTriggerComment(this._handle, Window, Trace, Array_Size, Comment);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches triggering on TV signals on or off.
        /// 
        /// Note(s):
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FSP-B6 (TV and RF Trigger).
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]TV[:STATe] ON | OFF
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="TV_Trigger">
        /// This control switches triggering on TV signals on or off.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetTVTrigger(int Window, bool TV_Trigger)
        {
            int pInvokeResult = PInvoke.SetTVTrigger(this._handle, Window, System.Convert.ToUInt16(TV_Trigger));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the state of triggering on TV signals.
        /// 
        /// Note(s):
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FSP-B6 (TV and RF Trigger).
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]TV[:STATe]?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="TV_Trigger">
        /// This control returns the state of triggering on TV signals.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetTVTrigger(int Window, out bool TV_Trigger)
        {
            ushort TV_TriggerAsUShort;
            int pInvokeResult = PInvoke.GetTVTrigger(this._handle, Window, out TV_TriggerAsUShort);
            TV_Trigger = System.Convert.ToBoolean(TV_TriggerAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// With active TV trigger this function activates triggering at the vertical sync signal. The measurement is triggered on both fields with selection 'All', on odd fields with selection 'Odd'
        /// and on even fields with selection 'Even'.
        /// 
        /// Note(s):
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FSP-B6 (TV and RF Trigger).
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// TRIGger&lt;1|2&gt;[:SEQuence]:VIDeo:FIELd:SELect ALL | ODD | EVEN
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Field">
        /// This control selects the field on which the measurement is triggered.
        /// 
        /// Valid Values:
        /// RSESCI_TVFIELD_ALL  (0) - All
        /// RSESCI_TVFIELD_ODD  (1) - Odd
        /// RSESCI_TVFIELD_EVEN (2) - Even
        /// 
        /// Default Value: RSESCI_TVFIELD_ALL  (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetTVTriggerVerticalSync(int Window, int Field)
        {
            int pInvokeResult = PInvoke.SetTVTriggerVerticalSync(this._handle, Window, Field);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns selected the field on which the measurement is triggered.
        /// 
        /// Note(s):
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FSP-B6 (TV and RF Trigger).
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// TRIGger&lt;1|2&gt;[:SEQuence]:VIDeo:FIELd:SELect?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Field">
        /// This control returns selected the field on which the measurement is triggered.
        /// 
        /// Valid Values:
        /// RSESCI_TVFIELD_ALL  (0) - All
        /// RSESCI_TVFIELD_ODD  (1) - Odd
        /// RSESCI_TVFIELD_EVEN (2) - Even
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetTVTriggerVerticalSync(int Window, out int Field)
        {
            int pInvokeResult = PInvoke.GetTVTriggerVerticalSync(this._handle, Window, out Field);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// With active TV trigger this function activates triggering at the horizontal sync signal of the indicated line number.
        /// 
        /// Note(s):
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FSP-B6 (TV and RF Trigger).
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// TRIGger&lt;1|2&gt;[:SEQuence]:VIDeo:LINE:NUMBer
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Line_Number">
        /// This control defines the line number for tirggering at the horizontal sync signal.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: 0
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetTVTriggerHorizontalSync(int Window, int Line_Number)
        {
            int pInvokeResult = PInvoke.SetTVTriggerHorizontalSync(this._handle, Window, Line_Number);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns line number for tirggering at the horizontal sync signal.
        /// 
        /// Note(s):
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FSP-B6 (TV and RF Trigger).
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// TRIGger&lt;1|2&gt;[:SEQuence]:VIDeo:LINE:NUMBer?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Line_Number">
        /// This control returns line number for tirggering at the horizontal sync signal.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetTVTriggerHorizontalSync(int Window, out int Line_Number)
        {
            int pInvokeResult = PInvoke.GetTVTriggerHorizontalSync(this._handle, Window, out Line_Number);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the slope of the video sync signal.
        /// 
        /// Note(s):
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FSP-B6 (TV and RF Trigger).
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// TRIGger&lt;1|2&gt;[:SEQuence]:VIDeo:SSIGnal:POLarity NEGative | POSitive
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Polarity">
        /// This control selects the slope of the video sync signal.
        /// 
        /// Valid Values:
        /// RSESCI_POL_POS (0) - Positive
        /// RSESCI_POL_NEG (1) - Negative
        /// 
        /// Default Value: RSESCI_POL_POS (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetTVTriggerPolarity(int Window, int Polarity)
        {
            int pInvokeResult = PInvoke.SetTVTriggerPolarity(this._handle, Window, Polarity);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the slope of the video sync signal.
        /// 
        /// Note(s):
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FSP-B6 (TV and RF Trigger).
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// TRIGger&lt;1|2&gt;[:SEQuence]:VIDeo:SSIGnal:POLarity?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Polarity">
        /// This control returns the slope of the video sync signal.
        /// 
        /// Valid Values:
        /// RSESCI_POL_POS (0) - Positive
        /// RSESCI_POL_NEG (1) - Negative
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetTVTriggerPolarity(int Window, out int Polarity)
        {
            int pInvokeResult = PInvoke.GetTVTriggerPolarity(this._handle, Window, out Polarity);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the line system in use (525 or 625 lines) with active TV trigger.
        /// 
        /// Note(s):
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FSP-B6 (TV and RF Trigger).
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// TRIGger&lt;1|2&gt;[:SEQuence]:VIDeo:FORMat:LPFRame 525 | 625
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Lines_System">
        /// This control defines the line system in use (525 or 625 lines) with active TV trigger.
        /// 
        /// Valid Range: 525, 625
        /// 
        /// Default Value: 525
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetTVTriggerLines(int Window, int Lines_System)
        {
            int pInvokeResult = PInvoke.SetTVTriggerLines(this._handle, Window, Lines_System);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the line system in use (525 or 625 lines) with active TV trigger.
        /// 
        /// Note(s):
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FSP-B6 (TV and RF Trigger).
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// TRIGger&lt;1|2&gt;[:SEQuence]:VIDeo:FORMat:LPFRame?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Lines_System">
        /// This control returns the line system in use (525 or 625 lines) with active TV trigger.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetTVTriggerLines(int Window, out int Lines_System)
        {
            int pInvokeResult = PInvoke.GetTVTriggerLines(this._handle, Window, out Lines_System);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects between an internal and an external CCVS signal as TV demodulator input
        /// 
        /// Note(s):
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FSP-B6 (TV and RF Trigger).
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]TV:CCVS INTernal | EXTernal
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="CCVS_Signal">
        /// This control selects between an internal and an external CCVS signal as TV demodulator input.
        /// 
        /// Valid Values:
        /// RSESCI_INT (0) - Internal
        /// RSESCI_EXT (1) - External
        /// 
        /// Default Value: RSESCI_INT (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetTVTriggerCCVS(int Window, int CCVS_Signal)
        {
            int pInvokeResult = PInvoke.SetTVTriggerCCVS(this._handle, Window, CCVS_Signal);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns selected CCVS signal as TV demodulator input.
        /// 
        /// Note(s):
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FSP-B6 (TV and RF Trigger).
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]TV:CCVS?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="CCVS_Signal">
        /// This control returns selected CCVS signal as TV demodulator input.
        /// 
        /// Valid Values:
        /// RSESCI_INT (0) - Internal
        /// RSESCI_EXT (1) - External
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetTVTriggerCCVS(int Window, out int CCVS_Signal)
        {
            int pInvokeResult = PInvoke.GetTVTriggerCCVS(this._handle, Window, out CCVS_Signal);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the type of display and the evaluation of the traces in the selected measurement window.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1...3&gt;:MODE WRITe | VIEW | AVERage | MAXHold | MINHold
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Trace">
        /// This control sets the trace.
        /// 
        /// Valid Range: 1 to 3
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Trace_Mode">
        /// This control defines the type of display and the evaluation of the traces in the selected measurement window.
        /// 
        /// Valid Values:
        /// RSESCI_TRACE_WRITE (0) - Write
        /// RSESCI_TRACE_VIEW  (1) - View
        /// RSESCI_TRACE_AVG   (2) - Average
        /// RSESCI_TRACE_MAXH  (3) - Max Hold
        /// RSESCI_TRACE_MINH  (4) - Min Hold
        /// 
        /// Default Value: RSESCI_TRACE_WRITE (0)
        /// 
        /// Note(s):
        /// 
        /// (1) 'Write' corresponds to the Clr/Write mode of manual operation.
        /// 
        /// (2) The number of measurements for 'Average', 'Max Hold' and 'Min Hold' is defined with the function rsesci_SetSweepCount().
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetTraceMode(int Window, int Trace, int Trace_Mode)
        {
            int pInvokeResult = PInvoke.SetTraceMode(this._handle, Window, Trace, Trace_Mode);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the type of display and the evaluation of the traces in the selected measurement window.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1...3&gt;:MODE?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Trace">
        /// This control sets the trace.
        /// 
        /// Valid Range: 1 to 3
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Trace_Mode">
        /// This control returns the type of display and the evaluation of the traces in the selected measurement window.
        /// 
        /// Valid Values:
        /// RSESCI_TRACE_WRITE (0) - Write
        /// RSESCI_TRACE_VIEW  (1) - View
        /// RSESCI_TRACE_AVG   (2) - Average
        /// RSESCI_TRACE_MAXH  (3) - Max Hold
        /// RSESCI_TRACE_MINH  (4) - Min Hold
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetTraceMode(int Window, int Trace, out int Trace_Mode)
        {
            int pInvokeResult = PInvoke.GetTraceMode(this._handle, Window, Trace, out Trace_Mode);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches on or off the display of the corresponding trace in the selected measurement window.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACE&lt;1...3&gt;[:STATe] ON | OFF
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Trace">
        /// This control sets the trace.
        /// 
        /// Valid Range: 1 to 3
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Trace_State">
        /// This control switches on or off the display of the corresponding trace in the selected measurement window.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_TRUE  (1)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetTraceBlanking(int Window, int Trace, bool Trace_State)
        {
            int pInvokeResult = PInvoke.SetTraceBlanking(this._handle, Window, Trace, System.Convert.ToUInt16(Trace_State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the state of the display of the corresponding trace in the selected measurement window.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACE&lt;1...3&gt;[:STATe]?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Trace">
        /// This control sets the trace.
        /// 
        /// Valid Range: 1 to 3
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Trace_State">
        /// This control returns the state of the display of the corresponding trace in the selected measurement window.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetTraceBlanking(int Window, int Trace, out bool Trace_State)
        {
            ushort Trace_StateAsUShort;
            int pInvokeResult = PInvoke.GetTraceBlanking(this._handle, Window, Trace, out Trace_StateAsUShort);
            Trace_State = System.Convert.ToBoolean(Trace_StateAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches on the detector for the data acquisition in the selected trace and the indicated measurement window.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]DETector&lt;1...3&gt;[:FUNCtion] APEak | NEGative | POSitive | SAMPle | RMS | AVERage | QPEak | CAVerage | CRMS
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Trace">
        /// This control sets the trace.
        /// 
        /// Valid Range: 1 to 3
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Trace_Detector">
        /// This control switches on the detector for the data acquisition in the selected trace and the indicated measurement window.
        /// 
        /// Valid Values:
        /// RSESCI_TRACEDET_APEAK  (0) - Auto Peak 
        /// RSESCI_TRACEDET_NEG    (1) - Negative
        /// RSESCI_TRACEDET_POS    (2) - Positive
        /// RSESCI_TRACEDET_SAMPLE (3) - Sample
        /// RSESCI_TRACEDET_RMS    (4) - RMS
        /// RSESCI_TRACEDET_AVG    (5) - Average
        /// RSESCI_TRACEDET_QPEAK  (6) - QPeak
        /// RSESCI_TRACEDET_CAVG   (7) - CISPR Average
        /// RSESCI_TRACEDET_CRMS   (8) - CISPR-RMS
        /// 
        /// Default Value: RSESCI_TRACEDET_APEAK (0)
        /// 
        /// Note(s):
        /// 
        /// (1) The 'Auto Peak' detector displays the positive and also the negative peak value of the noise floor. If a signal is detected, only the positive peak value is displayed.
        /// 
        /// (2) The 'Positive' or 'Negative' detector only displays the positive or the negative peak value.
        /// 
        /// (3) With the 'Sample' detector the value measured at the sampling time is displayed, whereas the RMS value of the power measured at each test point is displayed with the RMS detector.
        /// 
        /// (4) The 'Average' detector displays the power average value at each test point.
        /// 
        /// (5) The QPeak detector, "CAVerage" for CISPR average and "CRMS" for CISPRRMS perform a signal evaluation for EMC measurements.
        /// 
        /// (6) The "CAVerage" detector represents the average value weighted with a time constant. This detector is not available in the analyzer mode (ESCI instrument).
        /// 
        /// (7) For ESU instrument - CISPR average "CAVerage and CISPR-RMS "CRMS" are available only in the analyzer operating mode and zerospan.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetTraceDetector(int Window, int Trace, int Trace_Detector)
        {
            int pInvokeResult = PInvoke.SetTraceDetector(this._handle, Window, Trace, Trace_Detector);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function reads selected trace detector.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]DETector&lt;1...3&gt;[:FUNCtion]?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Trace">
        /// This control sets the trace.
        /// 
        /// Valid Range: 1 to 3
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Trace_Detector">
        /// This control returns selected trace detector.
        /// 
        /// Valid Values:
        /// RSESCI_TRACEDET_APEAK  (0) - Auto Peak 
        /// RSESCI_TRACEDET_NEG    (1) - Negative
        /// RSESCI_TRACEDET_POS    (2) - Positive
        /// RSESCI_TRACEDET_SAMPLE (3) - Sample
        /// RSESCI_TRACEDET_RMS    (4) - RMS
        /// RSESCI_TRACEDET_AVG    (5) - Average
        /// RSESCI_TRACEDET_QPEAK  (6) - QPeak
        /// RSESCI_TRACEDET_CAVG   (7) - CISPR Average
        /// RSESCI_TRACEDET_CRMS   (8) - CISPR-RMS
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetTraceDetector(int Window, int Trace, out int Trace_Detector)
        {
            int pInvokeResult = PInvoke.GetTraceDetector(this._handle, Window, Trace, out Trace_Detector);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function either couples the detector in the selected measurement window to the current trace setting or turns coupling off.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]DETector&lt;1...3&gt;[:FUNCtion]:AUTO ON | OFF
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Trace">
        /// This control sets the trace.
        /// 
        /// Valid Range: 1 to 3
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Couple_Detector">
        /// This control either couples the detector in the selected measurement window to the current trace setting or turns coupling off.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_TRUE (1)
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetTraceDetectorAuto(int Window, int Trace, bool Couple_Detector)
        {
            int pInvokeResult = PInvoke.SetTraceDetectorAuto(this._handle, Window, Trace, System.Convert.ToUInt16(Couple_Detector));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns detector coupling state.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]DETector&lt;1...3&gt;[:FUNCtion]:AUTO?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Trace">
        /// This control sets the trace.
        /// 
        /// Valid Range: 1 to 3
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Couple_Detector">
        /// This control returns detector coupling state.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetTraceDetectorAuto(int Window, int Trace, out bool Couple_Detector)
        {
            ushort Couple_DetectorAsUShort;
            int pInvokeResult = PInvoke.GetTraceDetectorAuto(this._handle, Window, Trace, out Couple_DetectorAsUShort);
            Couple_Detector = System.Convert.ToBoolean(Couple_DetectorAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function specifies whether or not the traces with peak or minimum value detection are reset after specific parameter changes.
        /// Usually the measurement must be restarted after a parameter change, before an evaluation of the measurement results is performed (e.g. with a marker). In cases in which a change causes a compulsory new measurement, the trace is automatically reset in order to prevent erroneous measurements of previous measurement results (e.g. when the span changes). For applications in which this behavior is not desired, this mechanism can be switched off.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1...3&gt;:MODE:HCONtinuous ON | OFF
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Trace">
        /// This control sets the trace.
        /// 
        /// Valid Range: 1 to 3
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Switch_Off_Reset">
        /// This control specifies whether or not the traces with peak or minimum value detection are reset after specific parameter changes.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Note(s):
        /// 
        /// (1) OFF: The traces are reset after specific parameter changes.
        /// 
        /// (2) ON: The reset mechanism is switched off.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetTraceResetBehavior(int Window, int Trace, bool Switch_Off_Reset)
        {
            int pInvokeResult = PInvoke.SetTraceResetBehavior(this._handle, Window, Trace, System.Convert.ToUInt16(Switch_Off_Reset));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns whether or not the traces with peak or minimum value detection are reset after specific parameter changes.
        /// Usually the measurement must be restarted after a parameter change, before an evaluation of the measurement results is performed (e.g. with a marker). In cases in which a change causes a compulsory new measurement, the trace is automatically reset in order to prevent erroneous measurements of previous measurement results (e.g. when the span changes). For applications in which this behavior is not desired, this mechanism can be switched off.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1...3&gt;:MODE:HCONtinuous?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Trace">
        /// This control sets the trace.
        /// 
        /// Valid Range: 1 to 3
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Switch_Off_Reset">
        /// This control returns the state of the reset.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetTraceResetBehavior(int Window, int Trace, out bool Switch_Off_Reset)
        {
            ushort Switch_Off_ResetAsUShort;
            int pInvokeResult = PInvoke.GetTraceResetBehavior(this._handle, Window, Trace, out Switch_Off_ResetAsUShort);
            Switch_Off_Reset = System.Convert.ToBoolean(Switch_Off_ResetAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches the mathematical relation of traces on or off.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MATH:STATe ON | OFF
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Math_Relation">
        /// This control switches the mathematical relation of traces on or off.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetTraceMath(int Window, bool Math_Relation)
        {
            int pInvokeResult = PInvoke.SetTraceMath(this._handle, Window, System.Convert.ToUInt16(Math_Relation));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the state of the mathematical relation of traces.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MATH:STATe?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Math_Relation">
        /// This control returns the state of the mathematical relation of traces.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetTraceMath(int Window, out bool Math_Relation)
        {
            ushort Math_RelationAsUShort;
            int pInvokeResult = PInvoke.GetTraceMath(this._handle, Window, out Math_RelationAsUShort);
            Math_Relation = System.Convert.ToBoolean(Math_RelationAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects linear or logarithmic (= video) calculation of the mathematical functions related to the traces. The calculation of the average is one of the affected functions. The setting is valid for all measurement windows.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MATH:MODE LINear | LOGarithmic
        /// </summary>
        /// <param name="Math_Mode">
        /// This control selects linear or logarithmic (= video) calculation of the mathematical functions related to the traces. The calculation of the average is one of the affected functions. The setting is valid for all measurement windows.
        /// 
        /// Valid Values:
        /// RSESCI_LIN (0) - Linear
        /// RSESCI_LOG (1) - Logarithmic
        /// 
        /// Default Value: RSESCI_LIN (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetTraceMathMode(int Math_Mode)
        {
            int pInvokeResult = PInvoke.SetTraceMathMode(this._handle, Math_Mode);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns selected mathematical mode.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MATH:MODE?
        /// </summary>
        /// <param name="Math_Mode">
        /// This control returns selected mathematical mode.
        /// 
        /// Valid Values:
        /// RSESCI_LIN (0) - Linear
        /// RSESCI_LOG (1) - Logarithmic
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetTraceMathMode(out int Math_Mode)
        {
            int pInvokeResult = PInvoke.GetTraceMathMode(this._handle, out Math_Mode);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the position of the result of the trace mathematics in the selected measurement window. The indication is in % of the screen height, with 100% corresponding to the
        /// upper diagram border.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MATH:POSition
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Position">
        /// This control defines the position of the result of the trace mathematics in the selected measurement window. The indication is in % of the screen height, with 100% corresponding to the
        /// upper diagram border.
        /// 
        /// Valid Range: -100 to 200 %
        /// 
        /// Default Value: 50.0 %
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetTraceMathPosition(int Window, double Position)
        {
            int pInvokeResult = PInvoke.SetTraceMathPosition(this._handle, Window, Position);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the position of the result of the trace mathematics in the selected measurement window.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MATH:POSition?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Position">
        /// This control returns the position of the result of the trace mathematics in the selected measurement window.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetTraceMathPosition(int Window, out double Position)
        {
            int pInvokeResult = PInvoke.GetTraceMathPosition(this._handle, Window, out Position);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the mathematical expression for relating traces to trace1.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MATH[:EXPression][:DEFine] &lt;expr&gt;
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Expression">
        /// This control defines the mathematical expression for relating traces to trace 1.
        /// 
        /// Valid Values:
        /// RSESCI_MATH_TR1_SUB_TR2 (0) - Trace 1 - Trace 2
        /// RSESCI_MATH_TR1_SUB_TR3 (1) - Trace 1 - Trace 3
        /// 
        /// Default Value: RSESCI_MATH_TR1_SUB_TR2 (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetTraceMathExpression(int Window, int Expression)
        {
            int pInvokeResult = PInvoke.SetTraceMathExpression(this._handle, Window, Expression);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the mathematical expression for relating traces to trace1.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MATH[:EXPression][:DEFine]?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Expression">
        /// This control returns the mathematical expression for relating traces to trace1.
        /// 
        /// Valid Values:
        /// RSESCI_MATH_TR1_SUB_TR2 (0) - Trace 1 - Trace 2
        /// RSESCI_MATH_TR1_SUB_TR3 (1) - Trace 1 - Trace 3
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetTraceMathExpression(int Window, out int Expression)
        {
            int pInvokeResult = PInvoke.GetTraceMathExpression(this._handle, Window, out Expression);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches block data transmission during a scan on and off. The block size depends on the scan time, the trace number is not evaluated.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// TRACe&lt;1|2&gt;:FEED:CONTrol&lt;1...3&gt; ALWays | NEVer
        /// </summary>
        /// <param name="Control">
        /// This control defines the control block.
        /// 
        /// Valid Range: 1 to 4
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Transmission_Mode">
        /// This control selects the transmission mode.
        /// 
        /// Valid Values:
        /// RSESCI_TRANS_NEVER  (0) - Never
        /// RSESCI_TRANS_ALWAYS (1) - Always
        /// 
        /// Default Value: RSESCI_TRANS_NEVER (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetTraceFeedControl(int Control, int Transmission_Mode)
        {
            int pInvokeResult = PInvoke.SetTraceFeedControl(this._handle, Control, Transmission_Mode);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns whether block data transmission during a scan is on or off.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// TRACe&lt;1|2&gt;:FEED:CONTrol&lt;1...3&gt;?
        /// </summary>
        /// <param name="Control">
        /// This control defines the control block.
        /// 
        /// Valid Range: 1 to 4
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Transmission_Mode">
        /// This control returns whether block data transmission during a scan is on or off.
        /// 
        /// Valid Values:
        /// RSESCI_TRANS_NEVER  (0) - Never
        /// RSESCI_TRANS_ALWAYS (1) - Always
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetTraceFeedControl(int Control, out int Transmission_Mode)
        {
            int pInvokeResult = PInvoke.GetTraceFeedControl(this._handle, Control, out Transmission_Mode);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches the I/Q data acquisition on or off.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// TRACe&lt;1|2&gt;:IQ[:STATe] ON | OFF
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="IQ_Trace_State">
        /// This control switches the I/Q data acquisition on or off.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetIQTrace(int Window, bool IQ_Trace_State)
        {
            int pInvokeResult = PInvoke.SetIQTrace(this._handle, Window, System.Convert.ToUInt16(IQ_Trace_State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the state of the I/Q data acquisition.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// TRACe&lt;1|2&gt;:IQ[:STATe]?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="IQ_Trace_State">
        /// This control returns the state of the I/Q data acquisition.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetIQTrace(int Window, out bool IQ_Trace_State)
        {
            ushort IQ_Trace_StateAsUShort;
            int pInvokeResult = PInvoke.GetIQTrace(this._handle, Window, out IQ_Trace_StateAsUShort);
            IQ_Trace_State = System.Convert.ToBoolean(IQ_Trace_StateAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function enables averaging of the recorded I/Q data provided that I/Q data acquisition was previously enabled and the sampling rate was set to 32 MHz.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// TRACe&lt;1|2&gt;:IQ:AVERage[:STATe] ON|OFF
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="IQ_Trace_State">
        /// This control switches averaging of the recorded I/Q data on or off.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetIQTraceAveraging(int Window, bool IQ_Trace_State)
        {
            int pInvokeResult = PInvoke.SetIQTraceAveraging(this._handle, Window, System.Convert.ToUInt16(IQ_Trace_State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the state of averaging of the recorded I/Q data.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// TRACe&lt;1|2&gt;:IQ:AVERage[:STATe]?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="IQ_Trace_State">
        /// This control returns the state of averaging of the recorded I/Q data.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetIQTraceAveraging(int Window, out bool IQ_Trace_State)
        {
            ushort IQ_Trace_StateAsUShort;
            int pInvokeResult = PInvoke.GetIQTraceAveraging(this._handle, Window, out IQ_Trace_StateAsUShort);
            IQ_Trace_State = System.Convert.ToBoolean(IQ_Trace_StateAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the number of I/Q data sets that are to serve as a basis for averaging.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// TRACe&lt;1|2&gt;:IQ:AVERage:COUNt
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Average_Count">
        /// This control defines the number of I/Q data sets that are to serve as a basis for averaging.
        /// 
        /// Valid Range: 0 to 32767
        /// 
        /// Default Value: 0
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetIQTraceAveragingCount(int Window, int Average_Count)
        {
            int pInvokeResult = PInvoke.SetIQTraceAveragingCount(this._handle, Window, Average_Count);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the number of I/Q data sets that are to serve as a basis for averaging.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// TRACe&lt;1|2&gt;:IQ:AVERage:COUNt?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Average_Count">
        /// This control returns the number of I/Q data sets that are to serve as a basis for averaging.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetIQTraceAveragingCount(int Window, out int Average_Count)
        {
            int pInvokeResult = PInvoke.GetIQTraceAveragingCount(this._handle, Window, out Average_Count);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches on or off the synchronization of the start of I/Q data acquisition to the trigger event. Prior to this functions, I/Q data acquisition has to be switched. The sampling rate must be 32 MHz. The synchronization ensures the correct calculation of I/Q average values as the data aquisition is always started with the same phase reference in relation to the trigger event.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Note(s): This command is not supported for sampling rates other than 32 MHz.
        /// 
        /// Remote-control command(s):
        /// TRACe&lt;1|2&gt;:IQ:SYNChronize[:STATe] ON | OFF
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="State">
        /// This control switches on or off the synchronization of the start of I/Q data acquisition to the trigger event.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_TRUE  (1)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetIQTraceSynchronization(int Window, bool State)
        {
            int pInvokeResult = PInvoke.SetIQTraceSynchronization(this._handle, Window, System.Convert.ToUInt16(State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the state of the synchronization of the start of I/Q data acquisition to the trigger event.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// TRACe&lt;1|2&gt;:IQ:SYNChronize[:STATe]?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="State">
        /// This control returns the state of the synchronization of the start of I/Q data acquisition to the trigger event.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetIQTraceSynchronization(int Window, out bool State)
        {
            ushort StateAsUShort;
            int pInvokeResult = PInvoke.GetIQTraceSynchronization(this._handle, Window, out StateAsUShort);
            State = System.Convert.ToBoolean(StateAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the sampling rate for the I/Q data acquisition. Thus the sample rate can be modified without affecting the other settings.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// TRACe&lt;1|2&gt;:IQ:SRATe
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Sampling_Rate">
        /// This control defines sampling rate for the I/Q data acquisition.
        /// 
        /// Valid Values: 15.625e3 to 32.0e6 Hz
        /// 
        /// Default Value: 32.0e6 Hz
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetIQTraceSamplingRate(int Window, double Sampling_Rate)
        {
            int pInvokeResult = PInvoke.SetIQTraceSamplingRate(this._handle, Window, Sampling_Rate);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function reads sampling rate for the I/Q data acquisition.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// TRACe&lt;1|2&gt;:IQ:SRATe?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Sampling_Rate">
        /// This control returns sampling rate for the I/Q data acquisition.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetIQTraceSamplingRate(int Window, out double Sampling_Rate)
        {
            int pInvokeResult = PInvoke.GetIQTraceSamplingRate(this._handle, Window, out Sampling_Rate);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the settings of the analyzer hardware for the measurement of I/Q data. This allows setting the bandwidth of the analog filters in front of the A/D converter as well as setting the sample rate, trigger conditions and the record length.
        /// 
        /// Notes:
        /// 
        /// (1) Function Trace I/Q State (rsesci_SetIQTrace) has to be switched ON before using this function.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// TRACe&lt;1|2&gt;:IQ:SET &lt;filter type&gt;,&lt;rbw&gt;,&lt;sample rate&gt;,&lt;trigger source&gt;,&lt;trigger slope&gt;,&lt;pretrigger samples&gt;,&lt;# of samples&gt;
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Sampling_Rate">
        /// This control defines sampling rate for the I/Q data acquisition.
        /// 
        /// Valid Values: 15.625e3 to 32.0e6 Hz
        /// 
        /// Default Value: 32.0e6 Hz
        /// </param>
        /// <param name="Trigger_Mode">
        /// This control selects the trigger source for the start of a sweep.
        /// 
        /// Valid Values:
        /// RSESCI_TRIG_IMM (0) - Immediate
        /// RSESCI_TRIG_EXT (1) - External
        /// RSESCI_TRIG_IFP (3) - IF Power
        /// RSESCI_TRIG_RFP (4) - RF Power
        /// 
        /// Default Value: RSESCI_TRIG_IMM (0)
        /// 
        /// Note(s):
        /// 
        /// (1) Immediate - automatic triggering the next measurement at the end of the previous one. The value 'Immediate' corresponds to the FREE RUN setting.
        /// 
        /// (2) External - the next measurement is triggered by the signal at the external trigger input.
        /// 
        /// (3) IF Power - the next measurement is triggered by the detection of a signal at the instrument IF (10 MHz bandwidth)
        /// 
        /// (4) RF Power - the next measurement is triggered by the detection of a signal at the instrument RF (80 MHz bandwidth)
        /// </param>
        /// <param name="Polarity">
        /// This control selects the slope of the trigger signal.
        /// 
        /// Valid Values:
        /// RSESCI_POL_POS (0) - Positive
        /// RSESCI_POL_NEG (1) - Negative
        /// 
        /// Default Value: RSESCI_POL_POS (0)
        /// </param>
        /// <param name="Bandwidth">
        /// This control sets bandwidth of the analog filters in front of the A/D converter.
        /// 
        /// Valid Range: (see table below) in steps of 1, 3, 10
        /// 
        /// Default Value: 3.0e6 Hz
        /// 
        /// Note:
        /// 
        /// Depending on the sample rate, the following maximum bandwidths can be obtained during the measurement:
        /// 
        /// Sample Rate - Max Bandwidth
        /// 
        ///   32 MHz - 9.6 MHz
        ///   16 MHz - 7.72 MHz
        ///   8 MHz - 4.8 MHz
        ///   4 MHz - 2.8 MHz
        ///   2 MHz - 1.6 MHz
        ///   1 MHz - 800 kHz
        ///   500 kHz - 400 kHz
        ///   250 kHz - 200 kHz
        ///   125 kHz - 100 kHz
        ///   62.5 kHz - 50 kHz
        ///   31.25 kHz - 25 kHz
        ///   15.625 kHz - 12.5 kHz
        /// 
        /// </param>
        /// <param name="Pretrigger_Samples">
        /// This control sets the number of measurement values to be recorded before the trigger point.
        /// 
        /// Valid Range: -16744447 to 65023
        /// 
        /// Default Value: 0
        /// 
        /// Note:
        /// 
        /// For &lt;trigger mode&gt; = Immediate the value must be 0.
        /// 
        /// </param>
        /// <param name="No_of_Samples">
        /// This control sets the number of measurement values to record.
        /// 
        /// Valid Range: 1 to 1130560 (= 128*1024 - 512)
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetIQTraceSettings(int Window, double Sampling_Rate, int Trigger_Mode, int Polarity, double Bandwidth, int Pretrigger_Samples, int No_of_Samples)
        {
            int pInvokeResult = PInvoke.SetIQTraceSettings(this._handle, Window, Sampling_Rate, Trigger_Mode, Polarity, Bandwidth, Pretrigger_Samples, No_of_Samples);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the settings of the analyzer hardware for the measurement of I/Q data. This allows setting the bandwidth of the analog filters in front of the A/D converter as well as setting the sample rate, trigger conditions and the record length.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// TRACe&lt;1|2&gt;:IQ:SET?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Sampling_Rate">
        /// This control returns sampling rate for the I/Q data acquisition.
        /// 
        /// </param>
        /// <param name="Trigger_Mode">
        /// This control returns the trigger source for the start of a sweep.
        /// 
        /// Valid Values:
        /// RSESCI_TRIG_IMM (0) - Immediate
        /// RSESCI_TRIG_EXT (1) - External
        /// RSESCI_TRIG_IFP (3) - IF Power
        /// RSESCI_TRIG_RFP (4) - RF Power
        /// 
        /// Note(s):
        /// 
        /// (1) Immediate - automatic triggering the next measurement at the end of the previous one. The value 'Immediate' corresponds to the FREE RUN setting.
        /// 
        /// (2) External - the next measurement is triggered by the signal at the external trigger input.
        /// 
        /// (3) IF Power - the next measurement is triggered by the detection of a signal at the instrument IF (10 MHz bandwidth)
        /// 
        /// (4) RF Power - the next measurement is triggered by the detection of a signal at the instrument RF (80 MHz bandwidth)
        /// </param>
        /// <param name="Polarity">
        /// This control returns the slope of the trigger signal.
        /// 
        /// Valid Values:
        /// RSESCI_POL_POS (0) - Positive
        /// RSESCI_POL_NEG (1) - Negative
        /// </param>
        /// <param name="Bandwidth">
        /// This control returns bandwidth of the analog filters in front of the A/D converter.
        /// </param>
        /// <param name="Pretrigger_Samples">
        /// This control returns the number of measurement values to be recorded before the trigger point
        /// </param>
        /// <param name="No_of_Samples">
        /// This control returns the number of measurement values to record.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetIQTraceSettings(int Window, out double Sampling_Rate, out int Trigger_Mode, out int Polarity, out double Bandwidth, out int Pretrigger_Samples, out int No_of_Samples)
        {
            int pInvokeResult = PInvoke.GetIQTraceSettings(this._handle, Window, out Sampling_Rate, out Trigger_Mode, out Polarity, out Bandwidth, out Pretrigger_Samples, out No_of_Samples);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function copies data from one trace to another.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// TRACe&lt;1|2&gt;:COPY TRACE1 | TRACE2 | TRACE3 |, TRACE1 | TRACE2 | TRACE3
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Source_Trace">
        /// This control defines the source trace.
        /// 
        /// Valid Range: 1 to 3
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Destination_Trace">
        /// This control defines the destination trace.
        /// 
        /// Valid Range: 1 to 3
        /// 
        /// Default Value: 2
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int CopyTrace(int Window, int Source_Trace, int Destination_Trace)
        {
            int pInvokeResult = PInvoke.CopyTrace(this._handle, Window, Source_Trace, Destination_Trace);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function reads out trace data from the instrument.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Note(s):
        /// 
        /// (1) If the FM demodulator (option FS-K7) is active, only the displayed trace data is read out and recalled. A portion of the measurement data that can be called by means of a marker, however, is calculated from the raw measurement data. These results are no longer available after recalling a trace; the associated queries generate a query error.
        /// 
        /// (2) With AUTO PEAK detector, only positive peak values can be read out.
        /// 
        /// Remote-control command(s):
        /// FORMat[:DATA] ASCii | REAL,32
        /// TRACe&lt;1|2&gt;:DATA? TRACE1| TRACE2| TRACE3
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Trace">
        /// This control sets the trace.
        /// 
        /// Valid Range: 1 to 3
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="No_of_Points">
        /// Returns the number of trace data points.
        /// 
        /// Note:
        /// 
        /// If VI_ATTR_INTF_TYPE is equal to VI_INTF_ASRL, then use this parameter to pass the expected number of points to be read from the instrument. Data points will be read until either number of data points are consumed or instruments output buffer is empty. This is due to limitation if the unit is remote-controlled via the RS-232-C interface.
        /// 
        /// </param>
        /// <param name="Trace_Data">
        /// Trace data array.
        /// 
        /// Note(s):
        /// 
        /// (1) The returned values are scaled in the current level unit. Returned FM-modulated measurement values (activated option FS-K7) are scaled in Hz.
        /// 
        /// (2) The array should consist of at least the number of elements defined with function Set Sweep Points (rsesci_SetSweepPoints).
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int ReadTraceData(int Window, int Trace, out int No_of_Points, double[] Trace_Data)
        {
            int pInvokeResult = PInvoke.ReadTraceData(this._handle, Window, Trace, out No_of_Points, Trace_Data);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function writes trace data to the instrument.
        /// 
        /// Note(s):
        /// 
        /// (1) Trace data can be written into the instrument with logarithmic display only in dBm, with linear display only in volts.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// FORMat[:DATA] ASCii | REAL,32
        /// TRACe[:DATA] TRACE1| TRACE2| TRACE3
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Trace">
        /// This control sets the trace.
        /// 
        /// Valid Range: 1 to 3
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="No_of_Points">
        /// Sets the number of trace points to be writen to the instrument.
        /// 
        /// Valid Range: 0 to 250000
        /// 
        /// Default Value: none
        /// 
        /// </param>
        /// <param name="Trace_Data">
        /// Trace data array.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int WriteTraceData(int Window, int Trace, int No_of_Points, double[] Trace_Data)
        {
            int pInvokeResult = PInvoke.WriteTraceData(this._handle, Window, Trace, No_of_Points, Trace_Data);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function starts a measurement with the settings defined via rsesci_SetIQTraceSettings() and returns the list of measurement results immediately after they are corrected in terms of frequency response. The number of measurement results depends on the settings defined with rsesci_SetIQTraceSettings().
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Note(s):
        /// 
        /// (1) The command requires that all response data are read out completely before the instrument accepts further commands.
        /// 
        /// Remote-control command(s):
        /// FORMat[:DATA] ASCii | REAL| UINT [, 8 | 32]
        /// TRACe&lt;1|2&gt;:IQ:DATA?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Buffer_Size">
        /// Pass the number of doubles in the array you specify for the real parts and imaginary parameters.
        /// 
        /// The buffer must contain at least as many elements as the value you specify with the Buffer Size parameter. If the data contains more elemens than you specify with the Buffer Size parameter, the function returns Buffer Size elements into the arrays and returns the buffer size you must pass to get the all data.  For example, if the data contains 16000 elements for real parts and 16000 elements for imaginary part and the Buffer Size is 10000, the function places 10000 elements into the arrays and returns 16000.
        /// 
        /// Notes:
        /// 
        /// (1) If VI_ATTR_INTF_TYPE is equal to VI_INTF_ASRL, then use this parameter to pass the expected number of points to be read from the instrument. Data points will be read until instruments output buffer is empty. If Buffer Size is lower than actual number of results available in the instruments output buffer, then this function returns an error code (BFFC0002  Parameter 2 (Buffer Size) out of range.) and measurement results will not be valid. This is due to limitation if the unit is remote-controlled via the RS-232-C interface.
        /// 
        /// </param>
        /// <param name="No_of_Points">
        /// Returns the number of I/Q trace data points.
        /// 
        /// </param>
        /// <param name="Real_Parts__I_">
        /// Returns an array of trace data (I-value x, real part) point. The buffer must contain at least as many elements as the value you specify with the Buffer Size parameter.
        /// 
        /// </param>
        /// <param name="Imaginary_Parts__Q_">
        /// Returns an array of trace data (Q-value y, imaginary part) point. The buffer must contain at least as many elements as the value you specify with the Buffer Size parameter.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int ReadIQTraceData(int Window, int Buffer_Size, out int No_of_Points, double[] Real_Parts__I_, double[] Imaginary_Parts__Q_)
        {
            int pInvokeResult = PInvoke.ReadIQTraceData(this._handle, Window, Buffer_Size, out No_of_Points, Real_Parts__I_, Imaginary_Parts__Q_);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function permits the readout of previously acquired (and frequency response corrected) I/Q data from the memory, with indication of the offset related to the start of measurement and with indication of the number of measurement values. Therefore a previously acquired data set can be read out in smaller portions. The result values are scaled linear in unit Volt and correspond to the voltage at the RF input of the instrument.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Note(s):
        /// 
        /// (1) The command requires that all response data are read out completely before the instrument accepts further commands.
        /// 
        /// (2) For large I/Q trace data is recommended to increase default timeout value to the appropriate value which allows instrument handle all pending tasks. To change default timeout value call following VISA function:
        /// 
        ///     viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, 60000);
        /// 
        /// This attribute specifies the minimum timeout value to use (in milliseconds) when accessing the device associated with the given session.
        /// 
        /// (2) If there are not I/Q data available in memory because the corresponding measurement had not been started, the function will cause a Query Error.
        /// 
        /// Example:
        /// 
        ///   (1) Switch on I/Q data acquisition:
        ///       rsesci_SetIQTrace (instrSession, RSESCI_SCREEN_A,
        ///                          VI_TRUE);
        /// 
        ///   (2) Choose configuration:
        ///       rsesci_SetIQTraceSettings (io, RSESCI_SCREEN_A, 32.0e6,
        ///                                  RSESCI_TRIG_IMM, 
        ///                                  RSESCI_POL_POS, 3e6, 0, 1);
        /// 
        ///   (3) Start measurement and read results:
        ///       rsesci_ReadIQTraceData (instrSession, RSESCI_SCREEN_A,
        ///                               bufferSize, &amp;points, I, Q);
        /// 
        ///   (4) Read memory results:
        ///       rsesci_ReadIQTraceDataMemory (instrSession,
        ///                                     RSESCI_SCREEN_A, offset,
        ///                                     noSamples, bufferSize,
        ///                                     &amp;points, I, Q);
        /// 
        ///   (5) Switch off I/Q data acquisition:
        ///       rsesci_SetIQTrace (instrSession, RSESCI_SCREEN_A,
        ///                          VI_FALSE);
        /// 
        /// Remote-control command(s):
        /// TRACe&lt;1|2&gt;:IQ:DATA:MEMory[:RF]? &lt;offset samples&gt;,&lt;# of samples&gt;
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Offset_Samples">
        /// This control sets offset of the values to be read related to the start of the acquired data.
        /// 
        /// Valid Range: 0 to &lt;# of samples&gt; - 1
        /// 
        /// Default Value: 0
        /// 
        /// </param>
        /// <param name="No_of_Samples">
        /// This control sets the number of measurement values to be read.
        /// 
        /// Valid Range: 1 to &lt;# of samples&gt; - &lt;offset samples&gt;
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <param name="Buffer_Size">
        /// Pass the number of doubles in the array you specify for the real parts and imaginary parameters.
        /// 
        /// The buffer must contain at least as many elements as the value you specify with the Buffer Size parameter. If the data contains more elemens than you specify with the Buffer Size parameter, the function returns Buffer Size elements into the arrays and returns the buffer size you must pass to get the all data.  For example, if the data contains 16000 elements for real parts and 16000 elements for imaginary part and the Buffer Size is 10000, the function places 10000 elements into the arrays and returns 16000.
        /// 
        /// Notes:
        /// 
        /// (1) If VI_ATTR_INTF_TYPE is equal to VI_INTF_ASRL, then use this parameter to pass the expected number of points to be read from the instrument. Data points will be read until instruments output buffer is empty. If Buffer Size is lower than actual number of results available in the instruments output buffer, then this function returns an error code (BFFC0002  Parameter 2 (Buffer Size) out of range.) and measurement results will not be valid. This is due to limitation if the unit is remote-controlled via the RS-232-C interface.
        /// 
        /// </param>
        /// <param name="No_of_Points">
        /// Returns the number of I/Q trace data points.
        /// 
        /// </param>
        /// <param name="Real_Parts__I_">
        /// Returns an array of trace data (I-value x, real part) point. The buffer must contain at least as many elements as the value you specify with the Buffer Size parameter.
        /// 
        /// </param>
        /// <param name="Imaginary_Parts__Q_">
        /// Returns an array of trace data (Q-value y, imaginary part) point. The buffer must contain at least as many elements as the value you specify with the Buffer Size parameter.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int ReadIQTraceDataMemory(int Window, int Offset_Samples, int No_of_Samples, int Buffer_Size, out int No_of_Points, double[] Real_Parts__I_, double[] Imaginary_Parts__Q_)
        {
            int pInvokeResult = PInvoke.ReadIQTraceDataMemory(this._handle, Window, Offset_Samples, No_of_Samples, Buffer_Size, out No_of_Points, Real_Parts__I_, Imaginary_Parts__Q_);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function permits the readout of previously acquired (and frequency response corrected) I/Q data from the memory, with indication of the offset related to the start of measurement and with indication of the number of measurement values. Therefore a previously acquired data set can be read out in smaller portions. The result values are scaled linear in unit Volt and correspond to the voltage at the RF input of the instrument.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with R&amp;S ESU instrument.
        /// 
        /// (2) The command requires that all response data are read out completely before the instrument accepts further commands.
        /// 
        /// (3) For large I/Q trace data is recommended to increase default timeout value to the appropriate value which allows instrument handle all pending tasks. To change default timeout value call following VISA function:
        /// 
        ///     viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, 60000);
        /// 
        /// This attribute specifies the minimum timeout value to use (in milliseconds) when accessing the device associated with the given session.
        /// 
        /// (4) If there are not I/Q data available in memory because the corresponding measurement had not been started, the function will cause a Query Error.
        /// 
        /// Example:
        /// 
        ///   (1) Switch on I/Q data acquisition:
        ///       rsesci_SetIQTrace (instrSession, RSESCI_SCREEN_A,
        ///                          VI_TRUE);
        /// 
        ///   (2) Choose configuration:
        ///       rsesci_SetIQTraceSettings (io, RSESCI_SCREEN_A, 32.0e6,
        ///                                  RSESCI_TRIG_IMM, 
        ///                                  RSESCI_POL_POS, 3e6, 0, 1);
        /// 
        ///   (3) Start measurement and read results:
        ///       rsesci_ReadIQTraceData (instrSession, RSESCI_SCREEN_A,
        ///                               bufferSize, &amp;points, I, Q);
        /// 
        ///   (4) Read memory results:
        ///       rsesci_ReadIQTraceDataMemoryAIQ (instrSession,
        ///                                        RSESCI_SCREEN_A, offset,
        ///                                        noSamples, bufferSize,
        ///                                        &amp;points, I, Q);
        /// 
        ///   (5) Switch off I/Q data acquisition:
        ///       rsesci_SetIQTrace (instrSession, RSESCI_SCREEN_A,
        ///                          VI_FALSE);
        /// 
        /// Remote-control command(s):
        /// TRACe&lt;1|2&gt;:IQ:DATA:MEMory:AIQ? &lt;offset samples&gt;,&lt;# of samples&gt;
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Offset_Samples">
        /// This control sets offset of the values to be read related to the start of the acquired data.
        /// 
        /// Valid Range: 0 to &lt;# of samples&gt; - 1
        /// 
        /// Default Value: 0
        /// 
        /// </param>
        /// <param name="No_of_Samples">
        /// This control sets the number of measurement values to be read.
        /// 
        /// Valid Range: 1 to &lt;# of samples&gt; - &lt;offset samples&gt;
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <param name="Buffer_Size">
        /// Pass the number of doubles in the array you specify for the real parts and imaginary parameters.
        /// 
        /// The buffer must contain at least as many elements as the value you specify with the Buffer Size parameter. If the data contains more elemens than you specify with the Buffer Size parameter, the function returns Buffer Size elements into the arrays and returns the buffer size you must pass to get the all data.  For example, if the data contains 16000 elements for real parts and 16000 elements for imaginary part and the Buffer Size is 10000, the function places 10000 elements into the arrays and returns 16000.
        /// 
        /// Notes:
        /// 
        /// (1) If VI_ATTR_INTF_TYPE is equal to VI_INTF_ASRL, then use this parameter to pass the expected number of points to be read from the instrument. Data points will be read until instruments output buffer is empty. If Buffer Size is lower than actual number of results available in the instruments output buffer, then this function returns an error code (BFFC0002  Parameter 2 (Buffer Size) out of range.) and measurement results will not be valid. This is due to limitation if the unit is remote-controlled via the RS-232-C interface.
        /// 
        /// </param>
        /// <param name="No_of_Points">
        /// Returns the number of I/Q trace data points.
        /// 
        /// </param>
        /// <param name="Real_Parts__I_">
        /// Returns an array of trace data (I-value x, real part) point. The buffer must contain at least as many elements as the value you specify with the Buffer Size parameter.
        /// 
        /// </param>
        /// <param name="Imaginary_Parts__Q_">
        /// Returns an array of trace data (Q-value y, imaginary part) point. The buffer must contain at least as many elements as the value you specify with the Buffer Size parameter.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int ReadIQTraceDataMemoryAIQ(int Window, int Offset_Samples, int No_of_Samples, int Buffer_Size, out int No_of_Points, double[] Real_Parts__I_, double[] Imaginary_Parts__Q_)
        {
            int pInvokeResult = PInvoke.ReadIQTraceDataMemoryAIQ(this._handle, Window, Offset_Samples, No_of_Samples, Buffer_Size, out No_of_Points, Real_Parts__I_, Imaginary_Parts__Q_);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function initiates the acquisition of system error correction data. A "0" is returned if the  acquisition was successful.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// CALibration[:ALL]?
        /// </summary>
        /// <param name="Result">
        /// This control returns the result of calibration.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int CalibrationTotal(out int Result)
        {
            int pInvokeResult = PInvoke.CalibrationTotal(this._handle, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function aborts the acquisition of correction data and restores the last complete correction data set.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// CALibration:ABORt
        /// </summary>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int CalibrationAbort()
        {
            int pInvokeResult = PInvoke.CalibrationAbort(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function outputs the results of the correction data acquisition. The lines of the result table (see manual section "Recording the correction data of R&amp;S ESCI - CAL key") are output as string data separated by commas.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// CALibration:RESult?
        /// </summary>
        /// <param name="Calibration_Result">
        /// This control returns the results of the correction data acquisition. The lines of the result table are output as string data separated by commas.
        /// 
        /// Notes:
        /// 
        /// (1) The array must contain at least 256 elements ViChar[256].
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int CalibrationResults(System.Text.StringBuilder Calibration_Result)
        {
            int pInvokeResult = PInvoke.CalibrationResults(this._handle, Calibration_Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function determines whether the current calibration data are taken into account by the instrument (On) or not (Off).
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// CALibration:STATe ON | OFF
        /// </summary>
        /// <param name="Calibration_State">
        /// This control determines whether the current calibration data are taken into account by the instrument (ON) or not (OFF).
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetCalibrationCorrection(bool Calibration_State)
        {
            int pInvokeResult = PInvoke.SetCalibrationCorrection(this._handle, System.Convert.ToUInt16(Calibration_State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function reads whether the current calibration data are taken into account by the instrument (On) or not (Off).
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// CALibration:STATe?
        /// </summary>
        /// <param name="Calibration_State">
        /// This control returns whether the current calibration data are taken into account by the instrument (ON) or not (OFF).
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetCalibrationCorrection(out bool Calibration_State)
        {
            ushort Calibration_StateAsUShort;
            int pInvokeResult = PInvoke.GetCalibrationCorrection(this._handle, out Calibration_StateAsUShort);
            Calibration_State = System.Convert.ToBoolean(Calibration_StateAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches on or off the automatic correction of the YIG filter frequency drift.
        /// When correction is switched on, it is checked once per minute whether the temperature on the YIG filter has changed by more than 5K relative to the last instance of correction. If this is the case, the frequency of the YIG filter is ? at the end of the next measurement ? adjusted as required for the new temperature. For time-critical applications, the correction function can be switched off after an operating period of &gt;= 30 minutes.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]CORRection:YIG:TEMPerature:AUTO ON | OFF
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="State">
        /// This control switches on or off the automatic correction of the YIG filter frequency drift.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_TRUE (1)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetYIGCorrectionState(int Window, bool State)
        {
            int pInvokeResult = PInvoke.SetYIGCorrectionState(this._handle, Window, System.Convert.ToUInt16(State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the state of the automatic correction of the YIG filter frequency drift.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]CORRection:YIG:TEMPerature:AUTO?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="State">
        /// This control returns the state of the automatic correction of the YIG filter frequency drift.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetYIGCorrectionState(int Window, out bool State)
        {
            ushort StateAsUShort;
            int pInvokeResult = PInvoke.GetYIGCorrectionState(this._handle, Window, out StateAsUShort);
            State = System.Convert.ToBoolean(StateAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function initiates a new sweep.
        /// In receiver mode with 'Single' selected, the R&amp;S ESCI performs a single scan and stops at the end frequency. With 'Continuous' selected, the scan is performed continuously until it is deliberately stopped.
        /// In analyzer mode, with Sweep Count &gt; 0 or Average Count &gt; 0, this means a restart of the indicated number of measurements. With trace functions 'Max Hold', 'Min Hold' and 'Average', the previous results are reset on restarting the measurement.
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// INITiate2[:IMMediate]
        /// </summary>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int RunScan()
        {
            int pInvokeResult = PInvoke.RunScan(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function interrupts a current scan measurement.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// HOLD
        /// </summary>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int HoldScan()
        {
            int pInvokeResult = PInvoke.HoldScan(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// When a scan is automatically interrupted by a transducer interrupt point, this function continues the measurement at the current receiver frequency. The function is useful especially for trace functions MAXHold, MINHold and AVERage if the previous
        /// results are not to be cleared with Sweep Count &gt; 0 or Average Count &gt; 0 on restarting the measurement (INIT:IMMediate resets the previous results on restarting the measurement). The single-sweep mode is automatically switched on. In the continuous-sweep mode, synchronization to the sweep end is not possible since the overall measurement "never" ends.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// INITiate2:CONMeas
        /// </summary>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int ContinueScan()
        {
            int pInvokeResult = PInvoke.ContinueScan(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function aborts a current measurement and resets the trigger system.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// ABORt
        /// </summary>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int StopScan()
        {
            int pInvokeResult = PInvoke.StopScan(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function determines whether the trigger system is continuously initiated (continuous) or performs single measurements (single).
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// INITiate2:CONTinuous ON | OFF
        /// </summary>
        /// <param name="Scan_Mode">
        /// This control determines whether the trigger system is continuously initiated (continuous) or performs single measurements (single).
        /// 
        /// Valid Values:
        /// RSESCI_SCAN_SING (0) - Single Scan
        /// RSESCI_SCAN_CONT (1) - Continuous Scan
        /// 
        /// Default Value: RSESCI_SCAN_SING (0)
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetScanMode(int Scan_Mode)
        {
            int pInvokeResult = PInvoke.SetScanMode(this._handle, Scan_Mode);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries whether the trigger system is continuously initiated (continuous) or performs single measurements (single).
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// INITiate2:CONTinuous?
        /// </summary>
        /// <param name="Scan_Mode">
        /// This control returns whether the trigger system is continuously initiated (continuous) or performs single measurements (single).
        /// 
        /// Valid Values:
        /// RSESCI_SCAN_SING (0) - Single Scan
        /// RSESCI_SCAN_CONT (1) - Continuous Scan
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetScanMode(out int Scan_Mode)
        {
            int pInvokeResult = PInvoke.GetScanMode(this._handle, out Scan_Mode);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the overall measurement time for time domain analysis.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SCAN:TDOMain
        /// </summary>
        /// <param name="Measurement_Time">
        /// This control sets the overall measurement time for time domain analysis.
        /// 
        /// Valid Range: 100.0e-6 to 10000.0 s
        /// 
        /// Default Value: 10.0
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetScanTimeDomain(double Measurement_Time)
        {
            int pInvokeResult = PInvoke.SetScanTimeDomain(this._handle, Measurement_Time);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the overall measurement time for time domain analysis.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SCAN:TDOMain?
        /// </summary>
        /// <param name="Measurement_Time">
        /// This control returns the overall measurement time for time domain analysis.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetScanTimeDomain(out double Measurement_Time)
        {
            int pInvokeResult = PInvoke.GetScanTimeDomain(this._handle, out Measurement_Time);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function toggles between linear and logarithmic display in receiver mode.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACE&lt;1...3&gt;:X:SPACing LINear | LOGarithmic
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Display_Spacing">
        /// This control toggles between linear and logarithmic display in receiver mode.
        /// 
        /// Valid Values:
        /// RSESCI_SPACING_LIN (0) - Linear
        /// RSESCI_SPACING_LOG (1) - Logarithmic
        /// 
        /// Default Value: RSESCI_SPACING_LIN (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetScanDisplaySpacing(int Window, int Display_Spacing)
        {
            int pInvokeResult = PInvoke.SetScanDisplaySpacing(this._handle, Window, Display_Spacing);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries setting of display spacing in receiver mode.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACE&lt;1...3&gt;:X:SPACing?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Display_Spacing">
        /// This control returns setting of display spacing in receiver mode.
        /// 
        /// Valid Values:
        /// RSESCI_LIN (0) - Linear
        /// RSESCI_LOG (1) - Logarithmic
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetScanDisplaySpacing(int Window, out int Display_Spacing)
        {
            int pInvokeResult = PInvoke.GetScanDisplaySpacing(this._handle, Window, out Display_Spacing);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function determines the number of ranges. With setting 0, the scan is performed using the current receiver settings and not the settings scan table group.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SCAN&lt;1...10&gt;:RANGes[:COUNt]
        /// </summary>
        /// <param name="Number_of_ranges">
        /// This control determines the number of ranges.
        /// 
        /// Valid Range: 1 to 10
        /// 
        /// Default Value: 1
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetScanRanges(int Number_of_ranges)
        {
            int pInvokeResult = PInvoke.SetScanRanges(this._handle, Number_of_ranges);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the number of ranges.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SCAN:RANGes[:COUNt]?
        /// </summary>
        /// <param name="Number_of_ranges">
        /// This control returns the number of ranges.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetScanRanges(out int Number_of_ranges)
        {
            int pInvokeResult = PInvoke.GetScanRanges(this._handle, out Number_of_ranges);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the maximum number of measurement points which are transferred in one block after the query command TRACE? SCAN. The total amount of bytes which is transferred depends on the number of active traces.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// TRACe&lt;1|2&gt;:POINTs
        /// </summary>
        /// <param name="Number_of_Points">
        /// This control defines the maximum number of measurement points which are transferred in one block after the query command TRACE? SCAN. The total amount of bytes which is transferred depends on the number of active traces.
        /// 
        /// Valid Values:
        /// 1 to 10000
        /// 
        /// Default Value: 1000
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetScanPoints(int Number_of_Points)
        {
            int pInvokeResult = PInvoke.SetScanPoints(this._handle, Number_of_Points);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the start frequency of the selected receiver scan range.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SCAN&lt;1...10&gt;:STARt
        /// </summary>
        /// <param name="Scan_Range">
        /// This control sets the scan range.
        /// 
        /// Valid Range: 1 to 10
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Start_Frequency">
        /// This control defines the start frequency of the selected receiver scan range.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: 150.e3 Hz
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetScanStart(int Scan_Range, double Start_Frequency)
        {
            int pInvokeResult = PInvoke.SetScanStart(this._handle, Scan_Range, Start_Frequency);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the start frequency of the selected receiver scan range.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SCAN&lt;1...10&gt;:STARt?
        /// </summary>
        /// <param name="Scan_Range">
        /// This control sets the scan range.
        /// 
        /// Valid Range: 1 to 10
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Start_Frequency">
        /// This control returns the start frequency of the selected receiver scan range.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetScanStart(int Scan_Range, out double Start_Frequency)
        {
            int pInvokeResult = PInvoke.GetScanStart(this._handle, Scan_Range, out Start_Frequency);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the stop frequency of the selected receiver scan range.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SCAN&lt;1...10&gt;:STOP
        /// </summary>
        /// <param name="Scan_Range">
        /// This control sets the scan range.
        /// 
        /// Valid Range: 1 to 10
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Stop_Frequency">
        /// This control defines the stop frequency of the selected receiver scan range.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: 30.e6 Hz
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetScanStop(int Scan_Range, double Stop_Frequency)
        {
            int pInvokeResult = PInvoke.SetScanStop(this._handle, Scan_Range, Stop_Frequency);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the stop frequency of the selected receiver scan range.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SCAN&lt;1...10&gt;:STOP?
        /// </summary>
        /// <param name="Scan_Range">
        /// This control sets the scan range.
        /// 
        /// Valid Range: 1 to 10
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Stop_Frequency">
        /// This control returns the stop frequency of the selected receiver scan range.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetScanStop(int Scan_Range, out double Stop_Frequency)
        {
            int pInvokeResult = PInvoke.GetScanStop(this._handle, Scan_Range, out Stop_Frequency);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the step size for the frequency of the selected receiver scan range.
        /// 
        /// Note(s):
        /// 
        /// With STEP AUTO selected, the step size cannot be changed because it is automatically set with respect to the IF bandwidth.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SCAN&lt;1...10&gt;:STEP
        /// </summary>
        /// <param name="Scan_Range">
        /// This control sets the scan range.
        /// 
        /// Valid Range: 1 to 10
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Step_Size">
        /// This control defines the step size for the frequency of the selected receiver scan range.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: 4.0e3 Hz
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetScanStepSize(int Scan_Range, double Step_Size)
        {
            int pInvokeResult = PInvoke.SetScanStepSize(this._handle, Scan_Range, Step_Size);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the step size for the frequency of the selected receiver scan range.
        /// 
        /// Note(s):
        /// 
        /// With STEP AUTO selected, the step size cannot be changed because it is automatically set with respect to the IF bandwidth.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SCAN&lt;1...10&gt;:STEP?
        /// </summary>
        /// <param name="Scan_Range">
        /// This control sets the scan range.
        /// 
        /// Valid Range: 1 to 10
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Step_Size">
        /// This control returns the step size for the frequency of the selected receiver scan range.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetScanStepSize(int Scan_Range, out double Step_Size)
        {
            int pInvokeResult = PInvoke.GetScanStepSize(this._handle, Scan_Range, out Step_Size);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the bandwidth for the selected receiver scan range.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SCAN&lt;1...10&gt;:BANDwidth:RESolution
        /// </summary>
        /// <param name="Scan_Range">
        /// This control sets the scan range.
        /// 
        /// Valid Range: 1 to 10
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Bandwidth">
        /// This control defines the bandwidth for the selected receiver scan range.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: 9.0e3 Hz
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetScanBandwidth(int Scan_Range, double Bandwidth)
        {
            int pInvokeResult = PInvoke.SetScanBandwidth(this._handle, Scan_Range, Bandwidth);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the bandwidth for the selected receiver scan range.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SCAN&lt;1...10&gt;:BANDwidth:RESolution?
        /// </summary>
        /// <param name="Scan_Range">
        /// This control sets the scan range.
        /// 
        /// Valid Range: 1 to 10
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Bandwidth">
        /// This control returns the bandwidth for the selected receiver scan range.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetScanBandwidth(int Scan_Range, out double Bandwidth)
        {
            int pInvokeResult = PInvoke.GetScanBandwidth(this._handle, Scan_Range, out Bandwidth);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the measurement time of the receiver subscan.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SCAN&lt;1...10&gt;:TIME
        /// </summary>
        /// <param name="Scan_Range">
        /// This control sets the scan range.
        /// 
        /// Valid Range: 1 to 10
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Measurement_Time">
        /// This control defines the measurement time of the receiver subscan.
        /// 
        /// Valid Range: 100.0e-6 to 100.0 s
        /// 
        /// Default Value: 0.001 s
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetScanMeasurementTime(int Scan_Range, double Measurement_Time)
        {
            int pInvokeResult = PInvoke.SetScanMeasurementTime(this._handle, Scan_Range, Measurement_Time);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the measurement time of the receiver subscan.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SCAN&lt;1...10&gt;:TIME?
        /// </summary>
        /// <param name="Scan_Range">
        /// This control sets the scan range.
        /// 
        /// Valid Range: 1 to 10
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Measurement_Time">
        /// This control returns the measurement time of the receiver subscan.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetScanMeasurementTime(int Scan_Range, out double Measurement_Time)
        {
            int pInvokeResult = PInvoke.GetScanMeasurementTime(this._handle, Scan_Range, out Measurement_Time);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches on or off the autoranging function in the selected receiver scan range.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SCAN&lt;1...10&gt;:INPut:ATTenuation:AUTO ON | OFF
        /// </summary>
        /// <param name="Scan_Range">
        /// This control sets the scan range.
        /// 
        /// Valid Range: 1 to 10
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Auto_Ranging">
        /// This control switches on or off the autoranging function in the selected receiver scan range.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetScanAutoRanging(int Scan_Range, bool Auto_Ranging)
        {
            int pInvokeResult = PInvoke.SetScanAutoRanging(this._handle, Scan_Range, System.Convert.ToUInt16(Auto_Ranging));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the state of the autoranging function in the selected receiver scan range.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SCAN&lt;1...10&gt;:INPut:ATTenuation:AUTO?
        /// </summary>
        /// <param name="Scan_Range">
        /// This control sets the scan range.
        /// 
        /// Valid Range: 1 to 10
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Auto_Ranging">
        /// This control returns the state of the autoranging function in the selected receiver scan range.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetScanAutoRanging(int Scan_Range, out bool Auto_Ranging)
        {
            ushort Auto_RangingAsUShort;
            int pInvokeResult = PInvoke.GetScanAutoRanging(this._handle, Scan_Range, out Auto_RangingAsUShort);
            Auto_Ranging = System.Convert.ToBoolean(Auto_RangingAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the RF attenuation for the selected receiver scan range.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SCAN&lt;1...10&gt;:INPut:ATTenuation
        /// </summary>
        /// <param name="Scan_Range">
        /// This control sets the scan range.
        /// 
        /// Valid Range: 1 to 10
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Attenuation">
        /// This control defines the RF attenuation for the selected receiver scan range.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: 10.0 dB
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetScanRFAttenuation(int Scan_Range, double Attenuation)
        {
            int pInvokeResult = PInvoke.SetScanRFAttenuation(this._handle, Scan_Range, Attenuation);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the RF attenuation for the selected receiver scan range.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SCAN&lt;1...10&gt;:INPut:ATTenuation?
        /// </summary>
        /// <param name="Scan_Range">
        /// This control sets the scan range.
        /// 
        /// Valid Range: 1 to 10
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Attenuation">
        /// This control returns the RF attenuation for the selected receiver scan range.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetScanRFAttenuation(int Scan_Range, out double Attenuation)
        {
            int pInvokeResult = PInvoke.GetScanRFAttenuation(this._handle, Scan_Range, out Attenuation);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches on or off the preamplifier in the selected receiver scan range.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SCAN&lt;1...10&gt;:INPut:GAIN[:STATe] ON | OFF
        /// </summary>
        /// <param name="Scan_Range">
        /// This control sets the scan range.
        /// 
        /// Valid Range: 1 to 10
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Preamplifier_State">
        /// This control switches on or off the autoranging function in the selected receiver scan range.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetScanPreamplifier(int Scan_Range, bool Preamplifier_State)
        {
            int pInvokeResult = PInvoke.SetScanPreamplifier(this._handle, Scan_Range, System.Convert.ToUInt16(Preamplifier_State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the state of the preamplifier in the selected receiver scan range.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SCAN&lt;1...10&gt;:INPut:GAIN[:STATe]?
        /// </summary>
        /// <param name="Scan_Range">
        /// This control sets the scan range.
        /// 
        /// Valid Range: 1 to 10
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Preamplifier_State">
        /// This control returns the state of the preamplifier in the selected receiver scan range.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetScanPreamplifier(int Scan_Range, out bool Preamplifier_State)
        {
            ushort Preamplifier_StateAsUShort;
            int pInvokeResult = PInvoke.GetScanPreamplifier(this._handle, Scan_Range, out Preamplifier_StateAsUShort);
            Preamplifier_State = System.Convert.ToBoolean(Preamplifier_StateAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function enables or disables the preamplifier in the autoranging function of the selected receiver scan range.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SCAN&lt;1...10&gt;:INPut:GAIN:AUTO ON | OFF
        /// </summary>
        /// <param name="Scan_Range">
        /// This control sets the scan range.
        /// 
        /// Valid Range: 1 to 10
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Preamplifier_In_Autoranging">
        /// This control enables or disables the preamplifier in the autoranging function of the selected receiver scan range.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetScanAutoPreamplifier(int Scan_Range, bool Preamplifier_In_Autoranging)
        {
            int pInvokeResult = PInvoke.SetScanAutoPreamplifier(this._handle, Scan_Range, System.Convert.ToUInt16(Preamplifier_In_Autoranging));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the state of the preamplifier in the autoranging function of the selected receiver scan range.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SCAN&lt;1...10&gt;:INPut:GAIN:AUTO?
        /// </summary>
        /// <param name="Scan_Range">
        /// This control sets the scan range.
        /// 
        /// Valid Range: 1 to 10
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Preamplifier_In_Autoranging">
        /// This control returns the state of the preamplifier in the autoranging function of the selected receiver scan range.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetScanAutoPreamplifier(int Scan_Range, out bool Preamplifier_In_Autoranging)
        {
            ushort Preamplifier_In_AutorangingAsUShort;
            int pInvokeResult = PInvoke.GetScanAutoPreamplifier(this._handle, Scan_Range, out Preamplifier_In_AutorangingAsUShort);
            Preamplifier_In_Autoranging = System.Convert.ToBoolean(Preamplifier_In_AutorangingAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches the low noise preamplifier on or off for a receiver subscan.
        /// 
        /// Note(s):
        /// 
        /// (1) Only with option R&amp;S ESU-B24
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SCAN&lt;1 to 10&gt;:INPut:GAIN:LNA[:STATe] ON | OFF
        /// </summary>
        /// <param name="Scan_Range">
        /// This control sets the scan range.
        /// 
        /// Valid Range: 1 to 10
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Low_Noise_Preamplifier_State">
        /// This control switches the low noise preamplifier on or off for a receiver subscan.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetScanLowNoisePreamplifierState(int Scan_Range, bool Low_Noise_Preamplifier_State)
        {
            int pInvokeResult = PInvoke.SetScanLowNoisePreamplifierState(this._handle, Scan_Range, System.Convert.ToUInt16(Low_Noise_Preamplifier_State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries state of the low noise preamplifier for a receiver subscan.
        /// 
        /// Note(s):
        /// 
        /// (1) Only with option R&amp;S ESU-B24
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SCAN&lt;1 to 10&gt;:INPut:GAIN:LNA[:STATe]?
        /// </summary>
        /// <param name="Scan_Range">
        /// This control sets the scan range.
        /// 
        /// Valid Range: 1 to 10
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Low_Noise_Preamplifier_State">
        /// This control returns state of the low noise preamplifier for a receiver subscan.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetScanLowNoisePreamplifierState(int Scan_Range, out bool Low_Noise_Preamplifier_State)
        {
            ushort Low_Noise_Preamplifier_StateAsUShort;
            int pInvokeResult = PInvoke.GetScanLowNoisePreamplifierState(this._handle, Scan_Range, out Low_Noise_Preamplifier_StateAsUShort);
            Low_Noise_Preamplifier_State = System.Convert.ToBoolean(Low_Noise_Preamplifier_StateAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the input for a receiver subscan.
        /// 
        /// Note(s):
        /// (1) This function is available with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SCAN&lt;1 to 10&gt;:INPut:TYPE INPUT1 | INPUT2
        /// </summary>
        /// <param name="Scan_Range">
        /// This control sets the scan range.
        /// 
        /// Valid Range: 1 to 10
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Input">
        /// This control defines the input for a receiver subscan.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetScanInput(int Scan_Range, int Input)
        {
            int pInvokeResult = PInvoke.SetScanInput(this._handle, Scan_Range, Input);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the input for a receiver subscan.
        /// 
        /// Note(s):
        /// (1) This function is available with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SCAN&lt;1 to 10&gt;:INPut:TYPE?
        /// </summary>
        /// <param name="Scan_Range">
        /// This control sets the scan range.
        /// 
        /// Valid Range: 1 to 10
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Input">
        /// This control returns the input for a receiver subscan.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetScanInput(int Scan_Range, out int Input)
        {
            int pInvokeResult = PInvoke.GetScanInput(this._handle, Scan_Range, out Input);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches on or off selected marker in the selected measurement window. If marker 2, 3 or 4 is selected and used as a delta marker, it is switched to marker mode.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;[:STATe] ON | OFF
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Marker">
        /// This control sets the marker.
        /// 
        /// Valid Range: 1 to 4
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="State">
        /// This control switches on or off the selected marker in the selected measurement window.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetMarker(int Window, int Marker, bool State)
        {
            int pInvokeResult = PInvoke.SetMarker(this._handle, Window, Marker, System.Convert.ToUInt16(State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function  returns the state of selected marker in the selected measurement window.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;[:STATe]?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Marker">
        /// This control sets the marker.
        /// 
        /// Valid Range: 1 to 4
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="State">
        /// This control returns the state of selected marker in the selected measurement window.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetMarker(int Window, int Marker, out bool State)
        {
            ushort StateAsUShort;
            int pInvokeResult = PInvoke.GetMarker(this._handle, Window, Marker, out StateAsUShort);
            State = System.Convert.ToBoolean(StateAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches off all active markers in the selected measurement window. All delta markers and active marker/delta marker measurement functions are switched off.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:AOFF
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int AllMarkersOff(int Window)
        {
            int pInvokeResult = PInvoke.AllMarkersOff(this._handle, Window);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function positions the selected marker to the indicated frequency (span &gt; 0), time (span = 0) or level (APD measurement or CCDF measurement ON) in the selected measurement window. If marker 2, 3 or 4 is selected and used as delta marker, it is switched to marker mode.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:X
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Marker">
        /// This control sets the marker.
        /// 
        /// Valid Range: 1 to 4
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="X">
        /// This control defines the marker's position.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: 0.0
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetMarkerPosition(int Window, int Marker, double X)
        {
            int pInvokeResult = PInvoke.SetMarkerPosition(this._handle, Window, Marker, X);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the marker's position.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:X?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Marker">
        /// This control sets the marker.
        /// 
        /// Valid Range: 1 to 4
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="X">
        /// This control returns the marker's position.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetMarkerPosition(int Window, int Marker, out double X)
        {
            int pInvokeResult = PInvoke.GetMarkerPosition(this._handle, Window, Marker, out X);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the measured value of the selected marker in the selected measurement window. The corresponding marker is activated before or switched to marker mode, if necessary. To obtain a valid query result, a complete sweep with synchronization to the sweep end must be performed between the activation of the marker and the query of the y value. This is only possible in single sweep mode.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:Y?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Marker">
        /// This control sets the marker.
        /// 
        /// Valid Range: 1 to 4
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Y">
        /// This control queries the marker's value.
        /// In the default setting, the output is made depending on the unit determined with rsesci_SetLevelAxisUnit(); only with linear level scaling is the output in %.
        /// 
        /// If the FM Demodulator (FS-K7) is activated, the query result is output in the following units:
        /// 
        /// - Result display FM: Hz
        /// - Result display RF POWER LOG: dBm
        /// - Result display RF POWER LIN: %
        /// - Result display SPECTRUM LOG: dBm
        /// - Result display SPECTRUM LIN: %
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetMarkerValue(int Window, int Marker, out double Y)
        {
            int pInvokeResult = PInvoke.GetMarkerValue(this._handle, Window, Marker, out Y);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the range to be zoomed around marker 1 in the selected measurement window. Marker 1 is activated first, if necessary.
        /// The subsequent frequency sweep is stopped at the marker position and the frequency of the signal is counted. This frequency becomes the new center frequency, and the zoomed span is set. In order to recognize the end of the operation the synchronization to the sweep end should be activated. This is only possible in single sweep mode.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis (frequency domain only)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:ZOOM
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Zoom">
        /// This control defines the range to be zoomed around marker 1 in the selected measurement window.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: 0.0
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetMarkerZoom(int Window, double Zoom)
        {
            int pInvokeResult = PInvoke.SetMarkerZoom(this._handle, Window, Zoom);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the range zoomed around marker 1 in the selected measurement window.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis (frequency domain only)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:ZOOM?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Zoom">
        /// This control returns the range zoomed around marker 1 in the selected measurement window.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetMarkerZoom(int Window, out double Zoom)
        {
            int pInvokeResult = PInvoke.GetMarkerZoom(this._handle, Window, out Zoom);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches on or off the frequency counter at the marker position in the selected measurement window. The count result is queried with rsesci_GetMarkerFrequencyCounterValue().
        /// 
        /// Frequency counting is possible only for one marker at a time for each measurement window. If it is activated for another marker, it is automatically de-activated for the previous marker.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Note(s):
        /// (1) Complete sweep must be performed after switching on the frequency counter to ensure that the frequency to be measured is actually reached. The synchronization to the sweep end required for this is possible only in single-sweep mode.
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:COUNt ON | OFF
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Marker">
        /// This control sets the marker.
        /// 
        /// Valid Range: 1 to 4
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Counter_State">
        /// This control switches on or off the frequency counter at the marker position in the selected measurement window.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetMarkerFrequencyCounter(int Window, int Marker, bool Counter_State)
        {
            int pInvokeResult = PInvoke.SetMarkerFrequencyCounter(this._handle, Window, Marker, System.Convert.ToUInt16(Counter_State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the state of frequency counter at the marker position in the selected measurement window.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:COUNt?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Marker">
        /// This control sets the marker.
        /// 
        /// Valid Range: 1 to 4
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Counter_State">
        /// This control returns the state of frequency counter at the marker position in the selected measurement window.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetMarkerFrequencyCounter(int Window, int Marker, out bool Counter_State)
        {
            ushort Counter_StateAsUShort;
            int pInvokeResult = PInvoke.GetMarkerFrequencyCounter(this._handle, Window, Marker, out Counter_StateAsUShort);
            Counter_State = System.Convert.ToBoolean(Counter_StateAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function specifies the resolution of the frequency counter in the selected measurement window.The setting is independent of the selected marker.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:COUNt:RESolution 0.1 | 1 | 10 | 100 | 1000 | 10000 Hz
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Resolution">
        /// This control specifies the resolution of the frequency counter in the selected measurement window.
        /// 
        /// Valid Values: 0.1, 1, 10, 100, 1000, 10000 Hz
        /// 
        /// Default Value: 1000.0 Hz
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetMarkerFrequencyCounterResolution(int Window, double Resolution)
        {
            int pInvokeResult = PInvoke.SetMarkerFrequencyCounterResolution(this._handle, Window, Resolution);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the resolution of the frequency counter in the selected measurement window.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:COUNt:RESolution?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Resolution">
        /// This control returns the resolution of the frequency counter in the selected measurement window.
        /// 
        /// Valid Values: 0.1, 1, 10, 100, 1000, 10000 Hz
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetMarkerFrequencyCounterResolution(int Window, out double Resolution)
        {
            int pInvokeResult = PInvoke.GetMarkerFrequencyCounterResolution(this._handle, Window, out Resolution);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the result of the frequency counter for the indicated marker in the selected measurement window. Before the command, the frequency counter should be switched on and a
        /// complete measurement performed to obtain a valid count result. Therefore, a single sweep with synchronization must be performed between switching on the frequency counter and querying the
        /// count result.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:COUNt:FREQuency?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Marker">
        /// This control sets the marker.
        /// 
        /// Valid Range: 1 to 4
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Frequency_Counter_Value">
        /// This function queries the result of the frequency counter for the indicated marker in the selected measurement window. Before the command, the frequency counter should be switched on and a
        /// complete measurement performed to obtain a valid count result. Therefore, a single sweep with synchronization must be performed between switching on the frequency counter and querying the
        /// count result.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetMarkerFrequencyCounterValue(int Window, int Marker, out double Frequency_Counter_Value)
        {
            int pInvokeResult = PInvoke.GetMarkerFrequencyCounterValue(this._handle, Window, Marker, out Frequency_Counter_Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches on and off the delta marker when delta marker 1 is selected. The corresponding marker becomes the delta marker when delta marker 2 to 4 is selected. If the corresponding marker is not activated, it will be activated and positioned on the maximum of the measurement curve.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:DELTamarker&lt;1...4&gt;[:STATe] ON | OFF
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Marker">
        /// This control sets the marker.
        /// 
        /// Valid Range: 1 to 4
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="State">
        /// This control switches on and off the delta marker.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetDeltaMarker(int Window, int Marker, bool State)
        {
            int pInvokeResult = PInvoke.SetDeltaMarker(this._handle, Window, Marker, System.Convert.ToUInt16(State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function This control returns the state of the delta marker.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:DELTamarker&lt;1...4&gt;[:STATe]?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Marker">
        /// This control sets the marker.
        /// 
        /// Valid Range: 1 to 4
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="State">
        /// This control returns the state of the delta marker.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetDeltaMarker(int Window, int Marker, out bool State)
        {
            ushort StateAsUShort;
            int pInvokeResult = PInvoke.GetDeltaMarker(this._handle, Window, Marker, out StateAsUShort);
            State = System.Convert.ToBoolean(StateAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches off all active delta markers in the selected measurement window (screen A or screen B).
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:DELTamarker&lt;1...4&gt;:AOFF
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int AllDeltaMarkersOff(int Window)
        {
            int pInvokeResult = PInvoke.AllDeltaMarkersOff(this._handle, Window);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches between relative and absolute frequency input of the delta marker (or time with span = 0). It affects all delta markers independent of the measurement window.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:DELTamarker&lt;1...4&gt;:MODE ABSolute | RELative
        /// </summary>
        /// <param name="Delta_Marker_Mode">
        /// This control switches between relative and absolute frequency input of the delta marker.
        /// 
        /// Valid Values:
        /// RSESCI_ABS (0) - Absolute
        /// RSESCI_REL (1) - Relative
        /// 
        /// Default Value: RSESCI_ABS (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetDeltaMarkerMode(int Delta_Marker_Mode)
        {
            int pInvokeResult = PInvoke.SetDeltaMarkerMode(this._handle, Delta_Marker_Mode);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns selected delta marker mode.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:DELTamarker&lt;1...4&gt;:MODE?
        /// </summary>
        /// <param name="Delta_Marker_Mode">
        /// This control returns selected delta marker mode.
        /// 
        /// Valid Values:
        /// RSESCI_ABS (0) - Absolute
        /// RSESCI_REL (1) - Relative
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetDeltaMarkerMode(out int Delta_Marker_Mode)
        {
            int pInvokeResult = PInvoke.GetDeltaMarkerMode(this._handle, out Delta_Marker_Mode);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function positions the selected delta marker in the indicated measurement window to the indicated frequency (span &gt; 0), time (span = 0) or level (APD measurement = ON or CCDFmeasurement = ON). The input is in absolute values or relative to marker 1 depending on the setting of rsesci_SetDeltaMarkerMode(). If Reference Fixed measurement
        /// (rsesci_SetDeltaMarkerReferenceFixed) is active, relative values refer to the reference position are entered.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:DELTamarker&lt;1...4&gt;:X
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Marker">
        /// This control sets the marker.
        /// 
        /// Valid Range: 1 to 4
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Position">
        /// This control positions the selected delta marker in the indicated measurement window to the indicated frequency (span &gt; 0), time (span = 0) or level (APD measurement = ON or CCDFmeasurement = ON).
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: 0.0
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetDeltaMarkerPosition(int Window, int Marker, double Position)
        {
            int pInvokeResult = PInvoke.SetDeltaMarkerPosition(this._handle, Window, Marker, Position);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the position of selected delta marker in selected window.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:DELTamarker&lt;1...4&gt;:X?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Marker">
        /// This control sets the marker.
        /// 
        /// Valid Range: 1 to 4
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Position">
        /// This control returns the position of selected delta marker in selected window.
        /// It always returns absolute values.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetDeltaMarkerPosition(int Window, int Marker, out double Position)
        {
            int pInvokeResult = PInvoke.GetDeltaMarkerPosition(this._handle, Window, Marker, out Position);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the frequency (span &gt; 0) or time (span = 0) of the selected delta marker relative to marker 1 or to the reference position ( for rsesci_SetDeltaMarkerReferenceFixed On). The function activates the corresponding delta marker, if necessary.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:DELTamarker&lt;1...4&gt;:X:RELative?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Marker">
        /// This control sets the marker.
        /// 
        /// Valid Range: 1 to 4
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Position">
        /// This control returns the relative position of selected delta marker in selected window.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetDeltaMarkerRelativePosition(int Window, int Marker, out double Position)
        {
            int pInvokeResult = PInvoke.GetDeltaMarkerRelativePosition(this._handle, Window, Marker, out Position);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the measured value of the selected delta marker in the indicated measurement window. The corresponding delta marker will be activated, if necessary. The output is
        /// always a relative value referred to marker 1 or to the reference position (reference fixed active). To obtain a valid query result, a complete sweep with synchronization to the sweep end must be performed between the activation of the delta marker and the query of the y value. This is only possible in single sweep mode.
        /// In receiver mode, the markers can be only activated after a scan is carried out.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:DELTamarker&lt;1...4&gt;:Y?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Marker">
        /// This control sets the marker.
        /// 
        /// Valid Range: 1 to 4
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Y">
        /// This control queries the delta marker's value.
        /// Depending on the unit determined with rsesci_SetLevelAxisUnit() or on the activated measuring functions, the query result is output in the units below:
        /// 
        /// - WATT | VOLT | AMPere: Output unit W | V | A
        /// - Statistics function (APD or CCDF) on: Dimensionless output
        /// - Result display FM (FS-K7): Hz
        /// - Result display RF POWER (FS-K7): dB
        /// - Result display SPECTRUM (FS-K7): dB
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetDeltaMarkerValue(int Window, int Marker, out double Y)
        {
            int pInvokeResult = PInvoke.GetDeltaMarkerValue(this._handle, Window, Marker, out Y);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches the relative measurement to a fixed reference value on or off. Marker 1 will be activated previously and a peak search will be performed, if necessary. If marker 1 is activated, its position becomes the reference point for the measurement. The reference point can then be modified with functions SetDeltaMarkerReferencePointPosition and 
        /// rsesci_SetDeltaMarkerReferencePointLevel independently of the position of marker 1 and of a trace.
        /// 
        /// It is valid for all delta markers in the selected measurement window as long as the function is active.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:DELTamarker&lt;1...4&gt;:FUNCtion:FIXed[:STATe] ON | OFF
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Fixed_Reference_Value">
        /// This control switches the relative measurement to a fixed reference value on or off.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetDeltaMarkerReferenceFixed(int Window, bool Fixed_Reference_Value)
        {
            int pInvokeResult = PInvoke.SetDeltaMarkerReferenceFixed(this._handle, Window, System.Convert.ToUInt16(Fixed_Reference_Value));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the setting of the relative measurement to a fixed reference value.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:DELTamarker&lt;1...4&gt;:FUNCtion:FIXed[:STATe]?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Fixed_Reference_Value">
        /// This control returns the setting of the relative measurement to a fixed reference value.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetDeltaMarkerReferenceFixed(int Window, out bool Fixed_Reference_Value)
        {
            ushort Fixed_Reference_ValueAsUShort;
            int pInvokeResult = PInvoke.GetDeltaMarkerReferenceFixed(this._handle, Window, out Fixed_Reference_ValueAsUShort);
            Fixed_Reference_Value = System.Convert.ToBoolean(Fixed_Reference_ValueAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines a new reference point level for all delta markers in the selected measurement window for a measurement with fixed reference point.
        /// For phase-noise measurements, the function defines a new reference point level for delta marker 2 in the selected measurement window.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:DELTamarker&lt;1...4&gt;:FUNCtion:FIXed:RPOint:Y
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Reference_Point_Level">
        /// This control defines a new reference point level for all delta markers in the selected measurement window for a measurement with fixed reference point.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: 0.0
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetDeltaMarkerReferencePointLevel(int Window, double Reference_Point_Level)
        {
            int pInvokeResult = PInvoke.SetDeltaMarkerReferencePointLevel(this._handle, Window, Reference_Point_Level);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns reference point level for all delta markers in the selected measurement window for a measurement with fixed reference point.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:DELTamarker&lt;1...4&gt;:FUNCtion:FIXed:RPOint:Y?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Reference_Point_Level">
        /// This control returns reference point level for all delta markers in the selected measurement window for a measurement with fixed reference point.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetDeltaMarkerReferencePointLevel(int Window, out double Reference_Point_Level)
        {
            int pInvokeResult = PInvoke.GetDeltaMarkerReferencePointLevel(this._handle, Window, out Reference_Point_Level);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the reference point level for all delta markers in the selected measurement window for a measurement with fixed reference point to the peak of the selected trace.
        /// For phase-noise measurements, the function defines a new reference point level for delta marker 2 in the selected measurement window.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:DELTamarker&lt;1...4&gt;:FUNCtion:FIXed:RPOint:MAXimum[:PEAK]
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetDeltaMarkerReferencePointToPeak(int Window)
        {
            int pInvokeResult = PInvoke.SetDeltaMarkerReferencePointToPeak(this._handle, Window);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines an additional level offset for the measurement with fixed reference value. For this measurement, the offset is included in the display of all delta markers of the selected measurement window.
        /// For phase-noise measurements, the function defines an additional level offset which is included in the display of delta marker 2 in the selected measurement window.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:DELTamarker&lt;1...4&gt;:FUNCtion:FIXed:RPOint:Y:OFFSet
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Offset">
        /// This control defines an additional level offset for the measurement with fixed reference value.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: 0.0
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetDeltaMarkerReferencePointLevelOffset(int Window, double Offset)
        {
            int pInvokeResult = PInvoke.SetDeltaMarkerReferencePointLevelOffset(this._handle, Window, Offset);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries an additional level offset for the measurement with fixed reference value.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:DELTamarker&lt;1...4&gt;:FUNCtion:FIXed:RPOint:Y:OFFSet?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Offset">
        /// This control returns an additional level offset for the measurement with fixed reference value.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetDeltaMarkerReferencePointLevelOffset(int Window, out double Offset)
        {
            int pInvokeResult = PInvoke.GetDeltaMarkerReferencePointLevelOffset(this._handle, Window, out Offset);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines a new reference frequency (span &gt; 0) or time (span = 0) for all delta markers in the selected measurement window for a measurement with fixed reference value.
        /// For phase-noise measurements, the function defines a new reference frequency or time for delta marker 2 in the selected measurement window.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:DELTamarker&lt;1...4&gt;:FUNCtion:FIXed:RPOint:X
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Reference_Point">
        /// This control defines a new reference frequency (span &gt; 0) or time (span = 0) for all delta markers in the selected measurement window for a measurement with fixed reference value.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: 0.0
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetDeltaMarkerReferencePointPosition(int Window, double Reference_Point)
        {
            int pInvokeResult = PInvoke.SetDeltaMarkerReferencePointPosition(this._handle, Window, Reference_Point);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns a reference frequency (span &gt; 0) or time (span = 0) for all delta markers in the selected measurement window for a measurement with fixed reference value.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:DELTamarker&lt;1...4&gt;:FUNCtion:FIXed:RPOint:X?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Reference_Point">
        /// This control returns a reference frequency (span &gt; 0) or time (span = 0) for all delta markers in the selected measurement window for a measurement with fixed reference value.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetDeltaMarkerReferencePointPosition(int Window, out double Reference_Point)
        {
            int pInvokeResult = PInvoke.GetDeltaMarkerReferencePointPosition(this._handle, Window, out Reference_Point);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function positions the marker to the desired position in the trace memory.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:MAXimum[:PEAK]
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:MAXimum:NEXT
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:MAXimum:RIGHt
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:MAXimum:LEFT
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:MINimum[:PEAK]
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:MINimum:NEXT
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:MINimum:RIGHt
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:MINimum:LEFT
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Marker">
        /// This control sets the marker.
        /// 
        /// Valid Range: 1 to 4
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Search_Mode">
        /// This control positions the marker to the desired position in the trace memory.
        /// 
        /// Valid Values:
        /// RSESCI_MARKER_PMAX (0) - Peak Maximum
        /// RSESCI_MARKER_NMAX (1) - Next Maximum
        /// RSESCI_MARKER_LMAX (2) - Left Maximum
        /// RSESCI_MARKER_RMAX (3) - Right Maximum
        /// RSESCI_MARKER_PMIN (4) - Peak Minimum
        /// RSESCI_MARKER_NMIN (5) - Next Minimum
        /// RSESCI_MARKER_LMIN (6) - Left Minimum
        /// RSESCI_MARKER_RMIN (7) - Right Minimum
        /// 
        /// Default Value: RSESCI_MARKER_PMAX (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int MarkerSearch(int Window, int Marker, int Search_Mode)
        {
            int pInvokeResult = PInvoke.MarkerSearch(this._handle, Window, Marker, Search_Mode);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches an automatic maximum or minimum peak search for marker 1 at the end of each particular sweep on and off. The current marker search limit settings (LEFT LIMIT, RIGHT LIMIT, THRESHOLD, EXCLUDE LO) are taken into account. 
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:MAXimum:AUTO ON | OFF
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:MINimum:AUTO ON | OFF
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Auto_Search_Mode">
        /// This control selects which auto search function will be set.
        /// 
        /// Valid Values:
        /// RSESCI_MARKER_PMAX (0) - Maximum
        /// RSESCI_MARKER_PMIN (4) - Minimum
        /// 
        /// Default Value: RSESCI_MARKER_PMAX (0)
        /// </param>
        /// <param name="Auto_Search_State">
        /// This control switches an automatic maximum or minimum peak search for marker 1 at the end of each particular sweep on and off.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetAutoMarkerSearch(int Window, int Auto_Search_Mode, bool Auto_Search_State)
        {
            int pInvokeResult = PInvoke.SetAutoMarkerSearch(this._handle, Window, Auto_Search_Mode, System.Convert.ToUInt16(Auto_Search_State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns state of an automatic maximum or minimum peak search for marker 1 at the end of each particular sweep on and off. The current marker search limit settings (LEFT LIMIT, RIGHT LIMIT, THRESHOLD, EXCLUDE LO) are taken into account. 
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:MAXimum:AUTO?
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:MINimum:AUTO?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Auto_Search_Mode">
        /// This control selects which auto search function will be set.
        /// 
        /// Valid Values:
        /// RSESCI_MARKER_PMAX (0) - Maximum
        /// RSESCI_MARKER_PMIN (4) - Minimum
        /// 
        /// Default Value: RSESCI_MARKER_PMAX (0)
        /// </param>
        /// <param name="Auto_Search_State">
        /// This control returns state of automatic maximum or minimum peak search for marker 1 at the end of each particular sweep on and off.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetAutoMarkerSearch(int Window, int Auto_Search_Mode, out bool Auto_Search_State)
        {
            ushort Auto_Search_StateAsUShort;
            int pInvokeResult = PInvoke.GetAutoMarkerSearch(this._handle, Window, Auto_Search_Mode, out Auto_Search_StateAsUShort);
            Auto_Search_State = System.Convert.ToBoolean(Auto_Search_StateAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function changes instrument parameters via currently active marker.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:CENTer
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:REFerence
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:CSTep
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Marker">
        /// This control sets the marker.
        /// 
        /// Valid Range: 1 to 4
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Mkr_Function">
        /// This control sets the marker function.
        /// 
        /// Valid Values:
        /// RSESCI_MARKER_TO_CENTER (0) - Center
        /// RSESCI_MARKER_TO_STEP   (1) - CF Step
        /// RSESCI_MARKER_TO_REF    (2) - Reference
        /// 
        /// Default Value: RSESCI_MARKER_TO_CENTER (0)
        /// 
        /// Note(s):
        /// 
        /// (1) Center:
        /// This command sets the center frequency to that of the current marker.
        /// 
        /// (2) CF Step:
        /// This command sets the step width of the center frequency to the x-value of the current marker.
        /// 
        /// (3) Reference:
        /// This command sets the reference level to that of the current marker.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetMarkerTo(int Window, int Marker, int Mkr_Function)
        {
            int pInvokeResult = PInvoke.SetMarkerTo(this._handle, Window, Marker, Mkr_Function);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function assigns the selected marker (1 to 4) to the indicated test curve.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:TRACe
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Marker">
        /// This control sets the marker.
        /// 
        /// Valid Range: 1 to 4
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Trace">
        /// This control sets the trace.
        /// 
        /// Valid Range: 1 to 3
        /// 
        /// Default Value: 1
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetMarkerToTrace(int Window, int Marker, int Trace)
        {
            int pInvokeResult = PInvoke.SetMarkerToTrace(this._handle, Window, Marker, Trace);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the number of trace associated with marker.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:TRACe?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Marker">
        /// This control sets the marker.
        /// 
        /// Valid Range: 1 to 4
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Trace">
        /// This control returns the number of marker's associated trace.
        /// 
        /// Valid Range: 1 to 3
        /// 
        /// Default Value: 1
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetMarkerToTrace(int Window, int Marker, out int Trace)
        {
            int pInvokeResult = PInvoke.GetMarkerToTrace(this._handle, Window, Marker, out Trace);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the peak excursion., i.e. the spacing below a trace maximum which must be attained before a new maximum is recognized, or the spacing above a trace minimum which must be attained before a new minimum is recognized. The set value is valid for all markers and delta markers. The unit depends on the selected operating mode.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:PEXCursion
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Peak_Excursion">
        /// This control defines the peak excursion value.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: 0.0
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetPeakExcursion(int Window, double Peak_Excursion)
        {
            int pInvokeResult = PInvoke.SetPeakExcursion(this._handle, Window, Peak_Excursion);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the peak excursion value.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:PEXCursion?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Peak_Excursion">
        /// This control returns the peak excursion value.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetPeakExcursion(int Window, out double Peak_Excursion)
        {
            int pInvokeResult = PInvoke.GetPeakExcursion(this._handle, Window, out Peak_Excursion);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches the local oscillator suppression for peak search on or off. This setting is valid for all markers and delta markers in all measurement windows.
        /// 
        /// Mode(s):
        /// Spectrum Analysis (frequency domain only)
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:LOEXclude ON | OFF
        /// </summary>
        /// <param name="Local_Oscillator_Suppression">
        /// This control switches the local oscillator suppression for peak search on or off.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetLocalOscillatorSuppression(bool Local_Oscillator_Suppression)
        {
            int pInvokeResult = PInvoke.SetLocalOscillatorSuppression(this._handle, System.Convert.ToUInt16(Local_Oscillator_Suppression));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the setting of the local oscillator suppression for peak search.
        /// 
        /// Mode(s):
        /// Spectrum Analysis (frequency domain only)
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:LOEXclude?
        /// </summary>
        /// <param name="Local_Oscillator_Suppression">
        /// This control returns the setting of the local oscillator suppression for peak search.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetLocalOscillatorSuppression(out bool Local_Oscillator_Suppression)
        {
            ushort Local_Oscillator_SuppressionAsUShort;
            int pInvokeResult = PInvoke.GetLocalOscillatorSuppression(this._handle, out Local_Oscillator_SuppressionAsUShort);
            Local_Oscillator_Suppression = System.Convert.ToBoolean(Local_Oscillator_SuppressionAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches the coupling of the receiver frequency at the marker frequency on or off.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:SCOupled[:STATe] ON | OFF
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Marker">
        /// This control sets the marker.
        /// 
        /// Valid Range: 1 to 4
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Coupling_State">
        /// This control switches the coupling of the receiver frequency at the marker frequency on or off.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetMarkerSettingsCoupled(int Window, int Marker, bool Coupling_State)
        {
            int pInvokeResult = PInvoke.SetMarkerSettingsCoupled(this._handle, Window, Marker, System.Convert.ToUInt16(Coupling_State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the state of the coupling of the receiver frequency at the marker frequency.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:SCOupled[:STATe]?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Marker">
        /// This control sets the marker.
        /// 
        /// Valid Range: 1 to 4
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Coupling_State">
        /// This control returns the state of the coupling of the receiver frequency at the marker frequency.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetMarkerSettingsCoupled(int Window, int Marker, out bool Coupling_State)
        {
            ushort Coupling_StateAsUShort;
            int pInvokeResult = PInvoke.GetMarkerSettingsCoupled(this._handle, Window, Marker, out Coupling_StateAsUShort);
            Coupling_State = System.Convert.ToBoolean(Coupling_StateAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches the coupling of the receiver frequency to the marker on or off.
        /// 
        /// Note(s):
        /// 
        /// (1) The numeric suffix in MARKer&lt;1...4&gt; is irrelevant.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:COUPled[:STATe] ON | OFF
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Coupling_State">
        /// This control switches the coupling of the receiver frequency to the marker on or off.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetMarkerCoupling(int Window, bool Coupling_State)
        {
            int pInvokeResult = PInvoke.SetMarkerCoupling(this._handle, Window, System.Convert.ToUInt16(Coupling_State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the state of the coupling of the receiver frequency to the marker.
        /// 
        /// Note(s):
        /// 
        /// (1) The numeric suffix in MARKer&lt;1...4&gt; is irrelevant.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:COUPled[:STATe]?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Coupling_State">
        /// This control returns the state of the coupling of the receiver frequency to the marker.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetMarkerCoupling(int Window, out bool Coupling_State)
        {
            ushort Coupling_StateAsUShort;
            int pInvokeResult = PInvoke.GetMarkerCoupling(this._handle, Window, out Coupling_StateAsUShort);
            Coupling_State = System.Convert.ToBoolean(Coupling_StateAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function positions the delta marker to the next smaller maximum value on the measured curve. The corresponding delta marker will be activated first, if necessary.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:DELTamarker&lt;1...4&gt;:MAXimum[:PEAK]
        /// CALCulate&lt;1|2&gt;:DELTamarker&lt;1...4&gt;:MAXimum:NEXT
        /// CALCulate&lt;1|2&gt;:DELTamarker&lt;1...4&gt;:MAXimum:RIGHt
        /// CALCulate&lt;1|2&gt;:DELTamarker&lt;1...4&gt;:MAXimum:LEFT
        /// CALCulate&lt;1|2&gt;:DELTamarker&lt;1...4&gt;:MINimum[:PEAK]
        /// CALCulate&lt;1|2&gt;:DELTamarker&lt;1...4&gt;:MINimum:NEXT
        /// CALCulate&lt;1|2&gt;:DELTamarker&lt;1...4&gt;:MINimum:RIGHt
        /// CALCulate&lt;1|2&gt;:DELTamarker&lt;1...4&gt;:MINimum:LEFT
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Marker">
        /// This control sets the marker.
        /// 
        /// Valid Range: 1 to 4
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Search_Mode">
        /// This control positions the delta marker to the desired position in the trace memory.
        /// 
        /// Valid Values:
        /// RSESCI_MARKER_PMAX (0) - Peak Maximum
        /// RSESCI_MARKER_NMAX (1) - Next Maximum
        /// RSESCI_MARKER_LMAX (2) - Left Maximum
        /// RSESCI_MARKER_RMAX (3) - Right Maximum
        /// RSESCI_MARKER_PMIN (4) - Peak Minimum
        /// RSESCI_MARKER_NMIN (5) - Next Minimum
        /// RSESCI_MARKER_LMIN (6) - Left Minimum
        /// RSESCI_MARKER_RMIN (7) - Right Minimum
        /// 
        /// Default Value: RSESCI_MARKER_PMAX (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int DeltaMarkerSearch(int Window, int Marker, int Search_Mode)
        {
            int pInvokeResult = PInvoke.DeltaMarkerSearch(this._handle, Window, Marker, Search_Mode);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function assigns the selected delta marker to the indicated measurement curve in the indicated measurement window. The selected measurement curve must be active.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:DELTamarker&lt;1...4&gt;:TRACe
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Marker">
        /// This control sets the marker.
        /// 
        /// Valid Range: 1 to 4
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Trace">
        /// This control sets the trace.
        /// 
        /// Valid Range: 1 to 3
        /// 
        /// Default Value: 1
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetDeltaMarkerToTrace(int Window, int Marker, int Trace)
        {
            int pInvokeResult = PInvoke.SetDeltaMarkerToTrace(this._handle, Window, Marker, Trace);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the number of trace associated with delta marker.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:DELTamarker&lt;1...4&gt;:TRACe?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Marker">
        /// This control sets the marker.
        /// 
        /// Valid Range: 1 to 4
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Trace">
        /// This control returns the number of trace associated with delta marker.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetDeltaMarkerToTrace(int Window, int Marker, out int Trace)
        {
            int pInvokeResult = PInvoke.GetDeltaMarkerToTrace(this._handle, Window, Marker, out Trace);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function links delta marker 1 to marker 1. The x-value of delta marker 1 is kept fixed at 0. Therefore, delta marker 1 always has the same x-value as marker 1. Changing the x-value of marker 1 automatically changes the value of delta marker 1. This function is useful if marker 1 is positioned on trace 1 and delta marker 2 on trace 2.
        /// 
        /// Note(s):
        /// 
        /// (1) For DELTamarker only numeric suffix 1 is permitted.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:DELTamarker&lt;1...4&gt;:LINK ON | OFF
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Marker">
        /// This control sets the marker.
        /// 
        /// Valid Range: 1
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Linking_State">
        /// This control links delta marker 1 to marker 1.
        /// 
        /// Valid Range: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetLinkMarkerAndDeltaMarkerState(int Window, int Marker, bool Linking_State)
        {
            int pInvokeResult = PInvoke.SetLinkMarkerAndDeltaMarkerState(this._handle, Window, Marker, System.Convert.ToUInt16(Linking_State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the state of the linked delta marker 1 to marker 1. 
        /// 
        /// Note(s):
        /// 
        /// (1) For DELTamarker only numeric suffix 1 is permitted.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:DELTamarker&lt;1&gt;:LINK?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Marker">
        /// This control sets the marker.
        /// 
        /// Valid Range: 1
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Linking_State">
        /// This control returns the state of the linked delta marker 1 to marker 1.
        /// 
        /// Valid Range: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetLinkMarkerAndDeltaMarkerState(int Window, int Marker, out bool Linking_State)
        {
            ushort Linking_StateAsUShort;
            int pInvokeResult = PInvoke.GetLinkMarkerAndDeltaMarkerState(this._handle, Window, Marker, out Linking_StateAsUShort);
            Linking_State = System.Convert.ToBoolean(Linking_StateAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches between a limited (ON) and unlimited (OFF) search range in the selected measurement window. The function is independent of the selection of a marker.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:X:SLIMits[:STATe] ON | OFF
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Limited_Range">
        /// This control switches between a limited (ON) and unlimited (OFF) search range in the selected measurement window.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetSearchLimitLines(int Window, bool Limited_Range)
        {
            int pInvokeResult = PInvoke.SetSearchLimitLines(this._handle, Window, System.Convert.ToUInt16(Limited_Range));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the setting of limited search range in the selected measurement window.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:X:SLIMits[:STATe]?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Limited_Range">
        /// This control returns the setting of limited search range in the selected measurement window.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetSearchLimitLines(int Window, out bool Limited_Range)
        {
            ushort Limited_RangeAsUShort;
            int pInvokeResult = PInvoke.GetSearchLimitLines(this._handle, Window, out Limited_RangeAsUShort);
            Limited_Range = System.Convert.ToBoolean(Limited_RangeAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the left limit of the search range for markers and delta markers in the selected measurement window. Depending on the x axis domain the indicated value defines a frequency (span &gt; 0) or time (span = 0). The function is independent of the selection of a marker.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:X:SLIMits:LEFT
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Left_Limit">
        /// This control sets the left limit of the search range for markers and delta markers in the selected measurement window.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: 0.0
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetSearchLeftLimitLine(int Window, double Left_Limit)
        {
            int pInvokeResult = PInvoke.SetSearchLeftLimitLine(this._handle, Window, Left_Limit);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function reads the left limit of the search range for markers and delta markers in the selected measurement window.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:X:SLIMits:LEFT?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Left_Limit">
        /// This control returns the left limit of the search range for markers and delta markers in the selected measurement window.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetSearchLeftLimitLine(int Window, out double Left_Limit)
        {
            int pInvokeResult = PInvoke.GetSearchLeftLimitLine(this._handle, Window, out Left_Limit);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the right limit of the search range for markers and delta markers in the selected measurement window. Depending on the x axis domain the indicated value defines a frequency (span &gt; 0) or time (span = 0). The function is independent of the selection of a marker.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:X:SLIMits:RIGHT
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Right_Limit">
        /// This control sets the right limit of the search range for markers and delta markers in the selected measurement window.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: 0.0
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetSearchRightLimitLine(int Window, double Right_Limit)
        {
            int pInvokeResult = PInvoke.SetSearchRightLimitLine(this._handle, Window, Right_Limit);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function reads the right limit of the search range for markers and delta markers in the selected measurement window.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:X:SLIMits:RIGHT?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Right_Limit">
        /// This control returns the right limit of the search range for markers and delta markers in the selected measurement window.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetSearchRightLimitLine(int Window, out double Right_Limit)
        {
            int pInvokeResult = PInvoke.GetSearchRightLimitLine(this._handle, Window, out Right_Limit);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches on or off the threshold line in the selected measurement window. The unit depends on the setting performed with CALC:UNIT.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:THReshold:STATe ON | OFF
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Search_Threshold">
        /// This control switches on or off the threshold line in the selected measurement window.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetSearchThreshold(int Window, bool Search_Threshold)
        {
            int pInvokeResult = PInvoke.SetSearchThreshold(this._handle, Window, System.Convert.ToUInt16(Search_Threshold));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the state of threshold line in the selected measurement window.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:THReshold:STATe?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Search_Threshold">
        /// This control returns the state of threshold line in the selected measurement window.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetSearchThreshold(int Window, out bool Search_Threshold)
        {
            ushort Search_ThresholdAsUShort;
            int pInvokeResult = PInvoke.GetSearchThreshold(this._handle, Window, out Search_ThresholdAsUShort);
            Search_Threshold = System.Convert.ToBoolean(Search_ThresholdAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the threshold value for the maximum/minimum search of markers with marker search functions  in the selected measurement window. The associated display line is automatically switched on.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:THReshold
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Threshold">
        /// This control defines the threshold value for the maximum/minimum search of markers with marker search functions MAX PEAK, NEXT PEAK, etc in the selected measurement window.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: 0.0
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetSearchThresholdLine(int Window, double Threshold)
        {
            int pInvokeResult = PInvoke.SetSearchThresholdLine(this._handle, Window, Threshold);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function reads the threshold value for the maximum/minimum search of markers with marker search functions in the selected measurement window.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:THReshold?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Threshold">
        /// This control returns the threshold value for the maximum/minimum search of markers with marker search functions MAX PEAK, NEXT PEAK, etc in the selected measurement window.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetSearchThresholdLine(int Window, out double Threshold)
        {
            int pInvokeResult = PInvoke.GetSearchThresholdLine(this._handle, Window, out Threshold);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches the noise measurement on or off for all markers of the indicated measurement window. The noise power density is measured at the position of the markers.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:NOISe[:STATe] ON | OFF
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Noise_Measurement">
        /// This control switches the noise measurement on or off for all markers of the indicated measurement window.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetNoiseDensityMeasurement(int Window, bool Noise_Measurement)
        {
            int pInvokeResult = PInvoke.SetNoiseDensityMeasurement(this._handle, Window, System.Convert.ToUInt16(Noise_Measurement));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function reads the state of the noise measurement for all markers of the indicated measurement window.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:NOISe[:STATe]?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Noise_Measurement">
        /// This control returns the state of the noise measurement for all markers of the indicated measurement window.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetNoiseDensityMeasurement(int Window, out bool Noise_Measurement)
        {
            ushort Noise_MeasurementAsUShort;
            int pInvokeResult = PInvoke.GetNoiseDensityMeasurement(this._handle, Window, out Noise_MeasurementAsUShort);
            Noise_Measurement = System.Convert.ToBoolean(Noise_MeasurementAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the result of the noise measurement. A complete sweep with synchronization to the sweep end must be performed between switching on the function and querying the measured value in order to obtain a valid query result. This is only possible in single sweep mode.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:NOISe:RESult?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Noise_Measurement">
        /// This control returns the result of the noise measurement.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetNoiseDensityMeasurementValue(int Window, out double Noise_Measurement)
        {
            int pInvokeResult = PInvoke.GetNoiseDensityMeasurementValue(this._handle, Window, out Noise_Measurement);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches on or off the phase-noise measurement with all active delta markers in the selected measurement window. The correction values for the bandwidth and the log amplifier are taken into account in the measurement.
        /// 
        /// Marker 1 will be activated, if necessary, and a peak search will be performed. If marker 1 is activated, its position becomes the reference point for the measurement.
        /// 
        /// The reference point can then be modified with functions rsesci_SetDeltaMarkerReferencePointPosition() and rsesci_SetDeltaMarkerReferencePointLevel() independently of the position of marker 1 and of a trace (the same functions used for the measurment with fixed reference point).
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:DELTamarker&lt;1...4&gt;:FUNCtion:PNOise[:STATe] ON | OFF
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Phase_Noise_Measurement">
        /// This control switches on or off the phase-noise measurement with all active delta markers in the selected measurement window.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetPhaseNoiseMeasurement(int Window, bool Phase_Noise_Measurement)
        {
            int pInvokeResult = PInvoke.SetPhaseNoiseMeasurement(this._handle, Window, System.Convert.ToUInt16(Phase_Noise_Measurement));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the state of phase-noise measurement with all active delta markers in the selected measurement window.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:DELTamarker&lt;1...4&gt;:FUNCtion:PNOise[:STATe]?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Phase_Noise_Measurement">
        /// This control returns the state of phase-noise measurement with all active delta markers in the selected measurement window.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetPhaseNoiseMeasurement(int Window, out bool Phase_Noise_Measurement)
        {
            ushort Phase_Noise_MeasurementAsUShort;
            int pInvokeResult = PInvoke.GetPhaseNoiseMeasurement(this._handle, Window, out Phase_Noise_MeasurementAsUShort);
            Phase_Noise_Measurement = System.Convert.ToBoolean(Phase_Noise_MeasurementAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function adds an automatic peak search action for the reference fixed marker 1 at the end of each particular sweep. This function may be used for tracking of a drifting source whilst phase noise measurement. The delta marker 2 which shows the phase noise measurement result keeps the delta frequency value. Therefore the phase noise measurement in a certain offset is valid although the source is drifting. Only when the marker 2 is reaching the border of the span the delta marker value is adjusted to be within the span. Choose a larger span in such situations.
        /// 
        /// Note(s):
        /// (1) This function is available with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:DELTamarker&lt;1 to 4&gt;:FUNCtion:PNOise:AUTO ON | OFF OFF
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Automatic_Peak_Search">
        /// This control switches on or off the automatic peak search action for the reference fixed marker 1 at the end of each particular sweep.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetPhaseNoiseMeasurementAuto(int Window, bool Automatic_Peak_Search)
        {
            int pInvokeResult = PInvoke.SetPhaseNoiseMeasurementAuto(this._handle, Window, System.Convert.ToUInt16(Automatic_Peak_Search));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries state of the automatic peak search action for the reference fixed marker 1 at the end of each particular sweep.
        /// 
        /// Note(s):
        /// (1) This function is available with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:DELTamarker&lt;1 to 4&gt;:FUNCtion:PNOise:AUTO?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Automatic_Peak_Search">
        /// This control returns state of the automatic peak search action for the reference fixed marker 1 at the end of each particular sweep.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetPhaseNoiseMeasurementAuto(int Window, out bool Automatic_Peak_Search)
        {
            ushort Automatic_Peak_SearchAsUShort;
            int pInvokeResult = PInvoke.GetPhaseNoiseMeasurementAuto(this._handle, Window, out Automatic_Peak_SearchAsUShort);
            Automatic_Peak_Search = System.Convert.ToBoolean(Automatic_Peak_SearchAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the result of the phase-noise measurement in the selected measurement window. The measurement will be switched on, if necessary.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:DELTamarker&lt;1...4&gt;:FUNCtion:PNOise:RESult?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Phase_Noise_Measurement">
        /// This control returns the result of the phase-noise measurement in the selected measurement window. The measurement will be switched on, if necessary.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetPhaseNoiseMeasurementValue(int Window, out double Phase_Noise_Measurement)
        {
            int pInvokeResult = PInvoke.GetPhaseNoiseMeasurementValue(this._handle, Window, out Phase_Noise_Measurement);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches the "N dB Down" function on or off in the selected measurement window. Marker 1 is activated first, if necessary.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:NDBDown:STATe ON | OFF
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="N_dB_Down_State">
        /// This control switches the "N dB Down" function on or off in the selected measurement window. Marker 1 is activated first, if necessary.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetNdBDown(int Window, bool N_dB_Down_State)
        {
            int pInvokeResult = PInvoke.SetNdBDown(this._handle, Window, System.Convert.ToUInt16(N_dB_Down_State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the state of "N dB Down" function in the selected measurement window.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:NDBDown:STATe?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="N_dB_Down_State">
        /// This control returns the state of "N dB Down" function in the selected measurement window.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetNdBDown(int Window, out bool N_dB_Down_State)
        {
            ushort N_dB_Down_StateAsUShort;
            int pInvokeResult = PInvoke.GetNdBDown(this._handle, Window, out N_dB_Down_StateAsUShort);
            N_dB_Down_State = System.Convert.ToBoolean(N_dB_Down_StateAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the level spacing of the two delta markers to the right and left of marker 1 in the selected measurement window. Marker 1 is always used as the reference marker. The temporary markers T1 and T2 are positioned by n dB below the active reference marker.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:NDBDown
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Level_Spacing">
        /// This control defines the level spacing of the two delta markers to the right and left of marker 1 in the selected measurement window.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: 6.0 dB
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetNdBDownLevelSpacing(int Window, double Level_Spacing)
        {
            int pInvokeResult = PInvoke.SetNdBDownLevelSpacing(this._handle, Window, Level_Spacing);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the level spacing of the two delta markers to the right and left of marker 1 in the selected measurement window.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:NDBDown?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Level_Spacing">
        /// This control returns the level spacing of the two delta markers to the right and left of marker 1 in the selected measurement window.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetNdBDownLevelSpacing(int Window, out double Level_Spacing)
        {
            int pInvokeResult = PInvoke.GetNdBDownLevelSpacing(this._handle, Window, out Level_Spacing);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the frequency spacing (bandwidth) of the N-dB-down markers in the selected measurement window. A complete sweep with synchronization to sweep end must be performed between switching on the function and querying the measured value in order to obtain a valid query result. This is only possible in single sweep mode.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:NDBDown:RESult?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Frequency_Spacing">
        /// This control returns queries the frequency spacing (bandwidth) of the N-dB-down markers in the selected measurement window.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetNdBDownBandwidth(int Window, out double Frequency_Spacing)
        {
            int pInvokeResult = PInvoke.GetNdBDownBandwidth(this._handle, Window, out Frequency_Spacing);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the two frequencies of the N-dB-down marker in the selected measurement window. A complete sweep with synchronization to sweep end must be performed between switching on the function and querying the measured value to obtain a valid query result. This is only possible in single sweep mode.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:NDBDown:FREQuency?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Frequencies">
        /// This control returns an array of two frequencies of the N-dB-down marker in ascending order.
        /// 
        /// Note(s):
        /// 
        /// (1) The array should have at least two ViReal64 elements.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetNdBDownFrequencies(int Window, double[] Frequencies)
        {
            int pInvokeResult = PInvoke.GetNdBDownFrequencies(this._handle, Window, Frequencies);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the two time values of the N-dB-down marker in the selected measurement window. A complete sweep with synchronization to sweep end must be performed between switching on the function and querying the measured value to obtain a valid query result. This is only possible in single sweep mode.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:NDBDown:TIME?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Times">
        /// This control returns an array of two time values of the N-dB-down marker in ascending order.
        /// 
        /// Note(s):
        /// 
        /// (1) The array should have at least two ViReal64 elements.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetNdBDownTimes(int Window, double[] Times)
        {
            int pInvokeResult = PInvoke.GetNdBDownTimes(this._handle, Window, Times);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function searches the selected trace for the indicated number of maxima.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Note(s):
        /// 
        /// (1) The number of maxima found depends on the waveform and value set for the Peak Excursion parameter, however, a maximum number of 50 maxima are determined. Only the signals which exceed their surrounding values at least by the value indicated by the peak excursion parameter will be recognized as maxima. Therefore, the number of maxima found is not automatically the same as the number of maxima desired.
        /// 
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:FPEaks[:IMMediate]
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Number_of_maxima">
        /// This control sets the number of maxima.
        /// 
        /// Valid Range: 1 to 50
        /// 
        /// Default Value: 1
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SearchForPeakList(int Window, int Number_of_maxima)
        {
            int pInvokeResult = PInvoke.SearchForPeakList(this._handle, Window, Number_of_maxima);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the sort mode for the search for maxima.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:FPEaks:SORT X | Y
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Sort_Mode">
        /// This control sets the sort mode for the search for maxima.
        /// 
        /// Valid Values:
        /// RSESCI_X (0) - X
        /// RSESCI_Y (1) - Y
        /// 
        /// Default Value: RSESCI_X (0)
        /// 
        /// Note(s):
        /// 
        /// (1) X - the maxima are sorted in the list of responses according to increasing X values
        /// 
        /// (2) Y - the maxima are sorted in the list of responses according to decreasing Y values
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetPeakListSortMode(int Window, int Sort_Mode)
        {
            int pInvokeResult = PInvoke.SetPeakListSortMode(this._handle, Window, Sort_Mode);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the sort mode for the search for maxima.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:FPEaks:SORT?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Sort_Mode">
        /// This control returns the sort mode for the search for maxima.
        /// 
        /// Valid Values:
        /// RSESCI_X (0) - X
        /// RSESCI_Y (1) - Y
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetPeakListSortMode(int Window, out int Sort_Mode)
        {
            int pInvokeResult = PInvoke.GetPeakListSortMode(this._handle, Window, out Sort_Mode);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function reads out the number of maxima found during the search. If no search for maxima has been performed, 0 is returned.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:FPEaks:COUNt?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Peak_Count">
        /// This control returns the number of maxima found during the search.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetPeakListCount(int Window, out int Peak_Count)
        {
            int pInvokeResult = PInvoke.GetPeakListCount(this._handle, Window, out Peak_Count);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function reads out the list of X values of the maxima found. The number of available values can be queried with rsesci_GetPeakListCount().
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:FPEaks:X?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="X_Values">
        /// This control returns an array of X values.
        /// 
        /// Note(s):
        /// 
        /// (1) The array should have number of elements equal or greater than number of peaks (can be queried using rsesci_GetPeakListCount). Maximum number of 50 maxima is determined.
        /// 
        /// (2) With sort mode X, the X values are in increasing order; with sort mode Y the order corresponds to the decreasing order of the Y values.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetPeakListXValues(int Window, double[] X_Values)
        {
            int pInvokeResult = PInvoke.GetPeakListXValues(this._handle, Window, X_Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function reads out the list of Y values of the maxima found. The number of available values can be queried with rsesci_GetPeakListCount().
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:FPEaks:Y?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Y_Values">
        /// This control returns an array of Y values.
        /// 
        /// Note(s):
        /// 
        /// (1) The array should have number of elements equal or greater than number of peaks (can be queried using rsesci_GetPeakListCount). Maximum number of 50 maxima is determined.
        /// 
        /// (2) With sort mode X, the X values are in increasing order; with sort mode Y the order corresponds to the decreasing order of the Y values.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetPeakListYValues(int Window, double[] Y_Values)
        {
            int pInvokeResult = PInvoke.GetPeakListYValues(this._handle, Window, Y_Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches on or off the audio demodulator when the indicated marker is reached in the selected measurement window. In the frequency domain (span &gt; 0) the hold time can be defined at the corresponding marker position with function rsesci_SetMarkerStopTime.
        /// In the time domain (span = 0) the demodulation is permanently active.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:DEModulation[:STATe] ON | OFF
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Marker">
        /// This control sets the marker.
        /// 
        /// Valid Range: 1 to 4
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Demodulation_State">
        /// This control switches on or off the audio demodulator when the indicated marker is reached in the selected measurement window.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetMarkerDemodulation(int Window, int Marker, bool Demodulation_State)
        {
            int pInvokeResult = PInvoke.SetMarkerDemodulation(this._handle, Window, Marker, System.Convert.ToUInt16(Demodulation_State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the state of the audio demodulator.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:DEModulation[:STATe]?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Marker">
        /// This control sets the marker.
        /// 
        /// Valid Range: 1 to 4
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Demodulation_State">
        /// This control returns the state of the audio demodulator.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetMarkerDemodulation(int Window, int Marker, out bool Demodulation_State)
        {
            ushort Demodulation_StateAsUShort;
            int pInvokeResult = PInvoke.GetMarkerDemodulation(this._handle, Window, Marker, out Demodulation_StateAsUShort);
            Demodulation_State = System.Convert.ToBoolean(Demodulation_StateAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the demodulation type for the audio demodulator. The function is independent of the measurement window and of the selected marker.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:DEModulation:SELect AM | FM
        /// </summary>
        /// <param name="Demodulation_Type">
        /// This control selects the demodulation type for the audio demodulator.
        /// 
        /// Valid Values:
        /// RSESCI_DEMOD_AM (1) - AM
        /// RSESCI_DEMOD_FM (2) - FM
        /// 
        /// Default Value: RSESCI_DEMOD_AM (1)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetMarkerDemodulationType(int Demodulation_Type)
        {
            int pInvokeResult = PInvoke.SetMarkerDemodulationType(this._handle, Demodulation_Type);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries selected demodulation type for the audio demodulator.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:DEModulation:SELect?
        /// </summary>
        /// <param name="Demodulation_Type">
        /// This control returns selected demodulation type for the audio demodulator.
        /// 
        /// Valid Values:
        /// RSESCI_DEMOD_AM (1) - AM
        /// RSESCI_DEMOD_FM (2) - FM
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetMarkerDemodulationType(out int Demodulation_Type)
        {
            int pInvokeResult = PInvoke.GetMarkerDemodulationType(this._handle, out Demodulation_Type);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function enables the input of a level threshold below which the audible AF is cut off. The squelch function is associated with the internal trigger function (TRIGGER menu), which will be switched on automatically with the squelch. Squelch level and trigger level do have the same value.
        /// 
        /// Mode(s):
        /// Spectrum Analyzer
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:DEModulation:SQUelch[:STATe] ON | OFF
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:DEModulation:SQUelch:LEVel 0...100 PCT
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Marker">
        /// This control sets the marker.
        /// 
        /// Valid Range: 1 to 4
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Squelch_State">
        /// This control enables the input of a level threshold below which the audible AF is cut off.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <param name="Squelch_Level">
        /// This control sets the squelch level.
        /// 
        /// Valid Values:
        /// 0.0 to 100.0 %
        /// 
        /// Default Value: 50.0 %
        /// 
        /// Note(s):
        /// 
        /// (1) This control is ignored if the squelch state is set to Off.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetSquelchAnalyzer(int Window, int Marker, bool Squelch_State, double Squelch_Level)
        {
            int pInvokeResult = PInvoke.SetSquelchAnalyzer(this._handle, Window, Marker, System.Convert.ToUInt16(Squelch_State), Squelch_Level);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the input of a level threshold below which the audible AF is cut off.
        /// 
        /// Mode(s):
        /// Spectrum Analyzer
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:DEModulation:SQUelch[:STATe]?
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:DEModulation:SQUelch:LEVel?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Marker">
        /// This control sets the marker.
        /// 
        /// Valid Range: 1 to 4
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Squelch_State">
        /// This control returns the input of a level threshold below which the audible AF is cut off.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <param name="Squelch_Level">
        /// This control gets the squelch level.
        /// 
        /// Valid Values:
        /// 0.0 to 100.0 %
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetSquelchAnalyzer(int Window, int Marker, out bool Squelch_State, out double Squelch_Level)
        {
            ushort Squelch_StateAsUShort;
            int pInvokeResult = PInvoke.GetSquelchAnalyzer(this._handle, Window, Marker, out Squelch_StateAsUShort, out Squelch_Level);
            Squelch_State = System.Convert.ToBoolean(Squelch_StateAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the hold time at the marker position for the demodulation in the frequency domain (span &gt; 0). The setting is independent of the measurement window and the selected marker,
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:DEModulation:HOLDoff
        /// </summary>
        /// <param name="Holdoff">
        /// This control defines the hold time at the marker position for the demodulation in the frequency domain (span &gt; 0).
        /// 
        /// Valid Range: 10e-3 to 1000 s
        /// 
        /// Default Value: 3 s
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetMarkerStopTime(double Holdoff)
        {
            int pInvokeResult = PInvoke.SetMarkerStopTime(this._handle, Holdoff);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the hold time at the marker position for the demodulation in the frequency domain (span &gt; 0).
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:DEModulation:HOLDoff?
        /// </summary>
        /// <param name="Holdoff">
        /// This control returns the hold time at the marker position for the demodulation in the frequency domain (span &gt; 0).
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetMarkerStopTime(out double Holdoff)
        {
            int pInvokeResult = PInvoke.GetMarkerStopTime(this._handle, out Holdoff);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches on or off the continuous demodulation in the frequency domain (span &gt;0) in the selected measurement window. Thus acoustic monitoring of the signals can be performed in the frequency domain. The function does not depend on the selected marker,
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:DEModulation:CONTinuous ON | OFF
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Continuous_Demodulation">
        /// This control switches on or off the continuous demodulation in the frequency domain (span &gt;0) in the selected measurement window.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetContinuousMarkerDemodulation(int Window, bool Continuous_Demodulation)
        {
            int pInvokeResult = PInvoke.SetContinuousMarkerDemodulation(this._handle, Window, System.Convert.ToUInt16(Continuous_Demodulation));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns setting of the continuous demodulation in the frequency domain (span &gt;0) in the selected measurement window.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:DEModulation:CONTinuous?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Continuous_Demodulation">
        /// This control returns setting of the continuous demodulation in the frequency domain (span &gt;0) in the selected measurement window.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetContinuousMarkerDemodulation(int Window, out bool Continuous_Demodulation)
        {
            ushort Continuous_DemodulationAsUShort;
            int pInvokeResult = PInvoke.GetContinuousMarkerDemodulation(this._handle, Window, out Continuous_DemodulationAsUShort);
            Continuous_Demodulation = System.Convert.ToBoolean(Continuous_DemodulationAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function starts a new spurious measurement.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// INITiate&lt;1|2&gt;:SPURious
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation.  The status code  either indicates success or describes an error or warning condition.  You examine the status code from each call to an instrument driver function to determine if an error occurred.
        /// 
        /// To obtain a text description of the status code, call the rsspecan_error_message function.  To obtain additional information about the error condition, call the rsspecan_GetError function.  To clear the error information from the driver, call the rsspecan_ClearError function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// WARNINGS:
        /// 3FFA2001  The instrument was in an uncalibrated state
        ///           when the measurement was taken.
        /// 3FFA2002  The measurement taken was over the instrument??s
        ///           range.
        ///   
        /// ERRORS:
        /// BFFA2001  The Active Marker is not enabled.
        /// BFFA2002  The Active Marker is not a delta marker.
        /// BFFA2003  The maximum waiting time for this operation was
        ///           exceeded.
        ///           
        /// This instrument driver also returns errors and warnings defined by other sources.  The following table defines the ranges of additional status codes that this driver can return.  The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFA0000 to 3FFA1FFF     IVI    Warnings
        /// 3FFF0000 to 3FFFFFFF     VISA   Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP Driver Warnings
        ///           
        /// BFFA0000 to BFFA1FFF     IVI    Errors
        /// BFFF0000 to BFFFFFFF     VISA   Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP Driver Errors
        /// 
        /// </returns>
        public int InitiateSpurious(int Window)
        {
            int pInvokeResult = PInvoke.InitiateSpurious(this._handle, Window);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the unit for power for the selected measurement window.
        /// 
        /// Note(s):
        /// 
        /// (1) These units: PCT, DBUV_MHZ, DBMV_MHZ, DBUA_MHZ, DBUV_MMHZ, DBUA_MMHZ, UNITLESS are available only with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// UNIT&lt;1|2&gt;:POWer DBM | V | A | W | DB | PCT | UNITless | DBPW | WATT | DBUV | DBMV | VOLT | DBUA | AMPere | DBPT | DBUV_MHZ | DBMV_MHZ | DBUA_MHZ | DBUV_M | DBUA_M | DBUV_MMHZ | DBUA_MMHZ
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Power_Unit">
        /// This control selects the unit for power for the selected measurement window.
        /// 
        /// Valid Values:
        /// RSESCI_UNIT_DBM       (0)  - dBm
        /// RSESCI_UNIT_VOLT      (1)  - Volt
        /// RSESCI_UNIT_AMP       (2)  - Ampere
        /// RSESCI_UNIT_WATT      (3)  - Watt
        /// RSESCI_UNIT_DBPW      (4)  - dBpW
        /// RSESCI_UNIT_DBUV      (5)  - dBuV
        /// RSESCI_UNIT_DBMV      (6)  - dBmV
        /// RSESCI_UNIT_DBUA      (7)  - dBuA
        /// RSESCI_UNIT_DBPT      (8)  - dBpT
        /// RSESCI_UNIT_DBUV_M    (9)  - dBuV/m
        /// RSESCI_UNIT_DBUA_M    (10) - dBuA/m
        /// RSESCI_UNIT_DB        (11) - dB
        /// RSESCI_UNIT_PCT       (17) - Percentage
        /// RSESCI_UNIT_DBUV_MHZ  (18) - dBuV/MHz
        /// RSESCI_UNIT_DBMV_MHZ  (19) - dBmV/MHz
        /// RSESCI_UNIT_DBUA_MHZ  (20) - dBuA/MHz
        /// RSESCI_UNIT_DBUV_MMHZ (23) - dBuV/mMhz
        /// RSESCI_UNIT_DBUA_MMHZ (24) - dBuA/mMhz
        /// RSESCI_UNIT_UNITLESS  (25) - Unitless
        /// 
        /// Default Value: RSESCI_UNIT_DBM (0)
        /// 
        /// Note(s):
        /// 
        /// (1) In receiver mode, the units DBM, DBUV, DBUA, DBPW, DBT, DBUV_MHZ, DBMV_MHZ and DBUA_MHZ are available
        /// 
        /// (3) In analyzer mode all units are available.
        /// 
        /// (4) These units: PCT, DBUV_MHZ, DBMV_MHZ, DBUA_MHZ, DBUV_MMHZ, DBUA_MMHZ, UNITLESS are available only with ESU instrument.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetPowerUnit(int Window, int Power_Unit)
        {
            int pInvokeResult = PInvoke.SetPowerUnit(this._handle, Window, Power_Unit);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the unit for power for the selected measurement window.
        /// 
        /// Note(s):
        /// 
        /// (1) These units: PCT, DBUV_MHZ, DBMV_MHZ, DBUA_MHZ, DBUV_MMHZ, DBUA_MMHZ, UNITless are available only with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// UNIT&lt;1|2&gt;:POWer?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Power_Unit">
        /// This control returns the unit for power for the selected measurement window.
        /// 
        /// Valid Values:
        /// RSESCI_UNIT_DBM       (0)  - dBm
        /// RSESCI_UNIT_VOLT      (1)  - Volt
        /// RSESCI_UNIT_AMP       (2)  - Ampere
        /// RSESCI_UNIT_WATT      (3)  - Watt
        /// RSESCI_UNIT_DBPW      (4)  - dBpW
        /// RSESCI_UNIT_DBUV      (5)  - dBuV
        /// RSESCI_UNIT_DBMV      (6)  - dBmV
        /// RSESCI_UNIT_DBUA      (7)  - dBuA
        /// RSESCI_UNIT_DBPT      (8)  - dBpT
        /// RSESCI_UNIT_DBUV_M    (9)  - dBuV/m
        /// RSESCI_UNIT_DBUA_M    (10) - dBuA/m
        /// RSESCI_UNIT_DB        (11) - dB
        /// RSESCI_UNIT_PCT       (17) - Percentage
        /// RSESCI_UNIT_DBUV_MHZ  (18) - dBuV/MHz
        /// RSESCI_UNIT_DBMV_MHZ  (19) - dBmV/MHz
        /// RSESCI_UNIT_DBUA_MHZ  (20) - dBuA/MHz
        /// RSESCI_UNIT_DBUV_MMHZ (23) - dBuV/mMhz
        /// RSESCI_UNIT_DBUA_MMHZ (24) - dBuA/mMhz
        /// RSESCI_UNIT_UNITLESS  (25) - Unitless
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetPowerUnit(int Window, out int Power_Unit)
        {
            int pInvokeResult = PInvoke.GetPowerUnit(this._handle, Window, out Power_Unit);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches on or off selected measurement in selected window.
        /// 
        /// Mode(s):
        /// Spectrum Analysis (time domain only (zero span))
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:SUMMary:AOFF
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:SUMMary[:STATe] ON | OFF
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:SUMMary:PPEak[:STATe] ON | OFF
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:SUMMary:RMS[:STATe] ON | OFF
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:SUMMary:MEAN[:STATe] ON | OFF
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:SUMMary:SDEViation[:STATe] ON | OFF
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Power_Measurement">
        /// This control selects the power measurement.
        /// 
        /// Valid Values:
        /// RSESCI_POWER_MEAS_AOFF (0) - All Off
        /// RSESCI_POWER_MEAS_PREV (1) - Previously Selected
        /// RSESCI_POWER_MEAS_PEAK (2) - Peak
        /// RSESCI_POWER_MEAS_RMS  (3) - RMS
        /// RSESCI_POWER_MEAS_MEAN (4) - Mean
        /// RSESCI_POWER_MEAS_SDEV (5) - Standard Deviation
        /// 
        /// Default Value: RSESCI_POWER_MEAS_AOFF (0)
        /// 
        /// Note(s):
        /// 
        /// (1) All Off - switches off all time domain measurements in the selected measurement window.
        /// 
        /// (2) Previously Selected - switches on or off the previously selected time domain power measurements.
        /// 
        /// (3) Peak - switches on or off the measurement of the positive peak value in the selected measurement window. It is only available in the time domain (span = 0).
        /// 
        /// (4) RMS - switches on or off the measurement of the effective (RMS) power in the selected measurement window.
        /// 
        /// (5) Mean - switches on or off the measurement of the mean value in the selected measurement window.
        /// 
        /// (6) Standard Deviation - switches on or off the measurement of the standard deviation in the selected measurement window.
        /// </param>
        /// <param name="Measurement_State">
        /// This control switches on or off selected measurement.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetTimeDomainPower(int Window, int Power_Measurement, bool Measurement_State)
        {
            int pInvokeResult = PInvoke.SetTimeDomainPower(this._handle, Window, Power_Measurement, System.Convert.ToUInt16(Measurement_State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the state of selected measurement.
        /// 
        /// Mode(s):
        /// Spectrum Analysis (time domain only (zero span))
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:SUMMary:PPEak[:STATe]?
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:SUMMary:RMS[:STATe]?
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:SUMMary:MEAN[:STATe]?
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:SUMMary:SDEViation[:STATe]?
        /// 
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Power_Measurement">
        /// This control selects the power measurement.
        /// 
        /// Valid Values:
        /// RSESCI_POWER_MEAS_PEAK (2) - Peak
        /// RSESCI_POWER_MEAS_RMS  (3) - RMS
        /// RSESCI_POWER_MEAS_MEAN (4) - Mean
        /// RSESCI_POWER_MEAS_SDEV (5) - Standard Deviation
        /// 
        /// Default Value: RSESCI_POWER_MEAS_PEAK (2)
        /// 
        /// Note(s):
        /// 
        /// (1) Peak - gets the state of the measurement of the positive peak value in the selected measurement window. It is only available in the time domain (span = 0).
        /// 
        /// (2) RMS - gets the state of the measurement of the effective (RMS) power in the selected measurement window.
        /// 
        /// (3) Mean - gets the state of the measurement of the mean value in the selected measurement window.
        /// 
        /// (4) Standard Deviation - gets the state of the measurement of the standard deviation in the selected measurement window.
        /// </param>
        /// <param name="Measurement_State">
        /// This control returns the state of selected measurement.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetTimeDomainPower(int Window, int Power_Measurement, out bool Measurement_State)
        {
            ushort Measurement_StateAsUShort;
            int pInvokeResult = PInvoke.GetTimeDomainPower(this._handle, Window, Power_Measurement, out Measurement_StateAsUShort);
            Measurement_State = System.Convert.ToBoolean(Measurement_StateAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches on or off averaging for the active time domain power measurement in the indicated window. The function is independent of the marker selection. It is only available in the time domain (span = 0). Averaging is reset by switching it off and on again. The number of results required for the calculation of average is defined with [SENSe&lt;1|2&gt;:]AVERage:COUNt. It should be noted that synchronization to the end of averaging is only possible in single sweep mode.
        /// 
        /// Mode(s):
        /// Spectrum Analysis (time domain only (zero span))
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:SUMMary:AVERage ON | OFF
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Averaging">
        /// This control switches on or off averaging for the active time domain power measurement in the indicated window.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetTimeDomainPowerAveraging(int Window, bool Averaging)
        {
            int pInvokeResult = PInvoke.SetTimeDomainPowerAveraging(this._handle, Window, System.Convert.ToUInt16(Averaging));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the state of averaging for the active time domain power measurement in the indicated window.
        /// 
        /// Mode(s):
        /// Spectrum Analysis (time domain only (zero span))
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:SUMMary:AVERage?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Averaging">
        /// This control returns the state of averaging for the active time domain power measurement in the indicated window.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetTimeDomainPowerAveraging(int Window, out bool Averaging)
        {
            ushort AveragingAsUShort;
            int pInvokeResult = PInvoke.GetTimeDomainPowerAveraging(this._handle, Window, out AveragingAsUShort);
            Averaging = System.Convert.ToBoolean(AveragingAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches on or off the peak-hold function for the active time domain power measurement in the indicated measurement window. The function is independent of the marker
        /// selection. It is only available in the time domain (span = 0).
        /// The peak-hold function is reset by switching it off and on again.
        /// 
        /// Mode(s):
        /// Spectrum Analysis (time domain only (zero span))
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:SUMMary:PHOLd ON | OFF
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Peak_Hold">
        /// This control switches on or off the peak-hold function for the active time domain power measurement.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetTimeDomainPowerPeakHold(int Window, bool Peak_Hold)
        {
            int pInvokeResult = PInvoke.SetTimeDomainPowerPeakHold(this._handle, Window, System.Convert.ToUInt16(Peak_Hold));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the state of the peak-hold function for the active time domain power measurement.
        /// 
        /// Mode(s):
        /// Spectrum Analysis (time domain only (zero span))
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:SUMMary:PHOLd?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Peak_Hold">
        /// This control returns the state of the peak-hold function for the active time domain power measurement.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetTimeDomainPowerPeakHold(int Window, out bool Peak_Hold)
        {
            ushort Peak_HoldAsUShort;
            int pInvokeResult = PInvoke.GetTimeDomainPowerPeakHold(this._handle, Window, out Peak_HoldAsUShort);
            Peak_Hold = System.Convert.ToBoolean(Peak_HoldAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects absolute or relative time domain power measurement in the indicated measurement window. The function is independent of the marker selection. It is only available in the time domain (span = 0).
        /// The reference power for relative measurement is defined with rsesci_SetTimeDomainPowerReference(). If the reference power is not defined, the value 0 dBm is used.
        /// 
        /// Mode(s):
        /// Spectrum Analysis (time domain only (zero span))
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:SUMMary:MODE ABSolute | RELative
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Mode">
        /// This control selects absolute or relative time domain power measurement in the indicated measurement window.
        /// 
        /// Valid Values:
        /// RSESCI_ABS (0) - Absolute
        /// RSESCI_REL (1) - Relative
        /// 
        /// Default Value: RSESCI_ABS (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetTimeDomainPowerMode(int Window, int Mode)
        {
            int pInvokeResult = PInvoke.SetTimeDomainPowerMode(this._handle, Window, Mode);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries mode of time domain power measurement in the indicated measurement window.
        /// 
        /// Mode(s):
        /// Spectrum Analysis (time domain only (zero span))
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:SUMMary:MODE?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Mode">
        /// This control returns mode of time domain power measurement in the indicated measurement window.
        /// 
        /// Valid Values:
        /// RSESCI_ABS (0) - Absolute
        /// RSESCI_REL (1) - Relative
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetTimeDomainPowerMode(int Window, out int Mode)
        {
            int pInvokeResult = PInvoke.GetTimeDomainPowerMode(this._handle, Window, out Mode);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// With this function the currently measured average value and RMS value are declared as reference values for relative measurements in the indicated measurement window. The function is independent of the marker selection. It is only available in the time domain (span = 0). If the measurement of RMS value and average is not activated, the reference value 0 dBm is used. If the function rsesci_SetTimeDomainPowerPeakHold is switched on, the current value is the accumulated measurement value at the time considered.
        /// 
        /// Mode(s):
        /// Spectrum Analysis (time domain only (zero span))
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:SUMMary:REFerence:AUTO ONCE
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetTimeDomainPowerReference(int Window)
        {
            int pInvokeResult = PInvoke.SetTimeDomainPowerReference(this._handle, Window);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the result of the measurement of the mean value in the selected measurement window. The function is independent of the marker selection. It is only available in the time domain (span = 0). A complete sweep with synchronization to sweep end must be performed between switching on the function and querying the measured value to obtain a valid query result. This is only possible in single sweep mode.
        /// 
        /// Mode(s):
        /// Spectrum Analysis (time domain only (zero span))
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:SUMMary:MEAN:RESult?
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:SUMMary:MEAN:AVERage:RESult?
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:SUMMary:MEAN:PHOLd:RESult?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Result_Type">
        /// This control selects the result type.
        /// 
        /// Valid Values:
        /// RSESCI_RESULT_CURR  (0) - Current
        /// RSESCI_RESULT_AVG   (1) - Average
        /// RSESCI_RESULT_PHOLD (2) - Peak Hold
        /// 
        /// Default Value: RSESCI_RESULT_CURR (0)
        /// 
        /// Note(s):
        /// 
        /// (1) Current - function returns the current mean value
        /// (2) Average - function returns the averaged mean value
        /// (3) Peak Hold - function returns the mean value with active peak hold
        /// </param>
        /// <param name="Result">
        /// This control returns the value of selected measurement.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetTimeDomainPowerMeanValue(int Window, int Result_Type, out double Result)
        {
            int pInvokeResult = PInvoke.GetTimeDomainPowerMeanValue(this._handle, Window, Result_Type, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function is used to query the result of the measurement of the positive peak value in the selected measurement window. The measurement may have to be switched on previously. The function is independent of the marker selection. It is only available in the time domain (span = 0). A complete sweep with synchronization to sweep end must be performed between switching on the function and querying the measured value to obtain a valid query result. This is only possible in single sweep mode.
        /// 
        /// Mode(s):
        /// Spectrum Analysis (time domain only (zero span))
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:SUMMary:PPEak:RESult?
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:SUMMary:PPEak:AVERage:RESult?
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:SUMMary:PPEak:PHOLd:RESult?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Result_Type">
        /// This control selects the result type.
        /// 
        /// Valid Values:
        /// RSESCI_RESULT_CURR  (0) - Current
        /// RSESCI_RESULT_AVG   (1) - Average
        /// RSESCI_RESULT_PHOLD (2) - Peak Hold
        /// 
        /// Default Value: RSESCI_RESULT_CURR (0)
        /// 
        /// Note(s):
        /// 
        /// (1) Current - function returns the current peak value
        /// (2) Average - function returns the averaged peak value
        /// (3) Peak Hold - function returns the peak value with active peak hold
        /// </param>
        /// <param name="Result">
        /// This control returns the value of selected measurement.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetTimeDomainPowerPeakValue(int Window, int Result_Type, out double Result)
        {
            int pInvokeResult = PInvoke.GetTimeDomainPowerPeakValue(this._handle, Window, Result_Type, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the result of the measurement of the RMS power value in the selected measurement window. The function is independent of the marker selection. It is only available in the time domain (span = 0). A complete sweep with synchronization to sweep end must be performed between switching on the function and querying the measured value to obtain a valid query result. This is only possible in single sweep mode.
        /// 
        /// Mode(s):
        /// Spectrum Analysis (time domain only (zero span))
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:SUMMary:RMS:RESult?
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:SUMMary:RMS:AVERage:RESult?
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:SUMMary:RMS:PHOLd:RESult?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Result_Type">
        /// This control selects the result type.
        /// 
        /// Valid Values:
        /// RSESCI_RESULT_CURR  (0) - Current
        /// RSESCI_RESULT_AVG   (1) - Average
        /// RSESCI_RESULT_PHOLD (2) - Peak Hold
        /// 
        /// Default Value: RSESCI_RESULT_CURR (0)
        /// 
        /// Note(s):
        /// 
        /// (1) Current - function returns the current RMS value
        /// (2) Average - function returns the averaged RMS value
        /// (3) Peak Hold - function returns the RMS value with active peak hold
        /// </param>
        /// <param name="Result">
        /// This control returns the value of selected measurement.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetTimeDomainPowerRMSValue(int Window, int Result_Type, out double Result)
        {
            int pInvokeResult = PInvoke.GetTimeDomainPowerRMSValue(this._handle, Window, Result_Type, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the results of the standard deviation measurement. The function is independent of the marker selection. It is only available in the time domain (span = 0).
        /// A complete sweep with synchronization to sweep end must be performed between switching on the function and querying the measured value to obtain a valid query result. This is only possible in single sweep mode.
        /// 
        /// Mode(s):
        /// Spectrum Analysis (time domain only (zero span))
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:SUMMary:SDEViation:RESult?
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:SUMMary:SDEViation:AVERage:RESult?
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:SUMMary:SDEViation:PHOLd:RESult?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Result_Type">
        /// This control selects the result type.
        /// 
        /// Valid Values:
        /// RSESCI_RESULT_CURR  (0) - Current
        /// RSESCI_RESULT_AVG   (1) - Average
        /// RSESCI_RESULT_PHOLD (2) - Peak Hold
        /// 
        /// Default Value: RSESCI_RESULT_CURR (0)
        /// 
        /// Note(s):
        /// 
        /// (1) Current - function returns the current standard deviation value
        /// (2) Average - function returns the averaged standard deviation value
        /// (3) Peak Hold - function returns the standard deviation value with active peak hold
        /// </param>
        /// <param name="Result">
        /// This control returns the value of selected measurement.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetTimeDomainPowerStandardDeviationValue(int Window, int Result_Type, out double Result)
        {
            int pInvokeResult = PInvoke.GetTimeDomainPowerStandardDeviationValue(this._handle, Window, Result_Type, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function is used to determine the power of a sequence of signal pulses having the same interval, as are typical for the slots of a GSM signal, for example. The number of pulses to be measured as well as the measurement time and the period can be set. To define the position of the first pulse in the trace, a suitable offset can be entered. The evaluation is performed on the measurement data of a previously recorded trace. The data recorded during the set measurement time is combined to a measured value for each pulse according to the detector specified and the indicated number of results is output as a list. TRACE 1 of the selected screen is always used by the function.
        /// 
        /// Mode(s):
        /// Spectrum Analysis (time domain only (zero span))
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:MSUMmary? &lt;time offset of first pulse&gt;, &lt;measurement time&gt;, &lt;period&gt;, &lt;# of pulses to measure&gt;
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Time_offset_of_first_pulse">
        /// This control sets the time offset of the first pulse.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: 0.0
        /// </param>
        /// <param name="Measurement_Time">
        /// This control defines the measurement time.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: 0.0
        /// </param>
        /// <param name="Period">
        /// This control defines the measurement period.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: 0.0
        /// </param>
        /// <param name="Pulses">
        /// This control defines the number of pulses to measure.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: 0
        /// </param>
        /// <param name="Result">
        /// This control returns the power of a sequence of signal pulses defined by the given parameters.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetTimeDomainPowerSignalPulsesValue(int Window, double Time_offset_of_first_pulse, double Measurement_Time, double Period, int Pulses, out double Result)
        {
            int pInvokeResult = PInvoke.GetTimeDomainPowerSignalPulsesValue(this._handle, Window, Time_offset_of_first_pulse, Measurement_Time, Period, Pulses, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects type of power measurement in the selected measurement window. This function is independent of the selected marker.
        /// 
        /// Mode(s):
        /// Spectrum Analysis (frequency domain only)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:POWer[:STATe] OFF
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:POWer:SELect ACPower | CPOWer | MCACpower | OBANdwidth | CN | CN0
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Measurement">
        /// This control selects the power measurement.
        /// 
        /// Valid Values:
        /// RSESCI_FDOMAIN_MEAS_OFF  (0) - All Off
        /// RSESCI_FDOMAIN_MEAS_CPOW (1) - Channel Power
        /// RSESCI_FDOMAIN_MEAS_ACP  (2) - Adjacent-channel Power
        /// RSESCI_FDOMAIN_MEAS_MCAC (3) - Multi-carrier Power
        /// RSESCI_FDOMAIN_MEAS_OBW  (4) - Occupied Bandwidth
        /// RSESCI_FDOMAIN_MEAS_CN   (5) - Carrier-to-noise Ratio
        /// RSESCI_FDOMAIN_MEAS_CN0  (6) - Carrier-to-noise Ratio Referenced to 1 Hz
        /// 
        /// Default Value: RSESCI_FDOMAIN_MEAS_OFF (0)
        /// 
        /// Note(s):
        /// 
        /// (1) Off - switches off the frequency domain measurements in the selected measurement window.
        /// 
        /// (2) Channel Power - channel power measurement with a single carrier signal (equivalent to adjacent-channel power measurement with NO. OF ADJ CHAN = 0).
        /// 
        /// (3) Adjacent-channel Power - adjacent-channel power measurement with a single carrier signal
        /// 
        /// (4) Multi-carrier Power - channel/adjacent-channel power measurement with several carrier signals
        /// 
        /// (5) Occupied Bandwidth - the occupied bandwidth measurement is performed for the trace on which marker 1 is positioned. To select another trace for the measurement, marker 1 is to be positioned on the desired trace.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetPowerMeasurement(int Window, int Measurement)
        {
            int pInvokeResult = PInvoke.SetPowerMeasurement(this._handle, Window, Measurement);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries for selected power measurement.
        /// 
        /// Mode(s):
        /// Spectrum Analysis (frequency domain only)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:POWer[:STATe]?
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:POWer:SELect?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Measurement">
        /// This control returns selected power measurement.
        /// 
        /// Valid Values:
        /// RSESCI_FDOMAIN_MEAS_OFF  (0) - All Off
        /// RSESCI_FDOMAIN_MEAS_CPOW (1) - Channel Power
        /// RSESCI_FDOMAIN_MEAS_ACP  (2) - Adjacent-channel Power
        /// RSESCI_FDOMAIN_MEAS_MCAC (3) - Multi-carrier Power
        /// RSESCI_FDOMAIN_MEAS_OBW  (4) - Occupied Bandwidth
        /// RSESCI_FDOMAIN_MEAS_CN   (5) - Carrier-to-noise Ratio
        /// RSESCI_FDOMAIN_MEAS_CN0  (6) - Carrier-to-noise Ratio Referenced to 1 Hz
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetPowerMeasurement(int Window, out int Measurement)
        {
            int pInvokeResult = PInvoke.GetPowerMeasurement(this._handle, Window, out Measurement);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function adjusts the frequency span, the measurement bandwidths and the detector as required for the number of channels, the channel bandwidths and the channel spacings selected in the active power measurement. If necessary, adjacent-channel power measurement is switched on prior to the adjustment. To obtain valid results, a complete sweep with synchronization to the end of the sweep must be performed after the adjustment. Synchronization is possible only in the single-sweep mode.
        /// 
        /// Mode(s):
        /// Spectrum Analysis (frequency domain only)
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]POWer:ACHannel:PRESet ACPower | CPOWer | MCACpower | OBANwidth | CN | CN0
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Measurement">
        /// This control selects the power measurement.
        /// 
        /// Valid Values:
        /// RSESCI_FDOMAIN_MEAS_CPOW (1) - Channel Power
        /// RSESCI_FDOMAIN_MEAS_ACP  (2) - Adjacent-channel Power
        /// RSESCI_FDOMAIN_MEAS_MCAC (3) - Multi-carrier Power
        /// RSESCI_FDOMAIN_MEAS_OBW  (4) - Occupied Bandwidth
        /// RSESCI_FDOMAIN_MEAS_CN   (5) - Carrier-to-noise Ratio
        /// RSESCI_FDOMAIN_MEAS_CN0  (6) - Carrier-to-noise Ratio Referenced to 1 Hz
        /// 
        /// Default Value: RSESCI_FDOMAIN_MEAS_OFF (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int AdjustPowerMeasurementSetting(int Window, int Measurement)
        {
            int pInvokeResult = PInvoke.AdjustPowerMeasurementSetting(this._handle, Window, Measurement);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function adapts the reference level to the measured channel power and - if required - switches on previously the adjacent channel power measurement. This ensures that the signal path of the instrument is not overloaded. Since the measurement  bandwidth is significantly smaller than the signal bandwidth in channel power measurements, the signal path can be overloaded although the trace is still significantly below the reference level. If the measured channel power equals the reference level, the signal path is not overloaded. 
        /// 
        /// !!!!!!!! DELETE AFTER IMPLEMENTATION !!!!!!!!!!!
        /// Subsequent commands have to be synchronized with *WAI, *OPC or *OPC? to the end of the autorange process which would otherwise be aborted.
        /// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
        /// 
        /// Mode(s):
        /// Spectrum Analysis (frequency domain only)
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]POWer:ACHannel:PRESet:RLEVel
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int AdjustReferenceLevel(int Window)
        {
            int pInvokeResult = PInvoke.AdjustReferenceLevel(this._handle, Window);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the result of the power measurement performed in the selected window. If necessary, the measurement is switched on prior to the query. To obtain a valid result, a complete sweep with synchronization to the end of the sweep must be performed before a query is output. Synchronization is possible only in the single-sweep mode.
        /// 
        /// Mode(s):
        /// Spectrum Analysis (frequency domain only)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:POWer:RESult? ACPower | CPOWer | MCACpower | OBANdwidth | OBWidth | CN | CN0
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Measurement">
        /// This control selects the power measurement.
        /// 
        /// Valid Values:
        /// RSESCI_FDOMAIN_MEAS_CPOW (1) - Channel Power
        /// RSESCI_FDOMAIN_MEAS_ACP  (2) - Adjacent-channel Power
        /// RSESCI_FDOMAIN_MEAS_MCAC (3) - Multi-carrier Power
        /// RSESCI_FDOMAIN_MEAS_OBW  (4) - Occupied Bandwidth
        /// RSESCI_FDOMAIN_MEAS_CN   (5) - Carrier-to-noise Ratio
        /// RSESCI_FDOMAIN_MEAS_CN0  (6) - Carrier-to-noise Ratio Referenced to 1 Hz
        /// 
        /// Default Value: RSESCI_FDOMAIN_MEAS_OFF (0)
        /// </param>
        /// <param name="Array_Size">
        /// This control defines the size of array in argument 'Result'.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: 100
        /// </param>
        /// <param name="Result">
        /// This control returns the result value.
        /// 
        /// Note(s):
        /// 
        /// (1) Adjacent-channel power measurement:
        /// 
        /// Results are output in the following sequence:
        /// 
        /// 1. Power of transmission channel
        /// 2. Power of lower adjacent channel
        /// 3. Power of upper adjacent channel
        /// 4. Power of lower alternate channel 1
        /// 5. Power of upper alternate channel 1
        /// 6. Power of lower alternate channel 2
        /// 7. Power of upper alternate channel 2
        /// 
        /// The number of measured values returned depends on the number of
        /// adjacent/alternate channels selected with SENSe:POWer:ACHannel:ACPairs.
        /// With logarithmic scaling (RANGE LOG), the power is output in the
        /// currently selected level unit; with linear scaling (RANGE LIN dB or LIN %), the power is output in W. If SENSe:POWer:ACHannel:MODE REL is selected, the adjacent/alternate-channel power is output in dB.
        /// 
        /// (2) Channel power measurement:
        /// With logarithmic scaling (RANGE LOG), the channel power is output in the currently selected level unit; with linear scaling (RANGE LIN dB or LIN %), the channel power is output in W.
        /// 
        /// (3) Mutli-carrier Power:
        /// Channel/adjacent-channel power measurement with several carrier signals.
        /// 
        /// Results are output in the following sequence, separated by commas:
        /// 
        /// 1. Power of carrier signal 1
        /// 2. Power of carrier signal 2
        /// 3. Power of carrier signal 3
        /// 4. Power of carrier signal 4
        /// 5. Power of carrier signal 5
        /// 6. Power of carrier signal 6
        /// 7. Power of carrier signal 7
        /// 8. Power of carrier signal 8
        /// 9. Power of carrier signal 9
        /// 10. Power of carrier signal 10
        /// 11. Power of carrier signal 11
        /// 12. Power of carrier signal 12
        /// 13. Total power of all carrier signals
        /// 14. Power of lower adjacent channel
        /// 15. Power of upper adjacent channel
        /// 16. Power of lower alternate channel 1
        /// 17. Power of upper alternate channel 1
        /// 18. Power of lower alternate channel 2
        /// 19. Power of upper alternate channel 2
        /// The number of measured values returned depends on the number of
        /// carrier signals and adjacent/alternate channels selected with SENSe:POWer:ACHannel:TXCHannel:COUNt and SENSe:POWer:ACHannel:ACPairs. If only one carrier signal is measured, the total value of all carrier signals will not be output. With logarithmic scaling (RANGE LOG), the power is output in dBm; with linear scaling (RANGE LIN dB or LIN %), the power is output in W. If SENSe:POWer:ACHannel:MODE REL is selected, the adjacent/alternate-channel power is output in dB.
        /// 
        /// (4) Measurement of occupied bandwidth:
        /// The occupied bandwidth in Hz is returned.
        /// 
        /// (5) Measurement of carrier-to-noise ratio:
        /// The carrier-to-noise ratio in dB is returned.
        /// 
        /// (6) Measurement of carrier-to-noise ratio referenced to 1 Hz bandwidth:
        /// The carrier-to-noise ratio in dB/Hz is returned.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetPowerMeasurementValue(int Window, int Measurement, int Array_Size, double[] Result)
        {
            int pInvokeResult = PInvoke.GetPowerMeasurementValue(this._handle, Window, Measurement, Array_Size, Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches the query response of the power measurement results in the indicated measurement window between output of absolute values (OFF) and output referred to the  measurement bandwith (ON).
        /// 
        /// Mode(s):
        /// Spectrum Analysis (frequency domain only)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:POWer:RESult:PHZ ON | OFF
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Relative_Result">
        /// This control switches the query response of the power measurement results in the indicated measurement window between output of absolute values (OFF) and output referred to the measurement bandwith (ON).
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetChannelPowerHzMeasurement(int Window, bool Relative_Result)
        {
            int pInvokeResult = PInvoke.SetChannelPowerHzMeasurement(this._handle, Window, System.Convert.ToUInt16(Relative_Result));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the state of the query response of the power measurement results in the indicated measurement window.
        /// 
        /// Mode(s):
        /// Spectrum Analysis (frequency domain only)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:POWer:RESult:PHZ?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Relative_Result">
        /// This control returns the state of the query response of the power measurement results in the indicated measurement window.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetChannelPowerHzMeasurement(int Window, out bool Relative_Result)
        {
            ushort Relative_ResultAsUShort;
            int pInvokeResult = PInvoke.GetChannelPowerHzMeasurement(this._handle, Window, out Relative_ResultAsUShort);
            Relative_Result = System.Convert.ToBoolean(Relative_ResultAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the Clear Write or Maxhold for Channel Power values. This function is independent of the selected marker.
        /// 
        /// Mode(s):
        /// Spectrum Analysis (frequency domain only)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:POWer:MODE WRITe | MAXHold
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Power_Mode">
        /// This control selects the Clear Write or Maxhold for Channel Power values.
        /// 
        /// Valid Values:
        /// RSESCI_TRACE_WRITE  (0) - Clear/Write
        /// RSESCI_TRACE_MAXH   (3) - Max Hold
        /// 
        /// 
        /// Default Value: RSESCI_TRACE_WRITE (0)
        /// 
        /// Note(s):
        /// 
        /// (1) In the CLEAR/WRITE mode the channel power and the adjacent channel powers are calculated directly from the current trace (default mode).
        /// 
        /// (2) In MAX HOLD mode the power values are still derived from the current trace, but they are compared with the previous power value using a maximum algorithm. The higher value is remained.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetPowerMode(int Window, int Power_Mode)
        {
            int pInvokeResult = PInvoke.SetPowerMode(this._handle, Window, Power_Mode);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the Clear Write or Maxhold for Channel Power values. This function is independent of the selected marker.
        /// 
        /// Mode(s):
        /// Spectrum Analysis (frequency domain only)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:POWer:MODE?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Power_Mode">
        /// This control returns the Clear Write or Maxhold for Channel Power values.
        /// 
        /// Valid Values:
        /// RSESCI_TRACE_WRITE  (0) - Clear/Write
        /// RSESCI_TRACE_MAXH   (3) - Max Hold
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetPowerMode(int Window, out int Power_Mode)
        {
            int pInvokeResult = PInvoke.GetPowerMode(this._handle, Window, out Power_Mode);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the power measurement setting for a standard in the indicated measurement window and previously switches on the corresponding measurement, if required. The function is independent of the marker selection. The configuration for a standard comprises of the parameters weighting filter, channel bandwidth and spacing, resolution and video bandwidth, as well as detector and sweep time.
        /// 
        /// Mode(s):
        /// Spectrum Analysis (frequency domain only)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:POWer:PRESet NADC | TETRA | PDC | PHS | CDPD | FWCDma | RWCDma | F8CDma | R8CDma | F19Cdma | R19Cdma | FW3Gppcdma | RW3Gppcdma | D2CDma | S2CDma | M2CDma | FIS95C0 | RIS95C0 | FIS95C1 | RIS95C1 | TCDMa | NONE | AWLan | BWLan
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Standard">
        /// This control selects the standard for which power measurement setting will be changed.
        /// 
        /// Valid Values:
        /// RSESCI_STANDARD_NONE    (0) - None
        /// RSESCI_STANDARD_NADC    (1) - NADC
        /// RSESCI_STANDARD_TETRA   (2) - TETRA
        /// RSESCI_STANDARD_PDC     (3) - PDC
        /// RSESCI_STANDARD_PHS     (4) - PHS
        /// RSESCI_STANDARD_CDPD    (5) - CDPD
        /// RSESCI_STANDARD_FWCD    (6) - W-CDMA 4.096 MHz forward
        /// RSESCI_STANDARD_RWCD    (7) - W-CDMA 4.096 MHz reverse
        /// RSESCI_STANDARD_F8CD    (8) - CDMA IS95A forward
        /// RSESCI_STANDARD_R8CD    (9) - CDMA IS95A reverse
        /// RSESCI_STANDARD_F19C    (10) - CDMA J-STD008 forward
        /// RSESCI_STANDARD_R19C    (11) - CDMA J-STD008 reverse
        /// RSESCI_STANDARD_FW3G    (12) - W-CDMA 3.84 MHz forward
        /// RSESCI_STANDARD_RW3G    (13) - W-CDMA 3.84 MHz reverse
        /// RSESCI_STANDARD_D2CD    (14) - CDMA 2000 direct sequence
        /// RSESCI_STANDARD_S2CD    (15) - CDMA 2000 MC1
        /// RSESCI_STANDARD_M2CD    (16) - CDMA 2000 MC3
        /// RSESCI_STANDARD_FIS95C0 (17) - CDMA IS95C Class 0 forward
        /// RSESCI_STANDARD_RIS95C0 (18) - CDMA IS95C Class 0 reverse
        /// RSESCI_STANDARD_FIS95C1 (19) - CDMA IS95C Class 1 forward
        /// RSESCI_STANDARD_RIS95C1 (20) - CDMA IS95C Class 1 reverse
        /// RSESCI_STANDARD_TCDM    (21) - TD-SCDMA
        /// RSESCI_STANDARD_AWL     (22) - WLAN 802.11a
        /// RSESCI_STANDARD_BWL     (23) - WLAN 802.11b
        /// 
        /// Default Value: RSESCI_STANDARD_NONE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetCPACPStandard(int Window, int Standard)
        {
            int pInvokeResult = PInvoke.SetCPACPStandard(this._handle, Window, Standard);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries selected standard for which power measurement setting are set.
        /// 
        /// Mode(s):
        /// Spectrum Analysis (frequency domain only)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:POWer:PRESet?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Standard">
        /// This control returns selected standard for which power measurement setting are set.
        /// 
        /// Valid Values:
        /// RSESCI_STANDARD_NONE    (0) - None
        /// RSESCI_STANDARD_NADC    (1) - NADC
        /// RSESCI_STANDARD_TETRA   (2) - TETRA
        /// RSESCI_STANDARD_PDC     (3) - PDC
        /// RSESCI_STANDARD_PHS     (4) - PHS
        /// RSESCI_STANDARD_CDPD    (5) - CDPD
        /// RSESCI_STANDARD_FWCD    (6) - W-CDMA 4.096 MHz forward
        /// RSESCI_STANDARD_RWCD    (7) - W-CDMA 4.096 MHz reverse
        /// RSESCI_STANDARD_F8CD    (8) - CDMA IS95A forward
        /// RSESCI_STANDARD_R8CD    (9) - CDMA IS95A reverse
        /// RSESCI_STANDARD_F19C    (10) - CDMA J-STD008 forward
        /// RSESCI_STANDARD_R19C    (11) - CDMA J-STD008 reverse
        /// RSESCI_STANDARD_FW3G    (12) - W-CDMA 3.84 MHz forward
        /// RSESCI_STANDARD_RW3G    (13) - W-CDMA 3.84 MHz reverse
        /// RSESCI_STANDARD_D2CD    (14) - CDMA 2000 direct sequence
        /// RSESCI_STANDARD_S2CD    (15) - CDMA 2000 MC1
        /// RSESCI_STANDARD_M2CD    (16) - CDMA 2000 MC3
        /// RSESCI_STANDARD_FIS95C0 (17) - CDMA IS95C Class 0 forward
        /// RSESCI_STANDARD_RIS95C0 (18) - CDMA IS95C Class 0 reverse
        /// RSESCI_STANDARD_FIS95C1 (19) - CDMA IS95C Class 1 forward
        /// RSESCI_STANDARD_RIS95C1 (20) - CDMA IS95C Class 1 reverse
        /// RSESCI_STANDARD_TCDM    (21) - TD-SCDMA
        /// RSESCI_STANDARD_AWL     (22) - WLAN 802.11a
        /// RSESCI_STANDARD_BWL     (23) - WLAN 802.11b
        /// RSESCI_STANDARD_USER    (24) - User defined
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetCPACPStandard(int Window, out int Standard)
        {
            int pInvokeResult = PInvoke.GetCPACPStandard(this._handle, Window, out Standard);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns all predefine ACP standards and all user standards.
        /// 
        /// Mode(s): 
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:POWer:STANdard:CATalog?
        /// 
        /// 
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Array_Size">
        /// This control defines the size of array for the standard catalog.
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: 256
        /// </param>
        /// <param name="Catalog">
        /// This control returns all standards for which power measurement setting are set.
        /// 
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetACPStandardCatalog(int Window, int Array_Size, System.Text.StringBuilder Catalog)
        {
            int pInvokeResult = PInvoke.GetACPStandardCatalog(this._handle, Window, Array_Size, Catalog);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects user defined power measurement setting for a standard in the indicated measurement window and previously switches on the corresponding measurement, if required. The function is independent of the marker selection. The configuration for a standard comprises of the parameters weighting filter, channel bandwidth and spacing, resolution and video bandwidth, as well as detector and sweep time.
        /// 
        /// Mode(s):
        /// Spectrum Analysis (frequency domain only)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:POWer:PRESet
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="User_Standard_Name">
        /// This control selects name of ACP user standard.
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: ""
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetCPACPUserStandard(int Window, string User_Standard_Name)
        {
            int pInvokeResult = PInvoke.SetCPACPUserStandard(this._handle, Window, User_Standard_Name);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function removes the ACP user standard &lt;name&gt;. It is not possible to remove predefine ACP standards.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:POWer:STANdard:DELete
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Standard_Name">
        /// This control removes the ACP user standard &lt;name&gt;.
        /// 
        /// Valid Values:
        /// Not checked
        /// 
        /// Default Value:"" 
        /// 
        /// Note(s):
        /// 
        /// (1) It is not possible to remove predefine ACP standards.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int DeleteACPUserStandard(int Window, string Standard_Name)
        {
            int pInvokeResult = PInvoke.DeleteACPUserStandard(this._handle, Window, Standard_Name);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This command saves the current ACP settings as an ACP user standard.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:POWer:STANdard:SAVE
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Standard_Name">
        /// This control is the file name of the saved current ACP as an ACP user standard.
        /// 
        /// Valid Values:
        /// Not checked
        /// 
        /// Default Value:""
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SaveACPUserStandard(int Window, string Standard_Name)
        {
            int pInvokeResult = PInvoke.SaveACPUserStandard(this._handle, Window, Standard_Name);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function allows to modify the ADJUST REF LEVEL function for user standards. It defines an additional level offset to be taken into account for the analyzer's reference level setting. It is only available if Fast ACP is not active and no predefine standard is selected.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]POWer:ACHannel:PRESet:RLEVel:OFFSet
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Reference_Level_Offset">
        /// This control sets Reference Level Offset for user standards.
        /// 
        /// Valid Values:
        /// 0.0 to 100.0 dB
        /// 
        /// Default Value: 0.0 dB
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetACPReferenceLevelOffset(int Window, double Reference_Level_Offset)
        {
            int pInvokeResult = PInvoke.SetACPReferenceLevelOffset(this._handle, Window, Reference_Level_Offset);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the modification of the ADJUST REF LEVEL function for user standards. It defines an additional level offset to be taken into account for the analyzer's reference level setting. It is only available if Fast ACP is not active and no predefine standard is selected.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]POWer:ACHannel:PRESet:RLEVel:OFFSet?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Reference_Level_Offset">
        /// This control returns Reference Level Offset for user standards.
        /// 
        /// Valid Values:
        /// 0.0 to 100.0 dB
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetACPReferenceLevelOffset(int Window, out double Reference_Level_Offset)
        {
            int pInvokeResult = PInvoke.GetACPReferenceLevelOffset(this._handle, Window, out Reference_Level_Offset);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the reference value for the relative measurement to the currently measured channel power.
        /// 
        /// Mode(s):
        /// Spectrum Analysis (frequency domain only)
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]POWer:ACHannel:REFerence:AUTO ONCE
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetCPReference(int Window)
        {
            int pInvokeResult = PInvoke.SetCPReference(this._handle, Window);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches on or off the correction of the  instrument inherent noise for channel power measurement. On activating this function, a reference measurement of the instrument inherent noise is performed. The measured noise power is then subtracted from the power in the examined channel. The instrument inherent noise is then re-determined after any change of the center frequency, resolution bandwidth, sweep time and level setting by repeating the reference measurement in the
        /// new instrument setting.
        /// 
        /// Mode(s):
        /// Spectrum Analysis (frequency domain only)
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]POWer:NCORrection ON | OFF
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Correction">
        /// This control switches on or off the correction of the  instrument inherent noise for channel power measurement.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetNoiseCorrection(int Window, bool Correction)
        {
            int pInvokeResult = PInvoke.SetNoiseCorrection(this._handle, Window, System.Convert.ToUInt16(Correction));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the state of the correction of the  instrument inherent noise for channel power measurement.
        /// 
        /// Mode(s):
        /// Spectrum Analysis (frequency domain only)
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]POWer:NCORrection?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Correction">
        /// This control returns the state of the correction of the  instrument inherent noise for channel power measurement.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetNoiseCorrection(int Window, out bool Correction)
        {
            ushort CorrectionAsUShort;
            int pInvokeResult = PInvoke.GetNoiseCorrection(this._handle, Window, out CorrectionAsUShort);
            Correction = System.Convert.ToBoolean(CorrectionAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches on or off the high-speed channel/adjacent channel power measurement. The measurement itself is performed in the time domain on the center frequencies of the individual channels. The command automatically switches to the time domain and back. Depending on the selected mobile radio standard, weighting filters with SQRT(cos) characteristic or very steep-sided channel filters are used for band limitation. The command is only available in the frequency domain (span &gt; 0).
        /// 
        /// Mode(s):
        /// Spectrum Analysis (frequency domain only)
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]POWer:HSPeed ON | OFF
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Fast_ACP">
        /// This control switches on or off the high-speed channel/adjacent channel power measurement.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetFastACP(int Window, bool Fast_ACP)
        {
            int pInvokeResult = PInvoke.SetFastACP(this._handle, Window, System.Convert.ToUInt16(Fast_ACP));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the state of the high-speed channel/adjacent channel power measurement.
        /// 
        /// Mode(s):
        /// Spectrum Analysis (frequency domain only)
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]POWer:HSPeed?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Fast_ACP">
        /// This control returns the state of the high-speed channel/adjacent channel power measurement.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetFastACP(int Window, out bool Fast_ACP)
        {
            ushort Fast_ACPAsUShort;
            int pInvokeResult = PInvoke.GetFastACP(this._handle, Window, out Fast_ACPAsUShort);
            Fast_ACP = System.Convert.ToBoolean(Fast_ACPAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches the measurement window for channel and adjacent-channel power measurements or for active FM demodulator (FS-K7) to full screen or half screen.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:SIZE LARGe | SMALl
        /// </summary>
        /// <param name="Size">
        /// This control switches the measurement window for channel and adjacent-channel power measurements or for active FM demodulator (FS-K7) to full screen or half screen.
        /// 
        /// Valid Values:
        /// RSESCI_WINDOW_SMALL (0) - Small
        /// RSESCI_WINDOW_LARGE (1) - Large
        /// 
        /// Default Value: RSESCI_WINDOW_SMALL (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetFullSizeDiagram(int Size)
        {
            int pInvokeResult = PInvoke.SetFullSizeDiagram(this._handle, Size);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the state of the measurement window for channel and adjacent-channel power measurements or for active FM demodulator (FS-K7) to full screen or half screen.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:SIZE?
        /// </summary>
        /// <param name="Size">
        /// This control returns the state of the measurement window for channel and adjacent-channel power measurements or for active FM demodulator (FS-K7) to full screen or half screen.
        /// 
        /// Valid Values:
        /// RSESCI_WINDOW_SMALL (0) - Small
        /// RSESCI_WINDOW_LARGE (1) - Large
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetFullSizeDiagram(out int Size)
        {
            int pInvokeResult = PInvoke.GetFullSizeDiagram(this._handle, out Size);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the number of adjacent channels (upper and lower channel in pairs).The figure 0 stands for pure channel power measurement. The function is only available in the frequency domain (span &gt; 0).
        /// 
        /// Mode(s):
        /// Spectrum Analysis (frequency domain only)
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]POWer:ACHannel:ACPairs 0..12
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Number_of_Adjacent_Channels">
        /// This control sets the number of adjacent channels (upper and lower channel in pairs).The figure 0 stands for pure channel power measurement.
        /// 
        /// Valid Range: 0 to 12
        /// 
        /// Default Value: 0
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetNumberAdjacentChannels(int Window, int Number_of_Adjacent_Channels)
        {
            int pInvokeResult = PInvoke.SetNumberAdjacentChannels(this._handle, Window, Number_of_Adjacent_Channels);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the number of adjacent channels (upper and lower channel in pairs).The figure 0 stands for pure channel power measurement.
        /// 
        /// Mode(s):
        /// Spectrum Analysis (frequency domain only)
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]POWer:ACHannel:ACPairs?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Number_of_Adjacent_Channels">
        /// This control returns the number of adjacent channels (upper and lower channel in pairs).The figure 0 stands for pure channel power measurement.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetNumberAdjacentChannels(int Window, out int Number_of_Adjacent_Channels)
        {
            int pInvokeResult = PInvoke.GetNumberAdjacentChannels(this._handle, Window, out Number_of_Adjacent_Channels);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the number of carrier signals. The function is available only for multicarrier channel and adjacent-channel power measurements in the frequency domain (span &gt; 0).
        /// 
        /// Mode(s):
        /// Spectrum Analysis (frequency domain only)
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]POWer:ACHannel:TXCHannel:COUNt 1..12
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Number_of_Carriers">
        /// This control selects the number of carrier signals.
        /// 
        /// Valid Range: 1 to 12
        /// 
        /// Default Value: 4
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetNumberTXChannels(int Window, int Number_of_Carriers)
        {
            int pInvokeResult = PInvoke.SetNumberTXChannels(this._handle, Window, Number_of_Carriers);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the number of carrier signals. The function is available only for multicarrier channel and adjacent-channel power measurements in the frequency domain (span &gt; 0).
        /// 
        /// Mode(s):
        /// Spectrum Analysis (frequency domain only)
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]POWer:ACHannel:TXCHannel:COUNt?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Number_of_Carriers">
        /// This control returns the number of carrier signals.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetNumberTXChannels(int Window, out int Number_of_Carriers)
        {
            int pInvokeResult = PInvoke.GetNumberTXChannels(this._handle, Window, out Number_of_Carriers);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the selected channel bandwidth value.
        /// 
        /// Mode(s):
        /// Spectrum Analysis (frequency domain only)
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]POWer:ACHannel:BANDwidth|BWIDth[:CHANnel]
        /// [SENSe&lt;1|2&gt;:]POWer:ACHannel:BANDwidth|BWIDth:ACHannel
        /// [SENSe&lt;1|2&gt;:]POWer:ACHannel:BANDwidth|BWIDth:ALTernate&lt;1...11&gt;
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Channel_Bandwidth">
        /// This control selects the channel bandwidth type.
        /// 
        /// Valid Values:
        /// RSESCI_BWID_TX    (0)  - TX
        /// RSESCI_BWID_ADJ   (1)  - Adjacent
        /// RSESCI_BWID_ALT1  (2)  - Alternate 1
        /// RSESCI_BWID_ALT2  (3)  - Alternate 2
        /// RSESCI_BWID_ALT3  (4)  - Alternate 3
        /// RSESCI_BWID_ALT4  (5)  - Alternate 4
        /// RSESCI_BWID_ALT5  (6)  - Alternate 5
        /// RSESCI_BWID_ALT6  (7)  - Alternate 6
        /// RSESCI_BWID_ALT7  (8)  - Alternate 7
        /// RSESCI_BWID_ALT8  (9)  - Alternate 8
        /// RSESCI_BWID_ALT9  (10) - Alternate 9
        /// RSESCI_BWID_ALT10 (11) - Alternate 10
        /// RSESCI_BWID_ALT11 (12) - Alternate 11
        /// 
        /// Default Value: RSESCI_BWID_TX (0)
        /// 
        /// Note(s):
        /// (1) Tx - sets the channel bandwidth of the radio communication system.The bandwidths of adjacent channels are not influenced by this modification (in contrast to the FSE family).
        /// 
        /// (2) Adjacent - sets the channel bandwidth of the adjacent channel of the radio transmission system. If the bandwidth of the adjacent channel is changed, the bandwidths of all alternate adjacent channels are automatically set to the same value.
        /// 
        /// (3) Alternate 1..11 - sets the channel bandwidth of the first/second alternate adjacent channel of the radio transmission system. If the channel bandwidth of alternate adjacent channel 1 is changed, the bandwidth of alternate adjacent channel 2 is automatically set to the same value.
        /// 
        /// (4) With SENS:POW:HSP ON the steep-edged channel filters from the table "List of available channel filters" in Section "Setting Bandwidths and Sweep Time - Key BW" are available.
        /// </param>
        /// <param name="Channel_Bandwidth_Value">
        /// This control sets the channel bandwidth value.
        /// 
        /// Valid Range: 100.0 to 1.0E9 Hz
        /// 
        /// Default Value: 14e3 Hz
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetTXACPChannelBandwidth(int Window, int Channel_Bandwidth, double Channel_Bandwidth_Value)
        {
            int pInvokeResult = PInvoke.SetTXACPChannelBandwidth(this._handle, Window, Channel_Bandwidth, Channel_Bandwidth_Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the channel bandwidth value.
        /// 
        /// Mode(s):
        /// Spectrum Analysis (frequency domain only)
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]POWer:ACHannel:BANDwidth|BWIDth[:CHANnel]?
        /// [SENSe&lt;1|2&gt;:]POWer:ACHannel:BANDwidth|BWIDth:ACHannel?
        /// [SENSe&lt;1|2&gt;:]POWer:ACHannel:BANDwidth|BWIDth:ALTernate&lt;1...11&gt;?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Channel_Bandwidth">
        /// This control selects the channel bandwidth type.
        /// 
        /// Valid Values:
        /// RSESCI_BWID_TX    (0)  - TX
        /// RSESCI_BWID_ADJ   (1)  - Adjacent
        /// RSESCI_BWID_ALT1  (2)  - Alternate 1
        /// RSESCI_BWID_ALT2  (3)  - Alternate 2
        /// RSESCI_BWID_ALT3  (4)  - Alternate 3
        /// RSESCI_BWID_ALT4  (5)  - Alternate 4
        /// RSESCI_BWID_ALT5  (6)  - Alternate 5
        /// RSESCI_BWID_ALT6  (7)  - Alternate 6
        /// RSESCI_BWID_ALT7  (8)  - Alternate 7
        /// RSESCI_BWID_ALT8  (9)  - Alternate 8
        /// RSESCI_BWID_ALT9  (10) - Alternate 9
        /// RSESCI_BWID_ALT10 (11) - Alternate 10
        /// RSESCI_BWID_ALT11 (12) - Alternate 11
        /// 
        /// Default Value: RSESCI_BWID_TX (0)
        /// 
        /// Note(s):
        /// (1) Tx - sets the channel bandwidth of the radio communication system.The bandwidths of adjacent channels are not influenced by this modification (in contrast to the FSE family).
        /// 
        /// (2) Adjacent - sets the channel bandwidth of the adjacent channel of the radio transmission system. If the bandwidth of the adjacent channel is changed, the bandwidths of all alternate adjacent channels are automatically set to the same value.
        /// 
        /// (3) Alternate 1..11 - sets the channel bandwidth of the first/second alternate adjacent channel of the radio transmission system. If the channel bandwidth of alternate adjacent channel 1 is changed, the bandwidth of alternate adjacent channel 2 is automatically set to the same value.
        /// 
        /// (4) With SENS:POW:HSP ON the steep-edged channel filters from the table "List of available channel filters" in Section "Setting Bandwidths and Sweep Time - Key BW" are available.
        /// </param>
        /// <param name="Channel_Bandwidth_Value">
        /// This control returns the channel bandwidth value.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetTXACPChannelBandwidth(int Window, int Channel_Bandwidth, out double Channel_Bandwidth_Value)
        {
            int pInvokeResult = PInvoke.GetTXACPChannelBandwidth(this._handle, Window, Channel_Bandwidth, out Channel_Bandwidth_Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the channel spacing for selected channel.
        /// 
        /// Mode(s):
        /// Spectrum Analysis (frequency domain only)
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]POWer:ACHannel:SPACing:CHANnel&lt;1...11&gt;
        /// [SENSe&lt;1|2&gt;:]POWer:ACHannel:SPACing:ACHannel
        /// [SENSe&lt;1|2&gt;:]POWer:ACHannel:SPACing:ALTernate&lt;1...11&gt;
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Channel_Spacing">
        /// This control selects the channel spacing type.
        /// 
        /// Valid Values:
        /// RSESCI_BWID_TX    (0)  - TX 1
        /// RSESCI_BWID_TX2   (13) - TX 2
        /// RSESCI_BWID_TX3   (14) - TX 3
        /// RSESCI_BWID_TX4   (15) - TX 4
        /// RSESCI_BWID_TX5   (16) - TX 5
        /// RSESCI_BWID_TX6   (17) - TX 6
        /// RSESCI_BWID_TX7   (18) - TX 7
        /// RSESCI_BWID_TX8   (19) - TX 8
        /// RSESCI_BWID_TX9   (20) - TX 9
        /// RSESCI_BWID_TX10  (21) - TX 10
        /// RSESCI_BWID_TX11  (22) - TX 11
        /// RSESCI_BWID_ADJ   (1)  - Adjacent
        /// RSESCI_BWID_ALT1  (2)  - Alternate 1
        /// RSESCI_BWID_ALT2  (3)  - Alternate 2
        /// RSESCI_BWID_ALT3  (4)  - Alternate 3
        /// RSESCI_BWID_ALT4  (5)  - Alternate 4
        /// RSESCI_BWID_ALT5  (6)  - Alternate 5
        /// RSESCI_BWID_ALT6  (7)  - Alternate 6
        /// RSESCI_BWID_ALT7  (8)  - Alternate 7
        /// RSESCI_BWID_ALT8  (9)  - Alternate 8
        /// RSESCI_BWID_ALT9  (10) - Alternate 9
        /// RSESCI_BWID_ALT10 (11) - Alternate 10
        /// RSESCI_BWID_ALT11 (12) - Alternate 11
        /// 
        /// Default Value: RSESCI_BWID_TX (0)
        /// 
        /// Note(s):
        /// (1) TX - defines the channel spacing for the carrier signals.
        /// 
        /// (2) Adjacent - defines the channel spacing of the adjacent channel to the TX channel. At the same time, the spacing of alternate adjacent channels 1 and 2 is set to the double or triple of the entered value.
        /// 
        /// (3) Alternate 1..11 - defines the spacing between the first (Alternate 1) or the second alternate adjacent channel (Alternate 2) and the TX channel.If the spacing to the alternate adjacent channel ALTernate1 is modified, the spacing to alternate adjacent channel 2 is set to 1.5 times the entered value.
        /// </param>
        /// <param name="Channel_Spacing_Value">
        /// This control sets the channel spacing value.
        /// 
        /// Valid Range: 100.0 to 2.0E9 Hz
        /// 
        /// Default Value: 14.0e3 Hz
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetACPChannelSpacing(int Window, int Channel_Spacing, double Channel_Spacing_Value)
        {
            int pInvokeResult = PInvoke.SetACPChannelSpacing(this._handle, Window, Channel_Spacing, Channel_Spacing_Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the channel spacing for selected channel.
        /// 
        /// Mode(s):
        /// Spectrum Analysis (frequency domain only)
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]POWer:ACHannel:SPACing:CHANnel&lt;1...11&gt;?
        /// [SENSe&lt;1|2&gt;:]POWer:ACHannel:SPACing:ACHannel?
        /// [SENSe&lt;1|2&gt;:]POWer:ACHannel:SPACing:ALTernate&lt;1...11&gt;?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Channel_Spacing">
        /// This control selects the channel spacing type.
        /// 
        /// Valid Values:
        /// RSESCI_BWID_TX    (0)  - TX 1
        /// RSESCI_BWID_TX2   (13) - TX 2
        /// RSESCI_BWID_TX3   (14) - TX 3
        /// RSESCI_BWID_TX4   (15) - TX 4
        /// RSESCI_BWID_TX5   (16) - TX 5
        /// RSESCI_BWID_TX6   (17) - TX 6
        /// RSESCI_BWID_TX7   (18) - TX 7
        /// RSESCI_BWID_TX8   (19) - TX 8
        /// RSESCI_BWID_TX9   (20) - TX 9
        /// RSESCI_BWID_TX10  (21) - TX 10
        /// RSESCI_BWID_TX11  (22) - TX 11
        /// RSESCI_BWID_ADJ   (1)  - Adjacent
        /// RSESCI_BWID_ALT1  (2)  - Alternate 1
        /// RSESCI_BWID_ALT2  (3)  - Alternate 2
        /// RSESCI_BWID_ALT3  (4)  - Alternate 3
        /// RSESCI_BWID_ALT4  (5)  - Alternate 4
        /// RSESCI_BWID_ALT5  (6)  - Alternate 5
        /// RSESCI_BWID_ALT6  (7)  - Alternate 6
        /// RSESCI_BWID_ALT7  (8)  - Alternate 7
        /// RSESCI_BWID_ALT8  (9)  - Alternate 8
        /// RSESCI_BWID_ALT9  (10) - Alternate 9
        /// RSESCI_BWID_ALT10 (11) - Alternate 10
        /// RSESCI_BWID_ALT11 (12) - Alternate 11
        /// 
        /// Default Value: RSESCI_BWID_TX (0)
        /// 
        /// Note(s):
        /// (1) TX - defines the channel spacing for the carrier signals.
        /// 
        /// (2) Adjacent - defines the channel spacing of the adjacent channel to the TX channel. At the same time, the spacing of alternate adjacent channels 1 and 2 is set to the double or triple of the entered value.
        /// 
        /// (3) Alternate 1..2 - defines the spacing between the first (Alternate 1) or the second alternate adjacent channel (Alternate 2) and the TX channel.If the spacing to the alternate adjacent channel ALTernate1 is modified, the spacing to alternate adjacent channel 2 is set to 1.5 times the entered value.
        /// </param>
        /// <param name="Channel_Spacing_Value">
        /// This control returns the channel spacing value.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetACPChannelSpacing(int Window, int Channel_Spacing, out double Channel_Spacing_Value)
        {
            int pInvokeResult = PInvoke.GetACPChannelSpacing(this._handle, Window, Channel_Spacing, out Channel_Spacing_Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects a transmission channel to be used as a reference channel in relative adjacent-channel power measurements.
        /// 
        /// Mode(s):
        /// Spectrum Analysis (frequency domain only)
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]POWer:ACHannel:REFerence:TXCHannel:MANual 1 | 2 | 3 | 4
        /// [SENSe&lt;1|2&gt;:]POWer:ACHannel:REFerence:TXCHannel:AUTO MINimum | MAXimum | LHIGhest
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="ACP_Reference_Channel">
        /// This control selects a transmission channel to be used as a reference channel in relative adjacent-channel power measurements.
        /// 
        /// Valid Values:
        /// RSESCI_ACP_REFERENCE_TX1      (0) - TX Channel 1
        /// RSESCI_ACP_REFERENCE_TX2      (1) - TX Channel 2
        /// RSESCI_ACP_REFERENCE_TX3      (2) - TX Channel 3
        /// RSESCI_ACP_REFERENCE_TX4      (3) - TX Channel 4
        /// RSESCI_ACP_REFERENCE_MIN      (4) - Min Power TX Channel
        /// RSESCI_ACP_REFERENCE_MAX      (5) - Max Power TX Channel
        /// RSESCI_ACP_REFERENCE_LOW_HIGH (6) - Lowest &amp; Highest Channel
        /// 
        /// Default Value: RSESCI_ACP_REFERENCE_TX1 (0)
        /// 
        /// Note(s):
        /// 
        /// (1) TX Channel 1 to 4 - selects a transmission channel to be used as a reference channel in relative adjacent-channel power measurements.
        /// 
        /// (2) Min Power TX Channel - automatic selection of transmission channel with the lowest power
        /// 
        /// (3) Max Power TX Channel - automatic selection of transmission channel with the highest power
        /// 
        /// (4) Lowest &amp; Highest Channel - automatic selection of lowermost transmission channel for the lower adjacent channels, uppermost transmission channel for the upper adjacent channels
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetACPReferenceChannel(int Window, int ACP_Reference_Channel)
        {
            int pInvokeResult = PInvoke.SetACPReferenceChannel(this._handle, Window, ACP_Reference_Channel);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function toggles between absolute and relative adjacent channel measurement. For the relative measurement the reference value is set to the currently measured channel power by function rsesci_SetCPReference().
        /// 
        /// Mode(s):
        /// Spectrum Analysis (frequency domain only)
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]POWer:ACHannel:MODE ABSolute | RELative
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Mode">
        /// This control toggles between absolute and relative adjacent channel measurement.
        /// 
        /// Valid Values:
        /// RSESCI_ABS (0) - Absolute
        /// RSESCI_REL (1) - Relative
        /// 
        /// Default Value: RSESCI_ABS (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetCPACPPowerMeasurementMode(int Window, int Mode)
        {
            int pInvokeResult = PInvoke.SetCPACPPowerMeasurementMode(this._handle, Window, Mode);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns if absolute and relative adjacent channel measurement is selected.
        /// 
        /// Mode(s):
        /// Spectrum Analysis (frequency domain only)
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]POWer:ACHannel:MODE?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Mode">
        /// This control returns selected adjacent channel measurement mode.
        /// 
        /// Valid Values:
        /// RSESCI_ABS (0) - Absolute
        /// RSESCI_REL (1) - Relative
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetCPACPPowerMeasurementMode(int Window, out int Mode)
        {
            int pInvokeResult = PInvoke.GetCPACPPowerMeasurementMode(this._handle, Window, out Mode);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function assigns the channel/adjacent channel power measurement to the indicated trace in the selected measurement window. The corresponding trace must be active, i.e. its state must be different from blank.
        /// 
        /// Note(s):
        /// 
        /// The measurement of the occupied bandwidth (OBW) is performed on the trace on which marker 1 is positioned. To evaluate another trace, marker 1 must be positioned to another trace with CALCulate:MARKer:TRACe.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]POWer:TRACe
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Trace">
        /// This control sets the trace.
        /// 
        /// Valid Range: 1 to 3
        /// 
        /// Default Value: 1
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetCPACPMeasurementTrace(int Window, int Trace)
        {
            int pInvokeResult = PInvoke.SetCPACPMeasurementTrace(this._handle, Window, Trace);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns to which channel/adjacent channel power measurement is the trace assigned in the selected measurement window.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]POWer:TRACe?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Trace">
        /// This control returns trace number.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetCPACPMeasurementTrace(int Window, out int Trace)
        {
            int pInvokeResult = PInvoke.GetCPACPMeasurementTrace(this._handle, Window, out Trace);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches the weigthing filter state for all the channels on on or off.
        ///  
        /// Note(s):
        /// 
        /// (1) This function is available only if Fast ACP is not active and no predefine standard is selected.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]POWer:ACHannel:FILTer:STATe:ALL
        /// 
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Weighting_Filter_State">
        /// This control activates the weighting filter for all channels.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetAllACPWeigthingFilterState(int Window, bool Weighting_Filter_State)
        {
            int pInvokeResult = PInvoke.SetAllACPWeigthingFilterState(this._handle, Window, System.Convert.ToUInt16(Weighting_Filter_State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the rolloff factor of the RRC weighting filter for all the channels.
        ///  
        /// Note(s):
        /// 
        /// (1) This function is available only if Fast ACP is not active and no predefine standard is selected.
        /// 
        /// Mode(s):
        /// Spectrum Analysis.
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]POWer:ACHannel:FILTer:ALPHa:ALL
        /// 
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Rolloff_Factor">
        /// This control sets rolloff factor value of the RRC weighting filter for all the channels.
        /// 
        /// Valid Values: 
        /// 0.0 to 1.0 
        /// 
        /// Default Value:0.26
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetAllACPWeightingFilterFactor(int Window, double Rolloff_Factor)
        {
            int pInvokeResult = PInvoke.SetAllACPWeightingFilterFactor(this._handle, Window, Rolloff_Factor);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches the weigthing filter state for the TX channel the adjacent channel and the alternate channel on on or off.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only if Fast ACP is not active and no predefine standard is selected.
        /// 
        /// Mode(s):
        /// Spectrum Analysis.
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]POWer:ACHannel:FILTer:STATe:CHANnel&lt;1...12&gt;
        /// [SENSe&lt;1|2&gt;:]POWer:ACHannel:FILTer:STATe:ACHannel
        /// [SENSe&lt;1|2&gt;:]POWer:ACHannel:FILTer:STATe:ALTernate&lt;1 to 11&gt;
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Channel">
        /// This control selects the adjacent channel limit type.
        /// 
        /// Valid Values:
        /// RSESCI_BWID_TX    (0)  - TX
        /// RSESCI_BWID_ADJ   (1)  - Adjacent
        /// RSESCI_BWID_ALT1  (2)  - Alternate 1
        /// RSESCI_BWID_ALT2  (3)  - Alternate 2
        /// RSESCI_BWID_ALT3  (4)  - Alternate 3
        /// RSESCI_BWID_ALT4  (5)  - Alternate 4
        /// RSESCI_BWID_ALT5  (6)  - Alternate 5
        /// RSESCI_BWID_ALT6  (7)  - Alternate 6
        /// RSESCI_BWID_ALT7  (8)  - Alternate 7
        /// RSESCI_BWID_ALT8  (9)  - Alternate 8
        /// RSESCI_BWID_ALT9  (10) - Alternate 9
        /// RSESCI_BWID_ALT10 (11) - Alternate 10
        /// RSESCI_BWID_ALT11 (12) - Alternate 11
        /// 
        /// Default Value: RSESCI_BWID_TX (0)
        /// 
        /// Note(s):
        /// 
        /// (1) Adjacent - defines the limit value for the lower/upper adjacent channel during adjacent channel power measurement (Adjacent Channel Power) in the selected measurement window.
        /// 
        /// (2) Alternate 1..11 - defines the limit for the alternate adjacent channel in the selected measurement window for adjacent channel power measurements.
        /// </param>
        /// <param name="Weighting_Filter_State">
        /// This control activates the weighting filter for TX channel the adjacent channel and the alternate channel.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetACPWeightingFilterState(int Window, int Channel, bool Weighting_Filter_State)
        {
            int pInvokeResult = PInvoke.SetACPWeightingFilterState(this._handle, Window, Channel, System.Convert.ToUInt16(Weighting_Filter_State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the weigthing filter state for the TX channel adjacent channel and the alternate channel on on or off.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only if Fast ACP is not active and no predefine standard is selected.
        /// 
        /// Mode(s):
        /// Spectrum Analysis.
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]POWer:ACHannel:FILTer:STATe:CHANnel&lt;1...12&gt;?
        /// [SENSe&lt;1|2&gt;:]POWer:ACHannel:FILTer:STATe:ACHannel?
        /// [SENSe&lt;1|2&gt;:]POWer:ACHannel:FILTer:STATe:ALTernate&lt;1 to 11&gt;?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Channel">
        /// This control selects the adjacent channel limit type.
        /// 
        /// Valid Values:
        /// RSESCI_BWID_TX    (0)  - TX
        /// RSESCI_BWID_ADJ   (1)  - Adjacent
        /// RSESCI_BWID_ALT1  (2)  - Alternate 1
        /// RSESCI_BWID_ALT2  (3)  - Alternate 2
        /// RSESCI_BWID_ALT3  (4)  - Alternate 3
        /// RSESCI_BWID_ALT4  (5)  - Alternate 4
        /// RSESCI_BWID_ALT5  (6)  - Alternate 5
        /// RSESCI_BWID_ALT6  (7)  - Alternate 6
        /// RSESCI_BWID_ALT7  (8)  - Alternate 7
        /// RSESCI_BWID_ALT8  (9)  - Alternate 8
        /// RSESCI_BWID_ALT9  (10) - Alternate 9
        /// RSESCI_BWID_ALT10 (11) - Alternate 10
        /// RSESCI_BWID_ALT11 (12) - Alternate 11
        /// 
        /// Default Value: RSESCI_BWID_TX (0)
        /// 
        /// Note(s):
        /// 
        /// (1) Adjacent - defines the limit value for the lower/upper adjacent channel during adjacent channel power measurement (Adjacent Channel Power) in the selected measurement window.
        /// 
        /// (2) Alternate 1..11 - defines the limit for the alternate adjacent channel in the selected measurement window for adjacent channel power measurements.
        /// </param>
        /// <param name="Weighting_Filter_State">
        /// This control returns the state of the weighting filter for the TX channel adjacent channel and the alternate channel.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetACPWeightingFilterState(int Window, int Channel, out bool Weighting_Filter_State)
        {
            ushort Weighting_Filter_StateAsUShort;
            int pInvokeResult = PInvoke.GetACPWeightingFilterState(this._handle, Window, Channel, out Weighting_Filter_StateAsUShort);
            Weighting_Filter_State = System.Convert.ToBoolean(Weighting_Filter_StateAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the rolloff factor of the RRC weighting filter for the TX channel adjacent channel and the alternate channel.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only if Fast ACP is not active and no predefine standard is selected.
        /// 
        /// Mode(s):
        /// Spectrum Analysis.
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]POWer:ACHannel:FILTer:ALPHa:CHANnel&lt;1...12&gt;
        /// [SENSe&lt;1|2&gt;:]POWer:ACHannel:FILTer:ALPHa:ACHannel
        /// [SENSe&lt;1|2&gt;:]POWer:ACHannel:FILTer:ALPHa:ALTernate&lt;1...11&gt;w
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Channel">
        /// This control selects the adjacent channel limit type.
        /// 
        /// Valid Values:
        /// RSESCI_BWID_TX    (0)  - TX
        /// RSESCI_BWID_ADJ   (1)  - Adjacent
        /// RSESCI_BWID_ALT1  (2)  - Alternate 1
        /// RSESCI_BWID_ALT2  (3)  - Alternate 2
        /// RSESCI_BWID_ALT3  (4)  - Alternate 3
        /// RSESCI_BWID_ALT4  (5)  - Alternate 4
        /// RSESCI_BWID_ALT5  (6)  - Alternate 5
        /// RSESCI_BWID_ALT6  (7)  - Alternate 6
        /// RSESCI_BWID_ALT7  (8)  - Alternate 7
        /// RSESCI_BWID_ALT8  (9)  - Alternate 8
        /// RSESCI_BWID_ALT9  (10) - Alternate 9
        /// RSESCI_BWID_ALT10 (11) - Alternate 10
        /// RSESCI_BWID_ALT11 (12) - Alternate 11
        /// 
        /// Default Value: RSESCI_BWID_TX (0)
        /// 
        /// Note(s):
        /// 
        /// (1) Adjacent - defines the limit value for the lower/upper adjacent channel during adjacent channel power measurement (Adjacent Channel Power) in the selected measurement window.
        /// 
        /// (2) Alternate 1..11 - defines the limit for the alternate adjacent channel in the selected measurement window for adjacent channel power measurements.
        /// </param>
        /// <param name="Rolloff_Factor">
        /// This control sets rolloff factor value of the RRC weighting filter for TX channel the adjacent channel and the alternate channel.
        /// 
        /// Valid Values: 
        /// 0.0 to 1.0 
        /// 
        /// Default Value: 0.22
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetACPWeightingFilterFactor(int Window, int Channel, double Rolloff_Factor)
        {
            int pInvokeResult = PInvoke.SetACPWeightingFilterFactor(this._handle, Window, Channel, Rolloff_Factor);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the rolloff factor of the RRC weighting filter for the TX channel adjacent channel and the alternate channel.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only if Fast ACP is not active and no predefine standard is selected.
        /// 
        /// Mode(s):
        /// Spectrum Analysis.
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]POWer:ACHannel:FILTer:ALPHa:CHANnel&lt;1...12&gt;?
        /// [SENSe&lt;1|2&gt;:]POWer:ACHannel:FILTer:ALPHa:ACHannel?
        /// [SENSe&lt;1|2&gt;:]POWer:ACHannel:FILTer:ALPHa:ALTernate&lt;1...11&gt;?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Channel">
        /// This control selects the adjacent channel limit type.
        /// 
        /// Valid Values:
        /// RSESCI_BWID_TX    (0)  - TX
        /// RSESCI_BWID_ADJ   (1)  - Adjacent
        /// RSESCI_BWID_ALT1  (2)  - Alternate 1
        /// RSESCI_BWID_ALT2  (3)  - Alternate 2
        /// RSESCI_BWID_ALT3  (4)  - Alternate 3
        /// RSESCI_BWID_ALT4  (5)  - Alternate 4
        /// RSESCI_BWID_ALT5  (6)  - Alternate 5
        /// RSESCI_BWID_ALT6  (7)  - Alternate 6
        /// RSESCI_BWID_ALT7  (8)  - Alternate 7
        /// RSESCI_BWID_ALT8  (9)  - Alternate 8
        /// RSESCI_BWID_ALT9  (10) - Alternate 9
        /// RSESCI_BWID_ALT10 (11) - Alternate 10
        /// RSESCI_BWID_ALT11 (12) - Alternate 11
        /// 
        /// Default Value: RSESCI_BWID_TX (0)
        /// 
        /// Note(s):
        /// 
        /// (1) Adjacent - defines the limit value for the lower/upper adjacent channel during adjacent channel power measurement (Adjacent Channel Power) in the selected measurement window.
        /// 
        /// (2) Alternate 1..11 - defines the limit for the alternate adjacent channel in the selected measurement window for adjacent channel power measurements.
        /// </param>
        /// <param name="Rolloff_Factor">
        /// This control returns rolloff factor value of the RRC weighting filter for TX channel the adjacent channel and the alternate channel.
        /// 
        /// Valid Values: 
        /// 0.0 to 1.0 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetACPWeightingFilterFactor(int Window, int Channel, out double Rolloff_Factor)
        {
            int pInvokeResult = PInvoke.GetACPWeightingFilterFactor(this._handle, Window, Channel, out Rolloff_Factor);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches on and off the limit check for adjacent channel power measurements in the selected measurement window. The limit line number is irrelevant for this function.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:ACPower[:STATe] ON | OFF
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Limit_Checking">
        /// This control switches on and off the limit check for adjacent channel power measurements in the selected measurement window.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetACPLimitChecking(int Window, bool Limit_Checking)
        {
            int pInvokeResult = PInvoke.SetACPLimitChecking(this._handle, Window, System.Convert.ToUInt16(Limit_Checking));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the state of the limit check for adjacent channel power measurements in the selected measurement window.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:ACPower[:STATe]?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Limit_Checking">
        /// This control returns the state of the limit check for adjacent channel power measurements in the selected measurement window.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetACPLimitChecking(int Window, out bool Limit_Checking)
        {
            ushort Limit_CheckingAsUShort;
            int pInvokeResult = PInvoke.GetACPLimitChecking(this._handle, Window, out Limit_CheckingAsUShort);
            Limit_Checking = System.Convert.ToBoolean(Limit_CheckingAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the state and value of selected adjacent channel measurement limit.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:ACPower:ACHannel[:RELative]
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:ACPower:ACHannel[:RELative]:STATe ON | OFF
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:ACPower:ACHannel:ABSolute
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:ACPower:ACHannel:ABSolute:STATe ON | OFF
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:ACPower:ALTernate&lt;1...11&gt;[:RELative]
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:ACPower:ALTernate&lt;1...11&gt;[:RELative]:STATe ON | OFF
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:ACPower:ALTernate&lt;1...11&gt;:ABSolute
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:ACPower:ALTernate&lt;1...11&gt;:ABSolute:STATe ON | OFF
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Limit_Type">
        /// This control selects the adjacent channel limit type.
        /// 
        /// Valid Values:
        /// RSESCI_BWID_ADJ   (1)  - Adjacent
        /// RSESCI_BWID_ALT1  (2)  - Alternate 1
        /// RSESCI_BWID_ALT2  (3)  - Alternate 2
        /// RSESCI_BWID_ALT3  (4)  - Alternate 3
        /// RSESCI_BWID_ALT4  (5)  - Alternate 4
        /// RSESCI_BWID_ALT5  (6)  - Alternate 5
        /// RSESCI_BWID_ALT6  (7)  - Alternate 6
        /// RSESCI_BWID_ALT7  (8)  - Alternate 7
        /// RSESCI_BWID_ALT8  (9)  - Alternate 8
        /// RSESCI_BWID_ALT9  (10) - Alternate 9
        /// RSESCI_BWID_ALT10 (11) - Alternate 10
        /// RSESCI_BWID_ALT11 (12) - Alternate 11
        /// 
        /// Default Value: RSESCI_BWID_ADJ (1)
        /// 
        /// Note(s):
        /// 
        /// (1) Adjacent - defines the limit value for the lower/upper adjacent channel during adjacent channel power measurement (Adjacent Channel Power) in the selected measurement window.
        /// 
        /// (2) Alternate 1..11 - defines the limit for the alternate adjacent channel in the selected measurement window for adjacent channel power measurements.
        /// </param>
        /// <param name="Mode">
        /// This control toggles between absolute and relative adjacent channel limit.
        /// 
        /// Valid Values:
        /// RSESCI_ABS (0) - Absolute
        /// RSESCI_REL (1) - Relative
        /// 
        /// Default Value: RSESCI_ABS (0)
        /// </param>
        /// <param name="State">
        /// This control activates or deactivates selected limit check.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <param name="Limit_Value">
        /// This control defines the limit value for select limit.
        /// 
        /// Valid Range:
        /// For Absolute mode: -200.0 dBm to 200.0 dBm
        /// For Relative mode: 0.0 dBm to 100.0 dB
        /// 
        /// Default Value: 0.0
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetACPLimits(int Window, int Limit_Type, int Mode, bool State, double Limit_Value)
        {
            int pInvokeResult = PInvoke.SetACPLimits(this._handle, Window, Limit_Type, Mode, System.Convert.ToUInt16(State), Limit_Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the limit check state and limit value for selected adjacent channel limit.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:ACPower:ACHannel[:RELative]?
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:ACPower:ACHannel[:RELative]:STATe?
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:ACPower:ACHannel:ABSolute?
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:ACPower:ACHannel:ABSolute:STATe?
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:ACPower:ALTernate&lt;1...11&gt;[:RELative]?
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:ACPower:ALTernate&lt;1...11&gt;[:RELative]:STATe?
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:ACPower:ALTernate&lt;1...11&gt;:ABSolute?
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:ACPower:ALTernate&lt;1...11&gt;:ABSolute:STATe?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Mode">
        /// This control toggles between absolute and relative adjacent channel limit.
        /// 
        /// Valid Values:
        /// RSESCI_ABS (0) - Absolute
        /// RSESCI_REL (1) - Relative
        /// 
        /// Default Value: RSESCI_ABS (0)
        /// </param>
        /// <param name="Limit_Type">
        /// This control selects the adjacent channel limit type.
        /// 
        /// Valid Values:
        /// RSESCI_BWID_ADJ   (1)  - Adjacent
        /// RSESCI_BWID_ALT1  (2)  - Alternate 1
        /// RSESCI_BWID_ALT2  (3)  - Alternate 2
        /// RSESCI_BWID_ALT3  (4)  - Alternate 3
        /// RSESCI_BWID_ALT4  (5)  - Alternate 4
        /// RSESCI_BWID_ALT5  (6)  - Alternate 5
        /// RSESCI_BWID_ALT6  (7)  - Alternate 6
        /// RSESCI_BWID_ALT7  (8)  - Alternate 7
        /// RSESCI_BWID_ALT8  (9)  - Alternate 8
        /// RSESCI_BWID_ALT9  (10) - Alternate 9
        /// RSESCI_BWID_ALT10 (11) - Alternate 10
        /// RSESCI_BWID_ALT11 (12) - Alternate 11
        /// 
        /// Default Value: RSESCI_BWID_ADJ (1)
        /// 
        /// Note(s):
        /// 
        /// (1) Adjacent - defines the limit value for the lower/upper adjacent channel during adjacent channel power measurement (Adjacent Channel Power) in the selected measurement window.
        /// 
        /// (2) Alternate 1..11 - defines the limit for the alternate adjacent channel in the selected measurement window for adjacent channel power measurements.
        /// </param>
        /// <param name="State">
        /// This control returns the state of selected limit check.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <param name="Limit_Value">
        /// This control returns the limit value for select limit.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetACPLimits(int Window, int Mode, int Limit_Type, out bool State, out double Limit_Value)
        {
            ushort StateAsUShort;
            int pInvokeResult = PInvoke.GetACPLimits(this._handle, Window, Mode, Limit_Type, out StateAsUShort, out Limit_Value);
            State = System.Convert.ToBoolean(StateAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the result of the limit check for selected limit.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:ACPower:ACHannel:RESult?
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:ACPower:ALTernate&lt;1...11&gt;:RESult?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Limit_Type">
        /// This control selects the adjacent channel limit type.
        /// 
        /// Valid Values:
        /// RSESCI_BWID_ADJ   (1) - Adjacent
        /// RSESCI_BWID_ALT1  (2) - Alternate 1
        /// RSESCI_BWID_ALT2  (3) - Alternate 2
        /// RSESCI_BWID_ALT3  (4) - Alternate 3
        /// RSESCI_BWID_ALT4  (5) - Alternate 4
        /// RSESCI_BWID_ALT5  (6) - Alternate 5
        /// RSESCI_BWID_ALT6  (7) - Alternate 6
        /// RSESCI_BWID_ALT7  (8) - Alternate 7
        /// RSESCI_BWID_ALT8  (9) - Alternate 8
        /// RSESCI_BWID_ALT9  (10) - Alternate 9
        /// RSESCI_BWID_ALT10 (11) - Alternate 10
        /// RSESCI_BWID_ALT11 (12) - Alternate 11
        /// 
        /// Default Value: RSESCI_BWID_ADJ (1)
        /// 
        /// Note(s):
        /// 
        /// (1) Adjacent - defines the limit value for the lower/upper adjacent channel during adjacent channel power measurement (Adjacent Channel Power) in the selected measurement window.
        /// 
        /// (2) Alternate 1..11 - defines the limit for the alternate adjacent channel in the selected measurement window for adjacent channel power measurements.
        /// </param>
        /// <param name="Result">
        /// This control returns the state of selected limit check.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Failed
        /// VI_TRUE  (1) - Passed
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetACPLimitCheckingResult(int Window, int Limit_Type, out bool Result)
        {
            ushort ResultAsUShort;
            int pInvokeResult = PInvoke.GetACPLimitCheckingResult(this._handle, Window, Limit_Type, out ResultAsUShort);
            Result = System.Convert.ToBoolean(ResultAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the percentage of the power with respect to the total power. This value is the basis for the occupied bandwidth measurement.
        /// 
        /// Mode(s):
        /// Spectrum Analysis (frequency domain only)
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]POWer:BANDwidth|BWIDth
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Percentage">
        /// This control defines the percentage of the power with respect to the total power. This value is the basis for the occupied bandwidth measurement.
        /// 
        /// Valid Range: 10.0 to 99.9 %
        /// 
        /// Default Value: 99 %
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetPowerBandwidthPercentage(int Window, double Percentage)
        {
            int pInvokeResult = PInvoke.SetPowerBandwidthPercentage(this._handle, Window, Percentage);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the percentage of the power with respect to the total power. This value is the basis for the occupied bandwidth measurement.
        /// 
        /// Mode(s):
        /// Spectrum Analysis (frequency domain only)
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]POWer:BANDwidth|BWIDth?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Percentage">
        /// This control returns the percentage of the power with respect to the total power. This value is the basis for the occupied bandwidth measurement.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetPowerBandwidthPercentage(int Window, out double Percentage)
        {
            int pInvokeResult = PInvoke.GetPowerBandwidthPercentage(this._handle, Window, out Percentage);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the channel bandwidth of the radio communication system. The bandwidths of adjacent channels are not influenced by this modification (in contrast to the FSE family).
        /// 
        /// Mode(s):
        /// Spectrum Analysis (frequency domain only)
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]POWer:ACHannel:BANDwidth|BWIDth[:CHANnel]
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Channel_Bandwidth">
        /// This control sets the channel bandwidth of the radio communication system.
        /// 
        /// Valid Range: 100.0 to 1.0e9 Hz
        /// 
        /// Default Value: 14.0e3 Hz
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetChannelBandwidth(int Window, double Channel_Bandwidth)
        {
            int pInvokeResult = PInvoke.SetChannelBandwidth(this._handle, Window, Channel_Bandwidth);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the channel bandwidth of the radio communication system.
        /// 
        /// Mode(s):
        /// Spectrum Analysis (frequency domain only)
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]POWer:ACHannel:BANDwidth|BWIDth[:CHANnel]?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Channel_Bandwidth">
        /// This control returns the channel bandwidth of the radio communication system.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetChannelBandwidth(int Window, out double Channel_Bandwidth)
        {
            int pInvokeResult = PInvoke.GetChannelBandwidth(this._handle, Window, out Channel_Bandwidth);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the statistical measurement.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate:STATistics:APD[:STATe] ON | OFF
        /// CALCulate:STATistics:CCDF[:STATe] ON | OFF
        /// </summary>
        /// <param name="Measurement">
        /// This control selects the measurement type.
        /// 
        /// Valid Values:
        /// RSESCI_STAT_AOFF (0) - All Off
        /// RSESCI_STAT_APD  (1) - APD
        /// RSESCI_STAT_CCDF (2) - CCDF
        /// 
        /// Default Value: RSESCI_STAT_AOFF (0)
        /// 
        /// Note(s):
        /// 
        /// (1) All Off - switches off both APD and CCDF statistical measurement.
        /// 
        /// (2) APD - switches on the measurement of amplitude distribution (APD). On activating this function, the CCDF measurement is switched off.
        /// 
        /// (3) CCDF - switches on the measurement of the complementary cumulative distribution function (CCDF). On activating this function, the APD measurement is switched off.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetSignalAmplitudeStatistics(int Measurement)
        {
            int pInvokeResult = PInvoke.SetSignalAmplitudeStatistics(this._handle, Measurement);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries selected statistical measurement.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate:STATistics:APD[:STATe]?
        /// CALCulate:STATistics:CCDF[:STATe]?
        /// </summary>
        /// <param name="Measurement">
        /// This control returns selected measurement type.
        /// 
        /// Valid Values:
        /// RSESCI_STAT_AOFF (0) - All Off
        /// RSESCI_STAT_APD  (1) - APD
        /// RSESCI_STAT_CCDF (2) - CCDF
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetSignalAmplitudeStatistics(out int Measurement)
        {
            int pInvokeResult = PInvoke.GetSignalAmplitudeStatistics(this._handle, out Measurement);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function positions the selected marker in the selected window to the given probability. If marker 2, 3 or 4 is selected and used as a delta marker, it is switched to marker mode.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Note(s):
        /// 
        /// (1) The function is only available with the CCDF measurement switched on.
        /// 
        /// (2) The associated level value can be determined with the rsesci_GetMarkerPosition() function.
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:Y:PERCent
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Marker">
        /// This control sets the marker.
        /// 
        /// Valid Range: 1 to 4
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Position">
        /// This control defines the probability at which the selected marker in the selected window will be positioned.
        /// 
        /// Valid Range: 0.0 to 100.0 %
        /// 
        /// Default Value: 0.0
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetCCDFPercentMarker(int Window, int Marker, double Position)
        {
            int pInvokeResult = PInvoke.SetCCDFPercentMarker(this._handle, Window, Marker, Position);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the position of selected marker in the selected window.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:Y:PERCent?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Marker">
        /// This control sets the marker.
        /// 
        /// Valid Range: 1 to 4
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Position">
        /// This control returns the probability at which the selected marker in the selected window is positioned.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetCCDFPercentMarker(int Window, int Marker, out double Position)
        {
            int pInvokeResult = PInvoke.GetCCDFPercentMarker(this._handle, Window, Marker, out Position);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the number of measurement points to be acquired for the statistical measurement functions.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate:STATistics:NSAMples
        /// </summary>
        /// <param name="Number_Of_Samples">
        /// This control sets the number of measurement points to be acquired for the statistical measurement functions.
        /// 
        /// Valid Range: 100.0 to 1.0E9
        /// 
        /// Default Value: 100000.0
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetStatisticsNumberSamples(double Number_Of_Samples)
        {
            int pInvokeResult = PInvoke.SetStatisticsNumberSamples(this._handle, Number_Of_Samples);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the number of measurement points to be acquired for the statistical measurement functions.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate:STATistics:NSAMples?
        /// </summary>
        /// <param name="Number_Of_Samples">
        /// This control returns the number of measurement points to be acquired for the statistical measurement functions.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetStatisticsNumberSamples(out double Number_Of_Samples)
        {
            int pInvokeResult = PInvoke.GetStatisticsNumberSamples(this._handle, out Number_Of_Samples);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function optimizes the level setting of the instrument depending on the measured peak power, in order to obtain maximum instrument sensitivity. To obtain maximum resolution, the level range is set as a function of the measured spacing between peak power and the minimum power for the APD measurement and of the spacing between peak power and mean power for the CCDF measurement. In addition, the probability scale for the number of test points is adapted.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate:STATistics:SCALe:AUTO ONCE
        /// </summary>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int StatisticsAdjustSettings()
        {
            int pInvokeResult = PInvoke.StatisticsAdjustSettings(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function resets the scaling of the X and Y axes in a statistical measurement. The following values are set:
        /// 
        /// X axis ref level: -20 dBm
        /// X axis range APD: 100 dB
        /// X axis range CCDF: 20 dB
        /// Y axis upper limit: 1.0
        /// Y axis lower limit: 1e-6
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate:STATistics:PRESet
        /// </summary>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int StatisticsDefaultSettings()
        {
            int pInvokeResult = PInvoke.StatisticsDefaultSettings(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function reads out the results of statistical measurements of a recorded trace.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate:STATistics:RESult&lt;1...3&gt;? MEAN | PEAK | CFACtor | ALL
        /// </summary>
        /// <param name="Trace">
        /// This control sets the trace.
        /// 
        /// Valid Range: 1 to 3
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Result_Type">
        /// This control define the type of result of statistical measurements of a recorded trace.
        /// 
        /// Valid Values:
        /// RSESCI_STAT_RESULT_ALL  (0) - All
        /// RSESCI_STAT_RESULT_MEAN (1) - Mean
        /// RSESCI_STAT_RESULT_PEAK (2) - Peak
        /// RSESCI_STAT_RESULT_CFAC (3) - CREST Factor
        /// 
        /// Default Value: RSESCI_STAT_RESULT_ALL (0)
        /// 
        /// Note(s):
        /// 
        /// (1) Mean:
        /// Average (=RMS) power in dBm measured during the measurement time.
        /// 
        /// (2) Peak:
        /// Peak power in dBm measured during the measurement time.
        /// 
        /// (3) CREST Factor:
        /// Determined CREST factor (= ratio of peak power to average power) in dB.
        /// 
        /// (4) All:
        /// Results of all three measurements mentioned before.
        /// 
        /// </param>
        /// <param name="Result">
        /// This control returns the array of results. In case of selected result type 'All', it returns three values, otherwise only one.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetStatisticalMeasurementsResults(int Trace, int Result_Type, double[] Result)
        {
            int pInvokeResult = PInvoke.GetStatisticalMeasurementsResults(this._handle, Trace, Result_Type, Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function reads out the level values for the probabilities 0.1%, 1% and 10%.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate:STATistics:CCDF:X&lt;1...3&gt;? P0_1 | P1 | P10
        /// </summary>
        /// <param name="Trace">
        /// Selects the trace.
        /// 
        /// Valid Values:
        /// 1 to 3
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Probability">
        /// Selects the probability.
        /// 
        /// Valid Values:
        /// RSESCI_STAT_PROB_0_1  (0) - 0.1 %
        /// RSESCI_STAT_PROB_1    (1) - 1 %
        /// RSESCI_STAT_PROB_10   (2) - 10 %
        /// 
        /// Default Value: RSESCI_STAT_PROB_0_1  (0)
        /// </param>
        /// <param name="Result">
        /// Returns the selected level value.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation.  The status code  either indicates success or describes an error or warning condition.  You examine the status code from each call to an instrument driver function to determine if an error occurred.
        /// 
        /// To obtain a text description of the status code, call the rsspecan_error_message function.  To obtain additional information about the error condition, call the rsspecan_GetError function.  To clear the error information from the driver, call the rsspecan_ClearError function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// WARNINGS:
        /// 3FFA2001  The instrument was in an uncalibrated state
        ///           when the measurement was taken.
        /// 3FFA2002  The measurement taken was over the instrument??s
        ///           range.
        ///   
        /// ERRORS:
        /// BFFA2001  The Active Marker is not enabled.
        /// BFFA2002  The Active Marker is not a delta marker.
        /// BFFA2003  The maximum waiting time for this operation was
        ///           exceeded.
        ///           
        /// This instrument driver also returns errors and warnings defined by other sources.  The following table defines the ranges of additional status codes that this driver can return.  The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFA0000 to 3FFA1FFF     IVI    Warnings
        /// 3FFF0000 to 3FFFFFFF     VISA   Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP Driver Warnings
        ///           
        /// BFFA0000 to BFFA1FFF     IVI    Errors
        /// BFFF0000 to BFFFFFFF     VISA   Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP Driver Errors
        /// 
        /// </returns>
        public int QuerySignalStatisticCCDFLevel(int Trace, int Probability, out double Result)
        {
            int pInvokeResult = PInvoke.QuerySignalStatisticCCDFLevel(this._handle, Trace, Probability, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the reference level for the X axis of the measurement diagram. With the reference level offset &lt;&gt; 0 the indicated value range of the reference level is modified by the
        /// offset. The unit depends on the setting performed with rsesci_SetLevelAxisUnit.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate:STATistics:SCALe:X:RLEVel
        /// </summary>
        /// <param name="Reference_Level">
        /// This control sets the reference level for the X axis of the measurement diagram.
        /// 
        /// Valid Range: -130 to 30dBm
        /// 
        /// Default Value: -20.0
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetStatisticsXAxisReferenceLevel(double Reference_Level)
        {
            int pInvokeResult = PInvoke.SetStatisticsXAxisReferenceLevel(this._handle, Reference_Level);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the reference level for the X axis of the measurement diagram.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate:STATistics:SCALe:X:RLEVel?
        /// </summary>
        /// <param name="Reference_Level">
        /// This control returns the reference level for the X axis of the measurement diagram.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetStatisticsXAxisReferenceLevel(out double Reference_Level)
        {
            int pInvokeResult = PInvoke.GetStatisticsXAxisReferenceLevel(this._handle, out Reference_Level);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the level range for the X axis of the measurement diagram.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate:STATistics:SCALe:X:RANGe
        /// </summary>
        /// <param name="Range">
        /// This control defines the level range for the X axis of the measurement diagram.
        /// 
        /// Valid Range: 10.0 to 200.0 dB
        /// 
        /// Default Value: 100.0 dB
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetStatisticsXAxisRange(double Range)
        {
            int pInvokeResult = PInvoke.SetStatisticsXAxisRange(this._handle, Range);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the level range for the X axis of the measurement diagram.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate:STATistics:SCALe:X:RANGe?
        /// </summary>
        /// <param name="Range">
        /// This control returns the level range for the X axis of the measurement diagram.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetStatisticsXAxisRange(out double Range)
        {
            int pInvokeResult = PInvoke.GetStatisticsXAxisRange(this._handle, out Range);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the lower limit for th Y axis of the diagram in statistical measurements.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate:STATistics:SCALe:Y:LOWer
        /// </summary>
        /// <param name="Min_Value">
        /// This control defines the lower limit for th Y axis of the diagram in statistical measurements. Since probabilities are specified on the Y axis, the entered numerical values are dimensionless.
        /// 
        /// Valid Range: 1e-9 to 0.1
        /// 
        /// Default Value: 1e-6
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetStatisticsYAxisMinValue(double Min_Value)
        {
            int pInvokeResult = PInvoke.SetStatisticsYAxisMinValue(this._handle, Min_Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the lower limit for th Y axis of the diagram in statistical measurements.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate:STATistics:SCALe:Y:LOWer?
        /// </summary>
        /// <param name="Min_Value">
        /// This control returns the lower limit for th Y axis of the diagram in statistical measurements. Since probabilities are specified on the Y axis, the entered numerical values are dimensionless.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetStatisticsYAxisMinValue(out double Min_Value)
        {
            int pInvokeResult = PInvoke.GetStatisticsYAxisMinValue(this._handle, out Min_Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the upper limit for th Y axis of the diagram in statistical measurements.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate:STATistics:SCALe:Y:UPPer
        /// </summary>
        /// <param name="Max_Value">
        /// This control defines the upper limit for th Y axis of the diagram in statistical measurements. Since probabilities are specified on the Y axis, the entered numerical values are dimensionless.
        /// 
        /// Valid Range: 1e-8 to 1.0
        /// 
        /// Default Value: 1.0
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetStatisticsYAxisMaxValue(double Max_Value)
        {
            int pInvokeResult = PInvoke.SetStatisticsYAxisMaxValue(this._handle, Max_Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the upper limit for th Y axis of the diagram in statistical measurements.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate:STATistics:SCALe:Y:UPPer?
        /// </summary>
        /// <param name="Max_Value">
        /// This control returns the upper limit for th Y axis of the diagram in statistical measurements. Since probabilities are specified on the Y axis, the entered numerical values are dimensionless.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetStatisticsYAxisMaxValue(out double Max_Value)
        {
            int pInvokeResult = PInvoke.GetStatisticsYAxisMaxValue(this._handle, out Max_Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function toggles the scaling of Y axis between percentage and absolute.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate:STATistics:SCALe:Y:UNIT PCT | ABS
        /// </summary>
        /// <param name="Units">
        /// This control toggles the scaling of Y axis between percentage and absolute.
        /// 
        /// Valid Range: 
        /// RSESCI_SSTAT_UNIT_ABS (0) - Absolute Value
        /// RSESCI_SSTAT_UNIT_PCT (1) - Percentage
        /// 
        /// Default Value: RSESCI_SSTAT_UNIT_ABS (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetStatisticsYAxisUnit(int Units)
        {
            int pInvokeResult = PInvoke.SetStatisticsYAxisUnit(this._handle, Units);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the scaling unit of Y axis.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate:STATistics:SCALe:Y:UNIT?
        /// </summary>
        /// <param name="Units">
        /// This control returns the scaling unit of Y axis.
        /// 
        /// Valid Range: 
        /// RSESCI_SSTAT_UNIT_ABS (0) - Absolute Value
        /// RSESCI_SSTAT_UNIT_PCT (1) - Percentage
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetStatisticsYAxisUnit(out int Units)
        {
            int pInvokeResult = PInvoke.GetStatisticsYAxisUnit(this._handle, out Units);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches on the measurement of the AM modulation depth. An AM-modulated carrier is required on the screen for correct operation. If necessary, marker 1 is previously activated and set to the largest signal available. The level value of marker 1 is regarded as the carrier level. On activating the function, marker 2 and marker 3 are automatically set as delta markers symetrically to the carrier to the adjacent maxima of the trace. If the position of delta marker 2 is changed, delta marker 3 is moved symetrically with respect to the reference marker (marker 1). If the position of delta marker 3 is changed, fine adjustment can be performed independently of delta marker 2. The R&amp;S ESCI calculates the power at the marker positions from the measured levels. The AM modulation depth is calculated from the ratio of power values at the reference marker and the delta markers. If the two AM sidebands differ in power, the average value of the two power values is used for calculating the AM modulation depth.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:MDEPth[:STATe] ON | OFF
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Modulation_Depth_Measurement">
        /// This control switches on the measurement of the AM modulation depth.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetModulationDepthMeasurement(int Window, bool Modulation_Depth_Measurement)
        {
            int pInvokeResult = PInvoke.SetModulationDepthMeasurement(this._handle, Window, System.Convert.ToUInt16(Modulation_Depth_Measurement));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the state of the measurement of the AM modulation depth.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:MDEPth[:STATe]?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Modulation_Depth_Measurement">
        /// This control returns the state of the measurement of the AM modulation depth.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetModulationDepthMeasurement(int Window, out bool Modulation_Depth_Measurement)
        {
            ushort Modulation_Depth_MeasurementAsUShort;
            int pInvokeResult = PInvoke.GetModulationDepthMeasurement(this._handle, Window, out Modulation_Depth_MeasurementAsUShort);
            Modulation_Depth_Measurement = System.Convert.ToBoolean(Modulation_Depth_MeasurementAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the AM modulation depth in the indicated measurement window. A complete sweep with synchronization to sweep end must be performed between switching on the function and querying the measured value to obtain a valid query result. This is only possible in single sweep mode.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:MDEPth:RESult?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Result">
        /// This control returns the AM modulation depth in the indicated measurement window.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetModulationDepthMeasurementValue(int Window, out double Result)
        {
            int pInvokeResult = PInvoke.GetModulationDepthMeasurementValue(this._handle, Window, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function controls the positioning of the intercept markers 3 and 4 for TOI measurement, based on the two carrier signals.
        ///  
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:TOI:MARKer
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="TOI_Marker">
        /// This control set the positioning of the intercept markers 3 and 4 for TOI measurement, based on the two carrier signals.
        /// 
        /// Valid Values:
        /// RSESCI_VAL_CALC (0) - Calculate
        /// RSESCI_VAL_SEAR (1) - Search
        /// 
        /// Default Value: RSESCI_VAL_CALC (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetThirdOrderInterceptFunction(int Window, int TOI_Marker)
        {
            int pInvokeResult = PInvoke.SetThirdOrderInterceptFunction(this._handle, Window, TOI_Marker);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the positioning of the intercept markers 3 and 4 for TOI measurement, based on the two carrier signals.
        ///  
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1 to 4&gt;:FUNCtion:TOI:MARKer?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="TOI_Marker">
        /// This control returns the positioning of the intercept markers 3 and 4 for TOI measurement, based on the two carrier signals.
        /// 
        /// Valid Values:
        /// RSESCI_VAL_CALC (0) - Calculate
        /// RSESCI_VAL_SEAR (1) - Search
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetThirdOrderInterceptFunction(int Window, out int TOI_Marker)
        {
            int pInvokeResult = PInvoke.GetThirdOrderInterceptFunction(this._handle, Window, out TOI_Marker);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function initiates the measurement of the third-order intercept point. A two-tone signal with equal carrier levels is expected at the RF input of the instrument. Marker 1 and marker 2 (both normal markers) are set to the maximum of the two signals. Delta marker 3 and delta marker 4 are positioned to the intermodulation products. The third-order intercept is calculated from the level spacing between the normal markers and the delta markers.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:TOI[:STATe] ON | OFF
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="State">
        /// This control switches on or off the third-order intercept point.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetThirdOrderInterceptMeasurement(int Window, bool State)
        {
            int pInvokeResult = PInvoke.SetThirdOrderInterceptMeasurement(this._handle, Window, System.Convert.ToUInt16(State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the state of the third-order intercept point measurement.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:TOI[:STATe]?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="State">
        /// This control returns the state of the third-order intercept point measurement.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetThirdOrderInterceptMeasurement(int Window, out bool State)
        {
            ushort StateAsUShort;
            int pInvokeResult = PInvoke.GetThirdOrderInterceptMeasurement(this._handle, Window, out StateAsUShort);
            State = System.Convert.ToBoolean(StateAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the third-order intercept point measurement in the indicated measurement window.
        /// A complete sweep with synchronization to sweep end must be performed between switching on the function and querying the measured value to obtain a valid query result. This is only possible in single sweep mode.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:TOI:RESult?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="TOI_Result">
        /// This control returns the third-order intercept point measurement in the indicated measurement window.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetThirdOrderInterceptMeasurementValue(int Window, out double TOI_Result)
        {
            int pInvokeResult = PInvoke.GetThirdOrderInterceptMeasurementValue(this._handle, Window, out TOI_Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the constant settings for the list during multiple power measurement.
        /// 
        /// Parameters &lt;Peak Detector&gt;, &lt;RMS Detector&gt; and &lt;Average Detector&gt; define, which measurements are to be performed at the same time at the frequency point. Correspondingly, one, two or three results per frequency point are returned for the SENS:LIST:POW? command.
        /// 
        /// If all three detectors are set to OFF, this function generates an execution error.
        /// 
        /// Mode(s):
        /// Spectrum Analysis (frequency domain only)
        /// Spectrum Analysis (time domain only (zero span))
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]LIST:POWer:SET &lt;PEAK meas&gt;,&lt;RMS meas&gt;,&lt;AVG meas&gt;, &lt;trigger mode&gt;,&lt;trigger slope&gt;,&lt;trigger offset&gt;,&lt;gate length&gt;
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Peak_Detector">
        /// This control activates/deactivates the measurement of the peak power (peak detector).
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_TRUE (1)
        /// 
        /// </param>
        /// <param name="RMS_Detector">
        /// This control activates/deactivates the measurement of the RMS power (RMS detector).
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// </param>
        /// <param name="Average_Detector">
        /// This control activates/deactivates the measurement of the average power (average detector).
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <param name="Trigger_Mode">
        /// Selection of the trigger source used for the list measurement.
        /// 
        /// Valid Values:
        /// RSESCI_TRIG_IMM (0) - Immediate
        /// RSESCI_TRIG_EXT (1) - External
        /// RSESCI_TRIG_VID (2) - Video
        /// RSESCI_TRIG_IFP (3) - IF Power
        /// 
        /// Default Value: RSESCI_TRIG_IMM (0)
        /// 
        /// Note(s):
        /// 
        /// (1) Immediate - automatic triggering the next measurement at the end of the previous one. The value 'Immediate' corresponds to the FREE RUN setting.
        /// 
        /// (2) External - the next measurement is triggered by the signal at the external trigger input.
        /// 
        /// (3) Video - the next measurement is triggered by the detection of a signal at the video filter output (not available for FM DEMOD mode).
        /// 
        /// (4) IF Power - the next measurement is triggered by the detection of a signal at the instrument IF (10 MHz bandwidth).
        /// 
        /// </param>
        /// <param name="Trigger_Slope">
        /// This control determines used trigger slope.
        /// 
        /// Valid Values:
        /// RSESCI_POS (0) - Positive
        /// RSESCI_NEG (1) - Negative
        /// 
        /// Default Value: RSESCI_POS (0)
        /// 
        /// </param>
        /// <param name="Trigger_Offset">
        /// This control defines offset between the detection of the trigger signal and the start of the measurement at the next frequency point.
        /// 
        /// Valid Range: 0.0 s, 125.0e-9 s to 100.0 s
        /// 
        /// Default Value: 0.0 s
        /// 
        /// </param>
        /// <param name="Gate_Length">
        /// This control defines gate length with Gated Sweep.
        /// 
        /// Valid Range: 0.0 s, 125.0e-9 s to 100.0 s
        /// 
        /// Default Value: 0.0 s
        /// 
        /// Note(s):
        /// 
        /// (1) The value 0.0 s deactivates the use of GATED TRIGGER; other values activate the GATED TRIGGER function.
        /// 
        /// (2) Values &lt;&gt; 0.0 s are only possible if &lt;trigger mode&gt; is different from Immediate. Otherwise, an execution error is triggered.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetMultiplePowerMeasurementParameters(int Window, bool Peak_Detector, bool RMS_Detector, bool Average_Detector, int Trigger_Mode, int Trigger_Slope, double Trigger_Offset, double Gate_Length)
        {
            int pInvokeResult = PInvoke.SetMultiplePowerMeasurementParameters(this._handle, Window, System.Convert.ToUInt16(Peak_Detector), System.Convert.ToUInt16(RMS_Detector), System.Convert.ToUInt16(Average_Detector), Trigger_Mode, Trigger_Slope, Trigger_Offset, Gate_Length);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns defined settings of the multiple power measurement.
        /// 
        /// Mode(s):
        /// Spectrum Analysis (frequency domain only)
        /// Spectrum Analysis (time domain only (zero span))
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]LIST:POWer:SET?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Peak_Detector">
        /// This control returns if the measurement of the peak power (peak detector) is activated or not.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// </param>
        /// <param name="RMS_Detector">
        /// This control returns if the measurement of the RMS power (RMS detector) is activated or not.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// </param>
        /// <param name="Average_Detector">
        /// This control returns if the measurement of the average power (average detector) is activated or not.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// </param>
        /// <param name="Trigger_Mode">
        /// Returns selection of the trigger source used for the list measurement.
        /// 
        /// Valid Values:
        /// RSESCI_TRIG_IMM (0) - Immediate
        /// RSESCI_TRIG_EXT (1) - External
        /// RSESCI_TRIG_VID (2) - Video
        /// RSESCI_TRIG_IFP (3) - IF Power
        /// 
        /// Note(s):
        /// 
        /// (1) Immediate - automatic triggering the next measurement at the end of the previous one. The value 'Immediate' corresponds to the FREE RUN setting.
        /// 
        /// (2) External - the next measurement is triggered by the signal at the external trigger input.
        /// 
        /// (3) Video - the next measurement is triggered by the detection of a signal at the video filter output (not available for FM DEMOD mode).
        /// 
        /// (4) IF Power - the next measurement is triggered by the detection of a signal at the instrument IF (10 MHz bandwidth).
        /// 
        /// </param>
        /// <param name="Trigger_Slope">
        /// This control returns used trigger slope.
        /// 
        /// Valid Values:
        /// RSESCI_POS (0) - Positive
        /// RSESCI_NEG (1) - Negative
        /// 
        /// </param>
        /// <param name="Trigger_Offset">
        /// This control returns offset between the detection of the trigger signal and the start of the measurement at the next frequency point.
        /// 
        /// </param>
        /// <param name="Gate_Length">
        /// This control returns gate length with Gated Sweep.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetMultiplePowerMeasurementParameters(int Window, out bool Peak_Detector, out bool RMS_Detector, out bool Average_Detector, out int Trigger_Mode, out int Trigger_Slope, out double Trigger_Offset, out double Gate_Length)
        {
            ushort Peak_DetectorAsUShort;
            ushort RMS_DetectorAsUShort;
            ushort Average_DetectorAsUShort;
            int pInvokeResult = PInvoke.GetMultiplePowerMeasurementParameters(this._handle, Window, out Peak_DetectorAsUShort, out RMS_DetectorAsUShort, out Average_DetectorAsUShort, out Trigger_Mode, out Trigger_Slope, out Trigger_Offset, out Gate_Length);
            Peak_Detector = System.Convert.ToBoolean(Peak_DetectorAsUShort);
            RMS_Detector = System.Convert.ToBoolean(RMS_DetectorAsUShort);
            Average_Detector = System.Convert.ToBoolean(Average_DetectorAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the list of settings (max. 100 entries) for the multiple power measurement and starts a measurement sequence. When synchronizing the command with *OPC, a service request is generated as soon as all frequency points are processed and the defined number of individual measurements is reached.
        /// 
        /// To reduce the setting time, all indicated parameters are set up simultaneously at each test point.
        /// 
        /// Note(s):
        /// 
        /// (1) The measurement is performed in the time domain and therefore the span is set to 0 Hz. If the time domain is abandoned, the function is automatically switched off.
        /// 
        /// (2) The measurement is not compatible with other measurements, especially as far as marker, adjacent channel power measurement or statistics are concerned. The corresponding commands thus automatically deactivate the function.
        /// 
        /// Mode(s):
        /// Spectrum Analysis (frequency domain only)
        /// Spectrum Analysis (time domain only (zero span))
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]LIST:POWer[:SEQuence] &lt;analyzer freq&gt;,&lt;ref level&gt;,&lt;rf att&gt;,OFF,&lt;filter type&gt;,&lt;rbw&gt;,&lt;vbw&gt;,&lt;meas time&gt;,&lt;trigger level&gt;,...
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Number_of_Entries">
        /// This control sets number of individual entries for the multiple power measurement parameters.
        /// 
        /// Valid Range: 1 to 100
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <param name="Analyzer_Frequency">
        /// Receive frequencies for the signal to be measured (= center frequency in manual operation).
        /// 
        /// Valid Range of values: 0.0 Hz to max. frequency, depending on the instrument model.
        /// 
        /// Default Value: none
        /// 
        /// </param>
        /// <param name="Reference_Level">
        /// Reference levels.
        /// 
        /// Valid Range of values: -75.0 dBm to 30.0 dBm in 5.0 dB steps
        /// 
        /// Default Value: none
        /// 
        /// </param>
        /// <param name="RF_Input_Attenuation">
        /// RF input attenuations.
        /// 
        /// Valid Range of values: 0.0 dB to 75.0 dB in 5.0 dB steps
        /// 
        /// Default Value: none
        /// 
        /// </param>
        /// <param name="Filter_Type">
        /// Filter types for the resolution bandwidths.
        /// 
        /// Valid Values:
        /// RSESCI_FILTER_NORMAL (0) - Normal
        /// RSESCI_FILTER_CFIL   (2) - Channel Filter
        /// RSESCI_FILTER_RRC    (3) - RRC (Root Raised Cosine)
        /// 
        /// Default Value: none
        /// 
        /// Notes:
        /// 
        /// (1) Normal:
        /// Normal resolution filter.
        /// 
        /// (2) Channel Filter:
        /// These are especially steep-edged filters, which are used for example in Fast ACP measurement to ensure the band-limiting of a transmission channel in the time domain.
        /// 
        /// (3) RRC:
        /// Root Raised Cosine filter. This special filter form is used to determine the channel power for some mobile radio standards.
        /// 
        /// </param>
        /// <param name="Resolution_Bandwidth">
        /// Resolution bandwidths.
        /// 
        /// Valid Range of values: 10.0 Hz to 10.0e6 Hz, 50.e6 Hz in 1, 3, 10 steps for &lt;Filter Type&gt; = Normal.
        /// 
        /// Default Value: none
        /// 
        /// Notes:
        /// 
        /// (1) See filter table for &lt;Filter Type&gt; = Channel Filter and &lt;Filter Type&gt; = RRC (Root Raised Cosine).
        /// 
        /// (2) Possible combinations of filter type and filter bandwidth see table "List of available channel filters" in section "Setting Bandwidths and Sweep Time - Key BW" of operating manual.
        /// 
        /// </param>
        /// <param name="Video_Bandwidth">
        /// Video bandwidths.
        /// 
        /// Valid Range of values: 1.0 Hz to 10.0e6 Hz in 1, 3, 10 steps.
        /// 
        /// Default Value: none
        /// 
        /// </param>
        /// <param name="Measurement_Time">
        /// Measurement times.
        /// 
        /// Valid Range of values: 1.0e-6 s to 30.0 s
        /// 
        /// Default Value: none
        /// 
        /// </param>
        /// <param name="Trigger_Level">
        /// Reserved.
        /// 
        /// Valid Range of values: Each element must be set to 0.
        /// 
        /// Default Value: none
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int MultiplePowerMeasurementSequence(int Window, int Number_of_Entries, double[] Analyzer_Frequency, double[] Reference_Level, double[] RF_Input_Attenuation, int[] Filter_Type, double[] Resolution_Bandwidth, double[] Video_Bandwidth, double[] Measurement_Time, double[] Trigger_Level)
        {
            int pInvokeResult = PInvoke.MultiplePowerMeasurementSequence(this._handle, Window, Number_of_Entries, Analyzer_Frequency, Reference_Level, RF_Input_Attenuation, Filter_Type, Resolution_Bandwidth, Video_Bandwidth, Measurement_Time, Trigger_Level);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the result of a previous list measurement as configured and initiated with SENSe:LIST:POWer[:SEQuence].
        /// 
        /// The unit of the results depends on the setting made with the CALC:UNIT command.
        /// 
        /// This function may be used to obtain measurement results in an asynchronous way, using the service request mechanism for synchronization with the end of the measurement.
        /// 
        /// If no measurement results are available, this function will return a query error.
        /// 
        /// Mode(s):
        /// Spectrum Analysis (frequency domain only)
        /// Spectrum Analysis (time domain only (zero span))
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]LIST:POWer:RESult?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Number_of_Results">
        /// Returns the number of results per test point.
        /// 
        /// </param>
        /// <param name="Power_Measurement_Results">
        /// Returns the list of multiple power measurement results. The number of results per test point depends on the setting of the "SENSe:LIST:POWer:SET" and "SENSe:LIST:POWer:SEQuence" command.
        /// 
        /// Note(s):
        /// 
        /// (1) The array should consist of at least 100 elements.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int MultiplePowerMeasurementResults(int Window, out int Number_of_Results, double[] Power_Measurement_Results)
        {
            int pInvokeResult = PInvoke.MultiplePowerMeasurementResults(this._handle, Window, out Number_of_Results, Power_Measurement_Results);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the the average type of the sense list measurement either to linear (default) or logarithmic. In linear mode voltages are averaged in logarithmic mode levels.
        /// 
        /// Mode(s):
        /// Spectrum Analysis (frequency domain only)
        /// Spectrum Analysis (time domain only (zero span))
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]LIST:POWer:SET:AVERage:TYPE LINear | LOGarithmic
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Averaging">
        /// This control sets the the average type of the sense list measurement either to linear (default) or logarithmic.
        /// 
        /// Valid Values:
        /// RSESCI_LIN (0) - Linear
        /// RSESCI_LOG (1) - Logarithmic
        /// 
        /// Default Value: RSESCI_LIN (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetMultiplePowerMeasurementAveraging(int Window, int Averaging)
        {
            int pInvokeResult = PInvoke.SetMultiplePowerMeasurementAveraging(this._handle, Window, Averaging);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function gets the the average type of the sense list measurement either to linear (default) or logarithmic. In linear mode voltages are averaged in logarithmic mode levels.
        /// 
        /// Mode(s):
        /// Spectrum Analysis (frequency domain only)
        /// Spectrum Analysis (time domain only (zero span))
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]LIST:POWer:SET:AVERage:TYPE?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Averaging">
        /// This control gets the the average type of the sense list measurement either to linear (default) or logarithmic.
        /// 
        /// Valid Values:
        /// RSESCI_LIN (0) - Linear
        /// RSESCI_LOG (1) - Logarithmic
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetMultiplePowerMeasurementAveraging(int Window, out int Averaging)
        {
            int pInvokeResult = PInvoke.GetMultiplePowerMeasurementAveraging(this._handle, Window, out Averaging);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function deactivates the multiple power (list) measurement.
        /// 
        /// Mode(s):
        /// Spectrum Analysis (frequency domain only)
        /// Spectrum Analysis (time domain only (zero span))
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]LIST:POWer:STATe OFF
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int DeactivateMultiplePowerMeasurement(int Window)
        {
            int pInvokeResult = PInvoke.DeactivateMultiplePowerMeasurement(this._handle, Window);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the instrument setup for multiple burst power measurement and starts a measurement sequence. When synchronizing this function with *OPC, a service request is generated as soon as the defined number of individual measurements (# of meas) is reached.
        /// 
        /// To reduce the setting time, the setup is performed simultaneously for all selected parameters.
        /// 
        /// Note(s):
        /// 
        /// (1) The measurement function always uses trace 1 in the selected screen, and activates the selected screen. Repeated use of this function without changes to its parameters (i.e. using the same settings again) will speed up the measurement since the previous hardware settings will be cached and therefore additional hardware settling times will be avoided. This also holds true if only part of the parameters (e.g. only the trigger delay) are changed, as in this case the rest of the parameters will be cached.
        /// 
        /// (2) This measurement is not compatible with other measurements, especially as far as marker functions, adjacent-channel measurement or statistics are concerned. The corresponding functions are therefore automatically switched off. In return incompatible commands will automatically deactivate the multi burst power function.
        /// 
        /// Mode(s):
        /// Spectrum Analysis (frequency domain only)
        /// Spectrum Analysis (time domain only (zero span))
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]MPOWer[:SEQuence] &lt;analyzer freq&gt;,&lt;rbw&gt;,&lt;meas time&gt;,&lt;trigger source&gt;,&lt;trigger level&gt;,&lt;trigger offset&gt;,&lt;type of meas&gt;,&lt;# of meas&gt;
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Analyzer_Frequency">
        /// Sets receive frequency for the burst signals to be measured (= center frequency in manual operation).
        /// 
        /// Valid Range: 0.0 Hz to max. frequency, depending on instrument model
        /// 
        /// Default Value: 935.2e6 Hz
        /// 
        /// </param>
        /// <param name="Resolution_Bandwidth">
        /// Sets resolution bandwidth for the measurement.
        /// 
        /// Valid Range: 10.0 Hz to 10.0e6 Hz in steps of 1, 3, 10
        /// 
        /// Default Value: 1.0e6 Hz
        /// 
        /// </param>
        /// <param name="Measurement_Time">
        /// Sets time span during which measurement samples are sampled for RMS / peak measurement. The type of measurement is selected by
        /// &lt;Measurement Type&gt;.
        /// 
        /// Valid Range: 1.0e-6 s to 30.0 s
        /// 
        /// Default Value: 434.0e-6 s
        /// 
        /// </param>
        /// <param name="Trigger_Source">
        /// Sets trigger signal source.
        /// 
        /// Valid Values:
        /// RSESCI_TRIG_EXT (1) - External
        /// RSESCI_TRIG_VID (2) - Video
        /// 
        /// Default Value: RSESCI_TRIG_VID (2)
        /// 
        /// Note(s):
        /// 
        /// (1) External - the next measurement is triggered by the signal at the external trigger input.
        /// 
        /// (2) Video - the next measurement is triggered by the detection of a signal at the video filter output (not available for FM DEMOD mode).
        /// 
        /// </param>
        /// <param name="Trigger_Level">
        /// Sets signal level at which the trigger becomes active. For &lt;Trigger Source&gt; = Video this is the level of the video signal as a percentage of the diagram height. If &lt;Trigger Source&gt; = External is selected, the value entered here is ignored, as in this case the trigger input uses TTL levels.
        /// 
        /// Valid Range: 0.0 to 100.0 PCT (&lt;Trigger Source&gt; = Video)
        /// 
        /// Default Value: 50.0 PCT
        /// 
        /// </param>
        /// <param name="Trigger_Offset">
        /// Sets offset between the detection of the trigger signal and the start of the measurement.
        /// 
        /// Valid Range: 125.0e-9 s to 100.0 s
        /// 
        /// Default Value: 5.0e-6 s
        /// 
        /// </param>
        /// <param name="Measurement_Type">
        /// Determines whether mean power (RMS) or peak power (PEAK) is to be measured. The detector is selected accordingly.
        /// 
        /// Valid Values:
        /// RSESCI_POWER_MEAS_PEAK (2) - Peak Power
        /// RSESCI_POWER_MEAS_RMS  (3) - Mean Power (RMS)
        /// 
        /// Default Value: RSESCI_POWER_MEAS_RMS (3)
        /// 
        /// </param>
        /// <param name="No_of_Measurements">
        /// Sets number of individual bursts to be measured.
        /// 
        /// Valid Range: 1 to 501
        /// 
        /// Default Value: 20
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int MultipleBurstPowerMeasurementSequence(int Window, double Analyzer_Frequency, double Resolution_Bandwidth, double Measurement_Time, int Trigger_Source, double Trigger_Level, double Trigger_Offset, int Measurement_Type, int No_of_Measurements)
        {
            int pInvokeResult = PInvoke.MultipleBurstPowerMeasurementSequence(this._handle, Window, Analyzer_Frequency, Resolution_Bandwidth, Measurement_Time, Trigger_Source, Trigger_Level, Trigger_Offset, Measurement_Type, No_of_Measurements);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the results of a multiple burst power measurement as configured and initiated with SENSe:MPOWer[:SEQuence]. 
        /// 
        /// The unit used for the return values is always dBm. This function may be used to obtain measurement results in an asynchronous way using the service request mechanism for synchronization with the end of the measurement.
        /// 
        /// If no measurement results are available, this function will return a query error.
        /// 
        /// Mode(s):
        /// Spectrum Analysis (frequency domain only)
        /// Spectrum Analysis (time domain only (zero span))
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]MPOWer:RESult[:LIST]?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Power_Measurement_Results">
        /// Returns the multiple burst power measurement results. The number of results depends on the setting of the "[SENSe&lt;1|2&gt;:]MPOWer[:SEQuence]" command.
        /// 
        /// Note(s):
        /// 
        /// (1) The array should consist of at least 501 elements.
        /// 
        /// </param>
        /// <param name="Number_of_Results">
        /// Returns the number of individual bursts measured.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int MultipleBurstPowerMeasurementResult(int Window, double[] Power_Measurement_Results, out int Number_of_Results)
        {
            int pInvokeResult = PInvoke.MultipleBurstPowerMeasurementResult(this._handle, Window, Power_Measurement_Results, out Number_of_Results);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the minimum power value in a multiple burst power measurement as configured and initiated with SENSe:MPOWer[:SEQuence].
        /// 
        /// If no measurement result is available, this function will return a query error.
        /// 
        /// Mode(s):
        /// Spectrum Analysis (frequency domain only)
        /// Spectrum Analysis (time domain only (zero span))
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]MPOWer:RESult:MIN?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Power_Measurement_Result">
        /// Returns the minimum power value in a multiple burst power measurement.
        /// 
        /// Note(s):
        /// 
        /// The unit used for the return values is always dBm.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int MultipleBurstPowerMeasurementResultMinimum(int Window, out double Power_Measurement_Result)
        {
            int pInvokeResult = PInvoke.MultipleBurstPowerMeasurementResultMinimum(this._handle, Window, out Power_Measurement_Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets harmonic distortion state.
        /// 
        /// Note(s):
        /// 
        /// (1) The function is independent of the marker selection, i.e. the suffix &lt;1|2&gt; or &lt;1...4&gt; of CALCulate or MARKer is irrelevant.
        /// 
        /// Mode(s):
        /// Spectrum Analysis (frequency domain)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:HARMonics[:STATe] ON | OFF
        /// </summary>
        /// <param name="State">
        /// Switches on or off the measurement of the harmonics of a carrier signal. The carrier signal is the first harmonic. The function is independent of the marker selection. It is only available in the frequency domain (span &gt; 0).
        /// If the measurement is started in the frequency domain (span &gt; 0), the last span will define the search range for the first harmonic. The level for the first harmonic will also be calculated in the frequency domain. However, the measurement can also be started in the time domain (span = 0), in which case the center frequency and the level used will remain unchanged. 
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation.  The status code  either indicates success or describes an error or warning condition.  You examine the status code from each call to an instrument driver function to determine if an error occurred.
        /// 
        /// To obtain a text description of the status code, call the rsspecan_error_message function.  To obtain additional information about the error condition, call the rsspecan_GetError function.  To clear the error information from the driver, call the rsspecan_ClearError function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// WARNINGS:
        /// 3FFA2001  The instrument was in an uncalibrated state
        ///           when the measurement was taken.
        /// 3FFA2002  The measurement taken was over the instrument??s
        ///           range.
        ///   
        /// ERRORS:
        /// BFFA2001  The Active Marker is not enabled.
        /// BFFA2002  The Active Marker is not a delta marker.
        /// BFFA2003  The maximum waiting time for this operation was
        ///           exceeded.
        ///           
        /// This instrument driver also returns errors and warnings defined by other sources.  The following table defines the ranges of additional status codes that this driver can return.  The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFA0000 to 3FFA1FFF     IVI    Warnings
        /// 3FFF0000 to 3FFFFFFF     VISA   Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP Driver Warnings
        ///           
        /// BFFA0000 to BFFA1FFF     IVI    Errors
        /// BFFF0000 to BFFFFFFF     VISA   Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP Driver Errors
        /// 
        /// </returns>
        public int SetHDistState(bool State)
        {
            int pInvokeResult = PInvoke.SetHDistState(this._handle, System.Convert.ToUInt16(State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function gets harmonic distortion state.
        /// 
        /// Note(s):
        /// 
        /// (1) The function is independent of the marker selection, i.e. the suffix &lt;1|2&gt; or &lt;1...4&gt; of CALCulate or MARKer is irrelevant.
        /// 
        /// Mode(s):
        /// Spectrum Analysis (frequency domain)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:HARMonics[:STATe]?
        /// </summary>
        /// <param name="State">
        /// Returns the state of the measurement of the harmonics of a carrier signal. 
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <returns>
        /// Returns the status code of this operation.  The status code  either indicates success or describes an error or warning condition.  You examine the status code from each call to an instrument driver function to determine if an error occurred.
        /// 
        /// To obtain a text description of the status code, call the rsspecan_error_message function.  To obtain additional information about the error condition, call the rsspecan_GetError function.  To clear the error information from the driver, call the rsspecan_ClearError function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// WARNINGS:
        /// 3FFA2001  The instrument was in an uncalibrated state
        ///           when the measurement was taken.
        /// 3FFA2002  The measurement taken was over the instrument??s
        ///           range.
        ///   
        /// ERRORS:
        /// BFFA2001  The Active Marker is not enabled.
        /// BFFA2002  The Active Marker is not a delta marker.
        /// BFFA2003  The maximum waiting time for this operation was
        ///           exceeded.
        ///           
        /// This instrument driver also returns errors and warnings defined by other sources.  The following table defines the ranges of additional status codes that this driver can return.  The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFA0000 to 3FFA1FFF     IVI    Warnings
        /// 3FFF0000 to 3FFFFFFF     VISA   Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP Driver Warnings
        ///           
        /// BFFA0000 to BFFA1FFF     IVI    Errors
        /// BFFF0000 to BFFFFFFF     VISA   Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP Driver Errors
        /// 
        /// </returns>
        public int GetHDistState(out bool State)
        {
            ushort StateAsUShort;
            int pInvokeResult = PInvoke.GetHDistState(this._handle, out StateAsUShort);
            State = System.Convert.ToBoolean(StateAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the number of harmonics of a carrier signal to be measured.
        /// 
        /// Note(s):
        /// 
        /// (1) The function is independent of the marker selection, i.e. the suffix &lt;1|2&gt; or &lt;1...4&gt; of CALCulate or MARKer is irrelevant.
        /// 
        /// Mode(s):
        /// Spectrum Analysis (frequency domain)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:HARMonics:NHARmonics
        /// </summary>
        /// <param name="No_of_Harmonics">
        /// Defines the number of harmonics of a carrier signal to be measured. 
        /// 
        /// Valid Values:
        /// 1 to 26
        /// 
        /// Default Value: 1
        /// </param>
        /// <returns>
        /// Returns the status code of this operation.  The status code  either indicates success or describes an error or warning condition.  You examine the status code from each call to an instrument driver function to determine if an error occurred.
        /// 
        /// To obtain a text description of the status code, call the rsspecan_error_message function.  To obtain additional information about the error condition, call the rsspecan_GetError function.  To clear the error information from the driver, call the rsspecan_ClearError function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// WARNINGS:
        /// 3FFA2001  The instrument was in an uncalibrated state
        ///           when the measurement was taken.
        /// 3FFA2002  The measurement taken was over the instrument??s
        ///           range.
        ///   
        /// ERRORS:
        /// BFFA2001  The Active Marker is not enabled.
        /// BFFA2002  The Active Marker is not a delta marker.
        /// BFFA2003  The maximum waiting time for this operation was
        ///           exceeded.
        ///           
        /// This instrument driver also returns errors and warnings defined by other sources.  The following table defines the ranges of additional status codes that this driver can return.  The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFA0000 to 3FFA1FFF     IVI    Warnings
        /// 3FFF0000 to 3FFFFFFF     VISA   Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP Driver Warnings
        ///           
        /// BFFA0000 to BFFA1FFF     IVI    Errors
        /// BFFF0000 to BFFFFFFF     VISA   Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP Driver Errors
        /// 
        /// </returns>
        public int SetNumberOfHDist(int No_of_Harmonics)
        {
            int pInvokeResult = PInvoke.SetNumberOfHDist(this._handle, No_of_Harmonics);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the number of harmonics of a carrier signal to be measured.
        /// 
        /// Note(s):
        /// 
        /// (1) The function is independent of the marker selection, i.e. the suffix &lt;1|2&gt; or &lt;1...4&gt; of CALCulate or MARKer is irrelevant.
        /// 
        /// Mode(s):
        /// Spectrum Analysis (frequency domain)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:HARMonics:NHARmonics?
        /// </summary>
        /// <param name="No_of_Harmonics">
        /// Returns the number of harmonics of a carrier signal to be measured. 
        /// 
        /// Valid Values:
        /// 1 to 26
        /// </param>
        /// <returns>
        /// Returns the status code of this operation.  The status code  either indicates success or describes an error or warning condition.  You examine the status code from each call to an instrument driver function to determine if an error occurred.
        /// 
        /// To obtain a text description of the status code, call the rsspecan_error_message function.  To obtain additional information about the error condition, call the rsspecan_GetError function.  To clear the error information from the driver, call the rsspecan_ClearError function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// WARNINGS:
        /// 3FFA2001  The instrument was in an uncalibrated state
        ///           when the measurement was taken.
        /// 3FFA2002  The measurement taken was over the instrument??s
        ///           range.
        ///   
        /// ERRORS:
        /// BFFA2001  The Active Marker is not enabled.
        /// BFFA2002  The Active Marker is not a delta marker.
        /// BFFA2003  The maximum waiting time for this operation was
        ///           exceeded.
        ///           
        /// This instrument driver also returns errors and warnings defined by other sources.  The following table defines the ranges of additional status codes that this driver can return.  The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFA0000 to 3FFA1FFF     IVI    Warnings
        /// 3FFF0000 to 3FFFFFFF     VISA   Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP Driver Warnings
        ///           
        /// BFFA0000 to BFFA1FFF     IVI    Errors
        /// BFFF0000 to BFFFFFFF     VISA   Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP Driver Errors
        /// 
        /// </returns>
        public int GetNumberOfHDist(out int No_of_Harmonics)
        {
            int pInvokeResult = PInvoke.GetNumberOfHDist(this._handle, out No_of_Harmonics);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function specifies whether the resolution bandwidth of the 2nd to the nth harmonic should be identical to the bandwidth of the first harmonic (OFF) or to the next largest bandwidth (corresponding to the harmonic) of the multiple of the bandwidth of the first harmonic (ON).
        /// 
        /// Note(s):
        /// 
        /// (1) The function is independent of the marker selection, i.e. the suffix &lt;1|2&gt; or &lt;1...4&gt; of CALCulate or MARKer is irrelevant.
        /// 
        /// Mode(s):
        /// Spectrum Analysis (frequency domain)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:HARMonics:BANDwidth:AUTO ON | OFF
        /// </summary>
        /// <param name="Harmonic_RBW_Auto">
        /// Specifies whether the resolution bandwidth of the 2nd to the nth harmonic should be identical to the bandwidth of the first harmonic (OFF) or to the next largest bandwidth (corresponding to the harmonic) of the multiple of the bandwidth of the first harmonic (ON). 
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation.  The status code  either indicates success or describes an error or warning condition.  You examine the status code from each call to an instrument driver function to determine if an error occurred.
        /// 
        /// To obtain a text description of the status code, call the rsspecan_error_message function.  To obtain additional information about the error condition, call the rsspecan_GetError function.  To clear the error information from the driver, call the rsspecan_ClearError function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// WARNINGS:
        /// 3FFA2001  The instrument was in an uncalibrated state
        ///           when the measurement was taken.
        /// 3FFA2002  The measurement taken was over the instrument??s
        ///           range.
        ///   
        /// ERRORS:
        /// BFFA2001  The Active Marker is not enabled.
        /// BFFA2002  The Active Marker is not a delta marker.
        /// BFFA2003  The maximum waiting time for this operation was
        ///           exceeded.
        ///           
        /// This instrument driver also returns errors and warnings defined by other sources.  The following table defines the ranges of additional status codes that this driver can return.  The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFA0000 to 3FFA1FFF     IVI    Warnings
        /// 3FFF0000 to 3FFFFFFF     VISA   Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP Driver Warnings
        ///           
        /// BFFA0000 to BFFA1FFF     IVI    Errors
        /// BFFF0000 to BFFFFFFF     VISA   Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP Driver Errors
        /// 
        /// </returns>
        public int SetHDistBandwidthState(bool Harmonic_RBW_Auto)
        {
            int pInvokeResult = PInvoke.SetHDistBandwidthState(this._handle, System.Convert.ToUInt16(Harmonic_RBW_Auto));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns whether the resolution bandwidth of the 2nd to the nth harmonic should be identical to the bandwidth of the first harmonic (OFF) or to the next largest bandwidth (corresponding to the harmonic) of the multiple of the bandwidth of the first harmonic (ON).
        /// 
        /// Note(s):
        /// 
        /// (1) The function is independent of the marker selection, i.e. the suffix &lt;1|2&gt; or &lt;1...4&gt; of CALCulate or MARKer is irrelevant.
        /// 
        /// Mode(s):
        /// Spectrum Analysis (frequency domain)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:HARMonics:BANDwidth:AUTO?
        /// </summary>
        /// <param name="Harmonic_RBW_Auto">
        /// Returns whether the resolution bandwidth of the 2nd to the nth harmonic should be identical to the bandwidth of the first harmonic (OFF) or to the next largest bandwidth (corresponding to the harmonic) of the multiple of the bandwidth of the first harmonic (ON). 
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <returns>
        /// Returns the status code of this operation.  The status code  either indicates success or describes an error or warning condition.  You examine the status code from each call to an instrument driver function to determine if an error occurred.
        /// 
        /// To obtain a text description of the status code, call the rsspecan_error_message function.  To obtain additional information about the error condition, call the rsspecan_GetError function.  To clear the error information from the driver, call the rsspecan_ClearError function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// WARNINGS:
        /// 3FFA2001  The instrument was in an uncalibrated state
        ///           when the measurement was taken.
        /// 3FFA2002  The measurement taken was over the instrument??s
        ///           range.
        ///   
        /// ERRORS:
        /// BFFA2001  The Active Marker is not enabled.
        /// BFFA2002  The Active Marker is not a delta marker.
        /// BFFA2003  The maximum waiting time for this operation was
        ///           exceeded.
        ///           
        /// This instrument driver also returns errors and warnings defined by other sources.  The following table defines the ranges of additional status codes that this driver can return.  The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFA0000 to 3FFA1FFF     IVI    Warnings
        /// 3FFF0000 to 3FFFFFFF     VISA   Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP Driver Warnings
        ///           
        /// BFFA0000 to BFFA1FFF     IVI    Errors
        /// BFFF0000 to BFFFFFFF     VISA   Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP Driver Errors
        /// 
        /// </returns>
        public int GetHDistBandwidthState(out bool Harmonic_RBW_Auto)
        {
            ushort Harmonic_RBW_AutoAsUShort;
            int pInvokeResult = PInvoke.GetHDistBandwidthState(this._handle, out Harmonic_RBW_AutoAsUShort);
            Harmonic_RBW_Auto = System.Convert.ToBoolean(Harmonic_RBW_AutoAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function optimizes the device settings depending on the mode in which the harmonic measurement was started:
        /// If the harmonic measurement was started in the frequency domain (span &gt; 0), the frequency and the level of the first harmonic are calculated, from which the measurement list is set up.
        /// If the measurement was started in the time domain (span = 0), the frequency of the first harmonic is not changed. The level, however, is not calculated. The function is independent of the marker selection.
        /// 
        /// Note(s):
        /// 
        /// (1) The function is independent of the marker selection, i.e. the suffix &lt;1|2&gt; or &lt;1...4&gt; of CALCulate or MARKer is irrelevant.
        /// 
        /// Mode(s):
        /// Spectrum Analysis (frequency domain)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:HARMonics:PRESet
        /// </summary>
        /// <returns>
        /// Returns the status code of this operation.  The status code  either indicates success or describes an error or warning condition.  You examine the status code from each call to an instrument driver function to determine if an error occurred.
        /// 
        /// To obtain a text description of the status code, call the rsspecan_error_message function.  To obtain additional information about the error condition, call the rsspecan_GetError function.  To clear the error information from the driver, call the rsspecan_ClearError function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// WARNINGS:
        /// 3FFA2001  The instrument was in an uncalibrated state
        ///           when the measurement was taken.
        /// 3FFA2002  The measurement taken was over the instrument??s
        ///           range.
        ///   
        /// ERRORS:
        /// BFFA2001  The Active Marker is not enabled.
        /// BFFA2002  The Active Marker is not a delta marker.
        /// BFFA2003  The maximum waiting time for this operation was
        ///           exceeded.
        ///           
        /// This instrument driver also returns errors and warnings defined by other sources.  The following table defines the ranges of additional status codes that this driver can return.  The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFA0000 to 3FFA1FFF     IVI    Warnings
        /// 3FFF0000 to 3FFFFFFF     VISA   Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP Driver Warnings
        ///           
        /// BFFA0000 to BFFA1FFF     IVI    Errors
        /// BFFF0000 to BFFFFFFF     VISA   Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP Driver Errors
        /// 
        /// </returns>
        public int AdjustHDistSettings()
        {
            int pInvokeResult = PInvoke.AdjustHDistSettings(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the results of the total harmonic distortion (THD).
        /// 
        /// A complete sweep with synchronization to sweep end must be performed between switching on the function and querying the measured value to obtain a valid query result. This is only possible in single sweep mode.
        /// 
        /// Note(s):
        /// 
        /// (1) The function is independent of the marker selection, i.e. the suffix &lt;1|2&gt; or &lt;1...4&gt; of CALCulate or MARKer is irrelevant.
        /// 
        /// Mode(s):
        /// Spectrum Analysis (frequency domain)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:HARMonics:DISTortion? TOTal
        /// </summary>
        /// <param name="Harmonic_Distortion_in_percent">
        /// Returns the results of the total harmonic distortion (THD) in %.
        /// </param>
        /// <param name="Harmonic_Distortion_in_dB">
        /// Returns the results of the total harmonic distortion (THD) in dB.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation.  The status code  either indicates success or describes an error or warning condition.  You examine the status code from each call to an instrument driver function to determine if an error occurred.
        /// 
        /// To obtain a text description of the status code, call the rsspecan_error_message function.  To obtain additional information about the error condition, call the rsspecan_GetError function.  To clear the error information from the driver, call the rsspecan_ClearError function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// WARNINGS:
        /// 3FFA2001  The instrument was in an uncalibrated state
        ///           when the measurement was taken.
        /// 3FFA2002  The measurement taken was over the instrument??s
        ///           range.
        ///   
        /// ERRORS:
        /// BFFA2001  The Active Marker is not enabled.
        /// BFFA2002  The Active Marker is not a delta marker.
        /// BFFA2003  The maximum waiting time for this operation was
        ///           exceeded.
        ///           
        /// This instrument driver also returns errors and warnings defined by other sources.  The following table defines the ranges of additional status codes that this driver can return.  The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFA0000 to 3FFA1FFF     IVI    Warnings
        /// 3FFF0000 to 3FFFFFFF     VISA   Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP Driver Warnings
        ///           
        /// BFFA0000 to BFFA1FFF     IVI    Errors
        /// BFFF0000 to BFFFFFFF     VISA   Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP Driver Errors
        /// 
        /// </returns>
        public int GetHDist(out double Harmonic_Distortion_in_percent, out double Harmonic_Distortion_in_dB)
        {
            int pInvokeResult = PInvoke.GetHDist(this._handle, out Harmonic_Distortion_in_percent, out Harmonic_Distortion_in_dB);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function reads out the list of harmonics. The first value is the absolute power of the first harmonic in the unit set via UNIT. The other values are relative to the carrier signal and are output in dB.
        /// 
        /// A complete sweep with synchronization to sweep end must be performed between switching on the function and querying the measured value to obtain a valid query result. This is only possible in single sweep mode.
        /// 
        /// Note(s):
        /// 
        /// (1) The function is independent of the marker selection, i.e. the suffix &lt;1|2&gt; or &lt;1...4&gt; of CALCulate or MARKer is irrelevant.
        /// 
        /// Mode(s):
        /// Spectrum Analysis (frequency domain)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:HARMonics:LIST?
        /// </summary>
        /// <param name="Harmonics_List">
        /// Returns the list of harmonics. The first value is the absolute power of the first harmonic in the unit set via UNIT. The other values are relative to the carrier signal and are output in dB.
        /// 
        /// Number of values corresponds to that of the harmonics to be measured, which are defined with CALC:MARK:FUNC:HARM:NHARM.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation.  The status code  either indicates success or describes an error or warning condition.  You examine the status code from each call to an instrument driver function to determine if an error occurred.
        /// 
        /// To obtain a text description of the status code, call the rsspecan_error_message function.  To obtain additional information about the error condition, call the rsspecan_GetError function.  To clear the error information from the driver, call the rsspecan_ClearError function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// WARNINGS:
        /// 3FFA2001  The instrument was in an uncalibrated state
        ///           when the measurement was taken.
        /// 3FFA2002  The measurement taken was over the instrument??s
        ///           range.
        ///   
        /// ERRORS:
        /// BFFA2001  The Active Marker is not enabled.
        /// BFFA2002  The Active Marker is not a delta marker.
        /// BFFA2003  The maximum waiting time for this operation was
        ///           exceeded.
        ///           
        /// This instrument driver also returns errors and warnings defined by other sources.  The following table defines the ranges of additional status codes that this driver can return.  The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFA0000 to 3FFA1FFF     IVI    Warnings
        /// 3FFF0000 to 3FFFFFFF     VISA   Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP Driver Warnings
        ///           
        /// BFFA0000 to BFFA1FFF     IVI    Errors
        /// BFFF0000 to BFFFFFFF     VISA   Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP Driver Errors
        /// 
        /// </returns>
        public int GetHDistHarmonicsList(double[] Harmonics_List)
        {
            int pInvokeResult = PInvoke.GetHDistHarmonicsList(this._handle, Harmonics_List);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the resolution bandwidth values used by the harmonic measurment. A separate value for every harmonic frequency is automatically calculated. 
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:HARMonics:BANDwidth[:LIST]?
        /// </summary>
        /// <param name="Array_Size">
        /// This control defines the size of array for the bandwidth resolution
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: 256
        /// </param>
        /// <param name="Resolution_Bandwidth_Value">
        /// This control returns the resolution bandwidth values used by the harmonic measurment.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetHDistBandwidthResolution(int Array_Size, double[] Resolution_Bandwidth_Value)
        {
            int pInvokeResult = PInvoke.GetHDistBandwidthResolution(this._handle, Array_Size, Resolution_Bandwidth_Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches the spurious emissions measurement on
        /// or off.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SWEep:MODE AUTO | LIST
        /// </summary>
        /// <param name="Mode">
        /// This control switches the spurious emissions measurement on
        /// or off. 
        /// 
        /// Valid Values: 
        /// RSESCI_SWEEP_MODE_AUTO (0) - Auto
        /// RSESCI_SWEEP_MODE_LIST (1) - List
        /// 
        /// Default Value: RSESCI_SWEEP_MODE_AUTO (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation.  The status code  either indicates success or describes an error or warning condition.  You examine the status code from each call to an instrument driver function to determine if an error occurred.
        /// 
        /// To obtain a text description of the status code, call the rsspecan_error_message function.  To obtain additional information about the error condition, call the rsspecan_GetError function.  To clear the error information from the driver, call the rsspecan_ClearError function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// WARNINGS:
        /// 3FFA2001  The instrument was in an uncalibrated state
        ///           when the measurement was taken.
        /// 3FFA2002  The measurement taken was over the instrument??s
        ///           range.
        ///   
        /// ERRORS:
        /// BFFA2001  The Active Marker is not enabled.
        /// BFFA2002  The Active Marker is not a delta marker.
        /// BFFA2003  The maximum waiting time for this operation was
        ///           exceeded.
        ///           
        /// This instrument driver also returns errors and warnings defined by other sources.  The following table defines the ranges of additional status codes that this driver can return.  The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFA0000 to 3FFA1FFF     IVI    Warnings
        /// 3FFF0000 to 3FFFFFFF     VISA   Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP Driver Warnings
        ///           
        /// BFFA0000 to BFFA1FFF     IVI    Errors
        /// BFFF0000 to BFFFFFFF     VISA   Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP Driver Errors
        /// 
        /// </returns>
        public int SetSEMode(int Mode)
        {
            int pInvokeResult = PInvoke.SetSEMode(this._handle, Mode);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the state of the spurious emissions measurement.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SWEep:MODE?
        /// </summary>
        /// <param name="Mode">
        /// This control returns the state of the spurious emissions measurement. 
        /// 
        /// Valid Values: 
        /// RSESCI_SWEEP_MODE_AUTO (0) - Auto
        /// RSESCI_SWEEP_MODE_LIST (1) - List
        /// </param>
        /// <returns>
        /// Returns the status code of this operation.  The status code  either indicates success or describes an error or warning condition.  You examine the status code from each call to an instrument driver function to determine if an error occurred.
        /// 
        /// To obtain a text description of the status code, call the rsspecan_error_message function.  To obtain additional information about the error condition, call the rsspecan_GetError function.  To clear the error information from the driver, call the rsspecan_ClearError function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// WARNINGS:
        /// 3FFA2001  The instrument was in an uncalibrated state
        ///           when the measurement was taken.
        /// 3FFA2002  The measurement taken was over the instrument??s
        ///           range.
        ///   
        /// ERRORS:
        /// BFFA2001  The Active Marker is not enabled.
        /// BFFA2002  The Active Marker is not a delta marker.
        /// BFFA2003  The maximum waiting time for this operation was
        ///           exceeded.
        ///           
        /// This instrument driver also returns errors and warnings defined by other sources.  The following table defines the ranges of additional status codes that this driver can return.  The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFA0000 to 3FFA1FFF     IVI    Warnings
        /// 3FFF0000 to 3FFFFFFF     VISA   Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP Driver Warnings
        ///           
        /// BFFA0000 to BFFA1FFF     IVI    Errors
        /// BFFF0000 to BFFFFFFF     VISA   Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP Driver Errors
        /// 
        /// </returns>
        public int GetSEMode(out int Mode)
        {
            int pInvokeResult = PInvoke.GetSEMode(this._handle, out Mode);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the sweep ranges.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]LIST:RANGe&lt;1...20&gt;[:FREQuency]:STARt
        /// [SENSe&lt;1|2&gt;:]LIST:RANGe&lt;1...20&gt;[:FREQuency]:STOP
        /// [SENSe&lt;1|2&gt;:]LIST:RANGe&lt;1...20&gt;:FILTer:TYPE NORMal | CHANnel | RRC | P5 | NOISe | PULSe
        /// [SENSe&lt;1|2&gt;:]LIST:RANGe&lt;1...20&gt;:BANDwidth[:RESolution]
        /// [SENSe&lt;1|2&gt;:]LIST:RANGe&lt;1...20&gt;:BANDwidth:VIDeo
        /// [SENSe&lt;1|2&gt;:]LIST:RANGe&lt;1...20&gt;:SWEep:TIME:AUTO ON | OFF
        /// [SENSe&lt;1|2&gt;:]LIST:RANGe&lt;1...20&gt;:SWEep:TIME
        /// [SENSe&lt;1|2&gt;:]LIST:RANGe&lt;1...20&gt;:DETector APEak | NEGative | POSitive | SAMPle | RMS | AVERage
        /// [SENSe&lt;1|2&gt;:]LIST:RANGe&lt;1...20&gt;:RLEVel
        /// [SENSe&lt;1|2&gt;:]LIST:RANGe&lt;1...20&gt;:INPut:ATTenuation:AUTO ON | OFF
        /// [SENSe&lt;1|2&gt;:]LIST:RANGe&lt;1...20&gt;:INPut:ATTenuation
        /// [SENSe&lt;1|2&gt;:]LIST:RANGe&lt;1...20&gt;:POINts
        /// [SENSe&lt;1|2&gt;:]LIST:RANGe&lt;1...20&gt;:BREAK ON | OFF
        /// [SENSe&lt;1|2&gt;:]LIST:RANGe&lt;1...20&gt;:TRANsducer
        /// </summary>
        /// <param name="Range">
        /// Sets the configured range.
        /// 
        /// Valid Values:
        /// 1 to 20
        /// 
        /// Default Value:
        /// 1
        /// </param>
        /// <param name="Start_Frequency">
        /// Sets the start frequency of a range in the spurious measurement. 
        /// 
        /// Units: Hz
        /// 
        /// Valid Values: no range checking
        /// 
        /// Default Value: 20.0
        /// </param>
        /// <param name="Stop_Frequency">
        /// Sets the stop frequency of a range in the spurious measurement. 
        /// 
        /// Units: Hz
        /// 
        /// Valid Values: no range checking
        /// 
        /// Default Value: 1.0e+9
        /// </param>
        /// <param name="Filter">
        /// This control sets the filter in the spurious measurement. 
        /// 
        /// Valid Values:
        /// RSESCI_LIST_FTYPE_NORM  (0) - Normal
        /// RSESCI_LIST_FTYPE_CHAN  (1) - Channel
        /// RSESCI_LIST_FTYPE_RRC   (2) - RRC
        /// RSESCI_LIST_FTYPE_P5    (3) - P5
        /// RSESCI_LIST_FTYPE_NOISE (4) - Noise
        /// RSESCI_LIST_FTYPE_PULSE (5) - Pulse
        /// 
        /// Default Value: RSESCI_LIST_FTYPE_NORM (0)
        /// 
        /// 
        /// </param>
        /// <param name="Resolution_Bandwidth">
        /// Sets the the resolution bandwidth. 
        /// 
        /// Valid Range: no range checking
        /// 
        /// Default Value: 10.0e3 Hz
        /// 
        /// </param>
        /// <param name="Video_Bandwidth">
        /// Sets the the video bandwidth. 
        /// 
        /// Valid Range: no range checking
        /// 
        /// Default Value: 30.0e3 Hz
        /// 
        /// </param>
        /// <param name="Sweep_Time_Auto">
        /// Sets the automatic coupling of the sweep duration on the frequency span and the bandwidth settings in a spurious measurement range.
        /// 
        /// Valid Range:
        /// VI_FALSE (0) - Off 
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_TRUE (1)
        /// 
        /// </param>
        /// <param name="Sweep_Time">
        /// Sets the sweep duration of a range in the spurious measurement. 
        /// 
        /// Valid Range: 0.001 s to 16000.0 s
        ///  
        /// Default Value: 0.1 s
        /// 
        /// Note:
        /// (1) This control has effect only if Sweep Time Auto is set Off
        /// 
        /// </param>
        /// <param name="Detector">
        /// This control sets the detector in the spurious measurement.
        /// 
        /// Valid Values:
        /// RSESCI_TRACEDET_APEAK  (0) - Auto Peak
        /// RSESCI_TRACEDET_NEG    (1) - Negative
        /// RSESCI_TRACEDET_POS    (2) - Positive
        /// RSESCI_TRACEDET_SAMPLE (3) - Sample
        /// RSESCI_TRACEDET_RMS    (4) - RMS
        /// RSESCI_TRACEDET_AVG    (5) - Average
        /// 
        /// Default Value: RSESCI_TRACEDET_APEAK  (0)
        /// </param>
        /// <param name="Ref_Level_Value">
        /// Specifies the reference level of a range in the spurious measurement.
        /// 
        /// Valid Range: -130.0 dBm to 30.0 dBm
        /// 
        /// Default Value: -30.0 dBm
        /// 
        /// </param>
        /// <param name="Attenuator_Auto">
        /// Sets the autoranging of a range in the spurious measurement on or off.
        /// 
        /// Valid Range:
        /// VI_FALSE (0) - Off 
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_TRUE (1)
        /// 
        /// </param>
        /// <param name="Manual_Attenuator_Value__dB_">
        /// This control sets the RF attenuation of a range.
        /// 
        /// Valid Range: 0.0 dB to 75.0 dB
        /// 
        /// Default Value: 15.0 dB
        /// 
        /// Note:
        /// (1) This control has effect only if Autoranging control is set Off
        /// 
        /// </param>
        /// <param name="Sweep_Points">
        /// Sets the number of sweep points of a range in the spurious measurement.
        /// 
        /// Valid Range: no range checking
        /// 
        /// Default Value: 625
        /// 
        /// </param>
        /// <param name="Stop_After_Sweep">
        /// Determines whether a sweep is stopped in the spurious measurement when a range is changed.
        /// 
        /// Valid Range:
        /// VI_FALSE (0) - Off 
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// </param>
        /// <param name="Transducer_Factor">
        /// Sets the transducer factor for a range in the spurious measurement.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: ""
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation.  The status code  either indicates success or describes an error or warning condition.  You examine the status code from each call to an instrument driver function to determine if an error occurred.
        /// 
        /// To obtain a text description of the status code, call the rsspecan_error_message function.  To obtain additional information about the error condition, call the rsspecan_GetError function.  To clear the error information from the driver, call the rsspecan_ClearError function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// WARNINGS:
        /// 3FFA2001  The instrument was in an uncalibrated state
        ///           when the measurement was taken.
        /// 3FFA2002  The measurement taken was over the instrument??s
        ///           range.
        ///   
        /// ERRORS:
        /// BFFA2001  The Active Marker is not enabled.
        /// BFFA2002  The Active Marker is not a delta marker.
        /// BFFA2003  The maximum waiting time for this operation was
        ///           exceeded.
        ///           
        /// This instrument driver also returns errors and warnings defined by other sources.  The following table defines the ranges of additional status codes that this driver can return.  The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFA0000 to 3FFA1FFF     IVI    Warnings
        /// 3FFF0000 to 3FFFFFFF     VISA   Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP Driver Warnings
        ///           
        /// BFFA0000 to BFFA1FFF     IVI    Errors
        /// BFFF0000 to BFFFFFFF     VISA   Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP Driver Errors
        /// 
        /// </returns>
        public int ConfigureSESweepRange(int Range, double Start_Frequency, double Stop_Frequency, int Filter, double Resolution_Bandwidth, double Video_Bandwidth, bool Sweep_Time_Auto, double Sweep_Time, int Detector, double Ref_Level_Value, bool Attenuator_Auto, double Manual_Attenuator_Value__dB_, int Sweep_Points, bool Stop_After_Sweep, string Transducer_Factor)
        {
            int pInvokeResult = PInvoke.ConfigureSESweepRange(this._handle, Range, Start_Frequency, Stop_Frequency, Filter, Resolution_Bandwidth, Video_Bandwidth, System.Convert.ToUInt16(Sweep_Time_Auto), Sweep_Time, Detector, Ref_Level_Value, System.Convert.ToUInt16(Attenuator_Auto), Manual_Attenuator_Value__dB_, Sweep_Points, System.Convert.ToUInt16(Stop_After_Sweep), Transducer_Factor);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures state of pre-amplifier for selected sweep range.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]LIST:RANGe&lt;1...20&gt;:INPut:GAIN[:STATe] ON | OFF
        /// 
        /// </summary>
        /// <param name="Range">
        /// Sets the configured range.
        /// 
        /// Valid Values:
        /// 1 to 20
        /// 
        /// Default Value:
        /// 1
        /// </param>
        /// <param name="Pre_namplifier">
        /// Switches the preamplifier of a range on and off.
        /// 
        /// Valid Range:
        /// VI_FALSE (0) - Off 
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// Note:
        /// (1) On value is available only with option B23 or B25
        /// </param>
        /// <returns>
        /// Returns the status code of this operation.  The status code  either indicates success or describes an error or warning condition.  You examine the status code from each call to an instrument driver function to determine if an error occurred.
        /// 
        /// To obtain a text description of the status code, call the rsspecan_error_message function.  To obtain additional information about the error condition, call the rsspecan_GetError function.  To clear the error information from the driver, call the rsspecan_ClearError function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// WARNINGS:
        /// 3FFA2001  The instrument was in an uncalibrated state
        ///           when the measurement was taken.
        /// 3FFA2002  The measurement taken was over the instrument??s
        ///           range.
        ///   
        /// ERRORS:
        /// BFFA2001  The Active Marker is not enabled.
        /// BFFA2002  The Active Marker is not a delta marker.
        /// BFFA2003  The maximum waiting time for this operation was
        ///           exceeded.
        ///           
        /// This instrument driver also returns errors and warnings defined by other sources.  The following table defines the ranges of additional status codes that this driver can return.  The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFA0000 to 3FFA1FFF     IVI    Warnings
        /// 3FFF0000 to 3FFFFFFF     VISA   Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP Driver Warnings
        ///           
        /// BFFA0000 to BFFA1FFF     IVI    Errors
        /// BFFF0000 to BFFFFFFF     VISA   Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP Driver Errors
        /// 
        /// </returns>
        public int SetSEPreamplifierState(int Range, bool Pre_namplifier)
        {
            int pInvokeResult = PInvoke.SetSEPreamplifierState(this._handle, Range, System.Convert.ToUInt16(Pre_namplifier));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the state of pre-amplifier for selected sweep range.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]LIST:RANGe&lt;1...20&gt;:INPut:GAIN[:STATe]?
        /// </summary>
        /// <param name="Range">
        /// Sets the configured range.
        /// 
        /// Valid Values:
        /// 1 to 20
        /// 
        /// Default Value:
        /// 1
        /// </param>
        /// <param name="Pre_namplifier">
        /// Returns the state of the preamplifier of a range.
        /// 
        /// Valid Range:
        /// VI_FALSE (0) - Off 
        /// VI_TRUE  (1) - On
        /// </param>
        /// <returns>
        /// Returns the status code of this operation.  The status code  either indicates success or describes an error or warning condition.  You examine the status code from each call to an instrument driver function to determine if an error occurred.
        /// 
        /// To obtain a text description of the status code, call the rsspecan_error_message function.  To obtain additional information about the error condition, call the rsspecan_GetError function.  To clear the error information from the driver, call the rsspecan_ClearError function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// WARNINGS:
        /// 3FFA2001  The instrument was in an uncalibrated state
        ///           when the measurement was taken.
        /// 3FFA2002  The measurement taken was over the instrument??s
        ///           range.
        ///   
        /// ERRORS:
        /// BFFA2001  The Active Marker is not enabled.
        /// BFFA2002  The Active Marker is not a delta marker.
        /// BFFA2003  The maximum waiting time for this operation was
        ///           exceeded.
        ///           
        /// This instrument driver also returns errors and warnings defined by other sources.  The following table defines the ranges of additional status codes that this driver can return.  The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFA0000 to 3FFA1FFF     IVI    Warnings
        /// 3FFF0000 to 3FFFFFFF     VISA   Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP Driver Warnings
        ///           
        /// BFFA0000 to BFFA1FFF     IVI    Errors
        /// BFFF0000 to BFFFFFFF     VISA   Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP Driver Errors
        /// 
        /// </returns>
        public int GetSEPreamplifierState(int Range, out bool Pre_namplifier)
        {
            ushort Pre_namplifierAsUShort;
            int pInvokeResult = PInvoke.GetSEPreamplifierState(this._handle, Range, out Pre_namplifierAsUShort);
            Pre_namplifier = System.Convert.ToBoolean(Pre_namplifierAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function deletes a range.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]LIST:RANGe&lt;1...20&gt;:DELete
        /// </summary>
        /// <param name="Range">
        /// This control specifies the range that will be deleted.
        /// 
        /// Valid Values: 1 to 20
        /// 
        /// Default Value: 1
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code
        /// either indicates success or describes an error or warning
        /// condition. You examine the status code from each call to an
        /// instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the
        /// rsfsq_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        /// 
        /// Status    Description
        /// -------------------------------------------------
        /// 
        /// BFFC0002  Parameter 2 (Range) out of range.
        /// 
        /// BFFC09F0  Instrument status error
        /// BFFC09F1  Option not installed
        /// BFFC09F2  Conflict with setup parameters
        /// BFFC09F3  Data are not available
        /// BFFC09F4  Unknown option
        /// BFFC09F5  Calibration failed
        /// BFFC09F6  Function does not support FSEx parameter value
        /// BFFC09F7  Overload. Instrument does not return I/Q values
        /// 
        /// This instrument driver also returns errors and warnings defined
        /// by other sources. The following table defines the ranges of
        /// additional status codes that this driver can return. The table
        /// lists the different include files that contain the defined
        /// constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SEDeleteRange(int Range)
        {
            int pInvokeResult = PInvoke.SEDeleteRange(this._handle, Range);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function stores the peak list of selected trace in ASCII format to a file.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// MMEMory:STORe:SPURious &lt;file_name&gt;
        /// </summary>
        /// <param name="File_Name">
        /// Specifies full path of the file, where spurios emissions data will be stored.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code
        /// either indicates success or describes an error or warning
        /// condition. You examine the status code from each call to an
        /// instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the
        /// rsfsq_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        /// 
        /// Status    Description
        /// -------------------------------------------------
        /// 
        /// BFFC0002  Parameter 2 (Window) out of range.
        /// BFFC0003  Parameter 3 (File Name) out of range.
        /// 
        /// BFFC09F0  Instrument status error
        /// BFFC09F1  Option not installed
        /// BFFC09F2  Conflict with setup parameters
        /// BFFC09F3  Data are not available
        /// BFFC09F4  Unknown option
        /// BFFC09F5  Calibration failed
        /// BFFC09F6  Function does not support FSEx parameter value
        /// BFFC09F7  Overload. Instrument does not return I/Q values
        /// 
        /// This instrument driver also returns errors and warnings defined
        /// by other sources. The following table defines the ranges of
        /// additional status codes that this driver can return. The table
        /// lists the different include files that contain the defined
        /// constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int StoreSEToFile(string File_Name)
        {
            int pInvokeResult = PInvoke.StoreSEToFile(this._handle, File_Name);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the spurious emissions peak list.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// TRACe&lt;1|2&gt;:DATA? SPUR
        /// :FORM REAL,32
        /// *CLS
        /// 
        /// 
        /// </summary>
        /// <param name="Trace">
        /// This control sets the trace.
        /// 
        /// Valid Range: 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="No_Of_Values">
        /// This control sets the number of values returned in arrays.
        /// The array size has to be at least equal to this number.
        /// 
        /// Valid Values: &gt; 0
        /// 
        /// Default Value: 10
        /// 
        /// </param>
        /// <param name="Frequency">
        /// Returns an array of frequency list in Hz.
        /// 
        /// </param>
        /// <param name="Level">
        /// Returns an array of level list in dBm.
        /// 
        /// </param>
        /// <param name="Delta_Limit">
        /// Returns an array of delta to limit line values in dB. A delta limit of +200dB indicates no limit check is active.
        /// 
        /// </param>
        /// <param name="Returned_Values">
        /// Returns number of value sets (frequency, level and delta to limit line) read (available to return) from the instrument. Number of values returned in arrays is limited with the parameter No of Values.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation.  The status code  either indicates success or describes an error or warning condition.  You examine the status code from each call to an instrument driver function to determine if an error occurred.
        /// 
        /// To obtain a text description of the status code, call the rsspecan_error_message function.  To obtain additional information about the error condition, call the rsspecan_GetError function.  To clear the error information from the driver, call the rsspecan_ClearError function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// WARNINGS:
        /// 3FFA2001  The instrument was in an uncalibrated state
        ///           when the measurement was taken.
        /// 3FFA2002  The measurement taken was over the instrument??s
        ///           range.
        ///   
        /// ERRORS:
        /// BFFA2001  The Active Marker is not enabled.
        /// BFFA2002  The Active Marker is not a delta marker.
        /// BFFA2003  The maximum waiting time for this operation was
        ///           exceeded.
        ///           
        /// This instrument driver also returns errors and warnings defined by other sources.  The following table defines the ranges of additional status codes that this driver can return.  The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFA0000 to 3FFA1FFF     IVI    Warnings
        /// 3FFF0000 to 3FFFFFFF     VISA   Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP Driver Warnings
        ///           
        /// BFFA0000 to BFFA1FFF     IVI    Errors
        /// BFFF0000 to BFFFFFFF     VISA   Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP Driver Errors
        /// 
        /// </returns>
        public int ReadSEPeakList(int Trace, int No_Of_Values, double[] Frequency, double[] Level, double[] Delta_Limit, out int Returned_Values)
        {
            int pInvokeResult = PInvoke.ReadSEPeakList(this._handle, Trace, No_Of_Values, Frequency, Level, Delta_Limit, out Returned_Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures start frequency of the sweep range.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]LIST:RANGe&lt;1...20&gt;[:FREQuency]:STARt
        /// </summary>
        /// <param name="Range">
        /// Sets the configured range.
        /// 
        /// Valid Values:
        /// 1 to 20
        /// 
        /// Default Value:
        /// 1
        /// </param>
        /// <param name="Start_Frequency">
        /// Sets the start frequency of a range in the spurious measurement. 
        /// 
        /// Units: Hz
        /// 
        /// Valid Values: no range checking
        /// 
        /// Default Value: 20.0
        /// </param>
        /// <returns>
        /// Returns the status code of this operation.  The status code  either indicates success or describes an error or warning condition.  You examine the status code from each call to an instrument driver function to determine if an error occurred.
        /// 
        /// To obtain a text description of the status code, call the rsspecan_error_message function.  To obtain additional information about the error condition, call the rsspecan_GetError function.  To clear the error information from the driver, call the rsspecan_ClearError function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// WARNINGS:
        /// 3FFA2001  The instrument was in an uncalibrated state
        ///           when the measurement was taken.
        /// 3FFA2002  The measurement taken was over the instrument??s
        ///           range.
        ///   
        /// ERRORS:
        /// BFFA2001  The Active Marker is not enabled.
        /// BFFA2002  The Active Marker is not a delta marker.
        /// BFFA2003  The maximum waiting time for this operation was
        ///           exceeded.
        ///           
        /// This instrument driver also returns errors and warnings defined by other sources.  The following table defines the ranges of additional status codes that this driver can return.  The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFA0000 to 3FFA1FFF     IVI    Warnings
        /// 3FFF0000 to 3FFFFFFF     VISA   Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP Driver Warnings
        ///           
        /// BFFA0000 to BFFA1FFF     IVI    Errors
        /// BFFF0000 to BFFFFFFF     VISA   Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP Driver Errors
        /// 
        /// </returns>
        public int SetSESweepStart(int Range, double Start_Frequency)
        {
            int pInvokeResult = PInvoke.SetSESweepStart(this._handle, Range, Start_Frequency);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries start frequency of the sweep range.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]LIST:RANGe&lt;1...20&gt;[:FREQuency]:STARt?
        /// </summary>
        /// <param name="Range">
        /// Sets the configured range.
        /// 
        /// Valid Values:
        /// 1 to 20
        /// 
        /// Default Value:
        /// 1
        /// </param>
        /// <param name="Start_Frequency">
        /// Returns the start frequency of a range in the spurious measurement. 
        /// 
        /// Units: Hz
        /// </param>
        /// <returns>
        /// Returns the status code of this operation.  The status code  either indicates success or describes an error or warning condition.  You examine the status code from each call to an instrument driver function to determine if an error occurred.
        /// 
        /// To obtain a text description of the status code, call the rsspecan_error_message function.  To obtain additional information about the error condition, call the rsspecan_GetError function.  To clear the error information from the driver, call the rsspecan_ClearError function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// WARNINGS:
        /// 3FFA2001  The instrument was in an uncalibrated state
        ///           when the measurement was taken.
        /// 3FFA2002  The measurement taken was over the instrument??s
        ///           range.
        ///   
        /// ERRORS:
        /// BFFA2001  The Active Marker is not enabled.
        /// BFFA2002  The Active Marker is not a delta marker.
        /// BFFA2003  The maximum waiting time for this operation was
        ///           exceeded.
        ///           
        /// This instrument driver also returns errors and warnings defined by other sources.  The following table defines the ranges of additional status codes that this driver can return.  The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFA0000 to 3FFA1FFF     IVI    Warnings
        /// 3FFF0000 to 3FFFFFFF     VISA   Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP Driver Warnings
        ///           
        /// BFFA0000 to BFFA1FFF     IVI    Errors
        /// BFFF0000 to BFFFFFFF     VISA   Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP Driver Errors
        /// 
        /// </returns>
        public int GetSESweepStart(int Range, out double Start_Frequency)
        {
            int pInvokeResult = PInvoke.GetSESweepStart(this._handle, Range, out Start_Frequency);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets stop frequency of the sweep range.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]LIST:RANGe&lt;1...20&gt;[:FREQuency]:STOP
        /// </summary>
        /// <param name="Range">
        /// Sets the configured range.
        /// 
        /// Valid Values:
        /// 1 to 20
        /// 
        /// Default Value:
        /// 1
        /// </param>
        /// <param name="Stop_Frequency">
        /// Sets the stop frequency of a range in the spurious measurement. 
        /// 
        /// Units: Hz
        /// 
        /// Valid Values: no range checking
        /// 
        /// Default Value: 1.0e+9
        /// </param>
        /// <returns>
        /// Returns the status code of this operation.  The status code  either indicates success or describes an error or warning condition.  You examine the status code from each call to an instrument driver function to determine if an error occurred.
        /// 
        /// To obtain a text description of the status code, call the rsspecan_error_message function.  To obtain additional information about the error condition, call the rsspecan_GetError function.  To clear the error information from the driver, call the rsspecan_ClearError function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// WARNINGS:
        /// 3FFA2001  The instrument was in an uncalibrated state
        ///           when the measurement was taken.
        /// 3FFA2002  The measurement taken was over the instrument??s
        ///           range.
        ///   
        /// ERRORS:
        /// BFFA2001  The Active Marker is not enabled.
        /// BFFA2002  The Active Marker is not a delta marker.
        /// BFFA2003  The maximum waiting time for this operation was
        ///           exceeded.
        ///           
        /// This instrument driver also returns errors and warnings defined by other sources.  The following table defines the ranges of additional status codes that this driver can return.  The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFA0000 to 3FFA1FFF     IVI    Warnings
        /// 3FFF0000 to 3FFFFFFF     VISA   Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP Driver Warnings
        ///           
        /// BFFA0000 to BFFA1FFF     IVI    Errors
        /// BFFF0000 to BFFFFFFF     VISA   Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP Driver Errors
        /// 
        /// </returns>
        public int SetSESweepStop(int Range, double Stop_Frequency)
        {
            int pInvokeResult = PInvoke.SetSESweepStop(this._handle, Range, Stop_Frequency);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries stop frequency of the sweep range.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]LIST:RANGe&lt;1...20&gt;[:FREQuency]:STOP?
        /// </summary>
        /// <param name="Range">
        /// Sets the configured range.
        /// 
        /// Valid Values:
        /// 1 to 20
        /// 
        /// Default Value:
        /// 1
        /// </param>
        /// <param name="Stop_Frequency">
        /// Returns the stop frequency of a range in the spurious measurement. 
        /// 
        /// Units: Hz
        /// </param>
        /// <returns>
        /// Returns the status code of this operation.  The status code  either indicates success or describes an error or warning condition.  You examine the status code from each call to an instrument driver function to determine if an error occurred.
        /// 
        /// To obtain a text description of the status code, call the rsspecan_error_message function.  To obtain additional information about the error condition, call the rsspecan_GetError function.  To clear the error information from the driver, call the rsspecan_ClearError function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// WARNINGS:
        /// 3FFA2001  The instrument was in an uncalibrated state
        ///           when the measurement was taken.
        /// 3FFA2002  The measurement taken was over the instrument??s
        ///           range.
        ///   
        /// ERRORS:
        /// BFFA2001  The Active Marker is not enabled.
        /// BFFA2002  The Active Marker is not a delta marker.
        /// BFFA2003  The maximum waiting time for this operation was
        ///           exceeded.
        ///           
        /// This instrument driver also returns errors and warnings defined by other sources.  The following table defines the ranges of additional status codes that this driver can return.  The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFA0000 to 3FFA1FFF     IVI    Warnings
        /// 3FFF0000 to 3FFFFFFF     VISA   Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP Driver Warnings
        ///           
        /// BFFA0000 to BFFA1FFF     IVI    Errors
        /// BFFF0000 to BFFFFFFF     VISA   Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP Driver Errors
        /// 
        /// </returns>
        public int GetSESweepStop(int Range, out double Stop_Frequency)
        {
            int pInvokeResult = PInvoke.GetSESweepStop(this._handle, Range, out Stop_Frequency);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the filter in the spurious measurement.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]LIST:RANGe&lt;1...20&gt;:FILTer:TYPE NORMal | CHANnel | RRC | P5 | NOISe | PULSe
        /// </summary>
        /// <param name="Range">
        /// Sets the configured range.
        /// 
        /// Valid Values:
        /// 1 to 20
        /// 
        /// Default Value:
        /// 1
        /// </param>
        /// <param name="Filter">
        /// This control sets the filter in the spurious measurement. 
        /// 
        /// Valid Values:
        /// RSESCI_LIST_FTYPE_NORM  (0) - Normal
        /// RSESCI_LIST_FTYPE_CHAN  (1) - Channel
        /// RSESCI_LIST_FTYPE_RRC   (2) - RRC
        /// RSESCI_LIST_FTYPE_P5    (3) - P5
        /// RSESCI_LIST_FTYPE_NOISE (4) - Noise
        /// RSESCI_LIST_FTYPE_PULSE (5) - Pulse
        /// 
        /// Default Value: RSESCI_LIST_FTYPE_NORM (0)
        /// 
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation.  The status code  either indicates success or describes an error or warning condition.  You examine the status code from each call to an instrument driver function to determine if an error occurred.
        /// 
        /// To obtain a text description of the status code, call the rsspecan_error_message function.  To obtain additional information about the error condition, call the rsspecan_GetError function.  To clear the error information from the driver, call the rsspecan_ClearError function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// WARNINGS:
        /// 3FFA2001  The instrument was in an uncalibrated state
        ///           when the measurement was taken.
        /// 3FFA2002  The measurement taken was over the instrument??s
        ///           range.
        ///   
        /// ERRORS:
        /// BFFA2001  The Active Marker is not enabled.
        /// BFFA2002  The Active Marker is not a delta marker.
        /// BFFA2003  The maximum waiting time for this operation was
        ///           exceeded.
        ///           
        /// This instrument driver also returns errors and warnings defined by other sources.  The following table defines the ranges of additional status codes that this driver can return.  The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFA0000 to 3FFA1FFF     IVI    Warnings
        /// 3FFF0000 to 3FFFFFFF     VISA   Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP Driver Warnings
        ///           
        /// BFFA0000 to BFFA1FFF     IVI    Errors
        /// BFFF0000 to BFFFFFFF     VISA   Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP Driver Errors
        /// 
        /// </returns>
        public int SetSESweepFilter(int Range, int Filter)
        {
            int pInvokeResult = PInvoke.SetSESweepFilter(this._handle, Range, Filter);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the filter in the spurious measurement.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]LIST:RANGe&lt;1...20&gt;:FILTer:TYPE?
        /// </summary>
        /// <param name="Range">
        /// Sets the configured range.
        /// 
        /// Valid Values:
        /// 1 to 20
        /// 
        /// Default Value:
        /// 1
        /// </param>
        /// <param name="Filter">
        /// This control returns the filter in the spurious measurement. 
        /// 
        /// Valid Values:
        /// RSESCI_LIST_FTYPE_NORM  (0) - Normal
        /// RSESCI_LIST_FTYPE_CHAN  (1) - Channel
        /// RSESCI_LIST_FTYPE_RRC   (2) - RRC
        /// RSESCI_LIST_FTYPE_P5    (3) - P5
        /// RSESCI_LIST_FTYPE_NOISE (4) - Noise
        /// RSESCI_LIST_FTYPE_PULSE (5) - Pulse
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation.  The status code  either indicates success or describes an error or warning condition.  You examine the status code from each call to an instrument driver function to determine if an error occurred.
        /// 
        /// To obtain a text description of the status code, call the rsspecan_error_message function.  To obtain additional information about the error condition, call the rsspecan_GetError function.  To clear the error information from the driver, call the rsspecan_ClearError function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// WARNINGS:
        /// 3FFA2001  The instrument was in an uncalibrated state
        ///           when the measurement was taken.
        /// 3FFA2002  The measurement taken was over the instrument??s
        ///           range.
        ///   
        /// ERRORS:
        /// BFFA2001  The Active Marker is not enabled.
        /// BFFA2002  The Active Marker is not a delta marker.
        /// BFFA2003  The maximum waiting time for this operation was
        ///           exceeded.
        ///           
        /// This instrument driver also returns errors and warnings defined by other sources.  The following table defines the ranges of additional status codes that this driver can return.  The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFA0000 to 3FFA1FFF     IVI    Warnings
        /// 3FFF0000 to 3FFFFFFF     VISA   Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP Driver Warnings
        ///           
        /// BFFA0000 to BFFA1FFF     IVI    Errors
        /// BFFF0000 to BFFFFFFF     VISA   Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP Driver Errors
        /// 
        /// </returns>
        public int GetSESweepFilter(int Range, out int Filter)
        {
            int pInvokeResult = PInvoke.GetSESweepFilter(this._handle, Range, out Filter);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the the resolution bandwidth.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]LIST:RANGe&lt;1...20&gt;:BANDwidth[:RESolution]
        /// 
        /// </summary>
        /// <param name="Range">
        /// Sets the configured range.
        /// 
        /// Valid Values:
        /// 1 to 20
        /// 
        /// Default Value:
        /// 1
        /// </param>
        /// <param name="Resolution_Bandwidth">
        /// Sets the the resolution bandwidth. 
        /// 
        /// Valid Range: no range checking
        /// 
        /// Default Value: 10.0e3 Hz
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation.  The status code  either indicates success or describes an error or warning condition.  You examine the status code from each call to an instrument driver function to determine if an error occurred.
        /// 
        /// To obtain a text description of the status code, call the rsspecan_error_message function.  To obtain additional information about the error condition, call the rsspecan_GetError function.  To clear the error information from the driver, call the rsspecan_ClearError function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// WARNINGS:
        /// 3FFA2001  The instrument was in an uncalibrated state
        ///           when the measurement was taken.
        /// 3FFA2002  The measurement taken was over the instrument??s
        ///           range.
        ///   
        /// ERRORS:
        /// BFFA2001  The Active Marker is not enabled.
        /// BFFA2002  The Active Marker is not a delta marker.
        /// BFFA2003  The maximum waiting time for this operation was
        ///           exceeded.
        ///           
        /// This instrument driver also returns errors and warnings defined by other sources.  The following table defines the ranges of additional status codes that this driver can return.  The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFA0000 to 3FFA1FFF     IVI    Warnings
        /// 3FFF0000 to 3FFFFFFF     VISA   Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP Driver Warnings
        ///           
        /// BFFA0000 to BFFA1FFF     IVI    Errors
        /// BFFF0000 to BFFFFFFF     VISA   Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP Driver Errors
        /// 
        /// </returns>
        public int SetSESweepRBW(int Range, double Resolution_Bandwidth)
        {
            int pInvokeResult = PInvoke.SetSESweepRBW(this._handle, Range, Resolution_Bandwidth);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the the resolution bandwidth.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]LIST:RANGe&lt;1...20&gt;:BANDwidth[:RESolution]?
        /// </summary>
        /// <param name="Range">
        /// Sets the configured range.
        /// 
        /// Valid Values:
        /// 1 to 20
        /// 
        /// Default Value:
        /// 1
        /// </param>
        /// <param name="Resolution_Bandwidth">
        /// Returns the the resolution bandwidth.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation.  The status code  either indicates success or describes an error or warning condition.  You examine the status code from each call to an instrument driver function to determine if an error occurred.
        /// 
        /// To obtain a text description of the status code, call the rsspecan_error_message function.  To obtain additional information about the error condition, call the rsspecan_GetError function.  To clear the error information from the driver, call the rsspecan_ClearError function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// WARNINGS:
        /// 3FFA2001  The instrument was in an uncalibrated state
        ///           when the measurement was taken.
        /// 3FFA2002  The measurement taken was over the instrument??s
        ///           range.
        ///   
        /// ERRORS:
        /// BFFA2001  The Active Marker is not enabled.
        /// BFFA2002  The Active Marker is not a delta marker.
        /// BFFA2003  The maximum waiting time for this operation was
        ///           exceeded.
        ///           
        /// This instrument driver also returns errors and warnings defined by other sources.  The following table defines the ranges of additional status codes that this driver can return.  The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFA0000 to 3FFA1FFF     IVI    Warnings
        /// 3FFF0000 to 3FFFFFFF     VISA   Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP Driver Warnings
        ///           
        /// BFFA0000 to BFFA1FFF     IVI    Errors
        /// BFFF0000 to BFFFFFFF     VISA   Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP Driver Errors
        /// 
        /// </returns>
        public int GetSESweepRBW(int Range, out double Resolution_Bandwidth)
        {
            int pInvokeResult = PInvoke.GetSESweepRBW(this._handle, Range, out Resolution_Bandwidth);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the the video bandwidth.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]LIST:RANGe&lt;1...20&gt;:BANDwidth:VIDeo
        /// </summary>
        /// <param name="Range">
        /// Sets the configured range.
        /// 
        /// Valid Values:
        /// 1 to 20
        /// 
        /// Default Value:
        /// 1
        /// </param>
        /// <param name="Video_Bandwidth">
        /// Sets the the video bandwidth. 
        /// 
        /// Valid Range: no range checking
        /// 
        /// Default Value: 30.0e3 Hz
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation.  The status code  either indicates success or describes an error or warning condition.  You examine the status code from each call to an instrument driver function to determine if an error occurred.
        /// 
        /// To obtain a text description of the status code, call the rsspecan_error_message function.  To obtain additional information about the error condition, call the rsspecan_GetError function.  To clear the error information from the driver, call the rsspecan_ClearError function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// WARNINGS:
        /// 3FFA2001  The instrument was in an uncalibrated state
        ///           when the measurement was taken.
        /// 3FFA2002  The measurement taken was over the instrument??s
        ///           range.
        ///   
        /// ERRORS:
        /// BFFA2001  The Active Marker is not enabled.
        /// BFFA2002  The Active Marker is not a delta marker.
        /// BFFA2003  The maximum waiting time for this operation was
        ///           exceeded.
        ///           
        /// This instrument driver also returns errors and warnings defined by other sources.  The following table defines the ranges of additional status codes that this driver can return.  The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFA0000 to 3FFA1FFF     IVI    Warnings
        /// 3FFF0000 to 3FFFFFFF     VISA   Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP Driver Warnings
        ///           
        /// BFFA0000 to BFFA1FFF     IVI    Errors
        /// BFFF0000 to BFFFFFFF     VISA   Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP Driver Errors
        /// 
        /// </returns>
        public int SetSESweepVBW(int Range, double Video_Bandwidth)
        {
            int pInvokeResult = PInvoke.SetSESweepVBW(this._handle, Range, Video_Bandwidth);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the the video bandwidth.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]LIST:RANGe&lt;1...20&gt;:BANDwidth:VIDeo?
        /// </summary>
        /// <param name="Range">
        /// Sets the configured range.
        /// 
        /// Valid Values:
        /// 1 to 20
        /// 
        /// Default Value:
        /// 1
        /// </param>
        /// <param name="Video_Bandwidth">
        /// Returns the the video bandwidth.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation.  The status code  either indicates success or describes an error or warning condition.  You examine the status code from each call to an instrument driver function to determine if an error occurred.
        /// 
        /// To obtain a text description of the status code, call the rsspecan_error_message function.  To obtain additional information about the error condition, call the rsspecan_GetError function.  To clear the error information from the driver, call the rsspecan_ClearError function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// WARNINGS:
        /// 3FFA2001  The instrument was in an uncalibrated state
        ///           when the measurement was taken.
        /// 3FFA2002  The measurement taken was over the instrument??s
        ///           range.
        ///   
        /// ERRORS:
        /// BFFA2001  The Active Marker is not enabled.
        /// BFFA2002  The Active Marker is not a delta marker.
        /// BFFA2003  The maximum waiting time for this operation was
        ///           exceeded.
        ///           
        /// This instrument driver also returns errors and warnings defined by other sources.  The following table defines the ranges of additional status codes that this driver can return.  The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFA0000 to 3FFA1FFF     IVI    Warnings
        /// 3FFF0000 to 3FFFFFFF     VISA   Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP Driver Warnings
        ///           
        /// BFFA0000 to BFFA1FFF     IVI    Errors
        /// BFFF0000 to BFFFFFFF     VISA   Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP Driver Errors
        /// 
        /// </returns>
        public int GetSESweepVBW(int Range, out double Video_Bandwidth)
        {
            int pInvokeResult = PInvoke.GetSESweepVBW(this._handle, Range, out Video_Bandwidth);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the automatic coupling of the sweep duration on the frequency span and the bandwidth settings in a spurious measurement range.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]LIST:RANGe&lt;1...20&gt;:SWEep:TIME:AUTO ON | OFF
        /// </summary>
        /// <param name="Range">
        /// Sets the configured range.
        /// 
        /// Valid Values:
        /// 1 to 20
        /// 
        /// Default Value:
        /// 1
        /// </param>
        /// <param name="Sweep_Time_Auto">
        /// Sets the automatic coupling of the sweep duration on the frequency span and the bandwidth settings in a spurious measurement range.
        /// 
        /// Valid Range:
        /// VI_FALSE (0) - Off 
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_TRUE (1)
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation.  The status code  either indicates success or describes an error or warning condition.  You examine the status code from each call to an instrument driver function to determine if an error occurred.
        /// 
        /// To obtain a text description of the status code, call the rsspecan_error_message function.  To obtain additional information about the error condition, call the rsspecan_GetError function.  To clear the error information from the driver, call the rsspecan_ClearError function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// WARNINGS:
        /// 3FFA2001  The instrument was in an uncalibrated state
        ///           when the measurement was taken.
        /// 3FFA2002  The measurement taken was over the instrument??s
        ///           range.
        ///   
        /// ERRORS:
        /// BFFA2001  The Active Marker is not enabled.
        /// BFFA2002  The Active Marker is not a delta marker.
        /// BFFA2003  The maximum waiting time for this operation was
        ///           exceeded.
        ///           
        /// This instrument driver also returns errors and warnings defined by other sources.  The following table defines the ranges of additional status codes that this driver can return.  The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFA0000 to 3FFA1FFF     IVI    Warnings
        /// 3FFF0000 to 3FFFFFFF     VISA   Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP Driver Warnings
        ///           
        /// BFFA0000 to BFFA1FFF     IVI    Errors
        /// BFFF0000 to BFFFFFFF     VISA   Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP Driver Errors
        /// 
        /// </returns>
        public int SetSESweepTimeAuto(int Range, bool Sweep_Time_Auto)
        {
            int pInvokeResult = PInvoke.SetSESweepTimeAuto(this._handle, Range, System.Convert.ToUInt16(Sweep_Time_Auto));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the state of automatic coupling of the sweep duration on the frequency span and the bandwidth settings in a spurious measurement range.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]LIST:RANGe&lt;1...20&gt;:SWEep:TIME:AUTO?
        /// </summary>
        /// <param name="Range">
        /// Sets the configured range.
        /// 
        /// Valid Values:
        /// 1 to 20
        /// 
        /// Default Value:
        /// 1
        /// </param>
        /// <param name="Sweep_Time_Auto">
        /// Returns the state of automatic coupling of the sweep duration on the frequency span and the bandwidth settings in a spurious measurement range.
        /// 
        /// Valid Range:
        /// VI_FALSE (0) - Off 
        /// VI_TRUE  (1) - On
        /// </param>
        /// <returns>
        /// Returns the status code of this operation.  The status code  either indicates success or describes an error or warning condition.  You examine the status code from each call to an instrument driver function to determine if an error occurred.
        /// 
        /// To obtain a text description of the status code, call the rsspecan_error_message function.  To obtain additional information about the error condition, call the rsspecan_GetError function.  To clear the error information from the driver, call the rsspecan_ClearError function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// WARNINGS:
        /// 3FFA2001  The instrument was in an uncalibrated state
        ///           when the measurement was taken.
        /// 3FFA2002  The measurement taken was over the instrument??s
        ///           range.
        ///   
        /// ERRORS:
        /// BFFA2001  The Active Marker is not enabled.
        /// BFFA2002  The Active Marker is not a delta marker.
        /// BFFA2003  The maximum waiting time for this operation was
        ///           exceeded.
        ///           
        /// This instrument driver also returns errors and warnings defined by other sources.  The following table defines the ranges of additional status codes that this driver can return.  The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFA0000 to 3FFA1FFF     IVI    Warnings
        /// 3FFF0000 to 3FFFFFFF     VISA   Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP Driver Warnings
        ///           
        /// BFFA0000 to BFFA1FFF     IVI    Errors
        /// BFFF0000 to BFFFFFFF     VISA   Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP Driver Errors
        /// 
        /// </returns>
        public int GetSESweepTimeAuto(int Range, out bool Sweep_Time_Auto)
        {
            ushort Sweep_Time_AutoAsUShort;
            int pInvokeResult = PInvoke.GetSESweepTimeAuto(this._handle, Range, out Sweep_Time_AutoAsUShort);
            Sweep_Time_Auto = System.Convert.ToBoolean(Sweep_Time_AutoAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the sweep duration of a range in the spurious measurement.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with R&amp;S ESU instrument.
        /// (2) This function has effect only if Sweep Time Auto is set Off
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]LIST:RANGe&lt;1...20&gt;:SWEep:TIME
        /// </summary>
        /// <param name="Range">
        /// Sets the configured range.
        /// 
        /// Valid Values:
        /// 1 to 20
        /// 
        /// Default Value:
        /// 1
        /// </param>
        /// <param name="Sweep_Time">
        /// Sets the sweep duration of a range in the spurious measurement. 
        /// 
        /// Valid Range: 0.001 s to 16000.0 s
        ///  
        /// Default Value: 0.1 s
        /// 
        /// Note:
        /// (1) This control has effect only if Sweep Time Auto is set Off
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation.  The status code  either indicates success or describes an error or warning condition.  You examine the status code from each call to an instrument driver function to determine if an error occurred.
        /// 
        /// To obtain a text description of the status code, call the rsspecan_error_message function.  To obtain additional information about the error condition, call the rsspecan_GetError function.  To clear the error information from the driver, call the rsspecan_ClearError function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// WARNINGS:
        /// 3FFA2001  The instrument was in an uncalibrated state
        ///           when the measurement was taken.
        /// 3FFA2002  The measurement taken was over the instrument??s
        ///           range.
        ///   
        /// ERRORS:
        /// BFFA2001  The Active Marker is not enabled.
        /// BFFA2002  The Active Marker is not a delta marker.
        /// BFFA2003  The maximum waiting time for this operation was
        ///           exceeded.
        ///           
        /// This instrument driver also returns errors and warnings defined by other sources.  The following table defines the ranges of additional status codes that this driver can return.  The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFA0000 to 3FFA1FFF     IVI    Warnings
        /// 3FFF0000 to 3FFFFFFF     VISA   Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP Driver Warnings
        ///           
        /// BFFA0000 to BFFA1FFF     IVI    Errors
        /// BFFF0000 to BFFFFFFF     VISA   Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP Driver Errors
        /// 
        /// </returns>
        public int SetSESweepTime(int Range, double Sweep_Time)
        {
            int pInvokeResult = PInvoke.SetSESweepTime(this._handle, Range, Sweep_Time);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the sweep duration of a range in the spurious measurement.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]LIST:RANGe&lt;1...20&gt;:SWEep:TIME?
        /// </summary>
        /// <param name="Range">
        /// Sets the configured range.
        /// 
        /// Valid Values:
        /// 1 to 20
        /// 
        /// Default Value:
        /// 1
        /// </param>
        /// <param name="Sweep_Time">
        /// Returns the sweep duration of a range in the spurious measurement. 
        /// 
        /// Valid Range: 0.001 s to 16000.0 s
        /// </param>
        /// <returns>
        /// Returns the status code of this operation.  The status code  either indicates success or describes an error or warning condition.  You examine the status code from each call to an instrument driver function to determine if an error occurred.
        /// 
        /// To obtain a text description of the status code, call the rsspecan_error_message function.  To obtain additional information about the error condition, call the rsspecan_GetError function.  To clear the error information from the driver, call the rsspecan_ClearError function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// WARNINGS:
        /// 3FFA2001  The instrument was in an uncalibrated state
        ///           when the measurement was taken.
        /// 3FFA2002  The measurement taken was over the instrument??s
        ///           range.
        ///   
        /// ERRORS:
        /// BFFA2001  The Active Marker is not enabled.
        /// BFFA2002  The Active Marker is not a delta marker.
        /// BFFA2003  The maximum waiting time for this operation was
        ///           exceeded.
        ///           
        /// This instrument driver also returns errors and warnings defined by other sources.  The following table defines the ranges of additional status codes that this driver can return.  The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFA0000 to 3FFA1FFF     IVI    Warnings
        /// 3FFF0000 to 3FFFFFFF     VISA   Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP Driver Warnings
        ///           
        /// BFFA0000 to BFFA1FFF     IVI    Errors
        /// BFFF0000 to BFFFFFFF     VISA   Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP Driver Errors
        /// 
        /// </returns>
        public int GetSESweepTime(int Range, out double Sweep_Time)
        {
            int pInvokeResult = PInvoke.GetSESweepTime(this._handle, Range, out Sweep_Time);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the detector in the spurious measurement.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]LIST:RANGe&lt;1...20&gt;:DETector APEak | NEGative | POSitive | SAMPle | RMS | AVERage
        /// </summary>
        /// <param name="Range">
        /// Sets the configured range.
        /// 
        /// Valid Values:
        /// 1 to 20
        /// 
        /// Default Value:
        /// 1
        /// </param>
        /// <param name="Detector">
        /// This control sets the detector in the spurious measurement.
        /// 
        /// Valid Values:
        /// RSESCI_TRACEDET_APEAK  (0) - Auto Peak
        /// RSESCI_TRACEDET_NEG    (1) - Negative
        /// RSESCI_TRACEDET_POS    (2) - Positive
        /// RSESCI_TRACEDET_SAMPLE (3) - Sample
        /// RSESCI_TRACEDET_RMS    (4) - RMS
        /// RSESCI_TRACEDET_AVG    (5) - Average
        /// 
        /// Default Value: RSESCI_TRACEDET_POS    (2)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation.  The status code  either indicates success or describes an error or warning condition.  You examine the status code from each call to an instrument driver function to determine if an error occurred.
        /// 
        /// To obtain a text description of the status code, call the rsspecan_error_message function.  To obtain additional information about the error condition, call the rsspecan_GetError function.  To clear the error information from the driver, call the rsspecan_ClearError function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// WARNINGS:
        /// 3FFA2001  The instrument was in an uncalibrated state
        ///           when the measurement was taken.
        /// 3FFA2002  The measurement taken was over the instrument??s
        ///           range.
        ///   
        /// ERRORS:
        /// BFFA2001  The Active Marker is not enabled.
        /// BFFA2002  The Active Marker is not a delta marker.
        /// BFFA2003  The maximum waiting time for this operation was
        ///           exceeded.
        ///           
        /// This instrument driver also returns errors and warnings defined by other sources.  The following table defines the ranges of additional status codes that this driver can return.  The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFA0000 to 3FFA1FFF     IVI    Warnings
        /// 3FFF0000 to 3FFFFFFF     VISA   Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP Driver Warnings
        ///           
        /// BFFA0000 to BFFA1FFF     IVI    Errors
        /// BFFF0000 to BFFFFFFF     VISA   Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP Driver Errors
        /// 
        /// </returns>
        public int SetSESweepDetector(int Range, int Detector)
        {
            int pInvokeResult = PInvoke.SetSESweepDetector(this._handle, Range, Detector);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the detector in the spurious measurement.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]LIST:RANGe&lt;1...20&gt;:DETector?
        /// </summary>
        /// <param name="Range">
        /// Sets the configured range.
        /// 
        /// Valid Values:
        /// 1 to 20
        /// 
        /// Default Value:
        /// 1
        /// </param>
        /// <param name="Detector">
        /// This control returns the detector in the spurious measurement.
        /// 
        /// Valid Values:
        /// RSESCI_TRACEDET_APEAK  (0) - Auto Peak
        /// RSESCI_TRACEDET_NEG    (1) - Negative
        /// RSESCI_TRACEDET_POS    (2) - Positive
        /// RSESCI_TRACEDET_SAMPLE (3) - Sample
        /// RSESCI_TRACEDET_RMS    (4) - RMS
        /// RSESCI_TRACEDET_AVG    (5) - Average
        /// </param>
        /// <returns>
        /// Returns the status code of this operation.  The status code  either indicates success or describes an error or warning condition.  You examine the status code from each call to an instrument driver function to determine if an error occurred.
        /// 
        /// To obtain a text description of the status code, call the rsspecan_error_message function.  To obtain additional information about the error condition, call the rsspecan_GetError function.  To clear the error information from the driver, call the rsspecan_ClearError function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// WARNINGS:
        /// 3FFA2001  The instrument was in an uncalibrated state
        ///           when the measurement was taken.
        /// 3FFA2002  The measurement taken was over the instrument??s
        ///           range.
        ///   
        /// ERRORS:
        /// BFFA2001  The Active Marker is not enabled.
        /// BFFA2002  The Active Marker is not a delta marker.
        /// BFFA2003  The maximum waiting time for this operation was
        ///           exceeded.
        ///           
        /// This instrument driver also returns errors and warnings defined by other sources.  The following table defines the ranges of additional status codes that this driver can return.  The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFA0000 to 3FFA1FFF     IVI    Warnings
        /// 3FFF0000 to 3FFFFFFF     VISA   Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP Driver Warnings
        ///           
        /// BFFA0000 to BFFA1FFF     IVI    Errors
        /// BFFF0000 to BFFFFFFF     VISA   Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP Driver Errors
        /// 
        /// </returns>
        public int GetSESweepDetector(int Range, out int Detector)
        {
            int pInvokeResult = PInvoke.GetSESweepDetector(this._handle, Range, out Detector);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function specifies the reference level of a range in the spurious measurement.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]LIST:RANGe&lt;1...20&gt;:RLEVel
        /// </summary>
        /// <param name="Range">
        /// Sets the configured range.
        /// 
        /// Valid Values:
        /// 1 to 20
        /// 
        /// Default Value:
        /// 1
        /// </param>
        /// <param name="Ref_Level_Value">
        /// Specifies the reference level of a range in the spurious measurement.
        /// 
        /// Valid Range: -130.0 dBm to 30.0 dBm
        /// 
        /// Default Value: -30.0 dBm
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation.  The status code  either indicates success or describes an error or warning condition.  You examine the status code from each call to an instrument driver function to determine if an error occurred.
        /// 
        /// To obtain a text description of the status code, call the rsspecan_error_message function.  To obtain additional information about the error condition, call the rsspecan_GetError function.  To clear the error information from the driver, call the rsspecan_ClearError function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// WARNINGS:
        /// 3FFA2001  The instrument was in an uncalibrated state
        ///           when the measurement was taken.
        /// 3FFA2002  The measurement taken was over the instrument??s
        ///           range.
        ///   
        /// ERRORS:
        /// BFFA2001  The Active Marker is not enabled.
        /// BFFA2002  The Active Marker is not a delta marker.
        /// BFFA2003  The maximum waiting time for this operation was
        ///           exceeded.
        ///           
        /// This instrument driver also returns errors and warnings defined by other sources.  The following table defines the ranges of additional status codes that this driver can return.  The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFA0000 to 3FFA1FFF     IVI    Warnings
        /// 3FFF0000 to 3FFFFFFF     VISA   Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP Driver Warnings
        ///           
        /// BFFA0000 to BFFA1FFF     IVI    Errors
        /// BFFF0000 to BFFFFFFF     VISA   Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP Driver Errors
        /// 
        /// </returns>
        public int SetSESweepRefLevel(int Range, double Ref_Level_Value)
        {
            int pInvokeResult = PInvoke.SetSESweepRefLevel(this._handle, Range, Ref_Level_Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the reference level of a range in the spurious measurement.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]LIST:RANGe&lt;1...20&gt;:RLEVel?
        /// </summary>
        /// <param name="Range">
        /// Sets the configured range.
        /// 
        /// Valid Values:
        /// 1 to 20
        /// 
        /// Default Value:
        /// 1
        /// </param>
        /// <param name="Ref_Level_Value">
        /// Returns the reference level of a range in the spurious measurement.
        /// 
        /// Valid Range: -130.0 dBm to 30.0 dBm
        /// </param>
        /// <returns>
        /// Returns the status code of this operation.  The status code  either indicates success or describes an error or warning condition.  You examine the status code from each call to an instrument driver function to determine if an error occurred.
        /// 
        /// To obtain a text description of the status code, call the rsspecan_error_message function.  To obtain additional information about the error condition, call the rsspecan_GetError function.  To clear the error information from the driver, call the rsspecan_ClearError function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// WARNINGS:
        /// 3FFA2001  The instrument was in an uncalibrated state
        ///           when the measurement was taken.
        /// 3FFA2002  The measurement taken was over the instrument??s
        ///           range.
        ///   
        /// ERRORS:
        /// BFFA2001  The Active Marker is not enabled.
        /// BFFA2002  The Active Marker is not a delta marker.
        /// BFFA2003  The maximum waiting time for this operation was
        ///           exceeded.
        ///           
        /// This instrument driver also returns errors and warnings defined by other sources.  The following table defines the ranges of additional status codes that this driver can return.  The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFA0000 to 3FFA1FFF     IVI    Warnings
        /// 3FFF0000 to 3FFFFFFF     VISA   Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP Driver Warnings
        ///           
        /// BFFA0000 to BFFA1FFF     IVI    Errors
        /// BFFF0000 to BFFFFFFF     VISA   Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP Driver Errors
        /// 
        /// </returns>
        public int GetSESweepRefLevel(int Range, out double Ref_Level_Value)
        {
            int pInvokeResult = PInvoke.GetSESweepRefLevel(this._handle, Range, out Ref_Level_Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the autoranging of a range in the spurious measurement on or off.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]LIST:RANGe&lt;1...20&gt;:INPut:ATTenuation:AUTO ON | OFF
        /// </summary>
        /// <param name="Range">
        /// Sets the configured range.
        /// 
        /// Valid Values:
        /// 1 to 20
        /// 
        /// Default Value:
        /// 1
        /// </param>
        /// <param name="Attenuator_Auto">
        /// Sets the autoranging of a range in the spurious measurement on or off.
        /// 
        /// Valid Range:
        /// VI_FALSE (0) - Off 
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_TRUE (1)
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation.  The status code  either indicates success or describes an error or warning condition.  You examine the status code from each call to an instrument driver function to determine if an error occurred.
        /// 
        /// To obtain a text description of the status code, call the rsspecan_error_message function.  To obtain additional information about the error condition, call the rsspecan_GetError function.  To clear the error information from the driver, call the rsspecan_ClearError function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// WARNINGS:
        /// 3FFA2001  The instrument was in an uncalibrated state
        ///           when the measurement was taken.
        /// 3FFA2002  The measurement taken was over the instrument??s
        ///           range.
        ///   
        /// ERRORS:
        /// BFFA2001  The Active Marker is not enabled.
        /// BFFA2002  The Active Marker is not a delta marker.
        /// BFFA2003  The maximum waiting time for this operation was
        ///           exceeded.
        ///           
        /// This instrument driver also returns errors and warnings defined by other sources.  The following table defines the ranges of additional status codes that this driver can return.  The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFA0000 to 3FFA1FFF     IVI    Warnings
        /// 3FFF0000 to 3FFFFFFF     VISA   Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP Driver Warnings
        ///           
        /// BFFA0000 to BFFA1FFF     IVI    Errors
        /// BFFF0000 to BFFFFFFF     VISA   Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP Driver Errors
        /// 
        /// </returns>
        public int SetSESweepAttenuatorAuto(int Range, bool Attenuator_Auto)
        {
            int pInvokeResult = PInvoke.SetSESweepAttenuatorAuto(this._handle, Range, System.Convert.ToUInt16(Attenuator_Auto));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the autoranging of a range in the spurious measurement on or off.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]LIST:RANGe&lt;1...20&gt;:INPut:ATTenuation:AUTO?
        /// </summary>
        /// <param name="Range">
        /// Sets the configured range.
        /// 
        /// Valid Values:
        /// 1 to 20
        /// 
        /// Default Value:
        /// 1
        /// </param>
        /// <param name="Attenuator_Auto">
        /// Returns the autoranging of a range in the spurious measurement on or off.
        /// 
        /// Valid Range:
        /// VI_FALSE (0) - Off 
        /// VI_TRUE  (1) - On
        /// </param>
        /// <returns>
        /// Returns the status code of this operation.  The status code  either indicates success or describes an error or warning condition.  You examine the status code from each call to an instrument driver function to determine if an error occurred.
        /// 
        /// To obtain a text description of the status code, call the rsspecan_error_message function.  To obtain additional information about the error condition, call the rsspecan_GetError function.  To clear the error information from the driver, call the rsspecan_ClearError function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// WARNINGS:
        /// 3FFA2001  The instrument was in an uncalibrated state
        ///           when the measurement was taken.
        /// 3FFA2002  The measurement taken was over the instrument??s
        ///           range.
        ///   
        /// ERRORS:
        /// BFFA2001  The Active Marker is not enabled.
        /// BFFA2002  The Active Marker is not a delta marker.
        /// BFFA2003  The maximum waiting time for this operation was
        ///           exceeded.
        ///           
        /// This instrument driver also returns errors and warnings defined by other sources.  The following table defines the ranges of additional status codes that this driver can return.  The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFA0000 to 3FFA1FFF     IVI    Warnings
        /// 3FFF0000 to 3FFFFFFF     VISA   Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP Driver Warnings
        ///           
        /// BFFA0000 to BFFA1FFF     IVI    Errors
        /// BFFF0000 to BFFFFFFF     VISA   Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP Driver Errors
        /// 
        /// </returns>
        public int GetSESweepAttenuatorAuto(int Range, out bool Attenuator_Auto)
        {
            ushort Attenuator_AutoAsUShort;
            int pInvokeResult = PInvoke.GetSESweepAttenuatorAuto(this._handle, Range, out Attenuator_AutoAsUShort);
            Attenuator_Auto = System.Convert.ToBoolean(Attenuator_AutoAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the RF attenuation of a range.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with R&amp;S ESU instrument.
        /// (2) This function has effect only if Autoranging control is set Off
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]LIST:RANGe&lt;1...20&gt;:INPut:ATTenuation
        /// 
        /// </summary>
        /// <param name="Range">
        /// Sets the configured range.
        /// 
        /// Valid Values:
        /// 1 to 20
        /// 
        /// Default Value:
        /// 1
        /// </param>
        /// <param name="Manual_Attenuator_Value__dB_">
        /// This control sets the RF attenuation of a range.
        /// 
        /// Valid Range: 0.0 dB to 75.0 dB
        /// 
        /// Default Value: 15.0 dB
        /// 
        /// Note:
        /// (1) This control has effect only if Autoranging control is set Off
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation.  The status code  either indicates success or describes an error or warning condition.  You examine the status code from each call to an instrument driver function to determine if an error occurred.
        /// 
        /// To obtain a text description of the status code, call the rsspecan_error_message function.  To obtain additional information about the error condition, call the rsspecan_GetError function.  To clear the error information from the driver, call the rsspecan_ClearError function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// WARNINGS:
        /// 3FFA2001  The instrument was in an uncalibrated state
        ///           when the measurement was taken.
        /// 3FFA2002  The measurement taken was over the instrument??s
        ///           range.
        ///   
        /// ERRORS:
        /// BFFA2001  The Active Marker is not enabled.
        /// BFFA2002  The Active Marker is not a delta marker.
        /// BFFA2003  The maximum waiting time for this operation was
        ///           exceeded.
        ///           
        /// This instrument driver also returns errors and warnings defined by other sources.  The following table defines the ranges of additional status codes that this driver can return.  The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFA0000 to 3FFA1FFF     IVI    Warnings
        /// 3FFF0000 to 3FFFFFFF     VISA   Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP Driver Warnings
        ///           
        /// BFFA0000 to BFFA1FFF     IVI    Errors
        /// BFFF0000 to BFFFFFFF     VISA   Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP Driver Errors
        /// 
        /// </returns>
        public int SetSESweepAttenuatorValue(int Range, double Manual_Attenuator_Value__dB_)
        {
            int pInvokeResult = PInvoke.SetSESweepAttenuatorValue(this._handle, Range, Manual_Attenuator_Value__dB_);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the RF attenuation of a range.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]LIST:RANGe&lt;1...20&gt;:INPut:ATTenuation?
        /// 
        /// </summary>
        /// <param name="Range">
        /// Sets the configured range.
        /// 
        /// Valid Values:
        /// 1 to 20
        /// 
        /// Default Value:
        /// 1
        /// </param>
        /// <param name="Manual_Attenuator_Value__dB_">
        /// This control returns the RF attenuation of a range.
        /// 
        /// Valid Range: 0.0 dB to 75.0 dBw
        /// </param>
        /// <returns>
        /// Returns the status code of this operation.  The status code  either indicates success or describes an error or warning condition.  You examine the status code from each call to an instrument driver function to determine if an error occurred.
        /// 
        /// To obtain a text description of the status code, call the rsspecan_error_message function.  To obtain additional information about the error condition, call the rsspecan_GetError function.  To clear the error information from the driver, call the rsspecan_ClearError function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// WARNINGS:
        /// 3FFA2001  The instrument was in an uncalibrated state
        ///           when the measurement was taken.
        /// 3FFA2002  The measurement taken was over the instrument??s
        ///           range.
        ///   
        /// ERRORS:
        /// BFFA2001  The Active Marker is not enabled.
        /// BFFA2002  The Active Marker is not a delta marker.
        /// BFFA2003  The maximum waiting time for this operation was
        ///           exceeded.
        ///           
        /// This instrument driver also returns errors and warnings defined by other sources.  The following table defines the ranges of additional status codes that this driver can return.  The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFA0000 to 3FFA1FFF     IVI    Warnings
        /// 3FFF0000 to 3FFFFFFF     VISA   Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP Driver Warnings
        ///           
        /// BFFA0000 to BFFA1FFF     IVI    Errors
        /// BFFF0000 to BFFFFFFF     VISA   Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP Driver Errors
        /// 
        /// </returns>
        public int GetSESweepAttenuatorValue(int Range, out double Manual_Attenuator_Value__dB_)
        {
            int pInvokeResult = PInvoke.GetSESweepAttenuatorValue(this._handle, Range, out Manual_Attenuator_Value__dB_);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the number of sweep points of a range in the spurious measurement.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]LIST:RANGe&lt;1...20&gt;:POINts
        /// </summary>
        /// <param name="Range">
        /// Sets the configured range.
        /// 
        /// Valid Values:
        /// 1 to 20
        /// 
        /// Default Value:
        /// 1
        /// </param>
        /// <param name="Sweep_Points">
        /// Sets the number of sweep points of a range in the spurious measurement.
        /// 
        /// Valid Range: no range checking
        /// 
        /// Default Value: 625
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation.  The status code  either indicates success or describes an error or warning condition.  You examine the status code from each call to an instrument driver function to determine if an error occurred.
        /// 
        /// To obtain a text description of the status code, call the rsspecan_error_message function.  To obtain additional information about the error condition, call the rsspecan_GetError function.  To clear the error information from the driver, call the rsspecan_ClearError function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// WARNINGS:
        /// 3FFA2001  The instrument was in an uncalibrated state
        ///           when the measurement was taken.
        /// 3FFA2002  The measurement taken was over the instrument??s
        ///           range.
        ///   
        /// ERRORS:
        /// BFFA2001  The Active Marker is not enabled.
        /// BFFA2002  The Active Marker is not a delta marker.
        /// BFFA2003  The maximum waiting time for this operation was
        ///           exceeded.
        ///           
        /// This instrument driver also returns errors and warnings defined by other sources.  The following table defines the ranges of additional status codes that this driver can return.  The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFA0000 to 3FFA1FFF     IVI    Warnings
        /// 3FFF0000 to 3FFFFFFF     VISA   Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP Driver Warnings
        ///           
        /// BFFA0000 to BFFA1FFF     IVI    Errors
        /// BFFF0000 to BFFFFFFF     VISA   Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP Driver Errors
        /// 
        /// </returns>
        public int SetSESweepPoints(int Range, int Sweep_Points)
        {
            int pInvokeResult = PInvoke.SetSESweepPoints(this._handle, Range, Sweep_Points);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the number of sweep points of a range in the spurious measurement.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]LIST:RANGe&lt;1...20&gt;:POINts?
        /// </summary>
        /// <param name="Range">
        /// Sets the configured range.
        /// 
        /// Valid Values:
        /// 1 to 20
        /// 
        /// Default Value:
        /// 1
        /// </param>
        /// <param name="Sweep_Points">
        /// Returns the number of sweep points of a range in the spurious measurement.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation.  The status code  either indicates success or describes an error or warning condition.  You examine the status code from each call to an instrument driver function to determine if an error occurred.
        /// 
        /// To obtain a text description of the status code, call the rsspecan_error_message function.  To obtain additional information about the error condition, call the rsspecan_GetError function.  To clear the error information from the driver, call the rsspecan_ClearError function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// WARNINGS:
        /// 3FFA2001  The instrument was in an uncalibrated state
        ///           when the measurement was taken.
        /// 3FFA2002  The measurement taken was over the instrument??s
        ///           range.
        ///   
        /// ERRORS:
        /// BFFA2001  The Active Marker is not enabled.
        /// BFFA2002  The Active Marker is not a delta marker.
        /// BFFA2003  The maximum waiting time for this operation was
        ///           exceeded.
        ///           
        /// This instrument driver also returns errors and warnings defined by other sources.  The following table defines the ranges of additional status codes that this driver can return.  The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFA0000 to 3FFA1FFF     IVI    Warnings
        /// 3FFF0000 to 3FFFFFFF     VISA   Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP Driver Warnings
        ///           
        /// BFFA0000 to BFFA1FFF     IVI    Errors
        /// BFFF0000 to BFFFFFFF     VISA   Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP Driver Errors
        /// 
        /// </returns>
        public int GetSESweepPoints(int Range, out int Sweep_Points)
        {
            int pInvokeResult = PInvoke.GetSESweepPoints(this._handle, Range, out Sweep_Points);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function determines whether a sweep is stopped in the spurious measurement when a range is changed.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]LIST:RANGe&lt;1...20&gt;:BREAK ON | OFF
        /// </summary>
        /// <param name="Range">
        /// Sets the configured range.
        /// 
        /// Valid Values:
        /// 1 to 20
        /// 
        /// Default Value:
        /// 1
        /// </param>
        /// <param name="Stop_After_Sweep">
        /// Determines whether a sweep is stopped in the spurious measurement when a range is changed.
        /// 
        /// Valid Range:
        /// VI_FALSE (0) - Off 
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation.  The status code  either indicates success or describes an error or warning condition.  You examine the status code from each call to an instrument driver function to determine if an error occurred.
        /// 
        /// To obtain a text description of the status code, call the rsspecan_error_message function.  To obtain additional information about the error condition, call the rsspecan_GetError function.  To clear the error information from the driver, call the rsspecan_ClearError function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// WARNINGS:
        /// 3FFA2001  The instrument was in an uncalibrated state
        ///           when the measurement was taken.
        /// 3FFA2002  The measurement taken was over the instrument??s
        ///           range.
        ///   
        /// ERRORS:
        /// BFFA2001  The Active Marker is not enabled.
        /// BFFA2002  The Active Marker is not a delta marker.
        /// BFFA2003  The maximum waiting time for this operation was
        ///           exceeded.
        ///           
        /// This instrument driver also returns errors and warnings defined by other sources.  The following table defines the ranges of additional status codes that this driver can return.  The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFA0000 to 3FFA1FFF     IVI    Warnings
        /// 3FFF0000 to 3FFFFFFF     VISA   Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP Driver Warnings
        ///           
        /// BFFA0000 to BFFA1FFF     IVI    Errors
        /// BFFF0000 to BFFFFFFF     VISA   Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP Driver Errors
        /// 
        /// </returns>
        public int SetSESweepStopAfterSweep(int Range, bool Stop_After_Sweep)
        {
            int pInvokeResult = PInvoke.SetSESweepStopAfterSweep(this._handle, Range, System.Convert.ToUInt16(Stop_After_Sweep));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries whether a sweep is stopped in the spurious measurement when a range is changed.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]LIST:RANGe&lt;1...20&gt;:BREAK?
        /// </summary>
        /// <param name="Range">
        /// Sets the configured range.
        /// 
        /// Valid Values:
        /// 1 to 20
        /// 
        /// Default Value:
        /// 1
        /// </param>
        /// <param name="Stop_After_Sweep">
        /// Returns whether a sweep is stopped in the spurious measurement when a range is changed.
        /// 
        /// Valid Range:
        /// VI_FALSE (0) - Off 
        /// VI_TRUE  (1) - On
        /// </param>
        /// <returns>
        /// Returns the status code of this operation.  The status code  either indicates success or describes an error or warning condition.  You examine the status code from each call to an instrument driver function to determine if an error occurred.
        /// 
        /// To obtain a text description of the status code, call the rsspecan_error_message function.  To obtain additional information about the error condition, call the rsspecan_GetError function.  To clear the error information from the driver, call the rsspecan_ClearError function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// WARNINGS:
        /// 3FFA2001  The instrument was in an uncalibrated state
        ///           when the measurement was taken.
        /// 3FFA2002  The measurement taken was over the instrument??s
        ///           range.
        ///   
        /// ERRORS:
        /// BFFA2001  The Active Marker is not enabled.
        /// BFFA2002  The Active Marker is not a delta marker.
        /// BFFA2003  The maximum waiting time for this operation was
        ///           exceeded.
        ///           
        /// This instrument driver also returns errors and warnings defined by other sources.  The following table defines the ranges of additional status codes that this driver can return.  The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFA0000 to 3FFA1FFF     IVI    Warnings
        /// 3FFF0000 to 3FFFFFFF     VISA   Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP Driver Warnings
        ///           
        /// BFFA0000 to BFFA1FFF     IVI    Errors
        /// BFFF0000 to BFFFFFFF     VISA   Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP Driver Errors
        /// 
        /// </returns>
        public int GetSESweepStopAfterSweep(int Range, out bool Stop_After_Sweep)
        {
            ushort Stop_After_SweepAsUShort;
            int pInvokeResult = PInvoke.GetSESweepStopAfterSweep(this._handle, Range, out Stop_After_SweepAsUShort);
            Stop_After_Sweep = System.Convert.ToBoolean(Stop_After_SweepAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the transducer factor for a range in the spurious measurement.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]LIST:RANGe&lt;1...20&gt;:TRANsducer
        /// </summary>
        /// <param name="Range">
        /// Sets the configured range.
        /// 
        /// Valid Values:
        /// 1 to 20
        /// 
        /// Default Value:
        /// 1
        /// </param>
        /// <param name="Transducer_Factor">
        /// Sets the transducer factor for a range in the spurious measurement.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: ""
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation.  The status code  either indicates success or describes an error or warning condition.  You examine the status code from each call to an instrument driver function to determine if an error occurred.
        /// 
        /// To obtain a text description of the status code, call the rsspecan_error_message function.  To obtain additional information about the error condition, call the rsspecan_GetError function.  To clear the error information from the driver, call the rsspecan_ClearError function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// WARNINGS:
        /// 3FFA2001  The instrument was in an uncalibrated state
        ///           when the measurement was taken.
        /// 3FFA2002  The measurement taken was over the instrument??s
        ///           range.
        ///   
        /// ERRORS:
        /// BFFA2001  The Active Marker is not enabled.
        /// BFFA2002  The Active Marker is not a delta marker.
        /// BFFA2003  The maximum waiting time for this operation was
        ///           exceeded.
        ///           
        /// This instrument driver also returns errors and warnings defined by other sources.  The following table defines the ranges of additional status codes that this driver can return.  The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFA0000 to 3FFA1FFF     IVI    Warnings
        /// 3FFF0000 to 3FFFFFFF     VISA   Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP Driver Warnings
        ///           
        /// BFFA0000 to BFFA1FFF     IVI    Errors
        /// BFFF0000 to BFFFFFFF     VISA   Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP Driver Errors
        /// 
        /// </returns>
        public int SetSESweepTransducerFactor(int Range, string Transducer_Factor)
        {
            int pInvokeResult = PInvoke.SetSESweepTransducerFactor(this._handle, Range, Transducer_Factor);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the transducer factor for a range in the spurious measurement.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]LIST:RANGe&lt;1...20&gt;:TRANsducer?
        /// </summary>
        /// <param name="Range">
        /// Sets the configured range.
        /// 
        /// Valid Values:
        /// 1 to 20
        /// 
        /// Default Value:
        /// 1
        /// </param>
        /// <param name="Array_Size">
        /// This control defines the size of array passed to Transducer Factor argument.
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: 256
        /// </param>
        /// <param name="Transducer_Factor">
        /// Returns the transducer factor for a range in the spurious measurement.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation.  The status code  either indicates success or describes an error or warning condition.  You examine the status code from each call to an instrument driver function to determine if an error occurred.
        /// 
        /// To obtain a text description of the status code, call the rsspecan_error_message function.  To obtain additional information about the error condition, call the rsspecan_GetError function.  To clear the error information from the driver, call the rsspecan_ClearError function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// WARNINGS:
        /// 3FFA2001  The instrument was in an uncalibrated state
        ///           when the measurement was taken.
        /// 3FFA2002  The measurement taken was over the instrument??s
        ///           range.
        ///   
        /// ERRORS:
        /// BFFA2001  The Active Marker is not enabled.
        /// BFFA2002  The Active Marker is not a delta marker.
        /// BFFA2003  The maximum waiting time for this operation was
        ///           exceeded.
        ///           
        /// This instrument driver also returns errors and warnings defined by other sources.  The following table defines the ranges of additional status codes that this driver can return.  The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFA0000 to 3FFA1FFF     IVI    Warnings
        /// 3FFF0000 to 3FFFFFFF     VISA   Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP Driver Warnings
        ///           
        /// BFFA0000 to BFFA1FFF     IVI    Errors
        /// BFFF0000 to BFFFFFFF     VISA   Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP Driver Errors
        /// 
        /// </returns>
        public int GetSESweepTransducerFactor(int Range, int Array_Size, System.Text.StringBuilder Transducer_Factor)
        {
            int pInvokeResult = PInvoke.GetSESweepTransducerFactor(this._handle, Range, Array_Size, Transducer_Factor);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the limit lines for the spurious emission measurement list evaluation.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]LIST:RANGe&lt;1...20&gt;:LIMit
        /// </summary>
        /// <param name="Range">
        /// Sets the configured range.
        /// 
        /// Valid Values:
        /// 1 to 20
        /// 
        /// Default Value:
        /// 1
        /// </param>
        /// <param name="Limit_Value">
        /// Defines the limit lines for the spurious emission measurement list evaluation.
        /// 
        /// Valid Range:
        /// -200.0 to 200.0 dB
        /// 
        /// Default Value: -13.0 dB
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation.  The status code  either indicates success or describes an error or warning condition.  You examine the status code from each call to an instrument driver function to determine if an error occurred.
        /// 
        /// To obtain a text description of the status code, call the rsspecan_error_message function.  To obtain additional information about the error condition, call the rsspecan_GetError function.  To clear the error information from the driver, call the rsspecan_ClearError function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// WARNINGS:
        /// 3FFA2001  The instrument was in an uncalibrated state
        ///           when the measurement was taken.
        /// 3FFA2002  The measurement taken was over the instrument??s
        ///           range.
        ///   
        /// ERRORS:
        /// BFFA2001  The Active Marker is not enabled.
        /// BFFA2002  The Active Marker is not a delta marker.
        /// BFFA2003  The maximum waiting time for this operation was
        ///           exceeded.
        ///           
        /// This instrument driver also returns errors and warnings defined by other sources.  The following table defines the ranges of additional status codes that this driver can return.  The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFA0000 to 3FFA1FFF     IVI    Warnings
        /// 3FFF0000 to 3FFFFFFF     VISA   Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP Driver Warnings
        ///           
        /// BFFA0000 to BFFA1FFF     IVI    Errors
        /// BFFF0000 to BFFFFFFF     VISA   Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP Driver Errors
        /// 
        /// </returns>
        public int SetSESweepLimit(int Range, double Limit_Value)
        {
            int pInvokeResult = PInvoke.SetSESweepLimit(this._handle, Range, Limit_Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the limit lines for the spurious emission measurement list evaluation.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]LIST:RANGe&lt;1...20&gt;:LIMit?
        /// </summary>
        /// <param name="Range">
        /// Sets the configured range.
        /// 
        /// Valid Values:
        /// 1 to 20
        /// 
        /// Default Value:
        /// 1
        /// </param>
        /// <param name="Limit_Value">
        /// Returns the limit lines for the spurious emission measurement list evaluation.
        /// 
        /// Valid Range:
        /// -200.0 to 200.0 dB
        /// </param>
        /// <returns>
        /// Returns the status code of this operation.  The status code  either indicates success or describes an error or warning condition.  You examine the status code from each call to an instrument driver function to determine if an error occurred.
        /// 
        /// To obtain a text description of the status code, call the rsspecan_error_message function.  To obtain additional information about the error condition, call the rsspecan_GetError function.  To clear the error information from the driver, call the rsspecan_ClearError function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// WARNINGS:
        /// 3FFA2001  The instrument was in an uncalibrated state
        ///           when the measurement was taken.
        /// 3FFA2002  The measurement taken was over the instrument??s
        ///           range.
        ///   
        /// ERRORS:
        /// BFFA2001  The Active Marker is not enabled.
        /// BFFA2002  The Active Marker is not a delta marker.
        /// BFFA2003  The maximum waiting time for this operation was
        ///           exceeded.
        ///           
        /// This instrument driver also returns errors and warnings defined by other sources.  The following table defines the ranges of additional status codes that this driver can return.  The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFA0000 to 3FFA1FFF     IVI    Warnings
        /// 3FFF0000 to 3FFFFFFF     VISA   Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP Driver Warnings
        ///           
        /// BFFA0000 to BFFA1FFF     IVI    Errors
        /// BFFF0000 to BFFFFFFF     VISA   Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP Driver Errors
        /// 
        /// </returns>
        public int GetSESweepLimit(int Range, out double Limit_Value)
        {
            int pInvokeResult = PInvoke.GetSESweepLimit(this._handle, Range, out Limit_Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function activates/deactivates the limit checking for the spurious emission measurement list evaluation.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]LIST:RANGe&lt;1...20&gt;:LIMit:STATe ON | OFF
        /// </summary>
        /// <param name="Limit_State">
        /// Activates/deactivates the limit checking for the spurious emission measurement list evaluation.
        /// 
        /// Valid Range:
        /// VI_FALSE (0) - Off 
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation.  The status code  either indicates success or describes an error or warning condition.  You examine the status code from each call to an instrument driver function to determine if an error occurred.
        /// 
        /// To obtain a text description of the status code, call the rsspecan_error_message function.  To obtain additional information about the error condition, call the rsspecan_GetError function.  To clear the error information from the driver, call the rsspecan_ClearError function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// WARNINGS:
        /// 3FFA2001  The instrument was in an uncalibrated state
        ///           when the measurement was taken.
        /// 3FFA2002  The measurement taken was over the instrument??s
        ///           range.
        ///   
        /// ERRORS:
        /// BFFA2001  The Active Marker is not enabled.
        /// BFFA2002  The Active Marker is not a delta marker.
        /// BFFA2003  The maximum waiting time for this operation was
        ///           exceeded.
        ///           
        /// This instrument driver also returns errors and warnings defined by other sources.  The following table defines the ranges of additional status codes that this driver can return.  The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFA0000 to 3FFA1FFF     IVI    Warnings
        /// 3FFF0000 to 3FFFFFFF     VISA   Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP Driver Warnings
        ///           
        /// BFFA0000 to BFFA1FFF     IVI    Errors
        /// BFFF0000 to BFFFFFFF     VISA   Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP Driver Errors
        /// 
        /// </returns>
        public int SetSESweepLimitState(bool Limit_State)
        {
            int pInvokeResult = PInvoke.SetSESweepLimitState(this._handle, System.Convert.ToUInt16(Limit_State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries state of the limit checking for the spurious emission measurement list evaluation.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]LIST:RANGe&lt;1...20&gt;:LIMit:STATe?
        /// </summary>
        /// <param name="Limit_State">
        /// Returns state of the limit checking for the spurious emission measurement list evaluation.
        /// 
        /// Valid Range:
        /// VI_FALSE (0) - Off 
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation.  The status code  either indicates success or describes an error or warning condition.  You examine the status code from each call to an instrument driver function to determine if an error occurred.
        /// 
        /// To obtain a text description of the status code, call the rsspecan_error_message function.  To obtain additional information about the error condition, call the rsspecan_GetError function.  To clear the error information from the driver, call the rsspecan_ClearError function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// WARNINGS:
        /// 3FFA2001  The instrument was in an uncalibrated state
        ///           when the measurement was taken.
        /// 3FFA2002  The measurement taken was over the instrument??s
        ///           range.
        ///   
        /// ERRORS:
        /// BFFA2001  The Active Marker is not enabled.
        /// BFFA2002  The Active Marker is not a delta marker.
        /// BFFA2003  The maximum waiting time for this operation was
        ///           exceeded.
        ///           
        /// This instrument driver also returns errors and warnings defined by other sources.  The following table defines the ranges of additional status codes that this driver can return.  The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFA0000 to 3FFA1FFF     IVI    Warnings
        /// 3FFF0000 to 3FFFFFFF     VISA   Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP Driver Warnings
        ///           
        /// BFFA0000 to BFFA1FFF     IVI    Errors
        /// BFFF0000 to BFFFFFFF     VISA   Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP Driver Errors
        /// 
        /// </returns>
        public int GetSESweepLimitState(out bool Limit_State)
        {
            ushort Limit_StateAsUShort;
            int pInvokeResult = PInvoke.GetSESweepLimitState(this._handle, out Limit_StateAsUShort);
            Limit_State = System.Convert.ToBoolean(Limit_StateAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the number of defined ranges of the spurious measurement.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with R&amp;S ESU instrument.
        /// 
        /// (2) The numeric suffix &lt;1 to 20&gt; with RANGe is not relevant for this command.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]LIST:RANGe&lt;1...20&gt;:COUNt?
        /// </summary>
        /// <param name="Number_Of_Ranges">
        /// Returns indicates the number of defined ranges of the spurious measurement. The driver sets the RSSPECAN_ATTR_LIST_RANG_COUNT to this value.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code
        /// either indicates success or describes an error or warning
        /// condition. You examine the status code from each call to an
        /// instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the
        /// rsfsq_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        /// 
        /// Status    Description
        /// -------------------------------------------------
        /// 
        /// BFFC0002  Parameter 2 (Subrange) out of range.
        /// BFFC0003  Parameter 3 (Margin) out of range.
        /// 
        /// BFFC09F0  Instrument status error
        /// BFFC09F1  Option not installed
        /// BFFC09F2  Conflict with setup parameters
        /// BFFC09F3  Data are not available
        /// BFFC09F4  Unknown option
        /// BFFC09F5  Calibration failed
        /// BFFC09F6  Function does not support FSEx parameter value
        /// BFFC09F7  Overload. Instrument does not return I/Q values
        /// 
        /// This instrument driver also returns errors and warnings defined
        /// by other sources. The following table defines the ranges of
        /// additional status codes that this driver can return. The table
        /// lists the different include files that contain the defined
        /// constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetSENumberOfListRanges(out int Number_Of_Ranges)
        {
            int pInvokeResult = PInvoke.GetSENumberOfListRanges(this._handle, out Number_Of_Ranges);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function starts the final measurement based on the peak list.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is only available with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// INITiate&lt;1|2&gt;:FMEasurement
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int InitiateFinalMeasurementTest(int Window)
        {
            int pInvokeResult = PInvoke.InitiateFinalMeasurementTest(this._handle, Window);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function starts an automatic sequence consisting of the preliminary measurement, determination of the peak list and subsequent final measurement.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is only available with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// INITiate&lt;1|2&gt;:EMITest
        /// </summary>
        /// <param name="Window">
        /// This control is reserved for future use.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int InitiateEMITest(int Window)
        {
            int pInvokeResult = PInvoke.InitiateEMITest(this._handle, Window);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function activates the detectors for single point measurements in EMI receiver operating mode.
        /// 
        /// Three detectors can be activated at the same time.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]DETector&lt;1...3&gt;:RECeiver[:FUNCtion] POSitive | NEGative | RMS | AVERage | QPEak | CAVerage
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Max_Peak">
        /// This control switches on or off max peak detector.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <param name="Min_Peak">
        /// This control switches on or off min peak detector.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <param name="RMS">
        /// This control switches on or off RMS detector.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <param name="Average">
        /// This control switches on or off average detector.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <param name="Quasipeak">
        /// This control switches on or off quasipeak detector.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <param name="CISPR_Average">
        /// This control switches on or off CISPR Average detector.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetReceiverDetector(int Window, bool Max_Peak, bool Min_Peak, bool RMS, bool Average, bool Quasipeak, bool CISPR_Average)
        {
            int pInvokeResult = PInvoke.SetReceiverDetector(this._handle, Window, System.Convert.ToUInt16(Max_Peak), System.Convert.ToUInt16(Min_Peak), System.Convert.ToUInt16(RMS), System.Convert.ToUInt16(Average), System.Convert.ToUInt16(Quasipeak), System.Convert.ToUInt16(CISPR_Average));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the selected receiver mode detectors.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]DETector:RECeiver[:FUNCtion]?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Max_Peak">
        /// This control returns the state of max peak detector.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <param name="Min_Peak">
        /// This control returns the state of min peak detector.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <param name="RMS">
        /// This control returns the state of RMS detector.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <param name="Average">
        /// This control returns the state of average detector.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <param name="Quasipeak">
        /// This control returns the state of quasipeak detector.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <param name="CISPR_Average">
        /// This control returns the state of CISPR Average detector.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetReceiverDetector(int Window, out bool Max_Peak, out bool Min_Peak, out bool RMS, out bool Average, out bool Quasipeak, out bool CISPR_Average)
        {
            ushort Max_PeakAsUShort;
            ushort Min_PeakAsUShort;
            ushort RMSAsUShort;
            ushort AverageAsUShort;
            ushort QuasipeakAsUShort;
            ushort CISPR_AverageAsUShort;
            int pInvokeResult = PInvoke.GetReceiverDetector(this._handle, Window, out Max_PeakAsUShort, out Min_PeakAsUShort, out RMSAsUShort, out AverageAsUShort, out QuasipeakAsUShort, out CISPR_AverageAsUShort);
            Max_Peak = System.Convert.ToBoolean(Max_PeakAsUShort);
            Min_Peak = System.Convert.ToBoolean(Min_PeakAsUShort);
            RMS = System.Convert.ToBoolean(RMSAsUShort);
            Average = System.Convert.ToBoolean(AverageAsUShort);
            Quasipeak = System.Convert.ToBoolean(QuasipeakAsUShort);
            CISPR_Average = System.Convert.ToBoolean(CISPR_AverageAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function activates the detectors for single point measurements in EMI receiver operating mode.
        /// 
        /// Three detectors can be activated at the same time.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]DETector:RECeiver[:FUNCtion] POSitive | NEGative | RMS | AVERage | QPEak | CAVerage | CRMS
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Max_Peak">
        /// This control switches on or off max peak detector.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <param name="Min_Peak">
        /// This control switches on or off min peak detector.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <param name="RMS">
        /// This control switches on or off RMS detector.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <param name="Average">
        /// This control switches on or off average detector.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <param name="Quasipeak">
        /// This control switches on or off quasipeak detector.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <param name="CISPR_Average">
        /// This control switches on or off CISPR Average detector.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <param name="CISPR_RMS">
        /// This control switches on or off CISPR RMS detector.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetReceiverDetectorWithCRMS(int Window, bool Max_Peak, bool Min_Peak, bool RMS, bool Average, bool Quasipeak, bool CISPR_Average, bool CISPR_RMS)
        {
            int pInvokeResult = PInvoke.SetReceiverDetectorWithCRMS(this._handle, Window, System.Convert.ToUInt16(Max_Peak), System.Convert.ToUInt16(Min_Peak), System.Convert.ToUInt16(RMS), System.Convert.ToUInt16(Average), System.Convert.ToUInt16(Quasipeak), System.Convert.ToUInt16(CISPR_Average), System.Convert.ToUInt16(CISPR_RMS));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the selected receiver mode detectors.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]DETector:RECeiver[:FUNCtion]?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Max_Peak">
        /// This control returns the state of max peak detector.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <param name="Min_Peak">
        /// This control returns the state of min peak detector.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <param name="RMS">
        /// This control returns the state of RMS detector.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <param name="Average">
        /// This control returns the state of average detector.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <param name="Quasipeak">
        /// This control returns the state of quasipeak detector.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <param name="CISPR_Average">
        /// This control returns the state of CISPR Average detector.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <param name="CISPR_RMS">
        /// This control returns the state of the CISPR RMS detector.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetReceiverDetectorWithCRMS(int Window, out bool Max_Peak, out bool Min_Peak, out bool RMS, out bool Average, out bool Quasipeak, out bool CISPR_Average, out bool CISPR_RMS)
        {
            ushort Max_PeakAsUShort;
            ushort Min_PeakAsUShort;
            ushort RMSAsUShort;
            ushort AverageAsUShort;
            ushort QuasipeakAsUShort;
            ushort CISPR_AverageAsUShort;
            ushort CISPR_RMSAsUShort;
            int pInvokeResult = PInvoke.GetReceiverDetectorWithCRMS(this._handle, Window, out Max_PeakAsUShort, out Min_PeakAsUShort, out RMSAsUShort, out AverageAsUShort, out QuasipeakAsUShort, out CISPR_AverageAsUShort, out CISPR_RMSAsUShort);
            Max_Peak = System.Convert.ToBoolean(Max_PeakAsUShort);
            Min_Peak = System.Convert.ToBoolean(Min_PeakAsUShort);
            RMS = System.Convert.ToBoolean(RMSAsUShort);
            Average = System.Convert.ToBoolean(AverageAsUShort);
            Quasipeak = System.Convert.ToBoolean(QuasipeakAsUShort);
            CISPR_Average = System.Convert.ToBoolean(CISPR_AverageAsUShort);
            CISPR_RMS = System.Convert.ToBoolean(CISPR_RMSAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the detector for the final measurement (the detector used subsequent to the prescan).
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]DETector&lt;1...3&gt;:FMEasurement NEGative | POSitive | RMS | AVERage| QPEak | CAVerage | CRMS
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Trace">
        /// This control sets the trace.
        /// 
        /// Valid Range: 1 to 3
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Final_Detector">
        /// This control selects the detector for the final measurement (the detector used subsequent to the prescan).
        /// 
        /// Valid Values:
        /// RSESCI_DETECTOR_MAXP  (0) - Max Peak
        /// RSESCI_DETECTOR_MINP  (1) - Min Peak
        /// RSESCI_DETECTOR_RMS   (2) - RMS
        /// RSESCI_DETECTOR_AVG   (3) - Average
        /// RSESCI_DETECTOR_QPEAK (4) - Quasipeak
        /// RSESCI_DETECTOR_CAV   (5) - CISPR Average
        /// RSESCI_DETECTOR_CRMS  (6) - CISPR RMS
        /// 
        /// Default Value: RSESCI_DETECTOR_MAXP (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetFinalReceiverDetector(int Window, int Trace, int Final_Detector)
        {
            int pInvokeResult = PInvoke.SetFinalReceiverDetector(this._handle, Window, Trace, Final_Detector);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries selected detector for the final measurement (the detector used subsequent to the prescan).
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]DETector&lt;1...3&gt;:FMEasurement?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Trace">
        /// This control sets the trace.
        /// 
        /// Valid Range: 1 to 3
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Final_Detector">
        /// This control returns selected detector for the final measurement (the detector used subsequent to the prescan).
        /// 
        /// Valid Values:
        /// RSESCI_DETECTOR_MAXP  (0) - Max Peak
        /// RSESCI_DETECTOR_MINP  (1) - Min Peak
        /// RSESCI_DETECTOR_RMS   (2) - RMS
        /// RSESCI_DETECTOR_AVG   (3) - Average
        /// RSESCI_DETECTOR_QPEAK (4) - Quasipeak
        /// RSESCI_DETECTOR_CAV   (5) - CISPR Average
        /// RSESCI_DETECTOR_CRMS  (6) - CISPR RMS
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetFinalReceiverDetector(int Window, int Trace, out int Final_Detector)
        {
            int pInvokeResult = PInvoke.GetFinalReceiverDetector(this._handle, Window, Trace, out Final_Detector);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function generates a peak list from the scan results. For this purpose, a scan must be performed prior to this function.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:PEAKsearch|PSEarch[:IMMediate]
        /// </summary>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int PeakSearch()
        {
            int pInvokeResult = PInvoke.PeakSearch(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines whether the peak list in the spurious measurement is calculated automatically after a measurement. For each range, exactly one peak value is calculated.
        /// 
        /// Mode(s):
        /// Spectrum Analyzer, Receiver
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:PEAKsearch|PSEarch:AUTO ON | OFF
        /// </summary>
        /// <param name="Peak_Search_Auto">
        /// This control defines whether the peak list in the spurious measurement is calculated automatically after a measurement. For each range, exactly one peak value is calculated.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetPeakSearchAuto(bool Peak_Search_Auto)
        {
            int pInvokeResult = PInvoke.SetPeakSearchAuto(this._handle, System.Convert.ToUInt16(Peak_Search_Auto));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns whether the peak list in the spurious measurement is calculated automatically after a measurement.
        /// 
        /// Mode(s):
        /// Spectrum Analyzer, Receiver
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:PEAKsearch|PSEarch:AUTO?
        /// </summary>
        /// <param name="Peak_Search_Auto">
        /// This control returns whether the peak list in the spurious measurement is calculated automatically after a measurement. For each range, exactly one peak value is calculated.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetPeakSearchAuto(out bool Peak_Search_Auto)
        {
            ushort Peak_Search_AutoAsUShort;
            int pInvokeResult = PInvoke.GetPeakSearchAuto(this._handle, out Peak_Search_AutoAsUShort);
            Peak_Search_Auto = System.Convert.ToBoolean(Peak_Search_AutoAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function adds a frequency value to the peaklist in receiver mode. Only frequencies in the currently displayed frequency range of the receiver scan are allowed.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:PEAKsearch|PSEarch:ADD
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Frequency_Addition">
        /// This control adds a frequency value to the peaklist in receiver mode. Only frequencies in the currently displayed frequency range of the receiver scan are allowed.
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: 0.0 Hz
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetPeakSearchAddition(int Window, double Frequency_Addition)
        {
            int pInvokeResult = PInvoke.SetPeakSearchAddition(this._handle, Window, Frequency_Addition);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the method that is used to determine the maxima of a scan.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:PEAKsearch|PSEarch:METHod SUBRange | PEAK
        /// </summary>
        /// <param name="Search_Method">
        /// This control defines the method that is used to determine the maxima of a scan.
        /// 
        /// Valid Values:
        /// RSESCI_SEARCH_SUBR (0) - Subrange
        /// RSESCI_SEARCH_PEAK (1) - Peak
        /// 
        /// Default Value: RSESCI_SEARCH_PEAK (1)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetPeakSearchMethod(int Search_Method)
        {
            int pInvokeResult = PInvoke.SetPeakSearchMethod(this._handle, Search_Method);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the method that is used to determine the maxima of a scan.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:PEAKsearch|PSEarch:METHod?
        /// </summary>
        /// <param name="Search_Method">
        /// This control returns the method that is used to determine the maxima of a scan.
        /// 
        /// Valid Values:
        /// RSESCI_SEARCH_SUBR (0) - Subrange
        /// RSESCI_SEARCH_PEAK (1) - Peak
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetPeakSearchMethod(out int Search_Method)
        {
            int pInvokeResult = PInvoke.GetPeakSearchMethod(this._handle, out Search_Method);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function determines the margin (distance to the limit line) for generating the peak list. This parameter is not used unless a limit line has been assigned to the trace. The margin, which can be individually assigned to each limit line in the limit line menu, is not used when generating the peak list.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:PEAKsearch|PSEarch:MARGin
        /// </summary>
        /// <param name="Margin">
        /// This control determines the margin (distance to the limit line) for generating the peak list.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: 6.0 dB
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetPeakSearchMargin(double Margin)
        {
            int pInvokeResult = PInvoke.SetPeakSearchMargin(this._handle, Margin);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the margin (distance to the limit line) for generating the peak list.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:PEAKsearch|PSEarch:MARGin?
        /// </summary>
        /// <param name="Margin">
        /// This control returns the margin (distance to the limit line) for generating the peak list.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetPeakSearchMargin(out double Margin)
        {
            int pInvokeResult = PInvoke.GetPeakSearchMargin(this._handle, out Margin);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function determines the maximum number of subranges or peaks obtained when generating the peak list.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:PEAKsearch|PSEarch:SUBRanges
        /// </summary>
        /// <param name="Subranges">
        /// This control determines the maximum number of subranges or peaks obtained when generating the peak list.
        /// 
        /// Valid Range: 1 to 500
        /// 
        /// Default Value: 25
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetPeakSearchSubranges(int Subranges)
        {
            int pInvokeResult = PInvoke.SetPeakSearchSubranges(this._handle, Subranges);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the maximum number of subranges or peaks obtained when generating the peak list.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:PEAKsearch|PSEarch:SUBRanges?
        /// </summary>
        /// <param name="Subranges">
        /// This control returns the maximum number of subranges or peaks obtained when generating the peak list.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetPeakSearchSubranges(out int Subranges)
        {
            int pInvokeResult = PInvoke.GetPeakSearchSubranges(this._handle, out Subranges);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches on or off the indication of the peak list or the final measurement result on the trace.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACE&lt;1...3&gt;:SYMBol CROSs | OFF
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Trace">
        /// This control sets the trace.
        /// 
        /// Valid Range: 1 to 3
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Indication">
        /// This control selects the indication symbol of the peak list or the final measurement result on the trace.
        /// 
        /// Valid Values:
        /// RSESCI_SYMBOL_OFF   (0) - Off
        /// RSESCI_SYMBOL_CROSS (1) - Cross
        /// 
        /// Default Value: RSESCI_SYMBOL_OFF (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetPeakSearchListDisplay(int Window, int Trace, int Indication)
        {
            int pInvokeResult = PInvoke.SetPeakSearchListDisplay(this._handle, Window, Trace, Indication);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries selected indication symbol of the peak list or the final measurement result on the trace.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACE&lt;1...3&gt;:SYMBol?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Trace">
        /// This control sets the trace.
        /// 
        /// Valid Range: 1 to 3
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Indication">
        /// This control returns selected indication symbol of the peak list or the final measurement result on the trace.
        /// 
        /// Valid Values:
        /// RSESCI_SYMBOL_OFF   (0) - Off
        /// RSESCI_SYMBOL_CROSS (1) - Cross
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetPeakSearchListDisplay(int Window, int Trace, out int Indication)
        {
            int pInvokeResult = PInvoke.GetPeakSearchListDisplay(this._handle, Window, Trace, out Indication);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function clears the peaklist.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:PEAKsearch|PSEarch:CLEar[:IMMediate]
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int ClearPeakSearchList(int Window)
        {
            int pInvokeResult = PInvoke.ClearPeakSearchList(this._handle, Window);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function activates or deactivates the THRESHOLD SCAN measurement function.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]FMEasurement:THReshold[:STATe] ON | OFF
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Threshold_Scan">
        /// This control activates or deactivates the THRESHOLD SCAN measurement function.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetThresholdScan(int Window, bool Threshold_Scan)
        {
            int pInvokeResult = PInvoke.SetThresholdScan(this._handle, Window, System.Convert.ToUInt16(Threshold_Scan));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the state of the THRESHOLD SCAN measurement function.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]FMEasurement:THReshold[:STATe]?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Threshold_Scan">
        /// This control returns the state of the THRESHOLD SCAN measurement function.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetThresholdScan(int Window, out bool Threshold_Scan)
        {
            ushort Threshold_ScanAsUShort;
            int pInvokeResult = PInvoke.GetThresholdScan(this._handle, Window, out Threshold_ScanAsUShort);
            Threshold_Scan = System.Convert.ToBoolean(Threshold_ScanAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function enables decision threshold to be used by the analyzer to discriminate between broadband and narrowband interference.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]FMEasurement:NBBB[:STATe] ON | OFF
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Discrimination_State">
        /// This control enables decision threshold to be used by the analyzer to discriminate between broadband and narrowband interference.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetNBBBDiscrimination(int Window, bool Discrimination_State)
        {
            int pInvokeResult = PInvoke.SetNBBBDiscrimination(this._handle, Window, System.Convert.ToUInt16(Discrimination_State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the state of decision threshold to be used by the analyzer to discriminate between broadband and narrowband interference.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]FMEasurement:NBBB[:STATe]?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Discrimination_State">
        /// This control returns the state of decision threshold to be used by the analyzer to discriminate between broadband and narrowband interference.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetNBBBDiscrimination(int Window, out bool Discrimination_State)
        {
            ushort Discrimination_StateAsUShort;
            int pInvokeResult = PInvoke.GetNBBBDiscrimination(this._handle, Window, out Discrimination_StateAsUShort);
            Discrimination_State = System.Convert.ToBoolean(Discrimination_StateAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function enters the decision threshold value to be used by the analyzer to discriminate between broadband and narrowband interference.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]FMEasurement:NBBB:LEVel
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Discrimination_Threshold">
        /// This control enters the decision threshold value to be used by the analyzer to discriminate between broadband and narrowband interference.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: 6.0 dB
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetNBBBDiscriminationValue(int Window, double Discrimination_Threshold)
        {
            int pInvokeResult = PInvoke.SetNBBBDiscriminationValue(this._handle, Window, Discrimination_Threshold);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the decision threshold value to be used by the analyzer to discriminate between broadband and narrowband interference.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]FMEasurement:NBBB:LEVel?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Discrimination_Threshold">
        /// This control returns the decision threshold value to be used by the analyzer to discriminate between broadband and narrowband interference.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetNBBBDiscriminationValue(int Window, out double Discrimination_Threshold)
        {
            int pInvokeResult = PInvoke.GetNBBBDiscriminationValue(this._handle, Window, out Discrimination_Threshold);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the measurement time in which the values indicated in the peak list (final measurement values) are re-checked.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]FMEasurement:TIME
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Measurement_Time">
        /// This control defines the measurement time in which the values indicated in the peak list (final measurement values) are re-checked.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: 1.0
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetFinalMeasurementTime(int Window, double Measurement_Time)
        {
            int pInvokeResult = PInvoke.SetFinalMeasurementTime(this._handle, Window, Measurement_Time);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the measurement time in which the values indicated in the peak list (final measurement values) are re-checked.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]FMEasurement:TIME?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Measurement_Time">
        /// This control returns the measurement time in which the values indicated in the peak list (final measurement values) are re-checked.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetFinalMeasurementTime(int Window, out double Measurement_Time)
        {
            int pInvokeResult = PInvoke.GetFinalMeasurementTime(this._handle, Window, out Measurement_Time);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches between automatic and interactive final measurement.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]FMEasurement:AUTO ON | OFF
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Automatic_Measurement">
        /// This control switches between automatic and interactive final measurement.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetFinalMeasurementAuto(int Window, bool Automatic_Measurement)
        {
            int pInvokeResult = PInvoke.SetFinalMeasurementAuto(this._handle, Window, System.Convert.ToUInt16(Automatic_Measurement));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the state of the final measurement.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]FMEasurement:AUTO?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Automatic_Measurement">
        /// This control the state of the final measurement.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off (Interactive)
        /// VI_TRUE  (1) - On  (Automatic)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetFinalMeasurementAuto(int Window, out bool Automatic_Measurement)
        {
            ushort Automatic_MeasurementAsUShort;
            int pInvokeResult = PInvoke.GetFinalMeasurementAuto(this._handle, Window, out Automatic_MeasurementAsUShort);
            Automatic_Measurement = System.Convert.ToBoolean(Automatic_MeasurementAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the V-network to be controlled via the user port.
        /// 
        /// Mode(s):
        /// Receiver (for both commands)
        /// Spectrum Analyzer (only for INPut&lt;1|2&gt;:LISN...)
        /// 
        /// Remote-control command(s):
        /// INPut&lt;1|2&gt;:LISN[:TYPE] TWOPhase | FOURphase | ESH3Z5 | ESH2Z5 | ENV4200 | ENV216 | OFF
        /// [SENSe&lt;1|2&gt;:]FMEasurement:LISN[:TYPE] TWOPhase | FOURphase | ESH3Z5 | ESH2Z5 | ENV4200 | ENV216 | OFF
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Measurement">
        /// This control selects the receiver measurement type.
        /// 
        /// Valid Values:
        /// RSESCI_VNET_MEAS_PRESCAN (0) - Prescan
        /// RSESCI_VNET_MEAS_FINAL   (1) - Final
        /// 
        /// Default Value: RSESCI_VNET_MEAS_PRESCAN (0)
        /// </param>
        /// <param name="V_network">
        /// This control selects the V-network to be controlled via user port.
        /// 
        /// Valid Values:
        /// RSESCI_VNET_OFF     (0) - Off, remote control is deactivated
        /// RSESCI_VNET_TWO     (1) - Two-line V-network R&amp;S ESH3-Z5 
        /// RSESCI_VNET_FOUR    (2) - Four-line V-network R&amp;S ESH2-Z5 or R&amp;S ENV4200
        /// RSESCI_VNET_ESH3Z5  (3) - ESH3 Z5
        /// RSESCI_VNET_ESH2Z5  (4) - ESH2 Z5
        /// RSESCI_VNET_ENV4200 (5) - ENV 4200
        /// RSESCI_VNET_ENV216  (6) - ENV 216
        /// 
        /// Default Value: RSESCI_VNET_OFF (0)
        /// 
        /// Note(s):
        /// 
        /// (1) The parameters ESH3 Z5, ESH2 Z5, ENV 4200 and ENV 216 are available only with ESU instrument.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetVnetworkType(int Window, int Measurement, int V_network)
        {
            int pInvokeResult = PInvoke.SetVnetworkType(this._handle, Window, Measurement, V_network);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries selected V-network controlled via the user port.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// INPut&lt;1|2&gt;:LISN[:TYPE]?
        /// [SENSe&lt;1|2&gt;:]FMEasurement:LISN[:TYPE]?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Measurement">
        /// This control selects the receiver measurement type.
        /// 
        /// Valid Values:
        /// RSESCI_VNET_MEAS_PRESCAN (0) - Prescan
        /// RSESCI_VNET_MEAS_FINAL   (1) - Final
        /// 
        /// Default Value: RSESCI_VNET_MEAS_PRESCAN (0)
        /// </param>
        /// <param name="V_network">
        /// This control returns selected the V-network controlled via user port.
        /// 
        /// Valid Values:
        /// RSESCI_VNET_OFF     (0) - Off, remote control is deactivated
        /// RSESCI_VNET_TWO     (1) - Two-line V-network R&amp;S ESH3-Z5 
        /// RSESCI_VNET_FOUR    (2) - Four-line V-network R&amp;S ESH2-Z5 or R&amp;S ENV4200
        /// RSESCI_VNET_ESH3Z5  (3) - ESH3 Z5
        /// RSESCI_VNET_ESH2Z5  (4) - ESH2 Z5
        /// RSESCI_VNET_ENV4200 (5) - ENV 4200
        /// RSESCI_VNET_ENV216  (6) - ENV 216
        /// 
        /// Note(s):
        /// 
        /// (1) The parameters ESH3 Z5, ESH2 Z5, ENV 4200 and ENV 216 are available only with ESU instrument.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetVnetworkType(int Window, int Measurement, out int V_network)
        {
            int pInvokeResult = PInvoke.GetVnetworkType(this._handle, Window, Measurement, out V_network);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the setting of the protection earth of the V-network.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// INPut&lt;1|2&gt;:LISN:PEARth GROunded | FLOating
        /// [SENSe&lt;1|2&gt;:]FMEasurement:LISN:PEARth GROunded | FLOating
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Measurement">
        /// This control selects the receiver measurement type.
        /// 
        /// Valid Values:
        /// RSESCI_VNET_MEAS_PRESCAN (0) - Prescan
        /// RSESCI_VNET_MEAS_FINAL   (1) - Final
        /// 
        /// Default Value: RSESCI_VNET_MEAS_PRESCAN (0)
        /// </param>
        /// <param name="Protective_Earth">
        /// This control selects the setting of the protection earth of the V-network.
        /// 
        /// Valid Values:
        /// RSESCI_EARTH_GROUNDED (0) - Grounded
        /// RSESCI_EARTH_FLOATING (1) - Floating
        /// 
        /// Default Value: RSESCI_EARTH_GROUNDED (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetVnetworkProtectionEarth(int Window, int Measurement, int Protective_Earth)
        {
            int pInvokeResult = PInvoke.SetVnetworkProtectionEarth(this._handle, Window, Measurement, Protective_Earth);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries selected setting of the protection earth of the V-network.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// INPut&lt;1|2&gt;:LISN:PEARth?
        /// [SENSe&lt;1|2&gt;:]FMEasurement:LISN:PEARth?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Measurement">
        /// This control selects the receiver measurement type.
        /// 
        /// Valid Values:
        /// RSESCI_VNET_MEAS_PRESCAN (0) - Prescan
        /// RSESCI_VNET_MEAS_FINAL   (1) - Final
        /// 
        /// Default Value: RSESCI_VNET_MEAS_PRESCAN (0)
        /// </param>
        /// <param name="Protective_Earth">
        /// This control returns selected setting of the protection earth of the V-network.
        /// 
        /// Valid Values:
        /// RSESCI_EARTH_GROUNDED (0) - Grounded
        /// RSESCI_EARTH_FLOATING (1) - Floating
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetVnetworkProtectionEarth(int Window, int Measurement, out int Protective_Earth)
        {
            int pInvokeResult = PInvoke.GetVnetworkProtectionEarth(this._handle, Window, Measurement, out Protective_Earth);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the ground settings on the V-network on which level measurements are consecutively performed in the final measurement. This function is available only for certain
        /// V-networks (FMEasurement:LISN FOURphase | TWOPhase | ESH3Z5 | ESH2Z5).
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]FMEasurement:LISN:PEARth GROunded | FLOating[,GROunded | FLOating]
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Number_Of_Protectives">
        /// This control sets the number of protectives.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Protective_Earth">
        /// This control selects the ground settings on the V-network on which level measurements are consecutively performed in the final measurement.
        /// 
        /// Valid Values:
        /// RSESCI_EARTH_GROUNDED (0) - Grounded
        /// RSESCI_EARTH_FLOATING (1) - Floating
        /// 
        /// Default Value: None
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetVnetworkProtectionEarthFinalMeasurement(int Window, int Number_Of_Protectives, int[] Protective_Earth)
        {
            int pInvokeResult = PInvoke.SetVnetworkProtectionEarthFinalMeasurement(this._handle, Window, Number_Of_Protectives, Protective_Earth);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the ground settings on the V-network on which level measurements are consecutively performed in the final measurement.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]FMEasurement:LISN:PEARth?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Number_Of_Protectives">
        /// This control gets the number of protectives.
        /// 
        /// Valid Values:
        /// 1 to 2
        /// </param>
        /// <param name="Protective_Earth">
        /// This control returns the ground settings on the V-network on which level measurements are consecutively performed in the final measurement.
        /// 
        /// Valid Values:
        /// RSESCI_EARTH_GROUNDED (0) - Grounded
        /// RSESCI_EARTH_FLOATING (1) - Floating
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetVnetworkProtectionEarthFinalMeasurement(int Window, out int Number_Of_Protectives, int[] Protective_Earth)
        {
            int pInvokeResult = PInvoke.GetVnetworkProtectionEarthFinalMeasurement(this._handle, Window, out Number_Of_Protectives, Protective_Earth);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the phase of the V-network on which the RFI voltage is to be measured.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// INPut&lt;1|2&gt;:LISN:PHASe L1 | L2 | L3 | N
        /// [SENSe&lt;1|2&gt;:]FMEasurement:LISN:PHASe L1 | L2 | L3 | N
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Measurement">
        /// This control selects the receiver measurement type.
        /// 
        /// Valid Values:
        /// RSESCI_VNET_MEAS_PRESCAN (0) - Prescan
        /// RSESCI_VNET_MEAS_FINAL   (1) - Final
        /// 
        /// Default Value: RSESCI_VNET_MEAS_PRESCAN (0)
        /// </param>
        /// <param name="Phase">
        /// This control selects the phase of the V-network on which the RFI voltage is to be measured.
        /// 
        /// Valid Values:
        /// RSESCI_PHASE_N  (0) - N
        /// RSESCI_PHASE_L1 (1) - L1
        /// RSESCI_PHASE_L2 (2) - L2
        /// RSESCI_PHASE_L3 (3) - L3
        /// 
        /// Default Value: RSESCI_PHASE_N (0)
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetVnetworkPhase(int Window, int Measurement, int Phase)
        {
            int pInvokeResult = PInvoke.SetVnetworkPhase(this._handle, Window, Measurement, Phase);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries selected phase of the V-network on which the RFI voltage is to be measured.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// INPut&lt;1|2&gt;:LISN:PHASe?
        /// [SENSe&lt;1|2&gt;:]FMEasurement:LISN:PHASe?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Measurement">
        /// This control selects the receiver measurement type.
        /// 
        /// Valid Values:
        /// RSESCI_VNET_MEAS_PRESCAN (0) - Prescan
        /// RSESCI_VNET_MEAS_FINAL   (1) - Final
        /// 
        /// Default Value: RSESCI_VNET_MEAS_PRESCAN (0)
        /// </param>
        /// <param name="Phase">
        /// This control returns selected phase of the V-network on which the RFI voltage is to be measured.
        /// 
        /// Valid Values:
        /// RSESCI_PHASE_N  (0) - N
        /// RSESCI_PHASE_L1 (1) - L1
        /// RSESCI_PHASE_L2 (2) - L2
        /// RSESCI_PHASE_L3 (3) - L3
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetVnetworkPhase(int Window, int Measurement, out int Phase)
        {
            int pInvokeResult = PInvoke.GetVnetworkPhase(this._handle, Window, Measurement, out Phase);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the phases of the V-network on which level measurements are consecutively performed in the final measurement. L2 and L3 are only available with four-line V-networks (FMEasurement:LISN FOURphase | ENV4200).
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]FMEasurement:LISN:PHASe L1 | L2 | L3 | N[[,L1 | L2 | L3 | N],...]
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Number_Of_Phases">
        /// This control sets the number of phases.
        /// 
        /// Valid Values:
        /// &gt;1
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Phases">
        /// This control sets the phases of the V-network on which level measurements are consecutively performed in the final measurement.
        /// 
        /// Valid Values:
        /// RSESCI_PHASE_N  (0) - N
        /// RSESCI_PHASE_L1 (1) - L1
        /// RSESCI_PHASE_L2 (2) - L2
        /// RSESCI_PHASE_L3 (3) - L3
        /// 
        /// Default Value: None
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetVnetworkPhaseFinalMeasurement(int Window, int Number_Of_Phases, int[] Phases)
        {
            int pInvokeResult = PInvoke.SetVnetworkPhaseFinalMeasurement(this._handle, Window, Number_Of_Phases, Phases);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the phases of the V-network on which level measurements are consecutively performed in the final measurement. L2 and L3 are only available with four-line V-networks (FMEasurement:LISN FOURphase | ENV4200).
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]FMEasurement:LISN:PHASe?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Number_Of_Phases">
        /// This control gets the number of phases.
        /// 
        /// Valid Values:
        /// &gt;1
        /// </param>
        /// <param name="Phases">
        /// This control returns the phases of the V-network on which level measurements are consecutively performed in the final measurement.
        /// 
        /// Valid Values:
        /// RSESCI_PHASE_N  (0) - N
        /// RSESCI_PHASE_L1 (1) - L1
        /// RSESCI_PHASE_L2 (2) - L2
        /// RSESCI_PHASE_L3 (3) - L3
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetVnetworkPhaseFinalMeasurement(int Window, out int Number_Of_Phases, int[] Phases)
        {
            int pInvokeResult = PInvoke.GetVnetworkPhaseFinalMeasurement(this._handle, Window, out Number_Of_Phases, Phases);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the setting for the high pass on the V-network.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only for the R&amp;S ENV 216 V-network (INP:LISN ENV216) - rsesci_SetVnetworkType.
        /// 
        /// (2) This function is available only with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// INPut&lt;1|2&gt;:LISN:FILTer:HPAS[:STATe] ON | OFF
        /// [SENSe&lt;1|2&gt;:]FMEasurement:LISN:FILTer:HPAS[:STATe] ON | OFF
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Measurement">
        /// This control selects the receiver measurement type.
        /// 
        /// Valid Values:
        /// RSESCI_VNET_MEAS_PRESCAN (0) - Prescan
        /// RSESCI_VNET_MEAS_FINAL   (1) - Final
        /// 
        /// Default Value: RSESCI_VNET_MEAS_PRESCAN (0)
        /// </param>
        /// <param name="Phase">
        /// This control selects the setting for the high pass on the V-network.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_TRUE (1)
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetVnetworkHighPass(int Window, int Measurement, bool Phase)
        {
            int pInvokeResult = PInvoke.SetVnetworkHighPass(this._handle, Window, Measurement, System.Convert.ToUInt16(Phase));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the setting of the high pass on the V-network.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only for the R&amp;S ENV 216 V-network (INP:LISN ENV216) - rsesci_SetVnetworkType.
        /// 
        /// (2) This function is available only with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// INPut&lt;1|2&gt;:LISN:FILTer:HPAS[:STATe]?
        /// [SENSe&lt;1|2&gt;:]FMEasurement:LISN:FILTer:HPAS[:STATe]?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Measurement">
        /// This control selects the receiver measurement type.
        /// 
        /// Valid Values:
        /// RSESCI_VNET_MEAS_PRESCAN (0) - Prescan
        /// RSESCI_VNET_MEAS_FINAL   (1) - Final
        /// 
        /// Default Value: RSESCI_VNET_MEAS_PRESCAN (0)
        /// </param>
        /// <param name="Phase">
        /// This control returns the setting of the high pass on the V-network.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetVnetworkHighPass(int Window, int Measurement, out bool Phase)
        {
            ushort PhaseAsUShort;
            int pInvokeResult = PInvoke.GetVnetworkHighPass(this._handle, Window, Measurement, out PhaseAsUShort);
            Phase = System.Convert.ToBoolean(PhaseAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the type of analog demodulation.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]DEMod OFF | AM | FM
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Demodulation">
        /// This control selects the type of analog demodulation.
        /// 
        /// Valid Values:
        /// RSESCI_DEMOD_OFF (0) - Off
        /// RSESCI_DEMOD_AM  (1) - AM
        /// RSESCI_DEMOD_FM  (2) - FM
        /// 
        /// Default Value: RSESCI_DEMOD_OFF (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetDemodulationType(int Window, int Demodulation)
        {
            int pInvokeResult = PInvoke.SetDemodulationType(this._handle, Window, Demodulation);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries selected type of analog demodulation.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]DEMod?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Demodulation">
        /// This control returns selected type of analog demodulation.
        /// 
        /// Valid Values:
        /// RSESCI_DEMOD_OFF (0) - Off
        /// RSESCI_DEMOD_AM  (1) - AM
        /// RSESCI_DEMOD_FM  (2) - FM
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetDemodulationType(int Window, out int Demodulation)
        {
            int pInvokeResult = PInvoke.GetDemodulationType(this._handle, Window, out Demodulation);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function enables the input of a level threshold below which the audible AF is cut off. The squelch function is associated with the internal trigger function (TRIGGER menu), which will be switched on automatically with the squelch. Squelch level and trigger level do have the same value.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]DEMod:SQUelch[:STATe] ON | OFF
        /// [SENSe&lt;1|2&gt;:]DEMod:SQUelch:LEVel
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Squelch_State">
        /// This control enables the input of a level threshold below which the audible AF is cut off.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <param name="Squelch_Level">
        /// This control sets the squelch level.
        /// 
        /// Valid Values:
        /// 0.0 to 100.0 %
        /// 
        /// Default Value: 50.0 %
        /// 
        /// Note(s):
        /// 
        /// (1) This control is ignored if the squelch state is set to Off.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetSquelchReceiver(int Window, bool Squelch_State, double Squelch_Level)
        {
            int pInvokeResult = PInvoke.SetSquelchReceiver(this._handle, Window, System.Convert.ToUInt16(Squelch_State), Squelch_Level);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the input of a level threshold below which the audible AF is cut off.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]DEMod:SQUelch[:STATe]?
        /// [SENSe&lt;1|2&gt;:]DEMod:SQUelch:LEVel?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Squelch_State">
        /// This control returns the input of a level threshold below which the audible AF is cut off.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <param name="Squelch_Level">
        /// This control gets the squelch level.
        /// 
        /// Valid Values:
        /// 0.0 to 100.0 %
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetSquelchReceiver(int Window, out bool Squelch_State, out double Squelch_Level)
        {
            ushort Squelch_StateAsUShort;
            int pInvokeResult = PInvoke.GetSquelchReceiver(this._handle, Window, out Squelch_StateAsUShort, out Squelch_Level);
            Squelch_State = System.Convert.ToBoolean(Squelch_StateAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function is possible to use only as a query for single measurements in the receiver mode. The values of all activated detectors are transferred. For inactive detectors, the value is omitted at the position(s) in question.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// FORMat[:DATA] ASCii
        /// TRACe&lt;1|2&gt;:DATA? SINGle
        /// </summary>
        /// <param name="First_Detector">
        /// Returns the value of first activated detector. Inactive detector returns zero.
        /// 
        /// </param>
        /// <param name="Second_Detector">
        /// Returns the value of second activated detector. Inactive detector returns zero.
        /// 
        /// </param>
        /// <param name="Third_Detector">
        /// Returns the value of third activated detector. Inactive detector returns zero.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int ReadReceiverDetector(out double First_Detector, out double Second_Detector, out double Third_Detector)
        {
            int pInvokeResult = PInvoke.ReadReceiverDetector(this._handle, out First_Detector, out Second_Detector, out Third_Detector);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function is possible to use only as a query for single measurements in the receiver mode. The peak hold values of all activated detectors are transferred. For inactive detectors, the value is omitted at the position(s) in question.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// FORMat[:DATA] ASCii
        /// TRACe&lt;1|2&gt;:DATA? PHOLd
        /// </summary>
        /// <param name="First_Detector">
        /// Returns the peak hold (level, frequency) values of first activated detector. Inactive detector returns zero.
        /// 
        /// Note(s):
        /// 
        /// (1) The array should consist of at least 2 elements.
        /// 
        /// </param>
        /// <param name="Second_Detector">
        /// Returns the peak hold (level, frequency) values of second activated detector. Inactive detector returns zero.
        /// 
        /// Note(s):
        /// 
        /// (1) The array should consist of at least 2 elements.
        /// 
        /// </param>
        /// <param name="Third_Detector">
        /// Returns the peak hold (level, frequency) values of third activated detector. Inactive detector returns zero.
        /// 
        /// Note(s):
        /// 
        /// (1) The array should consist of at least 2 elements.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int ReadReceiverDetectorPeakHold(double[] First_Detector, double[] Second_Detector, double[] Third_Detector)
        {
            int pInvokeResult = PInvoke.ReadReceiverDetectorPeakHold(this._handle, First_Detector, Second_Detector, Third_Detector);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the result of the scan measurement. The number of transmitted measurement results depends on the scan settings.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// FORMat[:DATA] REAL,32
        /// TRACe&lt;1|2&gt;:DATA? SCAN
        /// </summary>
        /// <param name="Array_Size">
        /// Pass the number of elements in the arrays you specify as the array output parameters.
        /// 
        /// If the data returned contains more elements that you indicate in this parameter, this function copies 'Array Size' elements into the output parameters.
        /// 
        /// </param>
        /// <param name="Trace_Status">
        /// Returns the trace status information (4 byte). Structure of data is:
        /// 
        /// - Bit 0 to 9: subscan
        /// - Bit 10: last block of subscan
        /// - Bit 11: last block of last subscan of scan
        /// - Bit 12: last of all blocks (for multiple scans after the
        ///           last scan)
        /// 
        /// </param>
        /// <param name="Number_of_Results">
        /// Returns the number of the transmitted measurement results of a trace.
        /// 
        /// </param>
        /// <param name="Status_Data">
        /// Returns the status information per measurement result during scan measurements.
        /// 
        /// One byte status information per measurement result is transmitted:
        /// 
        ///   bit 0: underrange trace1
        ///   bit 1: underrange trace2
        ///   bit 2: underrange trace3
        ///   bit 3: overrange trace1 to trace3
        /// 
        /// Note(s):
        /// 
        /// (1) The array should consist of at least the number of elements defined with 'Array Size' parameter.
        /// 
        /// </param>
        /// <param name="Results_of_Trace_1">
        /// Returns an array of measurement results  of trace 1 if trace 1 is active.
        /// 
        /// Note(s):
        /// 
        /// (1) The array should consist of at least the number of elements defined with 'Array Size' parameter.
        /// 
        /// </param>
        /// <param name="Results_of_Trace_2">
        /// Returns an array of measurement results of trace 2 if trace 2 is active.
        /// 
        /// Note(s):
        /// 
        /// (1) The array should consist of at least the number of elements defined with 'Array Size' parameter.
        /// 
        /// </param>
        /// <param name="Results_of_Trace_3">
        /// Returns an array of measurement results  of trace 3 if trace 3 is active.
        /// 
        /// Note(s):
        /// 
        /// (1) The array should consist of at least the number of elements defined with 'Array Size' parameter.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int ReadScanMeasurement(int Array_Size, out int Trace_Status, out int Number_of_Results, System.Text.StringBuilder Status_Data, double[] Results_of_Trace_1, double[] Results_of_Trace_2, double[] Results_of_Trace_3)
        {
            int pInvokeResult = PInvoke.ReadScanMeasurement(this._handle, Array_Size, out Trace_Status, out Number_of_Results, Status_Data, Results_of_Trace_1, Results_of_Trace_2, Results_of_Trace_3);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the status information per measurement result during scan measurements.
        /// 
        /// 1 byte status information per measurement result is transmitted:
        /// bit 0: underrange trace1;
        /// bit 1: underrange trace2;
        /// bit 2: underrange trace3;
        /// bit 3: overrange trace1 to trace3
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// FORMat[:DATA] UINT,8
        /// TRACe&lt;1|2&gt;:DATA? STATus
        /// </summary>
        /// <param name="Array_Size">
        /// Pass the number of bytes in the ViChar array you specify for the 'Status Data' parameter.  
        /// 
        /// If the status data, contains more bytes that you indicate in this parameter, the function copies 'Array Size' bytes into the buffer and as 'Number of Results' returns value you must pass to get entire buffer.
        /// 
        /// If you pass a negative number, the function copies the value to the buffer regardless of the number of bytes in the value.
        /// 
        /// If you pass 0, you can pass VI_NULL for the 'Status Data' array parameter.
        /// 
        /// </param>
        /// <param name="Number_of_Results">
        /// Returns the number of measurement results.
        /// 
        /// </param>
        /// <param name="Status_Data">
        /// Returns the status information per measurement result during scan measurements.
        /// 
        /// One byte status information per measurement result is transmitted:
        /// 
        ///   bit 0: underrange trace1
        ///   bit 1: underrange trace2
        ///   bit 2: underrange trace3
        ///   bit 3: overrange trace1 to trace3
        /// 
        /// Note(s):
        /// 
        /// (1) The array should consist of at least the number of elements defined with 'Array Size' parameter.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int ReadScanMeasurementStatus(int Array_Size, out int Number_of_Results, System.Text.StringBuilder Status_Data)
        {
            int pInvokeResult = PInvoke.ReadScanMeasurementStatus(this._handle, Array_Size, out Number_of_Results, Status_Data);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the results of the final scan measurement.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// FORMat[:DATA] ASCii
        /// TRACe&lt;1|2&gt;:DATA? FINAL1| FINAL2| FINAL3
        /// </summary>
        /// <param name="Trace">
        /// This control sets the trace.
        /// 
        /// Valid Range: 1 to 3
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Array_Size">
        /// Pass the number of elements in the arrays you specify as the array output parameters.
        /// 
        /// If the data returned contains more elements that you indicate in this parameter, this function copies only 'Array Size' elements into the output parameters.
        /// 
        /// Valid Range: &gt; 0
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <param name="Number_of_Results">
        /// Returns the number of the transmitted measurement results.
        /// 
        /// </param>
        /// <param name="Frequency">
        /// Returns an array of frequencies.
        /// 
        /// Note(s):
        /// 
        /// (1) The array should consist of at least the number of elements defined with 'Array Size' parameter.
        /// 
        /// </param>
        /// <param name="Level">
        /// Returns an array of levels.
        /// 
        /// Note(s):
        /// 
        /// (1) The array should consist of at least the number of elements defined with 'Array Size' parameter.
        /// 
        /// </param>
        /// <param name="Delta_Limit">
        /// Returns an array of delta limits.
        /// 
        /// Note(s):
        /// 
        /// (1) The array should consist of at least the number of elements defined with 'Array Size' parameter.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int ReadFinalMeasurement(int Trace, int Array_Size, out int Number_of_Results, double[] Frequency, double[] Level, double[] Delta_Limit)
        {
            int pInvokeResult = PInvoke.ReadFinalMeasurement(this._handle, Trace, Array_Size, out Number_of_Results, Frequency, Level, Delta_Limit);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns data that represent the values in array.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// TRACe&lt;1|2&gt;:DATA? ABITstream | PWCDp
        /// :FORM REAL,32
        /// *CLS
        /// 
        /// 
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Trace">
        /// Selects the type of measurement to be read out.
        /// 
        /// Valid Values:
        /// RSESCI_TRAC_ABIT (0) - Frame Bit-Stream
        /// RSESCI_TRAC_PWCD (1) - Code Power (PWCD)
        /// 
        /// Default Value: RSESCI_TRAC_ABIT (0)
        /// </param>
        /// <param name="Array_Length">
        /// Number of array points requested.
        /// 
        /// Default Value: None
        /// </param>
        /// <param name="Actual_Points">
        /// Number of points actually returned in the Values array.
        /// </param>
        /// <param name="Values">
        /// A user allocated buffer into which the trace values are stored.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation.  The status code  either indicates success or describes an error or warning condition.  You examine the status code from each call to an instrument driver function to determine if an error occurred.
        /// 
        /// To obtain a text description of the status code, call the rsspecan_error_message function.  To obtain additional information about the error condition, call the rsspecan_GetError function.  To clear the error information from the driver, call the rsspecan_ClearError function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// WARNINGS:
        /// 3FFA2001  The instrument was in an uncalibrated state
        ///           when the measurement was taken.
        /// 3FFA2002  The measurement taken was over the instrument??s
        ///           range.
        ///   
        /// ERRORS:
        /// BFFA2001  The Active Marker is not enabled.
        /// BFFA2002  The Active Marker is not a delta marker.
        /// BFFA2003  The maximum waiting time for this operation was
        ///           exceeded.
        ///           
        /// This instrument driver also returns errors and warnings defined by other sources.  The following table defines the ranges of additional status codes that this driver can return.  The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFA0000 to 3FFA1FFF     IVI    Warnings
        /// 3FFF0000 to 3FFFFFFF     VISA   Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP Driver Warnings
        ///           
        /// BFFA0000 to BFFA1FFF     IVI    Errors
        /// BFFF0000 to BFFFFFFF     VISA   Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP Driver Errors
        /// 
        /// </returns>
        public int ReadAdditionalMeasurementData(int Window, int Trace, int Array_Length, out int Actual_Points, double[] Values)
        {
            int pInvokeResult = PInvoke.ReadAdditionalMeasurementData(this._handle, Window, Trace, Array_Length, out Actual_Points, Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the current X and Y results of the sweep. During a sweep the last measured values are read out.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// TRACe&lt;1|2&gt;:IMMediate:RESult?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="X_Value">
        /// Returns the X value result of the sweep.
        /// 
        /// </param>
        /// <param name="Y_Value">
        /// Returns the Y value result of the sweep.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code
        /// either indicates success or describes an error or warning
        /// condition. You examine the status code from each call to an
        /// instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the
        /// rsfsq_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        /// 
        /// Status    Description
        /// -------------------------------------------------
        /// 
        /// BFFC0002  Parameter 2 (Source Trace) out of range.
        /// BFFC0003  Parameter 3 (Destination Trace) out of range.
        /// 
        /// BFFC09F0  Instrument status error
        /// BFFC09F1  Option not installed
        /// BFFC09F2  Conflict with setup parameters
        /// BFFC09F3  Data are not available
        /// BFFC09F4  Unknown option
        /// BFFC09F5  Calibration failed
        /// BFFC09F6  Function does not support FSEx parameter value
        /// BFFC09F7  Overload. Instrument does not return I/Q values
        /// 
        /// This instrument driver also returns errors and warnings defined
        /// by other sources. The following table defines the ranges of
        /// additional status codes that this driver can return. The table
        /// lists the different include files that contain the defined
        /// constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetCurrentResults(int Window, out double X_Value, out double Y_Value)
        {
            int pInvokeResult = PInvoke.GetCurrentResults(this._handle, Window, out X_Value, out Y_Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the current Y results of the sweep. During a sweep the last measured value is read out.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// TRACe&lt;1|2&gt;:IMMediate:LEVel?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Level">
        /// Returns the current Y results of the sweep.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code
        /// either indicates success or describes an error or warning
        /// condition. You examine the status code from each call to an
        /// instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the
        /// rsfsq_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        /// 
        /// Status    Description
        /// -------------------------------------------------
        /// 
        /// BFFC0002  Parameter 2 (Source Trace) out of range.
        /// BFFC0003  Parameter 3 (Destination Trace) out of range.
        /// 
        /// BFFC09F0  Instrument status error
        /// BFFC09F1  Option not installed
        /// BFFC09F2  Conflict with setup parameters
        /// BFFC09F3  Data are not available
        /// BFFC09F4  Unknown option
        /// BFFC09F5  Calibration failed
        /// BFFC09F6  Function does not support FSEx parameter value
        /// BFFC09F7  Overload. Instrument does not return I/Q values
        /// 
        /// This instrument driver also returns errors and warnings defined
        /// by other sources. The following table defines the ranges of
        /// additional status codes that this driver can return. The table
        /// lists the different include files that contain the defined
        /// constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetCurrentYResult(int Window, out double Level)
        {
            int pInvokeResult = PInvoke.GetCurrentYResult(this._handle, Window, out Level);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the points (channels) to be measured in the selected scan range by specifying the channel index. 
        /// 
        /// Note:
        /// (1) The channel lists are lost when the instrument is switched off. To save it use Channel List Store.
        /// (2) This function is only valid in conjunction with the option R&amp;S ESPI-K50 option.
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SCAN&lt;1...10&gt;:LIST[:SEQUence] &lt;numeric_value&gt;{,&lt;numeric_value&gt;}
        /// </summary>
        /// <param name="No_Of_Values">
        /// This control sets the number of values returned in arrays.
        /// The array size has to be at least equal to this number.
        /// 
        /// Valid Values: &gt; 0
        /// 
        /// Default Value: 10
        /// 
        /// </param>
        /// <param name="Scan_Table">
        /// Selects configured channel list.
        /// 
        /// Valid Values: 1 to 10
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <param name="Channel_List">
        /// The actual frequency is obtained as follows:
        /// 
        ///     start frequency of scan range + (index - 1) x step width.
        /// 
        /// If there are no entries in the list, all frequencies defined in the range are measured. A maximum of 1000 channels can be programmed for each scan range.
        /// 
        /// Example: 
        /// {1,5,15,234} defines the channels for channel list:
        ///   1 = start frequency,
        ///   5 = start frequency + 4 x step width
        ///   15 = start frequency + 14 x step width
        ///   234 = start frequency + 233 x step width
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetChannelList(int No_Of_Values, int Scan_Table, int[] Channel_List)
        {
            int pInvokeResult = PInvoke.SetChannelList(this._handle, No_Of_Values, Scan_Table, Channel_List);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the points (channels) to be measured in the selected scan range by specifying the channel index. 
        /// 
        /// Note:
        /// This function is only valid in conjunction with the option R&amp;S ESPI-K50 option.
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SCAN&lt;1...10&gt;:LIST[:SEQUence]?
        /// </summary>
        /// <param name="Array_Size">
        /// This control sets the number of values returned in arrays.
        /// The array size has to be at least equal to this number.
        /// 
        /// Valid Values: &gt; 0
        /// 
        /// Default Value: 10
        /// 
        /// </param>
        /// <param name="Scan_Table">
        /// Selects configured channel list.
        /// 
        /// Valid Values: 1 to 10
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <param name="Channel_List">
        /// Shows the actual scan table.
        /// </param>
        /// <param name="Returned_Values">
        /// Returns number of value read (available to return) from the instrument. Number of values returned in arrays is limited with the parameter Array Size.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetChannelList(int Array_Size, int Scan_Table, int[] Channel_List, out int Returned_Values)
        {
            int pInvokeResult = PInvoke.GetChannelList(this._handle, Array_Size, Scan_Table, Channel_List, out Returned_Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches between a continuous scan and measurements according to the channel list for the selected range
        /// 
        /// Note:
        /// This function is only valid in conjunction with the option R&amp;S ESPI-K50 option.
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SCAN&lt;1...10&gt;:LIST:STATe ON | OFF
        /// </summary>
        /// <param name="Scan_Table">
        /// Selects configured channel list.
        /// 
        /// Valid Values: 1 to 10
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <param name="Scan">
        /// This control defines whether channel list is scanned.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetChannelListScan(int Scan_Table, bool Scan)
        {
            int pInvokeResult = PInvoke.SetChannelListScan(this._handle, Scan_Table, System.Convert.ToUInt16(Scan));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function return if a continuous scan or measurements
        /// according to the channel list for the selected range is selected.
        /// 
        /// Note:
        /// This function is only valid in conjunction with the option R&amp;S ESPI-K50 option.
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SCAN&lt;1...10&gt;:LIST:STATe?
        /// </summary>
        /// <param name="Scan_Table">
        /// Selects configured channel list.
        /// 
        /// Valid Values: 1 to 10
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <param name="Scan">
        /// This control shows whether channel list is scanned.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetChannelListScan(int Scan_Table, out bool Scan)
        {
            ushort ScanAsUShort;
            int pInvokeResult = PInvoke.GetChannelListScan(this._handle, Scan_Table, out ScanAsUShort);
            Scan = System.Convert.ToBoolean(ScanAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function adds the channel lists to the list of device settings to be stored/loaded.
        /// 
        /// Note:
        /// This function is only valid in conjunction with the option R&amp;S ESPI-K50 option.
        /// 
        /// Remote-control command(s):
        /// MMEMory:SELect[:ITEM]:CLISt ON | OFF
        /// </summary>
        /// <param name="Store">
        /// This control defines whether is channel lists stored/loaded.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetChannelListStore(bool Store)
        {
            int pInvokeResult = PInvoke.SetChannelListStore(this._handle, System.Convert.ToUInt16(Store));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns if the channel lists to the list of device
        /// settings should be stored/loaded.
        /// 
        /// Note:
        /// This function is only valid in conjunction with the option R&amp;S ESPI-K50 option.
        /// 
        /// Remote-control command(s):
        /// MMEMory:SELect[:ITEM]:CLISt?
        /// </summary>
        /// <param name="Store">
        /// This control shows whether is channel lists stored/loaded.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetChannelListStore(out bool Store)
        {
            ushort StoreAsUShort;
            int pInvokeResult = PInvoke.GetChannelListStore(this._handle, out StoreAsUShort);
            Store = System.Convert.ToBoolean(StoreAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches between transmission of single measured values and continuous measurement with blockwise data transmission. The measurement is only performed on the set frequency and is started with INIT1.
        /// 
        /// Note:
        /// This function is only valid in conjunction with the option R&amp;S ESPI-K50 option.
        /// 
        /// Remote-control command(s):
        /// TRACe&lt;1|2&gt;:POINts SINGle, &lt;numeric_value&gt;
        /// </summary>
        /// <param name="Value">
        /// The number of single measurement results that are transmitted by the query. If set to 1, transmit only the results of a single
        /// measurement. If greater than 1 activates the blockwise  transmission of the level measurement results.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: 1
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetTraceSingle(int Value)
        {
            int pInvokeResult = PInvoke.SetTraceSingle(this._handle, Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the number of single measurement results that are transmitted by the query.
        /// 
        /// Note:
        /// This function is only valid in conjunction with the option R&amp;S ESPI-K50 option.
        /// 
        /// Remote-control command(s):
        /// TRACe&lt;1|2&gt;:POINts? SINGle
        /// </summary>
        /// <param name="Value">
        /// The number of single measurement results that are transmitted by the query. If set to 1, transmit only the results of a single
        /// measurement. If greater than 1 activates the blockwise  transmission of the level measurement results.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetTraceSingle(out int Value)
        {
            int pInvokeResult = PInvoke.GetTraceSingle(this._handle, out Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches the scan between measurement on all frequencies and the use of channel lists. The measurement is started with INIT2.
        /// 
        /// Note:
        /// This function is only valid in conjunction with the option R&amp;S ESPI-K50 option.
        /// 
        /// Remote-control command(s):
        /// TRACe&lt;1|2&gt;:POINts SCAN, &lt;numeric_value&gt;
        /// </summary>
        /// <param name="Value">
        /// The number of single measurement results that are transmitted by the query. If set to 1, transmit only the results of a single
        /// measurement. If greater than 1 activates the blockwise  transmission of the level measurement results.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: 1
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetTraceScan(int Value)
        {
            int pInvokeResult = PInvoke.SetTraceScan(this._handle, Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the number of scan measurement results that are transmitted by the query.
        /// 
        /// Note:
        /// This function is only valid in conjunction with the option R&amp;S ESPI-K50 option.
        /// 
        /// Remote-control command(s):
        /// TRACe&lt;1|2&gt;:POINts? SCAN
        /// </summary>
        /// <param name="Value">
        /// The number of scan measurement results that are transmitted by the query.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetTraceScan(out int Value)
        {
            int pInvokeResult = PInvoke.GetTraceScan(this._handle, out Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches between the instrument operating modes (Spectrum analysis, EMI receiver, FM demodulator and IF analyzer).
        /// 
        /// Note:
        /// 
        /// (1) Changeover to ADEMod is only possible with FM Demodulator R&amp;S FS-K7 option installed.
        /// 
        /// (2) Changeover to IFANalyzer is only possible with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// INSTrument[:SELect] SANalyzer | RECeiver | ADEMod | IFANalyzer
        /// </summary>
        /// <param name="Instrument_Mode">
        /// This control switches between the instrument operating modes.
        /// 
        /// Valid Values:
        /// RSESCI_INSTRUMENT_MODE_SANALYZER   (0) - Spectrum Analysis
        /// RSESCI_INSTRUMENT_MODE_RECEIVER    (1) - EMI Receiver
        /// RSESCI_INSTRUMENT_MODE_ADEMOD      (2) - FM Demodulator
        /// RSESCI_INSTRUMENT_MODE_IFANALYZER  (3) - IF Analyzer
        /// 
        /// Default Value: RSESCI_INSTRUMENT_MODE_RECEIVER (1)
        /// 
        /// Note:
        /// 
        /// (1) The IF Analyzer is only available with ESU instrument.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetInstrumentMode(int Instrument_Mode)
        {
            int pInvokeResult = PInvoke.SetInstrumentMode(this._handle, Instrument_Mode);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the instrument operating mode (Spectrum analysis, EMI receiver, FM demodulator and IF analyzer).
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// INSTrument[:SELect]?
        /// </summary>
        /// <param name="Instrument_Mode">
        /// This control returns the instrument operating mode.
        /// 
        /// Valid Values:
        /// RSESCI_INSTRUMENT_MODE_SANALYZER   (0) - Spectrum Analysis
        /// RSESCI_INSTRUMENT_MODE_RECEIVER    (1) - EMI Receiver
        /// RSESCI_INSTRUMENT_MODE_ADEMOD      (2) - FM Demodulator
        /// RSESCI_INSTRUMENT_MODE_IFANALYZER  (3) - IF Analyzer
        /// 
        /// Note:
        /// 
        /// (1) The IF Analyzer is only available with ESU instrument.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetInstrumentMode(out int Instrument_Mode)
        {
            int pInvokeResult = PInvoke.GetInstrumentMode(this._handle, out Instrument_Mode);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the R&amp;S ESCI in a combination of test receiver and analyzer mode.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// INSTrument:CONFigure:MIXed ON | OFF
        /// INSTrument:CONFigure&lt;1|2&gt;:DISPlay SANalyzer | RECeiver
        /// </summary>
        /// <param name="Mixed_Mode">
        /// This control splits the screen and decouples the two subscreens, permitting a mixed analyzer/receiver display.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_TRUE (1)
        /// 
        /// </param>
        /// <param name="Screens">
        /// This control selects the combination to be displayed in the mixed mode (INSTrument:CONFigure:MIXed ON).
        /// 
        /// Two combinations are available:
        /// 
        ///  Screen 1 (top):
        ///   Bargraph
        ///   Sweep
        /// 
        ///  Screen 2 (bottom):
        ///   Sweep
        ///   Scan
        /// 
        /// Valid Values:
        /// RSESCI_MIXED_MODE_DISPLAY_SANALYZER (0) - Spectrum + Scan
        /// RSESCI_MIXED_MODE_DISPLAY_RECEIVER  (1) - Bargraph + Spectrum
        /// 
        /// Default Value: RSESCI_MIXED_MODE_DISPLAY_RECEIVER (1)
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetMixedInstrumentMode(bool Mixed_Mode, int Screens)
        {
            int pInvokeResult = PInvoke.SetMixedInstrumentMode(this._handle, System.Convert.ToUInt16(Mixed_Mode), Screens);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the R&amp;S ESCI combination of test receiver and analyzer mode.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// INSTrument:CONFigure:MIXed?
        /// INSTrument:CONFigure&lt;1|2&gt;:DISPlay?
        /// </summary>
        /// <param name="Mixed_Mode">
        /// This control returns whether the mixed analyzer/receiver display is activated or not.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// </param>
        /// <param name="Screens">
        /// This control returns the combination displayed in the mixed mode (INSTrument:CONFigure:MIXed ON).
        /// 
        /// Two combinations are available:
        /// 
        ///  Screen 1 (top):
        ///   Bargraph
        ///   Sweep
        /// 
        ///  Screen 2 (bottom):
        ///   Sweep
        ///   Scan
        /// 
        /// Valid Values:
        /// RSESCI_MIXED_MODE_DISPLAY_SANALYZER (0) - Spectrum + Scan
        /// RSESCI_MIXED_MODE_DISPLAY_RECEIVER  (1) - Bargraph + Spectrum
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetMixedInstrumentMode(out bool Mixed_Mode, out int Screens)
        {
            ushort Mixed_ModeAsUShort;
            int pInvokeResult = PInvoke.GetMixedInstrumentMode(this._handle, out Mixed_ModeAsUShort, out Screens);
            Mixed_Mode = System.Convert.ToBoolean(Mixed_ModeAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches the tracking generator on or off.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FSP-B9.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// OUTPut&lt;1|2&gt;[:STATe] ON | OFF
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Tracking_Generator">
        /// This control switches the tracking generator on or off.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetTrackingGenerator(int Window, bool Tracking_Generator)
        {
            int pInvokeResult = PInvoke.SetTrackingGenerator(this._handle, Window, System.Convert.ToUInt16(Tracking_Generator));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the state of tracking generator.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FSP-B9.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// OUTPut&lt;1|2&gt;[:STATe]?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Tracking_Generator">
        /// This control returns the state of tracking generator.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetTrackingGenerator(int Window, out bool Tracking_Generator)
        {
            ushort Tracking_GeneratorAsUShort;
            int pInvokeResult = PInvoke.GetTrackingGenerator(this._handle, Window, out Tracking_GeneratorAsUShort);
            Tracking_Generator = System.Convert.ToBoolean(Tracking_GeneratorAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the output level of the tracking generator in the current measurement window.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FSP-B9.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// SOURce&lt;1|2&gt;:POWer[:LEVel][:IMMediate][:AMPLitude]
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Output_Level">
        /// This control defines the output level of the tracking generator in the current measurement window.
        /// 
        /// Valid Range: -30 to 5 dBm ((-100 to + 5dBm with option B12)
        /// 
        /// Default Value: -20 dBm
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetTrackingGeneratorSourcePower(int Window, double Output_Level)
        {
            int pInvokeResult = PInvoke.SetTrackingGeneratorSourcePower(this._handle, Window, Output_Level);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the output level of the tracking generator in the current measurement window.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FSP-B9.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// SOURce&lt;1|2&gt;:POWer[:LEVel][:IMMediate][:AMPLitude]?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Output_Level">
        /// This control returns the output level of the tracking generator in the current measurement window.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetTrackingGeneratorSourcePower(int Window, out double Output_Level)
        {
            int pInvokeResult = PInvoke.GetTrackingGeneratorSourcePower(this._handle, Window, out Output_Level);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines a level offset for the tracking generator level. Thus, for example, attenuators or amplifiers at the output of the tracking generator can be taken into account for the setting.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FSP-B9.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// SOURce&lt;1|2&gt;:POWer[:LEVel][:IMMediate]:OFFSet
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Level_Offset">
        /// This control defines a level offset for the tracking generator level.
        /// 
        /// Valid Range: -200.0 to 200.0 dB
        /// 
        /// Default Value: 0.0 dB
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetTrackingGeneratorPowerOffset(int Window, double Level_Offset)
        {
            int pInvokeResult = PInvoke.SetTrackingGeneratorPowerOffset(this._handle, Window, Level_Offset);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries a level offset for the tracking generator level.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FSP-B9.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// SOURce&lt;1|2&gt;:POWer[:LEVel][:IMMediate]:OFFSet?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Level_Offset">
        /// This control returns a level offset for the tracking generator level.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetTrackingGeneratorPowerOffset(int Window, out double Level_Offset)
        {
            int pInvokeResult = PInvoke.GetTrackingGeneratorPowerOffset(this._handle, Window, out Level_Offset);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines a frequency offset of the tracking generator for the indicated measurement window. Frequency-converting DUTs can be measured with this setting. It should be noted that the terms (start frequency - tracking frequency offset) and (stop frequency - tracking frequency offset) are both &gt; 1 kHz or both &lt; -1 kHz.
        /// External I/Q modulation is switched off, if active.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FSP-B9.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// SOURce&lt;1|2&gt;:FREQuency:OFFSet
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Frequency_Offset">
        /// This control defines a frequency offset of the tracking generator for the indicated measurement window.
        /// 
        /// Valid Range: -150.0E6 to 150.0E6 Hz
        /// 
        /// Default Value: 0.0 Hz
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetTrackingGeneratorFrequencyOffset(int Window, double Frequency_Offset)
        {
            int pInvokeResult = PInvoke.SetTrackingGeneratorFrequencyOffset(this._handle, Window, Frequency_Offset);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries a frequency offset of the tracking generator for the indicated measurement window.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FSP-B9.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// SOURce&lt;1|2&gt;:FREQuency:OFFSet?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Frequency_Offset">
        /// This control returns a frequency offset of the tracking generator for the indicated measurement window.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetTrackingGeneratorFrequencyOffset(int Window, out double Frequency_Offset)
        {
            int pInvokeResult = PInvoke.GetTrackingGeneratorFrequencyOffset(this._handle, Window, out Frequency_Offset);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the power sweep of the tracking generator.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FSP-B9.
        /// 
        /// Mode(s):
        /// All - for Power Mode
        /// Spectrum Analyzer - for Power Start and Stop
        /// 
        /// Remote-control command(s):
        /// SOURce&lt;1|2&gt;:POWer:MODE FIXed | SWEep
        /// SOURce&lt;1|2&gt;:POWer:STARt
        /// SOURce&lt;1|2&gt;:POWer:STOP
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Power_Sweep">
        /// Activates or deactivates the power sweep.
        /// 
        /// Valid Values: 
        /// RSESCI_TGEN_POW_FIX (0) - Fixed
        /// RSESCI_TGEN_POW_SWE (1) - Sweep
        /// 
        /// Default Value: RSESCI_TGEN_POW_FIX (0)
        /// </param>
        /// <param name="Power_Start">
        /// Sets the start power of the power sweep. 
        /// 
        /// Valid Values: 
        /// -30.0 to 0.0 dBm
        /// 
        /// Default Value: -30.0
        /// </param>
        /// <param name="Power_Stop">
        /// Sets the stop power of the power sweep. 
        /// 
        /// Valid Values: 
        /// -30.0 to 0.0 dBm
        /// 
        /// Default Value: 0.0
        /// </param>
        /// <returns>
        /// Returns the status code of this operation.  The status code  either indicates success or describes an error or warning condition.  You examine the status code from each call to an instrument driver function to determine if an error occurred.
        /// 
        /// To obtain a text description of the status code, call the rsspecan_error_message function.  To obtain additional information about the error condition, call the rsspecan_GetError function.  To clear the error information from the driver, call the rsspecan_ClearError function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// WARNINGS:
        /// 3FFA2001  The instrument was in an uncalibrated state
        ///           when the measurement was taken.
        /// 3FFA2002  The measurement taken was over the instrument??s
        ///           range.
        ///   
        /// ERRORS:
        /// BFFA2001  The Active Marker is not enabled.
        /// BFFA2002  The Active Marker is not a delta marker.
        /// BFFA2003  The maximum waiting time for this operation was
        ///           exceeded.
        ///           
        /// This instrument driver also returns errors and warnings defined by other sources.  The following table defines the ranges of additional status codes that this driver can return.  The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFA0000 to 3FFA1FFF     IVI    Warnings
        /// 3FFF0000 to 3FFFFFFF     VISA   Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP Driver Warnings
        ///           
        /// BFFA0000 to BFFA1FFF     IVI    Errors
        /// BFFF0000 to BFFFFFFF     VISA   Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP Driver Errors
        /// 
        /// </returns>
        public int ConfigureTrackingGeneratorPowerSweep(int Window, int Power_Sweep, double Power_Start, double Power_Stop)
        {
            int pInvokeResult = PInvoke.ConfigureTrackingGeneratorPowerSweep(this._handle, Window, Power_Sweep, Power_Start, Power_Stop);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function activates or deactivates the power sweep.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FSP-B9.
        /// 
        /// Mode(s):
        /// All - for Fixed Mode
        /// Spectrum Analyzer - for Sweep mode
        /// 
        /// Remote-control command(s):
        /// SOURce&lt;1|2&gt;:POWer:MODE FIXed | SWEep
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Power_Sweep">
        /// Activates or deactivates the power sweep.
        /// 
        /// Valid Values: 
        /// RSESCI_TGEN_POW_FIX (0) - Fixed
        /// RSESCI_TGEN_POW_SWE (1) - Sweep
        /// 
        /// Default Value: RSESCI_TGEN_POW_FIX (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation.  The status code  either indicates success or describes an error or warning condition.  You examine the status code from each call to an instrument driver function to determine if an error occurred.
        /// 
        /// To obtain a text description of the status code, call the rsspecan_error_message function.  To obtain additional information about the error condition, call the rsspecan_GetError function.  To clear the error information from the driver, call the rsspecan_ClearError function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// WARNINGS:
        /// 3FFA2001  The instrument was in an uncalibrated state
        ///           when the measurement was taken.
        /// 3FFA2002  The measurement taken was over the instrument??s
        ///           range.
        ///   
        /// ERRORS:
        /// BFFA2001  The Active Marker is not enabled.
        /// BFFA2002  The Active Marker is not a delta marker.
        /// BFFA2003  The maximum waiting time for this operation was
        ///           exceeded.
        ///           
        /// This instrument driver also returns errors and warnings defined by other sources.  The following table defines the ranges of additional status codes that this driver can return.  The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFA0000 to 3FFA1FFF     IVI    Warnings
        /// 3FFF0000 to 3FFFFFFF     VISA   Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP Driver Warnings
        ///           
        /// BFFA0000 to BFFA1FFF     IVI    Errors
        /// BFFF0000 to BFFFFFFF     VISA   Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP Driver Errors
        /// 
        /// </returns>
        public int SetTrackingGeneratorPowerSweepMode(int Window, int Power_Sweep)
        {
            int pInvokeResult = PInvoke.SetTrackingGeneratorPowerSweepMode(this._handle, Window, Power_Sweep);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the power sweep mode.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FSP-B9.
        /// 
        /// Mode(s):
        /// All - for Fixed Mode
        /// Spectrum Analyzer - for Sweep mode
        /// 
        /// Remote-control command(s):
        /// SOURce&lt;1|2&gt;:POWer:MODE?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Power_Sweep">
        /// Returns selected power sweep.
        /// 
        /// Valid Values: 
        /// RSESCI_TGEN_POW_FIX (0) - Fixed
        /// RSESCI_TGEN_POW_SWE (1) - Sweep
        /// </param>
        /// <returns>
        /// Returns the status code of this operation.  The status code  either indicates success or describes an error or warning condition.  You examine the status code from each call to an instrument driver function to determine if an error occurred.
        /// 
        /// To obtain a text description of the status code, call the rsspecan_error_message function.  To obtain additional information about the error condition, call the rsspecan_GetError function.  To clear the error information from the driver, call the rsspecan_ClearError function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// WARNINGS:
        /// 3FFA2001  The instrument was in an uncalibrated state
        ///           when the measurement was taken.
        /// 3FFA2002  The measurement taken was over the instrument??s
        ///           range.
        ///   
        /// ERRORS:
        /// BFFA2001  The Active Marker is not enabled.
        /// BFFA2002  The Active Marker is not a delta marker.
        /// BFFA2003  The maximum waiting time for this operation was
        ///           exceeded.
        ///           
        /// This instrument driver also returns errors and warnings defined by other sources.  The following table defines the ranges of additional status codes that this driver can return.  The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFA0000 to 3FFA1FFF     IVI    Warnings
        /// 3FFF0000 to 3FFFFFFF     VISA   Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP Driver Warnings
        ///           
        /// BFFA0000 to BFFA1FFF     IVI    Errors
        /// BFFF0000 to BFFFFFFF     VISA   Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP Driver Errors
        /// 
        /// </returns>
        public int GetTrackingGeneratorPowerSweepMode(int Window, out int Power_Sweep)
        {
            int pInvokeResult = PInvoke.GetTrackingGeneratorPowerSweepMode(this._handle, Window, out Power_Sweep);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the start power of the power sweep. 
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FSP-B9.
        /// 
        /// Mode(s):
        /// Spectrum Analyzer
        /// 
        /// Remote-control command(s):
        /// SOURce&lt;1|2&gt;:POWer:STARt
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Power_Start">
        /// Sets the start power of the power sweep. 
        /// 
        /// Valid Values: 
        /// -30.0 to 0.0 dBm
        /// 
        /// Default Value: -30.0
        /// </param>
        /// <returns>
        /// Returns the status code of this operation.  The status code  either indicates success or describes an error or warning condition.  You examine the status code from each call to an instrument driver function to determine if an error occurred.
        /// 
        /// To obtain a text description of the status code, call the rsspecan_error_message function.  To obtain additional information about the error condition, call the rsspecan_GetError function.  To clear the error information from the driver, call the rsspecan_ClearError function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// WARNINGS:
        /// 3FFA2001  The instrument was in an uncalibrated state
        ///           when the measurement was taken.
        /// 3FFA2002  The measurement taken was over the instrument??s
        ///           range.
        ///   
        /// ERRORS:
        /// BFFA2001  The Active Marker is not enabled.
        /// BFFA2002  The Active Marker is not a delta marker.
        /// BFFA2003  The maximum waiting time for this operation was
        ///           exceeded.
        ///           
        /// This instrument driver also returns errors and warnings defined by other sources.  The following table defines the ranges of additional status codes that this driver can return.  The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFA0000 to 3FFA1FFF     IVI    Warnings
        /// 3FFF0000 to 3FFFFFFF     VISA   Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP Driver Warnings
        ///           
        /// BFFA0000 to BFFA1FFF     IVI    Errors
        /// BFFF0000 to BFFFFFFF     VISA   Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP Driver Errors
        /// 
        /// </returns>
        public int SetTrackingGeneratorPowerSweepStart(int Window, double Power_Start)
        {
            int pInvokeResult = PInvoke.SetTrackingGeneratorPowerSweepStart(this._handle, Window, Power_Start);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the start power of the power sweep. 
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FSP-B9.
        /// 
        /// Mode(s):
        /// Spectrum Analyzer
        /// 
        /// Remote-control command(s):
        /// SOURce&lt;1|2&gt;:POWer:STARt?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Power_Start">
        /// Returns the start power of the power sweep. 
        /// 
        /// Valid Values: 
        /// -30.0 to 0.0 dBm
        /// </param>
        /// <returns>
        /// Returns the status code of this operation.  The status code  either indicates success or describes an error or warning condition.  You examine the status code from each call to an instrument driver function to determine if an error occurred.
        /// 
        /// To obtain a text description of the status code, call the rsspecan_error_message function.  To obtain additional information about the error condition, call the rsspecan_GetError function.  To clear the error information from the driver, call the rsspecan_ClearError function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// WARNINGS:
        /// 3FFA2001  The instrument was in an uncalibrated state
        ///           when the measurement was taken.
        /// 3FFA2002  The measurement taken was over the instrument??s
        ///           range.
        ///   
        /// ERRORS:
        /// BFFA2001  The Active Marker is not enabled.
        /// BFFA2002  The Active Marker is not a delta marker.
        /// BFFA2003  The maximum waiting time for this operation was
        ///           exceeded.
        ///           
        /// This instrument driver also returns errors and warnings defined by other sources.  The following table defines the ranges of additional status codes that this driver can return.  The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFA0000 to 3FFA1FFF     IVI    Warnings
        /// 3FFF0000 to 3FFFFFFF     VISA   Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP Driver Warnings
        ///           
        /// BFFA0000 to BFFA1FFF     IVI    Errors
        /// BFFF0000 to BFFFFFFF     VISA   Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP Driver Errors
        /// 
        /// </returns>
        public int GetTrackingGeneratorPowerSweepStart(int Window, out double Power_Start)
        {
            int pInvokeResult = PInvoke.GetTrackingGeneratorPowerSweepStart(this._handle, Window, out Power_Start);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the stop power of the power sweep.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FSP-B9.
        /// 
        /// Mode(s):
        /// Spectrum Analyzer
        /// 
        /// Remote-control command(s):
        /// SOURce&lt;1|2&gt;:POWer:STOP
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Power_Stop">
        /// Sets the stop power of the power sweep. 
        /// 
        /// Valid Values: 
        /// -30.0 to 0.0 dBm
        /// 
        /// Default Value: 0.0
        /// </param>
        /// <returns>
        /// Returns the status code of this operation.  The status code  either indicates success or describes an error or warning condition.  You examine the status code from each call to an instrument driver function to determine if an error occurred.
        /// 
        /// To obtain a text description of the status code, call the rsspecan_error_message function.  To obtain additional information about the error condition, call the rsspecan_GetError function.  To clear the error information from the driver, call the rsspecan_ClearError function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// WARNINGS:
        /// 3FFA2001  The instrument was in an uncalibrated state
        ///           when the measurement was taken.
        /// 3FFA2002  The measurement taken was over the instrument??s
        ///           range.
        ///   
        /// ERRORS:
        /// BFFA2001  The Active Marker is not enabled.
        /// BFFA2002  The Active Marker is not a delta marker.
        /// BFFA2003  The maximum waiting time for this operation was
        ///           exceeded.
        ///           
        /// This instrument driver also returns errors and warnings defined by other sources.  The following table defines the ranges of additional status codes that this driver can return.  The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFA0000 to 3FFA1FFF     IVI    Warnings
        /// 3FFF0000 to 3FFFFFFF     VISA   Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP Driver Warnings
        ///           
        /// BFFA0000 to BFFA1FFF     IVI    Errors
        /// BFFF0000 to BFFFFFFF     VISA   Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP Driver Errors
        /// 
        /// </returns>
        public int SetTrackingGeneratorPowerSweepStop(int Window, double Power_Stop)
        {
            int pInvokeResult = PInvoke.SetTrackingGeneratorPowerSweepStop(this._handle, Window, Power_Stop);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the stop power of the power sweep.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FSP-B9.
        /// 
        /// Mode(s):
        /// Spectrum Analyzer
        /// 
        /// Remote-control command(s):
        /// SOURce&lt;1|2&gt;:POWer:STOP?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Power_Stop">
        /// Returns the stop power of the power sweep. 
        /// 
        /// Valid Values: 
        /// -30.0 to 0.0 dBm
        /// </param>
        /// <returns>
        /// Returns the status code of this operation.  The status code  either indicates success or describes an error or warning condition.  You examine the status code from each call to an instrument driver function to determine if an error occurred.
        /// 
        /// To obtain a text description of the status code, call the rsspecan_error_message function.  To obtain additional information about the error condition, call the rsspecan_GetError function.  To clear the error information from the driver, call the rsspecan_ClearError function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// WARNINGS:
        /// 3FFA2001  The instrument was in an uncalibrated state
        ///           when the measurement was taken.
        /// 3FFA2002  The measurement taken was over the instrument??s
        ///           range.
        ///   
        /// ERRORS:
        /// BFFA2001  The Active Marker is not enabled.
        /// BFFA2002  The Active Marker is not a delta marker.
        /// BFFA2003  The maximum waiting time for this operation was
        ///           exceeded.
        ///           
        /// This instrument driver also returns errors and warnings defined by other sources.  The following table defines the ranges of additional status codes that this driver can return.  The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFA0000 to 3FFA1FFF     IVI    Warnings
        /// 3FFF0000 to 3FFFFFFF     VISA   Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP Driver Warnings
        ///           
        /// BFFA0000 to BFFA1FFF     IVI    Errors
        /// BFFF0000 to BFFFFFFF     VISA   Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP Driver Errors
        /// 
        /// </returns>
        public int GetTrackingGeneratorPowerSweepStop(int Window, out double Power_Stop)
        {
            int pInvokeResult = PInvoke.GetTrackingGeneratorPowerSweepStop(this._handle, Window, out Power_Stop);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the type of measurement with active tracking generator.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FSP-B9/B10.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]CORRection:METHod TRANsmission | REFLection
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Measurement_Type">
        /// This control selects the type of measurement with active tracking generator.
        /// 
        /// Valid Values:
        /// RSESCI_CORRECTION_TRAN (0) - Transimission
        /// RSESCI_CORRECTION_REFL (1) - Reflection
        /// 
        /// Default Value: RSESCI_CORRECTION_TRAN (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetTrackingGeneratorMeasurement(int Window, int Measurement_Type)
        {
            int pInvokeResult = PInvoke.SetTrackingGeneratorMeasurement(this._handle, Window, Measurement_Type);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the type of measurement with active tracking generator.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FSP-B9/B10.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]CORRection:METHod?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Measurement_Type">
        /// This control returns the type of measurement with active tracking generator.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetTrackingGeneratorMeasurement(int Window, out int Measurement_Type)
        {
            int pInvokeResult = PInvoke.GetTrackingGeneratorMeasurement(this._handle, Window, out Measurement_Type);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function activates/deactivates the normalization of the measurement results in the selected window provided that the tracking generator is active. The function is available only after acquisition of a reference trace for the selected type of measurement.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FSP-B9/B10.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]CORRection[:STATe] ON | OFF
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Normalization">
        /// This control activates/deactivates the normalization of the measurement results in the selected window provided that the tracking generator is active.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetTrackingGeneratorNormalization(int Window, bool Normalization)
        {
            int pInvokeResult = PInvoke.SetTrackingGeneratorNormalization(this._handle, Window, System.Convert.ToUInt16(Normalization));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns if the normalization of the measurement results in the selected window provided of the tracking generator is active. The function is available only after acquisition of a reference trace for the selected type of measurement.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FSP-B9/B10.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]CORRection[:STATe]?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Normalization">
        /// This control returns the state of the normalization of the measurement results in the selected window provided that the tracking generator is active.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetTrackingGeneratorNormalization(int Window, out bool Normalization)
        {
            ushort NormalizationAsUShort;
            int pInvokeResult = PInvoke.GetTrackingGeneratorNormalization(this._handle, Window, out NormalizationAsUShort);
            Normalization = System.Convert.ToBoolean(NormalizationAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// When the tracking generator is active, this function determines the type of result acquisition for the normalization reference measurement and starts the measurement selected.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FSP-B9/B10.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]CORRection:COLLect[:ACQuire] THRough | OPEN
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Acquisition_Type">
        /// This control determines the type of result acquisition for the
        /// normalization reference measurement
        /// 
        /// Valid Values:
        /// RSESCI_ACQ_THR  (0) - Through
        /// RSESCI_ACQ_OPEN (1) - Open
        /// 
        /// Default Value: RSESCI_ACQ_THR (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int TrackingGeneratorCalibration(int Window, int Acquisition_Type)
        {
            int pInvokeResult = PInvoke.TrackingGeneratorCalibration(this._handle, Window, Acquisition_Type);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function restores the instrument setting that was valid for the measurement of the reference data, provided that the tracking generator is active.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FSP-B9/B10.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]CORRection:RECall
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int TrackingGeneratorRecallSettings(int Window)
        {
            int pInvokeResult = PInvoke.TrackingGeneratorRecallSettings(this._handle, Window);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function uses the normalized measurement data to
        /// generate a transducer factor with up to 625 points.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is only valid in conjunction with the option R&amp;S FSP-B9/B10.
        /// 
        /// (2) The function is only available when normalization is switched on.
        /// 
        /// (3) This function is only available with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// Spectrum Analyzer
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]CORRection:TRANsducer:GENerate &lt;name&gt;
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Name">
        /// This control sets name of the transducer factors.
        /// 
        /// Valid Values: up to 8 characters
        /// 
        /// Default Value: ""
        /// </param>
        /// <returns>
        /// Returns the status code of this operation.  The status code  either indicates success or describes an error or warning condition.  You examine the status code from each call to an instrument driver function to determine if an error occurred.
        /// 
        /// To obtain a text description of the status code, call the rsspecan_error_message function.  To obtain additional information about the error condition, call the rsspecan_GetError function.  To clear the error information from the driver, call the rsspecan_ClearError function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// WARNINGS:
        /// 3FFA2001  The instrument was in an uncalibrated state
        ///           when the measurement was taken.
        /// 3FFA2002  The measurement taken was over the instrument??s
        ///           range.
        ///   
        /// ERRORS:
        /// BFFA2001  The Active Marker is not enabled.
        /// BFFA2002  The Active Marker is not a delta marker.
        /// BFFA2003  The maximum waiting time for this operation was
        ///           exceeded.
        ///           
        /// This instrument driver also returns errors and warnings defined by other sources.  The following table defines the ranges of additional status codes that this driver can return.  The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFA0000 to 3FFA1FFF     IVI    Warnings
        /// 3FFF0000 to 3FFFFFFF     VISA   Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP Driver Warnings
        ///           
        /// BFFA0000 to BFFA1FFF     IVI    Errors
        /// BFFF0000 to BFFFFFFF     VISA   Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP Driver Errors
        /// 
        /// </returns>
        public int TrackingGeneratorSaveAsTrdFactor(int Window, string Name)
        {
            int pInvokeResult = PInvoke.TrackingGeneratorSaveAsTrdFactor(this._handle, Window, Name);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches on or off the external amplitude modulation of the tracking generator in the selected measurement window.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FSP-B9.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// SOURce&lt;1|2&gt;:AM:STATe ON | OFF
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="External_AM">
        /// This control switches on or off the external amplitude modulation of the tracking generator in the selected measurement window.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetExternalAM(int Window, bool External_AM)
        {
            int pInvokeResult = PInvoke.SetExternalAM(this._handle, Window, System.Convert.ToUInt16(External_AM));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the state of the external amplitude modulation of the tracking generator in the selected measurement window.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FSP-B9.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// SOURce&lt;1|2&gt;:AM:STATe?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="External_AM">
        /// This control returns the state of the external amplitude modulation of the tracking generator in the selected measurement window.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetExternalAM(int Window, out bool External_AM)
        {
            ushort External_AMAsUShort;
            int pInvokeResult = PInvoke.GetExternalAM(this._handle, Window, out External_AMAsUShort);
            External_AM = System.Convert.ToBoolean(External_AMAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches on or off the external frequency modulation of the tracking generator in the selected measurement window.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FSP-B9.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// SOURce&lt;1|2&gt;:FM:STATe ON | OFF
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="External_FM">
        /// This control switches on or off the external frequency modulation of the tracking generator in the selected measurement window.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetExternalFM(int Window, bool External_FM)
        {
            int pInvokeResult = PInvoke.SetExternalFM(this._handle, Window, System.Convert.ToUInt16(External_FM));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the state of the external frequency modulation of the tracking generator in the selected measurement window.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FSP-B9.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// SOURce&lt;1|2&gt;:FM:STATe?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="External_FM">
        /// This control returns the state of the external frequency modulation of the tracking generator in the selected measurement window.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetExternalFM(int Window, out bool External_FM)
        {
            ushort External_FMAsUShort;
            int pInvokeResult = PInvoke.GetExternalFM(this._handle, Window, out External_FMAsUShort);
            External_FM = System.Convert.ToBoolean(External_FMAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches on or off the external I/Q modulation of the tracking generator in the selected measurement window.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FSP-B9.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// SOURce&lt;1|2&gt;:DM:STATe ON | OFF
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="IQ_Modulation">
        /// This control switches on or off the external I/Q modulation of the tracking generator in the selected measurement window.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetExternalIQModulation(int Window, bool IQ_Modulation)
        {
            int pInvokeResult = PInvoke.SetExternalIQModulation(this._handle, Window, System.Convert.ToUInt16(IQ_Modulation));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the state of the external I/Q modulation of the tracking generator in the selected measurement window.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FSP-B9.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// SOURce&lt;1|2&gt;:DM:STATe?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="IQ_Modulation">
        /// This control returns the state of the external I/Q modulation of the tracking generator in the selected measurement window.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetExternalIQModulation(int Window, out bool IQ_Modulation)
        {
            ushort IQ_ModulationAsUShort;
            int pInvokeResult = PInvoke.GetExternalIQModulation(this._handle, Window, out IQ_ModulationAsUShort);
            IQ_Modulation = System.Convert.ToBoolean(IQ_ModulationAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the maximum frequency deviation at 1 V input voltage at the FM input of the tracking generator.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FSP-B9.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// SOURce&lt;1|2&gt;:FM:DEViation
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="FM_Deviation">
        /// This control defines the maximum frequency deviation at 1 V input voltage at the FM input of the tracking generator.
        /// 
        /// Valid Range: 100.0 to 100.0e6 Hz
        /// 
        /// Default Value: 100.0 Hz
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetExternalFMDeviation(int Window, double FM_Deviation)
        {
            int pInvokeResult = PInvoke.SetExternalFMDeviation(this._handle, Window, FM_Deviation);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the maximum frequency deviation at 1 V input voltage at the FM input of the tracking generator.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FSP-B9.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// SOURce&lt;1|2&gt;:FM:DEViation?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="FM_Deviation">
        /// This control returns the maximum frequency deviation at 1 V input voltage at the FM input of the tracking generator.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetExternalFMDeviation(int Window, out double FM_Deviation)
        {
            int pInvokeResult = PInvoke.GetExternalFMDeviation(this._handle, Window, out FM_Deviation);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function activates or deactivates the external generator in the selected window.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FSP-B10.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// SOURce&lt;1|2&gt;:EXTernal&lt;1|2&gt;[:STATe] ON | OFF
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="External_Generator">
        /// This control activates or deactivates the external generator in the selected window.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetExternalGenerator(int Window, bool External_Generator)
        {
            int pInvokeResult = PInvoke.SetExternalGenerator(this._handle, Window, System.Convert.ToUInt16(External_Generator));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the state of the external generator in the selected window.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FSP-B10.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// SOURce&lt;1|2&gt;:EXTernal&lt;1|2&gt;[:STATe]?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="External_Generator">
        /// This control returns the state of the external generator in the selected window.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetExternalGenerator(int Window, out bool External_Generator)
        {
            ushort External_GeneratorAsUShort;
            int pInvokeResult = PInvoke.GetExternalGenerator(this._handle, Window, out External_GeneratorAsUShort);
            External_Generator = System.Convert.ToBoolean(External_GeneratorAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the external generator parameters.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FSP-B10.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// SYSTem:COMMunicate:GPIB:RDEVice:GENerator&lt;1|2&gt;:ADDRess
        /// SYSTem:COMMunicate:RDEVice:GENerator&lt;1|2&gt;:LINK GPIB | TTL
        /// SYSTem:COMMunicate:RDEVice:GENerator&lt;1|2&gt;:TYPE &lt;name&gt;
        /// </summary>
        /// <param name="Generator">
        /// This control selects the external generator.
        /// 
        /// Valid Range: 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Interface">
        /// This control selects the interface type of the external generator 1 or 2.
        /// 
        /// Valid Values:
        /// RSESCI_INTF_GPIB (0) - GPIB Alone
        /// RSESCI_INTF_TTL  (1) - GPIB And TTL interface for synchronization
        /// 
        /// Default Value: RSESCI_INTF_GPIB (0)
        /// </param>
        /// <param name="Type">
        /// This control selects the type of external generator 1 or 2. List of all available generator types is stated in operation manual.
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: "NONE"
        /// </param>
        /// <param name="GPIB_Address">
        /// This control changes the IEC/IEEe-bus address of the device selected as external generator 1 or 2.
        /// 
        /// Valid Range: 0 to 30
        /// 
        /// Default Value: 28
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetExternalGeneratorSelection(int Generator, int Interface, string Type, int GPIB_Address)
        {
            int pInvokeResult = PInvoke.SetExternalGeneratorSelection(this._handle, Generator, Interface, Type, GPIB_Address);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the external generator parameters.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FSP-B10.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// SYSTem:COMMunicate:GPIB:RDEVice:GENerator&lt;1|2&gt;:ADDRess?
        /// SYSTem:COMMunicate:RDEVice:GENerator&lt;1|2&gt;:LINK?
        /// SYSTem:COMMunicate:RDEVice:GENerator&lt;1|2&gt;:TYPE?
        /// </summary>
        /// <param name="Generator">
        /// This control selects the external generator.
        /// 
        /// Valid Range: 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Interface">
        /// This control returns selected interface type of the external generator 1 or 2.
        /// 
        /// Valid Values:
        /// RSESCI_INTF_GPIB (0) - GPIB Alone
        /// RSESCI_INTF_TTL  (1) - GPIB And TTL interface for synchronization
        /// </param>
        /// <param name="Type">
        /// This control returns selected type of external generator 1 or 2. List of all available generator types is stated in operation manual.
        /// </param>
        /// <param name="GPIB_Address">
        /// This control returns the IEC/IEEe-bus address of the device selected as external generator 1 or 2.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetExternalGeneratorSelection(int Generator, out int Interface, System.Text.StringBuilder Type, out int GPIB_Address)
        {
            int pInvokeResult = PInvoke.GetExternalGeneratorSelection(this._handle, Generator, out Interface, Type, out GPIB_Address);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the frequency offset of the selected generator 1 or 2 with reference to the receive frequency in the selected window.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FSP-B10.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// SOURce&lt;1|2&gt;:EXTernal&lt;1|2&gt;:FREQuency:OFFSet
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Generator">
        /// This control selects the external generator.
        /// 
        /// Valid Range: 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Offset">
        /// This control sets the frequency offset of the selected generator 1 or 2 with reference to the receive frequency in the selected window.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: 0.0 Hz
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetExternalGeneratorFrequencyOffset(int Window, int Generator, double Offset)
        {
            int pInvokeResult = PInvoke.SetExternalGeneratorFrequencyOffset(this._handle, Window, Generator, Offset);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the frequency offset of the selected generator 1 or 2 with reference to the receive frequency in the selected window.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FSP-B10.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// SOURce&lt;1|2&gt;:EXTernal&lt;1|2&gt;:FREQuency:OFFSet?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Generator">
        /// This control selects the external generator.
        /// 
        /// Valid Range: 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Offset">
        /// This control returns the frequency offset of the selected generator 1 or 2 with reference to the receive frequency in the selected window.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetExternalGeneratorFrequencyOffset(int Window, int Generator, out double Offset)
        {
            int pInvokeResult = PInvoke.GetExternalGeneratorFrequencyOffset(this._handle, Window, Generator, out Offset);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the numerator and the denominator of the factor with which the analyzer frequency is multiplied in order to obtain the transmit frequency of the selected generator 1 or 2 in the selected window.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FSP-B10.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// SOURce&lt;1|2&gt;:EXTernal&lt;1|2&gt;:FREQuency[:FACTor]:NUMerator
        /// SOURce&lt;1|2&gt;:EXTernal&lt;1|2&gt;:FREQuency[:FACTor]:DENominator
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Generator">
        /// This control selects the external generator.
        /// 
        /// Valid Range: 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Numerator">
        /// This control defines the numerator of the factor with which the analyzer frequency is multiplied to obtain the transmit frequency of the selected generator 1 or 2 in the selected window.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Denominator">
        /// This control defines the denominator of the factor with which the analyzer frequency is multiplied to obtain the transmit frequency of the selected generator 1 or 2 in the selected window.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: 1
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetExternalGeneratorMultiplicationFactor(int Window, int Generator, int Numerator, int Denominator)
        {
            int pInvokeResult = PInvoke.SetExternalGeneratorMultiplicationFactor(this._handle, Window, Generator, Numerator, Denominator);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the numerator and the denominator of the factor with which the analyzer frequency is multiplied in order to obtain the transmit frequency of the selected generator 1 or 2 in the selected window.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FSP-B10.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// SOURce&lt;1|2&gt;:EXTernal&lt;1|2&gt;:FREQuency[:FACTor]:NUMerator?
        /// SOURce&lt;1|2&gt;:EXTernal&lt;1|2&gt;:FREQuency[:FACTor]:DENominator?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Generator">
        /// This control selects the external generator.
        /// 
        /// Valid Range: 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Numerator">
        /// This control returns the numerator of the factor with which the analyzer frequency is multiplied to obtain the transmit frequency of the selected generator 1 or 2 in the selected window.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Denominator">
        /// This control returns the denominator of the factor with which the analyzer frequency is multiplied to obtain the transmit frequency of the selected generator 1 or 2 in the selected window.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: 1
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetExternalGeneratorMultiplicationFactor(int Window, int Generator, out int Numerator, out int Denominator)
        {
            int pInvokeResult = PInvoke.GetExternalGeneratorMultiplicationFactor(this._handle, Window, Generator, out Numerator, out Denominator);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function activates or deactivates the frequency sweep for generator 1 or 2 in the selected window.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FSP-B10.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// SOURce&lt;1|2&gt;:EXTernal&lt;1|2&gt;:FREQuency:SWEep[:STATe] ON | OFF
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Generator">
        /// This control selects the external generator.
        /// 
        /// Valid Range: 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Sweep">
        /// This control activates or deactivates the frequency sweep for generator 1 or 2 in the selected window.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetExternalGeneratorFrequencySweep(int Window, int Generator, bool Sweep)
        {
            int pInvokeResult = PInvoke.SetExternalGeneratorFrequencySweep(this._handle, Window, Generator, System.Convert.ToUInt16(Sweep));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the state of the frequency sweep for generator 1 or 2 in the selected window.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FSP-B10.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// SOURce&lt;1|2&gt;:EXTernal&lt;1|2&gt;:FREQuency:SWEep[:STATe]?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Generator">
        /// This control selects the external generator.
        /// 
        /// Valid Range: 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Sweep">
        /// This control returns the state of the frequency sweep for generator 1 or 2 in the selected window.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetExternalGeneratorFrequencySweep(int Window, int Generator, out bool Sweep)
        {
            ushort SweepAsUShort;
            int pInvokeResult = PInvoke.GetExternalGeneratorFrequencySweep(this._handle, Window, Generator, out SweepAsUShort);
            Sweep = System.Convert.ToBoolean(SweepAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the output power of the selected generator 1 or 2 in the selected window.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FSP-B9/B10.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// SOURce&lt;1|2&gt;:EXTernal&lt;1|2&gt;:POWer[:LEVel]
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Generator">
        /// This control selects the external generator.
        /// 
        /// Valid Range: 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Power">
        /// This control sets the output power of the selected generator 1 or 2 in the selected window.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: -20.0 dBm
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetExternalGeneratorPower(int Window, int Generator, double Power)
        {
            int pInvokeResult = PInvoke.SetExternalGeneratorPower(this._handle, Window, Generator, Power);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the output power of the selected generator 1 or 2 in the selected window.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FSP-B9/B10.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// SOURce&lt;1|2&gt;:EXTernal&lt;1|2&gt;:POWer[:LEVel]?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Generator">
        /// This control selects the external generator.
        /// 
        /// Valid Range: 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Power">
        /// This control returns the output power of the selected generator 1 or 2 in the selected window.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetExternalGeneratorPower(int Window, int Generator, out double Power)
        {
            int pInvokeResult = PInvoke.GetExternalGeneratorPower(this._handle, Window, Generator, out Power);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches between external and internal reference oscillator for the frequency processing of external generator 1 and 2.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FSP-B10.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// SOURce&lt;1|2&gt;:EXTernal&lt;1|2&gt;:ROSCillator[:SOURce] INTernal | EXTernal
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Generator">
        /// This control selects the external generator.
        /// 
        /// Valid Range: 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Reference_Oscillator">
        /// This control switches between external and internal reference oscillator for the frequency processing of external generator 1 and 2.
        /// 
        /// Valid Values:
        /// RSESCI_INT (0) - Internal
        /// RSESCI_EXT (1) - External
        /// 
        /// Default Value: RSESCI_INT (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetExternalGeneratorReferenceOscillator(int Window, int Generator, int Reference_Oscillator)
        {
            int pInvokeResult = PInvoke.SetExternalGeneratorReferenceOscillator(this._handle, Window, Generator, Reference_Oscillator);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries selected reference oscillator for the frequency processing of external generator 1 and 2.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FSP-B10.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// SOURce&lt;1|2&gt;:EXTernal&lt;1|2&gt;:ROSCillator[:SOURce]?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Generator">
        /// This control selects the external generator.
        /// 
        /// Valid Range: 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Reference_Oscillator">
        /// This control returns selected reference oscillator for the frequency processing of external generator 1 and 2.
        /// 
        /// Valid Values:
        /// RSESCI_INT (0) - Internal
        /// RSESCI_EXT (1) - External
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetExternalGeneratorReferenceOscillator(int Window, int Generator, out int Reference_Oscillator)
        {
            int pInvokeResult = PInvoke.GetExternalGeneratorReferenceOscillator(this._handle, Window, Generator, out Reference_Oscillator);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sends a command string to the external generator connected via the GPIB interface of the Ext. Generator Control, R&amp;S FSP-B10. 
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FSP-B10.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// SYSTem:COMMunicate:GPIB:RDEVice:COMMand 0...30,&lt;?command string?&gt;
        /// </summary>
        /// <param name="GPIB_Address">
        /// This control changes the IEC/IEEe-bus address of the device selected as external generator 1 or 2.
        /// 
        /// Valid Range: 0 to 30
        /// 
        /// Default Value: 28
        /// </param>
        /// <param name="Command">
        /// This control defines the command to be sent to external generator.
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: "*RST"
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SendExternalGeneratorCommand(int GPIB_Address, string Command)
        {
            int pInvokeResult = PInvoke.SendExternalGeneratorCommand(this._handle, GPIB_Address, Command);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sends query string to the external generator connected via the GPIB interface of the Ext. Generator Control, R&amp;S FSP-B10. 
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FSP-B10.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// SYSTem:COMMunicate:GPIB:RDEVice:COMMand? 0...30,&lt;?command string?&gt;
        /// </summary>
        /// <param name="GPIB_Address">
        /// This control changes the IEC/IEEe-bus address of the device selected as external generator 1 or 2.
        /// 
        /// Valid Range: 0 to 30
        /// 
        /// Default Value: 28
        /// </param>
        /// <param name="Command">
        /// This control defines the command to be sent to external generator.
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: "*RST"
        /// </param>
        /// <param name="Array_Size">
        /// This control defines the size of array in argument 'Comment'.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: 100
        /// </param>
        /// <param name="Result">
        /// Returns result of query.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int QueryExternalGeneratorCommand(int GPIB_Address, string Command, int Array_Size, System.Text.StringBuilder Result)
        {
            int pInvokeResult = PInvoke.QueryExternalGeneratorCommand(this._handle, GPIB_Address, Command, Array_Size, Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function activates the FM demodulator of the instrument. The instrument will be set to time domain measurement (span = 0) at the current center frequency. The detector will be set to SAMPle, the demodulator itself will be set up according to the parameters of command [SENSe:]ADEMod:SET.
        /// 
        /// Note(s):
        /// 
        /// (1) The measurement is always performed in Screen A. Split Screen operation will be switched off as soon as the Demodulator is activated.
        /// 
        /// (2) This function is only valid in conjunction with the option R&amp;S FS-K7.
        /// 
        /// Mode(s):
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// [SENSe:]ADEMod[:STATe] ON | OFF
        /// </summary>
        /// <param name="Analog_Demodulator">
        /// This control activates the FM demodulator.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetAnalogDemodulator(bool Analog_Demodulator)
        {
            int pInvokeResult = PInvoke.SetAnalogDemodulator(this._handle, System.Convert.ToUInt16(Analog_Demodulator));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns if the FM demodulator of the instrument is activated or not.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FS-K7.
        /// 
        /// Mode(s):
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// [SENSe:]ADEMod[:STATe]?
        /// </summary>
        /// <param name="Analog_Demodulator">
        /// Returns if the FM demodulator of the instrument is activated or not.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetAnalogDemodulator(out bool Analog_Demodulator)
        {
            ushort Analog_DemodulatorAsUShort;
            int pInvokeResult = PInvoke.GetAnalogDemodulator(this._handle, out Analog_DemodulatorAsUShort);
            Analog_Demodulator = System.Convert.ToBoolean(Analog_DemodulatorAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the AM/FM demodulator of the instrument.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FS-K7.
        /// 
        /// Mode(s):
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// [SENSe:]ADEMod:SET &lt;sample rate&gt;,&lt;record length&gt;,&lt;trigger source&gt;,&lt;trigger slope&gt;,&lt;offset samples&gt;,&lt;# of meas&gt;
        /// </summary>
        /// <param name="Sample_Rate">
        /// Sets the frequency at which measurement values are taken from the A/D converter and stored in I/Q memory (with filter type = normal).
        /// 
        /// Valid Values:
        /// 15.625 kHz, 31.25 kHz, 62.5 kHz, 125 kHz, 250 kHz, 500 kHz, 1 MHz, 2 MHz, 4 MHz, 8 MHz, 16 MHz, 32 MHz
        /// 
        /// Default Value: 8.0e6 Hz
        /// 
        /// </param>
        /// <param name="Record_Length">
        /// Sets the number of samples to be stored in I/Q memory.
        /// 
        /// Valid Range: 1 to 130560 (128 * 1024 - 512)
        /// 
        /// Default Value: 501
        /// 
        /// </param>
        /// <param name="Trigger_Source">
        /// Selection of the trigger source to use for the demodulator.
        /// 
        /// Valid Values:
        /// RSESCI_TRIG_IMM (0) - Immediate
        /// RSESCI_TRIG_EXT (1) - External
        /// RSESCI_TRIG_IFP (3) - IF Power
        /// RSESCI_TRIG_RFP (4) - RF Power
        /// RSESCI_TRIG_FM  (6) - FM
        /// RSESCI_TRIG_AM  (7) - AM
        /// RSESCI_TRIG_PM  (8) - PM
        /// RSESCI_TRIG_AF  (9) - AF
        /// 
        /// Default Value: RSESCI_TRIG_IMM (0)
        /// 
        /// </param>
        /// <param name="Trigger_Slope">
        /// Used slope of the trigger signal. The value indicated here will be ignored for &lt;trigger source&gt; = Immediate.
        /// 
        /// Valid Values:
        /// RSESCI_POS (0) - Positive
        /// RSESCI_NEG (1) - Negative
        /// 
        /// Default Value: RSESCI_POS (0)
        /// 
        /// </param>
        /// <param name="Offset_Samples">
        /// Sets the number of samples to be used as an offset to the trigger signal.
        /// 
        /// Valid Range: -65024 to 130559 (= -64 * 1024 + 512 to 128 * 1024 - 513)
        /// 
        /// Default Value: 0
        /// 
        /// Note:
        /// 
        /// The value indicated here will be ignored for &lt;trigger source&gt; = Immediate.
        /// 
        /// </param>
        /// <param name="Number_of_Meas">
        /// Sets the number of repetitions of the measurement to be executed. The value indicated here is especially necessary for the average/maxhold/minhold function.
        /// 
        /// Valid Range:0 to 32767
        /// 
        /// Default Value: 0
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetAnalogDemodulatorParameters(double Sample_Rate, int Record_Length, int Trigger_Source, int Trigger_Slope, int Offset_Samples, int Number_of_Meas)
        {
            int pInvokeResult = PInvoke.SetAnalogDemodulatorParameters(this._handle, Sample_Rate, Record_Length, Trigger_Source, Trigger_Slope, Offset_Samples, Number_of_Meas);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function reads configuration of the AM/FM demodulator of the instrument.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FS-K7.
        /// 
        /// Mode(s):
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// [SENSe:]ADEMod:SET?
        /// </summary>
        /// <param name="Sample_Rate">
        /// Gets the frequency at which measurement values are taken from the A/D converter and stored in I/Q memory.
        /// 
        /// </param>
        /// <param name="Record_Length">
        /// Gets the number of samples to be stored in I/Q memory.
        /// 
        /// </param>
        /// <param name="Trigger_Source">
        /// Returns selection of the trigger source to use for the demodulator.
        /// 
        /// Valid Values:
        /// RSESCI_TRIG_IMM (0) - Immediate
        /// RSESCI_TRIG_EXT (1) - External
        /// RSESCI_TRIG_IFP (3) - IF Power
        /// RSESCI_TRIG_RFP (4) - RF Power
        /// RSESCI_TRIG_FM  (6) - FM
        /// RSESCI_TRIG_AM  (7) - AM
        /// RSESCI_TRIG_PM  (8) - PM
        /// RSESCI_TRIG_AF  (9) - AF
        /// 
        /// </param>
        /// <param name="Trigger_Slope">
        /// Used slope of the trigger signal.
        /// 
        /// Valid Values:
        /// RSESCI_POS (0) - Positive
        /// RSESCI_NEG (1) - Negative
        /// 
        /// </param>
        /// <param name="Offset_Samples">
        /// Gets the number of samples to be used as an offset to the trigger signal.
        /// 
        /// </param>
        /// <param name="Number_of_Meas">
        /// Gets the number of repetitions of the measurement to be executed.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetAnalogDemodulatorParameters(out double Sample_Rate, out int Record_Length, out int Trigger_Source, out int Trigger_Slope, out int Offset_Samples, out int Number_of_Meas)
        {
            int pInvokeResult = PInvoke.GetAnalogDemodulatorParameters(this._handle, out Sample_Rate, out Record_Length, out Trigger_Source, out Trigger_Slope, out Offset_Samples, out Number_of_Meas);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the type of evaluation of the measured data. This corresponds to the selection of the Result Display in manual mode.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FS-K7.
        /// 
        /// Mode(s):
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:FEED 'XTIM:AM' | 'XTIM:RFPower[:TDOMain]' | 'XTIM:RFPower:AFSPectrum&lt;1...3&gt;' | 'XTIM:FM[:TDOMain]' | 'XTIM:FM:AFSPectrum&lt;1...3&gt;' | 'XTIM:SPECtrum' | 'XTIM:PM[:TDOMain]' | 'XTIM:PM:AFSPectrum&lt;1...3&gt;' | 'XTIM:AMSummary&lt;1...3&gt;' | 'XTIM:FMSummary&lt;1...3&gt;' | 'XTIM:PMSummary&lt;1...3&gt;' | 'XTIM:AM:REL' | 'XTIM:AMS:REL' | 'XTIM:AM:REL:AFSP'
        /// </summary>
        /// <param name="Result_Display">
        /// Selects the type of evaluation of the measured data.
        /// 
        /// Valid Values:
        /// RSESCI_FM_SUMMARY       (0)  - FM Summary
        /// RSESCI_DEMOD_FM_SIG     (1)  - Demodulated FM Signal
        /// RSESCI_AFS_FM_SIG       (2)  - AF Spectrum of FM Signal
        /// RSESCI_RFS_FM_SIG       (3)  - RF Spectrum of FM Signal
        /// RSESCI_PM_SUMMARY       (4)  - PM Summary
        /// RSESCI_DEMOD_PM_SIG     (5)  - Demodulated PM Signal
        /// RSESCI_AFS_PM_SIG       (6)  - AF Spectrum of PM Signal
        /// RSESCI_AM_SUMMARY       (7)  - AM Summary
        /// RSESCI_DEMOD_AM_SIG     (8)  - Demodulated AM Signal
        /// RSESCI_AFS_AM_SIG       (9)  - AF Spectrum of AM Signal
        /// RSESCI_RFP_AM_SIG       (10) - RF Power of AM Signal
        /// RSESCI_DEMOD_AM_SIG_REL (11) - Demodulated AM Signal Relative
        /// RSESCI_AM_SUMMARY_REL   (12) - AM Summary Relative
        /// RSESCI_AFS_AM_SIG_REL   (13) - AF Spectrum of AM Signal Relative
        /// 
        /// Default Value: RSESCI_FM_SUMMARY (0)
        /// 
        /// Notes:
        /// 
        /// (1) FM Summary ('XTIM:FMSummary&lt;1...3&gt;'):
        /// FM results, referred to trace 1 to 3.
        /// 
        /// (2) Demodulated FM Signal ('XTIM:FM[:TDOMain]'):
        /// Demodulated FM signal.
        /// 
        /// (3) AF Spectrum of FM Signal ('XTIM:FM:AFSPectrum&lt;1...3&gt;'):
        /// AF spectrum of demodulated FM signal; results referenced to trace 1 to 3.
        /// 
        /// (4) RF Spectrum of FM Signal ('XTIM:SPECtrum'):
        /// RF spectrum of FM signal determined from the measured data by means of FFT.
        /// 
        /// (5) PM Summary ('XTIM:PMSummary&lt;1...3&gt;'):
        /// PM results, referred to trace 1 to 3.
        /// 
        /// (6) Demodulated PM Signal ('XTIM:PM[:TDOMain]'):
        /// Demodulated PM signal.
        /// 
        /// (7) AF Spectrum of PM Signal ('XTIM:PM:AFSPectrum&lt;1...3&gt;'):
        /// AF spectrum of demodulated PM signal; results referenced to trace 1 to 3.
        /// 
        /// (8) AM Summary ('XTIM:AMSummary&lt;1...3&gt;'):
        /// AM results, referred to trace 1 to 3.
        /// 
        /// (9) Demodulated AM Signal ('XTIM:AM'):
        /// Demodulated AM signal in level display.
        /// 
        /// (10) AF Spectrum of AM Signal ('XTIM:RFPower:AFSPectrum&lt;1...3&gt;'):
        /// AF spectrum of demodulated AM signal; results referenced to trace 1 to 3.
        /// 
        /// (11) RF Power of AM Signal ('XTIM:RFPower[:TDOMain]'):
        /// Demodulated AM signal in level display.
        /// 
        /// (12) Demodulated AM Signal Relative ('XTIM:AM:REL'):
        /// Demodulated relative AM signal in level display.
        /// 
        /// (13) AM Summary Relative ('XTIM:AMSummary&lt;1...3&gt;:REL'):
        /// AM relative results, referred to trace 1 to 3.
        /// 
        /// (14) AF Spectrum of AM Signal Relative ('XTIM:AM:REL:AFSPectrum&lt;1...3&gt;'):
        /// AF spectrum of demodulated AM signal relative; results referenced to trace 1 to 3.
        /// </param>
        /// <param name="Trace">
        /// This control sets the trace.
        /// 
        /// Valid Range: 1 to 3
        /// 
        /// Default Value: 1
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetAnalogDemodulatorResultDisplay(int Result_Display, int Trace)
        {
            int pInvokeResult = PInvoke.SetAnalogDemodulatorResultDisplay(this._handle, Result_Display, Trace);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the measurement time for analog demodulation.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FS-K7.
        /// 
        /// Mode(s):
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// [SENSe:]ADEMod:MTIMe
        /// </summary>
        /// <param name="Measurement_Time">
        /// This control defines the measurement time for analog demodulation.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: 62.625e-6 s
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetAnalogDemodulationMeasurementTime(double Measurement_Time)
        {
            int pInvokeResult = PInvoke.SetAnalogDemodulationMeasurementTime(this._handle, Measurement_Time);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the measurement time for analog demodulation.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FS-K7.
        /// 
        /// Mode(s):
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// [SENSe:]ADEMod:MTIMe?
        /// </summary>
        /// <param name="Measurement_Time">
        /// This control returns the measurement time for analog demodulation.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetAnalogDemodulationMeasurementTime(out double Measurement_Time)
        {
            int pInvokeResult = PInvoke.GetAnalogDemodulationMeasurementTime(this._handle, out Measurement_Time);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the sample rate set up for the current analog demodulation measurement.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FS-K7.
        /// 
        /// Mode(s):
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// [SENSe:]ADEMod:SRATe?
        /// </summary>
        /// <param name="Sample_Rate">
        /// This control returns the sample rate set up for the current analog demodulation measurement.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetAnalogDemodulationSampleRate(out double Sample_Rate)
        {
            int pInvokeResult = PInvoke.GetAnalogDemodulationSampleRate(this._handle, out Sample_Rate);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the record length set up for the current analog demodulation measurement.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FS-K7.
        /// 
        /// Mode(s):
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// [SENSe:]ADEMod:RLENgth?
        /// </summary>
        /// <param name="Record_Length">
        /// This control returns the record length set up for the current analog demodulation measurement.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetAnalogDemodulationRecordLength(out double Record_Length)
        {
            int pInvokeResult = PInvoke.GetAnalogDemodulationRecordLength(this._handle, out Record_Length);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function activates the phase wrapping to ?PI ... +PI (-180 deg ... +180 deg).
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FS-K7.
        /// 
        /// Mode(s):
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:FORMat PHASe | UPHase
        /// </summary>
        /// <param name="Type">
        /// This control activates the phase wrapping.
        /// 
        /// Valid Values:
        /// RSESCI_PHASE_WRAP_PHAS  (0) - Phase
        /// RSESCI_PHASE_WRAP_UPH   (1) - U Phase
        /// 
        /// Default Value: RSESCI_PHASE_WRAP_PHAS  (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation.  The status code  either indicates success or describes an error or warning condition.  You examine the status code from each call to an instrument driver function to determine if an error occurred.
        /// 
        /// To obtain a text description of the status code, call the RsSpecAn_error_message function.  To obtain additional information about the error condition, call the RsSpecAn_GetError function.  To clear the error information from the driver, call the RsSpecAn_ClearError function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// WARNINGS:
        /// 3FFA2001  The instrument was in an uncalibrated state
        ///           when the measurement was taken.
        /// 3FFA2002  The measurement taken was over the instrument??s
        ///           range.
        ///   
        /// ERRORS:
        /// BFFA2001  The Active Marker is not enabled.
        /// BFFA2002  The Active Marker is not a delta marker.
        /// BFFA2003  The maximum waiting time for this operation was
        ///           exceeded.
        ///           
        /// This instrument driver also returns errors and warnings defined by other sources.  The following table defines the ranges of additional status codes that this driver can return.  The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFA0000 to 3FFA1FFF     IVI    Warnings
        /// 3FFF0000 to 3FFFFFFF     VISA   Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP Driver Warnings
        ///           
        /// BFFA0000 to BFFA1FFF     IVI    Errors
        /// BFFF0000 to BFFFFFFF     VISA   Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP Driver Errors
        /// 
        /// </returns>
        public int SetAnalogDemodulationPhaseWrap(int Type)
        {
            int pInvokeResult = PInvoke.SetAnalogDemodulationPhaseWrap(this._handle, Type);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the phase wrapping to ?PI ... +PI (-180 deg ... +180 deg).
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FS-K7.
        /// 
        /// Mode(s):
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:FORMat?
        /// </summary>
        /// <param name="Type">
        /// This control returns the phase wrapping.
        /// 
        /// Valid Values:
        /// RSESCI_PHASE_WRAP_PHAS  (0) - Phase
        /// RSESCI_PHASE_WRAP_UPH   (1) - U Phase
        /// </param>
        /// <returns>
        /// Returns the status code of this operation.  The status code  either indicates success or describes an error or warning condition.  You examine the status code from each call to an instrument driver function to determine if an error occurred.
        /// 
        /// To obtain a text description of the status code, call the RsSpecAn_error_message function.  To obtain additional information about the error condition, call the RsSpecAn_GetError function.  To clear the error information from the driver, call the RsSpecAn_ClearError function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        ///           
        /// Status    Description
        /// -------------------------------------------------
        /// WARNINGS:
        /// 3FFA2001  The instrument was in an uncalibrated state
        ///           when the measurement was taken.
        /// 3FFA2002  The measurement taken was over the instrument??s
        ///           range.
        ///   
        /// ERRORS:
        /// BFFA2001  The Active Marker is not enabled.
        /// BFFA2002  The Active Marker is not a delta marker.
        /// BFFA2003  The maximum waiting time for this operation was
        ///           exceeded.
        ///           
        /// This instrument driver also returns errors and warnings defined by other sources.  The following table defines the ranges of additional status codes that this driver can return.  The table lists the different include files that contain the defined constants for the particular status codes:
        ///           
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFA0000 to 3FFA1FFF     IVI    Warnings
        /// 3FFF0000 to 3FFFFFFF     VISA   Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP Driver Warnings
        ///           
        /// BFFA0000 to BFFA1FFF     IVI    Errors
        /// BFFF0000 to BFFFFFFF     VISA   Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP Driver Errors
        /// 
        /// </returns>
        public int GetAnalogDemodulationPhaseWrap(out int Type)
        {
            int pInvokeResult = PInvoke.GetAnalogDemodulationPhaseWrap(this._handle, out Type);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function activates the input of the reference
        /// point, where the phase is set to 0 rad.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FS-K7.
        /// 
        /// Mode(s):
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// [SENSe1:]ADEMod:PM:RPO:X
        /// </summary>
        /// <param name="Zero_Phase_Ref_Point">
        /// This control activates the input of the reference
        /// point, where the phase is set to 0 rad.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: 10.0e-6 s
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetAnalogDemodulationZeroPhaseRefPoint(double Zero_Phase_Ref_Point)
        {
            int pInvokeResult = PInvoke.SetAnalogDemodulationZeroPhaseRefPoint(this._handle, Zero_Phase_Ref_Point);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the input of the reference
        /// point, where the phase is set to 0 rad.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FS-K7.
        /// 
        /// Mode(s):
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// [SENSe1:]ADEMod:PM:RPO:X?
        /// </summary>
        /// <param name="Zero_Phase_Ref_Point">
        /// This control returns the input of the reference
        /// point, where the phase is set to 0 rad.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetAnalogDemodulationZeroPhaseRefPoint(out double Zero_Phase_Ref_Point)
        {
            int pInvokeResult = PInvoke.GetAnalogDemodulationZeroPhaseRefPoint(this._handle, out Zero_Phase_Ref_Point);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects demodulation bandwidth AF filters of FM demodulation.
        /// 
        /// Note:
        /// 
        /// (1) This function is only valid in conjunction with the option R&amp;S FS-K7.
        /// 
        /// Mode(s):
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// [SENSe:]FILTer:HPASs[:STATe] ON | OFF
        /// [SENSe:]FILTer:HPASs:FREQuency 50 Hz | 300 Hz
        /// [SENSe:]FILTer:LPASs[:STATe] ON | OFF
        /// [SENSe:]FILTer:LPASs:FREQuency 3 kHz | 15 kHz | 150 kHz
        /// [SENSe:]FILTer:LPASs:FREQuency:RELative 5 | 10 | 25
        /// [SENSe:]FILTer:DEMPhasis[:STATe] ON | OFF
        /// [SENSe:]FILTer:DEMPhasis:TCONstant 25 us | 50 us | 75 us | 750 us
        /// </summary>
        /// <param name="Filter_Type">
        /// This control selects the filter type.
        /// 
        /// Valid Values:
        /// RSESCI_FM_FILTER_HP      (0) - High Pass AF Filter
        /// RSESCI_FM_FILTER_LP      (1) - Low Pass AF Filter
        /// RSESCI_FM_FILTER_LP_REL  (2) - Low Pass AF Filter Relative
        /// RSESCI_FM_FILTER_DEEMP   (3) - AM/FM Deemphasis
        /// 
        /// Default Value: RSESCI_FILTER_HP (0)
        /// 
        /// Notes:
        /// 
        /// (1) High Pass AF Filter:
        /// Activates the high-pass filter in the AF-branch for analog demodulation.
        /// 
        /// (2) Low Pass AF Filter:
        /// Activates the low-pass filter in the AF-branch with analog demodulation.
        /// 
        /// (3) Low Pass AF Filter Relative:
        /// Activates the relative low-pass filter in the AF-branch with analog demodulation.
        /// 
        /// (4) AM/FM Deemphasis:
        /// Activates the selected de-emphasis for analog demodulation.
        /// </param>
        /// <param name="Filter_State">
        /// This control activates the selected filter.
        /// 
        /// Valid Range:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <param name="Filter_Frequency">
        /// This control sets the filter frequency.
        /// 
        /// Valid Values:
        ///   for High Pass filter:  50 Hz,   300 Hz
        ///   for Low Pass filter: 3000 Hz, 15000 Hz, 150000 Hz
        ///   for Low Pass Relative filter: 5 %, 10% and 25% Hz
        /// 
        /// Default Value: 25.0 Hz
        /// 
        /// Note(s):
        /// 
        /// (1) High Pass:
        /// 
        ///     50 Hz for 200 Hz &lt;= demodulation bandwidth &lt;=  4 MHz
        ///    300 Hz     800 Hz &lt;= demodulation bandwidth &lt;= 16 MHz
        /// 
        /// (2) Low Pass:
        /// 
        ///     3000 Hz for 6.4 kHz &lt;= demodulation bandwidth &lt;=  4 MHz
        ///    15000 Hz      50 kHz &lt;= demodulation bandwidth &lt;= 16 MHz
        ///   150000 Hz     400 kHz &lt;= demodulation bandwidth &lt;= 16 MHz
        /// </param>
        /// <param name="AM_FM_Deemphasis_Value">
        /// This control sets the time constant of the de-emphasis for analog demodulation.
        /// 
        /// Valid Values:
        /// 25.0E-6 s, 50.0E-6 s, 75.0E-6 s, 750.0E-6 s
        /// 
        /// Default Value: 25.0E-6 s
        /// 
        /// Note(s):
        /// 
        /// (1) The de-emphasis is active in the following demodulation bandwidth range:
        /// 
        ///      25 us for  25 kHz &lt;= demodulation bandwidth &lt;= 30 MHz
        ///      50 us     6.4 kHz &lt;= demodulation bandwidth &lt;= 18 MHz
        ///      75 us     6.4 kHz &lt;= demodulation bandwidth &lt;= 18 MHz
        ///     750 us     800  Hz &lt;= demodulation bandwidth &lt;=  4 MHz
        /// 
        /// (2) The R&amp;S FSP requires the option FSP-B70 for the de-emphasis.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code
        /// either indicates success or describes an error or warning
        /// condition. You examine the status code from each call to an
        /// instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the
        /// rsfsq_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        /// 
        /// Status    Description
        /// -------------------------------------------------
        /// 
        /// BFFC0002  Parameter 2 (Filter Type) out of range.
        /// BFFC0003  Parameter 3 (Filter State) out of range.
        /// BFFC0004  Parameter 4 (Filter Frequency) out of range.
        /// BFFC0005  Parameter 5 (AM/FM Deemphasis Value) out of range.
        /// 
        /// BFFC09F0  Instrument status error
        /// BFFC09F1  Option not installed
        /// BFFC09F2  Conflict with setup parameters
        /// BFFC09F3  Data are not available
        /// BFFC09F4  Unknown option
        /// BFFC09F5  Calibration failed
        /// BFFC09F6  Function does not support FSEx parameter value
        /// BFFC09F7  Overload. Instrument does not return I/Q values
        /// 
        /// This instrument driver also returns errors and warnings defined
        /// by other sources. The following table defines the ranges of
        /// additional status codes that this driver can return. The table
        /// lists the different include files that contain the defined
        /// constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int ConfigureAnalogDemodulationFilter(int Filter_Type, bool Filter_State, double Filter_Frequency, double AM_FM_Deemphasis_Value)
        {
            int pInvokeResult = PInvoke.ConfigureAnalogDemodulationFilter(this._handle, Filter_Type, System.Convert.ToUInt16(Filter_State), Filter_Frequency, AM_FM_Deemphasis_Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function activates the selected filter.
        /// 
        /// Note:
        /// 
        /// (1) This function is only valid in conjunction with the option R&amp;S FS-K7.
        /// 
        /// Mode(s):
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// [SENSe:]FILTer:HPASs[:STATe] ON | OFF
        /// [SENSe:]FILTer:LPASs[:STATe] ON | OFF
        /// [SENSe:]FILTer:DEMPhasis[:STATe] ON | OFF
        /// </summary>
        /// <param name="Filter_Type">
        /// This control selects the filter type.
        /// 
        /// Valid Values:
        /// RSESCI_FM_FILTER_HP      (0) - High Pass AF Filter
        /// RSESCI_FM_FILTER_LP      (1) - Low Pass AF Filter
        /// RSESCI_FM_FILTER_LP_REL  (2) - Low Pass AF Filter Relative
        /// RSESCI_FM_FILTER_DEEMP   (3) - AM/FM Deemphasis
        /// 
        /// Default Value: RSESCI_FILTER_HP (0)
        /// 
        /// Notes:
        /// 
        /// (1) High Pass AF Filter:
        /// Activates the high-pass filter in the AF-branch for analog demodulation.
        /// 
        /// (2) Low Pass AF Filter:
        /// Activates the low-pass filter in the AF-branch with analog demodulation.
        /// 
        /// (3) Low Pass AF Filter Relative:
        /// Activates the relative low-pass filter in the AF-branch with analog demodulation.
        /// 
        /// (4) AM/FM Deemphasis:
        /// Activates the selected de-emphasis for analog demodulation.
        /// </param>
        /// <param name="Filter_State">
        /// This control activates the selected filter.
        /// 
        /// Valid Range:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code
        /// either indicates success or describes an error or warning
        /// condition. You examine the status code from each call to an
        /// instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the
        /// rsfsq_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        /// 
        /// Status    Description
        /// -------------------------------------------------
        /// 
        /// BFFC0002  Parameter 2 (Filter Type) out of range.
        /// BFFC0003  Parameter 3 (Filter State) out of range.
        /// BFFC0004  Parameter 4 (Filter Frequency) out of range.
        /// BFFC0005  Parameter 5 (AM/FM Deemphasis Value) out of range.
        /// 
        /// BFFC09F0  Instrument status error
        /// BFFC09F1  Option not installed
        /// BFFC09F2  Conflict with setup parameters
        /// BFFC09F3  Data are not available
        /// BFFC09F4  Unknown option
        /// BFFC09F5  Calibration failed
        /// BFFC09F6  Function does not support FSEx parameter value
        /// BFFC09F7  Overload. Instrument does not return I/Q values
        /// 
        /// This instrument driver also returns errors and warnings defined
        /// by other sources. The following table defines the ranges of
        /// additional status codes that this driver can return. The table
        /// lists the different include files that contain the defined
        /// constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetAnalogDemodulationFilterState(int Filter_Type, bool Filter_State)
        {
            int pInvokeResult = PInvoke.SetAnalogDemodulationFilterState(this._handle, Filter_Type, System.Convert.ToUInt16(Filter_State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries state of selected filter.
        /// 
        /// Note:
        /// 
        /// (1) This function is only valid in conjunction with the option R&amp;S FS-K7.
        /// 
        /// Mode(s):
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// [SENSe:]FILTer:HPASs[:STATe]?
        /// [SENSe:]FILTer:LPASs[:STATe]?
        /// [SENSe:]FILTer:DEMPhasis[:STATe]?
        /// </summary>
        /// <param name="Filter_Type">
        /// This control selects the filter type.
        /// 
        /// Valid Values:
        /// RSESCI_FM_FILTER_HP      (0) - High Pass AF Filter
        /// RSESCI_FM_FILTER_LP      (1) - Low Pass AF Filter
        /// RSESCI_FM_FILTER_LP_REL  (2) - Low Pass AF Filter Relative
        /// RSESCI_FM_FILTER_DEEMP   (3) - AM/FM Deemphasis
        /// 
        /// Default Value: RSESCI_FILTER_HP (0)
        /// 
        /// Notes:
        /// 
        /// (1) High Pass AF Filter:
        /// Activates the high-pass filter in the AF-branch for analog demodulation.
        /// 
        /// (2) Low Pass AF Filter:
        /// Activates the low-pass filter in the AF-branch with analog demodulation.
        /// 
        /// (3) Low Pass AF Filter Relative:
        /// Activates the relative low-pass filter in the AF-branch with analog demodulation.
        /// 
        /// (4) AM/FM Deemphasis:
        /// Activates the selected de-emphasis for analog demodulation.
        /// </param>
        /// <param name="Filter_State">
        /// This control returns state of selected filter.
        /// 
        /// Valid Range:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code
        /// either indicates success or describes an error or warning
        /// condition. You examine the status code from each call to an
        /// instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the
        /// rsfsq_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        /// 
        /// Status    Description
        /// -------------------------------------------------
        /// 
        /// BFFC0002  Parameter 2 (Filter Type) out of range.
        /// BFFC0003  Parameter 3 (Filter State) out of range.
        /// BFFC0004  Parameter 4 (Filter Frequency) out of range.
        /// BFFC0005  Parameter 5 (AM/FM Deemphasis Value) out of range.
        /// 
        /// BFFC09F0  Instrument status error
        /// BFFC09F1  Option not installed
        /// BFFC09F2  Conflict with setup parameters
        /// BFFC09F3  Data are not available
        /// BFFC09F4  Unknown option
        /// BFFC09F5  Calibration failed
        /// BFFC09F6  Function does not support FSEx parameter value
        /// BFFC09F7  Overload. Instrument does not return I/Q values
        /// 
        /// This instrument driver also returns errors and warnings defined
        /// by other sources. The following table defines the ranges of
        /// additional status codes that this driver can return. The table
        /// lists the different include files that contain the defined
        /// constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetAnalogDemodulationFilterState(int Filter_Type, out bool Filter_State)
        {
            ushort Filter_StateAsUShort;
            int pInvokeResult = PInvoke.GetAnalogDemodulationFilterState(this._handle, Filter_Type, out Filter_StateAsUShort);
            Filter_State = System.Convert.ToBoolean(Filter_StateAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the filter frequency.
        /// 
        /// Note:
        /// 
        /// (1) This function is only valid in conjunction with the option R&amp;S FS-K7.
        /// 
        /// Mode(s):
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// [SENSe:]FILTer:HPASs:FREQuency 50 Hz | 300 Hz
        /// [SENSe:]FILTer:LPASs:FREQuency 3 kHz | 15 kHz | 150 kHz
        /// [SENSe:]FILTer:LPASs:FREQuency:RELative 5 | 10 | 25
        /// 
        /// </summary>
        /// <param name="Filter_Type">
        /// This control selects the filter type.
        /// 
        /// Valid Values:
        /// RSESCI_FM_FILTER_HP      (0) - High Pass AF Filter
        /// RSESCI_FM_FILTER_LP      (1) - Low Pass AF Filter
        /// RSESCI_FM_FILTER_LP_REL  (2) - Low Pass AF Filter Relative
        /// 
        /// Default Value: RSESCI_FILTER_HP (0)
        /// 
        /// Notes:
        /// 
        /// (1) High Pass AF Filter:
        /// Activates the high-pass filter in the AF-branch for analog demodulation.
        /// 
        /// (2) Low Pass AF Filter:
        /// Activates the low-pass filter in the AF-branch with analog demodulation.
        /// 
        /// (3) Low Pass AF Filter Relative:
        /// Activates the relative low-pass filter in the AF-branch with analog demodulation.
        /// </param>
        /// <param name="Filter_Frequency">
        /// This control sets the filter frequency.
        /// 
        /// Valid Values:
        ///   for High Pass filter:  50 Hz,   300 Hz
        ///   for Low Pass filter: 3000 Hz, 15000 Hz, 150000 Hz
        ///   for Low Pass Relative filter: 5 %, 10% and 25% Hz
        /// 
        /// Default Value: 25.0 Hz
        /// 
        /// Note(s):
        /// 
        /// (1) High Pass:
        /// 
        ///     50 Hz for 200 Hz &lt;= demodulation bandwidth &lt;=  4 MHz
        ///    300 Hz     800 Hz &lt;= demodulation bandwidth &lt;= 16 MHz
        /// 
        /// (2) Low Pass:
        /// 
        ///     3000 Hz for 6.4 kHz &lt;= demodulation bandwidth &lt;=  4 MHz
        ///    15000 Hz      50 kHz &lt;= demodulation bandwidth &lt;= 16 MHz
        ///   150000 Hz     400 kHz &lt;= demodulation bandwidth &lt;= 16 MHz
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code
        /// either indicates success or describes an error or warning
        /// condition. You examine the status code from each call to an
        /// instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the
        /// rsfsq_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        /// 
        /// Status    Description
        /// -------------------------------------------------
        /// 
        /// BFFC0002  Parameter 2 (Filter Type) out of range.
        /// BFFC0003  Parameter 3 (Filter State) out of range.
        /// BFFC0004  Parameter 4 (Filter Frequency) out of range.
        /// BFFC0005  Parameter 5 (AM/FM Deemphasis Value) out of range.
        /// 
        /// BFFC09F0  Instrument status error
        /// BFFC09F1  Option not installed
        /// BFFC09F2  Conflict with setup parameters
        /// BFFC09F3  Data are not available
        /// BFFC09F4  Unknown option
        /// BFFC09F5  Calibration failed
        /// BFFC09F6  Function does not support FSEx parameter value
        /// BFFC09F7  Overload. Instrument does not return I/Q values
        /// 
        /// This instrument driver also returns errors and warnings defined
        /// by other sources. The following table defines the ranges of
        /// additional status codes that this driver can return. The table
        /// lists the different include files that contain the defined
        /// constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetAnalogDemodulationFilterFrequency(int Filter_Type, double Filter_Frequency)
        {
            int pInvokeResult = PInvoke.SetAnalogDemodulationFilterFrequency(this._handle, Filter_Type, Filter_Frequency);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the filter frequency.
        /// 
        /// Note:
        /// 
        /// (1) This function is only valid in conjunction with the option R&amp;S FS-K7.
        /// 
        /// Mode(s):
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// [SENSe:]FILTer:HPASs:FREQuency?
        /// [SENSe:]FILTer:LPASs:FREQuency?
        /// [SENSe:]FILTer:LPASs:FREQuency:RELative?
        /// 
        /// </summary>
        /// <param name="Filter_Type">
        /// This control selects the filter type.
        /// 
        /// Valid Values:
        /// RSESCI_FM_FILTER_HP      (0) - High Pass AF Filter
        /// RSESCI_FM_FILTER_LP      (1) - Low Pass AF Filter
        /// RSESCI_FM_FILTER_LP_REL  (2) - Low Pass AF Filter Relative
        /// 
        /// Default Value: RSESCI_FILTER_HP (0)
        /// 
        /// Notes:
        /// 
        /// (1) High Pass AF Filter:
        /// Activates the high-pass filter in the AF-branch for analog demodulation.
        /// 
        /// (2) Low Pass AF Filter:
        /// Activates the low-pass filter in the AF-branch with analog demodulation.
        /// 
        /// (3) Low Pass AF Filter Relative:
        /// Activates the relative low-pass filter in the AF-branch with analog demodulation.
        /// </param>
        /// <param name="Filter_Frequency">
        /// This control returns the filter frequency.
        /// 
        /// Valid Values:
        ///   for High Pass filter:  50 Hz,   300 Hz
        ///   for Low Pass filter: 3000 Hz, 15000 Hz, 150000 Hz
        ///   for Low Pass Relative filter: 5 %, 10% and 25% Hz
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code
        /// either indicates success or describes an error or warning
        /// condition. You examine the status code from each call to an
        /// instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the
        /// rsfsq_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        /// 
        /// Status    Description
        /// -------------------------------------------------
        /// 
        /// BFFC0002  Parameter 2 (Filter Type) out of range.
        /// BFFC0003  Parameter 3 (Filter State) out of range.
        /// BFFC0004  Parameter 4 (Filter Frequency) out of range.
        /// BFFC0005  Parameter 5 (AM/FM Deemphasis Value) out of range.
        /// 
        /// BFFC09F0  Instrument status error
        /// BFFC09F1  Option not installed
        /// BFFC09F2  Conflict with setup parameters
        /// BFFC09F3  Data are not available
        /// BFFC09F4  Unknown option
        /// BFFC09F5  Calibration failed
        /// BFFC09F6  Function does not support FSEx parameter value
        /// BFFC09F7  Overload. Instrument does not return I/Q values
        /// 
        /// This instrument driver also returns errors and warnings defined
        /// by other sources. The following table defines the ranges of
        /// additional status codes that this driver can return. The table
        /// lists the different include files that contain the defined
        /// constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetAnalogDemodulationFilterFrequency(int Filter_Type, out double Filter_Frequency)
        {
            int pInvokeResult = PInvoke.GetAnalogDemodulationFilterFrequency(this._handle, Filter_Type, out Filter_Frequency);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the time constant of the de-emphasis for analog demodulation.
        /// 
        /// Note:
        /// 
        /// (1) This function is only valid in conjunction with the option R&amp;S FS-K7.
        /// 
        /// Mode(s):
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// [SENSe:]FILTer:DEMPhasis:TCONstant 25 us | 50 us | 75 us | 750 us
        /// </summary>
        /// <param name="AM_FM_Deemphasis_Value">
        /// This control sets the time constant of the de-emphasis for analog demodulation.
        /// 
        /// Valid Values:
        /// 25.0E-6 s, 50.0E-6 s, 75.0E-6 s, 750.0E-6 s
        /// 
        /// Default Value: 25.0E-6 s
        /// 
        /// Note(s):
        /// 
        /// (1) The de-emphasis is active in the following demodulation bandwidth range:
        /// 
        ///      25 us for  25 kHz &lt;= demodulation bandwidth &lt;= 30 MHz
        ///      50 us     6.4 kHz &lt;= demodulation bandwidth &lt;= 18 MHz
        ///      75 us     6.4 kHz &lt;= demodulation bandwidth &lt;= 18 MHz
        ///     750 us     800  Hz &lt;= demodulation bandwidth &lt;=  4 MHz
        /// 
        /// (2) The R&amp;S FSP requires the option FSP-B70 for the de-emphasis.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code
        /// either indicates success or describes an error or warning
        /// condition. You examine the status code from each call to an
        /// instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the
        /// rsfsq_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        /// 
        /// Status    Description
        /// -------------------------------------------------
        /// 
        /// BFFC0002  Parameter 2 (Filter Type) out of range.
        /// BFFC0003  Parameter 3 (Filter State) out of range.
        /// BFFC0004  Parameter 4 (Filter Frequency) out of range.
        /// BFFC0005  Parameter 5 (AM/FM Deemphasis Value) out of range.
        /// 
        /// BFFC09F0  Instrument status error
        /// BFFC09F1  Option not installed
        /// BFFC09F2  Conflict with setup parameters
        /// BFFC09F3  Data are not available
        /// BFFC09F4  Unknown option
        /// BFFC09F5  Calibration failed
        /// BFFC09F6  Function does not support FSEx parameter value
        /// BFFC09F7  Overload. Instrument does not return I/Q values
        /// 
        /// This instrument driver also returns errors and warnings defined
        /// by other sources. The following table defines the ranges of
        /// additional status codes that this driver can return. The table
        /// lists the different include files that contain the defined
        /// constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetAnalogDemodulationFilterDeemphasis(double AM_FM_Deemphasis_Value)
        {
            int pInvokeResult = PInvoke.SetAnalogDemodulationFilterDeemphasis(this._handle, AM_FM_Deemphasis_Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the time constant of the de-emphasis for analog demodulation.
        /// 
        /// Note:
        /// 
        /// (1) This function is only valid in conjunction with the option R&amp;S FS-K7.
        /// 
        /// Mode(s):
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// [SENSe:]FILTer:DEMPhasis:TCONstant?
        /// </summary>
        /// <param name="AM_FM_Deemphasis_Value">
        /// This control returns the time constant of the de-emphasis for analog demodulation.
        /// 
        /// Valid Values:
        /// 25.0E-6 s, 50.0E-6 s, 75.0E-6 s, 750.0E-6 s
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code
        /// either indicates success or describes an error or warning
        /// condition. You examine the status code from each call to an
        /// instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the
        /// rsfsq_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        /// 
        /// Status    Description
        /// -------------------------------------------------
        /// 
        /// BFFC0002  Parameter 2 (Filter Type) out of range.
        /// BFFC0003  Parameter 3 (Filter State) out of range.
        /// BFFC0004  Parameter 4 (Filter Frequency) out of range.
        /// BFFC0005  Parameter 5 (AM/FM Deemphasis Value) out of range.
        /// 
        /// BFFC09F0  Instrument status error
        /// BFFC09F1  Option not installed
        /// BFFC09F2  Conflict with setup parameters
        /// BFFC09F3  Data are not available
        /// BFFC09F4  Unknown option
        /// BFFC09F5  Calibration failed
        /// BFFC09F6  Function does not support FSEx parameter value
        /// BFFC09F7  Overload. Instrument does not return I/Q values
        /// 
        /// This instrument driver also returns errors and warnings defined
        /// by other sources. The following table defines the ranges of
        /// additional status codes that this driver can return. The table
        /// lists the different include files that contain the defined
        /// constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetAnalogDemodulationFilterDeemphasis(out double AM_FM_Deemphasis_Value)
        {
            int pInvokeResult = PInvoke.GetAnalogDemodulationFilterDeemphasis(this._handle, out AM_FM_Deemphasis_Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the center frequency for result display AF spectrum.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FS-K7.
        /// 
        /// Mode(s):
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// [SENSe:]ADEMod:AF:CENTer
        /// </summary>
        /// <param name="Center_Frequency">
        /// This control sets the center frequency for result display AF spectrum.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: 1.25e6 Hz
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetAFCenterFrequency(double Center_Frequency)
        {
            int pInvokeResult = PInvoke.SetAFCenterFrequency(this._handle, Center_Frequency);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the center frequency for result display AF spectrum.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FS-K7.
        /// 
        /// Mode(s):
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// [SENSe:]ADEMod:AF:CENTer?
        /// </summary>
        /// <param name="Center_Frequency">
        /// This control returns the center frequency for result display AF spectrum.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetAFCenterFrequency(out double Center_Frequency)
        {
            int pInvokeResult = PInvoke.GetAFCenterFrequency(this._handle, out Center_Frequency);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the start frequency for result display AF spectrum.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FS-K7.
        /// 
        /// Mode(s):
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// [SENSe:]ADEMod:AF:STARt
        /// </summary>
        /// <param name="Start_Frequency">
        /// This control sets the start frequency for result display AF spectrum.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: 0.0 Hz
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetAFStartFrequency(double Start_Frequency)
        {
            int pInvokeResult = PInvoke.SetAFStartFrequency(this._handle, Start_Frequency);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the start frequency for result display AF spectrum.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FS-K7.
        /// 
        /// Mode(s):
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// [SENSe:]ADEMod:AF:STARt?
        /// </summary>
        /// <param name="Start_Frequency">
        /// This control returns the start frequency for result display AF spectrum.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetAFStartFrequency(out double Start_Frequency)
        {
            int pInvokeResult = PInvoke.GetAFStartFrequency(this._handle, out Start_Frequency);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the stop frequency for result display AF spectrum. The stop frequency is limited to half the measurement bandwidth of analog demodulation (SENS:ADEM:BAND).
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FS-K7.
        /// 
        /// Mode(s):
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// [SENSe:]ADEMod:AF:STOP
        /// </summary>
        /// <param name="Stop_Frequency">
        /// This control sets the stop frequency for result display AF spectrum.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: 2.5e6 Hz
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetAFStopFrequency(double Stop_Frequency)
        {
            int pInvokeResult = PInvoke.SetAFStopFrequency(this._handle, Stop_Frequency);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the stop frequency for result display AF spectrum.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FS-K7.
        /// 
        /// Mode(s):
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// [SENSe:]ADEMod:AF:STOP?
        /// </summary>
        /// <param name="Stop_Frequency">
        /// This control returns the stop frequency for result display AF spectrum.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetAFStopFrequency(out double Stop_Frequency)
        {
            int pInvokeResult = PInvoke.GetAFStopFrequency(this._handle, out Stop_Frequency);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function activates an automatic signal search.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FS-K7.
        /// 
        /// Mode(s):
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// [SENSe:]FREQuency:CW:AFC ONCE
        /// </summary>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int AutomaticSignalSearch()
        {
            int pInvokeResult = PInvoke.AutomaticSignalSearch(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the span for result display AF spectrum. The span is limited to half the measurement bandwidth of analog demodulation.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FS-K7.
        /// 
        /// Mode(s):
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// [SENSe:]ADEMod:AF:SPAN
        /// </summary>
        /// <param name="Span">
        /// This control sets the span for result display AF spectrum.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: 2.5e6 Hz
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetAFSpan(double Span)
        {
            int pInvokeResult = PInvoke.SetAFSpan(this._handle, Span);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the span for result display AF spectrum.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FS-K7.
        /// 
        /// Mode(s):
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// [SENSe:]ADEMod:AF:SPAN?
        /// </summary>
        /// <param name="Span">
        /// This control returns the span for result display AF spectrum.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetAFSpan(out double Span)
        {
            int pInvokeResult = PInvoke.GetAFSpan(this._handle, out Span);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the maximum span for result display AF spectrum. The maximum span corresponds to half the measurement bandwidth of analog demodulation (SENS:ADEM:BAND).
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FS-K7.
        /// 
        /// Mode(s):
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// [SENSe:]ADEMod:AF:SPAN:FULL 
        /// </summary>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetAFFullSpan()
        {
            int pInvokeResult = PInvoke.SetAFFullSpan(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the frequency range for result display of RF spectrum determined from FM demodulation data. The frequency range for result display is limited to the maximum span (SENS:ADEM:SPEC:SPAN:MAX) or to the measurement bandwidth of analog demodulation (SENS:ADEM:BAND).
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FS-K7.
        /// 
        /// Mode(s):
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// [SENSe:]ADEMod:SPECtrum:SPAN:ZOOM
        /// </summary>
        /// <param name="Span">
        /// This control sets the frequency range for result display of RF spectrum determined from FM demodulation data.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: 5.0e6 Hz
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetRFFrequencySpan(double Span)
        {
            int pInvokeResult = PInvoke.SetRFFrequencySpan(this._handle, Span);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the frequency range for result display of RF spectrum determined from FM demodulation data.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FS-K7.
        /// 
        /// Mode(s):
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// [SENSe:]ADEMod:SPECtrum:SPAN:ZOOM?
        /// </summary>
        /// <param name="Span">
        /// This control returns the frequency range for result display of RF spectrum determined from FM demodulation data.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetRFFrequencySpan(out double Span)
        {
            int pInvokeResult = PInvoke.GetRFFrequencySpan(this._handle, out Span);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the maximum frequency range for displaying the RF spectrum that was determined from the FM demodulation data. The maximum span corresponds to the measurement bandwidth of analog demodulation (SENS:ADEM:BAND).
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FS-K7.
        /// 
        /// Mode(s):
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// [SENSe:]ADEMod:SPECtrum:SPAN[:MAXimum]
        /// </summary>
        /// <param name="Span">
        /// This control sets the maximum frequency range for displaying the RF spectrum that was determined from the FM demodulation data.
        /// 
        /// Only discrete values are possible for the span since the available sampling rates are discrete.
        /// 
        /// Valid Values (Span ... Sampling rate):
        /// 10 MHz   ... 32 MHz
        /// 8 MHz    ... 16 MHz
        /// 5 MHz    ... 8 MHz
        /// 3 MHz    ... 4 MHz
        /// 1.6 MHz  ... 2 MHz
        /// 800 kHz  ... 1 MHz
        /// 400 kHz  ... 500 kHz
        /// 200 kHz  ... 250 kHz
        /// 100 kHz  ... 125 kHz
        /// 50 kHz   ... 62.5 kHz
        /// 25 kHz   ... 31.25 kHz
        /// 12.5 kHz ... 15.625 kHz
        /// 
        /// Default Value: 5.0e6 Hz
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetMaxRFFrequencySpan(double Span)
        {
            int pInvokeResult = PInvoke.SetMaxRFFrequencySpan(this._handle, Span);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the maximum frequency range for displaying the RF spectrum that was determined from the FM demodulation data.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FS-K7.
        /// 
        /// Mode(s):
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// [SENSe:]ADEMod:SPECtrum:SPAN[:MAXimum]?
        /// </summary>
        /// <param name="Span">
        /// This control returns the maximum frequency range for displaying the RF spectrum that was determined from the FM demodulation data.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetMaxRFFrequencySpan(out double Span)
        {
            int pInvokeResult = PInvoke.GetMaxRFFrequencySpan(this._handle, out Span);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the demodulation bandwidth used for analog demodulation. The required sampling rate is automatically set depending on the selected demodulation bandwidth. The available demodulation bandwidths are determined by the existing sampling rates.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FS-K7.
        /// 
        /// Mode(s):
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// [SENSe:]ADEMod:BANDwidth | BWIDth:DEModulation
        /// </summary>
        /// <param name="Demodulation_Bandwidth">
        /// This control defines the demodulation bandwidth used for analog demodulation.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: 5.0e6 Hz
        /// 
        /// Note:
        /// 
        /// The available demodulation bandwidths are determined by the existing sampling rates.
        /// 
        /// Rounded demodulation bandwidth ... Sampling rate
        /// 
        ///   10 MHz   ... 32 MHz
        ///   8 MHz    ... 16 MHz
        ///   5 MHz    ... 8 MHz
        ///   3 MHz    ... 4 MHz
        ///   1.6 MHz  ... 2 MHz
        ///   800 kHz  ... 1 MHz
        ///   400 kHz  ... 500 kHz
        ///   200 kHz  ... 250 kHz
        ///   100 kHz  ... 125 kHz
        ///   50 kHz   ... 62.5 kHz
        ///   25 kHz   ... 31.25 kHz
        ///   12.5 kHz ... 15.625 kHz
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetAnalogDemodulationBandwidth(double Demodulation_Bandwidth)
        {
            int pInvokeResult = PInvoke.SetAnalogDemodulationBandwidth(this._handle, Demodulation_Bandwidth);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the demodulation bandwidth used for analog demodulation.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FS-K7.
        /// 
        /// Mode(s):
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// [SENSe:]ADEMod:BANDwidth | BWIDth:DEModulation?
        /// </summary>
        /// <param name="Demodulation_Bandwidth">
        /// This control returns the demodulation bandwidth used for analog demodulation.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetAnalogDemodulationBandwidth(out double Demodulation_Bandwidth)
        {
            int pInvokeResult = PInvoke.GetAnalogDemodulationBandwidth(this._handle, out Demodulation_Bandwidth);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the resolution bandwidth for spectrum represetation that was determined from the analog demodulation data.
        /// 
        /// The recording time required is calculated from the and the sampling rate indirectly set via ADEM:SPEC:SPAN:MAX or ADEM:BAND. If the available recording time is not sufficient for the given bandwidth, the recording time is set to ist maximum and the resolution bandwidth is enlarged to the resulting bandwidth.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FS-K7.
        /// 
        /// Mode(s):
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// [SENSe:]ADEMod:SPECtrum:BANDwidth|BWIDth[:RESolution]
        /// </summary>
        /// <param name="Resolution_Bandwidth">
        /// This control sets the resolution bandwidth for spectrum represetation that was determined from the analog demodulation data.
        /// 
        /// Valid Range: 1.0 Hz to 10.0e6 Hz
        /// 
        /// Default Value: 61.2e3 Hz
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetAnalogDemodulationResolutionBandwidth(double Resolution_Bandwidth)
        {
            int pInvokeResult = PInvoke.SetAnalogDemodulationResolutionBandwidth(this._handle, Resolution_Bandwidth);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the resolution bandwidth for spectrum represetation that was determined from the analog demodulation data.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FS-K7.
        /// 
        /// Mode(s):
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// [SENSe:]ADEMod:SPECtrum:BANDwidth|BWIDth[:RESolution]?
        /// </summary>
        /// <param name="Resolution_Bandwidth">
        /// This control returns the resolution bandwidth for spectrum represetation that was determined from the analog demodulation data.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetAnalogDemodulationResolutionBandwidth(out double Resolution_Bandwidth)
        {
            int pInvokeResult = PInvoke.GetAnalogDemodulationResolutionBandwidth(this._handle, out Resolution_Bandwidth);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the coupling of the AF path of the analyzer.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FS-K7.
        /// 
        /// Mode(s):
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// [SENSe:]ADEMod:AF:COUPling AC | DC
        /// </summary>
        /// <param name="AF_Input_Coupling">
        /// This control selects the coupling of the AF path of the analyzer.
        /// 
        /// Valid Values:
        /// RSESCI_COUPLING_AC (0) - AC
        /// RSESCI_COUPLING_DC (1) - DC
        /// 
        /// Default Value: RSESCI_COUPLING_AC (0)
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetAFInputCoupling(int AF_Input_Coupling)
        {
            int pInvokeResult = PInvoke.SetAFInputCoupling(this._handle, AF_Input_Coupling);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the coupling of the AF path of the analyzer.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FS-K7.
        /// 
        /// Mode(s):
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// [SENSe:]ADEMod:AF:COUPling?
        /// </summary>
        /// <param name="AF_Input_Coupling">
        /// This control returns the coupling of the AF path of the analyzer.
        /// 
        /// Valid Values:
        /// RSESCI_COUPLING_AC (0) - AC
        /// RSESCI_COUPLING_DC (1) - DC
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetAFInputCoupling(out int AF_Input_Coupling)
        {
            int pInvokeResult = PInvoke.GetAFInputCoupling(this._handle, out AF_Input_Coupling);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the default unit for angles.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FS-K7.
        /// 
        /// Mode(s):
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// UNIT&lt;1|2&gt;:ANGLe DEG | RAD
        /// </summary>
        /// <param name="Angle_Units">
        /// Selects the default unit for angles.
        /// 
        /// Valid Range:
        /// RSESCI_ANGLE_UNITS_RAD (0) - RAD
        /// RSESCI_ANGLE_UNITS_DEG (1) - DEG
        /// 
        /// Default Value: RSESCI_ANGLE_UNITS_RAD (0)
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetAnalogDemodulationAngleUnits(int Angle_Units)
        {
            int pInvokeResult = PInvoke.SetAnalogDemodulationAngleUnits(this._handle, Angle_Units);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the default unit for angles.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FS-K7.
        /// 
        /// Mode(s):
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// UNIT&lt;1|2&gt;:ANGLe?
        /// </summary>
        /// <param name="Angle_Units">
        /// Returns the default unit for angles.
        /// 
        /// Valid Range:
        /// RSESCI_ANGLE_UNITS_RAD (0) - RAD
        /// RSESCI_ANGLE_UNITS_DEG (1) - DEG
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetAnalogDemodulationAngleUnits(out int Angle_Units)
        {
            int pInvokeResult = PInvoke.GetAnalogDemodulationAngleUnits(this._handle, out Angle_Units);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function enables or disables the zoom function for the FM-demodulator measurement data.
        /// 
        /// Depending on the selected measurement time and the demodulation bandwidth, the number of recorded test points may be greater than that shown on the display.
        /// 
        /// With the zoom function enabled, exactly 501 test points of the result memory are displayed from the time defined with [SENS:]ADEM:ZOOM:STARt.
        /// 
        /// With the zoom function disabled, data reduction is used to adapt the test points to the number of points available on the display.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FS-K7.
        /// 
        /// Mode(s):
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]ADEMod:ZOOM[:STATe&gt;] ON | OFF
        /// </summary>
        /// <param name="Zoom">
        /// This control enables or disables the zoom function for the FM-demodulator measurement data.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetAnalogDemodulationZoom(bool Zoom)
        {
            int pInvokeResult = PInvoke.SetAnalogDemodulationZoom(this._handle, System.Convert.ToUInt16(Zoom));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns if the zoom function for the FM-demodulator measurement data is enabled or disabled.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FS-K7.
        /// 
        /// Mode(s):
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]ADEMod:ZOOM[:STATe&gt;]?
        /// </summary>
        /// <param name="Zoom">
        /// This control returns if the zoom function for the FM-demodulator measurement data is enabled or disabled.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetAnalogDemodulationZoom(out bool Zoom)
        {
            ushort ZoomAsUShort;
            int pInvokeResult = PInvoke.GetAnalogDemodulationZoom(this._handle, out ZoomAsUShort);
            Zoom = System.Convert.ToBoolean(ZoomAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the start time for the display of individual measured values of the FM demodulator. The maximum possible value depends on the measurement time, which is set in the instrument and can be queried with command [SENSe]:ADEMod:MTIMe?.
        /// 
        /// If the zoom function is activated, 501 test points are displayed from the specified start time.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FS-K7.
        /// 
        /// Mode(s):
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]ADEMod:ZOOM:STARt
        /// </summary>
        /// <param name="Zoom_Start">
        /// Selects the start time for the display of individual measured values of the FM demodulator.
        /// 
        /// Valid Range: 0.0 s to measurement time
        /// 
        /// Default Value: 0.0 s
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetAnalogDemodulationZoomStart(double Zoom_Start)
        {
            int pInvokeResult = PInvoke.SetAnalogDemodulationZoomStart(this._handle, Zoom_Start);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the start time for the display of individual measured values of the FM demodulator.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FS-K7.
        /// 
        /// Mode(s):
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]ADEMod:ZOOM:STARt?
        /// </summary>
        /// <param name="Zoom_Start">
        /// Returns the start time for the display of individual measured values of the FM demodulator.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetAnalogDemodulationZoomStart(out double Zoom_Start)
        {
            int pInvokeResult = PInvoke.GetAnalogDemodulationZoomStart(this._handle, out Zoom_Start);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the results of FM modulation measurement.
        /// 
        ///   Result of measurement with detector +PK
        ///   Result of measurement with detector -PK
        ///   Result of averaging ?PK/2
        ///   Result of rms measurement
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FS-K7.
        /// 
        /// Mode(s):
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:ADEMod:FM[:RESult&lt;1...3&gt;]? PPEak | MPEak | MIDDle | RMS
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Marker">
        /// This control sets the marker.
        /// 
        /// Valid Range: 1 to 4
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Trace">
        /// This control sets the trace.
        /// 
        /// Valid Range: 1 to 3
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Measurement">
        /// Specifies measurement result type.
        /// 
        /// Valid Range:
        /// RSESCI_MOD_MEAS_PPEAK  (0) - +PK
        /// RSESCI_MOD_MEAS_MPEAK  (1) - -PK
        /// RSESCI_MOD_MEAS_MIDDLE (2) - +/- PK/2
        /// RSESCI_MOD_MEAS_RMS    (3) - RMS
        /// 
        /// Default Value: RSESCI_MOD_MEAS_PPEAK (0)
        /// 
        /// </param>
        /// <param name="Result">
        /// This control returns the results of modulation measurement.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetFMModulationMeasurement(int Window, int Marker, int Trace, int Measurement, out double Result)
        {
            int pInvokeResult = PInvoke.GetFMModulationMeasurement(this._handle, Window, Marker, Trace, Measurement, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the results of PM modulation measurement.
        /// 
        ///   Result of measurement with detector +PK
        ///   Result of measurement with detector -PK
        ///   Result of averaging ?PK/2
        ///   Result of rms measurement
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FS-K7.
        /// 
        /// Mode(s):
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:ADEMod:PM[:RESult&lt;1...3&gt;]? PPEak | MPEak | MIDDle | RMS
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Marker">
        /// This control sets the marker.
        /// 
        /// Valid Range: 1 to 4
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Trace">
        /// This control sets the trace.
        /// 
        /// Valid Range: 1 to 3
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Measurement">
        /// Specifies measurement result type.
        /// 
        /// Valid Range:
        /// RSESCI_MOD_MEAS_PPEAK  (0) - +PK
        /// RSESCI_MOD_MEAS_MPEAK  (1) - -PK
        /// RSESCI_MOD_MEAS_MIDDLE (2) - +/- PK/2
        /// RSESCI_MOD_MEAS_RMS    (3) - RMS
        /// 
        /// Default Value: RSESCI_MOD_MEAS_PPEAK (0)
        /// 
        /// </param>
        /// <param name="Result">
        /// This control returns the results of modulation measurement.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetPMModulationMeasurement(int Window, int Marker, int Trace, int Measurement, out double Result)
        {
            int pInvokeResult = PInvoke.GetPMModulationMeasurement(this._handle, Window, Marker, Trace, Measurement, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the results of AM modulation measurement.
        /// 
        ///   Result of measurement with detector +PK
        ///   Result of measurement with detector -PK
        ///   Result of averaging ?PK/2
        ///   Result of rms measurement
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FS-K7.
        /// 
        /// Mode(s):
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:ADEMod:AM[:RESult&lt;1...3&gt;]? PPEak | MPEak | MIDDle | RMS
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Marker">
        /// This control sets the marker.
        /// 
        /// Valid Range: 1 to 4
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Trace">
        /// This control sets the trace.
        /// 
        /// Valid Range: 1 to 3
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Measurement">
        /// Specifies measurement result type.
        /// 
        /// Valid Range:
        /// RSESCI_MOD_MEAS_PPEAK  (0) - +PK
        /// RSESCI_MOD_MEAS_MPEAK  (1) - -PK
        /// RSESCI_MOD_MEAS_MIDDLE (2) - +/- PK/2
        /// RSESCI_MOD_MEAS_RMS    (3) - RMS
        /// 
        /// Default Value: RSESCI_MOD_MEAS_PPEAK (0)
        /// 
        /// </param>
        /// <param name="Result">
        /// This control returns the results of modulation measurement.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetAMModulationMeasurement(int Window, int Marker, int Trace, int Measurement, out double Result)
        {
            int pInvokeResult = PInvoke.GetAMModulationMeasurement(this._handle, Window, Marker, Trace, Measurement, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the audio frequency with analog demodulation.
        /// 
        /// If several demodulation modes are activated simultaneously with commands SENS:ADEM:FM:TYPE, SENS:ADEM:RFP:TYPE, SENS:ADEM:FM:AFSP:TYPE or SENS:ADEM:RFP:AFSP, the audio frequency of the display mode selected with CALC:FEED is returned.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FS-K7.
        /// 
        /// Mode(s):
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:ADEMod:AFRequency[:RESult&lt;1...3&gt;]?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Marker">
        /// This control sets the marker.
        /// 
        /// Valid Range: 1 to 4
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Trace">
        /// This control sets the trace.
        /// 
        /// Valid Range: 1 to 3
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Result">
        /// This control returns the results of demodulation measurement.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetAnalogDemodulationAudioFrequency(int Window, int Marker, int Trace, out double Result)
        {
            int pInvokeResult = PInvoke.GetAnalogDemodulationAudioFrequency(this._handle, Window, Marker, Trace, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the frequency error with FM demodulation. The offset thus determined differs from that calculated in the command [SENSe:]ADEMod:FM:OFFSet?, since, for determination of the frequency deviation, the modulation is removed by means of lowpass filtering, producing results that are different from those obtained by averaging with the SENSe:...command.
        /// 
        /// This function is only available for traces in the Result Display FM. If another result display is selected, the function is disabled.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FS-K7.
        /// 
        /// Mode(s):
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:ADEMod:FERRor[:RESult&lt;1...3&gt;]?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Marker">
        /// This control sets the marker.
        /// 
        /// Valid Range: 1 to 4
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Trace">
        /// This control sets the trace.
        /// 
        /// Valid Range: 1 to 3
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Result">
        /// This control returns the results of demodulation measurement.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetAnalogDemodulationFrequencyError(int Window, int Marker, int Trace, out double Result)
        {
            int pInvokeResult = PInvoke.GetAnalogDemodulationFrequencyError(this._handle, Window, Marker, Trace, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the result of the SINAD measurement.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FS-K7.
        /// 
        /// Mode(s):
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:ADEMod:SINad:RESult&lt;1...3&gt;?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Marker">
        /// This control sets the marker.
        /// 
        /// Valid Range: 1 to 4
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Trace">
        /// This control sets the trace.
        /// 
        /// Valid Range: 1 to 3
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Result">
        /// This control returns the results of demodulation measurement.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetAnalogDemodulationSINAD(int Window, int Marker, int Trace, out double Result)
        {
            int pInvokeResult = PInvoke.GetAnalogDemodulationSINAD(this._handle, Window, Marker, Trace, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the result of the THD measurement.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FS-K7.
        /// 
        /// Mode(s):
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:ADEMod:THD:RESult&lt;1...3&gt;?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Marker">
        /// This control sets the marker.
        /// 
        /// Valid Range: 1 to 4
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Trace">
        /// This control sets the trace.
        /// 
        /// Valid Range: 1 to 3
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Result">
        /// This control returns the results of demodulation measurement.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetAnalogDemodulationTHD(int Window, int Marker, int Trace, out double Result)
        {
            int pInvokeResult = PInvoke.GetAnalogDemodulationTHD(this._handle, Window, Marker, Trace, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the carrier power.
        /// 
        /// With result display FM and SPECTRUM, the carrier power is determined from the current trace data (CLR/WRITE trace). With result display RF Power, the carrier power is determined from trace 1 to 3.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FS-K7.
        /// 
        /// Mode(s):
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:MARKer&lt;1...4&gt;:FUNCtion:ADEMod:CARRier[:RESult&lt;1...3&gt;]?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Marker">
        /// This control sets the marker.
        /// 
        /// Valid Range: 1 to 4
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Trace">
        /// This control sets the trace.
        /// 
        /// Valid Range: 1 to 3
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Result">
        /// This control returns the results of demodulation measurement.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetAnalogDemodulationCarrierPower(int Window, int Marker, int Trace, out double Result)
        {
            int pInvokeResult = PInvoke.GetAnalogDemodulationCarrierPower(this._handle, Window, Marker, Trace, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the simultaneously measured result types of the AF spectrum of the demodulated signal.
        /// 
        /// Settings other than OFF can be assigned only to one result type at a time.
        /// 
        /// If all result types are set to OFF, the AF spectrum of the demodulated signal is not calculated.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FS-K7.
        /// 
        /// Mode(s):
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// [SENSe:]ADEMod:AM:AFSPectrum[:TYPE] &lt;result type 1&gt;,&lt;result type 2&gt;,&lt;result type 3&gt;
        /// [SENSe:]ADEMod:FM:AFSPectrum[:TYPE] &lt;result type 1&gt;,&lt;result type 2&gt;,&lt;result type 3&gt;
        /// [SENSe:]ADEMod:PM:AFSPectrum[:TYPE] &lt;result type 1&gt;,&lt;result type 2&gt;,&lt;result type 3&gt;
        /// </summary>
        /// <param name="Demodulation_Type">
        /// This parameter selects the demodulation type.
        /// 
        /// Valid Range:
        /// RSESCI_DEMOD_TYPE_AM (0) - AM
        /// RSESCI_DEMOD_TYPE_FM (1) - FM
        /// RSESCI_DEMOD_TYPE_PM (2) - PM
        /// 
        /// Default Value: RSESCI_DEMOD_TYPE_AM (0)
        /// 
        /// </param>
        /// <param name="Result_Type_1">
        /// Selects simultaneously measured result types of the AF spectrum of the demodulated signal.
        /// 
        /// Valid Range:
        /// RSESCI_RESULT_TYPE_OFF     (0) - Off
        /// RSESCI_RESULT_TYPE_WRITE   (1) - Write
        /// RSESCI_RESULT_TYPE_AVERAGE (2) - Average
        /// RSESCI_RESULT_TYPE_MAXHOLD (3) - Maxhold
        /// RSESCI_RESULT_TYPE_MINHOLD (4) - Minhold
        /// RSESCI_RESULT_TYPE_VIEW    (5) - View
        /// 
        /// Default Value: RSESCI_RESULT_TYPE_OFF (0)
        /// 
        /// Notes:
        /// 
        /// (1) Off:
        /// Result type is not used.
        /// 
        /// (2) Write:
        /// The current measurement results are recorded.
        /// 
        /// (3) Average:
        /// The measurement results averaged over the given number of measurements are obtained.
        /// 
        /// (4) Maxhold:
        /// The maximum result is determined over the given number of measurements.
        /// 
        /// (5) Minhold:
        /// The minimum result is determined over the given number of measurements.
        /// 
        /// (6) View:
        /// The measurement result in the display is frozen, i.e. not recalculated with further measurements.
        /// 
        /// </param>
        /// <param name="Result_Type_2">
        /// Selects simultaneously measured result types of the AF spectrum of the demodulated signal.
        /// 
        /// Valid Range:
        /// RSESCI_RESULT_TYPE_OFF     (0) - Off
        /// RSESCI_RESULT_TYPE_WRITE   (1) - Write
        /// RSESCI_RESULT_TYPE_AVERAGE (2) - Average
        /// RSESCI_RESULT_TYPE_MAXHOLD (3) - Maxhold
        /// RSESCI_RESULT_TYPE_MINHOLD (4) - Minhold
        /// RSESCI_RESULT_TYPE_VIEW    (5) - View
        /// 
        /// Default Value: RSESCI_RESULT_TYPE_OFF (0)
        /// 
        /// Notes:
        /// 
        /// (1) Off:
        /// Result type is not used.
        /// 
        /// (2) Write:
        /// The current measurement results are recorded.
        /// 
        /// (3) Average:
        /// The measurement results averaged over the given number of measurements are obtained.
        /// 
        /// (4) Maxhold:
        /// The maximum result is determined over the given number of measurements.
        /// 
        /// (5) Minhold:
        /// The minimum result is determined over the given number of measurements.
        /// 
        /// (6) View:
        /// The measurement result in the display is frozen, i.e. not recalculated with further measurements.
        /// 
        /// </param>
        /// <param name="Result_Type_3">
        /// Selects simultaneously measured result types of the AF spectrum of the demodulated signal.
        /// 
        /// Valid Range:
        /// RSESCI_RESULT_TYPE_OFF     (0) - Off
        /// RSESCI_RESULT_TYPE_WRITE   (1) - Write
        /// RSESCI_RESULT_TYPE_AVERAGE (2) - Average
        /// RSESCI_RESULT_TYPE_MAXHOLD (3) - Maxhold
        /// RSESCI_RESULT_TYPE_MINHOLD (4) - Minhold
        /// RSESCI_RESULT_TYPE_VIEW    (5) - View
        /// 
        /// Default Value: RSESCI_RESULT_TYPE_OFF (0)
        /// 
        /// Notes:
        /// 
        /// (1) Off:
        /// Result type is not used.
        /// 
        /// (2) Write:
        /// The current measurement results are recorded.
        /// 
        /// (3) Average:
        /// The measurement results averaged over the given number of measurements are obtained.
        /// 
        /// (4) Maxhold:
        /// The maximum result is determined over the given number of measurements.
        /// 
        /// (5) Minhold:
        /// The minimum result is determined over the given number of measurements.
        /// 
        /// (6) View:
        /// The measurement result in the display is frozen, i.e. not recalculated with further measurements.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetAFSpectrumResultTypes(int Demodulation_Type, int Result_Type_1, int Result_Type_2, int Result_Type_3)
        {
            int pInvokeResult = PInvoke.SetAFSpectrumResultTypes(this._handle, Demodulation_Type, Result_Type_1, Result_Type_2, Result_Type_3);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function reads out measurement results of the AF spectrum of the demodulated signal with the specified result type.
        /// 
        /// Traces in the VIEW state cannot be queried.
        /// 
        /// The specified result type must be one of those configured with command [SENSe:]ADEMod:AM[|FM|PM]:AFSPectrum[:TYPE]. Otherwise a query error will be generated.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FS-K7.
        /// 
        /// Mode(s):
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// [SENSe:]ADEMod:AM:AFSPectrum:RESult? &lt;result type&gt;
        /// [SENSe:]ADEMod:FM:AFSPectrum:RESult? &lt;result type&gt;
        /// [SENSe:]ADEMod:PM:AFSPectrum:RESult? &lt;result type&gt;
        /// </summary>
        /// <param name="Demodulation_Type">
        /// This parameter selects the demodulation type.
        /// 
        /// Valid Range:
        /// RSESCI_DEMOD_TYPE_AM (0) - AM
        /// RSESCI_DEMOD_TYPE_FM (1) - FM
        /// RSESCI_DEMOD_TYPE_PM (2) - PM
        /// 
        /// Default Value: RSESCI_DEMOD_TYPE_AM (0)
        /// 
        /// </param>
        /// <param name="Result_Type">
        /// Selects measurement result type of the AF spectrum of the demodulated signal.
        /// 
        /// Valid Range:
        /// RSESCI_RESULT_TYPE_WRITE   (1) - Write
        /// RSESCI_RESULT_TYPE_AVERAGE (2) - Average
        /// RSESCI_RESULT_TYPE_MAXHOLD (3) - Maxhold
        /// RSESCI_RESULT_TYPE_MINHOLD (4) - Minhold
        /// 
        /// Default Value: RSESCI_RESULT_TYPE_WRITE (1)
        /// 
        /// Notes:
        /// 
        /// (1) Write:
        /// The current measurement results are returned.
        /// 
        /// (2) Average:
        /// The measurement results averaged over the given number of measurements are returned.
        /// 
        /// (3) Maxhold:
        /// The maximum result determined over the given number of measurements is returned.
        /// 
        /// (4) Minhold:
        /// The minimum result determined over the given number of measurements is returned.
        /// 
        /// </param>
        /// <param name="Array_Size">
        /// Pass the number of elements in the arrays you specify as the array output parameters.
        /// 
        /// If the data returned contains more elements that you indicate in this parameter, this function copies 'Array Size' elements into the output parameters.
        /// 
        /// </param>
        /// <param name="Number_of_Results">
        /// Returns the number of measurement results.
        /// 
        /// </param>
        /// <param name="Results">
        /// Returns array of measurement results.
        /// 
        /// Note(s):
        /// 
        /// (1) The array should consist of at least the number of elements defined with 'Array Size' parameter.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetAFSpectrumResultValues(int Demodulation_Type, int Result_Type, int Array_Size, out int Number_of_Results, double[] Results)
        {
            int pInvokeResult = PInvoke.GetAFSpectrumResultValues(this._handle, Demodulation_Type, Result_Type, Array_Size, out Number_of_Results, Results);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the result types to be created in parallel by the RF Spectrum measurement with active analog demodulation.
        /// 
        /// Each value besides OFF can only be assigned to one result type at a time.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FS-K7.
        /// 
        /// Mode(s):
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// [SENSe:]ADEMod:SPECtrum[:TYPE] &lt;result type 1&gt;,&lt;result type 2&gt;,&lt;result type 3&gt;
        /// </summary>
        /// <param name="Result_Type_1">
        /// Selects the result types to be created in parallel by the RF Spectrum measurement with active analog demodulation.
        /// 
        /// Valid Range:
        /// RSESCI_RESULT_TYPE_OFF     (0) - Off
        /// RSESCI_RESULT_TYPE_WRITE   (1) - Write
        /// RSESCI_RESULT_TYPE_AVERAGE (2) - Average
        /// RSESCI_RESULT_TYPE_MAXHOLD (3) - Maxhold
        /// RSESCI_RESULT_TYPE_MINHOLD (4) - Minhold
        /// RSESCI_RESULT_TYPE_VIEW    (5) - View
        /// 
        /// Default Value: RSESCI_RESULT_TYPE_OFF (0)
        /// 
        /// Notes:
        /// 
        /// (1) Off:
        /// The result type will not be used.
        /// 
        /// (2) Write:
        /// The current measurement results will be obtained.
        /// 
        /// (3) Average:
        /// The measurement results will be averaged over the given # of measurements.
        /// 
        /// (4) Maxhold:
        /// The maximum result values will be obtained over the given # of measurements.
        /// 
        /// (5) Minhold:
        /// The minimum result values will be obtained over the given # of measurements.
        /// 
        /// (6) View:
        /// The measurement results are frozen and displayed, i.e. they are not calculated for subsequent measurements.
        /// 
        /// </param>
        /// <param name="Result_Type_2">
        /// Selects the result types to be created in parallel by the RF Spectrum measurement with active analog demodulation.
        /// 
        /// Valid Range:
        /// RSESCI_RESULT_TYPE_OFF     (0) - Off
        /// RSESCI_RESULT_TYPE_WRITE   (1) - Write
        /// RSESCI_RESULT_TYPE_AVERAGE (2) - Average
        /// RSESCI_RESULT_TYPE_MAXHOLD (3) - Maxhold
        /// RSESCI_RESULT_TYPE_MINHOLD (4) - Minhold
        /// RSESCI_RESULT_TYPE_VIEW    (5) - View
        /// 
        /// Default Value: RSESCI_RESULT_TYPE_OFF (0)
        /// 
        /// Notes:
        /// 
        /// (1) Off:
        /// The result type will not be used.
        /// 
        /// (2) Write:
        /// The current measurement results will be obtained.
        /// 
        /// (3) Average:
        /// The measurement results will be averaged over the given # of measurements.
        /// 
        /// (4) Maxhold:
        /// The maximum result values will be obtained over the given # of measurements.
        /// 
        /// (5) Minhold:
        /// The minimum result values will be obtained over the given # of measurements.
        /// 
        /// (6) View:
        /// The measurement results are frozen and displayed, i.e. they are not calculated for subsequent measurements.
        /// 
        /// </param>
        /// <param name="Result_Type_3">
        /// Selects the result types to be created in parallel by the RF Spectrum measurement with active analog demodulation.
        /// 
        /// Valid Range:
        /// RSESCI_RESULT_TYPE_OFF     (0) - Off
        /// RSESCI_RESULT_TYPE_WRITE   (1) - Write
        /// RSESCI_RESULT_TYPE_AVERAGE (2) - Average
        /// RSESCI_RESULT_TYPE_MAXHOLD (3) - Maxhold
        /// RSESCI_RESULT_TYPE_MINHOLD (4) - Minhold
        /// RSESCI_RESULT_TYPE_VIEW    (5) - View
        /// 
        /// Default Value: RSESCI_RESULT_TYPE_OFF (0)
        /// 
        /// Notes:
        /// 
        /// (1) Off:
        /// The result type will not be used.
        /// 
        /// (2) Write:
        /// The current measurement results will be obtained.
        /// 
        /// (3) Average:
        /// The measurement results will be averaged over the given # of measurements.
        /// 
        /// (4) Maxhold:
        /// The maximum result values will be obtained over the given # of measurements.
        /// 
        /// (5) Minhold:
        /// The minimum result values will be obtained over the given # of measurements.
        /// 
        /// (6) View:
        /// The measurement results are frozen and displayed, i.e. they are not calculated for subsequent measurements.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetRFSpectrumResultTypes(int Result_Type_1, int Result_Type_2, int Result_Type_3)
        {
            int pInvokeResult = PInvoke.SetRFSpectrumResultTypes(this._handle, Result_Type_1, Result_Type_2, Result_Type_3);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function reads the result data obtained by RF Spectrum measurement depending on the type indicated as a parameter.
        /// 
        /// It is not possible to read out trace data when result type VIEW is selected.
        /// 
        /// The result type indicated must be one of those configured by [SENSe:]ADEMod:SPEC[:TYPE]. Otherwise a query error will be generated.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FS-K7.
        /// 
        /// Mode(s):
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// [SENSe:]ADEMod:SPECtrum:RESult? &lt;result type&gt;
        /// </summary>
        /// <param name="Result_Type">
        /// Selects measurement result type of the RF spectrum measurement.
        /// 
        /// Valid Range:
        /// RSESCI_RESULT_TYPE_WRITE   (1) - Write
        /// RSESCI_RESULT_TYPE_AVERAGE (2) - Average
        /// RSESCI_RESULT_TYPE_MAXHOLD (3) - Maxhold
        /// RSESCI_RESULT_TYPE_MINHOLD (4) - Minhold
        /// 
        /// Default Value: RSESCI_RESULT_TYPE_WRITE (1)
        /// 
        /// Notes:
        /// 
        /// (1) Write:
        /// The current measurement results will be obtained.
        /// 
        /// (2) Average:
        /// The measurement results will be averaged over the given # of measurements.
        /// 
        /// (3) Maxhold:
        /// The maximum result values will be obtained over the given # of measurements.
        /// 
        /// (4) Minhold:
        /// The minimum result values will be obtained over the given # of measurements.
        /// 
        /// </param>
        /// <param name="Array_Size">
        /// Pass the number of elements in the arrays you specify as the array output parameters.
        /// 
        /// If the data returned contains more elements that you indicate in this parameter, this function copies 'Array Size' elements into the output parameters.
        /// 
        /// </param>
        /// <param name="Number_of_Results">
        /// Returns the number of measurement results.
        /// 
        /// </param>
        /// <param name="Results">
        /// Returns array of measurement results.
        /// 
        /// Note(s):
        /// 
        /// (1) The array should consist of at least the number of elements defined with 'Array Size' parameter.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetRFSpectrumResultValues(int Result_Type, int Array_Size, out int Number_of_Results, double[] Results)
        {
            int pInvokeResult = PInvoke.GetRFSpectrumResultValues(this._handle, Result_Type, Array_Size, out Number_of_Results, Results);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the result types to be created in parallel by currently selected demodulation.
        /// 
        /// Each value besides OFF can only be assigned to one result type at a time.
        /// 
        /// Setting all result types to OFF will deactivate the demodulator.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FS-K7.
        /// 
        /// Mode(s):
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// [SENSe:]ADEMod:AM[:TDOMain][:TYPE] &lt;result type 1&gt;,&lt;result type 2&gt;,&lt;result type 3&gt;
        /// [SENSe:]ADEMod:FM[:TDOMain][:TYPE] &lt;result type 1&gt;,&lt;result type 2&gt;,&lt;result type 3&gt;
        /// [SENSe:]ADEMod:PM[:TDOMain][:TYPE] &lt;result type 1&gt;,&lt;result type 2&gt;,&lt;result type 3&gt;
        /// [SENSe:]ADEMod:AM:REL:TYPE &lt;result type 1&gt;,&lt;result type 2&gt;,&lt;result type 3&gt;
        /// </summary>
        /// <param name="Demodulation_Type">
        /// This parameter selects the demodulation type.
        /// 
        /// Valid Range:
        /// RSESCI_DEMOD_TYPE_AM     (0) - AM
        /// RSESCI_DEMOD_TYPE_FM     (1) - FM
        /// RSESCI_DEMOD_TYPE_PM     (2) - PM
        /// RSESCI_DEMOD_TYPE_AM_REL (3) - AM Relative
        /// 
        /// Default Value: RSESCI_DEMOD_TYPE_AM (0)
        /// 
        /// </param>
        /// <param name="Result_Type_1">
        /// Selects the result types to be created in parallel by currently selected demodulation.
        /// 
        /// Valid Range:
        /// RSESCI_RESULT_TYPE_OFF     (0) - Off
        /// RSESCI_RESULT_TYPE_WRITE   (1) - Write
        /// RSESCI_RESULT_TYPE_AVERAGE (2) - Average
        /// RSESCI_RESULT_TYPE_MAXHOLD (3) - Maxhold
        /// RSESCI_RESULT_TYPE_MINHOLD (4) - Minhold
        /// RSESCI_RESULT_TYPE_VIEW    (5) - View
        /// 
        /// Default Value: RSESCI_RESULT_TYPE_OFF (0)
        /// 
        /// Notes:
        /// 
        /// (1) Off:
        /// The result type will not be used.
        /// 
        /// (2) Write:
        /// The current measurement results will be obtained.
        /// 
        /// (3) Average:
        /// The measurement results will be averaged over the given # of measurements.
        /// 
        /// (4) Maxhold:
        /// The maximum result values will be obtained over the given # of measurements.
        /// 
        /// (5) Minhold:
        /// The minimum result values will be obtained over the given # of measurements.
        /// 
        /// (6) View:
        /// The measurement results are frozen and displayed, i.e. they are not calculated for subsequent measurements.
        /// 
        /// </param>
        /// <param name="Result_Type_2">
        /// Selects the result types to be created in parallel by currently selected demodulation.
        /// 
        /// Valid Range:
        /// RSESCI_RESULT_TYPE_OFF     (0) - Off
        /// RSESCI_RESULT_TYPE_WRITE   (1) - Write
        /// RSESCI_RESULT_TYPE_AVERAGE (2) - Average
        /// RSESCI_RESULT_TYPE_MAXHOLD (3) - Maxhold
        /// RSESCI_RESULT_TYPE_MINHOLD (4) - Minhold
        /// RSESCI_RESULT_TYPE_VIEW    (5) - View
        /// 
        /// Default Value: RSESCI_RESULT_TYPE_OFF (0)
        /// 
        /// Notes:
        /// 
        /// (1) Off:
        /// The result type will not be used.
        /// 
        /// (2) Write:
        /// The current measurement results will be obtained.
        /// 
        /// (3) Average:
        /// The measurement results will be averaged over the given # of measurements.
        /// 
        /// (4) Maxhold:
        /// The maximum result values will be obtained over the given # of measurements.
        /// 
        /// (5) Minhold:
        /// The minimum result values will be obtained over the given # of measurements.
        /// 
        /// (6) View:
        /// The measurement results are frozen and displayed, i.e. they are not calculated for subsequent measurements.
        /// 
        /// </param>
        /// <param name="Result_Type_3">
        /// Selects the result types to be created in parallel by currently selected demodulation.
        /// 
        /// Valid Range:
        /// RSESCI_RESULT_TYPE_OFF     (0) - Off
        /// RSESCI_RESULT_TYPE_WRITE   (1) - Write
        /// RSESCI_RESULT_TYPE_AVERAGE (2) - Average
        /// RSESCI_RESULT_TYPE_MAXHOLD (3) - Maxhold
        /// RSESCI_RESULT_TYPE_MINHOLD (4) - Minhold
        /// RSESCI_RESULT_TYPE_VIEW    (5) - View
        /// 
        /// Default Value: RSESCI_RESULT_TYPE_OFF (0)
        /// 
        /// Notes:
        /// 
        /// (1) Off:
        /// The result type will not be used.
        /// 
        /// (2) Write:
        /// The current measurement results will be obtained.
        /// 
        /// (3) Average:
        /// The measurement results will be averaged over the given # of measurements.
        /// 
        /// (4) Maxhold:
        /// The maximum result values will be obtained over the given # of measurements.
        /// 
        /// (5) Minhold:
        /// The minimum result values will be obtained over the given # of measurements.
        /// 
        /// (6) View:
        /// The measurement results are frozen and displayed, i.e. they are not calculated for subsequent measurements.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetTimeDomainResultTypes(int Demodulation_Type, int Result_Type_1, int Result_Type_2, int Result_Type_3)
        {
            int pInvokeResult = PInvoke.SetTimeDomainResultTypes(this._handle, Demodulation_Type, Result_Type_1, Result_Type_2, Result_Type_3);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function reads the result data obtained by demodulation depending on the type indicated as a parameter.
        /// 
        /// The query of traces in the VIEW state is not possible.
        /// 
        /// The result type indicated must be one of those configured by [SENSe:]ADEMod:AM[|FM|PM][:TDOMain][:TYPE] or ADEM:AM:REL:TYPE. Otherwise a query error will be generated.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FS-K7.
        /// 
        /// Mode(s):
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// [SENSe:]ADEMod:AM[:TDOMain]:RESult? &lt;result type&gt;
        /// [SENSe:]ADEMod:FM[:TDOMain]:RESult? &lt;result type&gt;
        /// [SENSe:]ADEMod:PM[:TDOMain]:RESult? &lt;result type&gt;
        /// [SENSe:]ADEMod:AM:REL:RESult? &lt;result type&gt;
        /// </summary>
        /// <param name="Demodulation_Type">
        /// This parameter selects the demodulation type.
        /// 
        /// Valid Range:
        /// RSESCI_DEMOD_TYPE_AM     (0) - AM
        /// RSESCI_DEMOD_TYPE_FM     (1) - FM
        /// RSESCI_DEMOD_TYPE_PM     (2) - PM
        /// RSESCI_DEMOD_TYPE_AM_REL (3) - AM Relative
        /// 
        /// Default Value: RSESCI_DEMOD_TYPE_AM (0)
        /// 
        /// </param>
        /// <param name="Result_Type">
        /// Selects measurement result type..
        /// 
        /// Valid Range:
        /// RSESCI_RESULT_TYPE_WRITE   (1) - Write
        /// RSESCI_RESULT_TYPE_AVERAGE (2) - Average
        /// RSESCI_RESULT_TYPE_MAXHOLD (3) - Maxhold
        /// RSESCI_RESULT_TYPE_MINHOLD (4) - Minhold
        /// 
        /// Default Value: RSESCI_RESULT_TYPE_WRITE (1)
        /// 
        /// Notes:
        /// 
        /// (1) Write:
        /// The current measurement results are obtained.
        /// 
        /// (2) Average:
        /// The measurement results averaged over the given number of measurements are obtained.
        /// 
        /// (3) Maxhold:
        /// The maximum result is determined over the given number of measurements.
        /// 
        /// (4) Minhold:
        /// The minimum result is determined over the given number of measurements.
        /// 
        /// </param>
        /// <param name="Array_Size">
        /// Pass the number of elements in the arrays you specify as the array output parameters.
        /// 
        /// If the data returned contains more elements that you indicate in this parameter, this function copies 'Array Size' elements into the output parameters.
        /// 
        /// </param>
        /// <param name="Number_of_Results">
        /// Returns the number of measurement results.
        /// 
        /// </param>
        /// <param name="Results">
        /// Returns array of measurement results.
        /// 
        /// Note(s):
        /// 
        /// (1) The array should consist of at least the number of elements defined with 'Array Size' parameter.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetTimeDomainResultValues(int Demodulation_Type, int Result_Type, int Array_Size, out int Number_of_Results, double[] Results)
        {
            int pInvokeResult = PInvoke.GetTimeDomainResultValues(this._handle, Demodulation_Type, Result_Type, Array_Size, out Number_of_Results, Results);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function calculates the FM offset of the currently available measurement data set. If averaging has been activated before acquiring the data set (using [SENSe:]ADEMod:FM[:TYPE]), the averaged FM offset over several measurements can be obtained as well by setting &lt;result type&gt; = AVERage.
        /// 
        /// The offset thus determined differs from that calculated in the command CALC:MARK:FUNC:ADEM:FERR?, since using that command, for determination of the frequency deviation, the modulation is removed by means of lowpass filtering, producing results that are different from those obtained by averaging.
        /// 
        /// If no average measurement was active during the last measurement sequence only the command [SENSe:]ADEMod:FM:OFFSet? IMMediate will return a valid result (data to calculate the offset are taken from the last measured data set).
        /// 
        /// [SENSe:]ADEMod:FM:OFFSet? AVERage will cause a query error in this case.
        /// 
        /// Note:
        /// 
        /// This function is only valid in conjunction with the option R&amp;S FS-K7.
        /// 
        /// Mode(s):
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// [SENSe:]ADEMod:FM:OFFSet? IMMediate | AVERage
        /// </summary>
        /// <param name="Measurement">
        /// Specifies measurement type.
        /// 
        /// Valid Range:
        /// RSESCI_MEASUREMENT_IMM     (0) - Immediate
        /// RSESCI_MEASUREMENT_AVERAGE (1) - Average
        /// 
        /// Default Value: RSESCI_MEASUREMENT_IMM (0)
        /// 
        /// Notes:
        /// 
        /// (1) Immediate:
        /// The current measurement results will be used for calculating the FM offset.
        /// 
        /// (2) Average:
        /// The measurement results that were averaged over the given # of measurements will be used for calculating the FM offset.
        /// 
        /// </param>
        /// <param name="Result">
        /// This control returns calculated FM offset of the currently available measurement data set.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetFMOffsetValue(int Measurement, out double Result)
        {
            int pInvokeResult = PInvoke.GetFMOffsetValue(this._handle, Measurement, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches on or off the average calculation for the selected trace in the selected measurement window.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]AVERage[:STATe&lt;1...3&gt;] ON | OFF
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Trace">
        /// This control sets the trace.
        /// 
        /// Valid Range: 1 to 3
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Data_Averaging">
        /// This control switches on or off the average calculation for the selected trace in the selected measurement window.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetDataAveraging(int Window, int Trace, bool Data_Averaging)
        {
            int pInvokeResult = PInvoke.SetDataAveraging(this._handle, Window, Trace, System.Convert.ToUInt16(Data_Averaging));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns if the average calculation for the selected trace in the selected measurement window is activate dor not.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]AVERage[:STATe&lt;1...3&gt;]?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Trace">
        /// This control sets the trace.
        /// 
        /// Valid Range: 1 to 3
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Data_Averaging">
        /// Returns if the average calculation for the selected trace in the selected measurement window is activate dor not.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetDataAveraging(int Window, int Trace, out bool Data_Averaging)
        {
            ushort Data_AveragingAsUShort;
            int pInvokeResult = PInvoke.GetDataAveraging(this._handle, Window, Trace, out Data_AveragingAsUShort);
            Data_Averaging = System.Convert.ToBoolean(Data_AveragingAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// In receiver mode, this function defines the number of scan started with single scan. In analyzer mode, it defines the number of measurements which contribute to the average value. It should be noted that continuous averaging will be performed after the indicated number has been reached in continuous sweep mode.
        /// 
        /// In single sweep or scan mode, the scan/sweep is stopped as soon as the indicated number of measurements (scans/sweeps) is reached. Synchronization to the end of the indicated number of measurements is only possible in single scan/sweep mode.
        /// 
        /// The command [SENSe&lt;1|2&gt;:]AVERage:COUNt is the same as command [SENSe&lt;1|2&gt;:]SWEep:COUNt. In both cases, the number of measurements is defined whether the average calculation is active or not.
        /// 
        /// The number of measurements is valid for all traces in the indicated measurement window.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]AVERage:COUNt
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Average_Count">
        /// In receiver mode, this parameter defines the number of scan started with single scan. In analyzer mode, it defines the number of measurements which contribute to the average value.
        /// 
        /// Valid Range: 0 to 32767
        /// 
        /// Default Value: 0
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetAverageCount(int Window, int Average_Count)
        {
            int pInvokeResult = PInvoke.SetAverageCount(this._handle, Window, Average_Count);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// In receiver mode, this function returns the number of scan started with single scan. In analyzer mode, it returns the number of measurements which contribute to the average value.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]AVERage:COUNt?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Average_Count">
        /// In receiver mode, this parameter returns the number of scan started with single scan. In analyzer mode, it returns the number of measurements which contribute to the average value.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetAverageCount(int Window, out int Average_Count)
        {
            int pInvokeResult = PInvoke.GetAverageCount(this._handle, Window, out Average_Count);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the type of average function. If VIDeo is selected, the logarithmic power is averaged and, if LINear is selected, the power values are averaged before they are converted to logarithmic values.
        /// 
        /// The type of average calculation is equally set for all traces in one measurement window.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]AVERage:TYPE VIDeo | LINear
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Average_Type">
        /// This control selects the type of average function.
        /// 
        /// Valid Values:
        /// RSESCI_AVERAGE_LINEAR (0) - Linear
        /// RSESCI_AVERAGE_VIDEO  (1) - Video
        /// 
        /// Default Value: RSESCI_AVERAGE_LINEAR (0)
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetAverageType(int Window, int Average_Type)
        {
            int pInvokeResult = PInvoke.SetAverageType(this._handle, Window, Average_Type);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the type of average function.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]AVERage:TYPE?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Average_Type">
        /// This control returns the type of average function.
        /// 
        /// Valid Values:
        /// RSESCI_AVERAGE_LINEAR (0) - Linear
        /// RSESCI_AVERAGE_VIDEO  (1) - Video
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetAverageType(int Window, out int Average_Type)
        {
            int pInvokeResult = PInvoke.GetAverageType(this._handle, Window, out Average_Type);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function assigns a name to a limit line numbered 1 to 8. If it does not exist already, a limit line with this name is created.
        /// The command is independent of the measurement window.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:NAME
        /// </summary>
        /// <param name="Limit">
        /// This control selects the limit line.
        /// 
        /// Valid Range: 1 to 8
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Name">
        /// This control defines the limit line name. The name of the limit line may contain a maximum of 8 characters.
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: "LIMIT1"
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetLimitLineName(int Limit, string Name)
        {
            int pInvokeResult = PInvoke.SetLimitLineName(this._handle, Limit, Name);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the limit line name.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:NAME?
        /// </summary>
        /// <param name="Limit">
        /// This control selects the limit line.
        /// 
        /// Valid Range: 1 to 8
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Name">
        /// This control returns the limit line name.
        /// 
        /// Note(s):
        /// 
        /// (1) The array should consist of at least 9 elements.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetLimitLineName(int Limit, System.Text.StringBuilder Name)
        {
            int pInvokeResult = PInvoke.GetLimitLineName(this._handle, Limit, Name);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches on or off the indicated limit line in the selected measurement window.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:LOWer:STATe ON | OFF
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:UPPer:STATe ON | OFF
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Limit">
        /// This control selects the limit line.
        /// 
        /// Valid Range: 1 to 8
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Limit_Type">
        /// This control selects the limit type.
        /// 
        /// Valid Values:
        /// RSESCI_LIMIT_LOWER (0) - Lower
        /// RSESCI_LIMIT_UPPER (1) - Upper
        /// 
        /// Default Value: RSESCI_LIMIT_LOWER (0)
        /// </param>
        /// <param name="State">
        /// This control switches on or off the indicated limit line in the selected measurement window.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetLimitLineVisible(int Window, int Limit, int Limit_Type, bool State)
        {
            int pInvokeResult = PInvoke.SetLimitLineVisible(this._handle, Window, Limit, Limit_Type, System.Convert.ToUInt16(State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function the state of the indicated limit line in the selected measurement window.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:LOWer:STATe?
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:UPPer:STATe?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Limit">
        /// This control selects the limit line.
        /// 
        /// Valid Range: 1 to 8
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Limit_Type">
        /// This control selects the limit type.
        /// 
        /// Valid Values:
        /// RSESCI_LIMIT_LOWER (0) - Lower
        /// RSESCI_LIMIT_UPPER (1) - Upper
        /// 
        /// Default Value: RSESCI_LIMIT_LOWER (0)
        /// </param>
        /// <param name="State">
        /// This control returns the state of the indicated limit line in the selected measurement window.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetLimitLineVisible(int Window, int Limit, int Limit_Type, out bool State)
        {
            ushort StateAsUShort;
            int pInvokeResult = PInvoke.GetLimitLineVisible(this._handle, Window, Limit, Limit_Type, out StateAsUShort);
            State = System.Convert.ToBoolean(StateAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function assigns a limit line to a trace in the indicated measurement window.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:TRACe
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Limit">
        /// This control selects the limit line.
        /// 
        /// Valid Range: 1 to 8
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Trace">
        /// This control selects the trace.
        /// 
        /// Valid Range: 1 to 3
        /// 
        /// Default Value: 1
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetLimitLineTrace(int Window, int Limit, int Trace)
        {
            int pInvokeResult = PInvoke.SetLimitLineTrace(this._handle, Window, Limit, Trace);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the trace associated with indicated limit line.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:TRACe?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Limit">
        /// This control selects the limit line.
        /// 
        /// Valid Range: 1 to 8
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Trace">
        /// This control returns the trace associated with indicated limit line.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetLimitLineTrace(int Window, int Limit, out int Trace)
        {
            int pInvokeResult = PInvoke.GetLimitLineTrace(this._handle, Window, Limit, out Trace);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function copies one limit line onto another one. It is independent of the measurement window.
        /// The name of the limit line may consist of max 8 characters.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:COPY 1 to 8
        /// </summary>
        /// <param name="Source_Limit">
        /// This control selects the source limit line.
        /// 
        /// Valid Range: 1 to 8
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Destination_Limit">
        /// This control selects the destination limit line.
        /// 
        /// Valid Range: 1 to 8
        /// 
        /// Default Value: 1
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int CopyLimitLine(int Source_Limit, int Destination_Limit)
        {
            int pInvokeResult = PInvoke.CopyLimitLine(this._handle, Source_Limit, Destination_Limit);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function deletes the selected limit line. The command is independent of the measurement window.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:DELete
        /// </summary>
        /// <param name="Limit">
        /// This control selects the limit line.
        /// 
        /// Valid Range: 1 to 8
        /// 
        /// Default Value: 1
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int DeleteLimitLine(int Limit)
        {
            int pInvokeResult = PInvoke.DeleteLimitLine(this._handle, Limit);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines an offset for the X axis value of the selected relative limit line in the frequency or time domain.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:CONTrol:OFFSet
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Limit">
        /// This control selects the limit line.
        /// 
        /// Valid Range: 1 to 8
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Offset">
        /// This control defines an offset for the X axis value of the selected relative limit line in the frequency or time domain.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: 0.0
        /// 
        /// Note(s):
        /// 
        /// (1) The unit of values depends on the frequency or time domain of the X axis, i.e. it is HZ with CALC:LIM:CONT:DOM FREQ und S bei CALC:LIM:CONT:DOM TIME.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetLimitLineXOffset(int Window, int Limit, double Offset)
        {
            int pInvokeResult = PInvoke.SetLimitLineXOffset(this._handle, Window, Limit, Offset);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries an offset for the X axis value of the selected relative limit line in the frequency or time domain.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:CONTrol:OFFSet?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Limit">
        /// This control selects the limit line.
        /// 
        /// Valid Range: 1 to 8
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Offset">
        /// This control returns an offset for the X axis value of the selected relative limit line in the frequency or time domain.
        /// 
        /// Note(s):
        /// 
        /// (1) The unit of values depends on the frequency or time domain of the X axis, i.e. it is HZ with CALC:LIM:CONT:DOM FREQ und S bei CALC:LIM:CONT:DOM TIME.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetLimitLineXOffset(int Window, int Limit, out double Offset)
        {
            int pInvokeResult = PInvoke.GetLimitLineXOffset(this._handle, Window, Limit, out Offset);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines an offset for the Y axis of the selected relative lower limit line. In contrast to CALC:LIM:LOW:SHIFt, the line is not shifted by modifying the individual Y values but by means of an additive offset.
        /// The offset is independent of the measurement window.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:LOWer:OFFSet
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:UPPer:OFFSet
        /// </summary>
        /// <param name="Limit">
        /// This control selects the limit line.
        /// 
        /// Valid Range: 1 to 8
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Limit_Type">
        /// This control selects the limit type.
        /// 
        /// Valid Values:
        /// RSESCI_LIMIT_LOWER (0) - Lower
        /// RSESCI_LIMIT_UPPER (1) - Upper
        /// 
        /// Default Value: RSESCI_LIMIT_LOWER (0)
        /// </param>
        /// <param name="Offset">
        /// This control defines an offset for the Y axis of the selected relative lower limit line.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: 0.0
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetLimitLineYOffset(int Limit, int Limit_Type, double Offset)
        {
            int pInvokeResult = PInvoke.SetLimitLineYOffset(this._handle, Limit, Limit_Type, Offset);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries an offset for the Y axis of the selected relative lower limit line.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:LOWer:OFFSet?
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:UPPer:OFFSet?
        /// </summary>
        /// <param name="Limit">
        /// This control selects the limit line.
        /// 
        /// Valid Range: 1 to 8
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Limit_Type">
        /// This control selects the limit type.
        /// 
        /// Valid Values:
        /// RSESCI_LIMIT_LOWER (0) - Lower
        /// RSESCI_LIMIT_UPPER (1) - Upper
        /// 
        /// Default Value: RSESCI_LIMIT_LOWER (0)
        /// </param>
        /// <param name="Offset">
        /// This control returns an offset for the Y axis of the selected relative lower limit line.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetLimitLineYOffset(int Limit, int Limit_Type, out double Offset)
        {
            int pInvokeResult = PInvoke.GetLimitLineYOffset(this._handle, Limit, Limit_Type, out Offset);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the names of all active limit lines.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:ACTive?
        /// </summary>
        /// <param name="Array_Size">
        /// This control defines the size of array in argument 'Active Limit Lines'.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: 1000
        /// </param>
        /// <param name="Active_Limit_Lines">
        /// This control returns the state of the indicated limit line in the selected measurement window.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetActiveLimitLines(int Array_Size, System.Text.StringBuilder Active_Limit_Lines)
        {
            int pInvokeResult = PInvoke.GetActiveLimitLines(this._handle, Array_Size, Active_Limit_Lines);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches on or off the limit check for the  selected limit line in the selected measurement window. The result of the limit check can be queried with rsesci_GetLimitLineCheckResult().
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:STATe ON | OFF
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Limit">
        /// This control selects the limit line.
        /// 
        /// Valid Range: 1 to 8
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Limit_Check">
        /// This control switches on or off the limit check for the selected limit line in the selected measurement window.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetLimitLineCheck(int Window, int Limit, bool Limit_Check)
        {
            int pInvokeResult = PInvoke.SetLimitLineCheck(this._handle, Window, Limit, System.Convert.ToUInt16(Limit_Check));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the state of the limit check for the selected limit line in the selected measurement window.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:STATe?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Limit">
        /// This control selects the limit line.
        /// 
        /// Valid Range: 1 to 8
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Limit_Check">
        /// This control returns the state of the limit check for the selected limit line in the selected measurement window.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetLimitLineCheck(int Window, int Limit, out bool Limit_Check)
        {
            ushort Limit_CheckAsUShort;
            int pInvokeResult = PInvoke.GetLimitLineCheck(this._handle, Window, Limit, out Limit_CheckAsUShort);
            Limit_Check = System.Convert.ToBoolean(Limit_CheckAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the result of the limit check of the limit line indicated in the selected measurement window. It should be noted that a complete sweep must have been performed for obtaining a valid result. A synchronization with *OPC, *OPC? or *WAI should therefore be provided.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:FAIL?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Limit">
        /// This control selects the limit line.
        /// 
        /// Valid Range: 1 to 8
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Result">
        /// This control returns the result of the limit check of the limit line indicated in the selected measurement window.
        /// 
        /// Valid Values:
        /// 0 - PASS
        /// 1 - FAIL
        /// 2 - MARGIN
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetLimitLineCheckResult(int Window, int Limit, out int Result)
        {
            int pInvokeResult = PInvoke.GetLimitLineCheckResult(this._handle, Window, Limit, out Result);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function deletes the result of the current limit check for all limit lines in the selected measurement window.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:CLEar[:IMMediate]
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int ClearLimitLineCheckResult(int Window)
        {
            int pInvokeResult = PInvoke.ClearLimitLineCheckResult(this._handle, Window);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the frequency or time domain for the x axis values.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:CONTrol:DOMain FREQuency | TIME
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Limit">
        /// This control selects the limit line.
        /// 
        /// Valid Range: 1 to 8
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Domain">
        /// This control defines the frequency or time domain for the x axis values.
        /// 
        /// Valid Values:
        /// RSESCI_DOMAIN_FREQ (0) - Frequency
        /// RSESCI_DOMAIN_TIME (1) - Time
        /// 
        /// Default Value: RSESCI_DOMAIN_FREQ (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetLimitLineDomain(int Window, int Limit, int Domain)
        {
            int pInvokeResult = PInvoke.SetLimitLineDomain(this._handle, Window, Limit, Domain);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries selected domain for the x axis values.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:CONTrol:DOMain?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Limit">
        /// This control selects the limit line.
        /// 
        /// Valid Range: 1 to 8
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Domain">
        /// This control returns selected domain for the x axis values.
        /// 
        /// Valid Values:
        /// RSESCI_DOMAIN_FREQ (0) - Frequency
        /// RSESCI_DOMAIN_TIME (1) - Time
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetLimitLineDomain(int Window, int Limit, out int Domain)
        {
            int pInvokeResult = PInvoke.GetLimitLineDomain(this._handle, Window, Limit, out Domain);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects linear or logarithmic interpolation for the calculation of limit lines from frequency points.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:LOWer:SPACing LINear | LOGarithmic
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:UPPer:SPACing LINear | LOGarithmic
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:CONTrol:SPACing LINear | LOGarithmic
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Limit">
        /// This control selects the limit line.
        /// 
        /// Valid Range: 1 to 8
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Interpolation_Axis">
        /// This control selects the axis for which the interpolation should be set.
        /// 
        /// Valid Values:
        /// RSESCI_LIMIT_LOWER (0) - Lower
        /// RSESCI_LIMIT_UPPER (1) - Upper
        /// RSESCI_LIMIT_X     (2) - X Axis
        /// 
        /// Default Value: RSESCI_LIMIT_LOWER (0)
        /// </param>
        /// <param name="Interpolation">
        /// This control selects linear or logarithmic interpolation for the calculation of limit lines from frequency points.
        /// 
        /// Valid Values:
        /// RSESCI_LIN (0) - Linear
        /// RSESCI_LOG (1) - Logarithmic
        /// 
        /// Default Value: RSESCI_LIN (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetLimitLineInterpolation(int Window, int Limit, int Interpolation_Axis, int Interpolation)
        {
            int pInvokeResult = PInvoke.SetLimitLineInterpolation(this._handle, Window, Limit, Interpolation_Axis, Interpolation);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries selected interpolation for the calculation of limit lines from frequency points.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:LOWer:SPACing?
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:UPPer:SPACing?
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:CONTrol:SPACing?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Limit">
        /// This control selects the limit line.
        /// 
        /// Valid Range: 1 to 8
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Interpolation_Axis">
        /// This control selects the axis for which the interpolation should be set.
        /// 
        /// Valid Values:
        /// RSESCI_LIMIT_LOWER (0) - Lower
        /// RSESCI_LIMIT_UPPER (1) - Upper
        /// RSESCI_LIMIT_X     (2) - X Axis
        /// 
        /// Default Value: RSESCI_LIMIT_LOWER (0)
        /// </param>
        /// <param name="Interpolation">
        /// This control returns selected interpolation for the calculation of limit lines from frequency points.
        /// 
        /// Valid Values:
        /// RSESCI_LIN (0) - Linear
        /// RSESCI_LOG (1) - Logarithmic
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetLimitLineInterpolation(int Window, int Limit, int Interpolation_Axis, out int Interpolation)
        {
            int pInvokeResult = PInvoke.GetLimitLineInterpolation(this._handle, Window, Limit, Interpolation_Axis, out Interpolation);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the relative or absolute scaling for selected axis of the selected limit line. The definition is independent of the measurement window.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:LOWer:MODE RELative | ABSolute
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:UPPer:MODE RELative | ABSolute
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:CONTrol:MODE RELative | ABSolute
        /// </summary>
        /// <param name="Limit">
        /// This control selects the limit line.
        /// 
        /// Valid Range: 1 to 8
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Interpolation_Axis">
        /// This control selects the axis for which the interpolation should be set.
        /// 
        /// Valid Values:
        /// RSESCI_LIMIT_LOWER (0) - Lower
        /// RSESCI_LIMIT_UPPER (1) - Upper
        /// RSESCI_LIMIT_X     (2) - X Axis
        /// 
        /// Default Value: RSESCI_LIMIT_LOWER (0)
        /// </param>
        /// <param name="Limit_Line_Scaling">
        /// This control sets the scale type of the limit line axis in the selected measurement window.
        /// 
        /// Valid Values:
        /// RSESCI_ABS (0) - Absolute
        /// RSESCI_REL (1) - Relative
        /// 
        /// Default Value: RSESCI_ABS (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetLimitLineScaling(int Limit, int Interpolation_Axis, int Limit_Line_Scaling)
        {
            int pInvokeResult = PInvoke.SetLimitLineScaling(this._handle, Limit, Interpolation_Axis, Limit_Line_Scaling);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries selected scaling for selected axis of the selected limit line. The function is independent of the measurement window.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:LOWer:MODE?
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:UPPer:MODE?
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:CONTrol:MODE?
        /// </summary>
        /// <param name="Limit">
        /// This control selects the limit line.
        /// 
        /// Valid Range: 1 to 8
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Interpolation_Axis">
        /// This control selects the axis for which the interpolation should be set.
        /// 
        /// Valid Values:
        /// RSESCI_LIMIT_LOWER (0) - Lower
        /// RSESCI_LIMIT_UPPER (1) - Upper
        /// RSESCI_LIMIT_X     (2) - X Axis
        /// 
        /// Default Value: RSESCI_LIMIT_LOWER (0)
        /// </param>
        /// <param name="Limit_Line_Scaling">
        /// This control returns the scale type of the limit line axis in the selected measurement window.
        /// 
        /// Valid Values:
        /// RSESCI_ABS (0) - Absolute
        /// RSESCI_REL (1) - Relative
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetLimitLineScaling(int Limit, int Interpolation_Axis, out int Limit_Line_Scaling)
        {
            int pInvokeResult = PInvoke.GetLimitLineScaling(this._handle, Limit, Interpolation_Axis, out Limit_Line_Scaling);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the unit of the selected limit line. The definition is valid independently of the measurement window.
        /// 
        /// Note(s):
        /// 
        /// (1) These units: DBPT, DEG, RAD, S, HZ, PCT, DBUA_M, DBUV_MHZ, DBMV_MHZ, DBUA_MHZ, DBPW_MHZ, DBPT_MHZ, DBUV_MMHZ, DBUA_MMHZ, UNITless are available only with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:UNIT DBM | DBPW | WATT | DBUV | DBMV | VOLT | DBUA | AMPere | DB | DBUV_M | DBMV_M | DBPT | DEG | RAD | S | HZ | PCT | DBUA_M | DBUV_MHZ | DBMV_MHZ | DBUA_MHZ | DBPW_MHZ | DBPT_MHZ | DBUV_MMHZ | DBUA_MMHZ | UNITless
        /// </summary>
        /// <param name="Limit">
        /// This control selects the limit line.
        /// 
        /// Valid Range: 1 to 8
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Level_Axis_Unit">
        /// This control selects the unit of the selected limit line.
        /// 
        /// Valid Values:
        /// RSESCI_UNIT_DBM       (0)  - dBm
        /// RSESCI_UNIT_VOLT      (1)  - Volt
        /// RSESCI_UNIT_AMP       (2)  - Ampere
        /// RSESCI_UNIT_WATT      (3)  - Watt
        /// RSESCI_UNIT_DBPW      (4)  - dBpW
        /// RSESCI_UNIT_DBUV      (5)  - dBuV
        /// RSESCI_UNIT_DBMV      (6)  - dBmV
        /// RSESCI_UNIT_DBUA      (7)  - dBuA
        /// RSESCI_UNIT_DBPT      (8)  - dBpT
        /// RSESCI_UNIT_DBUV_M    (9)  - dBuV/m
        /// RSESCI_UNIT_DBUA_M    (10) - dBuA/m
        /// RSESCI_UNIT_DB        (11) - dB
        /// RSESCI_UNIT_DBMV_M    (12) - dBmV/m
        /// RSESCI_UNIT_DEG       (13) - deg
        /// RSESCI_UNIT_RAD       (14) - rad
        /// RSESCI_UNIT_S         (15) - s
        /// RSESCI_UNIT_HZ        (16) - Hz
        /// RSESCI_UNIT_PCT       (17) - Percentage
        /// RSESCI_UNIT_DBUV_MHZ  (18) - dBuV/MHz
        /// RSESCI_UNIT_DBMV_MHZ  (19) - dBmV/MHz
        /// RSESCI_UNIT_DBUA_MHZ  (20) - dBuA/MHz
        /// RSESCI_UNIT_DBPW_MHZ  (21) - dBpW/MHz
        /// RSESCI_UNIT_DBPT_MHZ  (22) - dBpT/MHz
        /// RSESCI_UNIT_DBUV_MMHZ (23) - dBuV/mMhz
        /// RSESCI_UNIT_DBUA_MMHZ (24) - dBuA/mMhz
        /// RSESCI_UNIT_UNITLESS  (25) - Unitless
        /// 
        /// Default Value: RSESCI_UNIT_DBM (0)
        /// 
        /// Note(s):
        /// 
        /// (1) Upon selection of the unit DB the limit line is automatically switched to the relative mode. For units different from DB the limit line is automatically switched to absolute mode.
        /// 
        /// (2) In receiver mode the units DBM, DBUV, DBUA, DBPW, DBUV_M, and DBUA_M are available.
        /// 
        /// (3) In analyzer mode all units are available.
        /// 
        /// (4) These units: DBPT, DEG, RAD, S, HZ, PCT, DBUA_M, DBUV_MHZ, DBMV_MHZ, DBUA_MHZ, DBPW_MHZ, DBPT_MHZ, DBUV_MMHZ, DBUA_MMHZ, UNITless are available only with ESU instrument.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetLimitLineUnit(int Limit, int Level_Axis_Unit)
        {
            int pInvokeResult = PInvoke.SetLimitLineUnit(this._handle, Limit, Level_Axis_Unit);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the unit of the selected limit line.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:UNIT?
        /// </summary>
        /// <param name="Limit">
        /// This control selects the limit line.
        /// 
        /// Valid Range: 1 to 8
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Level_Axis_Unit">
        /// This control returns the unit of the selected limit line.
        /// 
        /// Valid Values:
        /// RSESCI_UNIT_DBM       (0)  - dBm
        /// RSESCI_UNIT_VOLT      (1)  - Volt
        /// RSESCI_UNIT_AMP       (2)  - Ampere
        /// RSESCI_UNIT_WATT      (3)  - Watt
        /// RSESCI_UNIT_DBPW      (4)  - dBpW
        /// RSESCI_UNIT_DBUV      (5)  - dBuV
        /// RSESCI_UNIT_DBMV      (6)  - dBmV
        /// RSESCI_UNIT_DBUA      (7)  - dBuA
        /// RSESCI_UNIT_DBPT      (8)  - dBpT
        /// RSESCI_UNIT_DBUV_M    (9)  - dBuV/m
        /// RSESCI_UNIT_DBUA_M    (10) - dBuA/m
        /// RSESCI_UNIT_DB        (11) - dB
        /// RSESCI_UNIT_DBMV_M    (12) - dBmV/m
        /// RSESCI_UNIT_DEG       (13) - deg
        /// RSESCI_UNIT_RAD       (14) - rad
        /// RSESCI_UNIT_S         (15) - s
        /// RSESCI_UNIT_HZ        (16) - Hz
        /// RSESCI_UNIT_PCT       (17) - Percentage
        /// RSESCI_UNIT_DBUV_MHZ  (18) - dBuV/MHz
        /// RSESCI_UNIT_DBMV_MHZ  (19) - dBmV/MHz
        /// RSESCI_UNIT_DBUA_MHZ  (20) - dBuA/MHz
        /// RSESCI_UNIT_DBPW_MHZ  (21) - dBpW/MHz
        /// RSESCI_UNIT_DBPT_MHZ  (22) - dBpT/MHz
        /// RSESCI_UNIT_DBUV_MMHZ (23) - dBuV/mMhz
        /// RSESCI_UNIT_DBUA_MMHZ (24) - dBuA/mMhz
        /// RSESCI_UNIT_UNITLESS  (25) - Unitless
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetLimitLineUnit(int Limit, out int Level_Axis_Unit)
        {
            int pInvokeResult = PInvoke.GetLimitLineUnit(this._handle, Limit, out Level_Axis_Unit);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines a margin to a lower limit line, at which out-of-limit values are signalled (if the limit check is active), but not handled as a violation of the limit value.
        /// The margin is independent of the measurement window.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:LOWer:MARGin
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:UPPer:MARGin
        /// </summary>
        /// <param name="Limit">
        /// This control selects the limit line.
        /// 
        /// Valid Range: 1 to 8
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Limit_Line_Type">
        /// This control selects the limit line type.
        /// 
        /// Valid Values:
        /// RSESCI_LIMIT_LOWER (0) - Lower
        /// RSESCI_LIMIT_UPPER (1) - Upper
        /// 
        /// Default Value: RSESCI_LIMIT_LOWER (0)
        /// </param>
        /// <param name="Margin">
        /// This control defines a margin to a lower limit line, at which out-of-limit values are signalled (if the limit check is active), but not handled as a violation of the limit value.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: 0.0
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetLimitLineMargin(int Limit, int Limit_Line_Type, double Margin)
        {
            int pInvokeResult = PInvoke.SetLimitLineMargin(this._handle, Limit, Limit_Line_Type, Margin);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries a margin to a lower limit line
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:LOWer:MARGin?
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:UPPer:MARGin?
        /// </summary>
        /// <param name="Limit">
        /// This control selects the limit line.
        /// 
        /// Valid Range: 1 to 8
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Limit_Line_Type">
        /// This control selects the limit line type.
        /// 
        /// Valid Values:
        /// RSESCI_LIMIT_LOWER (0) - Lower
        /// RSESCI_LIMIT_UPPER (1) - Upper
        /// 
        /// Default Value: RSESCI_LIMIT_LOWER (0)
        /// </param>
        /// <param name="Margin">
        /// This control returns a margin to a lower limit line.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetLimitLineMargin(int Limit, int Limit_Line_Type, out double Margin)
        {
            int pInvokeResult = PInvoke.GetLimitLineMargin(this._handle, Limit, Limit_Line_Type, out Margin);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines an absolute threshold value for limit lines with relative Y axis scaling independently of the measurement window. The absolute threshold value is used in the limit check as soon as it exceeds the relative limit value. The unit must correspond to the unit selected with CALC:LIM:UNIT (except dB which is not allowed). If no unit is indicated, the unit defined with CALC:LIM:UNIT is automatically used (exception: dBm instead of dB).
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:LOWer:THReshold
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:UPPer:THReshold
        /// </summary>
        /// <param name="Limit">
        /// This control selects the limit line.
        /// 
        /// Valid Range: 1 to 8
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Limit_Line_Type">
        /// This control selects the limit line type.
        /// 
        /// Valid Values:
        /// RSESCI_LIMIT_LOWER (0) - Lower
        /// RSESCI_LIMIT_UPPER (1) - Upper
        /// 
        /// Default Value: RSESCI_LIMIT_LOWER (0)
        /// </param>
        /// <param name="Threshold">
        /// This control defines an absolute threshold value for limit lines with relative Y axis scaling.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: -200.0 dBm
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetLimitLineThreshold(int Limit, int Limit_Line_Type, double Threshold)
        {
            int pInvokeResult = PInvoke.SetLimitLineThreshold(this._handle, Limit, Limit_Line_Type, Threshold);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries an absolute threshold value for limit lines with relative Y axis scaling.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:LOWer:THReshold?
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:UPPer:THReshold?
        /// </summary>
        /// <param name="Limit">
        /// This control selects the limit line.
        /// 
        /// Valid Range: 1 to 8
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Limit_Line_Type">
        /// This control selects the limit line type.
        /// 
        /// Valid Values:
        /// RSESCI_LIMIT_LOWER (0) - Lower
        /// RSESCI_LIMIT_UPPER (1) - Upper
        /// 
        /// Default Value: RSESCI_LIMIT_LOWER (0)
        /// </param>
        /// <param name="Threshold">
        /// This control returns an absolute threshold value for limit lines with relative Y axis scaling.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetLimitLineThreshold(int Limit, int Limit_Line_Type, out double Threshold)
        {
            int pInvokeResult = PInvoke.GetLimitLineThreshold(this._handle, Limit, Limit_Line_Type, out Threshold);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines a comment for the limit line selected.
        /// The comment is independent from the measurement window.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:COMMent &lt;string&gt;
        /// </summary>
        /// <param name="Limit">
        /// This control selects the limit line.
        /// 
        /// Valid Range: 1 to 8
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Comment">
        /// This control defines a comment for the limit line selected.
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: "Comment"
        /// 
        /// Note(s):
        /// 
        /// (1) The comment can be up to 40 characters long.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetLimitLineComment(int Limit, string Comment)
        {
            int pInvokeResult = PInvoke.SetLimitLineComment(this._handle, Limit, Comment);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries a comment for the limit line selected.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:COMMent?
        /// </summary>
        /// <param name="Limit">
        /// This control selects the limit line.
        /// 
        /// Valid Range: 1 to 8
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Comment">
        /// This control returns a comment for the limit line selected.
        /// 
        /// Note(s):
        /// 
        /// (1) The array should consist of at least 41 elements.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetLimitLineComment(int Limit, System.Text.StringBuilder Comment)
        {
            int pInvokeResult = PInvoke.GetLimitLineComment(this._handle, Limit, Comment);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the selected axis values of the upper or lower limit lines. The values are defined independently of the measurement window.
        /// The number of values for the X axis and for the corresponding Upper and/or Lower limit lines have to be identical. Otherwise default values are entered for missing values or not required values are deleted.
        /// In analyzer mode, the unit of values depends on the frequency or time domain of the X axis.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:LOWer[:DATA]
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:UPPer[:DATA]
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:CONTrol[:DATA]
        /// </summary>
        /// <param name="Limit">
        /// This control selects the limit line.
        /// 
        /// Valid Range: 1 to 8
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Axis">
        /// This control selects the axis for which the data are set.
        /// 
        /// Valid Values:
        /// RSESCI_LIMIT_LOWER (0) - Lower
        /// RSESCI_LIMIT_UPPER (1) - Upper
        /// RSESCI_LIMIT_X     (2) - X Axis
        /// 
        /// Default Value: RSESCI_LIMIT_LOWER (0)
        /// </param>
        /// <param name="Array_Size">
        /// This control defines the size of array in argument 'Data'.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: 100
        /// </param>
        /// <param name="Data">
        /// This control defines values of the selected limit lines.
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: none
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetLimitLineValues(int Limit, int Axis, int Array_Size, double[] Data)
        {
            int pInvokeResult = PInvoke.SetLimitLineValues(this._handle, Limit, Axis, Array_Size, Data);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns values of the selected limit lines.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:LOWer[:DATA]?
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:UPPer[:DATA]?
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:CONTrol[:DATA]?
        /// </summary>
        /// <param name="Limit">
        /// This control selects the limit line.
        /// 
        /// Valid Range: 1 to 8
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Axis">
        /// This control selects the axis for which the data are set.
        /// 
        /// Valid Values:
        /// RSESCI_LIMIT_LOWER (0) - Lower
        /// RSESCI_LIMIT_UPPER (1) - Upper
        /// RSESCI_LIMIT_X     (2) - X Axis
        /// 
        /// Default Value: RSESCI_LIMIT_LOWER (0)
        /// </param>
        /// <param name="Array_Size">
        /// This control defines the size of array in argument 'Data'.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: 100
        /// </param>
        /// <param name="Data">
        /// This control returns values of the selected limit lines.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetLimitLineValues(int Limit, int Axis, int Array_Size, double[] Data)
        {
            int pInvokeResult = PInvoke.GetLimitLineValues(this._handle, Limit, Axis, Array_Size, Data);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function moves a limit line by the indicated value in x direction. In contrast to CALC:LIM:CONT:OFFS, the line is shifted by modifying the individual x values and not by means of an additive offset.
        /// The shift is independent of the measurement window.
        /// In the analyzer mode, the unit of values depends on the frequency or time domain of the X axis.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:CONTrol:SHIFt
        /// </summary>
        /// <param name="Limit">
        /// This control selects the limit line.
        /// 
        /// Valid Range: 1 to 8
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Shift">
        /// This control moves a limit line by the indicated value in x direction.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: 0.0
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int LimitLineShiftX(int Limit, double Shift)
        {
            int pInvokeResult = PInvoke.LimitLineShiftX(this._handle, Limit, Shift);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function shifts a limit line by the indicated value in Y direction. In contrast to CALC:LIM:LOW:OFFS, the line is shifted by modifying the individual Y values but not by means of an additive offset.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:LOWer:SHIFt
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:UPPer:SHIFt
        /// </summary>
        /// <param name="Limit">
        /// This control selects the limit line.
        /// 
        /// Valid Range: 1 to 8
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Axis_Type">
        /// This control selects the axis which should be shifted.
        /// 
        /// Valid Values:
        /// RSESCI_LIMIT_LOWER (0) - Lower
        /// RSESCI_LIMIT_UPPER (1) - Upper
        /// 
        /// Default Value: RSESCI_LIMIT_LOWER (0)
        /// </param>
        /// <param name="Shift">
        /// This control moves a limit line by the indicated value in y direction.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: 0.0
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int LimitLineShiftY(int Limit, int Axis_Type, double Shift)
        {
            int pInvokeResult = PInvoke.LimitLineShiftY(this._handle, Limit, Axis_Type, Shift);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries all the names of the limit lines saved on the hard disc.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analyzer
        /// Analog Demodulation
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:LIMit&lt;1...8&gt;:CATalog?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Limit">
        /// This control selects the limit line.
        /// 
        /// Valid Range: 1 to 8
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Buffer_Size">
        /// Pass the number of bytes in the string you specify for the catalog list parameter.
        /// 
        /// The buffer must contain at least as many elements as the value you specify with the Buffer Size parameter. If the value contains more bytes than you specify with the Buffer Size parameter, the function copies Buffer Size - 1 bytes into the buffer and returns the buffer size you must pass to get the entire value. For example, if the value is "file1,file2" and the Buffer Size is 6, the function places "file1" into the buffer and returns 12.
        /// 
        /// </param>
        /// <param name="Limit_Line_List">
        /// This control queries all the names of the limit lines saved on the hard disc.
        /// 
        /// The syntax of the output is:
        /// &lt;sum of file length of all following files&gt;,&lt;free spaces on hard disc&gt;,&lt;1st file name,,&lt;1st file length&gt;,&lt;2nd file name&gt;,&lt;2 nd file length&gt;,....,&lt;n th file name&gt;,&lt;nth file . length&gt;
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code
        /// either indicates success or describes an error or warning
        /// condition. You examine the status code from each call to an
        /// instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the
        /// rsfsq_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        /// 
        /// Status    Description
        /// -------------------------------------------------
        /// 
        /// BFFC0002  Parameter 2 (Number of Channel Tables) is null pointer.
        /// BFFC0003  Parameter 3 (Buffer Size) out of range.
        /// BFFC0004  Parameter 4 (Channel Tables List) is null pointer.
        /// 
        /// BFFC09F0  Instrument status error
        /// BFFC09F1  Option not installed
        /// BFFC09F2  Conflict with setup parameters
        /// BFFC09F3  Data are not available
        /// BFFC09F4  Unknown option
        /// BFFC09F5  Calibration failed
        /// BFFC09F6  Function does not support FSEx parameter value
        /// BFFC09F7  Overload. Instrument does not return I/Q values
        /// 
        /// This instrument driver also returns errors and warnings defined
        /// by other sources. The following table defines the ranges of
        /// additional status codes that this driver can return. The table
        /// lists the different include files that contain the defined
        /// constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetLimitLineCatalog(int Window, int Limit, int Buffer_Size, System.Text.StringBuilder Limit_Line_List)
        {
            int pInvokeResult = PInvoke.GetLimitLineCatalog(this._handle, Window, Limit, Buffer_Size, Limit_Line_List);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches Display Line 1 or 2 (level lines) on or off.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:DLINe&lt;1|2&gt;:STATe ON | OFF
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Line">
        /// This control selects the display line.
        /// 
        /// Valid Range: 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="State">
        /// This control switches Display Line 1 or 2 (level lines) on or off.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetDisplayLine(int Window, int Line, bool State)
        {
            int pInvokeResult = PInvoke.SetDisplayLine(this._handle, Window, Line, System.Convert.ToUInt16(State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the state of selected display line.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:DLINe&lt;1|2&gt;:STATe?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Line">
        /// This control selects the display line.
        /// 
        /// Valid Range: 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="State">
        /// This control returns the state of Display Line 1 or 2 (level lines) on or off.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetDisplayLine(int Window, int Line, out bool State)
        {
            ushort StateAsUShort;
            int pInvokeResult = PInvoke.GetDisplayLine(this._handle, Window, Line, out StateAsUShort);
            State = System.Convert.ToBoolean(StateAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the position of Display Line 1 or 2. These lines enable the user to mark any levels in the diagram.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:DLINe&lt;1|2&gt;
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Line">
        /// This control selects the display line.
        /// 
        /// Valid Range: 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Position">
        /// This control defines the position of Display Line 1 or 2.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: 0.0
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetDisplayLinePosition(int Window, int Line, double Position)
        {
            int pInvokeResult = PInvoke.SetDisplayLinePosition(this._handle, Window, Line, Position);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the position of Display Line 1 or 2.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:DLINe&lt;1|2&gt;?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Line">
        /// This control selects the display line.
        /// 
        /// Valid Range: 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Position">
        /// This control returns the position of Display Line 1 or 2.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetDisplayLinePosition(int Window, int Line, out double Position)
        {
            int pInvokeResult = PInvoke.GetDisplayLinePosition(this._handle, Window, Line, out Position);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches the frequency line on or off.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis (frequency domain only)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:FLINe&lt;1|2&gt;:STATe ON | OFF
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Line">
        /// This control selects the display line.
        /// 
        /// Valid Range: 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="State">
        /// This control switches the frequency line on or off.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetFrequencyLine(int Window, int Line, bool State)
        {
            int pInvokeResult = PInvoke.SetFrequencyLine(this._handle, Window, Line, System.Convert.ToUInt16(State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the state of the frequency line
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis (frequency domain only)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:FLINe&lt;1|2&gt;:STATe?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Line">
        /// This control selects the display line.
        /// 
        /// Valid Range: 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="State">
        /// This control returns the state of the frequency line.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetFrequencyLine(int Window, int Line, out bool State)
        {
            ushort StateAsUShort;
            int pInvokeResult = PInvoke.GetFrequencyLine(this._handle, Window, Line, out StateAsUShort);
            State = System.Convert.ToBoolean(StateAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the position of the frequency lines.
        /// The frequency lines mark the frequencies specified in the measurement window. Frequency lines are only available with SPAN &gt; 0.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis (frequency domain only)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:FLINe&lt;1|2&gt;
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Line">
        /// This control selects the frequency line.
        /// 
        /// Valid Range: 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Position">
        /// This control defines the position of Frequency Line 1 or 2.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: 0.0
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetFrequencyLinePosition(int Window, int Line, double Position)
        {
            int pInvokeResult = PInvoke.SetFrequencyLinePosition(this._handle, Window, Line, Position);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the position of Frequency Line 1 or 2.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis (frequency domain only)
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:FLINe&lt;1|2&gt;?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Line">
        /// This control selects the frequency line.
        /// 
        /// Valid Range: 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Position">
        /// This control returns the position of Frequency Line 1 or 2.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetFrequencyLinePosition(int Window, int Line, out double Position)
        {
            int pInvokeResult = PInvoke.GetFrequencyLinePosition(this._handle, Window, Line, out Position);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches the time line on or off.
        /// 
        /// Mode(s):
        /// Spectrum Analysis (time domain only (zero span))
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:TLINe&lt;1|2&gt;:STATe ON | OFF
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Line">
        /// This control selects the time line.
        /// 
        /// Valid Range: 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="State">
        /// This control switches the time line on or off.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetTimeLine(int Window, int Line, bool State)
        {
            int pInvokeResult = PInvoke.SetTimeLine(this._handle, Window, Line, System.Convert.ToUInt16(State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the state of time line.
        /// 
        /// Mode(s):
        /// Spectrum Analysis (time domain only (zero span))
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:TLINe&lt;1|2&gt;:STATe?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Line">
        /// This control selects the time line.
        /// 
        /// Valid Range: 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="State">
        /// This control returns the state of the time line.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetTimeLine(int Window, int Line, out bool State)
        {
            ushort StateAsUShort;
            int pInvokeResult = PInvoke.GetTimeLine(this._handle, Window, Line, out StateAsUShort);
            State = System.Convert.ToBoolean(StateAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the position of the time lines.
        /// The time lines mark the times specified in the measurement window. Time lines are only available with SPAN = 0.
        /// 
        /// Mode(s):
        /// Spectrum Analysis (time domain only (zero span))
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:TLINe&lt;1|2&gt;
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Line">
        /// This control selects the time line.
        /// 
        /// Valid Range: 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Position">
        /// This control defines the position of Time Line 1 or 2.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: 0.0
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetTimeLinePosition(int Window, int Line, double Position)
        {
            int pInvokeResult = PInvoke.SetTimeLinePosition(this._handle, Window, Line, Position);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the position of the time line.
        /// 
        /// Mode(s):
        /// Spectrum Analysis (time domain only (zero span))
        /// 
        /// Remote-control command(s):
        /// CALCulate&lt;1|2&gt;:TLINe&lt;1|2&gt;?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Line">
        /// This control selects the time line.
        /// 
        /// Valid Range: 1 to 2
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Position">
        /// This control returns the position of Time Line 1 or 2.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetTimeLinePosition(int Window, int Line, out double Position)
        {
            int pInvokeResult = PInvoke.GetTimeLinePosition(this._handle, Window, Line, out Position);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the active measurement window. In FULL SCREEN mode, the measurements are only performed in the active measurement window. Measurements are therefore initiated in the active window and result queries (marker, trace data and other results) answered also in the active window. Initiating measurements and querying results in the inactive window yields an error message (execution error).
        /// 
        /// Note(s):
        /// 
        /// (1) In FULL SCREEN mode, settings can also be performed in the inactive measurement window. They become effective as soon as the corresponding window becomes active.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:SELect
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetActiveWindow(int Window)
        {
            int pInvokeResult = PInvoke.SetActiveWindow(this._handle, Window);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the active measurement window.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:ACTive?
        /// </summary>
        /// <param name="Window">
        /// Returns the active measurement window.
        /// 
        /// Valid Values:
        /// 1 - Screen A
        /// 2 - Screen B
        /// 3 - Screen C
        /// 4 - Screen D
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetActiveWindow(out int Window)
        {
            int pInvokeResult = PInvoke.GetActiveWindow(this._handle, out Window);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches the measurement result display between FULL SCREEN and SPLIT SCREEN. The coupling of settings between screen A and screen B can be selected with the function rsesci_SetInstrumentCoupling().
        /// 
        /// In full-screen display the active measurement window can be selected with rsesci_SetActiveWindow().
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// DISPlay:FORMat SINGle | SPLit
        /// </summary>
        /// <param name="Mode">
        /// This control switches the measurement result display between FULL SCREEN and SPLIT SCREEN.
        /// 
        /// Valid Values:
        /// RSESCI_SCREEN_SINGLE (0) - Single
        /// RSESCI_SCREEN_SPLIT  (1) - Split
        /// 
        /// Default Value: RSESCI_SCREEN_SINGLE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetScreenMode(int Mode)
        {
            int pInvokeResult = PInvoke.SetScreenMode(this._handle, Mode);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns screen mode.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// DISPlay:FORMat?
        /// </summary>
        /// <param name="Mode">
        /// This control returns screen mode.
        /// 
        /// Valid Values:
        /// RSESCI_SCREEN_SINGLE (0) - Single
        /// RSESCI_SCREEN_SPLIT  (1) - Split
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetScreenMode(out int Mode)
        {
            int pInvokeResult = PInvoke.GetScreenMode(this._handle, out Mode);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches the measurement window for channel and adjacent-channel power measurements or for active FM demodulator (FS-K7) to full screen or half screen.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:SIZE LARGe | SMALl
        /// </summary>
        /// <param name="Display_Size">
        /// This control switches the measurement window for channel and adjacent-channel power measurements or for active FM demodulator (FS-K7) to full screen or half screen.
        /// 
        /// Valid Values:
        /// RSESCI_SCREEN_SMALL (0) - Small
        /// RSESCI_SCREEN_LARGE (1) - Large
        /// 
        /// Default Value: RSESCI_SCREEN_SMALL (0)
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetDisplaySize(int Display_Size)
        {
            int pInvokeResult = PInvoke.SetDisplaySize(this._handle, Display_Size);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns if the measurement window for channel and adjacent-channel power measurements or for active FM demodulator (FS-K7) is set to full screen or half screen.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// FM Demodulator (option R&amp;S FS-K7)
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:SIZE?
        /// </summary>
        /// <param name="Display_Size">
        /// Returns if the measurement window for channel and adjacent-channel power measurements or for active FM demodulator (FS-K7) is set to full screen or half screen.
        /// 
        /// Valid Values:
        /// RSESCI_SCREEN_SMALL (0) - Small
        /// RSESCI_SCREEN_LARGE (1) - Large
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetDisplaySize(out int Display_Size)
        {
            int pInvokeResult = PInvoke.GetDisplaySize(this._handle, out Display_Size);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches on or off the update of all display elements during remote control.
        /// 
        /// Note(s):
        /// 
        /// (1) The best performance is obtained when the display output is switched off during remote control.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// SYSTem:DISPlay:UPDate ON | OFF
        /// </summary>
        /// <param name="Display_Update">
        /// This control switches on or off the update of all display elements during remote control.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetDisplayUpdate(bool Display_Update)
        {
            int pInvokeResult = PInvoke.SetDisplayUpdate(this._handle, System.Convert.ToUInt16(Display_Update));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the state the update of all display elements during remote control.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// SYSTem:DISPlay:UPDate?
        /// </summary>
        /// <param name="Display_Update">
        /// This control returns the state the update of all display elements during remote control.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetDisplayUpdate(out bool Display_Update)
        {
            ushort Display_UpdateAsUShort;
            int pInvokeResult = PInvoke.GetDisplayUpdate(this._handle, out Display_UpdateAsUShort);
            Display_Update = System.Convert.ToBoolean(Display_UpdateAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches on the numeric indication of the maximum value of the bargraph.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// DISPlay:BARGraph:PHOLd ON | OFF
        /// </summary>
        /// <param name="Bargraph_Maxhold">
        /// This control switches on the numeric indication of the maximum value of the bargraph.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetDisplayBargraphMaxhold(bool Bargraph_Maxhold)
        {
            int pInvokeResult = PInvoke.SetDisplayBargraphMaxhold(this._handle, System.Convert.ToUInt16(Bargraph_Maxhold));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the state of the numeric indication of maximum bargraph value.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// DISPlay:BARGraph:PHOLd?
        /// </summary>
        /// <param name="Bargraph_Maxhold">
        /// This control returns the state of the numeric indication of maximum bargraph value.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetDisplayBargraphMaxhold(out bool Bargraph_Maxhold)
        {
            ushort Bargraph_MaxholdAsUShort;
            int pInvokeResult = PInvoke.GetDisplayBargraphMaxhold(this._handle, out Bargraph_MaxholdAsUShort);
            Bargraph_Maxhold = System.Convert.ToBoolean(Bargraph_MaxholdAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the minimum and maximum level of the bargraph.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// DISPlay:BARGraph:LEVel:LOWer?
        /// DISPlay:BARGraph:LEVel:UPPer?
        /// </summary>
        /// <param name="Minimum_Bargraph_Level">
        /// Returns the minimum level of the bargraph.
        /// 
        /// </param>
        /// <param name="Maximum_Bargraph_Level">
        /// Returns the maximum level of the bargraph.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetDisplayBargraphLevel(out double Minimum_Bargraph_Level, out double Maximum_Bargraph_Level)
        {
            int pInvokeResult = PInvoke.GetDisplayBargraphLevel(this._handle, out Minimum_Bargraph_Level, out Maximum_Bargraph_Level);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function resets the maxhold value of the numeric indication of the bargraph measurement.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// DISPlay:BARGraph:PHOLd:RESet
        /// </summary>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int ResetDisplayBargraphMaxhold()
        {
            int pInvokeResult = PInvoke.ResetDisplayBargraphMaxhold(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the coupled parameters of receiver and analyzer mode.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// INSTrument:COUPle:CENTer ALL | NONE
        /// INSTrument:COUPle:SPAN ALL | NONE
        /// INSTrument:COUPle:ATTenuation ALL | NONE
        /// INSTrument:COUPle:PROTection ALL | NONE
        /// INSTrument:COUPle:GAIN ALL | NONE
        /// INSTrument:COUPle:PRESelector ALL | NONE
        /// INSTrument:COUPle:DEModulation ALL | NONE
        /// INSTrument:COUPle:BANDwidth|BWIDth ALL | NONE
        /// </summary>
        /// <param name="Coupled_Parameters">
        /// Selects the coupled parameters.
        /// 
        /// Valid Values:
        /// RSESCI_INST_COUP_CENT (0) - Analyzer Center &lt;-&gt; Receiver Frequency
        /// RSESCI_INST_COUP_SPAN (1) - Start-Stop (Sweep &lt;-&gt; Scan)
        /// RSESCI_INST_COUP_ATT  (2) - Attenuation + Unit
        /// RSESCI_INST_COUP_PROT (3) - Minimum Attenuation
        /// RSESCI_INST_COUP_GAIN (4) - Preamp
        /// RSESCI_INST_COUP_PRES (5) - Preselector
        /// RSESCI_INST_COUP_DEM  (6) - Demodulator
        /// RSESCI_INST_COUP_BWID (7) - Bandwidth
        /// 
        /// Default Value: RSESCI_INST_COUP_CENT (0)
        /// 
        /// Notes:
        /// 
        /// (1) Analyzer Center &lt;-&gt; Receiver Frequency:
        /// Couples the center frequency or receive frequency settings for the analyzer and receiver modes to each other.
        /// 
        /// (2) Start-Stop (Sweep &lt;-&gt; Scan):
        /// Couples the start and stop frequencies of sweep and scan for the analyzer and receiver modes to each other.
        /// 
        /// (3) Attenuation + Unit:
        /// Couples the RF attenuation settings for the analyzer and receiver modes to each other. The RF attenuation and level unit are set to the same value in the analyzer and receiver screens.
        /// 
        /// (4) Minimum Attenuation:
        /// Couples the RF safety attenuation of 10 dB for the analyzer and receiver modes to each other.
        /// 
        /// (5) Preamp:
        /// Couples the preamplifier settings for the analyzer and receiver modes to each other.
        /// 
        /// (6) Preselector:
        /// Couples the activation of preselection for the analyzer and receiver modes to each other. Preselection cannot be switched off in the receiver mode. In practice, this means that preselection is always switched on automatically when changing to the analyzer mode with coupling activated.
        /// 
        /// (7) Demodulator:
        /// Couples the audio demodulator settings for the analyzer and receiver modes to each other.
        /// 
        /// (8) Bandwidth:
        /// Couples the bandwidth settings for the analyzer and receiver modes to each other. The resolution bandwidth (RBW) and filter type are set to the same value in the analyzer and receiver screens. Other settings, such as the quasipeak detector, may prevent these settings.
        /// 
        /// </param>
        /// <param name="Coupling">
        /// This control switches selected couplings on or off.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetParameterCoupling(int Coupled_Parameters, bool Coupling)
        {
            int pInvokeResult = PInvoke.SetParameterCoupling(this._handle, Coupled_Parameters, System.Convert.ToUInt16(Coupling));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns whether is selected coupled parameters of receiver and analyzer mode switched on or off.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// INSTrument:COUPle:CENTer?
        /// INSTrument:COUPle:SPAN?
        /// INSTrument:COUPle:ATTenuation?
        /// INSTrument:COUPle:PROTection?
        /// INSTrument:COUPle:GAIN?
        /// INSTrument:COUPle:PRESelector?
        /// INSTrument:COUPle:DEModulation?
        /// INSTrument:COUPle:BANDwidth|BWIDth?
        /// </summary>
        /// <param name="Coupled_Parameters">
        /// Selects the coupled parameters.
        /// 
        /// Valid Values:
        /// RSESCI_INST_COUP_CENT (0) - Analyzer Center &lt;-&gt; Receiver Frequency
        /// RSESCI_INST_COUP_SPAN (1) - Start-Stop (Sweep &lt;-&gt; Scan)
        /// RSESCI_INST_COUP_ATT  (2) - Attenuation + Unit
        /// RSESCI_INST_COUP_PROT (3) - Minimum Attenuation
        /// RSESCI_INST_COUP_GAIN (4) - Preamp
        /// RSESCI_INST_COUP_PRES (5) - Preselector
        /// RSESCI_INST_COUP_DEM  (6) - Demodulator
        /// RSESCI_INST_COUP_BWID (7) - Bandwidth
        /// 
        /// Default Value: RSESCI_INST_COUP_CENT (0)
        /// 
        /// Notes:
        /// 
        /// (1) Analyzer Center &lt;-&gt; Receiver Frequency:
        /// Couples the center frequency or receive frequency settings for the analyzer and receiver modes to each other.
        /// 
        /// (2) Start-Stop (Sweep &lt;-&gt; Scan):
        /// Couples the start and stop frequencies of sweep and scan for the analyzer and receiver modes to each other.
        /// 
        /// (3) Attenuation + Unit:
        /// Couples the RF attenuation settings for the analyzer and receiver modes to each other. The RF attenuation and level unit are set to the same value in the analyzer and receiver screens.
        /// 
        /// (4) Minimum Attenuation:
        /// Couples the RF safety attenuation of 10 dB for the analyzer and receiver modes to each other.
        /// 
        /// (5) Preamp:
        /// Couples the preamplifier settings for the analyzer and receiver modes to each other.
        /// 
        /// (6) Preselector:
        /// Couples the activation of preselection for the analyzer and receiver modes to each other. Preselection cannot be switched off in the receiver mode. In practice, this means that preselection is always switched on automatically when changing to the analyzer mode with coupling activated.
        /// 
        /// (7) Demodulator:
        /// Couples the audio demodulator settings for the analyzer and receiver modes to each other.
        /// 
        /// (8) Bandwidth:
        /// Couples the bandwidth settings for the analyzer and receiver modes to each other. The resolution bandwidth (RBW) and filter type are set to the same value in the analyzer and receiver screens. Other settings, such as the quasipeak detector, may prevent these settings.
        /// 
        /// </param>
        /// <param name="Coupling">
        /// This control returns whether is selected couplings switched on or off.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetParameterCoupling(int Coupled_Parameters, out bool Coupling)
        {
            ushort CouplingAsUShort;
            int pInvokeResult = PInvoke.GetParameterCoupling(this._handle, Coupled_Parameters, out CouplingAsUShort);
            Coupling = System.Convert.ToBoolean(CouplingAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the parameter coupling between the two measurement windows screen A and B.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// INSTrument:COUPle NONE | RLEVel | CF_B | CF_A
        /// </summary>
        /// <param name="Coupling">
        /// Selects the parameter coupling between the two measurement windows screen A and B.
        /// 
        /// Valid Values:
        /// RSESCI_INST_COUP_NONE   (0) - None
        /// RSESCI_INST_COUP_RLEVEL (1) - Reference Levels
        /// RSESCI_INST_COUP_CF_A   (2) - Center Frequency of Screen A
        /// RSESCI_INST_COUP_CF_B   (3) - Center Frequency of Screen B
        /// 
        /// Default Value: RSESCI_INST_COUP_NONE (0)
        /// 
        /// Note(s):
        /// 
        /// (1) None:
        /// No coupling. The two measurement windows are operated like two independent "virtual" devices.
        /// 
        /// (2) Reference Levels:
        /// The reference levels of the two measurement windows are coupled.
        /// 
        /// (3) Center Frequency of Screen A:
        /// The center frequency of screen A is coupled to the frequency of marker 1 in screen B.
        /// 
        /// (4) Center Frequency of Screen B:
        /// The center frequency of screen B is coupled to the frequency of marker 1 in screen A.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetInstrumentCoupling(int Coupling)
        {
            int pInvokeResult = PInvoke.SetInstrumentCoupling(this._handle, Coupling);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function toggles between linear and logarithmic display in receiver mode.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACE&lt;1...3&gt;:X:SPACing LINear | LOGarithmic
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Display_Spacing">
        /// This control toggles between linear and logarithmic display in receiver mode.
        /// 
        /// Valid Values:
        /// RSESCI_SPACING_LIN (0) - Linear
        /// RSESCI_SPACING_LOG (1) - Logarithmic
        /// 
        /// Default Value: RSESCI_SPACING_LIN (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetXAxisDisplaySpacing(int Window, int Display_Spacing)
        {
            int pInvokeResult = PInvoke.SetXAxisDisplaySpacing(this._handle, Window, Display_Spacing);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries setting of display spacing in receiver mode.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACE&lt;1...3&gt;:X:SPACing?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Display_Spacing">
        /// This control returns setting of display spacing in receiver mode.
        /// 
        /// Valid Values:
        /// RSESCI_LIN (0) - Linear
        /// RSESCI_LOG (1) - Logarithmic
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetXAxisDisplaySpacing(int Window, out int Display_Spacing)
        {
            int pInvokeResult = PInvoke.GetXAxisDisplaySpacing(this._handle, Window, out Display_Spacing);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches the zoom on or off.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:TRACe&lt;1...3&gt;:X[:SCALe]:ZOOM
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Zoom_State">
        /// This control switches the zoom on or off.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetXAxisDisplayZoom(int Window, bool Zoom_State)
        {
            int pInvokeResult = PInvoke.SetXAxisDisplayZoom(this._handle, Window, System.Convert.ToUInt16(Zoom_State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines a comment (max. 20 characters) which can be displayed on the screen in the selected measurement window.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// DISPlay[:WINDow&lt;1|2&gt;]:TEXT[:DATA] &lt;string&gt;
        /// DISPlay[:WINDow&lt;1|2&gt;]:TEXT:STATe ON | OFF
        /// </summary>
        /// <param name="Screen_Title">
        /// Switches on or off the display of the comment (screen title) in the selected measurement window.
        /// 
        /// Valid Range:
        /// VI_FALSE (0) - Off (Default Value)
        /// VI_TRUE  (1) - On
        /// 
        /// </param>
        /// <param name="Comment_String">
        /// Defines a comment (max. 20 characters) which can be displayed on the screen in the selected measurement window.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: "Screen Title"
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetScreenTitle(bool Screen_Title, string Comment_String)
        {
            int pInvokeResult = PInvoke.SetScreenTitle(this._handle, System.Convert.ToUInt16(Screen_Title), Comment_String);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function controls the apperance of display elements (annotation, company logo, display date + time).
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// DISPlay:LOGO ON | OFF
        /// DISPlay[:WINDow&lt;1|2&gt;]:TIME ON | OFF
        /// DISPlay:ANNotation:FREQuency ON | OFF
        /// </summary>
        /// <param name="Annotation">
        /// This control switches the x-axis annotation on or off.
        /// 
        /// Valid Range:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On (Default Value)
        /// 
        /// </param>
        /// <param name="Logo">
        /// This control switches the factory logo on the screen on or off.
        /// 
        /// Valid Range:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On (Default Value)
        /// 
        /// </param>
        /// <param name="Time___Date">
        /// This control switches on or off the screen display of date and time.
        /// 
        /// Valid Range:
        /// VI_FALSE (0) - Off (Default Value)
        /// VI_TRUE  (1) - On
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetDisplayElements(bool Annotation, bool Logo, bool Time___Date)
        {
            int pInvokeResult = PInvoke.SetDisplayElements(this._handle, System.Convert.ToUInt16(Annotation), System.Convert.ToUInt16(Logo), System.Convert.ToUInt16(Time___Date));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the color table of the instrument.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// DISPlay:CMAP&lt;1...34&gt;:HSL &lt;hue&gt;,&lt;sat&gt;,&lt;lum&gt;
        /// DISPlay:CMAP&lt;1...34&gt;:PDEFined BLACk | BLUE | BROWn | GREen | CYAN | RED | MAGenta | YELLow | WHITe | DGRAy | LGRAy | LBLUe | LGREen | LCYan | LRED | LMAGenta
        /// </summary>
        /// <param name="Color_Map">
        /// This control defines the color table of the instrument.
        /// 
        /// Valid Range:
        /// RSESCI_CMAP_1 (1) - Background
        /// RSESCI_CMAP_2 (2) - Grid
        /// RSESCI_CMAP_3 (3) - Function and status field + data entry text
        /// RSESCI_CMAP_4 (4) - Function field LED on
        /// RSESCI_CMAP_5 (5) - Function field LED warn
        /// RSESCI_CMAP_6 (6) - Enhancement label text
        /// RSESCI_CMAP_7 (7) - Status field background
        /// RSESCI_CMAP_8 (8) - Trace 1
        /// RSESCI_CMAP_9 (9) - Trace 2
        /// RSESCI_CMAP_10 (10) - Trace 3
        /// RSESCI_CMAP_11 (11) - Marker
        /// RSESCI_CMAP_12 (12) - Lines
        /// RSESCI_CMAP_13 (13) - Measurement status + Limit check pass
        /// RSESCI_CMAP_14 (14) - Limit check fail
        /// RSESCI_CMAP_15 (15) - Table + softkey background
        /// RSESCI_CMAP_16 (16) - Table + softkey text
        /// RSESCI_CMAP_17 (17) - Table selected field text
        /// RSESCI_CMAP_18 (18) - Table selected field background
        /// RSESCI_CMAP_19 (19) - Table + data entry field opaque titlebar
        /// RSESCI_CMAP_20 (20) - Data entry field opaque text
        /// RSESCI_CMAP_21 (21) - Data entry field opaque background
        /// RSESCI_CMAP_22 (22) - 3D shade bright part
        /// RSESCI_CMAP_23 (23) - 3D shade dark part
        /// RSESCI_CMAP_24 (24) - Softkey state on
        /// RSESCI_CMAP_25 (25) - Softkey state data entry
        /// RSESCI_CMAP_26 (26) - Logo
        /// RSESCI_CMAP_27 (27) - Bargraph PK+
        /// RSESCI_CMAP_28 (28) - Bargraph PK-
        /// RSESCI_CMAP_29 (29) - Bargraph QPK
        /// RSESCI_CMAP_30 (30) - Bargraph AVER
        /// RSESCI_CMAP_31 (31) - Bargraph CAV
        /// RSESCI_CMAP_32 (32) - Bargraph RMS
        /// RSESCI_CMAP_33 (33) - Bargraph CRMS
        /// RSESCI_CMAP_32 (34) - Final Meas
        /// 
        /// Default Value: RSESCI_CMAP_1 (1)
        /// 
        /// </param>
        /// <param name="Predefined_Colors">
        /// This control defines the color table of the instrument using predefined color values or user defined settings via tint, saturation and brightness parameters.
        /// 
        /// Valid Range:
        /// RSESCI_USER_DEFINED (-1) - User defined settings
        /// RSESCI_BLACK        (0)  - Black
        /// RSESCI_BLUE         (1)  - Blue
        /// RSESCI_BROWN        (2)  - Brown
        /// RSESCI_GREEN        (3)  - Green
        /// RSESCI_CYAN         (4)  - Cyan
        /// RSESCI_RED          (5)  - Red
        /// RSESCI_MAGNETA      (6)  - Magneta
        /// RSESCI_YELLOW       (7)  - Yellow
        /// RSESCI_WHITE        (8)  - White
        /// RSESCI_DGRAY        (9)  - Dark Gray
        /// RSESCI_LGRAY        (10) - Light Gray
        /// RSESCI_LBLUE        (11) - Light Blue
        /// RSESCI_LGREEN       (12) - Light Green
        /// RSESCI_LCYAN        (13) - Light Cyan
        /// RSESCI_LRED         (14) - Light Red
        /// RSESCI_LMAGNETA     (15) - Light Magneta
        /// 
        /// Default Value: RSESCI_BLACK (0)
        /// 
        /// </param>
        /// <param name="Tint">
        /// This control sets the hue = Tint.
        /// 
        /// Valid Range: 0.0 to 100.0 %
        /// 
        /// Default Value: 30.0 %
        /// 
        /// </param>
        /// <param name="Saturation">
        /// This control sets the sat = Saturation.
        /// 
        /// Valid Range: 0.0 to 100.0 %
        /// 
        /// Default Value: 80.0 %
        /// 
        /// </param>
        /// <param name="Brightness">
        /// This control sets the lum = Brightness.
        /// 
        /// Valid Range: 0.0 to 100.0 %
        /// 
        /// Default Value: 100.0 %
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetDisplayUserDefinedColors(int Color_Map, int Predefined_Colors, double Tint, double Saturation, double Brightness)
        {
            int pInvokeResult = PInvoke.SetDisplayUserDefinedColors(this._handle, Color_Map, Predefined_Colors, Tint, Saturation, Brightness);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function resets the screen colors of all display items to their default settings.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// DISPlay:CMAP&lt;1...34&gt;:DEFault&lt;1|2&gt;
        /// </summary>
        /// <param name="Default_Colors">
        /// The default colors 1 and 2 restores the default settings for brightness, color tint and color saturation for all display screen elements.
        /// 
        /// The color schemes have been selected to give optimum visibility of all picture elements at an angle of vision from above or below.
        /// 
        /// Valid Values:
        /// RSESCI_DEFAULT_1 (1) - Default Colors 1
        /// RSESCI_DEFAULT_2 (2) - Default Colors 2
        /// 
        /// Default Value: RSESCI_DEFAULT_1 (1)
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetDisplayDefaultColors(int Default_Colors)
        {
            int pInvokeResult = PInvoke.SetDisplayDefaultColors(this._handle, Default_Colors);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches on or off the power-save mode of the display. With the power-save mode activated the display including backlight is completely switched off after the elapse of the response time.
        /// 
        /// Note:
        /// 
        /// This mode is recommended for preserving the display especially if the instrument is exclusively operated via remote control.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// DISPlay:PSAVe[:STATe] ON | OFF
        /// DISPlay:PSAVe:HOLDoff 1 to 60
        /// </summary>
        /// <param name="Power_Saving">
        /// Switches on or off the power-save mode of the display.
        /// 
        /// Valid Range:
        /// VI_FALSE (0) - Off (Default Value)
        /// VI_TRUE  (1) - On
        /// 
        /// </param>
        /// <param name="Holdoff_Time">
        /// Sets the holdoff time for the power-save mode of the display.
        /// 
        /// Valid Range: 1 to 60 minutes
        /// 
        /// Default Value: 15 minutes
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetDisplayPowerSaving(bool Power_Saving, int Holdoff_Time)
        {
            int pInvokeResult = PInvoke.SetDisplayPowerSaving(this._handle, System.Convert.ToUInt16(Power_Saving), Holdoff_Time);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function controls selection of the reference oscillator.
        /// 
        /// If the external reference oscillator is selected, the reference signal must be connected to the rear panel of the instrument.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]ROSCillator:SOURce INTernal | EXTernal | EAUTo
        /// </summary>
        /// <param name="Reference_Oscillator">
        /// This control selects between internal and external reference oscillator.
        /// 
        /// Valid Values:
        /// RSESCI_INT (0) - Internal
        /// RSESCI_EXT (1) - External
        /// RSESCI_EAUT(2) - Fail-safe External
        /// 
        /// Default Value: RSESCI_INT (0)
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetExternalReferenceOscillator(int Reference_Oscillator)
        {
            int pInvokeResult = PInvoke.SetExternalReferenceOscillator(this._handle, Reference_Oscillator);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns selection of the reference oscillator source.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]ROSCillator:SOURce?
        /// </summary>
        /// <param name="Reference_Oscillator">
        /// This control returns if internal or external reference oscillator is used.
        /// 
        /// Valid Values:
        /// RSESCI_INT (0) - Internal
        /// RSESCI_EXT (1) - External
        /// RSESCI_EAUT(2) - Fail-safe External
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetExternalReferenceOscillator(out int Reference_Oscillator)
        {
            int pInvokeResult = PInvoke.GetExternalReferenceOscillator(this._handle, out Reference_Oscillator);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function informs the instrument on the frequency of the external reference oscillator. This frequency is used as starting point for the synchronization of the internal reference frequencies.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]ROSCillator:EXTernal:FREQuency
        /// </summary>
        /// <param name="External_Frequency">
        /// This control sets the external reference oscillator frequency.
        /// 
        /// Valid Values:
        /// 1.0e6 to 20.0e6 Hz
        /// 
        /// Default Value: 5.0e6 Hz
        /// 
        /// Note(s):
        /// 
        /// (1) This control has effect only for source External.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetExternalReferenceOscillatorFrequency(double External_Frequency)
        {
            int pInvokeResult = PInvoke.SetExternalReferenceOscillatorFrequency(this._handle, External_Frequency);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the frequency of the external reference oscillator. This frequency is used as starting point for the synchronization of the internal reference frequencies.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]ROSCillator:EXTernal:FREQuency?
        /// </summary>
        /// <param name="External_Frequency">
        /// This control gets the external reference oscillator frequency.
        /// 
        /// Valid Values:
        /// 1.0e6 to 20.0e6 Hz
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetExternalReferenceOscillatorFrequency(out double External_Frequency)
        {
            int pInvokeResult = PInvoke.GetExternalReferenceOscillatorFrequency(this._handle, out External_Frequency);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches the 28-V supply of the noise source at the rear panel on or off.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// DIAGnostic&lt;1|2&gt;:SERVice:NSOurce ON | OFF
        /// </summary>
        /// <param name="Noise_Source">
        /// This control switches on or off the 28-V supply of the noise source at the rear panel
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetExternalNoiseSource(bool Noise_Source)
        {
            int pInvokeResult = PInvoke.SetExternalNoiseSource(this._handle, System.Convert.ToUInt16(Noise_Source));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns if the 28-V supply of the noise source at the rear panel is switched on or off.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// DIAGnostic&lt;1|2&gt;:SERVice:NSOurce?
        /// </summary>
        /// <param name="Noise_Source">
        /// This control returns if the 28-V supply of the noise source at the rear panel is switched on or off.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetExternalNoiseSource(out bool Noise_Source)
        {
            ushort Noise_SourceAsUShort;
            int pInvokeResult = PInvoke.GetExternalNoiseSource(this._handle, out Noise_SourceAsUShort);
            Noise_Source = System.Convert.ToBoolean(Noise_SourceAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// In analyzer mode, this function switches the preselection on or off.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// INPut&lt;1|2&gt;:PRESelection[:STATe] ON | OFF
        /// </summary>
        /// <param name="Preselector">
        /// This control switches the preselection on or off.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetPreselector(bool Preselector)
        {
            int pInvokeResult = PInvoke.SetPreselector(this._handle, System.Convert.ToUInt16(Preselector));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns if in analyzer mode the preselection is switched on or off.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// INPut&lt;1|2&gt;:PRESelection[:STATe]?
        /// </summary>
        /// <param name="Preselector">
        /// This control returns if the preselection is switched on or off.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetPreselector(out bool Preselector)
        {
            ushort PreselectorAsUShort;
            int pInvokeResult = PInvoke.GetPreselector(this._handle, out PreselectorAsUShort);
            Preselector = System.Convert.ToBoolean(PreselectorAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches the selected transducer factor on or off.
        /// 
        /// Note(s):
        /// 
        /// Prior to this function, the function Set Transducer Factor Name (rsesci_SetTransducerFactorName) must be called.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]CORRection:TRANsducer[:STATe] ON | OFF
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Transducer_Factor">
        /// This control switches the selected transducer factor on or off.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetTransducerFactor(int Window, bool Transducer_Factor)
        {
            int pInvokeResult = PInvoke.SetTransducerFactor(this._handle, Window, System.Convert.ToUInt16(Transducer_Factor));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns if the selected transducer factor is on or off.
        /// 
        /// Note(s):
        /// 
        /// Prior to this function, the function Set Transducer Factor Name (rsesci_SetTransducerFactorName) must be called.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]CORRection:TRANsducer[:STATe]?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Transducer_Factor">
        /// This control returns if the selected transducer factor is on or off.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetTransducerFactor(int Window, out bool Transducer_Factor)
        {
            ushort Transducer_FactorAsUShort;
            int pInvokeResult = PInvoke.GetTransducerFactor(this._handle, Window, out Transducer_FactorAsUShort);
            Transducer_Factor = System.Convert.ToBoolean(Transducer_FactorAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the transducer factor designated by &lt;name&gt;. If &lt;name&gt; does not exist yet, a new transducer factor is created.
        /// 
        /// This function must be sent prior to the subsequent functions for modifying/activating transducer factors.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]CORRection:TRANsducer:SELect &lt;name&gt;
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Name">
        /// Selects the transducer factor. Name of the transducer factor in string data form with a maximum of 8 characters.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: "FACTOR1"
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetTransducerFactorName(int Window, string Name)
        {
            int pInvokeResult = PInvoke.SetTransducerFactorName(this._handle, Window, Name);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the name of the currently active transducer factor.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]CORRection:TRANsducer:SELect?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Name">
        /// Returns the name of the currently active transducer factor.
        /// 
        /// Note(s):
        /// 
        /// (1) The array should consist of at least 9 elements.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetTransducerFactorName(int Window, System.Text.StringBuilder Name)
        {
            int pInvokeResult = PInvoke.GetTransducerFactorName(this._handle, Window, Name);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function reads out the names of all transducer factors stored on the harddisk.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]CORRection:TRANsducer:CATalog?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Number_of_Transducer_Factors">
        /// This control returns number of names of all transducer factors.
        /// </param>
        /// <param name="Buffer_Size">
        /// Pass the number of bytes in the string you specify for the catalog list parameter.
        /// 
        /// The buffer must contain at least as many elements as the value you specify with the Buffer Size parameter. If the value contains more bytes than you specify with the Buffer Size parameter, the function copies Buffer Size - 1 bytes into the buffer and returns the buffer size you must pass to get the entire value. For example, if the value is "file1,file2" and the Buffer Size is 6, the function places "file1" into the buffer and returns 12.
        /// 
        /// </param>
        /// <param name="Transducer_Factors_List">
        /// This control returns the names of all transducer factors stored on the hard disk.
        /// 
        /// The buffer must contain at least as many elements as the value you specify with the Buffer Size parameter. If the value contains more bytes than you specify with the Buffer Size parameter, the function copies Buffer Size - 1 bytes into the buffer and returns the buffer size you must pass to get the entire value. For example, if the value is "file1,file2" and the Buffer Size is 6, the function places "file1" into the buffer and returns 12.
        /// 
        /// if you pass 0 for buffer Size, you can use VI_NULL for this parameter.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code
        /// either indicates success or describes an error or warning
        /// condition. You examine the status code from each call to an
        /// instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the
        /// rsfsq_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        /// 
        /// Status    Description
        /// -------------------------------------------------
        /// 
        /// BFFC0002  Parameter 2 (Number of Channel Tables) is null pointer.
        /// BFFC0003  Parameter 3 (Buffer Size) out of range.
        /// BFFC0004  Parameter 4 (Channel Tables List) is null pointer.
        /// 
        /// BFFC09F0  Instrument status error
        /// BFFC09F1  Option not installed
        /// BFFC09F2  Conflict with setup parameters
        /// BFFC09F3  Data are not available
        /// BFFC09F4  Unknown option
        /// BFFC09F5  Calibration failed
        /// BFFC09F6  Function does not support FSEx parameter value
        /// BFFC09F7  Overload. Instrument does not return I/Q values
        /// 
        /// This instrument driver also returns errors and warnings defined
        /// by other sources. The following table defines the ranges of
        /// additional status codes that this driver can return. The table
        /// lists the different include files that contain the defined
        /// constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetTransducerFactorCatalog(int Window, out int Number_of_Transducer_Factors, int Buffer_Size, System.Text.StringBuilder Transducer_Factors_List)
        {
            int pInvokeResult = PInvoke.GetTransducerFactorCatalog(this._handle, Window, out Number_of_Transducer_Factors, Buffer_Size, Transducer_Factors_List);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the active transducer factor. If no transducer factor is active, an empty string is returned.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]CORRection:TRANsducer:ACTive?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Buffer_Size">
        /// Pass the number of bytes in the string you specify for the active transducer factor.
        /// 
        /// The buffer must contain at least as many elements as the value you specify with the Buffer Size parameter. If the value contains more bytes than you specify with the Buffer Size parameter, the function copies Buffer Size - 1 bytes into the buffer and returns the buffer size you must pass to get the entire value. For example, if the value is "file1,file2" and the Buffer Size is 6, the function places "file1" into the buffer and returns 12.
        /// 
        /// </param>
        /// <param name="Active_Transducer_Factor">
        /// This control returns the active transducer factor. If no transducer factor is active, an empty string is returned.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code
        /// either indicates success or describes an error or warning
        /// condition. You examine the status code from each call to an
        /// instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the
        /// rsfsq_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        /// 
        /// Status    Description
        /// -------------------------------------------------
        /// 
        /// BFFC0002  Parameter 2 (Number of Channel Tables) is null pointer.
        /// BFFC0003  Parameter 3 (Buffer Size) out of range.
        /// BFFC0004  Parameter 4 (Channel Tables List) is null pointer.
        /// 
        /// BFFC09F0  Instrument status error
        /// BFFC09F1  Option not installed
        /// BFFC09F2  Conflict with setup parameters
        /// BFFC09F3  Data are not available
        /// BFFC09F4  Unknown option
        /// BFFC09F5  Calibration failed
        /// BFFC09F6  Function does not support FSEx parameter value
        /// BFFC09F7  Overload. Instrument does not return I/Q values
        /// 
        /// This instrument driver also returns errors and warnings defined
        /// by other sources. The following table defines the ranges of
        /// additional status codes that this driver can return. The table
        /// lists the different include files that contain the defined
        /// constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetActiveTransducerFactor(int Window, int Buffer_Size, System.Text.StringBuilder Active_Transducer_Factor)
        {
            int pInvokeResult = PInvoke.GetActiveTransducerFactor(this._handle, Window, Buffer_Size, Active_Transducer_Factor);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function deletes the selected transducer factor.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]CORRection:TRANsducer:SELect &lt;name&gt;
        /// [SENSe&lt;1|2&gt;:]CORRection:TRANsducer:DELete
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Name">
        /// Selects the transducer factor. Name of the transducer factor in string data form with a maximum of 8 characters.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: "FACTOR1"
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int DeleteTransducerFactor(int Window, string Name)
        {
            int pInvokeResult = PInvoke.DeleteTransducerFactor(this._handle, Window, Name);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches on the display of the active transducer factor or set.
        /// 
        /// Note(s):
        /// 
        /// Prior to this function, the function Set Transducer Factor Name (rsesci_SetTransducerFactorName) must be called.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]CORRection:TRANsducer:VIEW ON | OFF
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Transducer_Factor_View">
        /// This control switches on the display of the active transducer factor or set.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetTransducerFactorView(int Window, bool Transducer_Factor_View)
        {
            int pInvokeResult = PInvoke.SetTransducerFactorView(this._handle, Window, System.Convert.ToUInt16(Transducer_Factor_View));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns if the display of the active transducer factor or set is on or off.
        /// 
        /// Note(s):
        /// 
        /// Prior to this function, the function Set Transducer Factor Name (rsesci_SetTransducerFactorName) must be called.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]CORRection:TRANsducer:VIEW?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Transducer_Factor_View">
        /// This control returns if the display of the active transducer factor or set is on or off.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetTransducerFactorView(int Window, out bool Transducer_Factor_View)
        {
            ushort Transducer_Factor_ViewAsUShort;
            int pInvokeResult = PInvoke.GetTransducerFactorView(this._handle, Window, out Transducer_Factor_ViewAsUShort);
            Transducer_Factor_View = System.Convert.ToBoolean(Transducer_Factor_ViewAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches on the automatic adjustment of the reference level to the selected transducer factor.
        /// 
        /// Note(s):
        /// 
        /// Prior to this function, the function Set Transducer Factor Name (rsesci_SetTransducerFactorName) must be called.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]CORRection:TRANsducer:ADJust:RLEVel[:STATe] ON | OFF
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Auto_Adjust">
        /// This control switches on the automatic adjustment of the reference level to the selected transducer factor.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetTransducerFactorAutoAdjust(int Window, bool Auto_Adjust)
        {
            int pInvokeResult = PInvoke.SetTransducerFactorAutoAdjust(this._handle, Window, System.Convert.ToUInt16(Auto_Adjust));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns if the automatic adjustment of the reference level to the selected transducer factor is enabled or disabled.
        /// 
        /// Note(s):
        /// 
        /// Prior to this function, the function Set Transducer Factor Name (rsesci_SetTransducerFactorName) must be called.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]CORRection:TRANsducer:ADJust:RLEVel[:STATe]?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Auto_Adjust">
        /// This control returns if the automatic adjustment of the reference level to the selected transducer factor is enabled or disabled.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetTransducerFactorAutoAdjust(int Window, out bool Auto_Adjust)
        {
            ushort Auto_AdjustAsUShort;
            int pInvokeResult = PInvoke.GetTransducerFactorAutoAdjust(this._handle, Window, out Auto_AdjustAsUShort);
            Auto_Adjust = System.Convert.ToBoolean(Auto_AdjustAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches the selected transducer set on or off.
        /// 
        /// Note(s):
        /// 
        /// Prior to this function, the function Set Transducer Set Name (rsesci_SetTransducerSetName) must be called.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]CORRection:TSET[:STATe] ON | OFF
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Transducer_Set">
        /// This control switches the selected transducer set on or off.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetTransducerSet(int Window, bool Transducer_Set)
        {
            int pInvokeResult = PInvoke.SetTransducerSet(this._handle, Window, System.Convert.ToUInt16(Transducer_Set));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns if the selected transducer set is switched on or off.
        /// 
        /// Note(s):
        /// 
        /// Prior to this function, the function Set Transducer Set Name (rsesci_SetTransducerSetName) must be called.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]CORRection:TSET[:STATe]?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Transducer_Set">
        /// This control returns if the selected transducer set is switched on or off.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetTransducerSet(int Window, out bool Transducer_Set)
        {
            ushort Transducer_SetAsUShort;
            int pInvokeResult = PInvoke.GetTransducerSet(this._handle, Window, out Transducer_SetAsUShort);
            Transducer_Set = System.Convert.ToBoolean(Transducer_SetAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selected the transducer set designated by &lt;name&gt;. If &lt;name&gt; does not exist yet, a new set is created.
        /// 
        /// This function must be sent prior to the subsequent functions for changing/activating the transducer sets.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]CORRection:TSET:SELect &lt;name&gt;
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Name">
        /// Selects the transducer set. Name of the transducer set in string data form with a maximum of 8 characters.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: "SET1"
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetTransducerSetName(int Window, string Name)
        {
            int pInvokeResult = PInvoke.SetTransducerSetName(this._handle, Window, Name);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the name of the currently active transducer set.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]CORRection:TSET:SELect?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Name">
        /// Returns the name of the currently active transducer set.
        /// 
        /// Note(s):
        /// 
        /// (1) The array should consist of at least 9 elements.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetTransducerSetName(int Window, System.Text.StringBuilder Name)
        {
            int pInvokeResult = PInvoke.GetTransducerSetName(this._handle, Window, Name);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function reads out the names of all transducer sets stored on the harddisk.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]CORRection:TSET:CATalog?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Number_of_Transducer_Sets">
        /// This control returns number of names of all transducer sets.
        /// </param>
        /// <param name="Buffer_Size">
        /// Pass the number of bytes in the string you specify for the catalog list parameter.
        /// 
        /// The buffer must contain at least as many elements as the value you specify with the Buffer Size parameter. If the value contains more bytes than you specify with the Buffer Size parameter, the function copies Buffer Size - 1 bytes into the buffer and returns the buffer size you must pass to get the entire value. For example, if the value is "file1,file2" and the Buffer Size is 6, the function places "file1" into the buffer and returns 12.
        /// 
        /// </param>
        /// <param name="Transducer_Sets_List">
        /// This control returns the names of all transducer sets stored on the hard disk.
        /// 
        /// The buffer must contain at least as many elements as the value you specify with the Buffer Size parameter. If the value contains more bytes than you specify with the Buffer Size parameter, the function copies Buffer Size - 1 bytes into the buffer and returns the buffer size you must pass to get the entire value. For example, if the value is "file1,file2" and the Buffer Size is 6, the function places "file1" into the buffer and returns 12.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code
        /// either indicates success or describes an error or warning
        /// condition. You examine the status code from each call to an
        /// instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the
        /// rsfsq_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        /// 
        /// Status    Description
        /// -------------------------------------------------
        /// 
        /// BFFC0002  Parameter 2 (Number of Channel Tables) is null pointer.
        /// BFFC0003  Parameter 3 (Buffer Size) out of range.
        /// BFFC0004  Parameter 4 (Channel Tables List) is null pointer.
        /// 
        /// BFFC09F0  Instrument status error
        /// BFFC09F1  Option not installed
        /// BFFC09F2  Conflict with setup parameters
        /// BFFC09F3  Data are not available
        /// BFFC09F4  Unknown option
        /// BFFC09F5  Calibration failed
        /// BFFC09F6  Function does not support FSEx parameter value
        /// BFFC09F7  Overload. Instrument does not return I/Q values
        /// 
        /// This instrument driver also returns errors and warnings defined
        /// by other sources. The following table defines the ranges of
        /// additional status codes that this driver can return. The table
        /// lists the different include files that contain the defined
        /// constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetTransducerSetCatalog(int Window, out int Number_of_Transducer_Sets, int Buffer_Size, System.Text.StringBuilder Transducer_Sets_List)
        {
            int pInvokeResult = PInvoke.GetTransducerSetCatalog(this._handle, Window, out Number_of_Transducer_Sets, Buffer_Size, Transducer_Sets_List);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function deletes the selected transducer set.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]CORRection:TSET:SELect &lt;name&gt;
        /// [SENSe&lt;1|2&gt;:]CORRection:TSET:DELete
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Name">
        /// Selects the transducer set. Name of the transducer set in string data form with a maximum of 8 characters.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: "SET1"
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int DeleteTransducerSet(int Window, string Name)
        {
            int pInvokeResult = PInvoke.DeleteTransducerSet(this._handle, Window, Name);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the unit of the transducer factor selected.
        /// 
        /// Note(s):
        /// 
        /// Prior to this function, the function Set Transducer Factor Name (rsesci_SetTransducerFactorName) must be called.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]CORRection:TRANsducer:UNIT 'DB' | 'DBM' | 'DBUV' | 'DBUV/M' | 'DBUA' | 'DBUA/M' | 'DBPW' | 'DBPT' | 'DBMV'
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Transducer_Factor_Unit">
        /// This control defines the unit of the transducer factor selected.
        /// 
        /// Valid Values:
        /// RSESCI_UNIT_DBM    (0)  - dBm
        /// RSESCI_UNIT_DBPW   (4)  - dBpW
        /// RSESCI_UNIT_DBUV   (5)  - dBuV
        /// RSESCI_UNIT_DBMV   (6)  - dBmV
        /// RSESCI_UNIT_DBUA   (7)  - dBuA
        /// RSESCI_UNIT_DBPT   (8)  - dBpT
        /// RSESCI_UNIT_DBUV_M (9)  - dBuV/MHz
        /// RSESCI_UNIT_DBUA_M (10) - dBuA/MHz
        /// RSESCI_UNIT_DB     (11) - dB
        /// 
        /// Default Value: RSESCI_UNIT_DB (11)
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetTransducerFactorUnit(int Window, int Transducer_Factor_Unit)
        {
            int pInvokeResult = PInvoke.SetTransducerFactorUnit(this._handle, Window, Transducer_Factor_Unit);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the unit of the transducer factor selected.
        /// 
        /// Note(s):
        /// 
        /// Prior to this function, the function Set Transducer Factor Name (rsesci_SetTransducerFactorName) must be called.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]CORRection:TRANsducer:UNIT?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Transducer_Factor_Unit">
        /// This control returns the unit of the transducer factor selected.
        /// 
        /// Valid Values:
        /// RSESCI_UNIT_DBM    (0)  - dBm
        /// RSESCI_UNIT_DBPW   (4)  - dBpW
        /// RSESCI_UNIT_DBUV   (5)  - dBuV
        /// RSESCI_UNIT_DBMV   (6)  - dBmV
        /// RSESCI_UNIT_DBUA   (7)  - dBuA
        /// RSESCI_UNIT_DBPT   (8)  - dBpT
        /// RSESCI_UNIT_DBUV_M (9)  - dBuV/MHz
        /// RSESCI_UNIT_DBUA_M (10) - dBuA/MHz
        /// RSESCI_UNIT_DB     (11) - dB
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetTransducerFactorUnit(int Window, out int Transducer_Factor_Unit)
        {
            int pInvokeResult = PInvoke.GetTransducerFactorUnit(this._handle, Window, out Transducer_Factor_Unit);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines whether the frequency scaling of the transducer factor is linear or logarithmic.
        /// 
        /// Note(s):
        /// 
        /// Prior to this function, the function Set Transducer Factor Name (rsesci_SetTransducerFactorName) must be called.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]CORRection:TRANsducer:SCALing LINear | LOGarithmic
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Transducer_Factor_Interpolation">
        /// This control defines whether the frequency scaling of the transducer factor is linear or logarithmic.
        /// 
        /// Valid Values:
        /// RSESCI_LIN (0) - Linear
        /// RSESCI_LOG (1) - Logarithmic
        /// 
        /// Default Value: RSESCI_LIN (0)
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetTransducerFactorInterpolation(int Window, int Transducer_Factor_Interpolation)
        {
            int pInvokeResult = PInvoke.SetTransducerFactorInterpolation(this._handle, Window, Transducer_Factor_Interpolation);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns whether the frequency scaling of the transducer factor is linear or logarithmic.
        /// 
        /// Note(s):
        /// 
        /// Prior to this function, the function Set Transducer Factor Name (rsesci_SetTransducerFactorName) must be called.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]CORRection:TRANsducer:SCALing?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Transducer_Factor_Interpolation">
        /// This control returns whether the frequency scaling of the transducer factor is linear or logarithmic.
        /// 
        /// Valid Values:
        /// RSESCI_LIN (0) - Linear
        /// RSESCI_LOG (1) - Logarithmic
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetTransducerFactorInterpolation(int Window, out int Transducer_Factor_Interpolation)
        {
            int pInvokeResult = PInvoke.GetTransducerFactorInterpolation(this._handle, Window, out Transducer_Factor_Interpolation);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the comment for the selected transducer factor.
        /// 
        /// Note(s):
        /// 
        /// Prior to this function, the function Set Transducer Factor Name (rsesci_SetTransducerFactorName) must be called.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]CORRection:TRANsducer:COMMent &lt;string&gt;
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Comment">
        /// This control defines the comment for the selected transducer factor.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: ""
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetTransducerFactorComment(int Window, string Comment)
        {
            int pInvokeResult = PInvoke.SetTransducerFactorComment(this._handle, Window, Comment);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the comment for the selected transducer factor.
        /// 
        /// Note(s):
        /// 
        /// Prior to this function, the function Set Transducer Factor Name (rsesci_SetTransducerFactorName) must be called.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]CORRection:TRANsducer:COMMent?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Comment">
        /// This control returns the comment for the selected transducer factor.
        /// 
        /// Note(s):
        /// 
        /// (1) The array should consist of at least 256 elements.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetTransducerFactorComment(int Window, System.Text.StringBuilder Comment)
        {
            int pInvokeResult = PInvoke.GetTransducerFactorComment(this._handle, Window, Comment);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the reference values of the transducer factor selected. These values are entered as a sequence of frequency/level pairs. The frequencies must be sent in ascending order.
        /// 
        /// Note(s):
        /// 
        /// (1) The level values are sent as dimensionless numbers; the unit is specified by means of the command SENS:CORR:TRAN:UNIT.
        /// 
        /// (2) Prior to this function, the function Set Transducer Factor Name (rsesci_SetTransducerFactorName) must be called.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]CORRection:TRANsducer:DATA &lt;freq&gt;,&lt;level&gt;..
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Array_Size">
        /// This control defines the size of array in arguments 'Frequency Values' and 'Level Values'.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <param name="Frequency_Values">
        /// Frequency values of the transducer factor selected. The frequencies must be sent in ascending order.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: none
        /// 
        /// </param>
        /// <param name="Level_Values">
        /// Level values of the transducer factor selected. The level values are sent as dimensionless numbers.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: none
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetTransducerFactorValues(int Window, int Array_Size, double[] Frequency_Values, double[] Level_Values)
        {
            int pInvokeResult = PInvoke.SetTransducerFactorValues(this._handle, Window, Array_Size, Frequency_Values, Level_Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the reference values of the transducer factor selected. These values are entered as a sequence of frequency/level pairs.
        /// 
        /// Note(s):
        /// 
        /// (1) The level values are sent as dimensionless numbers; the unit is specified by means of the command SENS:CORR:TRAN:UNIT.
        /// 
        /// (2) Prior to this function, the function Set Transducer Factor Name (rsesci_SetTransducerFactorName) must be called.
        /// 
        /// Mode(s):
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]CORRection:TRANsducer:DATA?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Number_of_Results">
        /// Returns the number of results taken from the instrument.
        /// 
        /// </param>
        /// <param name="Frequency_Values">
        /// Frequency values of the transducer factor selected.
        /// 
        /// Note(s):
        /// 
        /// (1) The array should consist of at least 50 elements.
        /// 
        /// </param>
        /// <param name="Level_Values">
        /// Level values of the transducer factor selected. The level values are sent as dimensionless numbers.
        /// 
        /// Note(s):
        /// 
        /// (1) The array should consist of at least 50 elements.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetTransducerFactorValues(int Window, out int Number_of_Results, double[] Frequency_Values, double[] Level_Values)
        {
            int pInvokeResult = PInvoke.GetTransducerFactorValues(this._handle, Window, out Number_of_Results, Frequency_Values, Level_Values);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the unit of the selected transducer sets. When assigning transducer factors to the set, only factors which are compatible to the selected unit, i. e. factors with the same unit or the unit dB, are allowed.
        /// 
        /// Note(s):
        /// 
        /// Prior to this function, the function Set Transducer Set Name (rsesci_SetTransducerSetName) must be called.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]CORRection:TSET:UNIT 'DB' | 'DBM' | 'DBUV' | 'DBUV/M' | 'DBUA' | 'DBUA/M' | 'DBPW' | 'DBPT'
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Transducer_Set_Unit">
        /// This control defines the unit of the transducer set selected.
        /// 
        /// Valid Values:
        /// RSESCI_UNIT_DBM    (0)  - dBm
        /// RSESCI_UNIT_DBPW   (4)  - dBpW
        /// RSESCI_UNIT_DBUV   (5)  - dBuV
        /// RSESCI_UNIT_DBUA   (7)  - dBuA
        /// RSESCI_UNIT_DBPT   (8)  - dBpT
        /// RSESCI_UNIT_DBUV_M (9)  - dBuV/MHz
        /// RSESCI_UNIT_DBUA_M (10) - dBuA/MHz
        /// RSESCI_UNIT_DB     (11) - dB
        /// 
        /// Default Value: RSESCI_UNIT_DB (11)
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetTransducerSetUnit(int Window, int Transducer_Set_Unit)
        {
            int pInvokeResult = PInvoke.SetTransducerSetUnit(this._handle, Window, Transducer_Set_Unit);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the unit of the selected transducer sets. When assigning transducer factors to the set, only factors which are compatible to the selected unit, i. e. factors with the same unit or the unit dB, are allowed.
        /// 
        /// Note(s):
        /// 
        /// Prior to this function, the function Set Transducer Set Name (rsesci_SetTransducerSetName) must be called.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]CORRection:TSET:UNIT?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Transducer_Set_Unit">
        /// This control returns the unit of the transducer set selected.
        /// 
        /// Valid Values:
        /// RSESCI_UNIT_DBM    (0)  - dBm
        /// RSESCI_UNIT_DBPW   (4)  - dBpW
        /// RSESCI_UNIT_DBUV   (5)  - dBuV
        /// RSESCI_UNIT_DBUA   (7)  - dBuA
        /// RSESCI_UNIT_DBPT   (8)  - dBpT
        /// RSESCI_UNIT_DBUV_M (9)  - dBuV/MHz
        /// RSESCI_UNIT_DBUA_M (10) - dBuA/MHz
        /// RSESCI_UNIT_DB     (11) - dB
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetTransducerSetUnit(int Window, out int Transducer_Set_Unit)
        {
            int pInvokeResult = PInvoke.GetTransducerSetUnit(this._handle, Window, out Transducer_Set_Unit);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines if the sweep is to be stopped on changeover from range to another.
        /// 
        /// Note(s):
        /// 
        /// Prior to this function, the function Set Transducer Set Name (rsesci_SetTransducerSetName) must be called.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]CORRection:TSET:BREak ON | OFF
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Transducer_Set_Break">
        /// This control defines if the sweep is to be stopped on changeover from range to another.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetTransducerSetBreak(int Window, bool Transducer_Set_Break)
        {
            int pInvokeResult = PInvoke.SetTransducerSetBreak(this._handle, Window, System.Convert.ToUInt16(Transducer_Set_Break));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns if the sweep is to be stopped on changeover from range to another.
        /// 
        /// Note(s):
        /// 
        /// Prior to this function, the function Set Transducer Set Name (rsesci_SetTransducerSetName) must be called.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]CORRection:TSET:BREak?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Transducer_Set_Break">
        /// This control returns if the sweep is to be stopped on changeover from range to another.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetTransducerSetBreak(int Window, out bool Transducer_Set_Break)
        {
            ushort Transducer_Set_BreakAsUShort;
            int pInvokeResult = PInvoke.GetTransducerSetBreak(this._handle, Window, out Transducer_Set_BreakAsUShort);
            Transducer_Set_Break = System.Convert.ToBoolean(Transducer_Set_BreakAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the comment for the selected transducer set.
        /// 
        /// Note(s):
        /// 
        /// Prior to this function, the function Set Transducer Set Name (rsesci_SetTransducerSetName) must be called.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]CORRection:TSET:COMMent &lt;string&gt;
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Comment">
        /// This control defines the comment for the selected transducer set.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: ""
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetTransducerSetComment(int Window, string Comment)
        {
            int pInvokeResult = PInvoke.SetTransducerSetComment(this._handle, Window, Comment);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the comment for the selected transducer set.
        /// 
        /// Note(s):
        /// 
        /// Prior to this function, the function Set Transducer Set Name (rsesci_SetTransducerSetName) must be called.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]CORRection:TSET:COMMent?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Comment">
        /// This returns defines the comment for the selected transducer set.
        /// 
        /// Note(s):
        /// 
        /// (1) The array should consist of at least 256 elements.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetTransducerSetComment(int Window, System.Text.StringBuilder Comment)
        {
            int pInvokeResult = PInvoke.GetTransducerSetComment(this._handle, Window, Comment);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines a partial range of the selected transducer set. The partial range is determined by its start and stop frequencies plus a list of names of the assigned transducer factors. The ranges 1 to 10 must be sent in ascending order.
        /// 
        /// Note(s):
        /// 
        /// (1) The individual subranges have to be linked without a gap. That is why the start frequency is already defined from the second subrange (= stop frequency of previous range).
        /// 
        /// (2) Prior to this function, the function Set Transducer Set Name (rsesci_SetTransducerSetName) must be called.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]CORRection:TSET:RANGe&lt;1...10&gt; &lt;freq&gt;,&lt;freq&gt;,&lt;name&gt;..
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Range">
        /// Range number.
        /// 
        /// Valid Values:
        /// 1 to 10
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Start_Frequency">
        /// Sets start frequency of the subrange.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: none
        /// </param>
        /// <param name="Stop_Frequency">
        /// Sets stop frequency of the subrange.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: none
        /// </param>
        /// <param name="Factors">
        /// Selection of the transducer factors for the subrange. List of names separated by commas for the assigned transducer factors.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: ""
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetTransducerSetValues(int Window, int Range, double Start_Frequency, double Stop_Frequency, string Factors)
        {
            int pInvokeResult = PInvoke.SetTransducerSetValues(this._handle, Window, Range, Start_Frequency, Stop_Frequency, Factors);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns a partial range of the selected transducer set. The partial range is determined by its start and stop frequencies plus a list of names of the assigned transducer factors.
        /// 
        /// Note(s):
        /// 
        /// Prior to this function, the function Set Transducer Set Name (rsesci_SetTransducerSetName) must be called.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]CORRection:TSET:RANGe&lt;1...10&gt;?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Range">
        /// Range number.
        /// 
        /// Valid Values:
        /// 1 to 10
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="Start_Frequency">
        /// Returns start frequency of the subrange.
        /// 
        /// </param>
        /// <param name="Stop_Frequency">
        /// Returns stop frequency of the subrange.
        /// </param>
        /// <param name="Factors">
        /// Returns the transducer factors for the subrange. List of names for the assigned transducer factors.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetTransducerSetValues(int Window, int Range, out double Start_Frequency, out double Stop_Frequency, System.Text.StringBuilder Factors)
        {
            int pInvokeResult = PInvoke.GetTransducerSetValues(this._handle, Window, Range, out Start_Frequency, out Stop_Frequency, Factors);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function changes the IEC/IEEe-bus address of the unit.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// SYSTem:COMMunicate:GPIB[:SELF]:ADDRess
        /// </summary>
        /// <param name="GPIB_Address">
        /// Changes the IEC/IEEe-bus address of the unit.
        /// 
        /// Valid Range: 0 to 30
        /// 
        /// Default Value: 20
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetGPIBAddress(int GPIB_Address)
        {
            int pInvokeResult = PInvoke.SetGPIBAddress(this._handle, GPIB_Address);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the IEC/IEEe-bus address of the unit.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// SYSTem:COMMunicate:GPIB[:SELF]:ADDRess?
        /// </summary>
        /// <param name="GPIB_Address">
        /// Returns the IEC/IEEe-bus address of the unit.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetGPIBAddress(out int GPIB_Address)
        {
            int pInvokeResult = PInvoke.GetGPIBAddress(this._handle, out GPIB_Address);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the serial interface parameters.
        /// 
        /// The following parameters can be configured:
        /// 
        ///   - Baud rate data transmission rate
        ///   - Bits number of data bits
        ///   - Parity bit parity check
        ///   - Stop bits number of stop bits
        ///   - HW-Handshake hardware handshake protocol
        ///   - SW-Handshake software handshake protocol
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// SYSTem:COMMunicate:SERial:CONTrol:DTR IBFull | OFF
        /// SYSTem:COMMunicate:SERial:CONTrol:RTS IBFull | OFF
        /// SYSTem:COMMunicate:SERial[:RECeive]:BAUD 110 | 300 | 600 | 1200 | 2400 | 9600 | 19200
        /// SYSTem:COMMunicate:SERial[:RECeive]:BITS 7 | 8
        /// SYSTem:COMMunicate:SERial[:RECeive]:PARity[:TYPE] EVEN | ODD | NONE
        /// SYSTem:COMMunicate:SERial[:RECeive]:SBITs 1|2
        /// SYSTem:COMMunicate:SERial[:RECeive]:PACE XON | NONE
        /// </summary>
        /// <param name="Baud_Rate">
        /// This control sets the transmission speed for the serial interface (COM).
        /// 
        /// Valid Values: 110, 300, 600, 1200, 2400, 4800, 9600, 19200
        /// 
        /// Default Value: 9600
        /// 
        /// </param>
        /// <param name="Data_Bits">
        /// This control defines the number of data bits per data word for the serial interface (COM).
        /// 
        /// Valid Values: 7, 8
        /// 
        /// Default Value: 8
        /// 
        /// </param>
        /// <param name="Stop_Bits">
        /// This control defines the number of stop bits per data word for the serial interface (COM).
        /// 
        /// Valid Values: 1, 2
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <param name="Hardware_Handshake">
        /// This control switch the hardware handshake procedure for the serial interface off (OFF) or on (IBFull).
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// </param>
        /// <param name="Software_Handshake">
        /// This control switches on or off the software handshake for the serial interface.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// </param>
        /// <param name="Parity">
        /// This control defines the parity check for the serial interface (COM).
        /// 
        /// Valid Range:
        /// RSESCI_NONE (0) - None
        /// RSESCI_EVEN (1) - Even
        /// RSESCI_ODD  (2) - Odd
        /// 
        /// Default Value: RSESCI_NONE (0)
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetSerialInterfaceConfiguration(int Baud_Rate, int Data_Bits, int Stop_Bits, bool Hardware_Handshake, bool Software_Handshake, int Parity)
        {
            int pInvokeResult = PInvoke.SetSerialInterfaceConfiguration(this._handle, Baud_Rate, Data_Bits, Stop_Bits, System.Convert.ToUInt16(Hardware_Handshake), System.Convert.ToUInt16(Software_Handshake), Parity);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the settings of serial interface parameters.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// SYSTem:COMMunicate:SERial:CONTrol:DTR?
        /// SYSTem:COMMunicate:SERial:CONTrol:RTS?
        /// SYSTem:COMMunicate:SERial[:RECeive]:BAUD?
        /// SYSTem:COMMunicate:SERial[:RECeive]:BITS?
        /// SYSTem:COMMunicate:SERial[:RECeive]:PARity[:TYPE]?
        /// SYSTem:COMMunicate:SERial[:RECeive]:SBITs?
        /// SYSTem:COMMunicate:SERial[:RECeive]:PACE?
        /// </summary>
        /// <param name="Baud_Rate">
        /// This control returns the transmission speed for the serial interface (COM).
        /// 
        /// </param>
        /// <param name="Data_Bits">
        /// This control returns the number of data bits per data word for the serial interface (COM).
        /// 
        /// </param>
        /// <param name="Stop_Bits">
        /// This control returns the number of stop bits per data word for the serial interface (COM).
        /// 
        /// </param>
        /// <param name="Hardware_Handshake">
        /// This control returns if the hardware handshake procedure for the serial interface is off (OFF) or on (IBFull).
        /// 
        /// </param>
        /// <param name="Software_Handshake">
        /// This control returns if the software handshake for the serial interface is on or off.
        /// 
        /// </param>
        /// <param name="Parity">
        /// This control returns the parity check for the serial interface (COM).
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetSerialInterfaceConfiguration(out int Baud_Rate, out int Data_Bits, out int Stop_Bits, out bool Hardware_Handshake, out bool Software_Handshake, out int Parity)
        {
            ushort Hardware_HandshakeAsUShort;
            ushort Software_HandshakeAsUShort;
            int pInvokeResult = PInvoke.GetSerialInterfaceConfiguration(this._handle, out Baud_Rate, out Data_Bits, out Stop_Bits, out Hardware_HandshakeAsUShort, out Software_HandshakeAsUShort, out Parity);
            Hardware_Handshake = System.Convert.ToBoolean(Hardware_HandshakeAsUShort);
            Software_Handshake = System.Convert.ToBoolean(Software_HandshakeAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the internal clock. The sequence of entry is hour, minute, second.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// SYSTem:TIME
        /// </summary>
        /// <param name="Hour">
        /// Sets the internal clock hour.
        /// 
        /// Valid Range: 0 to 23
        /// 
        /// Default Value: 0
        /// </param>
        /// <param name="Minute">
        /// Sets the internal clock minutes.
        /// 
        /// Valid Range: 0 to 59
        /// 
        /// Default: 0
        /// </param>
        /// <param name="Second">
        /// Sets the internal clock seconds.
        /// 
        /// Valid Range: 0 to 59
        /// 
        /// Default Value: 0
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetSystemTime(int Hour, int Minute, int Second)
        {
            int pInvokeResult = PInvoke.SetSystemTime(this._handle, Hour, Minute, Second);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the internal clock. The sequence of entry is hour, minute, second.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// SYSTem:TIME?
        /// </summary>
        /// <param name="Hour">
        /// Returns the internal clock hour.
        /// 
        /// </param>
        /// <param name="Minute">
        /// Returns the internal clock minutes.
        /// 
        /// </param>
        /// <param name="Second">
        /// Returns the internal clock seconds.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetSystemTime(out int Hour, out int Minute, out int Second)
        {
            int pInvokeResult = PInvoke.GetSystemTime(this._handle, out Hour, out Minute, out Second);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function is used to enter the date for the internal calendar. The sequence of entry is year, month, day.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// SYSTem:DATE
        /// </summary>
        /// <param name="Day">
        /// Sets the calendar day.
        /// 
        /// Valid Range: 1 to 31
        /// 
        /// Default Value: 21
        /// </param>
        /// <param name="Month">
        /// Sets calendar month.
        /// 
        /// Valid Values:
        /// 1 - January
        /// 2 - February
        /// 3 - March
        /// 4 - April
        /// 5 - May
        /// 6 - June
        /// 7 - July
        /// 8 - August
        /// 9 - September
        /// 10 - October
        /// 11 - November
        /// 12 - December
        /// 
        /// Default Value: 6
        /// 
        /// </param>
        /// <param name="Year">
        /// Sets the year for the date function.
        /// 
        /// Valid Range: 1980 to 2099
        /// 
        /// Default Value: 2004
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetSystemDate(int Day, int Month, int Year)
        {
            int pInvokeResult = PInvoke.SetSystemDate(this._handle, Day, Month, Year);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the date for the internal calendar.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// SYSTem:DATE?
        /// </summary>
        /// <param name="Day">
        /// Returns the calendar day.
        /// 
        /// </param>
        /// <param name="Month">
        /// Returns calendar month.
        /// 
        /// Valid Values:
        /// 1 - January
        /// 2 - February
        /// 3 - March
        /// 4 - April
        /// 5 - May
        /// 6 - June
        /// 7 - July
        /// 8 - August
        /// 9 - September
        /// 10 - October
        /// 11 - November
        /// 12 - December
        /// 
        /// </param>
        /// <param name="Year">
        /// Returns the year for the date function.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetSystemDate(out int Day, out int Month, out int Year)
        {
            int pInvokeResult = PInvoke.GetSystemDate(this._handle, out Day, out Month, out Year);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the volume of the built-in loudspeaker for demodulated signals.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// SYSTem:SPEaker:VOLume
        /// </summary>
        /// <param name="Speaker_Volume">
        /// Sets the volume of the built-in loudspeaker for demodulated signals.
        /// 
        /// Valid Range: 0.0 to 100.0 PCT
        /// 
        /// Default Value: 50.0 PCT
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetSpeakerVolume(double Speaker_Volume)
        {
            int pInvokeResult = PInvoke.SetSpeakerVolume(this._handle, Speaker_Volume);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the volume of the built-in loudspeaker for demodulated signals.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// SYSTem:SPEaker:VOLume?
        /// </summary>
        /// <param name="Speaker_Volume">
        /// Returns the volume of the built-in loudspeaker for demodulated signals.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetSpeakerVolume(out double Speaker_Volume)
        {
            int pInvokeResult = PInvoke.GetSpeakerVolume(this._handle, out Speaker_Volume);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function activates or deactivates the display of the front panel keys on the screen.
        /// 
        /// With the display activated, the instrument can be operated on the screen using the mouse by pressing the corresponding buttons. This may be useful if the instrument is operated in a detached station by means of a remote program such as PCANYWHERE.
        /// 
        /// Note(s):
        /// 
        /// (1) With the display of the front panel keys activated, the screen resolution of the unit is set to 1024x768. Thus, only a section of the whole screen is visible on the internal LCD display, which will be moved by mouse moves.
        /// 
        /// (2) For a full display of the user interface, an external monitor has to be connected to the rear panel.
        /// 
        /// (3) When the front panel display is deactivated, the original screen resolution is restored.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// SYSTem:DISPlay:FPANel ON | OFF
        /// </summary>
        /// <param name="Soft_Frontpanel">
        /// This control activates or deactivates the display of the front panel keys on the screen.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetSoftFrontpanel(bool Soft_Frontpanel)
        {
            int pInvokeResult = PInvoke.SetSoftFrontpanel(this._handle, System.Convert.ToUInt16(Soft_Frontpanel));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns if the display of the front panel keys on the screen is activated or deactivated.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// SYSTem:DISPlay:FPANel?
        /// </summary>
        /// <param name="Soft_Frontpanel">
        /// This control returns if the display of the front panel keys on the screen is activated or deactivated.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetSoftFrontpanel(out bool Soft_Frontpanel)
        {
            ushort Soft_FrontpanelAsUShort;
            int pInvokeResult = PInvoke.GetSoftFrontpanel(this._handle, out Soft_FrontpanelAsUShort);
            Soft_Frontpanel = System.Convert.ToBoolean(Soft_FrontpanelAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects whether user port is used as output or input.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// INPut&lt;1|2&gt;:UPORt:STATe ON | OFF
        /// OUTPut:UPORt:STATe ON | OFF
        /// </summary>
        /// <param name="User_Port">
        /// Selects whether user port is used as output or input.
        /// 
        /// Valid Values:
        /// RSESCI_INPUT  (0) - Input
        /// RSESCI_OUTPUT (1) - Output
        /// 
        /// Default Value: RSESCI_INPUT (0)
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetUserPort(int User_Port)
        {
            int pInvokeResult = PInvoke.SetUserPort(this._handle, User_Port);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns whether user port is used as output or input.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// INPut&lt;1|2&gt;:UPORt:STATe?
        /// OUTPut:UPORt:STATe?
        /// </summary>
        /// <param name="User_Port">
        /// Returns whether user port is used as output or input.
        /// 
        /// Valid Values:
        /// RSESCI_INPUT  (0) - Input
        /// RSESCI_OUTPUT (1) - Output
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetUserPort(out int User_Port)
        {
            int pInvokeResult = PInvoke.GetUserPort(this._handle, out User_Port);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the control lines of the user ports.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// INPut&lt;1|2&gt;:UPORt[:VALue]?
        /// </summary>
        /// <param name="User_Port_Value">
        /// Returns the state of control lines of the user ports. In manual operation, the control lines are represented by softkeys PORT 0 to 7.
        /// 
        /// Note(s):
        /// 
        /// (1) The array should consist of at least 8 elements.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int ReadUserPortValue(ushort[] User_Port_Value)
        {
            int pInvokeResult = PInvoke.ReadUserPortValue(this._handle, User_Port_Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the control lines of the user ports. In manual operation, the control lines are represented by softkeys PORT 0 to 7.
        /// 
        /// The user port is written to with the given binary pattern. If the user port is programmed to Input instead of Output, the output value is temporarily stored.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// OUTPut:UPORt[:VALue]
        /// </summary>
        /// <param name="User_Port_Value">
        /// Sets the control lines of the user ports. In manual operation, the control lines are represented by softkeys PORT 0 to 7.
        /// 
        /// Valid Values: VI_FALSE (0), VI_TRUE (1)
        /// 
        /// Default Value: none
        /// 
        /// Note(s):
        /// 
        /// The array should consist of at least 8 elements.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int WriteUserPortValue(ushort[] User_Port_Value)
        {
            int pInvokeResult = PInvoke.WriteUserPortValue(this._handle, User_Port_Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function toggles between the RF input on the front panel and the internal 128-MHz reference signal in the selected measurement window. The level of the 128-MHz signals can be also selected.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// DIAGnostic&lt;1|2&gt;:SERVice:INPut[:SELect] CALibration | RF
        /// DIAGnostic&lt;1|2&gt;:SERVice:CSOource[:POWer]
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Service_Source">
        /// Toggles between the RF input on the front panel and the internal 128-MHz reference signal in the selected measurement window.
        /// 
        /// Valid Values:
        /// RSESCI_RF_INPUT         (0) - RF Input
        /// RSESCI_INT_CALIB_SOURCE (1) - Internal Calibration Source
        /// 
        /// Default Value: RSESCI_RF_INPUT (0)
        /// 
        /// </param>
        /// <param name="Calibration_Source_Level">
        /// This control switches the level of the 128 MHz reference signal source between 0 dBm and -30 dBm in the selected measurement window.
        /// 
        /// Valid Range: -30.0 dBm to 0.0 dBm
        /// 
        /// Default Value: -30.0 dBm
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetServiceSource(int Window, int Service_Source, double Calibration_Source_Level)
        {
            int pInvokeResult = PInvoke.SetServiceSource(this._handle, Window, Service_Source, Calibration_Source_Level);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns whether the RF input on the front panel or the internal 128-MHz reference signal in the selected measurement window is used. The level of the 128-MHz signals is also queried.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// DIAGnostic&lt;1|2&gt;:SERVice:INPut[:SELect]?
        /// DIAGnostic&lt;1|2&gt;:SERVice:CSOource[:POWer]?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Service_Source">
        /// Returns whether the RF input on the front panel or the internal 128-MHz reference signal in the selected measurement window is used.
        /// 
        /// Valid Values:
        /// RSESCI_RF_INPUT         (0) - RF Input
        /// RSESCI_INT_CALIB_SOURCE (1) - Internal Calibration Source
        /// 
        /// </param>
        /// <param name="Calibration_Source_Level">
        /// This control returns the level of the 128 MHz reference signal source.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetServiceSource(int Window, out int Service_Source, out double Calibration_Source_Level)
        {
            int pInvokeResult = PInvoke.GetServiceSource(this._handle, Window, out Service_Source, out Calibration_Source_Level);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function toggles the calibration signal in the selected measurement window between pulsed and non-pulsed. The selection takes effect only if the RF input has been set to the internal reference signal using DIAG:SERV:INP CAL.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// DIAGnostic&lt;1|2&gt;:SERVice:INPut:PULSed[:STATe] ON | OFF
        /// DIAGnostic&lt;1|2&gt;:SERVice:INPut:PULSed:PRATe
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Pulse_Generator">
        /// This control toggles the calibration signal in the selected measurement window between pulsed and non-pulsed.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <param name="Pulse_Rate">
        /// Selects the pulse rate for the pulsed calibration signal in the selected measurement window.
        /// 
        /// Valid Values: 10 kHz, 100 kHz, 200 kHz, 500 kHz, 1 MHz, 2 MHz, 4 MHz and 8 MHz
        /// 
        /// Default Value: 1.0e6 Hz
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetServicePulseGenerator(int Window, bool Pulse_Generator, double Pulse_Rate)
        {
            int pInvokeResult = PInvoke.SetServicePulseGenerator(this._handle, Window, System.Convert.ToUInt16(Pulse_Generator), Pulse_Rate);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns if the calibration signal in the selected measurement window is pulsed or non-pulsed.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// DIAGnostic&lt;1|2&gt;:SERVice:INPut:PULSed[:STATe]?
        /// DIAGnostic&lt;1|2&gt;:SERVice:INPut:PULSed:PRATe?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Pulse_Generator">
        /// This control returns if the calibration signal in the selected measurement window is pulsed or non-pulsed.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// </param>
        /// <param name="Pulse_Rate">
        /// Returns the pulse rate of the pulsed calibration signal in the selected measurement window.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetServicePulseGenerator(int Window, out bool Pulse_Generator, out double Pulse_Rate)
        {
            ushort Pulse_GeneratorAsUShort;
            int pInvokeResult = PInvoke.GetServicePulseGenerator(this._handle, Window, out Pulse_GeneratorAsUShort, out Pulse_Rate);
            Pulse_Generator = System.Convert.ToBoolean(Pulse_GeneratorAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures rectangle calibration signal. This function takes effect only if the RF input has been set to the internal reference signal.
        /// 
        /// Remote-control command(s):
        /// DIAGnostic&lt;1|2&gt;:SERVice:INPut:RECTangle[:STATe] ON | OFF
        /// DIAGnostic&lt;1|2&gt;:SERVice:INPut:RECTangle:PRATe
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="State">
        /// Switches the rectangle calibration signal on or off in the selected measurement window.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <param name="Frequency">
        /// Sets the frequency of the rectangle calibration signal in the selected measurement window.
        /// 
        /// Valid Values:
        /// 5.0e3, 31.25e3, 50.0e3, 250.0e3, 500.0e3 Hz.
        /// 
        /// Default Value: 5.0e3 Hz
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int ServiceConfigureRectangleCalibrationSignal(int Window, bool State, double Frequency)
        {
            int pInvokeResult = PInvoke.ServiceConfigureRectangleCalibrationSignal(this._handle, Window, System.Convert.ToUInt16(State), Frequency);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the shifting of the 1st IF.
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SWEep:IF:SHIFt OFF | A | B | AUTO
        /// </summary>
        /// <param name="IF_Shift">
        /// This control selects the shifting of the 1st IF. Input signals at a frequency of half the 1st IF (in the frequency range of 2270 MHz to 2350 MHz) will reduce the dynamic range of the analyzer. This problem only occurs for low RF attenuation values. It can be overcome by shifting the 1st IF. The 1st IF shifting is automatically done for ACP measurements if center frequency ( = signal frequency) is in the range of 2270 MHz to 2350 MHz. The IF SHIFT setting is therefore ignored for ACP measurements.
        /// 
        /// Valid Values:
        /// RSESCI_VAL_IF_SHIFT_OFF  (0) - Off
        /// RSESCI_VAL_IF_SHIFT_A    (1) - A
        /// RSESCI_VAL_IF_SHIFT_B    (2) - B
        /// RSESCI_VAL_IF_SHIFT_AUTO (3) - Auto
        /// 
        /// Default Value: RSESCI_VAL_IF_SHIFT_OFF (0)
        /// 
        /// Note(s):
        /// 
        /// (1) Off: Normal operation.
        /// 
        /// (2) A: For signals from 2270MHz to 2310 MHz.
        /// 
        /// (3) B: For signals from 2310 MHz to 2350 MHz.
        /// 
        /// (4) Auto: Automatically selects the suitable 1st IF shifting.
        /// The signal frequency has to specified with control IF Shift Frequency.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int SetIFShift(int IF_Shift)
        {
            int pInvokeResult = PInvoke.SetIFShift(this._handle, IF_Shift);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the shifting of the 1st IF.
        /// 
        /// Remote-control command(s):
        /// [SENSe&lt;1|2&gt;:]SWEep:IF:SHIFt?
        /// </summary>
        /// <param name="IF_Shift">
        /// This control returns the shifting of the 1st IF.
        /// 
        /// Valid Values:
        /// RSESCI_VAL_IF_SHIFT_OFF  (0) - Off
        /// RSESCI_VAL_IF_SHIFT_A    (1) - A
        /// RSESCI_VAL_IF_SHIFT_B    (2) - B
        /// RSESCI_VAL_IF_SHIFT_AUTO (3) - Auto
        /// 
        /// Default Value: RSESCI_VAL_IF_SHIFT_OFF (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetIFShift(out int IF_Shift)
        {
            int pInvokeResult = PInvoke.GetIFShift(this._handle, out IF_Shift);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches the calibration signal to the I or Q path.
        /// 
        /// Note(s):
        /// 
        /// (1) High selects the positive and low the negative input. The negative input is only connected through at the Balanced setting. The calibration signals are always positive. An inverted signal therefore appears in the output data during feeding into the negative input (Low).
        /// 
        /// (2) The numeric suffix &lt;1|2&gt; has no meaning with this command.
        /// 
        /// Mode(s):
        /// IQBB
        /// 
        /// Remote-control command(s):
        /// DIAGnostic&lt;1|2&gt;:SERVice:IQ:CALibration:DESTination IHIGh | ILOW | QHIGh | QLOW
        /// </summary>
        /// <param name="Signal_Destination">
        /// This control switches the calibration signal to the I or Q path.
        /// 
        /// Valid Values:
        /// RSESCI_IQ_CAL_IHIG  (0) - Positive I Path
        /// RSESCI_IQ_CAL_ILOW  (1) - Negative I Path
        /// RSESCI_IQ_CAL_QHIG  (2) - Positive Q Path
        /// RSESCI_IQ_CAL_QLOW  (3) - Negative Q Path
        /// 
        /// Default Value: RSESCI_IQ_CAL_IHIG (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetCalibrationSignalDestination(int Signal_Destination)
        {
            int pInvokeResult = PInvoke.SetCalibrationSignalDestination(this._handle, Signal_Destination);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the calibration signal to the I or Q path.
        /// 
        /// Note(s):
        /// 
        /// (1) The numeric suffix &lt;1|2&gt; has no meaning with this command.
        /// 
        /// Mode(s):
        /// IQBB
        /// 
        /// Remote-control command(s):
        /// DIAGnostic&lt;1|2&gt;:SERVice:IQ:CALibration:DESTination?
        /// </summary>
        /// <param name="Signal_Destination">
        /// This control returns the calibration signal to the I or Q path.
        /// 
        /// Valid Values:
        /// RSESCI_IQ_CAL_IHIG  (0) - Positive I Path
        /// RSESCI_IQ_CAL_ILOW  (1) - Negative I Path
        /// RSESCI_IQ_CAL_QHIG  (2) - Positive Q Path
        /// RSESCI_IQ_CAL_QLOW  (3) - Negative Q Path
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetCalibrationSignalDestination(out int Signal_Destination)
        {
            int pInvokeResult = PInvoke.GetCalibrationSignalDestination(this._handle, out Signal_Destination);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the voltage for the DC Cal signal in Volt.
        /// 
        /// Note(s):
        /// 
        /// (1) The numeric suffix &lt;1|2&gt; has no meaning with this command.
        /// 
        /// Mode(s):
        /// IQBB
        /// 
        /// Remote-control command(s):
        /// DIAGnostic&lt;1|2&gt;:SERVice:IQ:CALibration:DC 0 | 0.1 | 0.178 | 0.316 | 0.562 | 1.0
        /// </summary>
        /// <param name="Signal_Voltage">
        /// This control selects the voltage for the DC Cal signal in Volt.
        /// 
        /// Valid Values:
        /// RSESCI_IQ_CAL_VOLT_0     (0) - 0
        /// RSESCI_IQ_CAL_VOLT_0_1   (1) - 0.1
        /// RSESCI_IQ_CAL_VOLT_0_178 (2) - 0.178
        /// RSESCI_IQ_CAL_VOLT_0_316 (3) - 0.316
        /// RSESCI_IQ_CAL_VOLT_0_562 (4) - 0.562
        /// RSESCI_IQ_CAL_VOLT_1_0   (5) - 1.0
        /// 
        /// Default Value: RSESCI_IQ_CAL_VOLT_0 (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetCalibrationSignalVoltage(int Signal_Voltage)
        {
            int pInvokeResult = PInvoke.SetCalibrationSignalVoltage(this._handle, Signal_Voltage);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the voltage for the DC Cal signal in Volt.
        /// 
        /// Note(s):
        /// 
        /// (1) The numeric suffix &lt;1|2&gt; has no meaning with this command.
        /// 
        /// Mode(s):
        /// IQBB
        /// 
        /// Remote-control command(s):
        /// DIAGnostic&lt;1|2&gt;:SERVice:IQ:CALibration:DC?
        /// </summary>
        /// <param name="Signal_Voltage">
        /// This control returns the voltage for the DC Cal signal in Volt.
        /// 
        /// Valid Values:
        /// RSESCI_IQ_CAL_VOLT_0     (0) - 0
        /// RSESCI_IQ_CAL_VOLT_0_1   (1) - 0.1
        /// RSESCI_IQ_CAL_VOLT_0_178 (2) - 0.178
        /// RSESCI_IQ_CAL_VOLT_0_316 (3) - 0.316
        /// RSESCI_IQ_CAL_VOLT_0_562 (4) - 0.562
        /// RSESCI_IQ_CAL_VOLT_1_0   (5) - 1.0
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetCalibrationSignalVoltage(out int Signal_Voltage)
        {
            int pInvokeResult = PInvoke.GetCalibrationSignalVoltage(this._handle, out Signal_Voltage);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the frequency of the Pulse Cal signal.
        /// 
        /// Note(s):
        /// 
        /// (1) The numeric suffix &lt;1|2&gt; has no meaning with this command.
        /// 
        /// Mode(s):
        /// IQBB
        /// 
        /// Remote-control command(s):
        /// DIAGnostic&lt;1|2&gt;:SERVice:IQ:CALibration:PULSe:PRATe 10KHz | 62.5KHz | 80KHz | 100KHz | 102.4KHz | 200KHz | 500KHz | 1MHz | 2MHz | 4MHz
        /// </summary>
        /// <param name="Pulse_Frequency">
        /// This control sets the frequency of the Pulse Cal signal.
        /// 
        /// Valid Values:
        /// RSESCI_IQ_CAL_FREQ_10KHZ   (0) - 10 kHz
        /// RSESCI_IQ_CAL_FREQ_62KHZ   (1) - 62.5 kHz
        /// RSESCI_IQ_CAL_FREQ_80KHZ   (2) - 80 kHz
        /// RSESCI_IQ_CAL_FREQ_100KHZ  (3) - 100 kHz
        /// RSESCI_IQ_CAL_FREQ_102KHZ  (4) - 102.4 kHz
        /// RSESCI_IQ_CAL_FREQ_200KHZ  (5) - 200 kHz
        /// RSESCI_IQ_CAL_FREQ_500KHZ  (6) - 500 kHz
        /// RSESCI_IQ_CAL_FREQ_1MHZ    (7) - 1 MHz
        /// RSESCI_IQ_CAL_FREQ_2MHZ    (8) - 2 MHz
        /// RSESCI_IQ_CAL_FREQ_4MHZ    (9) - 4 MHz
        /// 
        /// Default Value: RSESCI_IQ_CAL_FREQ_62KHZ (1)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetCalibrationSignalPulseFrequency(int Pulse_Frequency)
        {
            int pInvokeResult = PInvoke.SetCalibrationSignalPulseFrequency(this._handle, Pulse_Frequency);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function gets the frequency of the Pulse Cal signal.
        /// 
        /// Note(s):
        /// 
        /// (1) The numeric suffix &lt;1|2&gt; has no meaning with this command.
        /// 
        /// Mode(s):
        /// IQBB
        /// 
        /// Remote-control command(s):
        /// DIAGnostic&lt;1|2&gt;:SERVice:IQ:CALibration:PULSe:PRATe?
        /// </summary>
        /// <param name="Pulse_Frequency">
        /// This control gets the frequency of the Pulse Cal signal.
        /// 
        /// Valid Values:
        /// RSESCI_IQ_CAL_FREQ_10KHZ   (0) - 10 kHz
        /// RSESCI_IQ_CAL_FREQ_62KHZ   (1) - 62.5 kHz
        /// RSESCI_IQ_CAL_FREQ_80KHZ   (2) - 80 kHz
        /// RSESCI_IQ_CAL_FREQ_100KHZ  (3) - 100 kHz
        /// RSESCI_IQ_CAL_FREQ_102KHZ  (4) - 102.4 kHz
        /// RSESCI_IQ_CAL_FREQ_200KHZ  (5) - 200 kHz
        /// RSESCI_IQ_CAL_FREQ_500KHZ  (6) - 500 kHz
        /// RSESCI_IQ_CAL_FREQ_1MHZ    (7) - 1 MHz
        /// RSESCI_IQ_CAL_FREQ_2MHZ    (8) - 2 MHz
        /// RSESCI_IQ_CAL_FREQ_4MHZ    (9) - 4 MHz
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetCalibrationSignalPulseFrequency(out int Pulse_Frequency)
        {
            int pInvokeResult = PInvoke.GetCalibrationSignalPulseFrequency(this._handle, out Pulse_Frequency);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the baseband signal source.
        /// 
        /// Note(s):
        /// 
        /// (1) The numeric suffix &lt;1|2&gt; has no meaning with this command.
        /// 
        /// Mode(s):
        /// IQBB
        /// 
        /// Remote-control command(s):
        /// DIAGnostic&lt;1|2&gt;:SERVice:IQ:INPut IQ | GND | CALDc | CALPulse
        /// </summary>
        /// <param name="Baseband_Source">
        /// This control selects the baseband signal source.
        /// 
        /// Valid Values:
        /// RSESCI_IQ_CAL_BBSRC_IQ   (0) - IQ
        /// RSESCI_IQ_CAL_BBSRC_GND  (1) - Ground
        /// RSESCI_IQ_CAL_BBSRC_CALD (2) - DC Cal Signal
        /// RSESCI_IQ_CAL_BBSRC_CALP (3) - Pulse Cal Signal
        /// 
        /// Default Value: RSESCI_IQ_CAL_BBSRC_IQ (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetCalibrationSignalBasebandSource(int Baseband_Source)
        {
            int pInvokeResult = PInvoke.SetCalibrationSignalBasebandSource(this._handle, Baseband_Source);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the baseband signal source.
        /// 
        /// Note(s):
        /// 
        /// (1) The numeric suffix &lt;1|2&gt; has no meaning with this command.
        /// 
        /// Mode(s):
        /// IQBB
        /// 
        /// Remote-control command(s):
        /// DIAGnostic&lt;1|2&gt;:SERVice:IQ:INPut?
        /// </summary>
        /// <param name="Baseband_Source">
        /// This control returns the baseband signal source.
        /// 
        /// Valid Values:
        /// RSESCI_IQ_CAL_BBSRC_IQ   (0) - IQ
        /// RSESCI_IQ_CAL_BBSRC_GND  (1) - Ground
        /// RSESCI_IQ_CAL_BBSRC_CALD (2) - DC Cal Signal
        /// RSESCI_IQ_CAL_BBSRC_CALP (3) - Pulse Cal Signal
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetCalibrationSignalBasebandSource(out int Baseband_Source)
        {
            int pInvokeResult = PInvoke.GetCalibrationSignalBasebandSource(this._handle, out Baseband_Source);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function enables or disables rectangle calibration signal. This function takes effect only if the RF input has been set to the internal reference signal.
        /// 
        /// Remote-control command(s):
        /// DIAGnostic&lt;1|2&gt;:SERVice:INPut:RECTangle[:STATe] ON | OFF
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="State">
        /// Switches the rectangle calibration signal on or off in the selected measurement window.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On 
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int SetServiceRectangleState(int Window, bool State)
        {
            int pInvokeResult = PInvoke.SetServiceRectangleState(this._handle, Window, System.Convert.ToUInt16(State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the state of rectangle calibration signal.
        /// 
        /// Remote-control command(s):
        /// DIAGnostic&lt;1|2&gt;:SERVice:INPut:RECTangle[:STATe]?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="State">
        /// Returns state of the rectangle calibration signal in the selected measurement window.
        /// 
        /// Valid Values: 
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetServiceRectangleState(int Window, out bool State)
        {
            ushort StateAsUShort;
            int pInvokeResult = PInvoke.GetServiceRectangleState(this._handle, Window, out StateAsUShort);
            State = System.Convert.ToBoolean(StateAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets rectangle calibration signal frequency.
        /// 
        /// Remote-control command(s):
        /// DIAGnostic&lt;1|2&gt;:SERVice:INPut:RECTangle:PRATe
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Frequency">
        /// Sets the frequency of the rectangle calibration signal in the selected measurement window.
        /// 
        /// Valid Values:
        /// 5.0e3, 31.25e3, 50.0e3, 250.0e3, 500.0e3 Hz.
        /// 
        /// Default Value: 5.0e3 Hz
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int SetServiceRectangleRate(int Window, double Frequency)
        {
            int pInvokeResult = PInvoke.SetServiceRectangleRate(this._handle, Window, Frequency);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries rectangle calibration signal frequency.
        /// 
        /// Remote-control command(s):
        /// DIAGnostic&lt;1|2&gt;:SERVice:INPut:RECTangle:PRATe?
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Frequency">
        /// Returns the frequency of the rectangle calibration signal in the selected measurement window.
        /// 
        /// Valid Values:
        /// 5.0e3, 31.25e3, 50.0e3, 250.0e3, 500.0e3 Hz
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsspecan_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value Meaning
        /// -------------------------------
        /// 0 Success
        /// Positive Values Warnings
        /// Negative Values Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex) Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF VISA Warnings
        /// 3FFC0000 to 3FFCFFFF VXIPnP Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF VISA Errors
        /// BFFC0000 to BFFCFFFF VXIPnP Driver Errors
        /// </returns>
        public int GetServiceRectangleRate(int Window, out double Frequency)
        {
            int pInvokeResult = PInvoke.GetServiceRectangleRate(this._handle, Window, out Frequency);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function starts a firmware update using the data set in the selected directory. Beforehand, the update files have to be stored in the following subdirectories using command MMEM:DATA:
        /// 
        /// Directory ... Content
        /// 
        ///   DISK1 ... _inst32i.ex_, _isdel.exe, _setup.dll, _sys1.cab, 
        ///             _user1.cab, data.tag, data1.cab, id.txt, lang.dat,
        ///             layout.bin, os.dat, Setup.exe, Setup.ini, setup.ins, 
        ///             setup.lid
        /// 
        ///   DISK2 ... data2.cab
        /// 
        ///   DISK3 ... data3.cab
        /// 
        ///   DISK4 ... data4.cab
        /// 
        ///   DISK5 ... data5.cab
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// SYSTem:FIRMware:UPDate
        /// </summary>
        /// <param name="Path">
        /// Selects the directory where the update files are stored.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: "D:\\USER\\FWUPDATE"
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int FirmwareUpdate(string Path)
        {
            int pInvokeResult = PInvoke.FirmwareUpdate(this._handle, Path);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function determines if the settings of the receiver after reset are compatible to the settings of the R&amp;S FSP (selection FSP). If Off selected, the receiver specific settings are active after a reset.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// SYSTem:PRESet:COMPatible FSP | OFF
        /// 
        /// </summary>
        /// <param name="Compatibility_Mode">
        /// Determines if the settings of the receiver after reset are compatible to the settings of the R&amp;S FSP (selection FSP).
        /// 
        /// Valid Values:
        /// RSESCI_OFF (0) - Off
        /// RSESCI_FSP (1) - FSP
        /// 
        /// Default Value: RSESCI_OFF (0)
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int CompatibilityMode(int Compatibility_Mode)
        {
            int pInvokeResult = PInvoke.CompatibilityMode(this._handle, Compatibility_Mode);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function stores the current device settings in a series of files which have the indicated file name, but different extensions. The file name includes indication of the path and may also include the drive name. The path name complies with DOS conventions.
        /// 
        /// Note(s):
        /// 
        /// A list of the extensions used is contained under MMEMory:LOAD:STATe.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// MMEMory:COMMent &lt;string&gt;
        /// MMEMory:STORe&lt;1|2&gt;:STATe 1,&lt;file_name&gt;
        /// </summary>
        /// <param name="File_Name">
        /// Sets the file name without extension, include indication of the path and may also include the drive name. The path name complies with DOS conventions.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: "TEST"
        /// 
        /// </param>
        /// <param name="Comment">
        /// Defines a comment for a device setting to be stored.
        /// 
        /// Valid Range: up to 60 characters
        /// 
        /// Default Value: ""
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int DataSetSave(string File_Name, string Comment)
        {
            int pInvokeResult = PInvoke.DataSetSave(this._handle, File_Name, Comment);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function loads device settings from files.The contents of the file are loaded and set as the new device state. The device automatically recognizes the files that are required for the whole setup from the list of file extensions:
        /// 
        /// (.SET) Current setting of measurement hardware and associated
        ///        title, if indicated.
        /// 
        /// (.LIN) Activated limit lines.
        /// 
        /// (.CFG) Current configuration of general device parameters.
        /// 
        /// (.HCS) Configuration for the hardcopy output.
        /// 
        /// (.COL) User-defined color setting.
        /// 
        /// (.LIA) All defined limit lines.
        /// 
        /// (.TR1 to 3) Measured data trace 1 to trace 3 screen A.
        /// 
        /// (.TR4 to 6) Measured data trace 1 to trace 3 screen B.
        /// 
        /// (.TCi) Tracking generator settings (only with Tracking Generator
        ///        Option B9 or Ext. Generator Control B10)
        /// 
        /// (.TS1 to 2) Setting for source calibration (only with Tracking
        ///             Generator Option B9 or Ext. Generator Control B10)
        /// 
        /// (.TC1 to 2) Correction data for source calibration (only with
        ///             Tracking Generator B9 or Ext. Generator Control B10)
        /// 
        /// (.TF) Activated transducer factors.
        /// 
        /// (.TS) Activated transducer set.
        /// 
        /// (TSA, .TFA) All defined transducer sets + transducer factors.
        /// 
        /// (.FIN) Peak list or final measurement results.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// MMEMory:LOAD:STATe 1,&lt;file_name&gt;
        /// </summary>
        /// <param name="File_Name">
        /// Sets the file name without extension, include indication of the path and may also include the drive name. The path name complies with DOS conventions.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: "TEST"
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int DataSetRecall(string File_Name)
        {
            int pInvokeResult = PInvoke.DataSetRecall(this._handle, File_Name);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines which device setting is automatically loaded after the device is switched on. The contents of the file are read after switching on the device and used to define the new device state.
        /// 
        /// Note(s):
        /// 
        /// The data set defined as auto recall set will also be restored by a *RST command.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// MMEMory:LOAD:AUTO 1,&lt;file_name&gt;
        /// </summary>
        /// <param name="File_Name">
        /// Sets the file name without extension, include indication of the path and may also include the drive name. The path name complies with DOS conventions.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: "FACTORY"
        /// 
        /// Note:
        /// 
        /// "FACTORY" denotes the data set previously in the instrument.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int DataSetRecallAuto(string File_Name)
        {
            int pInvokeResult = PInvoke.DataSetRecallAuto(this._handle, File_Name);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the list of device settings to be stored/loaded.
        /// 
        /// Note(s):
        /// 
        /// (1) Detailed description is provided with the 'Data Set Items' parameter.
        /// 
        /// (2) The command "MMEMory:SELect[:ITEM]:HCOPy" is available only with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// All
        /// Receiver - only for "MMEMory:SELect[:ITEM]:HCOPy"
        /// 
        /// Remote-control command(s):
        /// MMEMory:SELect[:ITEM]:DEFault
        /// MMEMory:SELect[:ITEM]:NONE
        /// MMEMory:SELect[:ITEM]:ALL
        /// MMEMory:SELect[:ITEM]:HWSettings ON | OFF
        /// MMEMory:SELect[:ITEM]:TRACe[:ACTive] ON | OFF
        /// MMEMory:SELect[:ITEM]:FINal ON | OFF
        /// MMEMory:SELect[:ITEM]:LINes:ALL ON | OFF
        /// MMEMory:SELect[:ITEM]:SCData ON | OFF
        /// MMEMory:SELect[:ITEM]:TRANsducer:ALL ON | OFF
        /// MMEMory:SELect[:ITEM]:HCOPy ON | OFF
        /// MMEMory:SELect[:ITEM]:CLISt ON | OFF
        /// </summary>
        /// <param name="Data_Set_Items">
        /// Selects device settings to be stored/loaded. Enable or disable it in the list using 'Selected Item' parameter.
        /// 
        /// Valid Values:
        /// RSESCI_DATA_SET_NONE        (0) - None
        /// RSESCI_DATA_SET_DEFAULT     (1) - Default Config
        /// RSESCI_DATA_SET_ALL         (2) - All Items
        /// RSESCI_DATA_SET_CURRENT     (3) - Current Setting
        /// RSESCI_DATA_SET_TRACES      (4) - All Traces
        /// RSESCI_DATA_SET_FINAL_LIST  (5) - Peak-Final List
        /// RSESCI_DATA_SET_LIMIT_LINES (6) - All Limit Lines
        /// RSESCI_DATA_SET_CALIBRATION (7) - Source Cal Data
        /// RSESCI_DATA_SET_TRANSDUCERS (8) - All Transducers
        /// RSESCI_DATA_SET_HCOPY       (9) - Test Report
        /// RSESCI_DATA_SET_SCAN        (10) - Scan Channel List
        /// 
        /// 
        /// Default Value: RSESCI_DATA_SET_DEFAULT (1)
        /// 
        /// Note(s):
        /// 
        /// (1) None:
        /// Deletes all data subsets from the list of device settings to be stored/loaded.
        /// 
        /// (2) Default Config:
        /// Sets the default list of device settings.
        /// 
        /// The latter includes:
        /// 
        /// - Current configuration of general device parameters (general setup)
        /// - Current setting of the measurement hardware including markers
        /// - Activated limit lines
        /// - User-defined color setting
        /// - Configuration for the hardcopy output
        /// - Tracking generator settings (only in conjunction with option Tracking Generator R&amp;S FSP-B9 or External Generator Control R&amp;S FSP-B10)
        /// - Correction data for source calibration (only in conjunction with option Tracking Generator R&amp;S FSP-B9 or External Generator Control R&amp;S FSP-B10)
        /// 
        /// Trace data, non-used transducer factors/sets and non-used limit lines are not included.
        /// 
        /// (3) All Items:
        /// Includes (Enabled) or deletes (Disabled) all data subsets from the list of device settings to be stored/loaded.
        /// 
        /// (4) Current Setting:
        /// Includes the hardware settings in the list of data subsets of a device setting to be stored/loaded. The hardware settings include:
        /// 
        /// - current configuration of general device parameters (general setup)
        /// - current setting of the measurement hardware including markers
        /// - activated limit lines:
        /// 
        /// A data set may include 8 limit lines at maximum in each measurement window. This number includes the activated limit lines and, if available, the de-activated limit lines last used. Therefore the combination of the non-activated restored limit lines depends on the sequence of use with the command MMEM:LOAD.
        /// 
        /// - user-defined color setting
        /// - configuration for the hardcopy output
        /// - the activated transducers:
        /// 
        /// A data set may include 4 transducer factors at maximum. It comprises the activated factors and, if available, the de-activated factors last used. Therefore the combination of non-activated restored transducer factors depends on the sequence of use with the command MMEM:LOAD.
        /// 
        /// - Tracking generator settings (only in conjunction with option Tracking Generator B9 or External Generator Control B10)
        /// - Correction data for source calibration (only in conjunction with option Tracking Generator B9 or External Generator Control B10)
        /// 
        /// (5) All Traces:
        /// Adds the active traces to the list of data subsets of a save/recall device setting. Active traces are all traces whose state is not blank.
        /// 
        /// (6) Peak-Final List:
        /// Includes the final measurement data or the peak list in the list of partial datasets of a device setting to be stored/loaded.
        /// 
        /// (7) All Limit Lines:
        /// Adds all limit lines (activated and de-activated) to the list of device settings to be stored/loaded.
        /// 
        /// (8) Source Cal Data:
        /// Adds the tracking generator calibration data to the list of device settings to be stored/loaded.
        /// 
        /// (9) All Transducers:
        /// Includes all transducer factors and sets in the list of data subsets of a device setting to be stored/loaded.
        /// 
        /// (10) Test Report:
        /// Copies the test report settings to the list of partial data records of a device setting to be stored/loaded.
        /// 
        /// (11) Scan Channel List:
        /// Includes the scan channel list to be stored/loaded.
        /// </param>
        /// <param name="Selected_Item">
        /// This control switches enables or disables whether the selected device settings is to be stored/loaded.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Disabled
        /// VI_TRUE  (1) - Enabbled
        /// 
        /// Default Value: VI_FALSE (0)
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int DataSetItems(int Data_Set_Items, bool Selected_Item)
        {
            int pInvokeResult = PInvoke.DataSetItems(this._handle, Data_Set_Items, System.Convert.ToUInt16(Selected_Item));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function reads the indicated directory. According to DOS convention, wild card characters can be entered in order to query e.g. a list of all files of a certain type.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// MMEMory:CATalog? &lt;path&gt;
        /// </summary>
        /// <param name="Directory">
        /// This control specifies the directory in which this function will be performed.
        /// 
        /// Valid Range: not tested
        /// 
        /// Default Value: "D:\\user\\data"
        /// 
        /// </param>
        /// <param name="Catalog">
        /// This indicator returns the files in the indicated directory as comma separated strings, for example:
        /// 
        /// 'SPOOL1.WMF','SPOOL2.WMF','SPOOL3.WMF'
        /// 
        /// The buffer must be of type ViChar and have at least as many bytes as indicated in the 'Buffer Size' parameter.
        /// 
        /// If the current value of the attribute, including the terminating NUL byte, contains more bytes that you indicate in this parameter, the function copies Buffer Size - 1 bytes into the buffer, places an ASCII NUL byte at the end of the buffer, and returns the buffer size you must pass to get the entire value. For example, if the value is "123456" and the Buffer Size is 4, the function places "123" into the buffer and returns 7.
        /// 
        /// If you specify 0 for the 'Buffer Size' parameter, you can pass VI_NULL for this parameter.
        /// 
        /// </param>
        /// <param name="Buffer_Size">
        /// Pass the number of bytes in the ViChar array you specify for the 'Catalog' parameter.  
        /// 
        /// If the data, including the terminating NUL byte, contains more bytes that you indicate in this parameter, the function copies Buffer Size - 1 bytes into the buffer, places an ASCII NUL byte at the end of the buffer, and returns the buffer size you must pass to get the entire value.  For example, if the value is "123456" and the Buffer Size is 4, the function places "123" into the buffer and returns 7.
        /// 
        /// If you pass a negative number, the function copies the value to the buffer regardless of the number of bytes in the value.
        /// 
        /// If you pass 0, you can pass VI_NULL for the 'Catalog' buffer parameter.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int DataSetCatalog(string Directory, System.Text.StringBuilder Catalog, int Buffer_Size)
        {
            int pInvokeResult = PInvoke.DataSetCatalog(this._handle, Directory, Catalog, Buffer_Size);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the directories and files in the given path.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// MMEMory:CATalog:LONG? &lt;path&gt;
        /// </summary>
        /// <param name="Directory">
        /// This control specifies the directory in which this function will be performed.
        /// 
        /// Valid Range: not tested
        /// 
        /// Default Value: "D:\\user\\data"
        /// 
        /// </param>
        /// <param name="Catalog">
        /// This indicator returns the directories and files in the indicated directory, for example:
        /// 
        /// &lt;used_bytes_in_this_directory&gt;,&lt;free_bytes_on_this_disk&gt;,
        /// "&lt;file_name&gt;,&lt;file_type&gt;,&lt;filesize_in_bytes&gt;",
        /// "&lt;file_name&gt;,&lt;file_type&gt;,&lt;filesize_in_bytes&gt;", ...
        /// 
        /// The buffer must be of type ViChar and have at least as many bytes as indicated in the 'Buffer Size' parameter.
        /// 
        /// If the current value of the attribute, including the terminating NUL byte, contains more bytes that you indicate in this parameter, the function copies Buffer Size - 1 bytes into the buffer, places an ASCII NUL byte at the end of the buffer, and returns the buffer size you must pass to get the entire value. For example, if the value is "123456" and the Buffer Size is 4, the function places "123" into the buffer and returns 7.
        /// 
        /// If you specify 0 for the 'Buffer Size' parameter, you can pass VI_NULL for this parameter.
        /// 
        /// </param>
        /// <param name="Buffer_Size">
        /// Pass the number of bytes in the ViChar array you specify for the 'Catalog' parameter.  
        /// 
        /// If the data, including the terminating NUL byte, contains more bytes that you indicate in this parameter, the function copies Buffer Size - 1 bytes into the buffer, places an ASCII NUL byte at the end of the buffer, and returns the buffer size you must pass to get the entire value.  For example, if the value is "123456" and the Buffer Size is 4, the function places "123" into the buffer and returns 7.
        /// 
        /// If you pass a negative number, the function copies the value to the buffer regardless of the number of bytes in the value.
        /// 
        /// If you pass 0, you can pass VI_NULL for the 'Catalog' buffer parameter.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int DataSetCatalogLong(string Directory, System.Text.StringBuilder Catalog, int Buffer_Size)
        {
            int pInvokeResult = PInvoke.DataSetCatalogLong(this._handle, Directory, Catalog, Buffer_Size);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function deletes the instrument setting selected by file names ('path' parameter) or deletes all device settings in the current directory ('path' parameter). All associated files on the mass memory storage are cleared. A list of the extensions used is included under MMEMory:LOAD:STATe.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// MMEMory:CDIRectory &lt;directory_name&gt;
        /// MMEMory:CLEar:STATe 1,&lt;file_name&gt;
        /// MMEMory:CLEar:ALL
        /// </summary>
        /// <param name="Clear">
        /// Selects whether deletes the instrument setting selected by file names ('path' parameter) or deletes all device settings in the current directory ('path' parameter).
        /// 
        /// Valid Values:
        /// RSESCI_CLEAR_ALL  (0) - Clear All
        /// RSESCI_CLEAR_FILE (1) - Clear File
        /// 
        /// Default Value: RSESCI_CLEAR_FILE (1)
        /// 
        /// </param>
        /// <param name="Path">
        /// Selects the directory or file where perform this function.
        /// 
        /// The file name without extension includes indication of the path and may also include the drive. The path name complies with DOS conventions.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: ""
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int DataSetClear(int Clear, string Path)
        {
            int pInvokeResult = PInvoke.DataSetClear(this._handle, Clear, Path);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function provides operations which allows to do basic file operations on the storage media of the instrument. It consists from the following operations:
        /// 
        ///   Change Drive
        ///   Change Current Directory
        ///   Create Directory
        ///   Delete Directory
        ///   Copy File
        ///   Move File
        ///   Delete File
        ///   Format Floppy Disk
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// MMEMory:MSIS &lt;device&gt;
        /// MMEMory:CDIRectory &lt;directory_name&gt;
        /// MMEMory:MDIRectory &lt;directory_name&gt;
        /// MMEMory:RDIRectory &lt;directory_name&gt;
        /// MMEMory:COPY &lt;file_source&gt;,&lt;file_destination&gt;
        /// MMEMory:MOVE &lt;file_source&gt;,&lt;file_destination&gt;
        /// MMEMory:DELete &lt;file_name&gt;
        /// MMEMory:INITialize &lt;msus&gt;
        /// </summary>
        /// <param name="Operation_to_be_Performed">
        /// This control selects the type of operation to be performed.
        /// 
        /// Valid Values:
        /// RSESCI_FILE_MAN_CDRIVE (0) - Change Drive
        /// RSESCI_FILE_MAN_CDIR   (1) - Change Current Directory
        /// RSESCI_FILE_MAN_MDIR   (2) - Create Directory
        /// RSESCI_FILE_MAN_RDIR   (3) - Delete Directory
        /// RSESCI_FILE_MAN_COPY   (4) - Copy File
        /// RSESCI_FILE_MAN_MOVE   (5) - Move File
        /// RSESCI_FILE_MAN_DELETE (6) - Delete File
        /// RSESCI_FILE_MAN_FORMAT (7) - Format Floppy Disk
        /// 
        /// Note(s):
        /// 
        /// (1) Change Drive:
        /// Changes to the drive indicated. The drive may be the internal hard disk D: or the floppy disk drive A: (Example "A:").
        /// 
        /// (2) Change Current Directory:
        /// Changes the current directory. In addition to the path name, the indication of the directory may contain the drive name. The path name complies with the DOS conventions (Example "D:\\USER\\DATA").
        /// 
        /// (3) Create Directory:
        /// Creates a new directory. The file name includes indication of the path and may also include the drive name. The path name complies with DOS conventions (Example "D:\\USER\\DATA").
        /// 
        /// (4) Delete Directory:
        /// Deletes the indicated directory. The directory name includes indication of the path and may also include the drive name. The path name complies with DOS conventions (Example "D:\\TEST").
        /// 
        /// (5) Copy File:
        /// Copies the files indicated in 'Source' to the destination directory indicated with 'Destination' or to the destination file indicated by 'Destination' when 'Source' is just a file. The indication of the file name may include the path and the drive name. The file names and path information must be in accordance with the DOS conventions (Example "D:\\USER\\DATA\\SETUP.CFG", "A:").
        /// 
        /// (6) Move File:
        /// Renames existing files, if 'Destination' contains no path indication. Otherwise the file is moved to the indicated path and stored under the file name specified there, if any. The file name includes indication of the path and may also include the drive. The path name complies with DOS conventions (Example of renaming "D:\\TEST01.CFG", "SETUP.CFG" and exampe of moving "D:\\TEST01.CFG", "D:\\USER\\DATA").
        /// 
        /// (7) Delete File:
        /// Deletes the indicated files. The indication of the file name contains the path and, optionally, the drive name. Indication of the path complies with DOS conventions (Example "TEST01.HCP").
        /// 
        /// (8) Format Floppy Disk:
        /// Formats the disk in drive A. Formatting deletes all data stored on the floppy disk.
        /// 
        /// </param>
        /// <param name="Source">
        /// This control specifies the source of the operation to be performed. If the operation requires a single parameter, this control specifies that parameter.
        /// 
        /// Valid Range: not tested
        /// 
        /// Default Value: "D:\\user\\data"
        /// 
        /// </param>
        /// <param name="Destination">
        /// This control specifies the destination of the operation to be performed.
        /// 
        /// Valid Range: not tested
        /// 
        /// Default Value: ""
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int FileManager(int Operation_to_be_Performed, string Source, string Destination)
        {
            int pInvokeResult = PInvoke.FileManager(this._handle, Operation_to_be_Performed, Source, Destination);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function stores the selected trace (1 to 3) in the measurement window (screen A or B) in a file with ASCII format.
        /// 
        /// Note(s):
        /// 
        /// (1) The file format is described in chapter 4 in the TRACE menu under the ASCII-FILE EXPORT softkey.
        /// 
        /// (2) The decimal separator (decimal point or comma) for floating-point numerals contained in the file is defined with the command FORMat:DEXPort:DSEParator.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// FORMat[:DATA] ASCii
        /// MMEMory:STORe&lt;1|2&gt;:TRACe 1 to 3,&lt;file_name&gt;
        /// </summary>
        /// <param name="Trace">
        /// This control sets the trace.
        /// 
        /// Valid Range: 1 to 3
        /// 
        /// Default Value: 1
        /// </param>
        /// <param name="File_Name">
        /// Sets the file name where to stores data on a disk. The file name includes indication of the path and the drive name. Indication of the path complies with DOS conventions.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: "A:\\TEST.ASC"
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int StoreTraceData(int Trace, string File_Name)
        {
            int pInvokeResult = PInvoke.StoreTraceData(this._handle, Trace, File_Name);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function stores the final measurement data in an ASCII file.
        /// 
        /// Mode(s):
        /// Receiver
        /// 
        /// Remote-control command(s):
        /// FORMat[:DATA] ASCii
        /// MMEMory:STORe:FINal &lt;file_name&gt;
        /// </summary>
        /// <param name="File_Name">
        /// Sets the file name where to store data on a disk. The file name includes indication of the path and the drive name. Indication of the path complies with DOS conventions.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: "A:\\TEST.ASC"
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int StoreFinalMeasurementData(string File_Name)
        {
            int pInvokeResult = PInvoke.StoreFinalMeasurementData(this._handle, File_Name);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function stores the content of the marker peak list in ASCII format to the specified file.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// MMEMory:STORe:PEAKlist
        /// </summary>
        /// <param name="File_Name">
        /// Sets the file name where to stores data on a disk. The file name includes indication of the path and the drive name. Indication of the path complies with DOS conventions.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: "A:\\TEST.ASC"
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int StorePeakList(string File_Name)
        {
            int pInvokeResult = PInvoke.StorePeakList(this._handle, File_Name);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function saves the data of all active markers to the specified file.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// MMEMory:STORe&lt;1|2&gt;:MARKer
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="File_Name">
        /// Sets the file name where to stores data on a disk. The file name includes indication of the path and the drive name. Indication of the path complies with DOS conventions.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: "A:\\TEST.ASC"
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int StoreMarkers(int Window, string File_Name)
        {
            int pInvokeResult = PInvoke.StoreMarkers(this._handle, Window, File_Name);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function is used to read data from the instrument and write it to a user specified file on the host computer.
        /// 
        /// Remote-control command(s):
        /// MMEMory:DATA? &lt;file_name&gt;
        /// 
        /// </summary>
        /// <param name="Source">
        /// This control selects file for the data transfer from instrument to control computer.
        /// 
        /// Valid Range: not tested
        /// 
        /// Default Value: "D:\\user\\data\\test.hcp"
        /// 
        /// </param>
        /// <param name="Destination">
        /// This control defines destination file to which the data transfer from instrument to control computer applies.
        /// 
        /// Valid Range: not tested
        /// 
        /// Default Value: "\\test.hcp"
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int readToFile(string Source, string Destination)
        {
            int pInvokeResult = PInvoke.readToFile(this._handle, Source, Destination);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function is used to read data from the host computer and write it to a user specified file in the instrument.
        /// 
        /// Remote-control command(s):
        /// SYSTem:COMMunicate:GPIB[:SELF]:RTERminator LFEOI | EOI
        /// MMEMory:DATA &lt;file_name&gt;
        /// 
        /// </summary>
        /// <param name="Source">
        /// This control selects file for the data transfer from control computer to instrument.
        /// 
        /// Valid Range: not tested
        /// 
        /// Default Value: "\\test.hcp"
        /// 
        /// </param>
        /// <param name="Destination">
        /// This control defines destination file to which the data transfer from control computer to instrument applies.
        /// 
        /// Valid Range: not tested
        /// 
        /// Default Value: "D:\\user\\data\\test.hcp"
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int writeFromFile(string Source, string Destination)
        {
            int pInvokeResult = PInvoke.writeFromFile(this._handle, Source, Destination);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines which decimal separator (decimal point or comma) is to be used for outputting measurement data to the file in ASCII format. Different languages of evaluation programs (e.g. MS-Excel) can thus be supported.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// FORMat:DEXPort:DSEParator POINt|COMMa
        /// </summary>
        /// <param name="Decimal_Separator">
        /// Defines which decimal separator (decimal point or comma) is to be used for outputting measurement data to the file in ASCII format.
        /// 
        /// Valid Values:
        /// RSESCI_DEC_SEPARATOR_POINT (0) - Point
        /// RSESCI_DEC_SEPARATOR_COMMA (1) - Comma
        /// 
        /// Default Value: RSESCI_DEC_SEPARATOR_POINT (0)
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetDecimalPointNotation(int Decimal_Separator)
        {
            int pInvokeResult = PInvoke.SetDecimalPointNotation(this._handle, Decimal_Separator);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns which decimal separator (decimal point or comma) is to be used for outputting measurement data to the file in ASCII format.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// FORMat:DEXPort:DSEParator?
        /// </summary>
        /// <param name="Decimal_Separator">
        /// Returns which decimal separator (decimal point or comma) is to be used for outputting measurement data to the file in ASCII format.
        /// 
        /// Valid Values:
        /// RSESCI_DEC_SEPARATOR_POINT (0) - Point
        /// RSESCI_DEC_SEPARATOR_COMMA (1) - Comma
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetDecimalPointNotation(out int Decimal_Separator)
        {
            int pInvokeResult = PInvoke.GetDecimalPointNotation(this._handle, out Decimal_Separator);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects one of the printers installed under Windows. Prior to use this function get list of available printers using function Get Printer List.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// SYSTem:COMMunicate:PRINter:SELect &lt;1|2&gt; &lt;printer_name&gt;
        /// </summary>
        /// <param name="Device_Number">
        /// Selects hardcopy device.
        /// 
        /// Valid Range: 1 to 2
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <param name="Printer_Name">
        /// This control sets printer name.
        /// 
        /// Valid Range: up to 128 chars
        /// 
        /// Default Value: ""
        /// 
        /// Note:
        /// 
        /// Valid names are taken using function Get Printer List.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SelectPrinter(int Device_Number, string Printer_Name)
        {
            int pInvokeResult = PInvoke.SelectPrinter(this._handle, Device_Number, Printer_Name);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function queries the list of printers under Windows.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// SYSTem:COMMunicate:PRINter:ENUMerate:FIRSt?
        /// SYSTem:COMMunicate:PRINter:ENUMerate:NEXT?
        /// </summary>
        /// <param name="Printer_List">
        /// This control returns list of printers available under Windows system. Names are separated using commas (,).
        /// 
        /// Note:
        /// 
        /// If no printer is configured an empty string is output.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int QueryPrinterList(System.Text.StringBuilder Printer_List)
        {
            int pInvokeResult = PInvoke.QueryPrinterList(this._handle, Printer_List);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function aborts a running hardcopy output.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// HCOPy:ABORt
        /// </summary>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int AbortHardcopy()
        {
            int pInvokeResult = PInvoke.AbortHardcopy(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function starts a hardcopy output. Device number parameter selects which printer configuration (1 or 2) is to be used for the hardcopy output.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// HCOPy[:IMMediate&lt;1|2&gt;]
        /// </summary>
        /// <param name="Device_Number">
        /// Selects hardcopy device.
        /// 
        /// Valid Range: 1 to 2
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int InitiateHardcopy(int Device_Number)
        {
            int pInvokeResult = PInvoke.InitiateHardcopy(this._handle, Device_Number);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the interface and configuration for the hardcopy operation. Destination and data format is also configured.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// MMEMory:NAME &lt;file_name&gt;
        /// HCOPy:DESTination&lt;1|2&gt;
        /// HCOPy:DEVice:LANGuage&lt;1|2&gt; GDI | WMF | EWMF | BMP
        /// HCOPy:PAGE:ORIentation&lt;1|2&gt; LANDscape | PORTrait
        /// </summary>
        /// <param name="Device_Number">
        /// Selects hardcopy device.
        /// 
        /// Valid Range: 1 to 2
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <param name="Destination">
        /// Selects the printer output medium (Disk, Printer or Clipboard) associated with configuration 1 or 2.
        /// 
        /// The type of instrument is selected with SYSTem:COMMunicate:PRINter:SELect, which will automatically select a default output medium. Therefore the command HCOPy:DESTination should always be sent after setting the device type.
        /// 
        /// Valid Range:
        /// RSESCI_HARDCOPY_DISK      (0) - Disk
        /// RSESCI_HARDCOPY_PRINTER   (1) - Printer
        /// RSESCI_HARDCOPY_CLIPBOARD (2) - Clipboard
        /// 
        /// Default Value: RSESCI_HARDCOPY_DISK (0)
        /// 
        /// Notes:
        /// 
        /// (1) Disk:
        /// Directs the hardcopy to a file. Command MMEM:NAME &lt;file_name&gt; defines the file name. All formats can be selected for HCOPy:DEVice:LANGuage.
        /// 
        /// (2) Printer:
        /// Directs the hardcopy to the printer. The printer is selected with command SYSTEM:COMMunicate:PRINter:SELect. GDI should be selected for HCOPy:DEVice:LANGuage.
        /// 
        /// (3) Clipboard:
        /// Directs the hardcopy to the clipboard. EWMF should be selected for HCOPy:DEVice:LANGuage.
        /// 
        /// </param>
        /// <param name="Data_Format">
        /// This control determines the data format of the printout.
        /// 
        /// Valid Range:
        /// RSESCI_HARDCOPY_WMF  (0) - WMF
        /// RSESCI_HARDCOPY_GDI  (1) - GDI
        /// RSESCI_HARDCOPY_EWMF (2) - EWMF
        /// RSESCI_HARDCOPY_BMP  (3) - BMP
        /// 
        /// Default Value: RSESCI_HARDCOPY_WMF  (0)
        /// 
        /// Notes:
        /// 
        /// (1) GDI:
        /// Graphics Device Interface. Default format for the output to a printer configured under Windows. Must be selected for the output to the printer interface (HCOPy:DEVice 'SYST:COMM:PRIN'). Can be used for the output to a file (HCOPy:DEVice 'SYST:COMM:MMEM'). The printer driver configured under Windows is used in this case and a printer-specific file format is thus generated.
        /// 
        /// (2) WMF and EWMF:
        /// WINDOWS Metafile and Enhanced Metafile Format. Data formats for output files which can be integrated in corresponding programs for documentation purposes at a later time. WMF can only be used for output to a file (HCOPy:DEVice 'SYST:COMM:MMEM') and EWMF also for the output to the clipboard (HCOPy:DEVice 'SYST:COMM:CLIP').
        /// 
        /// (3) BMP:
        /// Bitmap. Data format for output to files only (HCOPy:DEVice 'SYST:COMM:MMEM').
        /// 
        /// </param>
        /// <param name="Page_Orientation">
        /// Selects the format of the output (portrait and landscape) (hardcopy unit 1 or 2).
        /// 
        /// Valid Range:
        /// RSESCI_HARDCOPY_PORTRAIT  (0) - Portrait
        /// RSESCI_HARDCOPY_LANDSCAPE (1) - Landscape
        /// 
        /// Default Value: RSESCI_HARDCOPY_PORTRAIT (0)
        /// 
        /// Note:
        /// 
        /// This setting will be ignored for Destination &lt;&gt; Printer.
        /// 
        /// </param>
        /// <param name="File_Name">
        /// This control defines a destination file for the printout started. In this case the printer output must be routed to destination Disk.
        /// 
        /// Indication of the path complies with DOS conventions.
        /// 
        /// Valid Range: not checked
        /// 
        /// Default Value: "PRINT1.BMP"
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int HardcopyDeviceSetup(int Device_Number, int Destination, int Data_Format, int Page_Orientation, string File_Name)
        {
            int pInvokeResult = PInvoke.HardcopyDeviceSetup(this._handle, Device_Number, Destination, Data_Format, Page_Orientation, File_Name);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// Defines the comment text for measurement window 1 or 2 for printout.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// HCOPy:ITEM:WINDow&lt;1|2&gt;:TEXT &lt;string&gt;
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Comment">
        /// Defines the comment text for measurement window 1 or 2 for printout, with a maximum of 100 characters (line feed by means of character @).
        /// 
        /// Valid Range: 100 characters
        /// 
        /// Default Value: "Comment ..."
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int HardcopyComment(int Window, string Comment)
        {
            int pInvokeResult = PInvoke.HardcopyComment(this._handle, Window, Comment);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This hardcopy output is always provided with comments, title, time and date. As an alternative to the whole screen, only traces (commands 'HCOPy:ITEM:WINDow:TRACe: STATe ON') or tables (command 'HCOPy:ITEM:WINDow:TABLe:STATe ON') can be output.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// HCOPy:ITEM:ALL
        /// HCOPy:ITEM:WINDow&lt;1|2&gt;:TABLe:STATe ON | OFF
        /// HCOPy:ITEM:WINDow&lt;1|2&gt;:TRACe:STATe ON | OFF
        /// </summary>
        /// <param name="Window">
        /// The measurement windows are selected by means of this parameter.
        /// 
        /// Valid Range:
        /// RSESCI_SCREEN_A (1) - Screen A
        /// RSESCI_SCREEN_B (2) - Screen B
        /// 
        /// Default Value: RSESCI_SCREEN_A (1)
        /// 
        /// Notes:
        /// 
        /// (1) If the instrument is in full screen mode and the screen selected is not the active screen then the instrument will generate an error.
        /// 
        /// </param>
        /// <param name="Hardcopy_Items">
        /// This control selects items to be output.
        /// 
        /// Valid Range:
        /// RSESCI_HARDCOPY_ALL   (0) - All Items
        /// RSESCI_HARDCOPY_TABLE (1) - Table
        /// RSESCI_HARDCOPY_TRACE (2) - Trace
        /// 
        /// Default Value: RSESCI_HARDCOPY_ALL (0)
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int HardcopyItems(int Window, int Hardcopy_Items)
        {
            int pInvokeResult = PInvoke.HardcopyItems(this._handle, Window, Hardcopy_Items);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches over from colour output to black-and-white output. All colour-highlighted areas are printed in white and all colour lines in black. This improves the contrast on the printout.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// HCOPy:DEVice:COLor ON|OFF
        /// </summary>
        /// <param name="Color">
        /// This control selects between the color and monochrome hardcopy of the screen.
        /// 
        /// Valid Range:
        /// VI_TRUE  (1) - On (Default Value)
        /// VI_FALSE (0) - Off
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int HardcopyColor(bool Color)
        {
            int pInvokeResult = PInvoke.HardcopyColor(this._handle, System.Convert.ToUInt16(Color));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function allows selection of colors for the printouts of the screen contents.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// HCOPy:CMAP&lt;1...34&gt;:HSL &lt;hue&gt;,&lt;sat&gt;,&lt;lum&gt;
        /// HCOPy:CMAP&lt;1...34&gt;:PDEFined BLACk | BLUE | BROWn | GREen | CYAN | RED | MAGenta | YELLow | WHITe | DGRAy | LGRAy | LBLUe | LGREen | LCYan | LRED | LMAGenta
        /// </summary>
        /// <param name="Color_Map">
        /// This control defines the color table of the instrument.
        /// 
        /// Valid Range:
        /// RSESCI_CMAP_1 (1) - Background
        /// RSESCI_CMAP_2 (2) - Grid
        /// RSESCI_CMAP_3 (3) - Function and status field + data entry text
        /// RSESCI_CMAP_4 (4) - Function field LED on
        /// RSESCI_CMAP_5 (5) - Function field LED warn
        /// RSESCI_CMAP_6 (6) - Enhancement label text
        /// RSESCI_CMAP_7 (7) - Status field background
        /// RSESCI_CMAP_8 (8) - Trace 1
        /// RSESCI_CMAP_9 (9) - Trace 2
        /// RSESCI_CMAP_10 (10) - Trace 3
        /// RSESCI_CMAP_11 (11) - Marker
        /// RSESCI_CMAP_12 (12) - Lines
        /// RSESCI_CMAP_13 (13) - Measurement status + Limit check pass
        /// RSESCI_CMAP_14 (14) - Limit check fail
        /// RSESCI_CMAP_15 (15) - Table + softkey background
        /// RSESCI_CMAP_16 (16) - Table + softkey text
        /// RSESCI_CMAP_17 (17) - Table selected field text
        /// RSESCI_CMAP_18 (18) - Table selected field background
        /// RSESCI_CMAP_19 (19) - Table + data entry field opaque titlebar
        /// RSESCI_CMAP_20 (20) - Data entry field opaque text
        /// RSESCI_CMAP_21 (21) - Data entry field opaque background
        /// RSESCI_CMAP_22 (22) - 3D shade bright part
        /// RSESCI_CMAP_23 (23) - 3D shade dark part
        /// RSESCI_CMAP_24 (24) - Softkey state on
        /// RSESCI_CMAP_25 (25) - Softkey state data entry
        /// RSESCI_CMAP_26 (26) - Logo
        /// RSESCI_CMAP_27 (27) - Bargraph PK+
        /// RSESCI_CMAP_28 (28) - Bargraph PK-
        /// RSESCI_CMAP_29 (29) - Bargraph QPK
        /// RSESCI_CMAP_30 (30) - Bargraph AVER
        /// RSESCI_CMAP_31 (31) - Bargraph CAV
        /// RSESCI_CMAP_32 (32) - Bargraph RMS
        /// RSESCI_CMAP_33 (33) - Bargraph CRMS
        /// RSESCI_CMAP_32 (34) - Final Meas
        /// 
        /// Default Value: RSESCI_CMAP_1 (1)
        /// 
        /// </param>
        /// <param name="Predefined_Colors">
        /// This control defines the color table of the instrument using predefined color values or user defined settings via tint, saturation and brightness parameters.
        /// 
        /// Valid Range:
        /// RSESCI_USER_DEFINED (-1) - User defined settings
        /// RSESCI_BLACK        (0)  - Black
        /// RSESCI_BLUE         (1)  - Blue
        /// RSESCI_BROWN        (2)  - Brown
        /// RSESCI_GREEN        (3)  - Green
        /// RSESCI_CYAN         (4)  - Cyan
        /// RSESCI_RED          (5)  - Red
        /// RSESCI_MAGNETA      (6)  - Magneta
        /// RSESCI_YELLOW       (7)  - Yellow
        /// RSESCI_WHITE        (8)  - White
        /// RSESCI_DGRAY        (9)  - Dark Gray
        /// RSESCI_LGRAY        (10) - Light Gray
        /// RSESCI_LBLUE        (11) - Light Blue
        /// RSESCI_LGREEN       (12) - Light Green
        /// RSESCI_LCYAN        (13) - Light Cyan
        /// RSESCI_LRED         (14) - Light Red
        /// RSESCI_LMAGNETA     (15) - Light Magneta
        /// 
        /// Default Value: RSESCI_BLACK (0)
        /// 
        /// </param>
        /// <param name="Tint">
        /// This control sets the hue = Tint.
        /// 
        /// Valid Range: 0.0 to 100.0 %
        /// 
        /// Default Value: 30.0 %
        /// 
        /// </param>
        /// <param name="Saturation">
        /// This control sets the sat = Saturation.
        /// 
        /// Valid Range: 0.0 to 100.0 %
        /// 
        /// Default Value: 80.0 %
        /// 
        /// </param>
        /// <param name="Brightness">
        /// This control sets the lum = Brightness.
        /// 
        /// Valid Range: 0.0 to 100.0 %
        /// 
        /// Default Value: 100.0 %
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int HardcopyUserDefinedColors(int Color_Map, int Predefined_Colors, double Tint, double Saturation, double Brightness)
        {
            int pInvokeResult = PInvoke.HardcopyUserDefinedColors(this._handle, Color_Map, Predefined_Colors, Tint, Saturation, Brightness);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function resets the colors for a hardcopy to the selected default settings.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// HCOPy:CMAP&lt;1...33&gt;:DEFault&lt;1|2|3&gt;
        /// </summary>
        /// <param name="Default_Colors">
        /// This control resets the colors for a hardcopy to the selected default settings.
        /// 
        /// Valid Values:
        /// RSESCI_DEFAULT_USER_DEFINED  (0) - User Defined
        /// RSESCI_DEFAULT_COLOR_SET     (1) - Optimized Color Set
        /// RSESCI_DEFAULT_SCREEN_COLORS (2) - Screen Colors
        /// 
        /// Default Value: RSESCI_DEFAULT_USER_DEFINED (0)
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int HardcopyDefaultColors(int Default_Colors)
        {
            int pInvokeResult = PInvoke.HardcopyDefaultColors(this._handle, Default_Colors);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function starts stores a hardcopy output for selected device to file on local disk.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// MMEMory:DELete &lt;file_name&gt;
        /// MMEMory:CDIRectory &lt;directory_name&gt;
        /// MMEMory:MSIS &lt;device&gt;
        /// MMEMory:NAME &lt;file_name&gt;
        /// MMEMory:DATA? &lt;file_name&gt;
        /// HCOPy:DESTination&lt;1|2&gt;
        /// HCOPy:DEVice:LANGuage&lt;1|2&gt; GDI | WMF | EWMF | BMP
        /// HCOPy:PAGE:ORIentation&lt;1|2&gt; LANDscape | PORTrait
        /// </summary>
        /// <param name="Device_Number">
        /// Selects hardcopy device.
        /// 
        /// Valid Range: 1 to 2
        /// 
        /// Default Value: 1
        /// 
        /// </param>
        /// <param name="Destination">
        /// This control specifies the destination of the operation to be performed.
        /// 
        /// Valid Range: not tested
        /// 
        /// Default Value: "C:\\PRINT1.BMP"
        /// 
        /// </param>
        /// <param name="Data_Format">
        /// This control determines the data format of the printout.
        /// 
        /// Valid Range:
        /// RSESCI_HARDCOPY_WMF  (0) - WMF
        /// RSESCI_HARDCOPY_GDI  (1) - GDI
        /// RSESCI_HARDCOPY_EWMF (2) - EWMF
        /// RSESCI_HARDCOPY_BMP  (3) - BMP
        /// 
        /// Default Value: RSESCI_HARDCOPY_BMP  (3) - BMP
        /// 
        /// Notes:
        /// 
        /// (1) GDI:
        /// Graphics Device Interface. Default format for the output to a printer configured under Windows. Must be selected for the output to the printer interface (HCOPy:DEVice 'SYST:COMM:PRIN'). Can be used for the output to a file (HCOPy:DEVice 'SYST:COMM:MMEM'). The printer driver configured under Windows is used in this case and a printer-specific file format is thus generated.
        /// 
        /// (2) WMF and EWMF:
        /// WINDOWS Metafile and Enhanced Metafile Format. Data formats for output files which can be integrated in corresponding programs for documentation purposes at a later time. WMF can only be used for output to a file (HCOPy:DEVice 'SYST:COMM:MMEM') and EWMF also for the output to the clipboard (HCOPy:DEVice 'SYST:COMM:CLIP').
        /// 
        /// (3) BMP:
        /// Bitmap. Data format for output to files only (HCOPy:DEVice 'SYST:COMM:MMEM').
        /// 
        /// </param>
        /// <param name="Page_Orientation">
        /// Selects the format of the output (portrait and landscape) (hardcopy unit 1 or 2).
        /// 
        /// Valid Range:
        /// RSESCI_HARDCOPY_PORTRAIT  (0) - Portrait
        /// RSESCI_HARDCOPY_LANDSCAPE (1) - Landscape
        /// 
        /// Default Value: RSESCI_HARDCOPY_PORTRAIT (0)
        /// 
        /// Note:
        /// 
        /// This setting will be ignored for Destination &lt;&gt; Printer.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int InitiateHardcopyToFile(int Device_Number, string Destination, int Data_Format, int Page_Orientation)
        {
            int pInvokeResult = PInvoke.InitiateHardcopyToFile(this._handle, Device_Number, Destination, Data_Format, Page_Orientation);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects the type of output that is used to document the measurements.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is only available with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// HCOPy:MODE SCReen | TREPort
        /// </summary>
        /// <param name="Output">
        /// This control selects the type of output that is used to document the measurements.
        /// 
        /// Valid Values:
        /// RSESCI_HCOP_SCREEN  (0) - Screen
        /// RSESCI_HCOP_TREP    (1) - Test Report
        /// 
        /// Default Value: RSESCI_HCOP_SCREEN  (0)
        /// 
        /// Note(s):
        /// 
        /// (1) Screen: outputs the information that is visible on the screen in a graphical format.
        /// 
        /// (2) Test Report: prepares a configurable collection of measurement results and the instrument settings that had an influence on the measurement results.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetHardcopyOutput(int Output)
        {
            int pInvokeResult = PInvoke.SetHardcopyOutput(this._handle, Output);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the type of output that is used to document the measurements.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is only available with ESU instrument.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analysis
        /// 
        /// Remote-control command(s):
        /// HCOPy:MODE?
        /// </summary>
        /// <param name="Output">
        /// This control returns the type of output that is used to document the measurements.
        /// 
        /// Valid Values:
        /// RSESCI_HCOP_SCREEN  (0) - Screen
        /// RSESCI_HCOP_TREP    (1) - Test Report
        /// 
        /// Note(s):
        /// 
        /// (1) Screen: outputs the information that is visible on the screen in a graphical format.
        /// 
        /// (2) Test Report: prepares a configurable collection of measurement results and the instrument settings that had an influence on the measurement results.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetHardcopyOutput(out int Output)
        {
            int pInvokeResult = PInvoke.GetHardcopyOutput(this._handle, out Output);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function generates a new test report in a new document in accordance with the test report configuration settings. An existing report document is deleted. This report document is initially stored internally until it is printed using the HCOPy:IMMediate command or exported to a portable file format.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analyzer
        /// 
        /// Remote-control command(s):
        /// HCOPy:TREPort:NEW
        /// </summary>
        /// <returns>
        /// Returns the status code of this operation. The status code
        /// either indicates success or describes an error or warning
        /// condition. You examine the status code from each call to an
        /// instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the
        /// rsfsq_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        /// 
        /// Status    Description
        /// -------------------------------------------------
        /// 
        /// BFFC0002  Parameter 2 (Number of Channel Tables) is null pointer.
        /// BFFC0003  Parameter 3 (Buffer Size) out of range.
        /// BFFC0004  Parameter 4 (Channel Tables List) is null pointer.
        /// 
        /// BFFC09F0  Instrument status error
        /// BFFC09F1  Option not installed
        /// BFFC09F2  Conflict with setup parameters
        /// BFFC09F3  Data are not available
        /// BFFC09F4  Unknown option
        /// BFFC09F5  Calibration failed
        /// BFFC09F6  Function does not support FSEx parameter value
        /// BFFC09F7  Overload. Instrument does not return I/Q values
        /// 
        /// This instrument driver also returns errors and warnings defined
        /// by other sources. The following table defines the ranges of
        /// additional status codes that this driver can return. The table
        /// lists the different include files that contain the defined
        /// constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int CreateNewTestReport()
        {
            int pInvokeResult = PInvoke.CreateNewTestReport(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function adds a new test report to an existing document in accordance with the test report configuration settings. This report document is initially stored internally until it is printed using the HCOPy:IMMediate command or exported to a portable file format.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analyzer
        /// 
        /// Remote-control command(s):
        /// HCOPy:TREPort:APPend
        /// </summary>
        /// <returns>
        /// Returns the status code of this operation. The status code
        /// either indicates success or describes an error or warning
        /// condition. You examine the status code from each call to an
        /// instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the
        /// rsfsq_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        /// 
        /// Status    Description
        /// -------------------------------------------------
        /// 
        /// BFFC0002  Parameter 2 (Number of Channel Tables) is null pointer.
        /// BFFC0003  Parameter 3 (Buffer Size) out of range.
        /// BFFC0004  Parameter 4 (Channel Tables List) is null pointer.
        /// 
        /// BFFC09F0  Instrument status error
        /// BFFC09F1  Option not installed
        /// BFFC09F2  Conflict with setup parameters
        /// BFFC09F3  Data are not available
        /// BFFC09F4  Unknown option
        /// BFFC09F5  Calibration failed
        /// BFFC09F6  Function does not support FSEx parameter value
        /// BFFC09F7  Overload. Instrument does not return I/Q values
        /// 
        /// This instrument driver also returns errors and warnings defined
        /// by other sources. The following table defines the ranges of
        /// additional status codes that this driver can return. The table
        /// lists the different include files that contain the defined
        /// constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int AppendTestReport()
        {
            int pInvokeResult = PInvoke.AppendTestReport(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the test report configuration to a defined default status. A newly generated test report will then contain the following:
        /// Header line
        /// Diagram
        /// Scan table
        /// Final measured values
        /// Time of day and date
        /// Page number
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analyzer
        /// 
        /// Remote-control command(s):
        /// HCOPy:TREPort:ITEM:DEFault
        /// </summary>
        /// <returns>
        /// Returns the status code of this operation. The status code
        /// either indicates success or describes an error or warning
        /// condition. You examine the status code from each call to an
        /// instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the
        /// rsfsq_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        /// 
        /// Status    Description
        /// -------------------------------------------------
        /// 
        /// BFFC0002  Parameter 2 (Number of Channel Tables) is null pointer.
        /// BFFC0003  Parameter 3 (Buffer Size) out of range.
        /// BFFC0004  Parameter 4 (Channel Tables List) is null pointer.
        /// 
        /// BFFC09F0  Instrument status error
        /// BFFC09F1  Option not installed
        /// BFFC09F2  Conflict with setup parameters
        /// BFFC09F3  Data are not available
        /// BFFC09F4  Unknown option
        /// BFFC09F5  Calibration failed
        /// BFFC09F6  Function does not support FSEx parameter value
        /// BFFC09F7  Overload. Instrument does not return I/Q values
        /// 
        /// This instrument driver also returns errors and warnings defined
        /// by other sources. The following table defines the ranges of
        /// additional status codes that this driver can return. The table
        /// lists the different include files that contain the defined
        /// constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetTestReportDefaultSettings()
        {
            int pInvokeResult = PInvoke.SetTestReportDefaultSettings(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the page settings of the test report.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analyzer
        /// 
        /// Remote-control command(s):
        /// HCOPy:TREPort:ITEM:HEADer:STATe ON | OFF
        /// HCOPy:TREPort:ITEM:DIAGram:STATe ON | OFF
        /// HCOPy:TREPort:ITEM:FRESults:STATe ON | OFF
        /// HCOPy:TREPort:ITEM:PAGecount:STATe ON | OFF
        /// HCOPy:TREPort:ITEM:SCANtable:STATe ON | OFF
        /// HCOPy:TREPort:ITEM:SRESults:STATe ON | OFF
        /// HCOPy:TREPort:ITEM:TDSTamp:STATe ON | OFF
        /// HCOPy:TREPort:ITEM:TRANsducer:STATe ON | OFF
        /// </summary>
        /// <param name="Item_Selector">
        /// This control selects the test report page item.
        /// 
        /// Valid Values:
        /// RSESCI_TREP_HEAD (0) - Header
        /// RSESCI_TREP_DIAG (1) - Diagram
        /// RSESCI_TREP_FRES (2) - Final Results
        /// RSESCI_TREP_PAGE (3) - Page Numbering
        /// RSESCI_TREP_STAB (4) - Scan Table
        /// RSESCI_TREP_SRES (5) - Scan Results
        /// RSESCI_TREP_TDST (6) - Time and Date
        /// RSESCI_TREP_TRAN (7) - Transducer Factor
        /// 
        /// Default Value: RSESCI_TREP_HEAD (0)
        /// 
        /// Note(s):
        /// 
        /// (1) Header - switches the output of the report header either
        /// on or off. The individual lines can also be controlled individually with ":HCOP:TREP:ITEM:HEAD:LINE:CONT ALW | ONCE | NEV".
        /// 
        /// (2) Diagram - switches the output of the diagram on the test
        /// instrument screen either on or off. 
        /// 
        /// (3) Final Results - switches the output of the final measured values in tabular format either on or off.
        /// 
        /// (4) Page Numbering - switches the output of the page numbering either on or off.
        /// 
        /// (5) Scan Table - switches the output of the scan table with the
        /// measurement settings either on or off.
        /// 
        /// (6) Scan Results - switches the output of the results on the
        /// preliminary measurement in tabular format either on or off. Depending on the current scan settings, this table can be very long in certain cases.
        /// 
        /// (7) Date and Time - switches the output of the date and time of day at the start of the documented measurement either on or off.
        /// 
        /// (8) Transducer Factor - switches the output of the enabled transducer factor(s) either on or off.
        /// </param>
        /// <param name="State">
        /// This control sets the state of the selected page item of the test report.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// 
        /// Default Value: VI_FALSE (0)
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code
        /// either indicates success or describes an error or warning
        /// condition. You examine the status code from each call to an
        /// instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the
        /// rsfsq_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        /// 
        /// Status    Description
        /// -------------------------------------------------
        /// 
        /// BFFC0002  Parameter 2 (Number of Channel Tables) is null pointer.
        /// BFFC0003  Parameter 3 (Buffer Size) out of range.
        /// BFFC0004  Parameter 4 (Channel Tables List) is null pointer.
        /// 
        /// BFFC09F0  Instrument status error
        /// BFFC09F1  Option not installed
        /// BFFC09F2  Conflict with setup parameters
        /// BFFC09F3  Data are not available
        /// BFFC09F4  Unknown option
        /// BFFC09F5  Calibration failed
        /// BFFC09F6  Function does not support FSEx parameter value
        /// BFFC09F7  Overload. Instrument does not return I/Q values
        /// 
        /// This instrument driver also returns errors and warnings defined
        /// by other sources. The following table defines the ranges of
        /// additional status codes that this driver can return. The table
        /// lists the different include files that contain the defined
        /// constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int ConfigureTestReportPageSettings(int Item_Selector, bool State)
        {
            int pInvokeResult = PInvoke.ConfigureTestReportPageSettings(this._handle, Item_Selector, System.Convert.ToUInt16(State));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the page settings of the test report.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analyzer
        /// 
        /// Remote-control command(s):
        /// HCOPy:TREPort:ITEM:HEADer:STATe?
        /// HCOPy:TREPort:ITEM:DIAGram:STATe?
        /// HCOPy:TREPort:ITEM:FRESults:STATe?
        /// HCOPy:TREPort:ITEM:PAGecount:STATe?
        /// HCOPy:TREPort:ITEM:SCANtable:STATe?
        /// HCOPy:TREPort:ITEM:SRESults:STATe?
        /// HCOPy:TREPort:ITEM:TDSTamp:STATe?
        /// HCOPy:TREPort:ITEM:TRANsducer:STATe?
        /// </summary>
        /// <param name="Item_Selector">
        /// This control selects the test report page item.
        /// 
        /// Valid Values:
        /// RSESCI_TREP_HEAD (0) - Header
        /// RSESCI_TREP_DIAG (1) - Diagram
        /// RSESCI_TREP_FRES (2) - Final Results
        /// RSESCI_TREP_PAGE (3) - Page Numbering
        /// RSESCI_TREP_STAB (4) - Scan Table
        /// RSESCI_TREP_SRES (5) - Scan Results
        /// RSESCI_TREP_TDST (6) - Time and Date
        /// RSESCI_TREP_TRAN (7) - Transducer Factor
        /// 
        /// Default Value: RSESCI_TREP_HEAD (0)
        /// 
        /// Note(s):
        /// 
        /// (1) Header - switches the output of the report header either
        /// on or off. The individual lines can also be controlled individually with ":HCOP:TREP:ITEM:HEAD:LINE:CONT ALW | ONCE | NEV".
        /// 
        /// (2) Diagram - switches the output of the diagram on the test
        /// instrument screen either on or off. 
        /// 
        /// (3) Final Results - switches the output of the final measured values in tabular format either on or off.
        /// 
        /// (4) Page Numbering - switches the output of the page numbering either on or off.
        /// 
        /// (5) Scan Table - switches the output of the scan table with the
        /// measurement settings either on or off.
        /// 
        /// (6) Scan Results - switches the output of the results on the
        /// preliminary measurement in tabular format either on or off. Depending on the current scan settings, this table can be very long in certain cases.
        /// 
        /// (7) Date and Time - switches the output of the date and time of day at the start of the documented measurement either on or off.
        /// 
        /// (8) Transducer Factor - switches the output of the enabled transducer factor(s) either on or off.
        /// </param>
        /// <param name="State">
        /// This control gets the state of the selected page item of the test report.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code
        /// either indicates success or describes an error or warning
        /// condition. You examine the status code from each call to an
        /// instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the
        /// rsfsq_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        /// 
        /// Status    Description
        /// -------------------------------------------------
        /// 
        /// BFFC0002  Parameter 2 (Number of Channel Tables) is null pointer.
        /// BFFC0003  Parameter 3 (Buffer Size) out of range.
        /// BFFC0004  Parameter 4 (Channel Tables List) is null pointer.
        /// 
        /// BFFC09F0  Instrument status error
        /// BFFC09F1  Option not installed
        /// BFFC09F2  Conflict with setup parameters
        /// BFFC09F3  Data are not available
        /// BFFC09F4  Unknown option
        /// BFFC09F5  Calibration failed
        /// BFFC09F6  Function does not support FSEx parameter value
        /// BFFC09F7  Overload. Instrument does not return I/Q values
        /// 
        /// This instrument driver also returns errors and warnings defined
        /// by other sources. The following table defines the ranges of
        /// additional status codes that this driver can return. The table
        /// lists the different include files that contain the defined
        /// constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int QueryTestReportPageSettings(int Item_Selector, out bool State)
        {
            ushort StateAsUShort;
            int pInvokeResult = PInvoke.QueryTestReportPageSettings(this._handle, Item_Selector, out StateAsUShort);
            State = System.Convert.ToBoolean(StateAsUShort);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function controls whether the line indicated in the numeric
        /// suffix of the report header is output. 
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with R&amp;S ESU instrument.
        /// 
        /// (2) A further condition for the output is that outputting of the report header must be switched on with "HCOP:TREP:ITEM:HEAD:STAT ON" (function rsesci_ConfigureTestReportPageSettings).
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analyzer
        /// 
        /// Remote-control command(s):
        /// HCOPy:TREPort:ITEM:HEADer:LINE&lt;1...7&gt;:CONTrol ALWays | ONCE | NEVer
        /// </summary>
        /// <param name="Line_Number">
        /// This control sets the line number.
        /// 
        /// Valid Values:
        /// 1 to 7
        /// 
        /// Default Value: 1
        /// 
        /// Note(s):
        /// 
        /// (1) This control is ignored when Logo State is selected by Item Selector.
        /// </param>
        /// <param name="State">
        /// This control sets the state of the test report header.
        /// 
        /// Valid Values:
        /// RSESCI_TREP_ALW  (0) - Always
        /// RSESCI_TREP_ONCE (1) - Once
        /// RSESCI_TREP_NEV  (2) - Never
        /// 
        /// Default Value: RSESCI_TREP_ALW  (0)
        /// 
        /// 
        /// Note(s):
        /// 
        /// (1) The meanings are as follows:
        /// ALWays - On each page
        /// ONCE   - Only on the first page
        /// NEVer  - Not on any page
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code
        /// either indicates success or describes an error or warning
        /// condition. You examine the status code from each call to an
        /// instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the
        /// rsfsq_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        /// 
        /// Status    Description
        /// -------------------------------------------------
        /// 
        /// BFFC0002  Parameter 2 (Number of Channel Tables) is null pointer.
        /// BFFC0003  Parameter 3 (Buffer Size) out of range.
        /// BFFC0004  Parameter 4 (Channel Tables List) is null pointer.
        /// 
        /// BFFC09F0  Instrument status error
        /// BFFC09F1  Option not installed
        /// BFFC09F2  Conflict with setup parameters
        /// BFFC09F3  Data are not available
        /// BFFC09F4  Unknown option
        /// BFFC09F5  Calibration failed
        /// BFFC09F6  Function does not support FSEx parameter value
        /// BFFC09F7  Overload. Instrument does not return I/Q values
        /// 
        /// This instrument driver also returns errors and warnings defined
        /// by other sources. The following table defines the ranges of
        /// additional status codes that this driver can return. The table
        /// lists the different include files that contain the defined
        /// constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetTestReportHeaderLineState(int Line_Number, int State)
        {
            int pInvokeResult = PInvoke.SetTestReportHeaderLineState(this._handle, Line_Number, State);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns whether the line indicated in the numeric
        /// suffix of the report header is output. 
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analyzer
        /// 
        /// Remote-control command(s):
        /// HCOPy:TREPort:ITEM:HEADer:LINE&lt;1...7&gt;:CONTrol?
        /// </summary>
        /// <param name="Line_Number">
        /// This control sets the line number.
        /// 
        /// Valid Values:
        /// 1 to 7
        /// 
        /// Default Value: 1
        /// 
        /// Note(s):
        /// 
        /// (1) This control is ignored when Logo State is selected by Item Selector.
        /// </param>
        /// <param name="State">
        /// This control returns whether the line indicated in the numeric
        /// suffix of the report header is output.
        /// 
        /// Valid Values:
        /// RSESCI_TREP_ALW  (0) - Always
        /// RSESCI_TREP_ONCE (1) - Once
        /// RSESCI_TREP_NEV  (2) - Never
        /// 
        /// Note(s):
        /// 
        /// (1) The meanings are as follows:
        /// ALWays - On each page
        /// ONCE   - Only on the first page
        /// NEVer  - Not on any page
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code
        /// either indicates success or describes an error or warning
        /// condition. You examine the status code from each call to an
        /// instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the
        /// rsfsq_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        /// 
        /// Status    Description
        /// -------------------------------------------------
        /// 
        /// BFFC0002  Parameter 2 (Number of Channel Tables) is null pointer.
        /// BFFC0003  Parameter 3 (Buffer Size) out of range.
        /// BFFC0004  Parameter 4 (Channel Tables List) is null pointer.
        /// 
        /// BFFC09F0  Instrument status error
        /// BFFC09F1  Option not installed
        /// BFFC09F2  Conflict with setup parameters
        /// BFFC09F3  Data are not available
        /// BFFC09F4  Unknown option
        /// BFFC09F5  Calibration failed
        /// BFFC09F6  Function does not support FSEx parameter value
        /// BFFC09F7  Overload. Instrument does not return I/Q values
        /// 
        /// This instrument driver also returns errors and warnings defined
        /// by other sources. The following table defines the ranges of
        /// additional status codes that this driver can return. The table
        /// lists the different include files that contain the defined
        /// constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetTestReportHeaderLineState(int Line_Number, out int State)
        {
            int pInvokeResult = PInvoke.GetTestReportHeaderLineState(this._handle, Line_Number, out State);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches the output of the diagram on the test
        /// instrument screen either on or off.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with R&amp;S ESU instrument.
        /// 
        /// (2) A further condition for the output is that outputting of the report header must be switched on with "HCOP:TREP:ITEM:HEAD:STAT ON" (function rsesci_ConfigureTestReportPageSettings).
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analyzer
        /// 
        /// Remote-control command(s):
        /// HCOPy:TREPort:ITEM:LOGO:CONTrol ALWays | ONCE | NEVer
        /// </summary>
        /// <param name="State">
        /// This control switches the output of the diagram on the test
        /// instrument screen either on or off.
        /// 
        /// Valid Values:
        /// RSESCI_TREP_ALW  (0) - Always
        /// RSESCI_TREP_ONCE (1) - Once
        /// RSESCI_TREP_NEV  (2) - Never
        /// 
        /// Default Value: RSESCI_TREP_NEV  (2)
        /// 
        /// 
        /// Note(s):
        /// 
        /// (1) The meanings are as follows:
        /// ALWays - On each page
        /// ONCE   - Only on the first page
        /// NEVer  - Not on any page
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code
        /// either indicates success or describes an error or warning
        /// condition. You examine the status code from each call to an
        /// instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the
        /// rsfsq_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        /// 
        /// Status    Description
        /// -------------------------------------------------
        /// 
        /// BFFC0002  Parameter 2 (Number of Channel Tables) is null pointer.
        /// BFFC0003  Parameter 3 (Buffer Size) out of range.
        /// BFFC0004  Parameter 4 (Channel Tables List) is null pointer.
        /// 
        /// BFFC09F0  Instrument status error
        /// BFFC09F1  Option not installed
        /// BFFC09F2  Conflict with setup parameters
        /// BFFC09F3  Data are not available
        /// BFFC09F4  Unknown option
        /// BFFC09F5  Calibration failed
        /// BFFC09F6  Function does not support FSEx parameter value
        /// BFFC09F7  Overload. Instrument does not return I/Q values
        /// 
        /// This instrument driver also returns errors and warnings defined
        /// by other sources. The following table defines the ranges of
        /// additional status codes that this driver can return. The table
        /// lists the different include files that contain the defined
        /// constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetTestReportHeaderLogoState(int State)
        {
            int pInvokeResult = PInvoke.SetTestReportHeaderLogoState(this._handle, State);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the state of the output of the diagram on the test instrument screen.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analyzer
        /// 
        /// Remote-control command(s):
        /// HCOPy:TREPort:ITEM:LOGO:CONTrol?
        /// </summary>
        /// <param name="State">
        /// This control returns the state of the output of the diagram on the test instrument screen.
        /// 
        /// Valid Values:
        /// RSESCI_TREP_ALW  (0) - Always
        /// RSESCI_TREP_ONCE (1) - Once
        /// RSESCI_TREP_NEV  (2) - Never
        /// 
        /// Note(s):
        /// 
        /// (1) The meanings are as follows:
        /// ALWays - On each page
        /// ONCE   - Only on the first page
        /// NEVer  - Not on any page
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code
        /// either indicates success or describes an error or warning
        /// condition. You examine the status code from each call to an
        /// instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the
        /// rsfsq_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        /// 
        /// Status    Description
        /// -------------------------------------------------
        /// 
        /// BFFC0002  Parameter 2 (Number of Channel Tables) is null pointer.
        /// BFFC0003  Parameter 3 (Buffer Size) out of range.
        /// BFFC0004  Parameter 4 (Channel Tables List) is null pointer.
        /// 
        /// BFFC09F0  Instrument status error
        /// BFFC09F1  Option not installed
        /// BFFC09F2  Conflict with setup parameters
        /// BFFC09F3  Data are not available
        /// BFFC09F4  Unknown option
        /// BFFC09F5  Calibration failed
        /// BFFC09F6  Function does not support FSEx parameter value
        /// BFFC09F7  Overload. Instrument does not return I/Q values
        /// 
        /// This instrument driver also returns errors and warnings defined
        /// by other sources. The following table defines the ranges of
        /// additional status codes that this driver can return. The table
        /// lists the different include files that contain the defined
        /// constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetTestReportHeaderLogoState(out int State)
        {
            int pInvokeResult = PInvoke.GetTestReportHeaderLogoState(this._handle, out State);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines a name for the line of the report header indicated in the numeric suffix. This name is also output in the test report.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with R&amp;S ESU instrument.
        /// 
        /// (2) A further condition for the output is that outputting of the report header must be switched on with "HCOP:TREP:ITEM:HEAD:STAT ON" (function rsesci_ConfigureTestReportPageSettings).
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analyzer
        /// 
        /// Remote-control command(s):
        /// HCOPy:TREPort:ITEM:HEADer:LINE&lt;1...7&gt;:TITLe &lt;string&gt;
        /// </summary>
        /// <param name="Line_Number">
        /// This control sets the line number.
        /// 
        /// Valid Values:
        /// 1 to 7
        /// 
        /// Default Value: 1
        /// 
        /// Note(s):
        /// 
        /// (1) This control is ignored when Logo State is selected by Item Selector.
        /// </param>
        /// <param name="Line_Title">
        /// This control sets the line title.
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: ""
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code
        /// either indicates success or describes an error or warning
        /// condition. You examine the status code from each call to an
        /// instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the
        /// rsfsq_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        /// 
        /// Status    Description
        /// -------------------------------------------------
        /// 
        /// BFFC0002  Parameter 2 (Number of Channel Tables) is null pointer.
        /// BFFC0003  Parameter 3 (Buffer Size) out of range.
        /// BFFC0004  Parameter 4 (Channel Tables List) is null pointer.
        /// 
        /// BFFC09F0  Instrument status error
        /// BFFC09F1  Option not installed
        /// BFFC09F2  Conflict with setup parameters
        /// BFFC09F3  Data are not available
        /// BFFC09F4  Unknown option
        /// BFFC09F5  Calibration failed
        /// BFFC09F6  Function does not support FSEx parameter value
        /// BFFC09F7  Overload. Instrument does not return I/Q values
        /// 
        /// This instrument driver also returns errors and warnings defined
        /// by other sources. The following table defines the ranges of
        /// additional status codes that this driver can return. The table
        /// lists the different include files that contain the defined
        /// constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetTestReportHeaderLineTitle(int Line_Number, string Line_Title)
        {
            int pInvokeResult = PInvoke.SetTestReportHeaderLineTitle(this._handle, Line_Number, Line_Title);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns a name for the line of the report header indicated in the numeric suffix.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analyzer
        /// 
        /// Remote-control command(s):
        /// HCOPy:TREPort:ITEM:HEADer:LINE&lt;1...7&gt;:TITLe?
        /// </summary>
        /// <param name="Line_Number">
        /// This control sets the line number.
        /// 
        /// Valid Values:
        /// 1 to 7
        /// 
        /// Default Value: 1
        /// 
        /// Note(s):
        /// 
        /// (1) This control is ignored when Logo State is selected by Item Selector.
        /// </param>
        /// <param name="Array_Size">
        /// This control defines the size of array passed to argument Line Title.
        /// 
        /// Valid Values:
        /// not checked
        /// 
        /// Default Value: 256
        /// </param>
        /// <param name="Line_Title">
        /// This control gets the line title.
        /// 
        /// Valid Values: not checked
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code
        /// either indicates success or describes an error or warning
        /// condition. You examine the status code from each call to an
        /// instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the
        /// rsfsq_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        /// 
        /// Status    Description
        /// -------------------------------------------------
        /// 
        /// BFFC0002  Parameter 2 (Number of Channel Tables) is null pointer.
        /// BFFC0003  Parameter 3 (Buffer Size) out of range.
        /// BFFC0004  Parameter 4 (Channel Tables List) is null pointer.
        /// 
        /// BFFC09F0  Instrument status error
        /// BFFC09F1  Option not installed
        /// BFFC09F2  Conflict with setup parameters
        /// BFFC09F3  Data are not available
        /// BFFC09F4  Unknown option
        /// BFFC09F5  Calibration failed
        /// BFFC09F6  Function does not support FSEx parameter value
        /// BFFC09F7  Overload. Instrument does not return I/Q values
        /// 
        /// This instrument driver also returns errors and warnings defined
        /// by other sources. The following table defines the ranges of
        /// additional status codes that this driver can return. The table
        /// lists the different include files that contain the defined
        /// constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetTestReportHeaderLineTitle(int Line_Number, int Array_Size, System.Text.StringBuilder Line_Title)
        {
            int pInvokeResult = PInvoke.GetTestReportHeaderLineTitle(this._handle, Line_Number, Array_Size, Line_Title);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function defines the text used for the line of the report header indicated in the numeric suffix.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with R&amp;S ESU instrument.
        /// 
        /// (2) A further condition for the output is that outputting of the report header must be switched on with "HCOP:TREP:ITEM:HEAD:STAT ON" (function rsesci_ConfigureTestReportPageSettings).
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analyzer
        /// 
        /// Remote-control command(s):
        /// HCOPy:TREPort:ITEM:HEADer:LINE&lt;1...7&gt;:TEXT &lt;string&gt;
        /// </summary>
        /// <param name="Line_Number">
        /// This control sets the line number.
        /// 
        /// Valid Values:
        /// 1 to 7
        /// 
        /// Default Value: 1
        /// 
        /// Note(s):
        /// 
        /// (1) This control is ignored when Logo State is selected by Item Selector.
        /// </param>
        /// <param name="Line_Text">
        /// This control sets the line text.
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: ""
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code
        /// either indicates success or describes an error or warning
        /// condition. You examine the status code from each call to an
        /// instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the
        /// rsfsq_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        /// 
        /// Status    Description
        /// -------------------------------------------------
        /// 
        /// BFFC0002  Parameter 2 (Number of Channel Tables) is null pointer.
        /// BFFC0003  Parameter 3 (Buffer Size) out of range.
        /// BFFC0004  Parameter 4 (Channel Tables List) is null pointer.
        /// 
        /// BFFC09F0  Instrument status error
        /// BFFC09F1  Option not installed
        /// BFFC09F2  Conflict with setup parameters
        /// BFFC09F3  Data are not available
        /// BFFC09F4  Unknown option
        /// BFFC09F5  Calibration failed
        /// BFFC09F6  Function does not support FSEx parameter value
        /// BFFC09F7  Overload. Instrument does not return I/Q values
        /// 
        /// This instrument driver also returns errors and warnings defined
        /// by other sources. The following table defines the ranges of
        /// additional status codes that this driver can return. The table
        /// lists the different include files that contain the defined
        /// constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int SetTestReportHeaderLineText(int Line_Number, string Line_Text)
        {
            int pInvokeResult = PInvoke.SetTestReportHeaderLineText(this._handle, Line_Number, Line_Text);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the text used for the line of the report header indicated in the numeric suffix. 
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analyzer
        /// 
        /// Remote-control command(s):
        /// HCOPy:TREPort:ITEM:HEADer:LINE&lt;1...7&gt;:TEXT?
        /// </summary>
        /// <param name="Line_Number">
        /// This control sets the line number.
        /// 
        /// Valid Values:
        /// 1 to 7
        /// 
        /// Default Value: 1
        /// 
        /// Note(s):
        /// 
        /// (1) This control is ignored when Logo State is selected by Item Selector.
        /// </param>
        /// <param name="Array_Size">
        /// This control defines the size of array passed to argument Line Text.
        /// 
        /// Valid Values:
        /// not checked
        /// 
        /// Default Value: 256
        /// </param>
        /// <param name="Line_Text">
        /// This control gets the line text.
        /// 
        /// Valid Values: not checked
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code
        /// either indicates success or describes an error or warning
        /// condition. You examine the status code from each call to an
        /// instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the
        /// rsfsq_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        /// 
        /// Status    Description
        /// -------------------------------------------------
        /// 
        /// BFFC0002  Parameter 2 (Number of Channel Tables) is null pointer.
        /// BFFC0003  Parameter 3 (Buffer Size) out of range.
        /// BFFC0004  Parameter 4 (Channel Tables List) is null pointer.
        /// 
        /// BFFC09F0  Instrument status error
        /// BFFC09F1  Option not installed
        /// BFFC09F2  Conflict with setup parameters
        /// BFFC09F3  Data are not available
        /// BFFC09F4  Unknown option
        /// BFFC09F5  Calibration failed
        /// BFFC09F6  Function does not support FSEx parameter value
        /// BFFC09F7  Overload. Instrument does not return I/Q values
        /// 
        /// This instrument driver also returns errors and warnings defined
        /// by other sources. The following table defines the ranges of
        /// additional status codes that this driver can return. The table
        /// lists the different include files that contain the defined
        /// constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetTestReportHeaderLineText(int Line_Number, int Array_Size, System.Text.StringBuilder Line_Text)
        {
            int pInvokeResult = PInvoke.GetTestReportHeaderLineText(this._handle, Line_Number, Array_Size, Line_Text);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function configures the test report operations. 
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analyzer
        /// 
        /// Remote-control command(s):
        /// HCOPy:TREPort:ITEM:TEMPlate:LOAD &lt;string&gt;
        /// HCOPy:TREPort:ITEM:TEMPlate:SAVE &lt;string&gt;
        /// HCOPy:TREPort:ITEM:TEMPlate:DELete &lt;string&gt;
        /// </summary>
        /// <param name="Operation">
        /// This control selects the operation.
        /// 
        /// Valid Values:
        /// RSESCI_TREP_LOAD (0) - Load
        /// RSESCI_TREP_SAVE (1) - Save
        /// RSESCI_TREP_DEL  (2) - Delete
        /// 
        /// Default Value: RSESCI_TREP_LOAD (0)
        /// 
        /// Note(s):
        /// 
        /// (1) Load - loads the specified test report configuration.
        /// 
        /// (2) Save - saves the current test report configuration using the specified name.
        /// 
        /// (3) Delete - deletes the specified test report configuration.
        /// </param>
        /// <param name="Test_Report_Name">
        /// This control sets the test report name.
        /// 
        /// Valid Values: not checked
        /// 
        /// Default Value: ""
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code
        /// either indicates success or describes an error or warning
        /// condition. You examine the status code from each call to an
        /// instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the
        /// rsfsq_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        /// 
        /// Status    Description
        /// -------------------------------------------------
        /// 
        /// BFFC0002  Parameter 2 (Number of Channel Tables) is null pointer.
        /// BFFC0003  Parameter 3 (Buffer Size) out of range.
        /// BFFC0004  Parameter 4 (Channel Tables List) is null pointer.
        /// 
        /// BFFC09F0  Instrument status error
        /// BFFC09F1  Option not installed
        /// BFFC09F2  Conflict with setup parameters
        /// BFFC09F3  Data are not available
        /// BFFC09F4  Unknown option
        /// BFFC09F5  Calibration failed
        /// BFFC09F6  Function does not support FSEx parameter value
        /// BFFC09F7  Overload. Instrument does not return I/Q values
        /// 
        /// This instrument driver also returns errors and warnings defined
        /// by other sources. The following table defines the ranges of
        /// additional status codes that this driver can return. The table
        /// lists the different include files that contain the defined
        /// constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int TestReportOperations(int Operation, string Test_Report_Name)
        {
            int pInvokeResult = PInvoke.TestReportOperations(this._handle, Operation, Test_Report_Name);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function selects a graphics file which will appear as the logo in the header of the test report.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// All
        /// 
        /// Remote-control command(s):
        /// MMEMory:LOGO:CDIRectory &lt;directory_name&gt;
        /// MMEMory:LOGO:NAME &lt;file_name&gt;
        /// </summary>
        /// <param name="Directory">
        /// This control selects the directory in which the graphics file is saved which contains the logo for the test report. The indication of the directory must contain the drive name in addition to the path indication. That path indication should comply with DOS conventions.
        /// 
        /// Valid Range: not tested
        /// 
        /// Default Value: "D:\\user\\data"
        /// 
        /// </param>
        /// <param name="File_Name">
        /// This control selects a graphics file which will appear as the logo in the header of the test report. The file name should comply with DOS conventions.
        /// 
        /// Valid Range: not tested
        /// 
        /// Default Value: ""
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int LoadLogo(string Directory, string File_Name)
        {
            int pInvokeResult = PInvoke.LoadLogo(this._handle, Directory, File_Name);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function outputs a list showing the complete test report configuration.
        /// 
        /// Note(s):
        /// 
        /// (1) This function is available only with R&amp;S ESU instrument.
        /// 
        /// Mode(s):
        /// Receiver
        /// Spectrum Analyzer
        /// 
        /// Remote-control command(s):
        /// HCOPy:TREPort:ITEM:TEMPlate:CATalog?
        /// </summary>
        /// <param name="Number_of_Test_Reports">
        /// This control returns number of names of all test reports.
        /// </param>
        /// <param name="Buffer_Size">
        /// Pass the number of bytes in the string you specify for the catalog list parameter.
        /// 
        /// The buffer must contain at least as many elements as the value you specify with the Buffer Size parameter. If the value contains more bytes than you specify with the Buffer Size parameter, the function copies Buffer Size - 1 bytes into the buffer and returns the buffer size you must pass to get the entire value. For example, if the value is "file1,file2" and the Buffer Size is 6, the function places "file1" into the buffer and returns 12.
        /// 
        /// </param>
        /// <param name="Test_Report_List">
        /// This control returns the names of all test reports.
        /// 
        /// The buffer must contain at least as many elements as the value you specify with the Buffer Size parameter. If the value contains more bytes than you specify with the Buffer Size parameter, the function copies Buffer Size - 1 bytes into the buffer and returns the buffer size you must pass to get the entire value. For example, if the value is "file1,file2" and the Buffer Size is 6, the function places "file1" into the buffer and returns 12.
        /// 
        /// if you pass 0 for buffer Size, you can use VI_NULL for this parameter.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code
        /// either indicates success or describes an error or warning
        /// condition. You examine the status code from each call to an
        /// instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the
        /// rsfsq_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This driver defines the following status codes:
        /// 
        /// Status    Description
        /// -------------------------------------------------
        /// 
        /// BFFC0002  Parameter 2 (Number of Channel Tables) is null pointer.
        /// BFFC0003  Parameter 3 (Buffer Size) out of range.
        /// BFFC0004  Parameter 4 (Channel Tables List) is null pointer.
        /// 
        /// BFFC09F0  Instrument status error
        /// BFFC09F1  Option not installed
        /// BFFC09F2  Conflict with setup parameters
        /// BFFC09F3  Data are not available
        /// BFFC09F4  Unknown option
        /// BFFC09F5  Calibration failed
        /// BFFC09F6  Function does not support FSEx parameter value
        /// BFFC09F7  Overload. Instrument does not return I/Q values
        /// 
        /// This instrument driver also returns errors and warnings defined
        /// by other sources. The following table defines the ranges of
        /// additional status codes that this driver can return. The table
        /// lists the different include files that contain the defined
        /// constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int GetTestReportCatalog(out int Number_of_Test_Reports, int Buffer_Size, System.Text.StringBuilder Test_Report_List)
        {
            int pInvokeResult = PInvoke.GetTestReportCatalog(this._handle, out Number_of_Test_Reports, Buffer_Size, Test_Report_List);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function switches state checking of the instrument (reading of the Standard Event Register and checking it for error) status subsystem. Driver functions are using state checking which is by default enabled.
        /// 
        /// Note:
        /// 
        /// (1) In debug mode enable state checking.
        /// 
        /// (2) For better bus throughput and instruments performance disable state checking.
        /// 
        /// (3) When state checking is disabled driver does not check if correct instrument model or option is used with each of the functions. This might cause unexpected behaviour of the instrument.
        /// 
        /// </summary>
        /// <param name="State_Checking">
        /// This control switches instrument state checking On or Off.
        /// 
        /// Valid Values:
        /// VI_FALSE (0) - Off
        /// VI_TRUE  (1) - On (Default Value)
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int errorCheckState(bool State_Checking)
        {
            int pInvokeResult = PInvoke.errorCheckState(this._handle, System.Convert.ToUInt16(State_Checking));
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function sets the Enable, NTransition, and PTransition bits     of the Main, Power, Limit, LMargin, ACP Limit, Frequency, status questionable and operating registers.
        /// 
        /// Note:
        /// 
        /// (1) For detailed description of Status Reporting System see also operating manual.
        /// 
        /// Remote-control command(s):
        /// STATus:PRESet
        /// STATus:QUEStionable:POWer:ENABle
        /// STATus:QUEStionable:LIMit&lt;1|2&gt;:ENABle 
        /// STATus:QUEStionable:LMARgin&lt;1|2&gt;:ENABle 
        /// STATus:QUEStionable:SYNC:ENABle 
        /// STATus:QUEStionable:ACPLimit:ENABle 
        /// STATus:QUEStionable:FREQuency:ENABle 
        /// STATus:QUEStionable:TRANsducer:ENABle
        /// STATus:QUEStionable:POWer:PTRansition
        /// STATus:QUEStionable:LIMit&lt;1|2&gt;:PTRansition 
        /// STATus:QUEStionable:LMARgin&lt;1|2&gt;:PTRansition 
        /// STATus:QUEStionable:SYNC:PTRansition 
        /// STATus:QUEStionable:ACPLimit:PTRansition 
        /// STATus:QUEStionable:FREQuency:PTRansition 
        /// STATus:QUEStionable:TRANsducer:PTRansition
        /// STATus:QUEStionable:POWer:NTRansition
        /// STATus:QUEStionable:LIMit&lt;1|2&gt;:NTRansition 
        /// STATus:QUEStionable:LMARgin&lt;1|2&gt;:NTRansition 
        /// STATus:QUEStionable:SYNC:NTRansition 
        /// STATus:QUEStionable:ACPLimit:NTRansition 
        /// STATus:QUEStionable:FREQuency:NTRansition 
        /// STATus:QUEStionable:TRANsducer:NTRansition
        /// STATus:QUEStionable:ENABle
        /// STATus:QUEStionable:PTRansition
        /// STATus:QUEStionable:NTRansition
        /// STATus:OPERation:ENABle
        /// STATus:OPERation:PTRansition
        /// STATus:OPERation:NTRansition
        /// </summary>
        /// <param name="Register_Operation">
        /// Specifies the operation to perform.
        /// 
        /// Valid Range:
        /// 0 - Preset Registers (Default Value)
        /// 1 - Set Questionable Register
        /// 2 - Set Operation Register
        /// 
        /// Notes:
        /// 
        /// (1) Preset Registers:
        /// This command resets the edge detectors and ENABle parts of all registers to a defined value. All PTRansition parts are set to FFFFh, i.e., all transitions from 0 to 1 are detected. All NTRansition parts are set to 0, i.e., a transition from 1 to 0 in a CONDition bit is not detected. The ENABle part of the STATus:OPERation and STATus:QUEStionable registers are set to 0, i.e., all events in these registers are not passed on.
        /// 
        /// </param>
        /// <param name="Questionable_Register">
        /// This control selects the registers that will be configured. 
        /// 
        /// Valid Range:
        /// 0 - Main(Default Value)
        /// 1 - Power
        /// 2 - Limit Screen A
        /// 3 - LMargin Screen A
        /// 4 - Sync
        /// 5 - ACP Limit
        /// 6 - Frequency
        /// 7 - Limit Screen B
        /// 8 - LMargin Screen B
        /// 9 - Transducer
        /// 
        /// </param>
        /// <param name="Enable">
        /// This control sets the enable bits of the status register.  
        /// 
        /// Valid range: 0 to 65535
        /// 
        /// Default Value: 0
        /// 
        /// Note:
        /// 
        /// (1) The ENABle register selectively enables the individual events of the associated EVENt section for the sum bit in the status byte.
        /// 
        /// </param>
        /// <param name="PTransition">
        /// This control sets the PTransition bits (edge detectors) of the status register.  
        /// 
        /// Valid range: 0 to 65535
        /// 
        /// Default Value: 0
        /// 
        /// </param>
        /// <param name="NTransition">
        /// This control sets the NTransition bits (edge detectors) of the status register.  
        /// 
        /// Valid range: 0 to 65535
        /// 
        /// Default Value: 0
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int setStatusRegister(int Register_Operation, int Questionable_Register, int Enable, int PTransition, int NTransition)
        {
            int pInvokeResult = PInvoke.setStatusRegister(this._handle, Register_Operation, Questionable_Register, Enable, PTransition, NTransition);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function contains the commands for the querying status reporting system.
        /// 
        /// Note:
        /// 
        /// For detailed description of Status Reporting System see also operating manual.
        /// 
        /// Remote-control command(s):
        /// STATus:PRESet
        /// STATus:OPERation[:EVENt]?
        /// STATus:OPERation:CONDition?
        /// STATus:QUEStionable:EVENt?
        /// STATus:QUEStionable:CONDition?
        /// STATus:QUEStionable:POWer[:EVENt]? 
        /// STATus:QUEStionable:POWer:CONDition? 
        /// STATus:QUEStionable:LIMit&lt;1|2&gt;[:EVENt]?
        /// STATus:QUEStionable:LIMit&lt;1|2&gt;:CONDition? 
        /// STATus:QUEStionable:SYNC:EVENt? 
        /// STATus:QUEStionable:SYNC:CONDition? 
        /// STATus:QUEStionable:ACPLimit[:EVENt]? 
        /// STATus:QUEStionable:ACPLimit:CONDition? 
        /// STATus:QUEStionable:FREQuency[:EVENt]? 
        /// STATus:QUEStionable:FREQuency:CONDition? 
        /// STATus:QUEStionable:TRANsducer[:EVENt]? 
        /// STATus:QUEStionable:TRANsducer:CONDition? 
        /// STATus:QUEStionable[:EVENt]?
        /// STATus:QUEStionable:LMARgin&lt;1|2&gt;[:EVENt]? 
        /// STATus:QUEStionable:LMARgin&lt;1|2&gt;:CONDition? 
        /// </summary>
        /// <param name="Status_Registers_Query">
        /// This control selects the registers to be queried.
        /// 
        /// Valid Range:
        /// 0 - Operation Event (Default Value)
        /// 1 - Operation Condition
        /// 2 - Main Event
        /// 3 - Main Condition
        /// 4 - Power Event
        /// 5 - Power Condition
        /// 6 - Limit Event Screen A
        /// 7 - Limit Condition Screen A
        /// 8 - LMargin Event Screen A
        /// 9 - LMargin Condition Screen A
        /// 10 - Sync Event
        /// 11 - Sync Condition
        /// 12 - ACP Limit Event
        /// 13 - ACP Limit Condition
        /// 14 - Frequency Event
        /// 15 - Frequency Condition
        /// 16 - Transducer Event
        /// 17 - Transducer Condition
        /// 18 - Queue Next
        /// 19 - Limit Event Screen B
        /// 20 - Limit Condition Screen B
        /// 21 - LMargin Event Screen B
        /// 22 - LMargin Condition Screen B
        /// 
        /// Notes:
        /// 
        /// (1) Operation Event:
        /// This command queries the contents of the EVENt section of the STATus:OPERation register.
        /// 
        /// (2) Operation Condition:
        /// This command queries the CONDition section of the STATus:OPERation register.
        /// 
        /// (3) Main Event:
        /// This command queries the contents of the EVENt section of the STATus:QUEStionable register.
        /// 
        /// (4) Main Condition:
        /// This command queries the CONDition section of the STATus:QUEStionable register.
        /// 
        /// (5) Power Event:
        /// This command queries the contents of the EVENt section of the STATus:QUEStionable:POWer register.
        /// 
        /// (6) Power Condition:
        /// This command queries the content of the CONDition section of the STATus:QUEStionable:POWer register.
        /// 
        /// (7) Limit Event Screen A(B):
        /// This command queries the contents of the EVENt section of the STATus:QUEStionable:LIMit register.
        /// 
        /// (8) Limit Condition Screen A(B):
        /// This command queries the contents of the CONDition section of the STATus:QUEStionable:LIMit register.
        /// 
        /// (9) LMargin Event Screen A(B):
        /// This command queries the contents of the EVENt section of the STATus:QUEStionable:LMARgin register.
        /// 
        /// (10) LMargin Condition Screen A(B):
        /// This command queries the contents of the CONDition section of the STATus:QUEStionable:LMARgin register.
        /// 
        /// (11) ACP Limit Event:
        /// This command queries the contents of the EVENt section of the STATus:QUEStionable:ACPLimit register.
        /// 
        /// (12) ACP Limit Condition:
        /// This command queries the contents of the CONDition section of the STATus:QUEStionable: ACPLimit register.
        /// 
        /// (13) Frequency Event:
        /// This command queries the contents of the EVENt section of the STATus:QUEStionableFREQuency register.
        /// 
        /// (14) Frequency Condition:
        /// This command queries the contents of the CONDition section of the STATus:QUEStionable:FREQuency register.
        /// 
        /// (15) Queue Next:
        /// This command queries the earliest entry to the error queue, thus deleting it.
        /// 
        /// (16) Sync Event:
        /// This command queries the contents of the EVENt section of the STATus:QUEStionable:SYNC register. 
        /// 
        /// (17) Sync Condition:
        /// This command queries the contents of the CONDition section of the STATus:QUEStionable:SYNC register. 
        /// 
        /// </param>
        /// <param name="Register_Value">
        /// Returns content of selected register.
        /// 
        /// Note:
        /// 
        /// For detailed description of Status Reporting System see also operating manual.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int getStatusRegister(int Status_Registers_Query, out int Register_Value)
        {
            int pInvokeResult = PInvoke.getStatusRegister(this._handle, Status_Registers_Query, out Register_Value);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function writes commands and queries to the instrument to modify parameters and query device settings.
        /// 
        /// </summary>
        /// <param name="Write_Buffer">
        /// The user can use this control to send common commands and queries to the instrument. This control can also be used to write any valid command to the instrument.
        /// 
        /// Default Value:  "*RST"
        /// 
        /// Notes:
        /// 
        /// (1) The command or query to be sent to the instrument may be a literal enclosed in double quotes i.e. "*RST" or may be contained in a variable of type string.
        /// 
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int writeInstrData(string Write_Buffer)
        {
            int pInvokeResult = PInvoke.writeInstrData(this._handle, Write_Buffer);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function reads data from the instrument's output buffer and returns it to the specified variable in memory.
        /// 
        /// Notes:
        /// 
        /// (1) Because the instrument may return both numeric and text data in response to queries this function returns the data in string format.
        /// 
        /// (2) If valid data is not available at the instrument's output buffer when this function is called the instrument will hang up and the function will not return until it times out.  If the time-out is disabled this function will hang indefinitely and it may be necessary to reboot the computer to break out.
        /// 
        /// </summary>
        /// <param name="Number_Bytes_To_Read">
        /// The number of bytes specified by this control should be greater than or equal to the number of bytes which are to be read from the instrument. If the actual number of bytes to be read is greater than the number this control specifies then multiple reads will be required to empty the instrument's output buffer.
        /// 
        /// If the instrument's output buffer is not emptied the instrument may return invalid data when future reads are performed.
        /// 
        /// Default Value:  50 (See NOTE 2)
        /// 
        /// Notes:
        /// 
        /// (1) If NO DATA is available at the instrument's output buffer when this function is called the instrument will hang up and the function will not return until it times out.  If the time-out is disabled, this function will hang indefinitely and it may be necessary to reboot the computer.
        /// 
        /// (2) If the number of bytes expected is greater than 50 the value of this control may be increased. If the actual number of bytes read is larger than the declared size of the read buffer a run-time error will be generated.
        /// 
        /// </param>
        /// <param name="Read_Buffer">
        /// The incoming data from the instrument is placed into this variable.
        /// 
        /// Notes:
        /// 
        /// (1) Because the instrument may return both numeric and text data in response to queries this function returns the data in string format.
        /// 
        /// (2) This function does not overwrite any old data left in the string variable from the last time the function was called. String data in LabWindows/CVI is terminated with an ASCII null(0x0) and string manipulation functions will only recognize data before the ASCII null.
        /// 
        /// (3) The declared size of the string variable must be greater than the actual number of bytes read from the instrument if it is not a run-time error will be generated.
        /// 
        /// </param>
        /// <param name="NumBytes_Read">
        /// This variable contains the actual number of bytes read from the instrument. This is the value which is returned by the read function.
        /// 
        /// Notes:
        /// 
        /// (1) If the actual number of bytes read is less than the number of bytes specified in the Number Bytes To Read control then the output buffer has probably been emptied. If the read function fails and the number of bytes read is 0, the most probable cause for the failure is there was no data available at the instrument's output buffer.
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int readInstrData(int Number_Bytes_To_Read, System.Text.StringBuilder Read_Buffer, out int NumBytes_Read)
        {
            int pInvokeResult = PInvoke.readInstrData(this._handle, Number_Bytes_To_Read, Read_Buffer, out NumBytes_Read);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function resets the instrument to a known state and sends initialization commands to the instrument that set any necessary programmatic variables to the state necessary for the operation of the instrument driver.
        /// 
        /// </summary>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int reset()
        {
            int pInvokeResult = PInvoke.reset(this._handle);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function runs the instrument's self test routine and returns the test result(s).
        /// 
        /// </summary>
        /// <param name="Self_Test_Result">
        /// This control contains the value returned from the instrument self test. Zero means success. For any other code, see the device's operator's manual.
        /// 
        /// </param>
        /// <param name="Self_Test_Message">
        /// This control contains the string returned from the self test. See the device's operation manual for an explanation of the string's contents.
        /// 
        /// Notes:
        /// 
        /// (1) The array must contain at least 256 elements ViChar[256].
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int self_test(out short Self_Test_Result, System.Text.StringBuilder Self_Test_Message)
        {
            int pInvokeResult = PInvoke.self_test(this._handle, out Self_Test_Result, Self_Test_Message);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function reads an error code from the instrument's error queue.
        /// 
        /// </summary>
        /// <param name="Error_Code">
        /// This control returns the error code read from the instrument's error queue.
        /// 
        /// </param>
        /// <param name="Error_Message">
        /// This control returns the error message string read from the instrument's error message queue.
        /// 
        /// Notes:
        /// 
        /// (1) The array must contain at least 256 elements ViChar[256].
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int error_query(out int Error_Code, System.Text.StringBuilder Error_Message)
        {
            int pInvokeResult = PInvoke.error_query(this._handle, out Error_Code, Error_Message);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        /// <summary>
        /// This function returns the revision numbers of the instrument driver and instrument firmware, and tells the user with which  instrument firmware this revision of the driver is compatible.
        /// 
        /// </summary>
        /// <param name="Instrument_Driver_Revision">
        /// This control returns the Instrument Driver Software Revision.
        /// 
        /// Notes:
        /// 
        /// (1) The array must contain at least 256 elements ViChar[256].
        /// </param>
        /// <param name="Firmware_Revision">
        /// This control returns the Instrument Firmware Revision.
        /// 
        /// Notes:
        /// 
        /// (1) The array must contain at least 256 elements ViChar[256].
        /// </param>
        /// <returns>
        /// Returns the status code of this operation. The status code  either indicates success or describes an error or warning condition. You examine the status code from each call to an instrument driver function to determine if an error occurred. To obtain a text description of the status code, call the rsesci_error_message function.
        /// 
        /// The general meaning of the status code is as follows:
        /// 
        /// Value                  Meaning
        /// -------------------------------
        /// 0                      Success
        /// Positive Values        Warnings
        /// Negative Values        Errors
        /// 
        /// This instrument driver also returns errors and warnings defined by other sources. The following table defines the ranges of additional status codes that this driver can return. The table lists the different include files that contain the defined constants for the particular status codes:
        /// 
        /// Numeric Range (in Hex)   Status Code Types    
        /// -------------------------------------------------
        /// 3FFF0000 to 3FFFFFFF     VISA     Warnings
        /// 3FFC0000 to 3FFCFFFF     VXIPnP   Driver Warnings
        /// 
        /// BFFF0000 to BFFFFFFF     VISA     Errors
        /// BFFC0000 to BFFCFFFF     VXIPnP   Driver Errors
        /// 
        /// </returns>
        public int revision_query(System.Text.StringBuilder Instrument_Driver_Revision, System.Text.StringBuilder Firmware_Revision)
        {
            int pInvokeResult = PInvoke.revision_query(this._handle, Instrument_Driver_Revision, Firmware_Revision);
            PInvoke.TestForError(this._handle, pInvokeResult);
            return pInvokeResult;
        }

        public void Dispose()
        {
            this.Dispose(true);
            System.GC.SuppressFinalize(this);
        }

        private void Dispose(bool disposing)
        {
            if ((this._disposed == false))
            {
                PInvoke.close(this._handle);
                this._handle = new System.Runtime.InteropServices.HandleRef(null, System.IntPtr.Zero);
            }
            this._disposed = true;
        }

        private class PInvoke
        {

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_init", CallingConvention = CallingConvention.StdCall)]
            public static extern int init(string Resource_Name, ushort ID_Query, ushort Reset_Device, out System.IntPtr Instrument_Handle);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_ReadSpectrum", CallingConvention = CallingConvention.StdCall)]
            public static extern int ReadSpectrum(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Measurement_Mode, int Frequency_Parameter, double Start_Frequency, double Stop_Frequency, out int Samples_Returned, [In, Out] double[] Trace_Data);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetCenterFrequency", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetCenterFrequency(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, double Center_Frequency);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetCenterFrequency", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetCenterFrequency(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out double Center_Frequency);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetStartFrequency", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetStartFrequency(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, double Start_Frequency);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetStartFrequency", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetStartFrequency(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out double Start_Frequency);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetStopFrequency", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetStopFrequency(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, double Stop_Frequency);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetStopFrequency", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetStopFrequency(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out double Stop_Frequency);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetFrequencyOffset", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetFrequencyOffset(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, double Frequency_Offset);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetFrequencyOffset", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetFrequencyOffset(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out double Frequency_Offset);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetFrequencyMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetFrequencyMode(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Frequency_Mode);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetFrequencyMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetFrequencyMode(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out int Frequency_Mode);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetCenterFrequencyStepWidth", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetCenterFrequencyStepWidth(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, double Step_Width);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetCenterFrequencyStepWidth", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetCenterFrequencyStepWidth(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out double Step_Width);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetCenterFrequencyStepLink", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetCenterFrequencyStepLink(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Step_Link);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetCenterFrequencyStepLink", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetCenterFrequencyStepLink(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out int Step_Link);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetCenterFrequencyStepLinkFactor", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetCenterFrequencyStepLinkFactor(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Step_Link_Factor);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetCenterFrequencyStepLinkFactor", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetCenterFrequencyStepLinkFactor(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out int Step_Link_Factor);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetSignalTracking", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetSignalTracking(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, ushort Signal_Tracking);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetSignalTracking", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetSignalTracking(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out ushort Signal_Tracking);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetSignalTrackingBandwidth", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetSignalTrackingBandwidth(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, double Signal_Tracking_Bandwidth);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetSignalTrackingBandwidth", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetSignalTrackingBandwidth(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out double Signal_Tracking_Bandwidth);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetSignalTrackingThreshold", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetSignalTrackingThreshold(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, double Signal_Tracking_Threshold);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetSignalTrackingThreshold", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetSignalTrackingThreshold(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out double Signal_Tracking_Threshold);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetSignalTrackingTrace", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetSignalTrackingTrace(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Signal_Tracking_Trace);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetSignalTrackingTrace", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetSignalTrackingTrace(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out int Signal_Tracking_Trace);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetFrequencySpan", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetFrequencySpan(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, double Frequency_Span);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetFrequencySpan", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetFrequencySpan(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out double Frequency_Span);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetFrequencyFullSpan", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetFrequencyFullSpan(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetReferenceLevel", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetReferenceLevel(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, double Reference_Level);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetReferenceLevel", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetReferenceLevel(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out double Reference_Level);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetReferenceLevelOffset", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetReferenceLevelOffset(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, double Reference_Level_Offset);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetReferenceLevelOffset", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetReferenceLevelOffset(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out double Reference_Level_Offset);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetReferenceLevelPosition", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetReferenceLevelPosition(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, double Reference_Level_Position);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetReferenceLevelPosition", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetReferenceLevelPosition(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out double Reference_Level_Position);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetReferenceLevelPositionValue", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetReferenceLevelPositionValue(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, double Reference_Level_Position);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetReferenceLevelPositionValue", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetReferenceLevelPositionValue(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out double Reference_Level_Position);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetYAxisScaleDivision", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetYAxisScaleDivision(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, double Y_Scale_Div);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetYAxisScaleDivision", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetYAxisScaleDivision(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out double Y_Scale_Div);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetLevelAxisRange", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetLevelAxisRange(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, double Level_Axis_Range);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetLevelAxisRange", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetLevelAxisRange(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out double Level_Axis_Range);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetLevelAxisMinimum", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetLevelAxisMinimum(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, double Level_Axis_Minimum);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetLevelAxisMinimum", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetLevelAxisMinimum(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out double Level_Axis_Minimum);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetLevelAxisRangeSpacing", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetLevelAxisRangeSpacing(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Level_Axis_Range_Spacing);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetLevelAxisRangeSpacing", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetLevelAxisRangeSpacing(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out int Level_Axis_Range_Spacing);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetLevelAxisUnit", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetLevelAxisUnit(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Level_Axis_Unit);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetLevelAxisUnit", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetLevelAxisUnit(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out int Level_Axis_Unit);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetLevelAxisScaling", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetLevelAxisScaling(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Level_Axis_Scaling);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetLevelAxisScaling", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetLevelAxisScaling(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out int Level_Axis_Scaling);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetRFInputCoupling", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetRFInputCoupling(System.Runtime.InteropServices.HandleRef Instrument_Handle, int RF_Input_Coupling);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetRFInputCoupling", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetRFInputCoupling(System.Runtime.InteropServices.HandleRef Instrument_Handle, out int RF_Input_Coupling);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetRFInputAttenuation", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetRFInputAttenuation(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, double Attenuation);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetRFInputAttenuation", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetRFInputAttenuation(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out double Attenuation);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetRFInputAttenuationAuto", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetRFInputAttenuationAuto(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, ushort Attenuation_Auto);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetRFInputAttenuationAuto", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetRFInputAttenuationAuto(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out ushort Attenuation_Auto);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetRFInputAttenuationProtection", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetRFInputAttenuationProtection(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, ushort Attenuation_Protection);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetRFInputAttenuationProtection", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetRFInputAttenuationProtection(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out ushort Attenuation_Protection);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_ResetRFInputAttenuationProtection", CallingConvention = CallingConvention.StdCall)]
            public static extern int ResetRFInputAttenuationProtection(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetRFInputImpedance", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetRFInputImpedance(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int RF_Input_Impedance);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetRFInputImpedance", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetRFInputImpedance(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out int RF_Input_Impedance);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetRFInputType", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetRFInputType(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Input_Type);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetRFInputType", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetRFInputType(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out int Input_Type);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetPreamplifierAuto", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetPreamplifierAuto(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, ushort Preamplifier_Auto);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetPreamplifierAuto", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetPreamplifierAuto(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out ushort Preamplifier_Auto);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetPreamplifier", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetPreamplifier(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, ushort Preamplifier);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetPreamplifier", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetPreamplifier(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out ushort Preamplifier);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_ConfigureInputMixerLevel", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureInputMixerLevel(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, ushort Automatic_Setup, double Power);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetInputMixerLevelState", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetInputMixerLevelState(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, ushort Automatic_Setup);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetInputMixerLevelState", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetInputMixerLevelState(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out ushort Automatic_Setup);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetInputMixerPower", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetInputMixerPower(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, double Power);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetInputMixerPower", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetInputMixerPower(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out double Power);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetMainPLLBandwidth", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetMainPLLBandwidth(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int PLL_Bandwidth);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetMainPLLBandwidth", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetMainPLLBandwidth(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out int PLL_Bandwidth);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetPLLBandwidthSynchronization", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetPLLBandwidthSynchronization(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int PLL_Bandwidth);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetPLLBandwidthSynchronization", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetPLLBandwidthSynchronization(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out int PLL_Bandwidth);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetFFTFilterMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetFFTFilterMode(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int FFT_Filter_Mode);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetFFTFilterMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetFFTFilterMode(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out int FFT_Filter_Mode);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetIFBandwidth", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetIFBandwidth(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, double IF_Bandwidth);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetIFBandwidth", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetIFBandwidth(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out double IF_Bandwidth);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetDemodulationBandwidth", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetDemodulationBandwidth(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, double Demodulation_Bandwidth);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetDemodulationBandwidth", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetDemodulationBandwidth(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out double Demodulation_Bandwidth);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetResolutionBandwidth", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetResolutionBandwidth(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, double Resolution_Bandwidth);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetResolutionBandwidth", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetResolutionBandwidth(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out double Resolution_Bandwidth);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetResolutionBandwidthAuto", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetResolutionBandwidthAuto(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, ushort Auto_Resolution_Bandwidth);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetResolutionBandwidthAuto", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetResolutionBandwidthAuto(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out ushort Auto_Resolution_Bandwidth);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetResolutionBandwidthCouplingRatio", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetResolutionBandwidthCouplingRatio(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, double Ratio);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetResolutionBandwidthCouplingRatio", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetResolutionBandwidthCouplingRatio(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out double Ratio);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetResolutionBandwidthFilterType", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetResolutionBandwidthFilterType(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Filter_Type);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetResolutionBandwidthFilterType", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetResolutionBandwidthFilterType(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out int Filter_Type);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetVideoBandwidth", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetVideoBandwidth(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, double Video_Bandwidth);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetVideoBandwidth", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetVideoBandwidth(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out double Video_Bandwidth);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetVideoBandwidthAuto", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetVideoBandwidthAuto(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, ushort Auto_Video_Bandwidth);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetVideoBandwidthAuto", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetVideoBandwidthAuto(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out ushort Auto_Video_Bandwidth);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetVideoBandwidthCouplingRatio", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetVideoBandwidthCouplingRatio(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, double Ratio);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetVideoBandwidthCouplingRatio", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetVideoBandwidthCouplingRatio(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out double Ratio);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetVideoBandwidthMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetVideoBandwidthMode(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Bandwidth_Mode);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetVideoBandwidthMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetVideoBandwidthMode(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out int Bandwidth_Mode);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_ContinueSingleSweep", CallingConvention = CallingConvention.StdCall)]
            public static extern int ContinueSingleSweep(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetNumberStartedSweeps", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetNumberStartedSweeps(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out int Started_Scans);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetSweepMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetSweepMode(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Continuous_Sweep);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetSweepMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetSweepMode(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out int Continuous_Sweep);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetSweepTime", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetSweepTime(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, double Sweep_Time);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetSweepTime", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetSweepTime(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out double Sweep_Time);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetSweepTimeAuto", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetSweepTimeAuto(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, ushort Auto_Sweep_Time);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetSweepTimeAuto", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetSweepTimeAuto(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out ushort Auto_Sweep_Time);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetSweepCount", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetSweepCount(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Sweep_Count);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetSweepCount", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetSweepCount(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out int Sweep_Count);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetSweepPoints", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetSweepPoints(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Sweep_Points);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetSweepPoints", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetSweepPoints(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out int Sweep_Points);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetSingleSweepDisplay", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetSingleSweepDisplay(System.Runtime.InteropServices.HandleRef Instrument_Handle, ushort Display_State);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetSingleSweepDisplay", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetSingleSweepDisplay(System.Runtime.InteropServices.HandleRef Instrument_Handle, out ushort Display_State);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetSweepSpacing", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetSweepSpacing(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Sweep_Spacing);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetSweepSpacing", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetSweepSpacing(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out int Sweep_Spacing);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_Abort", CallingConvention = CallingConvention.StdCall)]
            public static extern int Abort(System.Runtime.InteropServices.HandleRef Instrument_Handle);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_Hold", CallingConvention = CallingConvention.StdCall)]
            public static extern int Hold(System.Runtime.InteropServices.HandleRef Instrument_Handle);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SendTrigger", CallingConvention = CallingConvention.StdCall)]
            public static extern int SendTrigger(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SendTriggerWaitOPC", CallingConvention = CallingConvention.StdCall)]
            public static extern int SendTriggerWaitOPC(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Timeout);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetTriggerSource", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetTriggerSource(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Trigger_Source);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetTriggerSource", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetTriggerSource(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out int Trigger_Source);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetTriggerLevel", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetTriggerLevel(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Signal_Type, double Trigger_Level);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetTriggerLevel", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetTriggerLevel(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Signal_Type, out double Trigger_Level);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetTriggerOffset", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetTriggerOffset(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, double Trigger_Delay);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetTriggerOffset", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetTriggerOffset(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out double Trigger_Delay);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetTriggerOffsetAdjust", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetTriggerOffsetAdjust(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, ushort Offset_Adjust);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetTriggerOffsetAdjust", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetTriggerOffsetAdjust(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out ushort Offset_Adjust);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetTriggerSourcePolarity", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetTriggerSourcePolarity(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Polarity);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetTriggerSourcePolarity", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetTriggerSourcePolarity(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out int Polarity);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetGatedTrigger", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetGatedTrigger(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, ushort Gated_Trigger);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetGatedTrigger", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetGatedTrigger(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out ushort Gated_Trigger);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetGatedTriggerSource", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetGatedTriggerSource(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Trigger_Source);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetGatedTriggerSource", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetGatedTriggerSource(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out int Trigger_Source);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetGatedTriggerMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetGatedTriggerMode(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Trigger_Mode);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetGatedTriggerMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetGatedTriggerMode(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out int Trigger_Mode);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetGatedTriggerPolarity", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetGatedTriggerPolarity(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Trigger_Polarity);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetGatedTriggerPolarity", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetGatedTriggerPolarity(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out int Trigger_Polarity);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetGatedTriggerDelay", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetGatedTriggerDelay(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, double Trigger_Delay);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetGatedTriggerDelay", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetGatedTriggerDelay(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out double Trigger_Delay);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetGatedTriggerLength", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetGatedTriggerLength(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, double Trigger_Length);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetGatedTriggerLength", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetGatedTriggerLength(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out double Trigger_Length);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetGatedTriggerState", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetGatedTriggerState(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Range, int Trace, ushort Gate_Range_Table_State);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetGatedTriggerState", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetGatedTriggerState(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Range, int Trace, out ushort Gate_Range_Table_State);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetGatedTriggerRange", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetGatedTriggerRange(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Range, int Trace, double Start, double Stop);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetGatedTriggerRange", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetGatedTriggerRange(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Range, int Trace, out double Start, out double Stop);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetGatedTriggerPeriod", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetGatedTriggerPeriod(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Trace, double Period);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetGatedTriggerPeriod", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetGatedTriggerPeriod(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Trace, out double Period);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetGatedTriggerComment", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetGatedTriggerComment(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Trace, string Comment);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetGatedTriggerComment", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetGatedTriggerComment(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Trace, int Array_Size, System.Text.StringBuilder Comment);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetTVTrigger", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetTVTrigger(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, ushort TV_Trigger);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetTVTrigger", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetTVTrigger(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out ushort TV_Trigger);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetTVTriggerVerticalSync", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetTVTriggerVerticalSync(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Field);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetTVTriggerVerticalSync", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetTVTriggerVerticalSync(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out int Field);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetTVTriggerHorizontalSync", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetTVTriggerHorizontalSync(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Line_Number);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetTVTriggerHorizontalSync", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetTVTriggerHorizontalSync(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out int Line_Number);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetTVTriggerPolarity", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetTVTriggerPolarity(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Polarity);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetTVTriggerPolarity", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetTVTriggerPolarity(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out int Polarity);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetTVTriggerLines", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetTVTriggerLines(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Lines_System);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetTVTriggerLines", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetTVTriggerLines(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out int Lines_System);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetTVTriggerCCVS", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetTVTriggerCCVS(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int CCVS_Signal);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetTVTriggerCCVS", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetTVTriggerCCVS(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out int CCVS_Signal);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetTraceMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetTraceMode(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Trace, int Trace_Mode);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetTraceMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetTraceMode(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Trace, out int Trace_Mode);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetTraceBlanking", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetTraceBlanking(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Trace, ushort Trace_State);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetTraceBlanking", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetTraceBlanking(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Trace, out ushort Trace_State);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetTraceDetector", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetTraceDetector(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Trace, int Trace_Detector);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetTraceDetector", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetTraceDetector(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Trace, out int Trace_Detector);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetTraceDetectorAuto", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetTraceDetectorAuto(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Trace, ushort Couple_Detector);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetTraceDetectorAuto", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetTraceDetectorAuto(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Trace, out ushort Couple_Detector);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetTraceResetBehavior", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetTraceResetBehavior(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Trace, ushort Switch_Off_Reset);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetTraceResetBehavior", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetTraceResetBehavior(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Trace, out ushort Switch_Off_Reset);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetTraceMath", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetTraceMath(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, ushort Math_Relation);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetTraceMath", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetTraceMath(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out ushort Math_Relation);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetTraceMathMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetTraceMathMode(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Math_Mode);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetTraceMathMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetTraceMathMode(System.Runtime.InteropServices.HandleRef Instrument_Handle, out int Math_Mode);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetTraceMathPosition", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetTraceMathPosition(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, double Position);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetTraceMathPosition", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetTraceMathPosition(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out double Position);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetTraceMathExpression", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetTraceMathExpression(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Expression);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetTraceMathExpression", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetTraceMathExpression(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out int Expression);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetTraceFeedControl", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetTraceFeedControl(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Control, int Transmission_Mode);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetTraceFeedControl", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetTraceFeedControl(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Control, out int Transmission_Mode);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetIQTrace", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetIQTrace(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, ushort IQ_Trace_State);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetIQTrace", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetIQTrace(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out ushort IQ_Trace_State);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetIQTraceAveraging", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetIQTraceAveraging(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, ushort IQ_Trace_State);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetIQTraceAveraging", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetIQTraceAveraging(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out ushort IQ_Trace_State);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetIQTraceAveragingCount", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetIQTraceAveragingCount(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Average_Count);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetIQTraceAveragingCount", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetIQTraceAveragingCount(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out int Average_Count);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetIQTraceSynchronization", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetIQTraceSynchronization(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, ushort State);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetIQTraceSynchronization", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetIQTraceSynchronization(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out ushort State);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetIQTraceSamplingRate", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetIQTraceSamplingRate(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, double Sampling_Rate);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetIQTraceSamplingRate", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetIQTraceSamplingRate(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out double Sampling_Rate);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetIQTraceSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetIQTraceSettings(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, double Sampling_Rate, int Trigger_Mode, int Polarity, double Bandwidth, int Pretrigger_Samples, int No_of_Samples);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetIQTraceSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetIQTraceSettings(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out double Sampling_Rate, out int Trigger_Mode, out int Polarity, out double Bandwidth, out int Pretrigger_Samples, out int No_of_Samples);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_CopyTrace", CallingConvention = CallingConvention.StdCall)]
            public static extern int CopyTrace(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Source_Trace, int Destination_Trace);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_ReadTraceData", CallingConvention = CallingConvention.StdCall)]
            public static extern int ReadTraceData(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Trace, out int No_of_Points, [In, Out] double[] Trace_Data);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_WriteTraceData", CallingConvention = CallingConvention.StdCall)]
            public static extern int WriteTraceData(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Trace, int No_of_Points, double[] Trace_Data);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_ReadIQTraceData", CallingConvention = CallingConvention.StdCall)]
            public static extern int ReadIQTraceData(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Buffer_Size, out int No_of_Points, [In, Out] double[] Real_Parts__I_, [In, Out] double[] Imaginary_Parts__Q_);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_ReadIQTraceDataMemory", CallingConvention = CallingConvention.StdCall)]
            public static extern int ReadIQTraceDataMemory(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Offset_Samples, int No_of_Samples, int Buffer_Size, out int No_of_Points, [In, Out] double[] Real_Parts__I_, [In, Out] double[] Imaginary_Parts__Q_);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_ReadIQTraceDataMemoryAIQ", CallingConvention = CallingConvention.StdCall)]
            public static extern int ReadIQTraceDataMemoryAIQ(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Offset_Samples, int No_of_Samples, int Buffer_Size, out int No_of_Points, [In, Out] double[] Real_Parts__I_, [In, Out] double[] Imaginary_Parts__Q_);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_CalibrationTotal", CallingConvention = CallingConvention.StdCall)]
            public static extern int CalibrationTotal(System.Runtime.InteropServices.HandleRef Instrument_Handle, out int Result);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_CalibrationAbort", CallingConvention = CallingConvention.StdCall)]
            public static extern int CalibrationAbort(System.Runtime.InteropServices.HandleRef Instrument_Handle);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_CalibrationResults", CallingConvention = CallingConvention.StdCall)]
            public static extern int CalibrationResults(System.Runtime.InteropServices.HandleRef Instrument_Handle, System.Text.StringBuilder Calibration_Result);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetCalibrationCorrection", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetCalibrationCorrection(System.Runtime.InteropServices.HandleRef Instrument_Handle, ushort Calibration_State);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetCalibrationCorrection", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetCalibrationCorrection(System.Runtime.InteropServices.HandleRef Instrument_Handle, out ushort Calibration_State);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetYIGCorrectionState", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetYIGCorrectionState(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, ushort State);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetYIGCorrectionState", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetYIGCorrectionState(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out ushort State);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_RunScan", CallingConvention = CallingConvention.StdCall)]
            public static extern int RunScan(System.Runtime.InteropServices.HandleRef Instrument_Handle);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_HoldScan", CallingConvention = CallingConvention.StdCall)]
            public static extern int HoldScan(System.Runtime.InteropServices.HandleRef Instrument_Handle);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_ContinueScan", CallingConvention = CallingConvention.StdCall)]
            public static extern int ContinueScan(System.Runtime.InteropServices.HandleRef Instrument_Handle);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_StopScan", CallingConvention = CallingConvention.StdCall)]
            public static extern int StopScan(System.Runtime.InteropServices.HandleRef Instrument_Handle);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetScanMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetScanMode(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Scan_Mode);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetScanMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetScanMode(System.Runtime.InteropServices.HandleRef Instrument_Handle, out int Scan_Mode);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetScanTimeDomain", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetScanTimeDomain(System.Runtime.InteropServices.HandleRef Instrument_Handle, double Measurement_Time);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetScanTimeDomain", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetScanTimeDomain(System.Runtime.InteropServices.HandleRef Instrument_Handle, out double Measurement_Time);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetScanDisplaySpacing", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetScanDisplaySpacing(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Display_Spacing);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetScanDisplaySpacing", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetScanDisplaySpacing(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out int Display_Spacing);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetScanRanges", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetScanRanges(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Number_of_ranges);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetScanRanges", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetScanRanges(System.Runtime.InteropServices.HandleRef Instrument_Handle, out int Number_of_ranges);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetScanPoints", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetScanPoints(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Number_of_Points);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetScanStart", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetScanStart(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Scan_Range, double Start_Frequency);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetScanStart", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetScanStart(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Scan_Range, out double Start_Frequency);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetScanStop", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetScanStop(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Scan_Range, double Stop_Frequency);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetScanStop", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetScanStop(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Scan_Range, out double Stop_Frequency);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetScanStepSize", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetScanStepSize(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Scan_Range, double Step_Size);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetScanStepSize", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetScanStepSize(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Scan_Range, out double Step_Size);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetScanBandwidth", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetScanBandwidth(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Scan_Range, double Bandwidth);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetScanBandwidth", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetScanBandwidth(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Scan_Range, out double Bandwidth);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetScanMeasurementTime", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetScanMeasurementTime(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Scan_Range, double Measurement_Time);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetScanMeasurementTime", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetScanMeasurementTime(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Scan_Range, out double Measurement_Time);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetScanAutoRanging", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetScanAutoRanging(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Scan_Range, ushort Auto_Ranging);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetScanAutoRanging", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetScanAutoRanging(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Scan_Range, out ushort Auto_Ranging);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetScanRFAttenuation", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetScanRFAttenuation(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Scan_Range, double Attenuation);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetScanRFAttenuation", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetScanRFAttenuation(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Scan_Range, out double Attenuation);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetScanPreamplifier", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetScanPreamplifier(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Scan_Range, ushort Preamplifier_State);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetScanPreamplifier", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetScanPreamplifier(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Scan_Range, out ushort Preamplifier_State);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetScanAutoPreamplifier", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetScanAutoPreamplifier(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Scan_Range, ushort Preamplifier_In_Autoranging);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetScanAutoPreamplifier", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetScanAutoPreamplifier(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Scan_Range, out ushort Preamplifier_In_Autoranging);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetScanLowNoisePreamplifierState", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetScanLowNoisePreamplifierState(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Scan_Range, ushort Low_Noise_Preamplifier_State);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetScanLowNoisePreamplifierState", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetScanLowNoisePreamplifierState(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Scan_Range, out ushort Low_Noise_Preamplifier_State);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetScanInput", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetScanInput(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Scan_Range, int Input);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetScanInput", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetScanInput(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Scan_Range, out int Input);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetMarker", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetMarker(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Marker, ushort State);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetMarker", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetMarker(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Marker, out ushort State);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_AllMarkersOff", CallingConvention = CallingConvention.StdCall)]
            public static extern int AllMarkersOff(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetMarkerPosition", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetMarkerPosition(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Marker, double X);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetMarkerPosition", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetMarkerPosition(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Marker, out double X);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetMarkerValue", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetMarkerValue(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Marker, out double Y);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetMarkerZoom", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetMarkerZoom(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, double Zoom);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetMarkerZoom", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetMarkerZoom(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out double Zoom);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetMarkerFrequencyCounter", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetMarkerFrequencyCounter(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Marker, ushort Counter_State);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetMarkerFrequencyCounter", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetMarkerFrequencyCounter(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Marker, out ushort Counter_State);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetMarkerFrequencyCounterResolution", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetMarkerFrequencyCounterResolution(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, double Resolution);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetMarkerFrequencyCounterResolution", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetMarkerFrequencyCounterResolution(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out double Resolution);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetMarkerFrequencyCounterValue", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetMarkerFrequencyCounterValue(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Marker, out double Frequency_Counter_Value);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetDeltaMarker", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetDeltaMarker(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Marker, ushort State);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetDeltaMarker", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetDeltaMarker(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Marker, out ushort State);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_AllDeltaMarkersOff", CallingConvention = CallingConvention.StdCall)]
            public static extern int AllDeltaMarkersOff(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetDeltaMarkerMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetDeltaMarkerMode(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Delta_Marker_Mode);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetDeltaMarkerMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetDeltaMarkerMode(System.Runtime.InteropServices.HandleRef Instrument_Handle, out int Delta_Marker_Mode);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetDeltaMarkerPosition", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetDeltaMarkerPosition(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Marker, double Position);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetDeltaMarkerPosition", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetDeltaMarkerPosition(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Marker, out double Position);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetDeltaMarkerRelativePosition", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetDeltaMarkerRelativePosition(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Marker, out double Position);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetDeltaMarkerValue", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetDeltaMarkerValue(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Marker, out double Y);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetDeltaMarkerReferenceFixed", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetDeltaMarkerReferenceFixed(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, ushort Fixed_Reference_Value);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetDeltaMarkerReferenceFixed", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetDeltaMarkerReferenceFixed(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out ushort Fixed_Reference_Value);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetDeltaMarkerReferencePointLevel", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetDeltaMarkerReferencePointLevel(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, double Reference_Point_Level);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetDeltaMarkerReferencePointLevel", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetDeltaMarkerReferencePointLevel(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out double Reference_Point_Level);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetDeltaMarkerReferencePointToPeak", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetDeltaMarkerReferencePointToPeak(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetDeltaMarkerReferencePointLevelOffset", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetDeltaMarkerReferencePointLevelOffset(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, double Offset);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetDeltaMarkerReferencePointLevelOffset", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetDeltaMarkerReferencePointLevelOffset(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out double Offset);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetDeltaMarkerReferencePointPosition", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetDeltaMarkerReferencePointPosition(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, double Reference_Point);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetDeltaMarkerReferencePointPosition", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetDeltaMarkerReferencePointPosition(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out double Reference_Point);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_MarkerSearch", CallingConvention = CallingConvention.StdCall)]
            public static extern int MarkerSearch(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Marker, int Search_Mode);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetAutoMarkerSearch", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetAutoMarkerSearch(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Auto_Search_Mode, ushort Auto_Search_State);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetAutoMarkerSearch", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetAutoMarkerSearch(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Auto_Search_Mode, out ushort Auto_Search_State);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetMarkerTo", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetMarkerTo(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Marker, int Mkr_Function);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetMarkerToTrace", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetMarkerToTrace(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Marker, int Trace);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetMarkerToTrace", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetMarkerToTrace(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Marker, out int Trace);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetPeakExcursion", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetPeakExcursion(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, double Peak_Excursion);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetPeakExcursion", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetPeakExcursion(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out double Peak_Excursion);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetLocalOscillatorSuppression", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetLocalOscillatorSuppression(System.Runtime.InteropServices.HandleRef Instrument_Handle, ushort Local_Oscillator_Suppression);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetLocalOscillatorSuppression", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetLocalOscillatorSuppression(System.Runtime.InteropServices.HandleRef Instrument_Handle, out ushort Local_Oscillator_Suppression);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetMarkerSettingsCoupled", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetMarkerSettingsCoupled(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Marker, ushort Coupling_State);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetMarkerSettingsCoupled", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetMarkerSettingsCoupled(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Marker, out ushort Coupling_State);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetMarkerCoupling", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetMarkerCoupling(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, ushort Coupling_State);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetMarkerCoupling", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetMarkerCoupling(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out ushort Coupling_State);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_DeltaMarkerSearch", CallingConvention = CallingConvention.StdCall)]
            public static extern int DeltaMarkerSearch(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Marker, int Search_Mode);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetDeltaMarkerToTrace", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetDeltaMarkerToTrace(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Marker, int Trace);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetDeltaMarkerToTrace", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetDeltaMarkerToTrace(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Marker, out int Trace);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetLinkMarkerAndDeltaMarkerState", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetLinkMarkerAndDeltaMarkerState(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Marker, ushort Linking_State);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetLinkMarkerAndDeltaMarkerState", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetLinkMarkerAndDeltaMarkerState(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Marker, out ushort Linking_State);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetSearchLimitLines", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetSearchLimitLines(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, ushort Limited_Range);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetSearchLimitLines", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetSearchLimitLines(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out ushort Limited_Range);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetSearchLeftLimitLine", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetSearchLeftLimitLine(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, double Left_Limit);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetSearchLeftLimitLine", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetSearchLeftLimitLine(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out double Left_Limit);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetSearchRightLimitLine", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetSearchRightLimitLine(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, double Right_Limit);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetSearchRightLimitLine", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetSearchRightLimitLine(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out double Right_Limit);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetSearchThreshold", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetSearchThreshold(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, ushort Search_Threshold);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetSearchThreshold", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetSearchThreshold(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out ushort Search_Threshold);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetSearchThresholdLine", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetSearchThresholdLine(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, double Threshold);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetSearchThresholdLine", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetSearchThresholdLine(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out double Threshold);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetNoiseDensityMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetNoiseDensityMeasurement(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, ushort Noise_Measurement);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetNoiseDensityMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetNoiseDensityMeasurement(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out ushort Noise_Measurement);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetNoiseDensityMeasurementValue", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetNoiseDensityMeasurementValue(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out double Noise_Measurement);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetPhaseNoiseMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetPhaseNoiseMeasurement(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, ushort Phase_Noise_Measurement);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetPhaseNoiseMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetPhaseNoiseMeasurement(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out ushort Phase_Noise_Measurement);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetPhaseNoiseMeasurementAuto", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetPhaseNoiseMeasurementAuto(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, ushort Automatic_Peak_Search);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetPhaseNoiseMeasurementAuto", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetPhaseNoiseMeasurementAuto(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out ushort Automatic_Peak_Search);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetPhaseNoiseMeasurementValue", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetPhaseNoiseMeasurementValue(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out double Phase_Noise_Measurement);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetNdBDown", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetNdBDown(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, ushort N_dB_Down_State);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetNdBDown", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetNdBDown(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out ushort N_dB_Down_State);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetNdBDownLevelSpacing", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetNdBDownLevelSpacing(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, double Level_Spacing);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetNdBDownLevelSpacing", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetNdBDownLevelSpacing(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out double Level_Spacing);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetNdBDownBandwidth", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetNdBDownBandwidth(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out double Frequency_Spacing);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetNdBDownFrequencies", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetNdBDownFrequencies(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, [In, Out] double[] Frequencies);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetNdBDownTimes", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetNdBDownTimes(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, [In, Out] double[] Times);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SearchForPeakList", CallingConvention = CallingConvention.StdCall)]
            public static extern int SearchForPeakList(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Number_of_maxima);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetPeakListSortMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetPeakListSortMode(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Sort_Mode);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetPeakListSortMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetPeakListSortMode(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out int Sort_Mode);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetPeakListCount", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetPeakListCount(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out int Peak_Count);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetPeakListXValues", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetPeakListXValues(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, [In, Out] double[] X_Values);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetPeakListYValues", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetPeakListYValues(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, [In, Out] double[] Y_Values);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetMarkerDemodulation", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetMarkerDemodulation(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Marker, ushort Demodulation_State);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetMarkerDemodulation", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetMarkerDemodulation(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Marker, out ushort Demodulation_State);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetMarkerDemodulationType", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetMarkerDemodulationType(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Demodulation_Type);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetMarkerDemodulationType", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetMarkerDemodulationType(System.Runtime.InteropServices.HandleRef Instrument_Handle, out int Demodulation_Type);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetSquelchAnalyzer", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetSquelchAnalyzer(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Marker, ushort Squelch_State, double Squelch_Level);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetSquelchAnalyzer", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetSquelchAnalyzer(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Marker, out ushort Squelch_State, out double Squelch_Level);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetMarkerStopTime", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetMarkerStopTime(System.Runtime.InteropServices.HandleRef Instrument_Handle, double Holdoff);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetMarkerStopTime", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetMarkerStopTime(System.Runtime.InteropServices.HandleRef Instrument_Handle, out double Holdoff);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetContinuousMarkerDemodulation", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetContinuousMarkerDemodulation(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, ushort Continuous_Demodulation);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetContinuousMarkerDemodulation", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetContinuousMarkerDemodulation(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out ushort Continuous_Demodulation);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_InitiateSpurious", CallingConvention = CallingConvention.StdCall)]
            public static extern int InitiateSpurious(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetPowerUnit", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetPowerUnit(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Power_Unit);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetPowerUnit", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetPowerUnit(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out int Power_Unit);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetTimeDomainPower", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetTimeDomainPower(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Power_Measurement, ushort Measurement_State);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetTimeDomainPower", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetTimeDomainPower(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Power_Measurement, out ushort Measurement_State);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetTimeDomainPowerAveraging", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetTimeDomainPowerAveraging(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, ushort Averaging);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetTimeDomainPowerAveraging", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetTimeDomainPowerAveraging(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out ushort Averaging);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetTimeDomainPowerPeakHold", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetTimeDomainPowerPeakHold(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, ushort Peak_Hold);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetTimeDomainPowerPeakHold", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetTimeDomainPowerPeakHold(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out ushort Peak_Hold);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetTimeDomainPowerMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetTimeDomainPowerMode(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Mode);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetTimeDomainPowerMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetTimeDomainPowerMode(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out int Mode);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetTimeDomainPowerReference", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetTimeDomainPowerReference(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetTimeDomainPowerMeanValue", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetTimeDomainPowerMeanValue(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Result_Type, out double Result);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetTimeDomainPowerPeakValue", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetTimeDomainPowerPeakValue(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Result_Type, out double Result);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetTimeDomainPowerRMSValue", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetTimeDomainPowerRMSValue(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Result_Type, out double Result);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetTimeDomainPowerStandardDeviationValue", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetTimeDomainPowerStandardDeviationValue(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Result_Type, out double Result);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetTimeDomainPowerSignalPulsesValue", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetTimeDomainPowerSignalPulsesValue(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, double Time_offset_of_first_pulse, double Measurement_Time, double Period, int Pulses, out double Result);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetPowerMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetPowerMeasurement(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Measurement);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetPowerMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetPowerMeasurement(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out int Measurement);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_AdjustPowerMeasurementSetting", CallingConvention = CallingConvention.StdCall)]
            public static extern int AdjustPowerMeasurementSetting(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Measurement);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_AdjustReferenceLevel", CallingConvention = CallingConvention.StdCall)]
            public static extern int AdjustReferenceLevel(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetPowerMeasurementValue", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetPowerMeasurementValue(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Measurement, int Array_Size, [In, Out] double[] Result);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetChannelPowerHzMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetChannelPowerHzMeasurement(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, ushort Relative_Result);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetChannelPowerHzMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetChannelPowerHzMeasurement(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out ushort Relative_Result);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetPowerMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetPowerMode(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Power_Mode);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetPowerMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetPowerMode(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out int Power_Mode);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetCPACPStandard", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetCPACPStandard(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Standard);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetCPACPStandard", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetCPACPStandard(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out int Standard);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetACPStandardCatalog", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetACPStandardCatalog(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Array_Size, System.Text.StringBuilder Catalog);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetCPACPUserStandard", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetCPACPUserStandard(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, string User_Standard_Name);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_DeleteACPUserStandard", CallingConvention = CallingConvention.StdCall)]
            public static extern int DeleteACPUserStandard(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, string Standard_Name);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SaveACPUserStandard", CallingConvention = CallingConvention.StdCall)]
            public static extern int SaveACPUserStandard(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, string Standard_Name);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetACPReferenceLevelOffset", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetACPReferenceLevelOffset(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, double Reference_Level_Offset);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetACPReferenceLevelOffset", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetACPReferenceLevelOffset(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out double Reference_Level_Offset);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetCPReference", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetCPReference(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetNoiseCorrection", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetNoiseCorrection(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, ushort Correction);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetNoiseCorrection", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetNoiseCorrection(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out ushort Correction);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetFastACP", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetFastACP(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, ushort Fast_ACP);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetFastACP", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetFastACP(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out ushort Fast_ACP);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetFullSizeDiagram", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetFullSizeDiagram(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Size);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetFullSizeDiagram", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetFullSizeDiagram(System.Runtime.InteropServices.HandleRef Instrument_Handle, out int Size);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetNumberAdjacentChannels", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetNumberAdjacentChannels(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Number_of_Adjacent_Channels);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetNumberAdjacentChannels", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetNumberAdjacentChannels(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out int Number_of_Adjacent_Channels);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetNumberTXChannels", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetNumberTXChannels(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Number_of_Carriers);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetNumberTXChannels", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetNumberTXChannels(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out int Number_of_Carriers);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetTXACPChannelBandwidth", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetTXACPChannelBandwidth(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Channel_Bandwidth, double Channel_Bandwidth_Value);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetTXACPChannelBandwidth", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetTXACPChannelBandwidth(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Channel_Bandwidth, out double Channel_Bandwidth_Value);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetACPChannelSpacing", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetACPChannelSpacing(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Channel_Spacing, double Channel_Spacing_Value);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetACPChannelSpacing", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetACPChannelSpacing(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Channel_Spacing, out double Channel_Spacing_Value);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetACPReferenceChannel", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetACPReferenceChannel(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int ACP_Reference_Channel);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetCPACPPowerMeasurementMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetCPACPPowerMeasurementMode(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Mode);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetCPACPPowerMeasurementMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetCPACPPowerMeasurementMode(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out int Mode);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetCPACPMeasurementTrace", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetCPACPMeasurementTrace(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Trace);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetCPACPMeasurementTrace", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetCPACPMeasurementTrace(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out int Trace);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetAllACPWeigthingFilterState", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetAllACPWeigthingFilterState(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, ushort Weighting_Filter_State);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetAllACPWeightingFilterFactor", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetAllACPWeightingFilterFactor(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, double Rolloff_Factor);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetACPWeightingFilterState", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetACPWeightingFilterState(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Channel, ushort Weighting_Filter_State);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetACPWeightingFilterState", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetACPWeightingFilterState(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Channel, out ushort Weighting_Filter_State);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetACPWeightingFilterFactor", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetACPWeightingFilterFactor(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Channel, double Rolloff_Factor);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetACPWeightingFilterFactor", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetACPWeightingFilterFactor(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Channel, out double Rolloff_Factor);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetACPLimitChecking", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetACPLimitChecking(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, ushort Limit_Checking);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetACPLimitChecking", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetACPLimitChecking(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out ushort Limit_Checking);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetACPLimits", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetACPLimits(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Limit_Type, int Mode, ushort State, double Limit_Value);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetACPLimits", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetACPLimits(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Mode, int Limit_Type, out ushort State, out double Limit_Value);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetACPLimitCheckingResult", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetACPLimitCheckingResult(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Limit_Type, out ushort Result);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetPowerBandwidthPercentage", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetPowerBandwidthPercentage(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, double Percentage);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetPowerBandwidthPercentage", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetPowerBandwidthPercentage(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out double Percentage);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetChannelBandwidth", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetChannelBandwidth(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, double Channel_Bandwidth);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetChannelBandwidth", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetChannelBandwidth(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out double Channel_Bandwidth);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetSignalAmplitudeStatistics", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetSignalAmplitudeStatistics(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Measurement);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetSignalAmplitudeStatistics", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetSignalAmplitudeStatistics(System.Runtime.InteropServices.HandleRef Instrument_Handle, out int Measurement);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetCCDFPercentMarker", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetCCDFPercentMarker(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Marker, double Position);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetCCDFPercentMarker", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetCCDFPercentMarker(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Marker, out double Position);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetStatisticsNumberSamples", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetStatisticsNumberSamples(System.Runtime.InteropServices.HandleRef Instrument_Handle, double Number_Of_Samples);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetStatisticsNumberSamples", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetStatisticsNumberSamples(System.Runtime.InteropServices.HandleRef Instrument_Handle, out double Number_Of_Samples);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_StatisticsAdjustSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int StatisticsAdjustSettings(System.Runtime.InteropServices.HandleRef Instrument_Handle);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_StatisticsDefaultSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int StatisticsDefaultSettings(System.Runtime.InteropServices.HandleRef Instrument_Handle);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetStatisticalMeasurementsResults", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetStatisticalMeasurementsResults(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Trace, int Result_Type, [In, Out] double[] Result);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_QuerySignalStatisticCCDFLevel", CallingConvention = CallingConvention.StdCall)]
            public static extern int QuerySignalStatisticCCDFLevel(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Trace, int Probability, out double Result);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetStatisticsXAxisReferenceLevel", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetStatisticsXAxisReferenceLevel(System.Runtime.InteropServices.HandleRef Instrument_Handle, double Reference_Level);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetStatisticsXAxisReferenceLevel", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetStatisticsXAxisReferenceLevel(System.Runtime.InteropServices.HandleRef Instrument_Handle, out double Reference_Level);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetStatisticsXAxisRange", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetStatisticsXAxisRange(System.Runtime.InteropServices.HandleRef Instrument_Handle, double Range);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetStatisticsXAxisRange", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetStatisticsXAxisRange(System.Runtime.InteropServices.HandleRef Instrument_Handle, out double Range);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetStatisticsYAxisMinValue", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetStatisticsYAxisMinValue(System.Runtime.InteropServices.HandleRef Instrument_Handle, double Min_Value);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetStatisticsYAxisMinValue", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetStatisticsYAxisMinValue(System.Runtime.InteropServices.HandleRef Instrument_Handle, out double Min_Value);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetStatisticsYAxisMaxValue", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetStatisticsYAxisMaxValue(System.Runtime.InteropServices.HandleRef Instrument_Handle, double Max_Value);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetStatisticsYAxisMaxValue", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetStatisticsYAxisMaxValue(System.Runtime.InteropServices.HandleRef Instrument_Handle, out double Max_Value);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetStatisticsYAxisUnit", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetStatisticsYAxisUnit(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Units);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetStatisticsYAxisUnit", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetStatisticsYAxisUnit(System.Runtime.InteropServices.HandleRef Instrument_Handle, out int Units);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetModulationDepthMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetModulationDepthMeasurement(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, ushort Modulation_Depth_Measurement);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetModulationDepthMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetModulationDepthMeasurement(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out ushort Modulation_Depth_Measurement);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetModulationDepthMeasurementValue", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetModulationDepthMeasurementValue(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out double Result);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetThirdOrderInterceptFunction", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetThirdOrderInterceptFunction(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int TOI_Marker);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetThirdOrderInterceptFunction", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetThirdOrderInterceptFunction(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out int TOI_Marker);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetThirdOrderInterceptMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetThirdOrderInterceptMeasurement(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, ushort State);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetThirdOrderInterceptMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetThirdOrderInterceptMeasurement(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out ushort State);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetThirdOrderInterceptMeasurementValue", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetThirdOrderInterceptMeasurementValue(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out double TOI_Result);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetMultiplePowerMeasurementParameters", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetMultiplePowerMeasurementParameters(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, ushort Peak_Detector, ushort RMS_Detector, ushort Average_Detector, int Trigger_Mode, int Trigger_Slope, double Trigger_Offset, double Gate_Length);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetMultiplePowerMeasurementParameters", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetMultiplePowerMeasurementParameters(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out ushort Peak_Detector, out ushort RMS_Detector, out ushort Average_Detector, out int Trigger_Mode, out int Trigger_Slope, out double Trigger_Offset, out double Gate_Length);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_MultiplePowerMeasurementSequence", CallingConvention = CallingConvention.StdCall)]
            public static extern int MultiplePowerMeasurementSequence(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Number_of_Entries, double[] Analyzer_Frequency, double[] Reference_Level, double[] RF_Input_Attenuation, int[] Filter_Type, double[] Resolution_Bandwidth, double[] Video_Bandwidth, double[] Measurement_Time, double[] Trigger_Level);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_MultiplePowerMeasurementResults", CallingConvention = CallingConvention.StdCall)]
            public static extern int MultiplePowerMeasurementResults(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out int Number_of_Results, [In, Out] double[] Power_Measurement_Results);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetMultiplePowerMeasurementAveraging", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetMultiplePowerMeasurementAveraging(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Averaging);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetMultiplePowerMeasurementAveraging", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetMultiplePowerMeasurementAveraging(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out int Averaging);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_DeactivateMultiplePowerMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int DeactivateMultiplePowerMeasurement(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_MultipleBurstPowerMeasurementSequence", CallingConvention = CallingConvention.StdCall)]
            public static extern int MultipleBurstPowerMeasurementSequence(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, double Analyzer_Frequency, double Resolution_Bandwidth, double Measurement_Time, int Trigger_Source, double Trigger_Level, double Trigger_Offset, int Measurement_Type, int No_of_Measurements);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_MultipleBurstPowerMeasurementResult", CallingConvention = CallingConvention.StdCall)]
            public static extern int MultipleBurstPowerMeasurementResult(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, [In, Out] double[] Power_Measurement_Results, out int Number_of_Results);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_MultipleBurstPowerMeasurementResultMinimum", CallingConvention = CallingConvention.StdCall)]
            public static extern int MultipleBurstPowerMeasurementResultMinimum(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out double Power_Measurement_Result);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetHDistState", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetHDistState(System.Runtime.InteropServices.HandleRef Instrument_Handle, ushort State);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetHDistState", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetHDistState(System.Runtime.InteropServices.HandleRef Instrument_Handle, out ushort State);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetNumberOfHDist", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetNumberOfHDist(System.Runtime.InteropServices.HandleRef Instrument_Handle, int No_of_Harmonics);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetNumberOfHDist", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetNumberOfHDist(System.Runtime.InteropServices.HandleRef Instrument_Handle, out int No_of_Harmonics);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetHDistBandwidthState", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetHDistBandwidthState(System.Runtime.InteropServices.HandleRef Instrument_Handle, ushort Harmonic_RBW_Auto);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetHDistBandwidthState", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetHDistBandwidthState(System.Runtime.InteropServices.HandleRef Instrument_Handle, out ushort Harmonic_RBW_Auto);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_AdjustHDistSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int AdjustHDistSettings(System.Runtime.InteropServices.HandleRef Instrument_Handle);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetHDist", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetHDist(System.Runtime.InteropServices.HandleRef Instrument_Handle, out double Harmonic_Distortion_in_percent, out double Harmonic_Distortion_in_dB);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetHDistHarmonicsList", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetHDistHarmonicsList(System.Runtime.InteropServices.HandleRef Instrument_Handle, [In, Out] double[] Harmonics_List);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetHDistBandwidthResolution", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetHDistBandwidthResolution(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Array_Size, [In, Out] double[] Resolution_Bandwidth_Value);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetSEMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetSEMode(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Mode);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetSEMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetSEMode(System.Runtime.InteropServices.HandleRef Instrument_Handle, out int Mode);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_ConfigureSESweepRange", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureSESweepRange(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Range, double Start_Frequency, double Stop_Frequency, int Filter, double Resolution_Bandwidth, double Video_Bandwidth, ushort Sweep_Time_Auto, double Sweep_Time, int Detector, double Ref_Level_Value, ushort Attenuator_Auto, double Manual_Attenuator_Value__dB_, int Sweep_Points, ushort Stop_After_Sweep, string Transducer_Factor);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetSEPreamplifierState", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetSEPreamplifierState(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Range, ushort Pre_namplifier);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetSEPreamplifierState", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetSEPreamplifierState(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Range, out ushort Pre_namplifier);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SEDeleteRange", CallingConvention = CallingConvention.StdCall)]
            public static extern int SEDeleteRange(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Range);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_StoreSEToFile", CallingConvention = CallingConvention.StdCall)]
            public static extern int StoreSEToFile(System.Runtime.InteropServices.HandleRef Instrument_Handle, string File_Name);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_ReadSEPeakList", CallingConvention = CallingConvention.StdCall)]
            public static extern int ReadSEPeakList(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Trace, int No_Of_Values, [In, Out] double[] Frequency, [In, Out] double[] Level, [In, Out] double[] Delta_Limit, out int Returned_Values);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetSESweepStart", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetSESweepStart(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Range, double Start_Frequency);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetSESweepStart", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetSESweepStart(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Range, out double Start_Frequency);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetSESweepStop", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetSESweepStop(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Range, double Stop_Frequency);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetSESweepStop", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetSESweepStop(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Range, out double Stop_Frequency);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetSESweepFilter", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetSESweepFilter(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Range, int Filter);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetSESweepFilter", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetSESweepFilter(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Range, out int Filter);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetSESweepRBW", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetSESweepRBW(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Range, double Resolution_Bandwidth);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetSESweepRBW", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetSESweepRBW(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Range, out double Resolution_Bandwidth);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetSESweepVBW", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetSESweepVBW(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Range, double Video_Bandwidth);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetSESweepVBW", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetSESweepVBW(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Range, out double Video_Bandwidth);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetSESweepTimeAuto", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetSESweepTimeAuto(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Range, ushort Sweep_Time_Auto);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetSESweepTimeAuto", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetSESweepTimeAuto(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Range, out ushort Sweep_Time_Auto);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetSESweepTime", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetSESweepTime(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Range, double Sweep_Time);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetSESweepTime", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetSESweepTime(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Range, out double Sweep_Time);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetSESweepDetector", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetSESweepDetector(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Range, int Detector);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetSESweepDetector", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetSESweepDetector(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Range, out int Detector);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetSESweepRefLevel", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetSESweepRefLevel(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Range, double Ref_Level_Value);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetSESweepRefLevel", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetSESweepRefLevel(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Range, out double Ref_Level_Value);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetSESweepAttenuatorAuto", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetSESweepAttenuatorAuto(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Range, ushort Attenuator_Auto);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetSESweepAttenuatorAuto", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetSESweepAttenuatorAuto(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Range, out ushort Attenuator_Auto);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetSESweepAttenuatorValue", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetSESweepAttenuatorValue(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Range, double Manual_Attenuator_Value__dB_);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetSESweepAttenuatorValue", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetSESweepAttenuatorValue(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Range, out double Manual_Attenuator_Value__dB_);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetSESweepPoints", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetSESweepPoints(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Range, int Sweep_Points);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetSESweepPoints", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetSESweepPoints(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Range, out int Sweep_Points);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetSESweepStopAfterSweep", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetSESweepStopAfterSweep(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Range, ushort Stop_After_Sweep);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetSESweepStopAfterSweep", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetSESweepStopAfterSweep(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Range, out ushort Stop_After_Sweep);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetSESweepTransducerFactor", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetSESweepTransducerFactor(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Range, string Transducer_Factor);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetSESweepTransducerFactor", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetSESweepTransducerFactor(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Range, int Array_Size, System.Text.StringBuilder Transducer_Factor);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetSESweepLimit", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetSESweepLimit(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Range, double Limit_Value);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetSESweepLimit", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetSESweepLimit(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Range, out double Limit_Value);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetSESweepLimitState", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetSESweepLimitState(System.Runtime.InteropServices.HandleRef Instrument_Handle, ushort Limit_State);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetSESweepLimitState", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetSESweepLimitState(System.Runtime.InteropServices.HandleRef Instrument_Handle, out ushort Limit_State);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetSENumberOfListRanges", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetSENumberOfListRanges(System.Runtime.InteropServices.HandleRef Instrument_Handle, out int Number_Of_Ranges);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_InitiateFinalMeasurementTest", CallingConvention = CallingConvention.StdCall)]
            public static extern int InitiateFinalMeasurementTest(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_InitiateEMITest", CallingConvention = CallingConvention.StdCall)]
            public static extern int InitiateEMITest(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetReceiverDetector", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetReceiverDetector(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, ushort Max_Peak, ushort Min_Peak, ushort RMS, ushort Average, ushort Quasipeak, ushort CISPR_Average);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetReceiverDetector", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetReceiverDetector(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out ushort Max_Peak, out ushort Min_Peak, out ushort RMS, out ushort Average, out ushort Quasipeak, out ushort CISPR_Average);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetReceiverDetectorWithCRMS", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetReceiverDetectorWithCRMS(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, ushort Max_Peak, ushort Min_Peak, ushort RMS, ushort Average, ushort Quasipeak, ushort CISPR_Average, ushort CISPR_RMS);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetReceiverDetectorWithCRMS", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetReceiverDetectorWithCRMS(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out ushort Max_Peak, out ushort Min_Peak, out ushort RMS, out ushort Average, out ushort Quasipeak, out ushort CISPR_Average, out ushort CISPR_RMS);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetFinalReceiverDetector", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetFinalReceiverDetector(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Trace, int Final_Detector);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetFinalReceiverDetector", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetFinalReceiverDetector(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Trace, out int Final_Detector);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_PeakSearch", CallingConvention = CallingConvention.StdCall)]
            public static extern int PeakSearch(System.Runtime.InteropServices.HandleRef Instrument_Handle);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetPeakSearchAuto", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetPeakSearchAuto(System.Runtime.InteropServices.HandleRef Instrument_Handle, ushort Peak_Search_Auto);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetPeakSearchAuto", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetPeakSearchAuto(System.Runtime.InteropServices.HandleRef Instrument_Handle, out ushort Peak_Search_Auto);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetPeakSearchAddition", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetPeakSearchAddition(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, double Frequency_Addition);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetPeakSearchMethod", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetPeakSearchMethod(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Search_Method);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetPeakSearchMethod", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetPeakSearchMethod(System.Runtime.InteropServices.HandleRef Instrument_Handle, out int Search_Method);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetPeakSearchMargin", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetPeakSearchMargin(System.Runtime.InteropServices.HandleRef Instrument_Handle, double Margin);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetPeakSearchMargin", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetPeakSearchMargin(System.Runtime.InteropServices.HandleRef Instrument_Handle, out double Margin);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetPeakSearchSubranges", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetPeakSearchSubranges(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Subranges);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetPeakSearchSubranges", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetPeakSearchSubranges(System.Runtime.InteropServices.HandleRef Instrument_Handle, out int Subranges);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetPeakSearchListDisplay", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetPeakSearchListDisplay(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Trace, int Indication);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetPeakSearchListDisplay", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetPeakSearchListDisplay(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Trace, out int Indication);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_ClearPeakSearchList", CallingConvention = CallingConvention.StdCall)]
            public static extern int ClearPeakSearchList(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetThresholdScan", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetThresholdScan(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, ushort Threshold_Scan);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetThresholdScan", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetThresholdScan(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out ushort Threshold_Scan);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetNBBBDiscrimination", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetNBBBDiscrimination(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, ushort Discrimination_State);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetNBBBDiscrimination", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetNBBBDiscrimination(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out ushort Discrimination_State);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetNBBBDiscriminationValue", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetNBBBDiscriminationValue(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, double Discrimination_Threshold);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetNBBBDiscriminationValue", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetNBBBDiscriminationValue(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out double Discrimination_Threshold);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetFinalMeasurementTime", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetFinalMeasurementTime(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, double Measurement_Time);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetFinalMeasurementTime", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetFinalMeasurementTime(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out double Measurement_Time);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetFinalMeasurementAuto", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetFinalMeasurementAuto(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, ushort Automatic_Measurement);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetFinalMeasurementAuto", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetFinalMeasurementAuto(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out ushort Automatic_Measurement);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetVnetworkType", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetVnetworkType(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Measurement, int V_network);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetVnetworkType", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetVnetworkType(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Measurement, out int V_network);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetVnetworkProtectionEarth", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetVnetworkProtectionEarth(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Measurement, int Protective_Earth);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetVnetworkProtectionEarth", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetVnetworkProtectionEarth(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Measurement, out int Protective_Earth);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetVnetworkProtectionEarthFinalMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetVnetworkProtectionEarthFinalMeasurement(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Number_Of_Protectives, int[] Protective_Earth);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetVnetworkProtectionEarthFinalMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetVnetworkProtectionEarthFinalMeasurement(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out int Number_Of_Protectives, [In, Out] int[] Protective_Earth);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetVnetworkPhase", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetVnetworkPhase(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Measurement, int Phase);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetVnetworkPhase", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetVnetworkPhase(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Measurement, out int Phase);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetVnetworkPhaseFinalMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetVnetworkPhaseFinalMeasurement(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Number_Of_Phases, int[] Phases);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetVnetworkPhaseFinalMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetVnetworkPhaseFinalMeasurement(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out int Number_Of_Phases, [In, Out] int[] Phases);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetVnetworkHighPass", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetVnetworkHighPass(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Measurement, ushort Phase);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetVnetworkHighPass", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetVnetworkHighPass(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Measurement, out ushort Phase);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetDemodulationType", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetDemodulationType(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Demodulation);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetDemodulationType", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetDemodulationType(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out int Demodulation);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetSquelchReceiver", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetSquelchReceiver(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, ushort Squelch_State, double Squelch_Level);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetSquelchReceiver", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetSquelchReceiver(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out ushort Squelch_State, out double Squelch_Level);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_ReadReceiverDetector", CallingConvention = CallingConvention.StdCall)]
            public static extern int ReadReceiverDetector(System.Runtime.InteropServices.HandleRef Instrument_Handle, out double First_Detector, out double Second_Detector, out double Third_Detector);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_ReadReceiverDetectorPeakHold", CallingConvention = CallingConvention.StdCall)]
            public static extern int ReadReceiverDetectorPeakHold(System.Runtime.InteropServices.HandleRef Instrument_Handle, [In, Out] double[] First_Detector, [In, Out] double[] Second_Detector, [In, Out] double[] Third_Detector);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_ReadScanMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int ReadScanMeasurement(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Array_Size, out int Trace_Status, out int Number_of_Results, System.Text.StringBuilder Status_Data, [In, Out] double[] Results_of_Trace_1, [In, Out] double[] Results_of_Trace_2, [In, Out] double[] Results_of_Trace_3);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_ReadScanMeasurementStatus", CallingConvention = CallingConvention.StdCall)]
            public static extern int ReadScanMeasurementStatus(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Array_Size, out int Number_of_Results, System.Text.StringBuilder Status_Data);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_ReadFinalMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int ReadFinalMeasurement(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Trace, int Array_Size, out int Number_of_Results, [In, Out] double[] Frequency, [In, Out] double[] Level, [In, Out] double[] Delta_Limit);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_ReadAdditionalMeasurementData", CallingConvention = CallingConvention.StdCall)]
            public static extern int ReadAdditionalMeasurementData(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Trace, int Array_Length, out int Actual_Points, [In, Out] double[] Values);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetCurrentResults", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetCurrentResults(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out double X_Value, out double Y_Value);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetCurrentYResult", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetCurrentYResult(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out double Level);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetChannelList", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetChannelList(System.Runtime.InteropServices.HandleRef Instrument_Handle, int No_Of_Values, int Scan_Table, int[] Channel_List);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetChannelList", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetChannelList(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Array_Size, int Scan_Table, [In, Out] int[] Channel_List, out int Returned_Values);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetChannelListScan", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetChannelListScan(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Scan_Table, ushort Scan);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetChannelListScan", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetChannelListScan(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Scan_Table, out ushort Scan);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetChannelListStore", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetChannelListStore(System.Runtime.InteropServices.HandleRef Instrument_Handle, ushort Store);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetChannelListStore", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetChannelListStore(System.Runtime.InteropServices.HandleRef Instrument_Handle, out ushort Store);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetTraceSingle", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetTraceSingle(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Value);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetTraceSingle", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetTraceSingle(System.Runtime.InteropServices.HandleRef Instrument_Handle, out int Value);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetTraceScan", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetTraceScan(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Value);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetTraceScan", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetTraceScan(System.Runtime.InteropServices.HandleRef Instrument_Handle, out int Value);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetInstrumentMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetInstrumentMode(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Instrument_Mode);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetInstrumentMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetInstrumentMode(System.Runtime.InteropServices.HandleRef Instrument_Handle, out int Instrument_Mode);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetMixedInstrumentMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetMixedInstrumentMode(System.Runtime.InteropServices.HandleRef Instrument_Handle, ushort Mixed_Mode, int Screens);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetMixedInstrumentMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetMixedInstrumentMode(System.Runtime.InteropServices.HandleRef Instrument_Handle, out ushort Mixed_Mode, out int Screens);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetTrackingGenerator", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetTrackingGenerator(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, ushort Tracking_Generator);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetTrackingGenerator", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetTrackingGenerator(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out ushort Tracking_Generator);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetTrackingGeneratorSourcePower", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetTrackingGeneratorSourcePower(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, double Output_Level);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetTrackingGeneratorSourcePower", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetTrackingGeneratorSourcePower(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out double Output_Level);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetTrackingGeneratorPowerOffset", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetTrackingGeneratorPowerOffset(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, double Level_Offset);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetTrackingGeneratorPowerOffset", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetTrackingGeneratorPowerOffset(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out double Level_Offset);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetTrackingGeneratorFrequencyOffset", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetTrackingGeneratorFrequencyOffset(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, double Frequency_Offset);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetTrackingGeneratorFrequencyOffset", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetTrackingGeneratorFrequencyOffset(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out double Frequency_Offset);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_ConfigureTrackingGeneratorPowerSweep", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureTrackingGeneratorPowerSweep(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Power_Sweep, double Power_Start, double Power_Stop);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetTrackingGeneratorPowerSweepMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetTrackingGeneratorPowerSweepMode(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Power_Sweep);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetTrackingGeneratorPowerSweepMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetTrackingGeneratorPowerSweepMode(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out int Power_Sweep);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetTrackingGeneratorPowerSweepStart", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetTrackingGeneratorPowerSweepStart(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, double Power_Start);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetTrackingGeneratorPowerSweepStart", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetTrackingGeneratorPowerSweepStart(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out double Power_Start);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetTrackingGeneratorPowerSweepStop", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetTrackingGeneratorPowerSweepStop(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, double Power_Stop);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetTrackingGeneratorPowerSweepStop", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetTrackingGeneratorPowerSweepStop(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out double Power_Stop);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetTrackingGeneratorMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetTrackingGeneratorMeasurement(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Measurement_Type);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetTrackingGeneratorMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetTrackingGeneratorMeasurement(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out int Measurement_Type);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetTrackingGeneratorNormalization", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetTrackingGeneratorNormalization(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, ushort Normalization);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetTrackingGeneratorNormalization", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetTrackingGeneratorNormalization(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out ushort Normalization);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_TrackingGeneratorCalibration", CallingConvention = CallingConvention.StdCall)]
            public static extern int TrackingGeneratorCalibration(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Acquisition_Type);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_TrackingGeneratorRecallSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int TrackingGeneratorRecallSettings(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_TrackingGeneratorSaveAsTrdFactor", CallingConvention = CallingConvention.StdCall)]
            public static extern int TrackingGeneratorSaveAsTrdFactor(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, string Name);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetExternalAM", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetExternalAM(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, ushort External_AM);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetExternalAM", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetExternalAM(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out ushort External_AM);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetExternalFM", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetExternalFM(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, ushort External_FM);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetExternalFM", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetExternalFM(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out ushort External_FM);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetExternalIQModulation", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetExternalIQModulation(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, ushort IQ_Modulation);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetExternalIQModulation", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetExternalIQModulation(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out ushort IQ_Modulation);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetExternalFMDeviation", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetExternalFMDeviation(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, double FM_Deviation);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetExternalFMDeviation", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetExternalFMDeviation(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out double FM_Deviation);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetExternalGenerator", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetExternalGenerator(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, ushort External_Generator);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetExternalGenerator", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetExternalGenerator(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out ushort External_Generator);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetExternalGeneratorSelection", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetExternalGeneratorSelection(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Generator, int Interface, string Type, int GPIB_Address);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetExternalGeneratorSelection", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetExternalGeneratorSelection(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Generator, out int Interface, System.Text.StringBuilder Type, out int GPIB_Address);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetExternalGeneratorFrequencyOffset", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetExternalGeneratorFrequencyOffset(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Generator, double Offset);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetExternalGeneratorFrequencyOffset", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetExternalGeneratorFrequencyOffset(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Generator, out double Offset);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetExternalGeneratorMultiplicationFactor", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetExternalGeneratorMultiplicationFactor(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Generator, int Numerator, int Denominator);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetExternalGeneratorMultiplicationFactor", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetExternalGeneratorMultiplicationFactor(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Generator, out int Numerator, out int Denominator);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetExternalGeneratorFrequencySweep", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetExternalGeneratorFrequencySweep(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Generator, ushort Sweep);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetExternalGeneratorFrequencySweep", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetExternalGeneratorFrequencySweep(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Generator, out ushort Sweep);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetExternalGeneratorPower", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetExternalGeneratorPower(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Generator, double Power);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetExternalGeneratorPower", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetExternalGeneratorPower(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Generator, out double Power);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetExternalGeneratorReferenceOscillator", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetExternalGeneratorReferenceOscillator(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Generator, int Reference_Oscillator);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetExternalGeneratorReferenceOscillator", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetExternalGeneratorReferenceOscillator(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Generator, out int Reference_Oscillator);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SendExternalGeneratorCommand", CallingConvention = CallingConvention.StdCall)]
            public static extern int SendExternalGeneratorCommand(System.Runtime.InteropServices.HandleRef Instrument_Handle, int GPIB_Address, string Command);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_QueryExternalGeneratorCommand", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryExternalGeneratorCommand(System.Runtime.InteropServices.HandleRef Instrument_Handle, int GPIB_Address, string Command, int Array_Size, System.Text.StringBuilder Result);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetAnalogDemodulator", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetAnalogDemodulator(System.Runtime.InteropServices.HandleRef Instrument_Handle, ushort Analog_Demodulator);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetAnalogDemodulator", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetAnalogDemodulator(System.Runtime.InteropServices.HandleRef Instrument_Handle, out ushort Analog_Demodulator);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetAnalogDemodulatorParameters", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetAnalogDemodulatorParameters(System.Runtime.InteropServices.HandleRef Instrument_Handle, double Sample_Rate, int Record_Length, int Trigger_Source, int Trigger_Slope, int Offset_Samples, int Number_of_Meas);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetAnalogDemodulatorParameters", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetAnalogDemodulatorParameters(System.Runtime.InteropServices.HandleRef Instrument_Handle, out double Sample_Rate, out int Record_Length, out int Trigger_Source, out int Trigger_Slope, out int Offset_Samples, out int Number_of_Meas);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetAnalogDemodulatorResultDisplay", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetAnalogDemodulatorResultDisplay(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Result_Display, int Trace);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetAnalogDemodulationMeasurementTime", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetAnalogDemodulationMeasurementTime(System.Runtime.InteropServices.HandleRef Instrument_Handle, double Measurement_Time);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetAnalogDemodulationMeasurementTime", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetAnalogDemodulationMeasurementTime(System.Runtime.InteropServices.HandleRef Instrument_Handle, out double Measurement_Time);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetAnalogDemodulationSampleRate", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetAnalogDemodulationSampleRate(System.Runtime.InteropServices.HandleRef Instrument_Handle, out double Sample_Rate);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetAnalogDemodulationRecordLength", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetAnalogDemodulationRecordLength(System.Runtime.InteropServices.HandleRef Instrument_Handle, out double Record_Length);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetAnalogDemodulationPhaseWrap", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetAnalogDemodulationPhaseWrap(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Type);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetAnalogDemodulationPhaseWrap", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetAnalogDemodulationPhaseWrap(System.Runtime.InteropServices.HandleRef Instrument_Handle, out int Type);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetAnalogDemodulationZeroPhaseRefPoint", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetAnalogDemodulationZeroPhaseRefPoint(System.Runtime.InteropServices.HandleRef Instrument_Handle, double Zero_Phase_Ref_Point);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetAnalogDemodulationZeroPhaseRefPoint", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetAnalogDemodulationZeroPhaseRefPoint(System.Runtime.InteropServices.HandleRef Instrument_Handle, out double Zero_Phase_Ref_Point);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_ConfigureAnalogDemodulationFilter", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureAnalogDemodulationFilter(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Filter_Type, ushort Filter_State, double Filter_Frequency, double AM_FM_Deemphasis_Value);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetAnalogDemodulationFilterState", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetAnalogDemodulationFilterState(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Filter_Type, ushort Filter_State);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetAnalogDemodulationFilterState", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetAnalogDemodulationFilterState(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Filter_Type, out ushort Filter_State);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetAnalogDemodulationFilterFrequency", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetAnalogDemodulationFilterFrequency(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Filter_Type, double Filter_Frequency);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetAnalogDemodulationFilterFrequency", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetAnalogDemodulationFilterFrequency(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Filter_Type, out double Filter_Frequency);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetAnalogDemodulationFilterDeemphasis", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetAnalogDemodulationFilterDeemphasis(System.Runtime.InteropServices.HandleRef Instrument_Handle, double AM_FM_Deemphasis_Value);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetAnalogDemodulationFilterDeemphasis", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetAnalogDemodulationFilterDeemphasis(System.Runtime.InteropServices.HandleRef Instrument_Handle, out double AM_FM_Deemphasis_Value);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetAFCenterFrequency", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetAFCenterFrequency(System.Runtime.InteropServices.HandleRef Instrument_Handle, double Center_Frequency);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetAFCenterFrequency", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetAFCenterFrequency(System.Runtime.InteropServices.HandleRef Instrument_Handle, out double Center_Frequency);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetAFStartFrequency", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetAFStartFrequency(System.Runtime.InteropServices.HandleRef Instrument_Handle, double Start_Frequency);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetAFStartFrequency", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetAFStartFrequency(System.Runtime.InteropServices.HandleRef Instrument_Handle, out double Start_Frequency);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetAFStopFrequency", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetAFStopFrequency(System.Runtime.InteropServices.HandleRef Instrument_Handle, double Stop_Frequency);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetAFStopFrequency", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetAFStopFrequency(System.Runtime.InteropServices.HandleRef Instrument_Handle, out double Stop_Frequency);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_AutomaticSignalSearch", CallingConvention = CallingConvention.StdCall)]
            public static extern int AutomaticSignalSearch(System.Runtime.InteropServices.HandleRef Instrument_Handle);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetAFSpan", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetAFSpan(System.Runtime.InteropServices.HandleRef Instrument_Handle, double Span);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetAFSpan", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetAFSpan(System.Runtime.InteropServices.HandleRef Instrument_Handle, out double Span);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetAFFullSpan", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetAFFullSpan(System.Runtime.InteropServices.HandleRef Instrument_Handle);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetRFFrequencySpan", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetRFFrequencySpan(System.Runtime.InteropServices.HandleRef Instrument_Handle, double Span);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetRFFrequencySpan", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetRFFrequencySpan(System.Runtime.InteropServices.HandleRef Instrument_Handle, out double Span);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetMaxRFFrequencySpan", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetMaxRFFrequencySpan(System.Runtime.InteropServices.HandleRef Instrument_Handle, double Span);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetMaxRFFrequencySpan", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetMaxRFFrequencySpan(System.Runtime.InteropServices.HandleRef Instrument_Handle, out double Span);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetAnalogDemodulationBandwidth", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetAnalogDemodulationBandwidth(System.Runtime.InteropServices.HandleRef Instrument_Handle, double Demodulation_Bandwidth);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetAnalogDemodulationBandwidth", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetAnalogDemodulationBandwidth(System.Runtime.InteropServices.HandleRef Instrument_Handle, out double Demodulation_Bandwidth);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetAnalogDemodulationResolutionBandwidth", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetAnalogDemodulationResolutionBandwidth(System.Runtime.InteropServices.HandleRef Instrument_Handle, double Resolution_Bandwidth);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetAnalogDemodulationResolutionBandwidth", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetAnalogDemodulationResolutionBandwidth(System.Runtime.InteropServices.HandleRef Instrument_Handle, out double Resolution_Bandwidth);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetAFInputCoupling", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetAFInputCoupling(System.Runtime.InteropServices.HandleRef Instrument_Handle, int AF_Input_Coupling);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetAFInputCoupling", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetAFInputCoupling(System.Runtime.InteropServices.HandleRef Instrument_Handle, out int AF_Input_Coupling);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetAnalogDemodulationAngleUnits", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetAnalogDemodulationAngleUnits(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Angle_Units);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetAnalogDemodulationAngleUnits", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetAnalogDemodulationAngleUnits(System.Runtime.InteropServices.HandleRef Instrument_Handle, out int Angle_Units);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetAnalogDemodulationZoom", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetAnalogDemodulationZoom(System.Runtime.InteropServices.HandleRef Instrument_Handle, ushort Zoom);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetAnalogDemodulationZoom", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetAnalogDemodulationZoom(System.Runtime.InteropServices.HandleRef Instrument_Handle, out ushort Zoom);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetAnalogDemodulationZoomStart", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetAnalogDemodulationZoomStart(System.Runtime.InteropServices.HandleRef Instrument_Handle, double Zoom_Start);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetAnalogDemodulationZoomStart", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetAnalogDemodulationZoomStart(System.Runtime.InteropServices.HandleRef Instrument_Handle, out double Zoom_Start);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetFMModulationMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetFMModulationMeasurement(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Marker, int Trace, int Measurement, out double Result);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetPMModulationMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetPMModulationMeasurement(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Marker, int Trace, int Measurement, out double Result);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetAMModulationMeasurement", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetAMModulationMeasurement(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Marker, int Trace, int Measurement, out double Result);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetAnalogDemodulationAudioFrequency", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetAnalogDemodulationAudioFrequency(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Marker, int Trace, out double Result);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetAnalogDemodulationFrequencyError", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetAnalogDemodulationFrequencyError(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Marker, int Trace, out double Result);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetAnalogDemodulationSINAD", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetAnalogDemodulationSINAD(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Marker, int Trace, out double Result);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetAnalogDemodulationTHD", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetAnalogDemodulationTHD(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Marker, int Trace, out double Result);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetAnalogDemodulationCarrierPower", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetAnalogDemodulationCarrierPower(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Marker, int Trace, out double Result);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetAFSpectrumResultTypes", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetAFSpectrumResultTypes(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Demodulation_Type, int Result_Type_1, int Result_Type_2, int Result_Type_3);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetAFSpectrumResultValues", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetAFSpectrumResultValues(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Demodulation_Type, int Result_Type, int Array_Size, out int Number_of_Results, [In, Out] double[] Results);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetRFSpectrumResultTypes", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetRFSpectrumResultTypes(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Result_Type_1, int Result_Type_2, int Result_Type_3);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetRFSpectrumResultValues", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetRFSpectrumResultValues(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Result_Type, int Array_Size, out int Number_of_Results, [In, Out] double[] Results);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetTimeDomainResultTypes", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetTimeDomainResultTypes(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Demodulation_Type, int Result_Type_1, int Result_Type_2, int Result_Type_3);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetTimeDomainResultValues", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetTimeDomainResultValues(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Demodulation_Type, int Result_Type, int Array_Size, out int Number_of_Results, [In, Out] double[] Results);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetFMOffsetValue", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetFMOffsetValue(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Measurement, out double Result);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetDataAveraging", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetDataAveraging(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Trace, ushort Data_Averaging);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetDataAveraging", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetDataAveraging(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Trace, out ushort Data_Averaging);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetAverageCount", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetAverageCount(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Average_Count);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetAverageCount", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetAverageCount(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out int Average_Count);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetAverageType", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetAverageType(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Average_Type);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetAverageType", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetAverageType(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out int Average_Type);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetLimitLineName", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetLimitLineName(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Limit, string Name);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetLimitLineName", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetLimitLineName(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Limit, System.Text.StringBuilder Name);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetLimitLineVisible", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetLimitLineVisible(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Limit, int Limit_Type, ushort State);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetLimitLineVisible", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetLimitLineVisible(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Limit, int Limit_Type, out ushort State);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetLimitLineTrace", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetLimitLineTrace(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Limit, int Trace);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetLimitLineTrace", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetLimitLineTrace(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Limit, out int Trace);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_CopyLimitLine", CallingConvention = CallingConvention.StdCall)]
            public static extern int CopyLimitLine(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Source_Limit, int Destination_Limit);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_DeleteLimitLine", CallingConvention = CallingConvention.StdCall)]
            public static extern int DeleteLimitLine(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Limit);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetLimitLineXOffset", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetLimitLineXOffset(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Limit, double Offset);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetLimitLineXOffset", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetLimitLineXOffset(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Limit, out double Offset);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetLimitLineYOffset", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetLimitLineYOffset(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Limit, int Limit_Type, double Offset);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetLimitLineYOffset", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetLimitLineYOffset(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Limit, int Limit_Type, out double Offset);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetActiveLimitLines", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetActiveLimitLines(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Array_Size, System.Text.StringBuilder Active_Limit_Lines);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetLimitLineCheck", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetLimitLineCheck(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Limit, ushort Limit_Check);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetLimitLineCheck", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetLimitLineCheck(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Limit, out ushort Limit_Check);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetLimitLineCheckResult", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetLimitLineCheckResult(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Limit, out int Result);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_ClearLimitLineCheckResult", CallingConvention = CallingConvention.StdCall)]
            public static extern int ClearLimitLineCheckResult(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetLimitLineDomain", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetLimitLineDomain(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Limit, int Domain);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetLimitLineDomain", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetLimitLineDomain(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Limit, out int Domain);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetLimitLineInterpolation", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetLimitLineInterpolation(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Limit, int Interpolation_Axis, int Interpolation);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetLimitLineInterpolation", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetLimitLineInterpolation(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Limit, int Interpolation_Axis, out int Interpolation);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetLimitLineScaling", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetLimitLineScaling(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Limit, int Interpolation_Axis, int Limit_Line_Scaling);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetLimitLineScaling", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetLimitLineScaling(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Limit, int Interpolation_Axis, out int Limit_Line_Scaling);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetLimitLineUnit", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetLimitLineUnit(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Limit, int Level_Axis_Unit);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetLimitLineUnit", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetLimitLineUnit(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Limit, out int Level_Axis_Unit);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetLimitLineMargin", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetLimitLineMargin(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Limit, int Limit_Line_Type, double Margin);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetLimitLineMargin", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetLimitLineMargin(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Limit, int Limit_Line_Type, out double Margin);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetLimitLineThreshold", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetLimitLineThreshold(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Limit, int Limit_Line_Type, double Threshold);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetLimitLineThreshold", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetLimitLineThreshold(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Limit, int Limit_Line_Type, out double Threshold);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetLimitLineComment", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetLimitLineComment(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Limit, string Comment);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetLimitLineComment", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetLimitLineComment(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Limit, System.Text.StringBuilder Comment);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetLimitLineValues", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetLimitLineValues(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Limit, int Axis, int Array_Size, double[] Data);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetLimitLineValues", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetLimitLineValues(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Limit, int Axis, int Array_Size, [In, Out] double[] Data);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_LimitLineShiftX", CallingConvention = CallingConvention.StdCall)]
            public static extern int LimitLineShiftX(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Limit, double Shift);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_LimitLineShiftY", CallingConvention = CallingConvention.StdCall)]
            public static extern int LimitLineShiftY(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Limit, int Axis_Type, double Shift);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetLimitLineCatalog", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetLimitLineCatalog(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Limit, int Buffer_Size, System.Text.StringBuilder Limit_Line_List);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetDisplayLine", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetDisplayLine(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Line, ushort State);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetDisplayLine", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetDisplayLine(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Line, out ushort State);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetDisplayLinePosition", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetDisplayLinePosition(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Line, double Position);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetDisplayLinePosition", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetDisplayLinePosition(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Line, out double Position);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetFrequencyLine", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetFrequencyLine(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Line, ushort State);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetFrequencyLine", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetFrequencyLine(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Line, out ushort State);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetFrequencyLinePosition", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetFrequencyLinePosition(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Line, double Position);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetFrequencyLinePosition", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetFrequencyLinePosition(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Line, out double Position);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetTimeLine", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetTimeLine(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Line, ushort State);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetTimeLine", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetTimeLine(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Line, out ushort State);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetTimeLinePosition", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetTimeLinePosition(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Line, double Position);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetTimeLinePosition", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetTimeLinePosition(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Line, out double Position);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetActiveWindow", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetActiveWindow(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetActiveWindow", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetActiveWindow(System.Runtime.InteropServices.HandleRef Instrument_Handle, out int Window);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetScreenMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetScreenMode(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Mode);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetScreenMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetScreenMode(System.Runtime.InteropServices.HandleRef Instrument_Handle, out int Mode);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetDisplaySize", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetDisplaySize(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Display_Size);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetDisplaySize", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetDisplaySize(System.Runtime.InteropServices.HandleRef Instrument_Handle, out int Display_Size);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetDisplayUpdate", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetDisplayUpdate(System.Runtime.InteropServices.HandleRef Instrument_Handle, ushort Display_Update);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetDisplayUpdate", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetDisplayUpdate(System.Runtime.InteropServices.HandleRef Instrument_Handle, out ushort Display_Update);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetDisplayBargraphMaxhold", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetDisplayBargraphMaxhold(System.Runtime.InteropServices.HandleRef Instrument_Handle, ushort Bargraph_Maxhold);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetDisplayBargraphMaxhold", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetDisplayBargraphMaxhold(System.Runtime.InteropServices.HandleRef Instrument_Handle, out ushort Bargraph_Maxhold);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetDisplayBargraphLevel", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetDisplayBargraphLevel(System.Runtime.InteropServices.HandleRef Instrument_Handle, out double Minimum_Bargraph_Level, out double Maximum_Bargraph_Level);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_ResetDisplayBargraphMaxhold", CallingConvention = CallingConvention.StdCall)]
            public static extern int ResetDisplayBargraphMaxhold(System.Runtime.InteropServices.HandleRef Instrument_Handle);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetParameterCoupling", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetParameterCoupling(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Coupled_Parameters, ushort Coupling);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetParameterCoupling", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetParameterCoupling(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Coupled_Parameters, out ushort Coupling);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetInstrumentCoupling", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetInstrumentCoupling(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Coupling);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetXAxisDisplaySpacing", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetXAxisDisplaySpacing(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Display_Spacing);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetXAxisDisplaySpacing", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetXAxisDisplaySpacing(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out int Display_Spacing);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetXAxisDisplayZoom", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetXAxisDisplayZoom(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, ushort Zoom_State);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetScreenTitle", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetScreenTitle(System.Runtime.InteropServices.HandleRef Instrument_Handle, ushort Screen_Title, string Comment_String);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetDisplayElements", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetDisplayElements(System.Runtime.InteropServices.HandleRef Instrument_Handle, ushort Annotation, ushort Logo, ushort Time___Date);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetDisplayUserDefinedColors", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetDisplayUserDefinedColors(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Color_Map, int Predefined_Colors, double Tint, double Saturation, double Brightness);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetDisplayDefaultColors", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetDisplayDefaultColors(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Default_Colors);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetDisplayPowerSaving", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetDisplayPowerSaving(System.Runtime.InteropServices.HandleRef Instrument_Handle, ushort Power_Saving, int Holdoff_Time);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetExternalReferenceOscillator", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetExternalReferenceOscillator(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Reference_Oscillator);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetExternalReferenceOscillator", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetExternalReferenceOscillator(System.Runtime.InteropServices.HandleRef Instrument_Handle, out int Reference_Oscillator);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetExternalReferenceOscillatorFrequency", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetExternalReferenceOscillatorFrequency(System.Runtime.InteropServices.HandleRef Instrument_Handle, double External_Frequency);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetExternalReferenceOscillatorFrequency", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetExternalReferenceOscillatorFrequency(System.Runtime.InteropServices.HandleRef Instrument_Handle, out double External_Frequency);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetExternalNoiseSource", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetExternalNoiseSource(System.Runtime.InteropServices.HandleRef Instrument_Handle, ushort Noise_Source);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetExternalNoiseSource", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetExternalNoiseSource(System.Runtime.InteropServices.HandleRef Instrument_Handle, out ushort Noise_Source);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetPreselector", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetPreselector(System.Runtime.InteropServices.HandleRef Instrument_Handle, ushort Preselector);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetPreselector", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetPreselector(System.Runtime.InteropServices.HandleRef Instrument_Handle, out ushort Preselector);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetTransducerFactor", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetTransducerFactor(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, ushort Transducer_Factor);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetTransducerFactor", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetTransducerFactor(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out ushort Transducer_Factor);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetTransducerFactorName", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetTransducerFactorName(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, string Name);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetTransducerFactorName", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetTransducerFactorName(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, System.Text.StringBuilder Name);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetTransducerFactorCatalog", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetTransducerFactorCatalog(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out int Number_of_Transducer_Factors, int Buffer_Size, System.Text.StringBuilder Transducer_Factors_List);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetActiveTransducerFactor", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetActiveTransducerFactor(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Buffer_Size, System.Text.StringBuilder Active_Transducer_Factor);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_DeleteTransducerFactor", CallingConvention = CallingConvention.StdCall)]
            public static extern int DeleteTransducerFactor(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, string Name);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetTransducerFactorView", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetTransducerFactorView(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, ushort Transducer_Factor_View);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetTransducerFactorView", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetTransducerFactorView(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out ushort Transducer_Factor_View);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetTransducerFactorAutoAdjust", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetTransducerFactorAutoAdjust(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, ushort Auto_Adjust);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetTransducerFactorAutoAdjust", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetTransducerFactorAutoAdjust(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out ushort Auto_Adjust);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetTransducerSet", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetTransducerSet(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, ushort Transducer_Set);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetTransducerSet", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetTransducerSet(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out ushort Transducer_Set);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetTransducerSetName", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetTransducerSetName(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, string Name);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetTransducerSetName", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetTransducerSetName(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, System.Text.StringBuilder Name);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetTransducerSetCatalog", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetTransducerSetCatalog(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out int Number_of_Transducer_Sets, int Buffer_Size, System.Text.StringBuilder Transducer_Sets_List);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_DeleteTransducerSet", CallingConvention = CallingConvention.StdCall)]
            public static extern int DeleteTransducerSet(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, string Name);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetTransducerFactorUnit", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetTransducerFactorUnit(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Transducer_Factor_Unit);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetTransducerFactorUnit", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetTransducerFactorUnit(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out int Transducer_Factor_Unit);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetTransducerFactorInterpolation", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetTransducerFactorInterpolation(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Transducer_Factor_Interpolation);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetTransducerFactorInterpolation", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetTransducerFactorInterpolation(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out int Transducer_Factor_Interpolation);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetTransducerFactorComment", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetTransducerFactorComment(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, string Comment);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetTransducerFactorComment", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetTransducerFactorComment(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, System.Text.StringBuilder Comment);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetTransducerFactorValues", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetTransducerFactorValues(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Array_Size, double[] Frequency_Values, double[] Level_Values);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetTransducerFactorValues", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetTransducerFactorValues(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out int Number_of_Results, [In, Out] double[] Frequency_Values, [In, Out] double[] Level_Values);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetTransducerSetUnit", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetTransducerSetUnit(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Transducer_Set_Unit);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetTransducerSetUnit", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetTransducerSetUnit(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out int Transducer_Set_Unit);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetTransducerSetBreak", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetTransducerSetBreak(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, ushort Transducer_Set_Break);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetTransducerSetBreak", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetTransducerSetBreak(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out ushort Transducer_Set_Break);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetTransducerSetComment", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetTransducerSetComment(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, string Comment);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetTransducerSetComment", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetTransducerSetComment(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, System.Text.StringBuilder Comment);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetTransducerSetValues", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetTransducerSetValues(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Range, double Start_Frequency, double Stop_Frequency, string Factors);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetTransducerSetValues", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetTransducerSetValues(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Range, out double Start_Frequency, out double Stop_Frequency, System.Text.StringBuilder Factors);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetGPIBAddress", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetGPIBAddress(System.Runtime.InteropServices.HandleRef Instrument_Handle, int GPIB_Address);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetGPIBAddress", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetGPIBAddress(System.Runtime.InteropServices.HandleRef Instrument_Handle, out int GPIB_Address);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetSerialInterfaceConfiguration", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetSerialInterfaceConfiguration(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Baud_Rate, int Data_Bits, int Stop_Bits, ushort Hardware_Handshake, ushort Software_Handshake, int Parity);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetSerialInterfaceConfiguration", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetSerialInterfaceConfiguration(System.Runtime.InteropServices.HandleRef Instrument_Handle, out int Baud_Rate, out int Data_Bits, out int Stop_Bits, out ushort Hardware_Handshake, out ushort Software_Handshake, out int Parity);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetSystemTime", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetSystemTime(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Hour, int Minute, int Second);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetSystemTime", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetSystemTime(System.Runtime.InteropServices.HandleRef Instrument_Handle, out int Hour, out int Minute, out int Second);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetSystemDate", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetSystemDate(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Day, int Month, int Year);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetSystemDate", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetSystemDate(System.Runtime.InteropServices.HandleRef Instrument_Handle, out int Day, out int Month, out int Year);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetSpeakerVolume", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetSpeakerVolume(System.Runtime.InteropServices.HandleRef Instrument_Handle, double Speaker_Volume);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetSpeakerVolume", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetSpeakerVolume(System.Runtime.InteropServices.HandleRef Instrument_Handle, out double Speaker_Volume);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetSoftFrontpanel", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetSoftFrontpanel(System.Runtime.InteropServices.HandleRef Instrument_Handle, ushort Soft_Frontpanel);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetSoftFrontpanel", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetSoftFrontpanel(System.Runtime.InteropServices.HandleRef Instrument_Handle, out ushort Soft_Frontpanel);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetUserPort", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetUserPort(System.Runtime.InteropServices.HandleRef Instrument_Handle, int User_Port);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetUserPort", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetUserPort(System.Runtime.InteropServices.HandleRef Instrument_Handle, out int User_Port);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_ReadUserPortValue", CallingConvention = CallingConvention.StdCall)]
            public static extern int ReadUserPortValue(System.Runtime.InteropServices.HandleRef Instrument_Handle, [In, Out] ushort[] User_Port_Value);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_WriteUserPortValue", CallingConvention = CallingConvention.StdCall)]
            public static extern int WriteUserPortValue(System.Runtime.InteropServices.HandleRef Instrument_Handle, ushort[] User_Port_Value);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetServiceSource", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetServiceSource(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Service_Source, double Calibration_Source_Level);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetServiceSource", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetServiceSource(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out int Service_Source, out double Calibration_Source_Level);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetServicePulseGenerator", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetServicePulseGenerator(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, ushort Pulse_Generator, double Pulse_Rate);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetServicePulseGenerator", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetServicePulseGenerator(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out ushort Pulse_Generator, out double Pulse_Rate);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_ServiceConfigureRectangleCalibrationSignal", CallingConvention = CallingConvention.StdCall)]
            public static extern int ServiceConfigureRectangleCalibrationSignal(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, ushort State, double Frequency);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetIFShift", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetIFShift(System.Runtime.InteropServices.HandleRef Instrument_Handle, int IF_Shift);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetIFShift", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetIFShift(System.Runtime.InteropServices.HandleRef Instrument_Handle, out int IF_Shift);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetCalibrationSignalDestination", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetCalibrationSignalDestination(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Signal_Destination);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetCalibrationSignalDestination", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetCalibrationSignalDestination(System.Runtime.InteropServices.HandleRef Instrument_Handle, out int Signal_Destination);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetCalibrationSignalVoltage", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetCalibrationSignalVoltage(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Signal_Voltage);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetCalibrationSignalVoltage", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetCalibrationSignalVoltage(System.Runtime.InteropServices.HandleRef Instrument_Handle, out int Signal_Voltage);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetCalibrationSignalPulseFrequency", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetCalibrationSignalPulseFrequency(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Pulse_Frequency);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetCalibrationSignalPulseFrequency", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetCalibrationSignalPulseFrequency(System.Runtime.InteropServices.HandleRef Instrument_Handle, out int Pulse_Frequency);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetCalibrationSignalBasebandSource", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetCalibrationSignalBasebandSource(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Baseband_Source);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetCalibrationSignalBasebandSource", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetCalibrationSignalBasebandSource(System.Runtime.InteropServices.HandleRef Instrument_Handle, out int Baseband_Source);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetServiceRectangleState", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetServiceRectangleState(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, ushort State);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetServiceRectangleState", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetServiceRectangleState(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out ushort State);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetServiceRectangleRate", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetServiceRectangleRate(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, double Frequency);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetServiceRectangleRate", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetServiceRectangleRate(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, out double Frequency);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_FirmwareUpdate", CallingConvention = CallingConvention.StdCall)]
            public static extern int FirmwareUpdate(System.Runtime.InteropServices.HandleRef Instrument_Handle, string Path);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_CompatibilityMode", CallingConvention = CallingConvention.StdCall)]
            public static extern int CompatibilityMode(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Compatibility_Mode);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_DataSetSave", CallingConvention = CallingConvention.StdCall)]
            public static extern int DataSetSave(System.Runtime.InteropServices.HandleRef Instrument_Handle, string File_Name, string Comment);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_DataSetRecall", CallingConvention = CallingConvention.StdCall)]
            public static extern int DataSetRecall(System.Runtime.InteropServices.HandleRef Instrument_Handle, string File_Name);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_DataSetRecallAuto", CallingConvention = CallingConvention.StdCall)]
            public static extern int DataSetRecallAuto(System.Runtime.InteropServices.HandleRef Instrument_Handle, string File_Name);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_DataSetItems", CallingConvention = CallingConvention.StdCall)]
            public static extern int DataSetItems(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Data_Set_Items, ushort Selected_Item);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_DataSetCatalog", CallingConvention = CallingConvention.StdCall)]
            public static extern int DataSetCatalog(System.Runtime.InteropServices.HandleRef Instrument_Handle, string Directory, System.Text.StringBuilder Catalog, int Buffer_Size);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_DataSetCatalogLong", CallingConvention = CallingConvention.StdCall)]
            public static extern int DataSetCatalogLong(System.Runtime.InteropServices.HandleRef Instrument_Handle, string Directory, System.Text.StringBuilder Catalog, int Buffer_Size);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_DataSetClear", CallingConvention = CallingConvention.StdCall)]
            public static extern int DataSetClear(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Clear, string Path);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_FileManager", CallingConvention = CallingConvention.StdCall)]
            public static extern int FileManager(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Operation_to_be_Performed, string Source, string Destination);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_StoreTraceData", CallingConvention = CallingConvention.StdCall)]
            public static extern int StoreTraceData(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Trace, string File_Name);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_StoreFinalMeasurementData", CallingConvention = CallingConvention.StdCall)]
            public static extern int StoreFinalMeasurementData(System.Runtime.InteropServices.HandleRef Instrument_Handle, string File_Name);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_StorePeakList", CallingConvention = CallingConvention.StdCall)]
            public static extern int StorePeakList(System.Runtime.InteropServices.HandleRef Instrument_Handle, string File_Name);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_StoreMarkers", CallingConvention = CallingConvention.StdCall)]
            public static extern int StoreMarkers(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, string File_Name);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_readToFile", CallingConvention = CallingConvention.StdCall)]
            public static extern int readToFile(System.Runtime.InteropServices.HandleRef Instrument_Handle, string Source, string Destination);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_writeFromFile", CallingConvention = CallingConvention.StdCall)]
            public static extern int writeFromFile(System.Runtime.InteropServices.HandleRef Instrument_Handle, string Source, string Destination);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetDecimalPointNotation", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetDecimalPointNotation(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Decimal_Separator);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetDecimalPointNotation", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetDecimalPointNotation(System.Runtime.InteropServices.HandleRef Instrument_Handle, out int Decimal_Separator);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SelectPrinter", CallingConvention = CallingConvention.StdCall)]
            public static extern int SelectPrinter(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Device_Number, string Printer_Name);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_QueryPrinterList", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryPrinterList(System.Runtime.InteropServices.HandleRef Instrument_Handle, System.Text.StringBuilder Printer_List);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_AbortHardcopy", CallingConvention = CallingConvention.StdCall)]
            public static extern int AbortHardcopy(System.Runtime.InteropServices.HandleRef Instrument_Handle);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_InitiateHardcopy", CallingConvention = CallingConvention.StdCall)]
            public static extern int InitiateHardcopy(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Device_Number);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_HardcopyDeviceSetup", CallingConvention = CallingConvention.StdCall)]
            public static extern int HardcopyDeviceSetup(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Device_Number, int Destination, int Data_Format, int Page_Orientation, string File_Name);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_HardcopyComment", CallingConvention = CallingConvention.StdCall)]
            public static extern int HardcopyComment(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, string Comment);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_HardcopyItems", CallingConvention = CallingConvention.StdCall)]
            public static extern int HardcopyItems(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Window, int Hardcopy_Items);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_HardcopyColor", CallingConvention = CallingConvention.StdCall)]
            public static extern int HardcopyColor(System.Runtime.InteropServices.HandleRef Instrument_Handle, ushort Color);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_HardcopyUserDefinedColors", CallingConvention = CallingConvention.StdCall)]
            public static extern int HardcopyUserDefinedColors(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Color_Map, int Predefined_Colors, double Tint, double Saturation, double Brightness);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_HardcopyDefaultColors", CallingConvention = CallingConvention.StdCall)]
            public static extern int HardcopyDefaultColors(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Default_Colors);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_InitiateHardcopyToFile", CallingConvention = CallingConvention.StdCall)]
            public static extern int InitiateHardcopyToFile(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Device_Number, string Destination, int Data_Format, int Page_Orientation);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetHardcopyOutput", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetHardcopyOutput(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Output);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetHardcopyOutput", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetHardcopyOutput(System.Runtime.InteropServices.HandleRef Instrument_Handle, out int Output);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_CreateNewTestReport", CallingConvention = CallingConvention.StdCall)]
            public static extern int CreateNewTestReport(System.Runtime.InteropServices.HandleRef Instrument_Handle);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_AppendTestReport", CallingConvention = CallingConvention.StdCall)]
            public static extern int AppendTestReport(System.Runtime.InteropServices.HandleRef Instrument_Handle);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetTestReportDefaultSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetTestReportDefaultSettings(System.Runtime.InteropServices.HandleRef Instrument_Handle);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_ConfigureTestReportPageSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int ConfigureTestReportPageSettings(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Item_Selector, ushort State);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_QueryTestReportPageSettings", CallingConvention = CallingConvention.StdCall)]
            public static extern int QueryTestReportPageSettings(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Item_Selector, out ushort State);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetTestReportHeaderLineState", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetTestReportHeaderLineState(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Line_Number, int State);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetTestReportHeaderLineState", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetTestReportHeaderLineState(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Line_Number, out int State);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetTestReportHeaderLogoState", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetTestReportHeaderLogoState(System.Runtime.InteropServices.HandleRef Instrument_Handle, int State);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetTestReportHeaderLogoState", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetTestReportHeaderLogoState(System.Runtime.InteropServices.HandleRef Instrument_Handle, out int State);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetTestReportHeaderLineTitle", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetTestReportHeaderLineTitle(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Line_Number, string Line_Title);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetTestReportHeaderLineTitle", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetTestReportHeaderLineTitle(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Line_Number, int Array_Size, System.Text.StringBuilder Line_Title);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_SetTestReportHeaderLineText", CallingConvention = CallingConvention.StdCall)]
            public static extern int SetTestReportHeaderLineText(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Line_Number, string Line_Text);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetTestReportHeaderLineText", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetTestReportHeaderLineText(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Line_Number, int Array_Size, System.Text.StringBuilder Line_Text);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_TestReportOperations", CallingConvention = CallingConvention.StdCall)]
            public static extern int TestReportOperations(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Operation, string Test_Report_Name);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_LoadLogo", CallingConvention = CallingConvention.StdCall)]
            public static extern int LoadLogo(System.Runtime.InteropServices.HandleRef Instrument_Handle, string Directory, string File_Name);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_GetTestReportCatalog", CallingConvention = CallingConvention.StdCall)]
            public static extern int GetTestReportCatalog(System.Runtime.InteropServices.HandleRef Instrument_Handle, out int Number_of_Test_Reports, int Buffer_Size, System.Text.StringBuilder Test_Report_List);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_errorCheckState", CallingConvention = CallingConvention.StdCall)]
            public static extern int errorCheckState(System.Runtime.InteropServices.HandleRef Instrument_Handle, ushort State_Checking);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_setStatusRegister", CallingConvention = CallingConvention.StdCall)]
            public static extern int setStatusRegister(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Register_Operation, int Questionable_Register, int Enable, int PTransition, int NTransition);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_getStatusRegister", CallingConvention = CallingConvention.StdCall)]
            public static extern int getStatusRegister(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Status_Registers_Query, out int Register_Value);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_writeInstrData", CallingConvention = CallingConvention.StdCall)]
            public static extern int writeInstrData(System.Runtime.InteropServices.HandleRef Instrument_Handle, string Write_Buffer);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_readInstrData", CallingConvention = CallingConvention.StdCall)]
            public static extern int readInstrData(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Number_Bytes_To_Read, System.Text.StringBuilder Read_Buffer, out int NumBytes_Read);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_reset", CallingConvention = CallingConvention.StdCall)]
            public static extern int reset(System.Runtime.InteropServices.HandleRef Instrument_Handle);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_self_test", CallingConvention = CallingConvention.StdCall)]
            public static extern int self_test(System.Runtime.InteropServices.HandleRef Instrument_Handle, out short Self_Test_Result, System.Text.StringBuilder Self_Test_Message);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_error_query", CallingConvention = CallingConvention.StdCall)]
            public static extern int error_query(System.Runtime.InteropServices.HandleRef Instrument_Handle, out int Error_Code, System.Text.StringBuilder Error_Message);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_revision_query", CallingConvention = CallingConvention.StdCall)]
            public static extern int revision_query(System.Runtime.InteropServices.HandleRef Instrument_Handle, System.Text.StringBuilder Instrument_Driver_Revision, System.Text.StringBuilder Firmware_Revision);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_close", CallingConvention = CallingConvention.StdCall)]
            public static extern int close(System.Runtime.InteropServices.HandleRef Instrument_Handle);

            [DllImport("rsesci_32.dll", EntryPoint = "rsesci_error_message", CallingConvention = CallingConvention.StdCall)]
            public static extern int error_message(System.Runtime.InteropServices.HandleRef Instrument_Handle, int Status_Code, System.Text.StringBuilder Message);


            public static int TestForError(System.Runtime.InteropServices.HandleRef handle, int status)
            {
                if ((status < 0))
                {
                    PInvoke.ThrowError(handle, status);
                }
                return status;
            }

            public static int ThrowError(System.Runtime.InteropServices.HandleRef handle, int code)
            {
                System.Text.StringBuilder msg = new System.Text.StringBuilder(256);
                PInvoke.error_message(handle, code, msg);
                throw new System.Runtime.InteropServices.ExternalException(msg.ToString(), code);
            }
        }
    }

    public class rsesciConstants
    {

        public const int ScreenA = 1;

        public const int ScreenB = 2;

        public const int FreqFixed = 0;

        public const int FreqSweep = 1;

        public const int FreqScan = 2;

        public const int FreqTdom = 3;

        public const int FreqStepLinkOff = 0;

        public const int FreqStepLinkRbw = 1;

        public const int FreqStepLinkSpan = 2;

        public const int SpacingLin = 0;

        public const int SpacingLog = 1;

        public const int SpacingLdb = 2;

        public const int UnitDbm = 0;

        public const int UnitVolt = 1;

        public const int UnitAmp = 2;

        public const int UnitWatt = 3;

        public const int UnitDbpw = 4;

        public const int UnitDbuv = 5;

        public const int UnitDbmv = 6;

        public const int UnitDbua = 7;

        public const int UnitDbpt = 8;

        public const int UnitDbuvM = 9;

        public const int UnitDbuaM = 10;

        public const int UnitDbuvMhz = 18;

        public const int UnitDbmvMhz = 19;

        public const int UnitDbuaMhz = 20;

        public const int Abs = 0;

        public const int Rel = 1;

        public const int CouplingAc = 0;

        public const int CouplingDc = 1;

        public const int InputImp50 = 0;

        public const int InputImp75 = 1;

        public const int InputType1 = 0;

        public const int InputType2 = 1;

        public const int PllBwidthAuto = 0;

        public const int PllBwidthHigh = 1;

        public const int PllBwidthMed = 2;

        public const int PllBwidthLow = 3;

        public const int Norm = 0;

        public const int Wide = 1;

        public const int FftFilterAuto = 0;

        public const int FftFilterWide = 1;

        public const int FftFilterNarr = 2;

        public const int FilterNormal = 0;

        public const int FilterFft = 1;

        public const int FilterCfil = 2;

        public const int FilterRrc = 3;

        public const int FilterNoise = 4;

        public const int FilterPulse = 5;

        public const int Lin = 0;

        public const int Log = 1;

        public const int ContinuousSweep = 1;

        public const int SingleSweep = 0;

        public const int Auto = 2;

        public const int Apul = 3;

        public const int Acw = 4;

        public const int TrigImm = 0;

        public const int TrigExt = 1;

        public const int TrigVid = 2;

        public const int TrigIfp = 3;

        public const int TrigRfp = 4;

        public const int TrigTv = 5;

        public const int TrigFm = 6;

        public const int TrigAm = 7;

        public const int TrigPm = 8;

        public const int TrigAmRel = 10;

        public const int PolPos = 0;

        public const int PolNeg = 1;

        public const int TrigLevel = 0;

        public const int TrigEdge = 1;

        public const int Pos = 0;

        public const int Neg = 1;

        public const int TvfieldAll = 0;

        public const int TvfieldOdd = 1;

        public const int TvfieldEven = 2;

        public const int Int = 0;

        public const int Ext = 1;

        public const int TraceWrite = 0;

        public const int TraceView = 1;

        public const int TraceAvg = 2;

        public const int TraceMaxh = 3;

        public const int TraceMinh = 4;

        public const int TracedetApeak = 0;

        public const int TracedetNeg = 1;

        public const int TracedetPos = 2;

        public const int TracedetSample = 3;

        public const int TracedetRms = 4;

        public const int TracedetAvg = 5;

        public const int TracedetQpeak = 6;

        public const int TracedetCavg = 7;

        public const int TracedetCrms = 8;

        public const int MathTr1SubTr2 = 0;

        public const int MathTr1SubTr3 = 1;

        public const int TransNever = 0;

        public const int TransAlways = 1;

        public const int ScanSing = 0;

        public const int ScanCont = 1;

        public const int MarkerPmax = 0;

        public const int MarkerNmax = 1;

        public const int MarkerLmax = 2;

        public const int MarkerRmax = 3;

        public const int MarkerPmin = 4;

        public const int MarkerNmin = 5;

        public const int MarkerLmin = 6;

        public const int MarkerRmin = 7;

        public const int MarkerToCenter = 0;

        public const int MarkerToStep = 1;

        public const int MarkerToRef = 2;

        public const int X = 0;

        public const int Y = 1;

        public const int DemodAm = 1;

        public const int DemodFm = 2;

        public const int UnitDb = 11;

        public const int UnitPct = 17;

        public const int UnitDbuvMmhz = 23;

        public const int UnitDbuaMmhz = 24;

        public const int UnitUnitless = 25;

        public const int PowerMeasAoff = 0;

        public const int PowerMeasPrev = 1;

        public const int PowerMeasPeak = 2;

        public const int PowerMeasRms = 3;

        public const int PowerMeasMean = 4;

        public const int PowerMeasSdev = 5;

        public const int ResultCurr = 0;

        public const int ResultAvg = 1;

        public const int ResultPhold = 2;

        public const int FdomainMeasOff = 0;

        public const int FdomainMeasCpow = 1;

        public const int FdomainMeasAcp = 2;

        public const int FdomainMeasMcac = 3;

        public const int FdomainMeasObw = 4;

        public const int FdomainMeasCn = 5;

        public const int FdomainMeasCn0 = 6;

        public const int StandardNone = 0;

        public const int StandardNadc = 1;

        public const int StandardTetra = 2;

        public const int StandardPdc = 3;

        public const int StandardPhs = 4;

        public const int StandardCdpd = 5;

        public const int StandardFwcd = 6;

        public const int StandardRwcd = 7;

        public const int StandardF8cd = 8;

        public const int StandardR8cd = 9;

        public const int StandardF19c = 10;

        public const int StandardR19c = 11;

        public const int StandardFw3g = 12;

        public const int StandardRw3g = 13;

        public const int StandardD2cd = 14;

        public const int StandardS2cd = 15;

        public const int StandardM2cd = 16;

        public const int StandardFis95c0 = 17;

        public const int StandardRis95c0 = 18;

        public const int StandardFis95c1 = 19;

        public const int StandardRis95c1 = 20;

        public const int StandardTcdm = 21;

        public const int StandardAwl = 22;

        public const int StandardBwl = 23;

        public const int WindowSmall = 0;

        public const int WindowLarge = 1;

        public const int BwidTx = 0;

        public const int BwidAdj = 1;

        public const int BwidAlt1 = 2;

        public const int BwidAlt2 = 3;

        public const int BwidAlt3 = 4;

        public const int BwidAlt4 = 5;

        public const int BwidAlt5 = 6;

        public const int BwidAlt6 = 7;

        public const int BwidAlt7 = 8;

        public const int BwidAlt8 = 9;

        public const int BwidAlt9 = 10;

        public const int BwidAlt10 = 11;

        public const int BwidAlt11 = 12;

        public const int BwidTx2 = 13;

        public const int BwidTx3 = 14;

        public const int BwidTx4 = 15;

        public const int BwidTx5 = 16;

        public const int BwidTx6 = 17;

        public const int BwidTx7 = 18;

        public const int BwidTx8 = 19;

        public const int BwidTx9 = 20;

        public const int BwidTx10 = 21;

        public const int BwidTx11 = 22;

        public const int AcpReferenceTx1 = 0;

        public const int AcpReferenceTx2 = 1;

        public const int AcpReferenceTx3 = 2;

        public const int AcpReferenceTx4 = 3;

        public const int AcpReferenceMin = 4;

        public const int AcpReferenceMax = 5;

        public const int AcpReferenceLowHigh = 6;

        public const int StatAoff = 0;

        public const int StatApd = 1;

        public const int StatCcdf = 2;

        public const int StatResultAll = 0;

        public const int StatResultMean = 1;

        public const int StatResultPeak = 2;

        public const int StatResultCfac = 3;

        public const int StatProb01 = 0;

        public const int StatProb1 = 1;

        public const int StatProb10 = 2;

        public const int SstatUnitAbs = 0;

        public const int SstatUnitPct = 1;

        public const int Calc = 0;

        public const int Sear = 1;

        public const int SweepModeAuto = 0;

        public const int SweepModeList = 1;

        public const int ListFtypeNorm = 0;

        public const int ListFtypeChan = 1;

        public const int ListFtypeRrc = 2;

        public const int ListFtypeP5 = 3;

        public const int ListFtypeNoise = 4;

        public const int ListFtypePulse = 5;

        public const int DetectorMaxp = 0;

        public const int DetectorMinp = 1;

        public const int DetectorRms = 2;

        public const int DetectorAvg = 3;

        public const int DetectorQpeak = 4;

        public const int DetectorCav = 5;

        public const int DetectorCrms = 6;

        public const int SearchSubr = 0;

        public const int SearchPeak = 1;

        public const int SymbolOff = 0;

        public const int SymbolCross = 1;

        public const int VnetMeasPrescan = 0;

        public const int VnetMeasFinal = 1;

        public const int VnetOff = 0;

        public const int VnetTwo = 1;

        public const int VnetFour = 2;

        public const int VnetEsh3z5 = 3;

        public const int VnetEsh2z5 = 4;

        public const int VnetEnv4200 = 5;

        public const int VnetEnv216 = 6;

        public const int EarthGrounded = 0;

        public const int EarthFloating = 1;

        public const int PhaseN = 0;

        public const int PhaseL1 = 1;

        public const int PhaseL2 = 2;

        public const int PhaseL3 = 3;

        public const int DemodOff = 0;

        public const int TracAbit = 0;

        public const int TracPwcd = 1;

        public const int InstrumentModeSanalyzer = 0;

        public const int InstrumentModeReceiver = 1;

        public const int InstrumentModeAdemod = 2;

        public const int InstrumentModeIfanalyzer = 3;

        public const int MixedModeDisplaySanalyzer = 0;

        public const int MixedModeDisplayReceiver = 1;

        public const int TgenPowFix = 0;

        public const int TgenPowSwe = 1;

        public const int CorrectionTran = 0;

        public const int CorrectionRefl = 1;

        public const int AcqThr = 0;

        public const int AcqOpen = 1;

        public const int IntfGpib = 0;

        public const int IntfTtl = 1;

        public const int TrigAf = 9;

        public const int FmSummary = 0;

        public const int DemodFmSig = 1;

        public const int AfsFmSig = 2;

        public const int RfsFmSig = 3;

        public const int PmSummary = 4;

        public const int DemodPmSig = 5;

        public const int AfsPmSig = 6;

        public const int AmSummary = 7;

        public const int DemodAmSig = 8;

        public const int AfsAmSig = 9;

        public const int RfpAmSig = 10;

        public const int DemodAmSigRel = 11;

        public const int AmSummaryRel = 12;

        public const int AfsAmSigRel = 13;

        public const int PhaseWrapPhas = 0;

        public const int PhaseWrapUph = 1;

        public const int FmFilterHp = 0;

        public const int FmFilterLp = 1;

        public const int FmFilterLpRel = 2;

        public const int FmFilterDeemp = 3;

        public const int AngleUnitsRad = 0;

        public const int AngleUnitsDeg = 1;

        public const int ModMeasPpeak = 0;

        public const int ModMeasMpeak = 1;

        public const int ModMeasMiddle = 2;

        public const int ModMeasRms = 3;

        public const int DemodTypeAm = 0;

        public const int DemodTypeFm = 1;

        public const int DemodTypePm = 2;

        public const int ResultTypeOff = 0;

        public const int ResultTypeWrite = 1;

        public const int ResultTypeAverage = 2;

        public const int ResultTypeMaxhold = 3;

        public const int ResultTypeMinhold = 4;

        public const int ResultTypeView = 5;

        public const int DemodTypeAmRel = 3;

        public const int MeasurementImm = 0;

        public const int MeasurementAverage = 1;

        public const int AverageLinear = 0;

        public const int AverageVideo = 1;

        public const int LimitLower = 0;

        public const int LimitUpper = 1;

        public const int DomainFreq = 0;

        public const int DomainTime = 1;

        public const int LimitX = 2;

        public const int UnitDbmvM = 12;

        public const int UnitDeg = 13;

        public const int UnitRad = 14;

        public const int UnitS = 15;

        public const int UnitHz = 16;

        public const int UnitDbpwMhz = 21;

        public const int UnitDbptMhz = 22;

        public const int ScreenSingle = 0;

        public const int ScreenSplit = 1;

        public const int ScreenSmall = 0;

        public const int ScreenLarge = 1;

        public const int InstCoupCent = 0;

        public const int InstCoupSpan = 1;

        public const int InstCoupAtt = 2;

        public const int InstCoupProt = 3;

        public const int InstCoupGain = 4;

        public const int InstCoupPres = 5;

        public const int InstCoupDem = 6;

        public const int InstCoupBwid = 7;

        public const int InstCoupNone = 0;

        public const int InstCoupRlevel = 1;

        public const int InstCoupCfA = 2;

        public const int InstCoupCfB = 3;

        public const int Cmap1 = 1;

        public const int Cmap2 = 2;

        public const int Cmap3 = 3;

        public const int Cmap4 = 4;

        public const int Cmap5 = 5;

        public const int Cmap6 = 6;

        public const int Cmap7 = 7;

        public const int Cmap8 = 8;

        public const int Cmap9 = 9;

        public const int Cmap10 = 10;

        public const int Cmap11 = 11;

        public const int Cmap12 = 12;

        public const int Cmap13 = 13;

        public const int Cmap14 = 14;

        public const int Cmap15 = 15;

        public const int Cmap16 = 16;

        public const int Cmap17 = 17;

        public const int Cmap18 = 18;

        public const int Cmap19 = 19;

        public const int Cmap20 = 20;

        public const int Cmap21 = 21;

        public const int Cmap22 = 22;

        public const int Cmap23 = 23;

        public const int Cmap24 = 24;

        public const int Cmap25 = 25;

        public const int Cmap26 = 26;

        public const int Cmap27 = 27;

        public const int Cmap28 = 28;

        public const int Cmap29 = 29;

        public const int Cmap30 = 30;

        public const int Cmap31 = 31;

        public const int Cmap32 = 32;

        public const int Cmap33 = 33;

        public const int Cmap34 = 34;

        public const int UserDefined = -1;

        public const int Black = 0;

        public const int Blue = 1;

        public const int Brown = 2;

        public const int Green = 3;

        public const int Cyan = 4;

        public const int Red = 5;

        public const int Magneta = 6;

        public const int Yellow = 7;

        public const int White = 8;

        public const int Dgray = 9;

        public const int Lgray = 10;

        public const int Lblue = 11;

        public const int Lgreen = 12;

        public const int Lcyan = 13;

        public const int Lred = 14;

        public const int Lmagneta = 15;

        public const int Default1 = 1;

        public const int Default2 = 2;

        public const int Eaut = 2;

        public const int None = 0;

        public const int Even = 1;

        public const int Odd = 2;

        public const int Input = 0;

        public const int Output = 1;

        public const int RfInput = 0;

        public const int IntCalibSource = 1;

        public const int IfShiftOff = 0;

        public const int IfShiftA = 1;

        public const int IfShiftB = 2;

        public const int IfShiftAuto = 3;

        public const int IqCalIhig = 0;

        public const int IqCalIlow = 1;

        public const int IqCalQhig = 2;

        public const int IqCalQlow = 3;

        public const int IqCalVolt0 = 0;

        public const int IqCalVolt01 = 1;

        public const int IqCalVolt0178 = 2;

        public const int IqCalVolt0316 = 3;

        public const int IqCalVolt0562 = 4;

        public const int IqCalVolt10 = 5;

        public const int IqCalFreq10khz = 0;

        public const int IqCalFreq62khz = 1;

        public const int IqCalFreq80khz = 2;

        public const int IqCalFreq100khz = 3;

        public const int IqCalFreq102khz = 4;

        public const int IqCalFreq200khz = 5;

        public const int IqCalFreq500khz = 6;

        public const int IqCalFreq1mhz = 7;

        public const int IqCalFreq2mhz = 8;

        public const int IqCalFreq4mhz = 9;

        public const int IqCalBbsrcIq = 0;

        public const int IqCalBbsrcGnd = 1;

        public const int IqCalBbsrcCald = 2;

        public const int IqCalBbsrcCalp = 3;

        public const int Off = 0;

        public const int Fsp = 1;

        public const int DataSetNone = 0;

        public const int DataSetDefault = 1;

        public const int DataSetAll = 2;

        public const int DataSetCurrent = 3;

        public const int DataSetTraces = 4;

        public const int DataSetFinalList = 5;

        public const int DataSetLimitLines = 6;

        public const int DataSetCalibration = 7;

        public const int DataSetTransducers = 8;

        public const int DataSetHcopy = 9;

        public const int DataSetScan = 10;

        public const int ClearAll = 0;

        public const int ClearFile = 1;

        public const int FileManCdrive = 0;

        public const int FileManCdir = 1;

        public const int FileManMdir = 2;

        public const int FileManRdir = 3;

        public const int FileManCopy = 4;

        public const int FileManMove = 5;

        public const int FileManDelete = 6;

        public const int FileManFormat = 7;

        public const int DecSeparatorPoint = 0;

        public const int DecSeparatorComma = 1;

        public const int HardcopyDisk = 0;

        public const int HardcopyPrinter = 1;

        public const int HardcopyClipboard = 2;

        public const int HardcopyWmf = 0;

        public const int HardcopyGdi = 1;

        public const int HardcopyEwmf = 2;

        public const int HardcopyBmp = 3;

        public const int HardcopyPortrait = 0;

        public const int HardcopyLandscape = 1;

        public const int HardcopyAll = 0;

        public const int HardcopyTable = 1;

        public const int HardcopyTrace = 2;

        public const int DefaultUserDefined = 0;

        public const int DefaultColorSet = 1;

        public const int DefaultScreenColors = 2;

        public const int HcopScreen = 0;

        public const int HcopTrep = 1;

        public const int TrepHead = 0;

        public const int TrepDiag = 1;

        public const int TrepFres = 2;

        public const int TrepPage = 3;

        public const int TrepStab = 4;

        public const int TrepSres = 5;

        public const int TrepTdst = 6;

        public const int TrepTran = 7;

        public const int TrepAlw = 0;

        public const int TrepOnce = 1;

        public const int TrepNev = 2;

        public const int TrepLoad = 0;

        public const int TrepSave = 1;

        public const int TrepDel = 2;
    }
}
